var Diagnostics;
(function (Diagnostics) {
    var NativeService = (function () {
        function NativeService($q) {
            this.$q = $q;
            this.iOS = false;
            this.android = false;
            this.win8 = false;
            this.browser = false;
            if (typeof window.webkit !== 'undefined') {
                this.iOS = true;
                if (typeof NativeInterface === 'undefined') {
                    NativeInterface = {};
                }
            }
            else if (window.external && typeof window.external['notify'] !== 'undefined') {
                this.win8 = true;
                var htmlElms = document.getElementById('html');
                if (htmlElms) {
                    var htmlElm = htmlElms[0];
                    htmlElm.style.overflow = 'hidden';
                }
                W8 = {
                    App: {
                        events: {},
                        addEventListener: function (eventName, callback) {
                            W8.App.events[eventName] = callback;
                        },
                        fireEvent: function (eventName, data) {
                            window.external['notify'](JSON.stringify({ eventName: eventName, data: data }));
                        }
                    }
                };
                fireJSEvent = function (eventName, data) {
                    W8.App.events[eventName](data);
                };
            }
            else if (typeof NativeInterface !== 'undefined') {
                this.android = true;
                NativeService.UseScootch = true;
            }
            else {
                this.browser = true;
            }
        }
        NativeService.prototype.cleanResult = function (result) {
            if (typeof result === 'string' && (result.substr(0, 1) === "{" || result.substr(0, 1) === "[")) {
                result = JSON.parse(result);
            }
            return result;
        };
        NativeService.prototype.call = function (taskName, data) {
            if (data === void 0) { data = null; }
            var self = this;
            var dfd = this.$q.defer();
            if (this.android) {
                if (data !== null) {
                    var result = NativeInterface[taskName](typeof (data) === "string" ? data : JSON.stringify(data));
                }
                else {
                    var result = NativeInterface[taskName]();
                }
                result = self.cleanResult(result);
                dfd.resolve(result);
            }
            else if (this.iOS) {
                if (window.webkit.messageHandlers[taskName]) {
                    NativeInterface['_' + taskName] = function (result) {
                        result = self.cleanResult(result);
                        dfd.resolve(result);
                    };
                    window.webkit.messageHandlers[taskName].postMessage(data);
                }
            }
            else if (this.win8) {
                W8.App.addEventListener('_' + taskName, function (data) {
                    data = self.cleanResult(data);
                    dfd.resolve(data);
                });
                if (data) {
                    W8.App.fireEvent(taskName, JSON.stringify(data));
                }
                else {
                    W8.App.fireEvent(taskName, JSON.stringify({}));
                }
            }
            else {
                dfd.reject("NOT_IMPLEMENTED");
                throw "NOT_IMPLEMENTED";
            }
            return dfd.promise;
        };
        NativeService.prototype.on = function (eventName, handler) {
            if (this.android || this.iOS) {
                NativeInterface[eventName] = handler;
            }
            else if (this.win8) {
                W8.App.addEventListener(eventName, handler);
            }
            else {
                throw "NOT_IMPLEMENTED";
            }
        };
        NativeService.getUUID = function (handler) {
            if (typeof window.webkit !== 'undefined') {
                if (uuid) {
                    handler(uuid);
                }
            }
            else if (typeof W8 !== 'undefined') {
                if (uuid) {
                    handler(uuid);
                }
            }
            else if (typeof NativeInterface !== 'undefined') {
                handler(NativeInterface.isUUIDAvailable());
            }
            else {
                handler(NativeInterface.isUUIDAvailable());
            }
        };
        NativeService.$inject = ['$q'];
        NativeService.UseScootch = false;
        NativeService.Tasks = {
            getDeviceName: "getDeviceName",
            getVersionName: "getVersionName",
            getOSVersion: "getOSVersion",
            getApplicationPackageName: "getApplicationPackageName",
            isBarCodeFeatureEnabled: "isBarCodeFeatureEnabled",
            connectionDropped: 'connectionDropped',
            vehicleConnectionDropped: 'vehicleConnectionDropped',
            negativeResponse: 'negativeResponse',
            onDeviceFound: 'onDeviceFound',
            clearUUID: 'clearUUID',
            vinStartScan: 'vinStartScan',
            vinScanCode: 'vinScanCode',
            isBTAvailable: 'isBTAvailable',
            isBTEnabled: 'isBTEnabled',
            btStatusChange: 'btStatusChange',
            enableDeviceBluetooth: 'enableDeviceBluetooth',
            goBack: 'goBack',
            isTestHookAvailable: 'isTestHookAvailable',
            nativeTestHook: 'nativeTestHook',
            nativeEventNotification: 'nativeEventNotification',
            newQuote: 'newQuote',
            savedQuotes: 'savedQuotes'
        };
        return NativeService;
    }());
    Diagnostics.NativeService = NativeService;
})(Diagnostics || (Diagnostics = {}));
angular.module('ad-ui-coyote-mobile-templates', ['adCatalogControl/partials/adCatalogModal.html', 'adCatalogControl/partials/catalogItemSelected.html', 'adCatalogControl/partials/selectMappedVehicleModal.html', 'login/LoginTemplate.html', 'quotes/partials/addCustomLaborModal.html', 'quotes/partials/addCustomPartModal.html', 'quotes/partials/addFlatFeeModal.html', 'quotes/partials/addNoteModal.html', 'quotes/partials/addPartsLaborModal.html', 'quotes/partials/additionalInfoModal.html', 'quotes/partials/customerFlyout.html', 'quotes/partials/customerInfoModal.html', 'quotes/partials/emailQuoteModal.html', 'quotes/partials/printPreviewTemplate.html', 'quotes/partials/quoteBannerDisplay.html', 'quotes/partials/quoteDisplay.html', 'quotes/partials/quoteHeader.html', 'quotes/partials/quoteHome.html', 'quotes/partials/quoteLineTemplates/customLabor.html', 'quotes/partials/quoteLineTemplates/customPart.html', 'quotes/partials/quoteLineTemplates/default.html', 'quotes/partials/quoteLineTemplates/flatFee.html', 'quotes/partials/quoteLineTemplates/note.html', 'quotes/partials/quoteList.html', 'quotes/partials/selectVehicleModal.html', 'setup/partials/ShopCreationTemplate.html', 'setup/partials/setup.html', 'shell/partials/shell.html']);
angular.module("adCatalogControl/partials/adCatalogModal.html", []).run(["$templateCache", function ($templateCache) {
        $templateCache.put("adCatalogControl/partials/adCatalogModal.html", "<ad-dialog-shell title=\"Add Parts and Labor\" buttons=\"vm.buttons\">\n" +
            "    <div class=\"catalogContent\">\n" +
            "        <div class=\"row top0\">\n" +
            "            <div class=\"col-md-12\">\n" +
            "                <div class=\"box\">\n" +
            "                    <i ng-hide=\"vm.readOnly\" class=\"appIcon-car2\"></i>\n" +
            "                    <span class=\"emphasize\" ng-bind=\"vm.quote.vehicleDescription\"></span>\n" +
            "                </div>\n" +
            "            </div>\n" +
            "            <div class=\"col-md-2\">\n" +
            "<!--\n" +
            "                <div class=\"box\">\n" +
            "                    <span class=\"glyphicon glyphicon-shopping-cart\"></span>\n" +
            "                    <span ng-bind=\"vm.itemsAddedToCatalog\" ng-show=\"vm.itemsAddedToCatalog > 0\"></span>\n" +
            "                </div>\n" +
            "-->\n" +
            "            </div>\n" +
            "        </div>\n" +
            "        <div class=\"row\">\n" +
            "            <div class=\"col-md-12\">\n" +
            "                <tabset justified=\"true\" >\n" +
            "                    <tab  heading=\"ALLDATA\" active=\"vm.tabALLDATAActive\" select=\"vm.ALLDATASystemLoad();vm.ShellService.sendAnalyticsEvents('ALLDATA Catalog - Tapped','Quote from Mobile','ALLDATA Catalog is tapped.')\" class=\"tabWidth\">\n" +
            "                        <div class=\"tabBorders tabPadding\">\n" +
            "                            <search-box ng-show=\"vm.ALLDATABreadcrumbs.length < 2\" placeholder=\"Search Catalog\" search-term=\"vm.ALLDATASearchTerm\" auto-search=\"false\" on-search=\"vm.ALLDATAStartCatalogSearch()\"></search-box>\n" +
            "                            <div ng-show=\"vm.ALLDATABreadcrumbs.length > 1\">\n" +
            "                                <ol class=\"breadcrumb breadcrumb-element\">\n" +
            "                                    <li ng-repeat=\"breadcrumb in vm.ALLDATABreadcrumbs\">\n" +
            "                                        <a class=\"breadcrumb-element\" ng-click=\"vm.ALLDATABreadcrumbClicked(breadcrumb)\">\n" +
            "                                            {{breadcrumb.Name}}\n" +
            "                                        </a>\n" +
            "                                    </li>\n" +
            "                                </ol>\n" +
            "                            </div>\n" +
            "                        </div>\n" +
            "                    </tab>\n" +
            "                    <tab heading=\"AutoZone\" active=\"vm.tabAutoZoneActive\" select=\"vm.AutoZoneSystemLoad();vm.ShellService.sendAnalyticsEvents('AutoZone Catalog - Tapped','Quote from Mobile','AutoZone Catalog is tapped.')\" class=\"tabWidth\">\n" +
            "                        <div class=\"tabBorders tabPadding\">\n" +
            "                            <search-box placeholder=\"Search Catalog\" search-term=\"vm.AutoZoneSearchTerm\" auto-search=\"false\" on-search=\"vm.AutoZoneStartCatalogSearch()\"></search-box>\n" +
            "                            <div ng-show=\"vm.AutoZoneBreadcrumbs.length > 1\">\n" +
            "                                <ol class=\"breadcrumb breadcrumb-element\">\n" +
            "                                    <li ng-repeat=\"breadcrumb in vm.AutoZoneBreadcrumbs\">\n" +
            "                                        <a class=\"breadcrumb-element\" ng-click=\"vm.AutoZoneBreadcrumbClicked(breadcrumb)\">\n" +
            "                                            {{breadcrumb.Name}}\n" +
            "                                        </a>\n" +
            "                                    </li>\n" +
            "                                </ol>\n" +
            "                            </div>\n" +
            "                        </div>\n" +
            "                    </tab>\n" +
            "                </tabset>\n" +
            "            </div>\n" +
            "        </div>\n" +
            "        <div class=\"adjustMargins\">\n" +
            "            <div class=\"col-md-12 col-sm-12 contentBody\">\n" +
            "                <div ng-show=\"vm.showALLDATAContent\" class=\"heightWithTotalSpace\">\n" +
            "                    <div ng-show=\"vm.showALLDATAComponents\">\n" +
            "                    <div class=\"lineSpace\">\n" +
            "                       <!-- <div ng-repeat=\"component in vm.ALLDATADisplayComponents\">\n" +
            "                            <div class=\"col-md-11 col-sm-11\">\n" +
            "                                <a class=\"leftMarginContent\" ng-click=\"vm.ALLDATAComponentClicked(component)\">{{ component.Name }}</a>\n" +
            "                            </div>\n" +
            "                            <div class=\"col-md-1 col-sm-1\">\n" +
            "                                <a class=\"leftMarginContent rightAlign\" ng-click=\"vm.ALLDATAComponentClicked(component)\"><span class=\"glyphicon glyphicon-chevron-right\"></span></a>\n" +
            "                            </div>\n" +
            "                            <div class=\"col-md-12 col-sm-12\">\n" +
            "                                <hr />\n" +
            "                            </div>\n" +
            "                        </div>-->\n" +
            "                        <ad-list items=\"vm.ALLDATADisplayComponents\" ng-hide=\"vm.noCatalogData\" on-select=\"vm.onALLDATAComponentSelected(item, $event, $index)\" template=\"'AllDataContent.html'\"></ad-list>\n" +
            "                        <script type=\"text/ng-template\" id=\"AllDataContent.html\">\n" +
            "                            <div class=\"listContent\" ng-bind=\"item.Name\"></div>\n" +
            "                        </script>\n" +
            "\n" +
            "                        <div ng-show=\"vm.noCatalogData\" class=\"content readable\">\n" +
            "                            <div class=\"article-header\">No parts of the selected type were found.</div>\n" +
            "                        </div>\n" +
            "                     </div>\n" +
            "                     </div>\n" +
            "                     <div ng-show=\"vm.invalidSearchItem && !vm.showALLDATAComponents\" class=\"content readable\">\n" +
            "                        <div class=\"article-header\">No matching results were found.</div>\n" +
            "                     </div>\n" +
            "                    <div ng-show=\"vm.showALLDATAParts\">\n" +
            "                        <h1>{{ vm.componentName }}</h1>\n" +
            "                        <!--<h3 ng-show=\"vm.ALLDATAParts != null\">Part Info</h3>-->\n" +
            "                        <div class=\"row tableHeader\" ng-show=\"vm.ALLDATAParts.length > 0\">\n" +
            "                            <div class=\"col-md-3 col-sm-3 catalogOffset\">Part Info</div>\n" +
            "                            <div class=\"col-md-9 col-sm-9 catalogOffset\">\n" +
            "                                <div class=\"row\">\n" +
            "                                    <div class=\"col-md-3 col-sm-3 catalogOffset\">Part #</div>\n" +
            "                                    <div class=\"col-md-2 col-sm-2 catalogOffset\">Price</div>\n" +
            "                                    <div class=\"col-md-7 col-sm-7 catalogOffset\">\n" +
            "                                        <div class=\"row\">\n" +
            "                                            <div class=\"col-md-4 col-sm-4 catalogOffset\">Qty</div>\n" +
            "                                            <div class=\"col-md-4 col-sm-4 catalogOffset text-center\">Taxable</div>\n" +
            "                                        </div>\n" +
            "                                    </div>\n" +
            "                                </div>\n" +
            "                            </div>\n" +
            "                        </div>\n" +
            "                        <div ng-repeat=\"part in vm.ALLDATAParts track by $index\" class=\"row\" ng-class=\"{alldataBorder: vm.isObject(part) == true}\">\n" +
            "                            <div class=\"col-md-3 col-sm-3 catalogOffset\" ng-show=\"vm.isObject(part) == false\">\n" +
            "                                <div>\n" +
            "                                    <pre class=\"indentable\" ng-bind=\"part\"></pre>\n" +
            "                                </div>\n" +
            "                            </div>\n" +
            "                            <div class=\"col-md-offset-3 col-sm-offset-3 col-md-9 col-sm-9 catalogOffset\" ng-show=\"vm.isObject(part) == true\">\n" +
            "                                <div>\n" +
            "                                <div class=\"row spaceOutRows\">\n" +
            "                                    <div class=\"col-md-3 col-sm-3 catalogOffset\" ng-bind=\"part.Value.PartNumber\"></div>\n" +
            "                                    <div class=\"col-md-2 col-sm-2 catalogOffset\" ng-bind=\"part.Value.Price | currency\"></div>\n" +
            "                                    <div class=\"col-md-7 col-sm-7 catalogOffset\">\n" +
            "                                        <catalog-item-selected callback=\"vm.addALLDATAPart(part, quantity, isTaxable, buttonLabel)\" item=\"part\" tax-parts-default=\"vm.shop.taxPartsDefault\" row-index=\"{{$index}}\" />\n" +
            "                                    </div>\n" +
            "                                </div>\n" +
            "                                <div class=\"row\" ng-show=\"part.Description != null\">\n" +
            "                                    <div class=\"col-md-offset-1 col-sm-offset-1 col-md-10 col-sm-10\" ng-bind=\"part.Value.Description\"></div>\n" +
            "                                </div>\n" +
            "                                <div class=\"row\" ng-show=\"part.PriceDescription != null\">\n" +
            "                                    <div class=\"col-md-offset-1 col-sm-offset-1 col-md-10 col-sm-10\" ng-bind=\"vm.getPartMarkupPrice(part.Value.PriceDescription)\"></div>\n" +
            "                                </div>\n" +
            "                                </div>\n" +
            "                            </div>\n" +
            "                        </div>\n" +
            "                        <div ng-show=\"vm.ALLDATALabor != null\">\n" +
            "                          <!--  <h3 ng-show=\"vm.ALLDATALabor != null\">Labor</h3>-->\n" +
            "                            <div class=\"row tableHeader\" ng-show=\"vm.ALLDATALabor.length > 0\">\n" +
            "                                <div class=\"col-md-3 col-sm-3 catalogOffset\">Labor Info</div>\n" +
            "                                <div class=\"col-md-9 col-sm-9 catalogOffset\">\n" +
            "                                    <div class=\"row\">\n" +
            "                                        <div class=\"col-md-3 col-sm-3 catalogOffset\">Skill</div>\n" +
            "                                        <div class=\"col-md-2 col-sm-2 catalogOffset\">Hrs</div>\n" +
            "                                        <div class=\"col-md-7 col-sm-7 catalogOffset\">\n" +
            "                                            <div class=\"row\">\n" +
            "                                                <div class=\"col-md-4 col-sm-4 catalogOffset\">Rate</div>\n" +
            "                                                <div class=\"col-md-4 col-sm-4 catalogOffset text-center\">Taxable</div>\n" +
            "                                            </div>\n" +
            "                                        </div>\n" +
            "                                    </div>\n" +
            "                                </div>\n" +
            "                            </div>\n" +
            "                        </div>\n" +
            "                        <div ng-repeat=\"labor in vm.ALLDATALabor track by $index\" class=\"row\" ng-class=\"{alldataBorder: vm.isObject(labor) == true}\">\n" +
            "                                <div ng-if=\"vm.isObject(labor) == false\" class=\"col-md-3 col-sm-3 catalogOffset\">\n" +
            "                                    <div>\n" +
            "                                     <pre class=\"indentable\" ng-bind=\"labor\"></pre>\n" +
            "                                    </div>\n" +
            "                                </div>\n" +
            "                                <div class=\"col-md-offset-3 col-sm-offset-3 col-md-9 col-sm-9 catalogOffset\" ng-if=\"labor.Value.Description !== undefined\">\n" +
            "                                    <div>\n" +
            "                                    <div class=\"row\"  ng-if=\"vm.isObject(labor) == true\">\n" +
            "                                        <div class=\"col-md-3 col-sm-3 catalogOffset\" ng-bind=\"labor.Value.Skill\"></div>\n" +
            "                                        <div class=\"col-md-2 col-sm-2 catalogOffset\" ng-bind=\"labor.Value.StandardTime\"></div>\n" +
            "                                        <div class=\"col-md-7 col-sm-7 catalogOffset\">\n" +
            "                                            <catalog-item-selected callback=\"vm.addALLDATALabor(labor, laborRateIndex, isTaxable, buttonLabel)\" item=\"labor\"\n" +
            "                                                                   rates=\"vm.laborRates\" default-rate-index=\"vm.getDefaultLaborRate(labor, rate)\" tax-labor-default=\"vm.shop.taxLaborDefault\" row-index=\"{{$index}}\" />\n" +
            "                                        </div>\n" +
            "                                    </div>\n" +
            "                                    <div class=\"row\" ng-show=\"labor.Description != null\">\n" +
            "                                        <div class=\"col-md-offset-1 col-sm-offset-1 col-md-10 col-sm-10\" ng-bind=\"labor.Value.Description\"></div>\n" +
            "                                    </div>\n" +
            "                                    </div>\n" +
            "                                </div>\n" +
            "                        </div>\n" +
            "                    </div>\n" +
            "                </div>\n" +
            "\n" +
            "                <div ng-show=\"vm.showAutoZoneContent\" class=\"heightWithTotalSpace\">\n" +
            "                    <div ng-show=\"vm.AZDataNotFound\" class=\"content readable\">\n" +
            "                        <div class=\"article-header\">No catalog information was found for the selected vehicle.</div>\n" +
            "                    </div>\n" +
            "                    <div ng-show=\"vm.showAutoZoneComponents\">\n" +
            "                        <div class=\"lineSpace\">\n" +
            "                           <!-- <div ng-repeat=\"component in vm.AutoZoneDisplayComponents\">\n" +
            "                                <div class=\"col-md-11\">\n" +
            "                                    <a class=\"leftMarginContent\" ng-click=\"vm.AutoZoneComponentClicked(component)\">{{ component.Name }}</a>\n" +
            "                                </div>\n" +
            "                                <div class=\"col-md-1\">\n" +
            "                                    <a class=\"leftMarginContent rightAlign\" ng-click=\"vm.AutoZoneComponentClicked(component)\"><span class=\"glyphicon glyphicon-chevron-right\"></span></a>\n" +
            "                                </div>\n" +
            "                                <div class=\"col-md-12\">\n" +
            "                                    <hr />\n" +
            "                                </div>\n" +
            "                            </div>-->\n" +
            "                            <ad-list items=\"vm.AutoZoneDisplayComponents\" ng-hide=\"vm.noCatalogData\" on-select=\"vm.onAutozoneComponentSelected(item, $event, $index)\" template=\"'AutoZoneContent.html'\"></ad-list>\n" +
            "                            <script type=\"text/ng-template\" id=\"AutoZoneContent.html\">\n" +
            "                                <div class=\"listContent\" ng-bind=\"item.Name\"></div>\n" +
            "                            </script>\n" +
            "                            <div ng-show=\"vm.noCatalogData\" class=\"content readable\">\n" +
            "                                <div class=\"article-header\">No parts of the selected type were found.</div>\n" +
            "                            </div>\n" +
            "                        </div>\n" +
            "                    </div>\n" +
            "                    <div ng-show=\"vm.invalidSearchItem && !vm.showAutoZoneComponents\" class=\"content readable\">\n" +
            "                        <div class=\"article-header\">No matching results were found.</div>\n" +
            "                    </div>\n" +
            "                    <div ng-show=\"vm.showAutoZoneParts\" class=\"fullHeight contentHeight\" id=\"AZContent\">\n" +
            "                        <h1>Parts</h1>\n" +
            "                        <div class=\"row\" ng-bind=\"vm.AutoZoneParts[0].elementName\"></div>\n" +
            "                        <div class=\"row tableHeader\">\n" +
            "                            <div class=\"col-md-5 col-sm-5 smallPadding\">Part Info</div>\n" +
            "                            <div class=\"col-md-2 col-sm-2 smallPadding\">Part #</div>\n" +
            "                            <div class=\"col-md-5 col-sm-5 smallPadding\">\n" +
            "                                <div class=\"row\">\n" +
            "                                    <div class=\"col-md-4 col-sm-4 smallPadding\">Price</div>\n" +
            "                                    <div class=\"col-md-8 col-sm-8 smallPadding\">\n" +
            "                                        <div class=\"row\">\n" +
            "                                            <div class=\"col-md-8 col-sm-8 smallPadding\">\n" +
            "                                                <div class=\"col-md-6 col-sm-6 smallPadding\">Qty</div>\n" +
            "                                                <div class=\"col-md-6 col-sm-6 smallPadding\">Taxable</div>\n" +
            "                                            </div>\n" +
            "                                        </div>\n" +
            "                                    </div>\n" +
            "                                </div>\n" +
            "                            </div>\n" +
            "                        </div>\n" +
            "                        <div class=\"row lineSpace autozoneBorder\" ng-repeat=\"part in vm.AutoZoneParts track by $index\">\n" +
            "                            <div class=\"col-md-5 col-sm-5 smallPadding\">\n" +
            "                                <div class=\"row\">\n" +
            "                                    <div class=\"col-md-4 col-sm-4 smallPadding\">\n" +
            "                                         <img src=\"{{ part.ImageURL }}\" ng-if=\"part.ImageURL\" style=\"width:60px\">\n" +
            "                                    </div>\n" +
            "                                    <div class=\"col-md-8 col-sm-8 smallPadding\">\n" +
            "                                        <span ng-bind=\"part.Description\"></span>\n" +
            "                                        <span>/</span>\n" +
            "                                        <span ng-bind=\"part.elementName\"></span>\n" +
            "                                        <div class=\"smallPadding\">Notes:</div>\n" +
            "                                        <div class=\"smallPadding\">\n" +
            "                                            <div ng-bind=\"part.Name\"></div>\n" +
            "                                            <div ng-bind=\"part.Notes\"></div>\n" +
            "                                        </div>\n" +
            "                                    </div>\n" +
            "                                </div>\n" +
            "                            </div>\n" +
            "                            <div class=\"col-md-2 col-sm-2 smallPadding\">\n" +
            "                                <div class=\"row\">\n" +
            "                                    <div class=\"smallPadding col-md-11 col-sm-11\">\n" +
            "                                        <span ng-bind=\"part.PartNumber\"></span>\n" +
            "                                    </div>\n" +
            "                                </div>\n" +
            "                            </div>\n" +
            "                            <div class=\"col-md-5 col-sm-5 smallPadding\">\n" +
            "                                <div class=\"row\">\n" +
            "                                    <div class=\"col-md-4 col-sm-4 smallPadding\" ng-bind=\"part.price | currency\"></div>\n" +
            "                                    <div class=\"col-md-8 col-sm-8 smallPadding\">\n" +
            "                                        <catalog-item-selected callback=\"vm.addAutoZonePart(part, quantity, isTaxable, buttonLabel)\" item=\"part\" tax-parts-default=\"vm.shop.taxPartsDefault\" row-index=\"{{$index}}\" />\n" +
            "                                    </div>\n" +
            "                                </div>\n" +
            "                            </div>\n" +
            "                        </div>\n" +
            "                    </div>\n" +
            "                </div>\n" +
            "            </div>\n" +
            "        </div>\n" +
            "    </div>\n" +
            "</ad-dialog-shell>");
    }]);
angular.module("adCatalogControl/partials/catalogItemSelected.html", []).run(["$templateCache", function ($templateCache) {
        $templateCache.put("adCatalogControl/partials/catalogItemSelected.html", "<div class=\"row\" ng-init=\"vm.isPartTaxableChecks[rowIndex]=vm.taxPartsDefault; vm.isLaborTaxableChecks[rowIndex]=vm.taxLaborDefault\">\n" +
            "        <div class=\"com-md-offset-8 com-sm-offset-8 col-md-8 col-sm-8 catalogItemSelected catalogOffset\">\n" +
            "            <div class=\"col-md-6 col-sm-6 catalogOffset\" ng-show=\"vm.rates == null\" scootch='{\"spaceClass\": \"contentBody\"}'>\n" +
            "                <input class=\"catalogQuantity\" type=\"number\" min=\"0\" max=\"100\" ng-model=\"vm.quantity\" ng-change=\"vm.updateQuantity()\" ng-hide=\"vm.hideQuantity\" floating />\n" +
            "            </div>\n" +
            "            <div class=\"col-md-6 col-sm-6 catalogOffset\" ng-show=\"vm.rates\">\n" +
            "                <select name=\"laborRates\" ng-init=\"vm.defaultRateIndex\" ng-model=\"vm.selectedLaborRateModel\" ng-hide=\"vm.selectedLaborRate == -1\" ng-change=\"vm.updateQuantity()\">\n" +
            "                    <option ng-repeat=\"rate in vm.rates track by $index\" ng-bind=\"rate | currency\" ng-selected=\"$index === vm.selectedLaborRate\"\n" +
            "                            ng-value=\"$index\"></option>\n" +
            "                </select>\n" +
            "            </div>\n" +
            "            <div class=\"col-md-6 col-sm-6 catalogOffset text-center\" ng-show=\"vm.rates == null\">\n" +
            "                <input type=\"checkbox\" ng-checked=\"vm.taxPartsDefault\" ng-model=\"vm.isPartTaxableChecks[rowIndex]\" ng-click=\"vm.updateTaxableFlag()\"/>\n" +
            "            </div>\n" +
            "            <div class=\"col-md-6 col-sm-6 catalogOffset text-center\" ng-show=\"vm.rates\">\n" +
            "                <input type=\"checkbox\" ng-checked=\"vm.taxLaborDefault\" ng-model=\"vm.isLaborTaxableChecks[rowIndex]\" ng-click=\"vm.updateTaxableFlag()\" />\n" +
            "            </div>\n" +
            "        </div>\n" +
            "        <div class=\"col-md-4 col-sm-4 catalogItemSelected catalogOffset\" ng-show=\"vm.rates == null\">\n" +
            "            <button class=\"btn btn-primary noPadding catalogBtn\" ng-click=\"vm.buttonClicked(item, vm.isPartTaxableChecks[rowIndex])\">{{vm.buttonLabel}}</button>\n" +
            "        </div>\n" +
            "        <div class=\"col-md-4 col-sm-4 catalogItemSelected catalogOffset\" ng-show=\"vm.rates\">\n" +
            "            <button class=\"btn btn-primary noPadding catalogBtn\" ng-click=\"vm.buttonClicked(item, vm.isLaborTaxableChecks[rowIndex])\">{{vm.buttonLabel}}</button>\n" +
            "        </div>\n" +
            "    </div>");
    }]);
angular.module("adCatalogControl/partials/selectMappedVehicleModal.html", []).run(["$templateCache", function ($templateCache) {
        $templateCache.put("adCatalogControl/partials/selectMappedVehicleModal.html", "<ad-dialog-shell class=\"mapVehicleDialog\" title=\"Select Vehicle\" buttons=\"vm.buttons\">\n" +
            "    <form name=\"form\">\n" +
            "        <fieldset class=\"group\" ng-if=\"vm.vehiclePartsDisplayFlag\">\n" +
            "           <!-- <div class=\"col-sm-12\">\n" +
            "                <div class=\"col-sm-12\" ng-repeat=\"desc in vm.acesVehicleDescriptions\">\n" +
            "                    <button class=\"vehicleButton\" ng-click=\"vm.vehicleSelected($index)\">{{desc}}</button>\n" +
            "                </div>\n" +
            "            </div>-->\n" +
            "            <ad-list items=\"vm.acesVehicleDescriptions\" on-select=\"vm.AutozoneVehicleSelected(item, $event, $index)\"></ad-list>\n" +
            "        </fieldset>\n" +
            "\n" +
            "        <fieldset class=\"group\" ng-if=\"!vm.vehiclePartsDisplayFlag\">\n" +
            "            <!--<div class=\"col-sm-12\">\n" +
            "                <div class=\"col-sm-12\" ng-repeat=\"desc in vm.acesVehiclePartUrls\">\n" +
            "                    <button class=\"vehicleButton\" ng-click=\"vm.vehicleSelected($index)\">{{vm.acesVehicleDescriptions[$index]}}</button>\n" +
            "                </div>\n" +
            "            </div>-->\n" +
            "            <ad-list items=\"vm.acesVehicleDescriptions\" on-select=\"vm.AutozoneVehicleSelected(item, $event, $index)\"></ad-list>\n" +
            "        </fieldset>\n" +
            "    </form>\n" +
            "</ad-dialog-shell>\n" +
            "");
    }]);
angular.module("login/LoginTemplate.html", []).run(["$templateCache", function ($templateCache) {
        $templateCache.put("login/LoginTemplate.html", "<div class=\"quote-component quote-login\">\n" +
            "    <form class=\"quote-body col-sm-12 col-md-6 col-lg-6\" ng-submit=\"vm.authenticate()\">\n" +
            "        <h1>Welcome to the ALLDATA Quote Preview.</h1>\n" +
            "        <H2>Please enter your ALLDATA login credentials:</H2>\n" +
            "        <input name=\"username\" ng-model=\"vm.credentials.username\" class=\"form-control col-sm-12 col-md-12 col-lg-12\" type=\"text\" required/>\n" +
            "        <input name=\"password\" ng-model=\"vm.credentials.password\" class=\"form-control col-sm-12 col-md-12 col-lg-12\" type=\"password\" required/>\n" +
            "        <div class=\"alert alert-info\" ng-if=\"vm.showBetaNotice\" style=\"clear:both\">\n" +
            "            <strong>ALLDATA® LLC BETA TEST AGREEMENT INSTRUCTIONS</strong>\n" +
            "            <p>The ALLDATA Beta team greatly appreciates your participation in testing and evaluating our products currently under development. Your feedback and critical review will help us to grow our program and create better products for you.</p>\n" +
            "            <p>By volunteering as a Field Tester, you agree to the following conditions and guidelines:</p>\n" +
            "            <ul>\n" +
            "                <li>The software being tested is a pre-release version, and may have errors and/or defects that are unknown at the start of the test. Should you encounter any error, please contact EG-Team-Mobile@alldata.com</li>\n" +
            "                <li>Please utilize the features and functions as you would normally use ALLDATA products in your daily operations.</li>\n" +
            "                <li>You will be asked to record any issues you may encounter, as well as your experience with the product. We request you note any issues, general comments or suggestions for improvement.</li>\n" +
            "                <li>During the Field Test, an ALLDATA representative may call and ask you questions regarding your test experience.</li>\n" +
            "            </ul>	\n" +
            "            <input type=\"checkbox\" ng-model=\"vm.betaAccepted\" required></input> I agree to the terms and conditions\n" +
            "        </div>\n" +
            "        <button type=\"submit\" class=\"btn btn-primary col-sm-12 col-md-12 col-lg-12\">Login</button>\n" +
            "    </form>\n" +
            "</div>");
    }]);
angular.module("quotes/partials/addCustomLaborModal.html", []).run(["$templateCache", function ($templateCache) {
        $templateCache.put("quotes/partials/addCustomLaborModal.html", "<ad-dialog-shell title=\"Add Custom Labor\" buttons=\"vm.buttons\">\n" +
            "    <form name=\"customLaborForm\" ng-init=\"vm.setFormScope(this)\" class=\"customLaborForm\">\n" +
            "        <div class=\"row\">\n" +
            "            <div class=\"col-sm-6\">\n" +
            "                <ad-form-input form-field-options=\"{\n" +
            "                    name:'duration',\n" +
            "                    label: 'Duration (Hours)',\n" +
            "                    type: 'number',\n" +
            "                    symbol: 'Hrs',\n" +
            "                    symbolLocation: 'right',\n" +
            "                    precision: 2\n" +
            "                }\" ng-model=\"vm.customLabor.quantity_Hours\" min=\"0\" ng-blur=\"vm.updateLaborExtended()\"></ad-form-input>\n" +
            "            </div>\n" +
            "            <div class=\"col-sm-6\">\n" +
            "                <ad-form-input form-field-options=\"{\n" +
            "                    name:'laborRate',\n" +
            "                    label: 'Labor Rate',\n" +
            "                    type: 'list',\n" +
            "                    listType: 'select',\n" +
            "                    itemValue: 'value',\n" +
            "                    itemLabel: 'label'\n" +
            "                }\" ad-form-field-items=\"vm.customLaborRates\" ng-model=\"vm.customLabor.price_Rate\" ng-change=\"vm.updateLaborExtended()\"></ad-form-input>\n" +
            "            </div>\n" +
            "        </div>\n" +
            "        <div class=\"row topPadding \">\n" +
            "            <div class=\"col-sm-6\">\n" +
            "                <label>\n" +
            "                    <input type=\"checkbox\" ng-model=\"vm.customLabor.taxable\"> Taxable</label>\n" +
            "            </div>\n" +
            "            <div class=\"col-sm-6\">\n" +
            "                <label>\n" +
            "                    <span>Extended:</span>\n" +
            "                    <span ng-bind=\"vm.customLabor.extended | currency\"></span>\n" +
            "                </label>\n" +
            "            </div>\n" +
            "        </div>\n" +
            "        <div class=\"row\">\n" +
            "            <div class=\"col-sm-12\">\n" +
            "                <ad-form-input form-field-options=\"{\n" +
            "                    name:'customLaborDescription',\n" +
            "                    label: 'Description',\n" +
            "                    type: 'textarea',\n" +
            "                    maxlength: 250,\n" +
            "                    rows: 5\n" +
            "                }\" ng-model=\"vm.customLabor.description\" scootch></ad-form-input>\n" +
            "            </div>\n" +
            "        </div>\n" +
            "    </form>\n" +
            "</ad-dialog-shell>");
    }]);
angular.module("quotes/partials/addCustomPartModal.html", []).run(["$templateCache", function ($templateCache) {
        $templateCache.put("quotes/partials/addCustomPartModal.html", "<ad-dialog-shell title=\"Add Custom Part\" buttons=\"vm.buttons\">\n" +
            "    <form name=\"customPartForm\" class=\"customPartForm\" ng-init=\"vm.setFormScope(this)\">\n" +
            "        <div class=\"row\">\n" +
            "            <div class=\"col-sm-6\">\n" +
            "                <label style=\"margin-top:0px\">Part Number</label>\n" +
            "                <input class=\"form-control\" type=\"text\" ng-model=\"vm.customPart.partNumber\" />\n" +
            "            </div>\n" +
            "            <div class=\"col-sm-3\">\n" +
            "                <ad-form-input form-field-options=\"{\n" +
            "                    name:'quantity',\n" +
            "                    label: 'Quantity',\n" +
            "                    type: 'number',\n" +
            "                    precision: 0\n" +
            "                }\" ng-model=\"vm.customPart.quantity_Hours\" min=\"0\" ng-blur=\"vm.updatePartExtended()\"></ad-form-input>\n" +
            "            </div>\n" +
            "            <div class=\"col-sm-3\">\n" +
            "                <ad-form-input form-field-options=\"{\n" +
            "                    name:'price',\n" +
            "                    label: 'Price',\n" +
            "                    type: 'number',\n" +
            "                    symbol: '$',\n" +
            "                    precision: 2\n" +
            "                }\" ng-model=\"vm.customPart.price_Rate\" min=\"0\" ng-blur=\"vm.updatePartExtended()\"></ad-form-input>\n" +
            "            </div>\n" +
            "        </div>\n" +
            "        <div class=\"row topPadding \">\n" +
            "            <div class=\"col-sm-6\">\n" +
            "                <label>\n" +
            "                    <input type=\"checkbox\" ng-model=\"vm.customPart.taxable\"> Taxable</label>\n" +
            "            </div>\n" +
            "            <div class=\"col-sm-6\">\n" +
            "                <label>Extended: {{ vm.customPart.extended | currency}}</label>\n" +
            "            </div>\n" +
            "        </div>\n" +
            "        <div class=\"row\">\n" +
            "            <div class=\"col-sm-12\">\n" +
            "                <ad-form-input form-field-options=\"{\n" +
            "                    name:'customPartDescription',\n" +
            "                    label: 'Description',\n" +
            "                    type: 'textarea',\n" +
            "                    maxlength: 250,\n" +
            "                    rows: 5\n" +
            "                }\" ng-model=\"vm.customPart.description\" scootch></ad-form-input>\n" +
            "            </div>\n" +
            "        </div>\n" +
            "    </form>\n" +
            "</ad-dialog-shell>");
    }]);
angular.module("quotes/partials/addFlatFeeModal.html", []).run(["$templateCache", function ($templateCache) {
        $templateCache.put("quotes/partials/addFlatFeeModal.html", "<ad-dialog-shell title=\"Add Flat Fee\" buttons=\"vm.buttons\">\n" +
            "    <form name=\"flatFeeForm\" ng-init=\"vm.setFormScope(this)\">\n" +
            "    <!--<div class=\"row\">\n" +
            "        <div class=\"col-sm-12\">\n" +
            "            <label>Fee Name</label>\n" +
            "            <input class=\"form-control\" type=\"text\" ng-model=\"vm.flatFee.name\" />\n" +
            "        </div>\n" +
            "    </div>-->\n" +
            "    <div class=\"row\">\n" +
            "        <div class=\"col-sm-12\">\n" +
            "            <ad-form-input form-field-options=\"{\n" +
            "                    name:'price',\n" +
            "                    label: 'Price',\n" +
            "                    type: 'number',\n" +
            "                    symbol: '$',\n" +
            "                    precision: 2\n" +
            "                }\" ng-model=\"vm.flatFee.price_Rate\" min=\"0\" scootch></ad-form-input>\n" +
            "        </div>\n" +
            "    </div>\n" +
            "    <div class=\"row\">\n" +
            "        <div class=\"col-sm-12\">\n" +
            "            <ad-form-input form-field-options=\"{\n" +
            "                    name:'flatFeeDescription',\n" +
            "                    label: 'Description',\n" +
            "                    type: 'textarea',\n" +
            "                    maxlength: 250,\n" +
            "                    rows: 5\n" +
            "                }\" ng-model=\"vm.flatFee.description\" scootch></ad-form-input>\n" +
            "        </div>\n" +
            "    </div>\n" +
            "    </form>\n" +
            "</ad-dialog-shell>");
    }]);
angular.module("quotes/partials/addNoteModal.html", []).run(["$templateCache", function ($templateCache) {
        $templateCache.put("quotes/partials/addNoteModal.html", "<ad-dialog-shell title=\"Add Note\" buttons=\"vm.buttons\">\n" +
            "   <form name=\"form\">\n" +
            "       <ad-form-input form-field-options=\"{\n" +
            "                    name:'noteDescription',\n" +
            "                    label: 'Description',\n" +
            "                    type: 'textarea',\n" +
            "                    maxlength: 250,\n" +
            "                    rows: 10\n" +
            "                }\" ng-model=\"vm.note\"></ad-form-input>\n" +
            "       </form>\n" +
            "</ad-dialog-shell>\n" +
            "");
    }]);
angular.module("quotes/partials/addPartsLaborModal.html", []).run(["$templateCache", function ($templateCache) {
        $templateCache.put("quotes/partials/addPartsLaborModal.html", "<ad-dialog-shell title=\"Add Parts and Labor\" buttons=\"vm.buttons\">\n" +
            "    <form name=\"form\">\n" +
            "        <div class=\"row\">\n" +
            "                <div class=\"col-sm-12\">\n" +
            "                    <label>Labor Name</label>\n" +
            "                    <input class=\"form-control\" type=\"text\" ng-model=\"vm.customLabor.name\" />\n" +
            "                </div>\n" +
            "            </div>\n" +
            "            <div class=\"row\">\n" +
            "                <div class=\"col-sm-6\">\n" +
            "                    <label>Labor Rate</label>\n" +
            "                    <input class=\"form-control\" type=\"number\" ng-model=\"vm.customLabor.rate\" ng-blur=\"vm.updateLaborExtended()\" floating />\n" +
            "                </div>\n" +
            "                <div class=\"col-sm-6\">\n" +
            "                    <label>Duration (Hours)</label>\n" +
            "                    <input class=\"form-control\" type=\"number\" ng-model=\"vm.customLabor.duration\" ng-blur=\"vm.updateLaborExtended()\"\n" +
            "                    floating />\n" +
            "                </div>\n" +
            "            </div>\n" +
            "            <div class=\"row topPadding \">\n" +
            "                <div class=\"col-sm-6\">\n" +
            "                    <label>\n" +
            "                        <input type=\"checkbox\" ng-model=\"vm.customLabor.taxable\" ng-change=\"vm.updateLaborExtended()\">Taxable</label>\n" +
            "                </div>\n" +
            "                <div class=\"col-sm-6\">\n" +
            "                    <label>\n" +
            "                        <span>Extended:</span>\n" +
            "                        <span ng-bind=\"vm.customLabor.extended | currency\"></span>\n" +
            "                    </label>\n" +
            "                </div>\n" +
            "            </div>\n" +
            "            <div class=\"row\">\n" +
            "                <div class=\"col-sm-12\">\n" +
            "                    <label>Description</label>\n" +
            "                    <textarea class=\"form-control\" rows=\"5\" ng-model=\"vm.customLabor.description\" />\n" +
            "                </div>\n" +
            "            </div>\n" +
            "    </form>\n" +
            "</ad-dialog-shell>");
    }]);
angular.module("quotes/partials/additionalInfoModal.html", []).run(["$templateCache", function ($templateCache) {
        $templateCache.put("quotes/partials/additionalInfoModal.html", "<ad-dialog-shell title=\"Customer Information\" buttons=\"vm.buttons\">\n" +
            "\n" +
            "    <form name=\"customerInfoForm\" ng-init=\"vm.setFormScope(this)\">\n" +
            "        <fieldset class=\"group\">\n" +
            "            <legend>Contact</legend>\n" +
            "            <div class=\"row\">\n" +
            "            <div class=\"col-sm-12 col-md-6 col-lg-6\">\n" +
            "                    <ad-form-input form-field-options=\"{\n" +
            "                    name:'firstName',\n" +
            "                    label: 'First Name',\n" +
            "                    updateOn: 'keyup',\n" +
            "                    debounce: 1000\n" +
            "                }\" ng-model=\"vm.quote.customerFirstName\" scootch ng-keyup=\"vm.firstNameChange()\"></ad-form-input>\n" +
            "            </div>\n" +
            "            <div class=\"col-sm-12 col-md-6 col-lg-6\">\n" +
            "                    <ad-form-input form-field-options=\"{\n" +
            "                    name:'lastName',\n" +
            "                    label: 'Last Name',\n" +
            "                    updateOn: 'keyup',\n" +
            "                    debounce: 1000\n" +
            "                }\" ng-model=\"vm.quote.customerLastName\" scootch ng-keyup=\"vm.lastNameChange()\"></ad-form-input>\n" +
            "            </div>\n" +
            "            </div>\n" +
            "            <div class=\"row\">\n" +
            "            <div class=\"col-sm-12 col-md-6 col-lg-6\">\n" +
            "                    <ad-form-input form-field-options=\"{\n" +
            "                    name:'phone',\n" +
            "                    label: 'Phone Number',\n" +
            "                    notes: 'example: (999) 999-9999',\n" +
            "                    type: 'tel',\n" +
            "                    updateOn: 'keyup',\n" +
            "                    allowInvalid: true,\n" +
            "                    debounce: 1000\n" +
            "                }\" ng-model=\"vm.quote.customerPhoneNumber\" scootch ng-keyup=\"vm.phoneChange()\"></ad-form-input>\n" +
            "            </div>\n" +
            "            <div class=\"col-sm-12 col-md-6 col-lg-6\">\n" +
            "                    <ad-form-input form-field-options=\"{\n" +
            "                    name:'email',\n" +
            "                    label: 'Email',\n" +
            "                    type: 'email',\n" +
            "                    notes: 'example: john.doe@example.com'\n" +
            "                }\" ng-model=\"vm.quote.customerEmail\" scootch></ad-form-input>\n" +
            "            </div>\n" +
            "            </div>\n" +
            "        </fieldset>\n" +
            "        <fieldset class=\"group\">\n" +
            "            <legend>Address</legend>\n" +
            "\n" +
            "            <div class=\"col-sm-12 col-md-12 col-lg-12\">\n" +
            "                <div class=\"form-group\" ng-class=\"{focused: isFocused()}\" scootch>\n" +
            "                    <label for=\"address1\">\n" +
            "                        <span>Street Address 1</span>\n" +
            "                    </label>\n" +
            "                    <input name=\"address1\" class=\"form-control\" ng-model=\"vm.quote.customerStreetAddress1\" />\n" +
            "                </div>\n" +
            "            </div>\n" +
            "            <div class=\"col-sm-12 col-md-12 col-lg-12\">\n" +
            "                <div class=\"form-group\" ng-class=\"{focused: isFocused()}\" scootch>\n" +
            "                    <label for=\"address2\">\n" +
            "                        <span>Street Address 2</span>\n" +
            "                    </label>\n" +
            "                    <input name=\"address2\" class=\"form-control\" ng-model=\"vm.quote.customerStreetAddress2\" />\n" +
            "                </div>\n" +
            "            </div>\n" +
            "            <div class=\"col-sm-12 col-md-6 col-lg-6\">\n" +
            "                <div class=\"form-group\" ng-class=\"{focused: isFocused()}\" scootch>\n" +
            "                    <label for=\"city\">\n" +
            "                        <span>City</span>\n" +
            "                    </label>\n" +
            "                    <input name=\"city\" class=\"form-control\" ng-model=\"vm.quote.customerCity\" />\n" +
            "                </div>\n" +
            "            </div>\n" +
            "            <div class=\"col-sm-12 col-md-6 col-lg-6\">\n" +
            "                <div class=\"form-group\" ng-class=\"{focused: isFocused()}\" scootch>\n" +
            "                    <label for=\"city\">\n" +
            "                        <span>State</span>\n" +
            "                    </label>\n" +
            "                    <input name=\"state\" class=\"form-control\" ng-model=\"vm.quote.customerState\" />\n" +
            "                </div>\n" +
            "            </div>\n" +
            "            <div class=\"col-sm-12 col-md-6 col-lg-6\">\n" +
            "                <div class=\"form-group\" ng-class=\"{focused: isFocused()}\" scootch>\n" +
            "                    <label for=\"country\">\n" +
            "                        <span>Country</span>\n" +
            "                    </label>\n" +
            "                    <input name=\"country\" class=\"form-control\" ng-model=\"vm.quote.customerCountry\" />\n" +
            "                </div>\n" +
            "            </div>\n" +
            "            <div class=\"col-sm-12 col-md-6 col-lg-6\">\n" +
            "                <div class=\"form-group\" ng-class=\"{focused: isFocused()}\" scootch>\n" +
            "                    <label for=\"postalCode\">\n" +
            "                        <span>Postal Code</span>\n" +
            "                    </label>\n" +
            "                    <input name=\"postalCode\" class=\"form-control\" ng-model=\"vm.quote.customerZipcode\" ng-pattern=\"/^(\\d{5}-\\d{4}|\\d{5}|[ABCEGHJKLMNPRSTVXY abceghjklmnprstvxy]\\d[ABCEGHJKLMNPRSTVWXYZ abceghjklmnprstvxyz]( )?\\d[ABCEGHJKLMNPRSTVWXYZ abceghjklmnprstvwxyz]\\d)$/\" />\n" +
            "                    <div class=\"validation-error\">\n" +
            "                        <span ng-show=\"customerInfoForm.postalCode.$invalid\">Invalid Postal Code</span>\n" +
            "                    </div>\n" +
            "                </div>\n" +
            "            </div>\n" +
            "        </fieldset>\n" +
            "    </form>\n" +
            "\n" +
            "</ad-dialog-shell>");
    }]);
angular.module("quotes/partials/customerFlyout.html", []).run(["$templateCache", function ($templateCache) {
        $templateCache.put("quotes/partials/customerFlyout.html", "<div class=\"customerInfoFlyout\">\n" +
            "    <div ng-show=\"vm.isLoading\" class=\"spinner\">\n" +
            "        <div class=\"rect1\"></div><div class=\"rect2\"></div><div class=\"rect3\"></div><div class=\"rect4\"></div><div class=\"rect5\"></div>\n" +
            "    </div>\n" +
            "    <div ng-show=\"!vm.isLoading\">\n" +
            "        <h2>{{vm.customerInfo.length}} Records Found</h2>\n" +
            "        <div>\n" +
            "            <a class=\"list-group-item\" ng-repeat=\"info in vm.customerInfo\" ng-click=\"vm.selectInformation(info)\">{{info.firstName}} {{info.lastName}}\n" +
            "                <br />\n" +
            "                {{info.addressLine1}}\n" +
            "            </a>\n" +
            "        </div>\n" +
            "    </div>\n" +
            "</div>\n" +
            "");
    }]);
angular.module("quotes/partials/customerInfoModal.html", []).run(["$templateCache", function ($templateCache) {
        $templateCache.put("quotes/partials/customerInfoModal.html", "<ad-dialog-shell title=\"Customer Information Found\" buttons=\"vm.buttons\">\n" +
            "<div class=\"customerInfoFlyout\">\n" +
            "<div ng-show=\"vm.isLoading\" class=\"spinner\">\n" +
            "    <div class=\"rect1\"></div><div class=\"rect2\"></div><div class=\"rect3\"></div><div class=\"rect4\"></div><div class=\"rect5\"></div>\n" +
            "</div>\n" +
            "<div ng-show=\"!vm.isLoading && vm.ShellService.molCustomerData.length\">\n" +
            "    <span>We found {{vm.ShellService.molCustomerData.length}} matching customer records for the selected vehicle. Select one to populate the quote with\n" +
            "        that customer's information or hit cancel to skip this step.</span>\n" +
            "    <div class=\"customerInfoDiv\">\n" +
            "        <div class=\"list-group-item\" ng-repeat=\"info in vm.ShellService.molCustomerData\" ng-click=\"vm.selectCustomer(info)\"><!--ng-repeat=\"info in vm.customerInfo.data\"-->\n" +
            "            <div>{{info.firstName}} </div>\n" +
            "            <div>{{info.addressLine1}} {{info.addressLine2}} {{info.state}} {{info.cellPhone}} </div>\n" +
            "        </div>\n" +
            "    </div>\n" +
            "</div>\n" +
            "    <div ng-show=\"!vm.isLoading && !vm.ShellService.molCustomerData.length\">\n" +
            "        <span>No record found</span>\n" +
            "    </div>\n" +
            "</div>\n" +
            "</ad-dialog-shell>\n" +
            "");
    }]);
angular.module("quotes/partials/emailQuoteModal.html", []).run(["$templateCache", function ($templateCache) {
        $templateCache.put("quotes/partials/emailQuoteModal.html", "<ad-dialog-shell title=\"Email Quote\" buttons=\"vm.buttons\">\n" +
            "    <form name=\"form\" ng-init=\"vm.setFormScope(this)\" novalidate>\n" +
            "        <fieldset class=\"group\">\n" +
            "            <div class=\"col-sm-12\">\n" +
            "           \n" +
            "                <ad-form-input form-field-options=\"{\n" +
            "                    name:'email',\n" +
            "                    label: 'Email Address',\n" +
            "                    type: 'email',\n" +
            "                    notes: 'example: john.doe@example.com'\n" +
            "                }\" ng-model=\"vm.emailModel.address\" required></ad-form-input>\n" +
            "            </div>\n" +
            "            <div class=\"col-sm-12\">\n" +
            "                <div class=\"form-group\">\n" +
            "                    <label for=\"message\">\n" +
            "                        <span>Message</span>\n" +
            "                    </label>\n" +
            "                    <textarea rows=\"5\" class=\"form-control messageField\" ng-model=\"vm.emailModel.message\"></textarea>\n" +
            "                </div>\n" +
            "            </div>\n" +
            "        </fieldset>\n" +
            "    </form>\n" +
            "</ad-dialog-shell>");
    }]);
angular.module("quotes/partials/printPreviewTemplate.html", []).run(["$templateCache", function ($templateCache) {
        $templateCache.put("quotes/partials/printPreviewTemplate.html", "<ad-dialog-shell title=\"Print Preview\" buttons=\"buttons\">\n" +
            "    <pdf-viewer host=\"host\" path=\"path\" url=\"getUrl(vm.quote)\" callback=\"pdfViewerCallback()\"></pdf-viewer>\n" +
            "</ad-dialog-shell>");
    }]);
angular.module("quotes/partials/quoteBannerDisplay.html", []).run(["$templateCache", function ($templateCache) {
        $templateCache.put("quotes/partials/quoteBannerDisplay.html", "<div class=\"bannerBand bg-primary\">\n" +
            "    <div class=\"row\">\n" +
            "        <div class=\"col-md-2\">\n" +
            "            <h1 ng-show=\"vm.quote.quoteNumber > 0\">\n" +
            "                Quote # {{ vm.quote.quoteNumber }}\n" +
            "            </h1>\n" +
            "\n" +
            "            <h1 ng-hide=\"vm.quote.quoteNumber > 0\">\n" +
            "                Quote\n" +
            "            </h1>\n" +
            "        </div>\n" +
            "\n" +
            "        <div class=\"col-md-offset-8 col-md-2 newQuote\">\n" +
            "            <button class=\"btn btn-secondary darkText alignRight moreBottomMargin\" ng-click=\"vm.newQuote()\">New Quote</button>\n" +
            "        </div>\n" +
            "    </div>\n" +
            "</div>");
    }]);
angular.module("quotes/partials/quoteDisplay.html", []).run(["$templateCache", function ($templateCache) {
        $templateCache.put("quotes/partials/quoteDisplay.html", "<div>\n" +
            "    <div class=\"quote\">\n" +
            "        <form name=\"quoteLineForm\">\n" +
            "            <div ng-show=\"quote != undefined\">\n" +
            "                <div class=\"noContent\" ng-show=\"quote.datalines.length === 0 && readOnly === false\">\n" +
            "                    <h3>This is a new quote:</h3>\n" +
            "                    <h4>Add items to get started.</h4>\n" +
            "                </div>\n" +
            "                \n" +
            "                <div class=\"noContent\" ng-show=\"quote.datalines.length === 0 && readOnly === true\">\n" +
            "                    <h3>No items in quote.</h3>\n" +
            "                </div>\n" +
            "                \n" +
            "                <div class=\"content quoteLines\" ng-show=\"quote.datalines.length > 0\">\n" +
            "                    <table class=\"table\" ng-class=\"{'table-hover': !readOnly, 'readOnly': readOnly}\">\n" +
            "                        <thead>\n" +
            "                            <tr>\n" +
            "                                <th width=\"75px\">Type</th>\n" +
            "                                <th>Description</th>\n" +
            "                                <th>Part #</th>\n" +
            "                                <th width=\"125px\" style=\"text-align:right\">Qty</th>\n" +
            "                                <th width=\"125px\" style=\"text-align:right\">Price</th>\n" +
            "                                <th width=\"75px\" style=\"text-align:right\">Extended</th>\n" +
            "                                <th width=\"75px\">Taxable</th>\n" +
            "                                <th width=\"50px\" class=\"deleteQuoteLine\"></th>\n" +
            "                            </tr>\n" +
            "                        </thead>             \n" +
            "                        <tbody>\n" +
            "                            <tr ng-repeat=\"quoteLine in quote.datalines\" ng-class=\"{'selected': quoteLine === $parent.selectedRow}\" \n" +
            "                                ng-click=\"!readOnly && $parent.selectRow(quoteLine, $event)\" ng-include=\"$parent.lineTemplate(quoteLine)\"\n" +
            "                                scootch='{\"scrollClass\": \"viewContent\"}'>\n" +
            "                            </tr>\n" +
            "                        </tbody>       \n" +
            "                    </table>\n" +
            "                </div>\n" +
            "            \n" +
            "                <div class=\"content row\" ng-show=\"quote.datalines.length > 0\">\n" +
            "                    <div ng-class=\"{'col-md-offset-6 col-md-6':readOnly, 'col-md-offset-8 col-md-4': !readOnly}\">\n" +
            "                        <div class=\"totals\">\n" +
            "                            <div class=\"subTotal\">\n" +
            "                                <span class=\"amount\" ng-bind=\"quote.totalSub | currency\"></span>\n" +
            "                                <span class=\"label\">Subtotal</span>\n" +
            "                            </div>\n" +
            "                            <div class=\"taxTotal\">\n" +
            "                                <span class=\"amount\" ng-bind=\"quote.totalTax | currency\"></span> \n" +
            "                                <span class=\"label\">Tax Total</span>\n" +
            "                            </div>\n" +
            "                            <div class=\"grandTotal\">\n" +
            "                                <span class=\"amount\" ng-bind=\"quote.total | currency\"></span> \n" +
            "                                <span class=\"label\">\n" +
            "                                    Total\n" +
            "                                    <span dropdown class=\"dropup\">\n" +
            "                                        <a dropdown-toggle class=\"pull-right\">\n" +
            "                                            <i class=\"appIcon-info\"></i>\n" +
            "                                        </a>\n" +
            "                                        <div class=\"dropdown-menu\">\n" +
            "                                            <div class=\"subTotal\">\n" +
            "                                                <span class=\"amount\" ng-bind=\"quote.totalParts | currency\"></span>\n" +
            "                                                <span class=\"label\">Parts Total</span>\n" +
            "                                            </div>\n" +
            "                                            <div class=\"subTotal\">\n" +
            "                                                <span class=\"amount\" ng-bind=\"quote.totalLabor | currency\"></span>\n" +
            "                                                <span class=\"label\">Labor Total</span>\n" +
            "                                            </div>\n" +
            "                                            <div class=\"subTotal\">\n" +
            "                                                <span class=\"amount\" ng-bind=\"(quote.totalSub - quote.totalLabor - quote.totalParts ) | currency\"></span>\n" +
            "                                                <span class=\"label\">Others Total</span>\n" +
            "                                            </div>\n" +
            "                                            \n" +
            "                                            <div class=\"subTotal\">\n" +
            "                                                <span class=\"amount\" ng-bind=\"quote.totalPartsTax | currency\"></span>\n" +
            "                                                <span class=\"label\">Parts Tax</span>\n" +
            "                                            </div>\n" +
            "                                            <div class=\"taxTotal\">\n" +
            "                                                <span class=\"amount\" ng-bind=\"quote.totalLaborTax | currency\"></span> \n" +
            "                                                <span class=\"label\">Labor Tax</span>\n" +
            "                                            </div>\n" +
            "                                        </div>\n" +
            "                                    </span>\n" +
            "                                </span>\n" +
            "                            </div>\n" +
            "                        </div>\n" +
            "                    </div>\n" +
            "                </div>\n" +
            "            </div>\n" +
            "        </form>\n" +
            "    </div>\n" +
            "\n" +
            "    <!--<print-preview vm=\"vm\"></print-preview>-->\n" +
            "    \n" +
            "</div>");
    }]);
angular.module("quotes/partials/quoteHeader.html", []).run(["$templateCache", function ($templateCache) {
        $templateCache.put("quotes/partials/quoteHeader.html", "<div class=\"quoteHeader header\">\n" +
            "    <ad-rich-content-button-group>\n" +
            "        <ad-rich-content-button link-type=\"vm.headerButtonType\" icon=\"'appIcon-car'\" ng-click=\"vm.changeVehicle();vm.ShellService.sendAnalyticsEvents('Vehicle Info - Tapped','Quote from Mobile','Vehicle Info option is tapped.')\" >\n" +
            "            <h3 ng-show=\"!vm.quote.vehicleDescription && !vm.readOnly\">Vehicle Info</h3>\n" +
            "            <h4 ng-show=\"!vm.quote.vehicleDescription && vm.readOnly\">No Vehicle Info</h4>\n" +
            "            <h4 class=\"top topDescrp\" ng-show=\"vm.quote.vehicleDescription\">{{vm.quote.vehicleDescription}}</h4>\n" +
            "            <div ng-show=\"vm.quote.vehicleVIN\">{{vm.quote.vehicleVIN}}</div>\n" +
            "        </ad-rich-content-button>\n" +
            "        <ad-rich-content-button link-type=\"vm.headerButtonType\" icon=\"'appIcon-user'\" ng-click=\"vm.showAdditionalInformation(); vm.ShellService.sendAnalyticsEvents('Customer Details - Tapped','Quote from Mobile','Customer details option is tapped.')\">\n" +
            "            <h3 ng-show=\"!(vm.quote.customerFirstName || vm.quote.customerLastName || vm.quote.customerPhoneNumber || vm.quote.customerEmail) && !vm.readOnly\">Customer Info</h3>  \n" +
            "            <h4 ng-show=\"!(vm.quote.customerFirstName || vm.quote.customerLastName || vm.quote.customerPhoneNumber || vm.quote.customerEmail) && vm.readOnly\">No Customer Info</h4>\n" +
            "            <h4 class=\"top\" ng-show=\"vm.quote.customerFirstName || vm.quote.customerLastName\">{{vm.quote.customerFirstName}} {{vm.quote.customerLastName}}</h4>\n" +
            "            <div class=\"sub\" ng-show=\"vm.quote.customerPhoneNumber\">{{vm.quote.customerPhoneNumber}}</div>\n" +
            "        </ad-rich-content-button>\n" +
            "        <ad-rich-content-button ng-show=\"!vm.readOnly\" link-type=\"'action'\" icon=\"'appIcon-dollar'\" \n" +
            "            ng-click=\"vm.showAddItems = !vm.showAddItems\"  ad-flyout-template=\"'addItems.html'\" ad-flyout-placement=\"bottom\" ad-flyout-is-open=\"!vm.readOnly && vm.showAddItems\">\n" +
            "            <h3 >Add Item</h3>  \n" +
            "            <script type=\"text/ng-template\" id=\"addItems.html\">\n" +
            "                <ul class=\"addItemsOptions\">\n" +
            "                    <li ng-click=\"vm.addPartsLabor(); vm.showAddItems = false;\">Catalog - Parts & Labor</li>\n" +
            "                    <li ng-click=\"vm.addCustomPart(); vm.showAddItems = false; vm.ShellService.sendAnalyticsEvents('Custom Part - Tapped','Quote from Mobile','Custom Part is tapped from the Add Item')\">Custom Part</li>\n" +
            "                    <li ng-click=\"vm.addCustomLabor(); vm.showAddItems = false; vm.ShellService.sendAnalyticsEvents('Custom Labor - Tapped','Quote from Mobile','Custom Labor is tapped from the Add Item')\">Custom Labor</li>\n" +
            "                    <li ng-click=\"vm.addFlatFee(); vm.showAddItems = false; vm.ShellService.sendAnalyticsEvents('Flat Fee - Tapped','Quote from Mobile','Flat Fee is tapped from the Add Item')\">Flat Fee</li>\n" +
            "                    <li ng-click=\"vm.addNote(); vm.showAddItems = false; vm.ShellService.sendAnalyticsEvents('Note - Tapped','Quote from Mobile','Note is tapped from the Add Item')\">Note</li>\n" +
            "                </ul>\n" +
            "            </script>\n" +
            "        </ad-rich-content-button>\n" +
            "    </ad-rich-content-button-group>\n" +
            "    \n" +
            "\n" +
            "</div>");
    }]);
angular.module("quotes/partials/quoteHome.html", []).run(["$templateCache", function ($templateCache) {
        $templateCache.put("quotes/partials/quoteHome.html", "<ad-view header-size=\"'small'\" class=\"fill\">\n" +
            "    <div transclude-to=\"header\">\n" +
            "        <h1>\n" +
            "            Quote <span ng-show=\"vm.quote.quoteNumber > 0\"># {{ vm.quote.quoteNumber }} </span>\n" +
            "            <span class=\"badge\" ng-show=\"vm.ShellService.checkDateExpiration(vm.quote.lastUpdateDate,vm.quote)\"> Expired </span>\n" +
            "        </h1>\n" +
            "        <div class=\"actions\">\n" +
            "            <div ng-if=\"vm.ShellService.needsSave\">\n" +
            "                <a class=\"btn btn-link\" ng-click=\"vm.cancelQuote()\">cancel</a>\n" +
            "                <a class=\"btn btn-primary\" ng-click=\"vm.saveQuote();vm.ShellService.sendAnalyticsEvents('Quote - Saved','Quote from Mobile','Quote is saved.')\">Save</a>\n" +
            "            </div>\n" +
            "            <div ng-if=\"!vm.ShellService.needsSave\">\n" +
            "                <a class=\"btn btn-primary pull-right\" ng-click=\"vm.cancelQuote()\">Done</a>\n" +
            "                <ad-action-bar actions=\"vm.quoteActions\"></ad-action-bar>\n" +
            "            </div>\n" +
            "        </div>\n" +
            "    </div>\n" +
            "    <div transclude-to=\"content\">\n" +
            "        <ad-quote-header></ad-quote-header>\n" +
            "        <ad-quote quote=\"vm.quote\" read-only=\"false\" ></ad-quote>\n" +
            "    </div>\n" +
            "</ad-view>");
    }]);
angular.module("quotes/partials/quoteLineTemplates/customLabor.html", []).run(["$templateCache", function ($templateCache) {
        $templateCache.put("quotes/partials/quoteLineTemplates/customLabor.html", "<td>Labor</td>\n" +
            "<td>\n" +
            "	<span ng-bind=\"quoteLine.displayDescription\"  ng-show=\"quoteLine !== $parent.selectedRow\"></span>\n" +
            "	<ad-form-input ng-model=\"quoteLine.description\" form-field-options=\"{name:'description'}\" ng-show=\"quoteLine === $parent.selectedRow\" autofocus></ad-form-input>\n" +
            "<td><span ng-bind=\"quoteLine.displayPartSkill\"></span></td>\n" +
            "<td style=\"text-align:right\">\n" +
            "	<span ng-bind=\"quoteLine.displayQuantity\" ng-show=\"quoteLine !== $parent.selectedRow\"></span>\n" +
            "	<ad-form-input ng-model=\"quoteLine.quantity_Hours\" form-field-options=\"{ type: 'number', name:'quantity'}\" ng-show=\"quoteLine === $parent.selectedRow\" ></ad-form-input></td>\n" +
            "<td style=\"text-align:right\">\n" +
            "	<span ng-bind=\"quoteLine.displayPrice\" ng-show=\"quoteLine !== $parent.selectedRow\"></span>\n" +
            "	<!--<ad-form-input ng-model=\"quoteLine.price_Rate\" form-field-options=\"{ type: 'number', name:'price', symbol: '$'}\" ng-show=\"quoteLine === $parent.selectedRow\" ></ad-form-input></td>-->\n" +
            "	<ad-form-input ng-show=\"quoteLine === $parent.selectedRow\" form-field-options=\"{\n" +
            "                            name:'defaultHourlyRate',\n" +
            "                            type: 'list',\n" +
            "                            listType: 'select',\n" +
            "                            itemLabel: 'label',\n" +
            "                            itemValue: 'value'\n" +
            "                        }\" ad-form-field-items=\"$parent.laborRates\" \n" +
            "                        ng-model=\"quoteLine.price_Rate\" required></ad-form-input>\n" +
            "<td style=\"text-align:right\"><span ng-bind=\"quoteLine.displayExtended\"></span></td>\n" +
            "<td style=\"text-align:center\">\n" +
            "	<span ng-if=\"quoteLine.taxable && quoteLine !== $parent.selectedRow\">&#10004;</span>\n" +
            "	<input type=\"checkbox\" ng-model=\"quoteLine.taxable\"  ng-show=\"quoteLine === $parent.selectedRow\"></input>\n" +
            "</td>\n" +
            "<td class=\"deleteQuoteLine\"><i class=\"appIcon-close\"></i></td>");
    }]);
angular.module("quotes/partials/quoteLineTemplates/customPart.html", []).run(["$templateCache", function ($templateCache) {
        $templateCache.put("quotes/partials/quoteLineTemplates/customPart.html", "<td>Part</td>\n" +
            "<td>\n" +
            "	<span ng-bind=\"quoteLine.displayDescription\"  ng-show=\"quoteLine !== $parent.selectedRow\"></span>\n" +
            "	<ad-form-input ng-model=\"quoteLine.description\" form-field-options=\"{name:'description'}\" ng-show=\"quoteLine === $parent.selectedRow\" autofocus></ad-form-input>\n" +
            "<td>\n" +
            "	<span ng-bind=\"quoteLine.partNumber\" ng-show=\"quoteLine !== $parent.selectedRow\"></span>\n" +
            "	<ad-form-input ng-model=\"quoteLine.partNumber\" form-field-options=\"{name:'skill'}\" ng-show=\"quoteLine === $parent.selectedRow\" autofocus></ad-form-input></td>\n" +
            "<td style=\"text-align:right\">\n" +
            "	<span ng-bind=\"quoteLine.displayQuantity\" ng-show=\"quoteLine !== $parent.selectedRow\"></span>\n" +
            "	<ad-form-input ng-model=\"quoteLine.quantity_Hours\" form-field-options=\"{ type: 'number', name:'quantity'}\" ng-show=\"quoteLine === $parent.selectedRow\" ></ad-form-input></td>\n" +
            "<td style=\"text-align:right\">\n" +
            "	<span ng-bind=\"quoteLine.displayPrice\" ng-show=\"quoteLine !== $parent.selectedRow\"></span>\n" +
            "	<ad-form-input ng-model=\"quoteLine.price_Rate\" form-field-options=\"{ type: 'number', name:'price', symbol: '$'}\" ng-show=\"quoteLine === $parent.selectedRow\" ></ad-form-input></td>\n" +
            "<td style=\"text-align:right\"><span ng-bind=\"quoteLine.displayExtended\"></span></td>\n" +
            "<td style=\"text-align:center\">\n" +
            "	<span ng-if=\"quoteLine.taxable && quoteLine !== $parent.selectedRow\">&#10004;</span>\n" +
            "	<input type=\"checkbox\" ng-model=\"quoteLine.taxable\"  ng-show=\"quoteLine === $parent.selectedRow\"></input>\n" +
            "</td>\n" +
            "<td class=\"deleteQuoteLine\"><i class=\"appIcon-close\"></i></td>");
    }]);
angular.module("quotes/partials/quoteLineTemplates/default.html", []).run(["$templateCache", function ($templateCache) {
        $templateCache.put("quotes/partials/quoteLineTemplates/default.html", "<td>default-<span ng-bind=\"quoteLine.displayLineType\"></span></td>\n" +
            "<td><span ng-bind=\"quoteLine.displayDescription\"></span></td>\n" +
            "<td><span ng-bind=\"quoteLine.displayPartSkill\"></span></td>\n" +
            "<td style=\"text-align:right\"><span ng-bind=\"quoteLine.displayQuantity\"></span></td>\n" +
            "<td style=\"text-align:right\"><span ng-bind=\"quoteLine.displayPrice\"></span></td>\n" +
            "<td style=\"text-align:right\"><span ng-bind=\"quoteLine.displayExtended\"></span></td>\n" +
            "<td style=\"text-align:center\"><span ng-if=\"quoteLine.taxable\">&#10004;</span></td>\n" +
            "<td class=\"deleteQuoteLine\"><i class=\"appIcon-close\"></i></td>");
    }]);
angular.module("quotes/partials/quoteLineTemplates/flatFee.html", []).run(["$templateCache", function ($templateCache) {
        $templateCache.put("quotes/partials/quoteLineTemplates/flatFee.html", "<td>Flat Fee</td>\n" +
            "<td>\n" +
            "	<span ng-bind=\"quoteLine.displayDescription\"  ng-show=\"quoteLine !== $parent.selectedRow\"></span>\n" +
            "	<ad-form-input ng-model=\"quoteLine.description\" form-field-options=\"{name:'description'}\" ng-show=\"quoteLine === $parent.selectedRow\" autofocus></ad-form-input>\n" +
            "<td><span ng-bind=\"quoteLine.displayPartSkill\"></span></td>\n" +
            "<td style=\"text-align:right\">&nbsp;</td>\n" +
            "<td style=\"text-align:right\">\n" +
            "	<span ng-bind=\"quoteLine.displayPrice\" ng-show=\"quoteLine !== $parent.selectedRow\"></span>\n" +
            "	<ad-form-input ng-model=\"quoteLine.price_Rate\" form-field-options=\"{ type: 'number', name:'price', symbol: '$'}\" ng-show=\"quoteLine === $parent.selectedRow\" ></ad-form-input></td>\n" +
            "<td style=\"text-align:right\"><span ng-bind=\"quoteLine.displayExtended\"></span></td>\n" +
            "<td style=\"text-align:center\">&nbsp;</td>\n" +
            "<td class=\"deleteQuoteLine\"><i class=\"appIcon-close\"></i></td>");
    }]);
angular.module("quotes/partials/quoteLineTemplates/note.html", []).run(["$templateCache", function ($templateCache) {
        $templateCache.put("quotes/partials/quoteLineTemplates/note.html", "<td>Note</td>\n" +
            "<td colspan=\"6\">\n" +
            "	<span ng-bind=\"quoteLine.displayDescription\"  ng-show=\"quoteLine !== $parent.selectedRow\"></span>\n" +
            "	<ad-form-input ng-model=\"quoteLine.description\" form-field-options=\"{}\" ng-show=\"quoteLine === $parent.selectedRow\" autofocus></ad-form-input>\n" +
            "</td>\n" +
            "<td class=\"deleteQuoteLine\"><i class=\"appIcon-close\"></i></td>");
    }]);
angular.module("quotes/partials/quoteList.html", []).run(["$templateCache", function ($templateCache) {
        $templateCache.put("quotes/partials/quoteList.html", "<ad-view header-size=\"'small'\" class=\"fill\">\n" +
            "    <div transclude-to=\"header\">\n" +
            "        <h1>Saved Quotes</h1>\n" +
            "        <div class=\"actions\">\n" +
            "            <a class=\"btn btn-primary\" ng-click=\"vm.ShellService.goToNewQuote()\">+ New Quote</a>\n" +
            "        </div>\n" +
            "    </div>\n" +
            "    <div transclude-to=\"content\">\n" +
            "        <ad-list class=\"fill quoteList\" items=\"vm.filteredQuoteList\"\n" +
            "                on-select=\"vm.adListQuoteSelected(item, $event, $index)\"\n" +
            "                template=\"'adListTemplate.html'\" allow-search=\"true\" show-details-pane=\"true\" start-selected-item=\"vm.filteredQuoteList[0]\">\n" +
            "            <div class=\"quoteSummaryHeader\">\n" +
            "                <h2>\n" +
            "                    Quote <span ng-bind=\"'#' + vm.quote.quoteNumber\"></span>\n" +
            "                    <span class=\"badge\" ng-show=\"vm.quote.quoteExpired\"> Expired </span>\n" +
            "                    <div class=\"creationDate\">Created On : {{vm.quote.creationDate | date:'MMM/dd/yyyy'}}</div>\n" +
            "                    <ad-action-bar actions=\"vm.quoteActions\" class=\"inverse\"></ad-action-bar>\n" +
            "                </h2>\n" +
            "            </div>\n" +
            "            <ad-quote-header></ad-quote-header>\n" +
            "            <ad-quote read-only=\"true\" quote=\"vm.quote\" shop=\"vm.shellService.shop\"></ad-quote>\n" +
            "        </ad-list>\n" +
            "        <script type=\"text/ng-template\" id=\"adListTemplate.html\">\n" +
            "            <div>\n" +
            "                <div class=\"quoteListItem\">\n" +
            "                    <div class=\"quoteNumber\" ng-bind=\"'#' + item.quoteNumber\"></div>\n" +
            "                    <div class=\"quoteAmount\" ng-bind=\"item.quoteAmount | currency\"></div>\n" +
            "\n" +
            "                    <div class=\"customerName\">\n" +
            "                        <span class=\"customerFirstName\" ng-bind=\"item.customerFirstName\"></span>\n" +
            "                        <span class=\"customerLastName\" ng-bind=\"item.customerLastName\"></span>\n" +
            "                    </div>\n" +
            "                    <div class=\"customerPhoneNumber\" ng-bind=\"item.customerPhoneNumber\"></div>\n" +
            "                    <div class=\"vehicleDescription\" ng-bind=\"item.vehicleDescription\"></div>\n" +
            "\n" +
            "                    <span class=\"updatedDate\">Updated On: </span>\n" +
            "                    <span class=\"updatedDate\" ng-bind=\"item.lastUpdateDate | date:'MMM/dd/yyyy'\"></span>\n" +
            "                    <span class=\"appIcon-itype-1 expiredImage\" ng-show=\"$parent.$parent.$parent.vm.ShellService.checkDateExpiration(item.lastUpdateDate, item)\"></span>\n" +
            "                </div>\n" +
            "            </div>\n" +
            "        </script>\n" +
            "    </div>\n" +
            "</ad-view>\n" +
            "\n" +
            "\n" +
            "\n" +
            "    <!--<div class=\"quoteList quote\">\n" +
            "        <div class=\"colorBand bg-primary\">\n" +
            "            <h1>\n" +
            "                \n" +
            "            </h1>\n" +
            "        </div>\n" +
            "        <div class=\"viewContent\">\n" +
            "           <div class=\"leftColumn\">\n" +
            "                <div id=\"savedQuoteList\">\n" +
            "                    \n" +
            "                </div>\n" +
            "            </div>\n" +
            "            \n" +
            "           <!-- <div class=\"leftColumn\">\n" +
            "                <div id=\"searchBoxContainer\">\n" +
            "                    <div class=\"searchBoxWrapper\">\n" +
            "                        <search-box placeholder=\"Search by Quote #, Customer Name or Vehicle\" on-search=\"vm.search(searchTerm)\" on-clear=\"vm.search(searchTerm)\"\n" +
            "                            search-term=\"vm.searchTerm\" number-of-character-delay=\"0\" time-delay=\"1\" is-focused=\"false\"></search-box>\n" +
            "                    </div>\n" +
            "                </div>\n" +
            "                <div id=\"savedQuoteList\">\n" +
            "                    <div>\n" +
            "                        <div  class=\"quoteListItem\" ng-repeat=\"quote in vm.filteredQuoteList | orderBy : '-creationDate'\"\n" +
            "                            ng-click=\"vm.quoteSelected(quote)\" ng-class=\"vm.quote.quoteNumber == quote.quoteNumber ? 'selected' : ''\">\n" +
            "                            <div class=\"quoteNumber\" ng-bind=\"'#' + quote.quoteNumber\"></div>\n" +
            "                            <div class=\"quoteAmount\" ng-bind=\"quote.quoteAmount | currency\"></div>\n" +
            "                            \n" +
            "                            <div class=\"customerName\">\n" +
            "                                <span class=\"customerFirstName\" ng-bind=\"quote.customerFirstName\"></span>\n" +
            "                                <span class=\"customerLastName\" ng-bind=\"quote.customerLastName\"></span>\n" +
            "                            </div>\n" +
            "                            <div class=\"customerPhoneNumber\" ng-bind=\"quote.customerPhoneNumber\"></div>\n" +
            "                            <div class=\"vehicleDescription\" ng-bind=\"quote.vehicleDescription\"></div>\n" +
            "                            \n" +
            "                            <span class=\"appIcon-itype-1 expiredImage\" ng-show=\"vm.ShellService.checkDateExpiration(quote.creationDate,quote)\"></span>\n" +
            "                            <div class=\"creationDate\" ng-bind=\"quote.creationDate | date:'MMM/dd/yyyy'\"></div>\n" +
            "                        </div>\n" +
            "                    </div>\n" +
            "                </div>\n" +
            "            </div>-->\n" +
            "            <!--<div class=\"rightColumn\">\n" +
            "                \n" +
            "            </div>\n" +
            "        </div>\n" +
            "    </div>\n" +
            "</div>-->");
    }]);
angular.module("quotes/partials/selectVehicleModal.html", []).run(["$templateCache", function ($templateCache) {
        $templateCache.put("quotes/partials/selectVehicleModal.html", "<ad-dialog-shell title=\"Select a vehicle\" buttons=\"vm.buttons\">\n" +
            "    <ymme-search ng-show=\"vm.isEngineSelected === false\" on-car-selected=\"vm.selectVehicle(engine)\"></ymme-search>\n" +
            "    \n" +
            "    <div class=\"quoteVehicleSelection\" ng-show=\"vm.isEngineSelected === true\">\n" +
            "        <div class=\"vehicleSelection\">\n" +
            "            <label>Selected Vehicle</label>\n" +
            "            \n" +
            "            <div class=\"col-xs-12 wrapper\" ng-click=\"vm.clearVehicleSelection()\">\n" +
            "                <div class=\"col-xs-1 icon appIcon-Car-3\"></div>\n" +
            "                \n" +
            "                <div class=\"col-xs-10 vehicle-description\" ng-class=\"{noVin: !vm.quote.vehicleVin}\">\n" +
            "                    <div ng-bind=\"vm.quote.vehicleDescription\"></div>\n" +
            "                    <div ng-show=\"vm.quote.vehicleVin\" ng-bind=\"vm.quote.vehicleVin\"></div>\n" +
            "                </div>\n" +
            "                \n" +
            "                <div class=\"col-xs-1 dropdown glyphicon glyphicon-chevron-down\"></div>\n" +
            "            </div>\n" +
            "        </div>\n" +
            "        \n" +
            "        <div class=\"vehicleInformation\">\n" +
            "            <div class=\"col-xs-5 noPadding\">\n" +
            "                <label>License Plate</label>\n" +
            "                <input class=\"col-xs-11 noPadding\" type=\"text\" ng-model=\"vm.quote.vehicleLicensePlate\" />\n" +
            "            </div>\n" +
            "            <div class=\"col-xs-2 noPadding\">\n" +
            "                <label>State</label>\n" +
            "                <select class=\"col-xs-8 noPadding\" type=\"text\" ng-model=\"vm.quote.vehicleLicenseState\" ng-options=\"state for state in vm.states\" style=\"height: 26px\">\n" +
            "                </select>\n" +
            "            </div>\n" +
            "            <div class=\"col-xs-5 noPadding\">\n" +
            "                <label>Mileage</label>\n" +
            "                <input class=\"col-xs-11 noPadding\" type=\"number\" min=\"0\"  ng-model=\"vm.quote.vehicleMileage\" />\n" +
            "            </div>\n" +
            "        </div>\n" +
            "    </div>\n" +
            "</ad-dialog-shell>");
    }]);
angular.module("setup/partials/ShopCreationTemplate.html", []).run(["$templateCache", function ($templateCache) {
        $templateCache.put("setup/partials/ShopCreationTemplate.html", "<ad-view header-size=\"'small'\" class=\"fill\">\n" +
            "    <div transclude-to=\"header\">\n" +
            "        <h1>Create Shop</h1>\n" +
            "        <div class=\"actions\">\n" +
            "            <div ng-if=\"!vm.setupFormIsPristine()\">\n" +
            "                <a class=\"btn btn-link\" ng-click=\"vm.cancelShopInfo()\">cancel</a>\n" +
            "                <a class=\"btn btn-primary\" ng-click=\"vm.submitShop()\">Save</a>\n" +
            "            </div>\n" +
            "            <div ng-if=\"vm.setupFormIsPristine()\">\n" +
            "                <a class=\"btn btn-primary\" ng-click=\"vm.cancelShopInfo()\">Done</a>\n" +
            "            </div>\n" +
            "        </div>\n" +
            "    </div>\n" +
            "    <div transclude-to=\"content\" redraw-on-scroll>\n" +
            "        <form name=\"setupForm\" ng-init=\"vm.setFormScope(this)\" novalidate>\n" +
            "            \n" +
            "        <!--<form name=\"setupForm\" ng-submit=\"vm.submitShop(formScope)\" novalidate ng-show=\"vm.displayPage\">-->\n" +
            "            <div class=\"row\">\n" +
            "                <div class=\"col-md-9\">\n" +
            "                    <ad-form-input form-field-options=\"{\n" +
            "                    name:'shopName',\n" +
            "                    label: 'Shop Name',\n" +
            "                    notes:'Be aware that the shop name cannot be changed in the future.'\n" +
            "                }\" ng-model=\"vm.shop.name\" ng-required=\"true\"></ad-form-input>\n" +
            "                </div>\n" +
            "            </div>\n" +
            "            <div class=\"row\">\n" +
            "                <div class=\"col-md-9\">\n" +
            "                    <ad-form-input form-field-options=\"{\n" +
            "                    name:'phone',\n" +
            "                    label: 'Phone Number',\n" +
            "                    type: 'tel'\n" +
            "                }\" ng-model=\"vm.shop.phone\"></ad-form-input>\n" +
            "                </div>\n" +
            "            </div>\n" +
            "            <div class=\"row\">\n" +
            "                <div class=\"col-md-9\">\n" +
            "                    <ad-form-input form-field-options=\"{\n" +
            "                    name:'website',\n" +
            "                    label: 'Website'\n" +
            "                }\" ng-model=\"vm.shop.website\"></ad-form-input>\n" +
            "                </div>\n" +
            "            </div>\n" +
            "        </form>\n" +
            "    </div>\n" +
            "</ad-view>\n" +
            "");
    }]);
angular.module("setup/partials/setup.html", []).run(["$templateCache", function ($templateCache) {
        $templateCache.put("setup/partials/setup.html", "<ad-view header-size=\"'small'\" class=\"fill\">\n" +
            "    <div transclude-to=\"header\">\n" +
            "        <h1>Settings</h1>\n" +
            "        <div class=\"actions\">\n" +
            "            <div ng-if=\"!setupForm.$pristine || !setupForm.$valid\">\n" +
            "                <a class=\"btn btn-link\" ng-click=\"vm.cancelShopInfo()\">cancel</a>\n" +
            "                <a class=\"btn btn-primary\" ng-click=\"vm.saveShopInfo()\">Save</a>\n" +
            "            </div>\n" +
            "            <div ng-if=\"setupForm.$pristine && setupForm.$valid\">\n" +
            "                <a class=\"btn btn-primary\" ng-click=\"vm.cancelShopInfo()\">Done</a>\n" +
            "            </div>\n" +
            "        </div>\n" +
            "    </div>\n" +
            "    <div transclude-to=\"content\" redraw-on-scroll>\n" +
            "        <form name=\"setupForm\" ng-init=\"vm.setFormScope(this)\" novalidate>\n" +
            "            <h2>Markup</h2>\n" +
            "            <div class=\"row\">\n" +
            "                <div class=\"col-sm-3\">\n" +
            "                    <ad-form-input form-field-options=\"{\n" +
            "                    name:'partsMarkup',\n" +
            "                    label: 'Parts Markup',\n" +
            "                    type: 'number',\n" +
            "                    symbol: '%',\n" +
            "                    symbolLocation: 'right',\n" +
            "                    precision: 3,\n" +
            "                    notes: 'Note: Applied to looked-up parts only' \n" +
            "                }\" ng-model=\"vm.shop.display_partsMarkup\" min=\"0\" max=\"99.99\" required scootch='{\"scrollClass\": \"viewContent\"}'></ad-form-input>\n" +
            "                </div>\n" +
            "            </div>\n" +
            "\n" +
            "            <h2>Labor Rates</h2>\n" +
            "            <div class=\"row\">\n" +
            "                <div class=\"col-sm-3\">\n" +
            "                    <ad-form-input form-field-options=\"{\n" +
            "                    name:'laborRate1',\n" +
            "                    label: 'Hourly Rate 1',\n" +
            "                    type: 'number',\n" +
            "                    symbol: '$',\n" +
            "                    precision: 2\n" +
            "                }\" ng-model=\"vm.shop.laborRate1\" min=\"0\" max=\"1000000\" required scootch='{\"scrollClass\": \"viewContent\"}'></ad-form-input>\n" +
            "                </div>\n" +
            "\n" +
            "                <div class=\"col-sm-3\">\n" +
            "                    <ad-form-input form-field-options=\"{\n" +
            "                    name:'laborRate2',\n" +
            "                    label: 'Hourly Rate 2',\n" +
            "                    type: 'number',\n" +
            "                    symbol: '$',\n" +
            "                    precision: 2\n" +
            "                }\" ng-model=\"vm.shop.laborRate2\" min=\"0\" max=\"1000000\" required scootch='{\"scrollClass\": \"viewContent\"}'></ad-form-input>\n" +
            "                </div>\n" +
            "\n" +
            "                <div class=\"col-sm-3\">\n" +
            "                    <ad-form-input form-field-options=\"{\n" +
            "                    name:'laborRate3',\n" +
            "                    label: 'Hourly Rate 3',\n" +
            "                    type: 'number',\n" +
            "                    symbol: '$',\n" +
            "                    precision: 2\n" +
            "                }\" ng-model=\"vm.shop.laborRate3\" min=\"0\" max=\"1000000\" required scootch='{\"scrollClass\": \"viewContent\"}'></ad-form-input>\n" +
            "                </div>\n" +
            "\n" +
            "                <div class=\"col-sm-3\">\n" +
            "                    <ad-form-input form-field-options=\"{\n" +
            "                            name:'defaultHourlyRate',\n" +
            "                            label: 'Default Hourly Rate',\n" +
            "                            type: 'list',\n" +
            "                            listType: 'select',\n" +
            "                            itemLabel: 'label',\n" +
            "                            itemValue: 'id'\n" +
            "                        }\" ad-form-field-items=\"[{id:1, label:'Hourly Rate 1'}, {id:2, label:'Hourly Rate 2'},{id:3, label:'Hourly Rate 3'}]\" \n" +
            "                        ng-model=\"vm.shop.defaultLaborRate\" required scootch='{\"scrollClass\": \"viewContent\"}'></ad-form-input>\n" +
            "                </div>\n" +
            "            </div>\n" +
            "\n" +
            "            <h2>Tax Rates</h2>\n" +
            "            <div class=\"row\">\n" +
            "                <div class=\"col-sm-6\">\n" +
            "                    <div class=\"col-sm-6\">\n" +
            "                        <ad-form-input form-field-options=\"{\n" +
            "                        name:'partsTax',\n" +
            "                        label: 'Parts Tax Rate',\n" +
            "                        type: 'number',\n" +
            "                        symbol: '%',\n" +
            "                        symbolLocation: 'right',\n" +
            "                        precision: 3\n" +
            "                    }\" ng-model=\"vm.shop.display_partsTax\" min=\"0\" max=\"99.9\" ng-required scootch='{\"scrollClass\": \"viewContent\"}'></ad-form-input>\n" +
            "                    </div>\n" +
            "                    <div class=\"col-sm-6\">\n" +
            "                        <input type=\"checkbox\" ng-model=\"vm.shop.taxPartsDefault\" /> Tax Parts by Default\n" +
            "                        <!--<ad-form-input form-field-options=\"{\n" +
            "                            type: 'list',\n" +
            "                            label: '',\n" +
            "                            itemLabel: 'label',\n" +
            "                            itemValue: 'label',\n" +
            "                            selectMultiple: true\n" +
            "                        }\" ad-form-field-items=\"[{id:0, label:'Tax Parts by Default'}]\" ng-model=\"vm.shop.taxPartsDefault\"></ad-form-input>-->\n" +
            "                    </div>\n" +
            "                </div>\n" +
            "\n" +
            "                <div class=\"col-sm-6\">\n" +
            "                    <div class=\"col-sm-6\">\n" +
            "                        <ad-form-input form-field-options=\"{\n" +
            "                        name:'laborsTax',\n" +
            "                        label: 'Labor Tax Rate',\n" +
            "                        type: 'number',\n" +
            "                        symbol: '%',\n" +
            "                        symbolLocation: 'right',\n" +
            "                        precision: 3\n" +
            "                          }\" ng-model=\"vm.shop.display_laborTax\" min=\"0\" max=\"99.9\" ng-required scootch='{\"scrollClass\": \"viewContent\"}'></ad-form-input>\n" +
            "                    </div>\n" +
            "\n" +
            "                    <div class=\"col-sm-6\">\n" +
            "                        <input type=\"checkbox\" ng-model=\"vm.shop.taxLaborDefault\" /> Tax Labor by Default\n" +
            "                       <!-- <ad-form-input form-field-options=\"{\n" +
            "                            type: 'list',\n" +
            "                            label: '',\n" +
            "                            itemLabel: 'label',\n" +
            "                            itemValue: 'label',\n" +
            "                            selectMultiple: true\n" +
            "                        }\" ad-form-field-items=\"[{id:0, label:'Tax Labor by Default'}]\" ng-model=\"vm.shop.taxLaborDefault\"></ad-form-input>-->\n" +
            "                    </div>\n" +
            "\n" +
            "                </div>\n" +
            "            </div>\n" +
            "\n" +
            "\n" +
            "            <h2>Quote Numbering and Expiration Date</h2>\n" +
            "            <div class=\"row\">\n" +
            "                <div class=\"col-sm-3\">\n" +
            "                    <ad-form-input form-field-options=\"{\n" +
            "                        name:'nextQuoteNumber',\n" +
            "                        label: 'Next Quote #',\n" +
            "                        type: 'number',\n" +
            "                        notes: 'The next quote number must be greater than the current quote number.'\n" +
            "                          }\" ng-model=\"vm.shop.nextQuoteNumber\" min=\"{{ vm.fixedQuoteNumber }}\" required scootch='{\"scrollClass\": \"viewContent\"}'></ad-form-input>\n" +
            "                </div>\n" +
            "\n" +
            "                <div class=\"col-sm-5 alignWithText\" scootch='{\"scrollClass\": \"viewContent\"}'>\n" +
            "                    <input type=\"checkbox\" ng-model=\"vm.shop.displayAfterExpirationDate\" /> Quote expires after\n" +
            "                    <input type=\"number\" ng-model=\"vm.shop.expirationDays\" min=\"1\" max=\"500\" numeric ng-disabled=\"!vm.shop.displayAfterExpirationDate\" ng-required=\"vm.shop.displayAfterExpirationDate\"/> days\n" +
            "                </div>\n" +
            "            </div>\n" +
            "\n" +
            "            <h2>Catalog Information</h2>\n" +
            "            <div class=\"row\">\n" +
            "                <div class=\"col-sm-6\">\n" +
            "\n" +
            "                    <ad-form-input form-field-options=\"{\n" +
            "                            label: 'AutoZone Catalog PIN',\n" +
            "                            name: 'azCatalogPIN',\n" +
            "                            type: 'number',\n" +
            "                            notes: 'PIN required to access AutoZone catalog. Please call 1-866-853-6459 for assistance or contact your local AutoZone Store.'\n" +
            "                        }\" ad-form-field-items=\"[{id:0, label:'Tax Labor by Default'}]\" ng-model=\"vm.shop.azcatalogPIN\" min=\"0\" scootch='{\"scrollClass\": \"viewContent\"}'></ad-form-input>\n" +
            "                    <!--<label>AutoZone Catalog PIN</label>\n" +
            "                    <input name=\"azCatalogPIN\" type=\"number\" class=\"form-control textRight\" ng-model=\"vm.shop.azcatalogPIN\" min=\"0\" numeric />\n" +
            "                    <div class=\"form-errors col-sm-12\" ng-show=\"formScope.azcatalogPIN.$error.min\">\n" +
            "                    </div>-->\n" +
            "                </div>\n" +
            "            </div>\n" +
            "\n" +
            "            <div ng-show=\"vm.ShellService.moEnabled\">\n" +
            "                <h2>Link Manage Online</h2>\n" +
            "                <div class=\"row\">\n" +
            "                    <div class=\"col-sm-6 manage-online-section\">\n" +
            "                        <div class=\"form-group\" ng-show=\"!vm.ShellService.molLinked || vm.ShellService.molLinked == undefined\">\n" +
            "                            <span>Entering your Manage Online Credentials will link ALLDATA Mobile with your Manage Online customer list. <a ng-click=\"vm.openLearnMoreDialog()\">Learn more</a></span>\n" +
            "                            <div class=\"form-group\" ng-class=\"{ 'has-error': setupForm.username.$dirty && setupForm.username.$error.required }\">\n" +
            "                                <input name=\"username\" ng-model=\"vm.shop.manageOnlineUsername\" placeholder=\"Enter username...\" class=\"form-control col-sm-12 col-md-12 col-lg-12\" type=\"text\" />\n" +
            "                                <span ng-show=\"setupForm.username.$dirty && setupForm.username.$error.required\" class=\"help-block\">Username is required</span>\n" +
            "                            </div>\n" +
            "                            <div class=\"form-group\" ng-class=\"{ 'has-error': setupForm.password.$dirty && setupForm.password.$error.required }\">\n" +
            "                                <input name=\"password\" ng-model=\"vm.shop.manageOnlinePassword\" placeholder=\"Enter password...\" class=\"form-control col-sm-12 col-md-12 col-lg-12\" type=\"password\" />\n" +
            "                                <span ng-show=\"setupForm.password.$dirty && setupForm.password.$error.required\" class=\"help-block\">Password is required</span>\n" +
            "                            </div>\n" +
            "                            <div class=\"form-actions\">\n" +
            "                                <button type=\"button\" ng-click=\"vm.verifyManageOnline()\" ng-disabled=\"setupForm.$invalid\" class=\"btn btn-primary col-sm-12 col-md-12 col-lg-12\">Link</button>\n" +
            "                            </div>\n" +
            "                            <div align=\"center\" class=\"col-sm-12 col-md-12 col-lg-12 mol-auth-message\">\n" +
            "                                <i class=\"appIcon-fail\" ng-if=\"(!vm.molVerified && (vm.molVerified != null))\"><span> Invalid Username or Password</span></i>\n" +
            "                            </div>\n" +
            "                        </div>\n" +
            "                        <div class=\"form-group\" ng-show=\"vm.ShellService.molLinked\">\n" +
            "                            <label>You are currently linked with Manage Online Account with username {{vm.shop.manageOnlineUsername}}</label>\n" +
            "                            <div class=\"form-actions\">\n" +
            "                                <button type=\"button\" ng-click=\"vm.unlinkManageOnline()\" class=\"btn btn-primary col-sm-6 col-md-6 col-lg-6\">Unlink Manage Online</button>\n" +
            "                            </div>\n" +
            "                        </div>\n" +
            "                    </div>\n" +
            "                </div>\n" +
            "            </div>\n" +
            "<!--\n" +
            "            <div class=\"row\">\n" +
            "                <div class=\"col-sm-10\">\n" +
            "                    <span class=\"note\">The AutoZone Catalog PIN is required to access the AutoZone Parts Catalog</span>\n" +
            "                </div>\n" +
            "            </div>-->\n" +
            "\n" +
            "            <h2>Printing Information</h2>\n" +
            "            <div class=\"row\">\n" +
            "                <div class=\"col-sm-3\">\n" +
            "                    <input type=\"checkbox\" ng-model=\"vm.shop.hideLaborWhenPrinting\" /> Hide Labor Rate and Hours\n" +
            "                    <!--<ad-form-input form-field-options=\"{\n" +
            "                            type: 'list',\n" +
            "                            itemLabel: 'label',\n" +
            "                            itemValue: 'label',\n" +
            "                            selectMultiple: true\n" +
            "                        }\" ad-form-field-items=\"[{id:0, label:'Hide Labor Rate and Hours'}]\" ng-model=\"vm.shop.hideLaborWhenPrinting\"></ad-form-input>-->\n" +
            "                </div>\n" +
            "                <div class=\"col-sm-3\">\n" +
            "                    <input type=\"checkbox\" ng-model=\"vm.shop.hidePartNumbersWhenPrinting\" /> Hide Part #\n" +
            "                </div>\n" +
            "            </div>\n" +
            "\n" +
            "            <div class=\"row\">\n" +
            "                <div class=\"col-sm-9\">\n" +
            "                    <ad-form-input form-field-options=\"{\n" +
            "                    name:'headerText',\n" +
            "                    label: 'Header Text',\n" +
            "                    type: 'textarea',\n" +
            "                    maxlength: 200,\n" +
            "                    rows: 5,\n" +
            "                    notes: 'Note: For company information.'\n" +
            "                }\" ng-model=\"vm.shop.headerContents\" scootch='{\"scrollClass\": \"viewContent\"}'></ad-form-input>\n" +
            "                </div>\n" +
            "            </div>\n" +
            "\n" +
            "            <div class=\"row\">\n" +
            "                <div class=\"col-sm-9\">\n" +
            "                    <ad-form-input form-field-options=\"{\n" +
            "                    name:'footerText',\n" +
            "                    label: 'Footer Text',\n" +
            "                    type: 'textarea',\n" +
            "                    maxlength: 200,\n" +
            "                    rows: 5,\n" +
            "                    notes: 'Note: For disclaimers, thank you messages, etc.'\n" +
            "                }\" ng-model=\"vm.shop.footerContents\" scootch='{\"scrollClass\": \"viewContent\"}'></ad-form-input>\n" +
            "                </div>\n" +
            "            </div>\n" +
            "\n" +
            "            <h2>Email Information</h2>\n" +
            "            <div class=\"row\">\n" +
            "                <div class=\"col-sm-6\">\n" +
            "                    <ad-form-input form-field-options=\"{\n" +
            "                    name:'fromEmail',\n" +
            "                    label: 'From Email Address',\n" +
            "                    type: 'email',\n" +
            "                    notes: 'example: john.doe@example.com'\n" +
            "                }\" ng-model=\"vm.shop.emailFromAddress\" required scootch='{\"scrollClass\": \"viewContent\"}'></ad-form-input>\n" +
            "                </div>\n" +
            "            </div>\n" +
            "\n" +
            "            <div class=\"row\">\n" +
            "                <div class=\"col-sm-9\">\n" +
            "                    <ad-form-input form-field-options=\"{\n" +
            "                    name:'emailMessage',\n" +
            "                    label: 'Email Message',\n" +
            "                    type: 'textarea',\n" +
            "                    maxlength: 200,\n" +
            "                    rows: 5\n" +
            "                }\" ng-model=\"vm.shop.emailMessage\" scootch='{\"scrollClass\": \"viewContent\"}'></ad-form-input>\n" +
            "                </div>\n" +
            "            </div>\n" +
            "\n" +
            "            <div class=\"row\">\n" +
            "                <div class=\"col-sm-9\">\n" +
            "                    <ad-form-input form-field-options=\"{\n" +
            "                    name:'emailSignature',\n" +
            "                    label: 'Email Signature',\n" +
            "                    type: 'textarea',\n" +
            "                    maxlength: 200,\n" +
            "                    rows: 5\n" +
            "                }\" ng-model=\"vm.shop.emailSignature\" scootch='{\"scrollClass\": \"viewContent\"}'></ad-form-input>\n" +
            "                </div>\n" +
            "            </div>\n" +
            "        </form>\n" +
            "    </div>\n" +
            "</ad-view>");
    }]);
angular.module("shell/partials/shell.html", []).run(["$templateCache", function ($templateCache) {
        $templateCache.put("shell/partials/shell.html", "<ad-shell app-name=\"Quote\">\n" +
            "    <div ui-view></div>\n" +
            "</ad-shell>\n" +
            "");
    }]);
var NgTsSeed;
(function (NgTsSeed) {
    var SelectMappedVehicleModalController = (function () {
        function SelectMappedVehicleModalController($scope, $modalInstance, acesVehicleDescriptions, vehiclePartsDisplayFlag, acesVehiclePartUrls) {
            this.$scope = $scope;
            this.$modalInstance = $modalInstance;
            this.acesVehicleDescriptions = acesVehicleDescriptions;
            this.vehiclePartsDisplayFlag = vehiclePartsDisplayFlag;
            this.acesVehiclePartUrls = acesVehiclePartUrls;
            var self = this;
            $scope.vm = self;
            self.buttons = self.initButtons();
        }
        SelectMappedVehicleModalController.prototype.initButtons = function () {
            var self = this;
            return [
                {
                    title: 'Cancel',
                    onClick: function () {
                        self.$modalInstance.dismiss();
                    }
                }
            ];
        };
        SelectMappedVehicleModalController.prototype.AutozoneVehicleSelected = function (component, event, index) {
            this.vehicleSelected(index);
        };
        SelectMappedVehicleModalController.prototype.vehicleSelected = function (index) {
            var self = this;
            self.$modalInstance.close(index);
        };
        SelectMappedVehicleModalController.$inject = ['$scope', '$modalInstance', 'acesVehicleDescriptions', 'vehiclePartsDisplayFlag', 'acesVehiclePartUrls'];
        return SelectMappedVehicleModalController;
    }());
    NgTsSeed.SelectMappedVehicleModalController = SelectMappedVehicleModalController;
})(NgTsSeed || (NgTsSeed = {}));
var NgTsSeed;
(function (NgTsSeed) {
    var Catalog;
    (function (Catalog) {
        var Controller = (function () {
            function Controller($scope, catalogDataService, spinner, $timeout, QuoteService, $modalInstance, DialogService, quote, shop, $rootScope, $q, ShellService) {
                this.$scope = $scope;
                this.catalogDataService = catalogDataService;
                this.spinner = spinner;
                this.$timeout = $timeout;
                this.QuoteService = QuoteService;
                this.$modalInstance = $modalInstance;
                this.DialogService = DialogService;
                this.quote = quote;
                this.shop = shop;
                this.$rootScope = $rootScope;
                this.$q = $q;
                this.ShellService = ShellService;
                this.showALLDATAContent = false;
                this.showALLDATAComponents = false;
                this.showALLDATAParts = false;
                this.noCatalogData = false;
                this.AZDataNotFound = false;
                this.invalidSearchItem = false;
                this.ALLDATASearchTerm = "";
                this.AutoZoneVehicleId = 0;
                this.showAutoZoneMapping = false;
                this.AutoZoneSearchTerm = "";
                this.showAutoZoneContent = false;
                this.showAutoZoneComponents = false;
                this.showAutoZoneParts = false;
                this.tabALLDATAActive = true;
                this.tabAutoZoneActive = false;
                this.laborRates = [];
                this.partsTotal = 0;
                this.laborTotal = 0;
                this.subTotal = 0;
                this.previousOnResize = null;
                $scope.vm = this;
                var self = this;
                self.quote = angular.copy(quote);
                self.buttons = self.initButtons();
                self.AutoZoneSystems = [];
                self.ALLDATABreadcrumbs = [];
                self.ALLDATASearchTerm = "";
                self.ALLDATAComponents = [];
                self.ALLDATADisplayComponents = [];
                self.ALLDATAParts = [];
                self.showALLDATAParts = false;
                self.AutoZoneSystems = [];
                self.AutoZoneBreadcrumbs = [];
                self.AutoZoneSearchTerm = "";
                self.AutoZoneComponents = [];
                self.AutoZoneDisplayComponents = [];
                self.AutoZoneParts = [];
                self.showAutoZoneParts = false;
                self.tabALLDATAActive = true;
                self.tabAutoZoneActive = false;
                self.itemsAddedToCatalog = 0;
                self.partsTotal = 0;
                self.laborTotal = 0;
                self.subTotal = 0;
                self.laborRates = self.getLaborRates();
            }
            Controller.prototype.initButtons = function () {
                var self = this;
                return [
                    {
                        title: 'Cancel',
                        onClick: function () {
                            self.$modalInstance.dismiss();
                        }
                    },
                    {
                        title: 'Done',
                        onClick: function () {
                            self.$modalInstance.close(self.quote);
                        }
                    }
                ];
            };
            Controller.prototype.ALLDATASystemLoad = function () {
                var self = this;
                self.ALLDATASearchTerm = "";
                self.invalidSearchItem = false;
                if (self.quote.vehicleALLDATAId == null || self.quote.vehicleALLDATAId == 0) {
                    this.DialogService.showAlert({
                        title: "ALLDATA Quote",
                        message: "Missing ALLDATA Vehicle ID!  No ALLDATA catalog data available"
                    });
                    return;
                }
                self.ALLDATABreadcrumbs = [];
                var url = "/vehicle/supplier_id/" + self.quote.vehicleALLDATAId + "/components/1";
                self.ALLDATAComponentClicked({ Name: 'Start', Value: url });
                self.showALLDATAContent = true;
                self.showAutoZoneContent = false;
            };
            Controller.prototype.retrieveALLDATAData = function (url) {
                var self = this;
                self.spinner.start();
                self.catalogDataService.requestALLDATAData(url)
                    .then(function (data) {
                    self.noCatalogData = false;
                    self.ALLDATADisplayComponents = data.Navigation;
                    self.showALLDATAComponents = true;
                    self.spinner.end();
                    if (data.Navigation && data.Navigation.length == 1 && data.Navigation[0].Name == "Parts and Labor") {
                        self.ALLDATAComponentClicked(data.Navigation[0]);
                        return;
                    }
                    if (self.ALLDATADisplayComponents == null) {
                        self.ALLDATADisplayPartsData(data);
                        return;
                    }
                    if (self.ALLDATADisplayComponents.length == 0) {
                        self.noCatalogData = true;
                        return;
                    }
                })
                    .catch(function (msg) {
                    self.spinner.end();
                    if (msg == "The part information requested is not available.") {
                        self.noCatalogData = true;
                        self.ALLDATADisplayComponents = null;
                    }
                });
            };
            Controller.prototype.ALLDATAStartCatalogSearch = function () {
                var self = this;
                self.invalidSearchItem = false;
                self.showALLDATAParts = false;
                self.ALLDATAParts = [];
                self.ALLDATALabor = [];
                var url = "/vehicle/supplier_id/" + self.quote.vehicleALLDATAId + "/search/" + encodeURIComponent(self.ALLDATASearchTerm);
                url = url.replace("%20", "+");
                while (self.ALLDATABreadcrumbs.length > 1) {
                    self.ALLDATABreadcrumbs.pop();
                }
                self.spinner.start();
                self.catalogDataService.requestALLDATAData(url).then(function (data) {
                    self.spinner.end();
                    self.ALLDATABreadcrumbs.push({ 'Name': 'search: ' + self.ALLDATASearchTerm, "Value": url });
                    self.ALLDATADisplayComponents = data.Navigation;
                    if (self.ALLDATADisplayComponents && self.ALLDATADisplayComponents.length == 1 && self.ALLDATADisplayComponents[0].Name == "Parts and Labor") {
                        self.ALLDATAComponentClicked(item);
                        return;
                    }
                    if (self.ALLDATADisplayComponents == null) {
                        self.ALLDATADisplayPartsData(data);
                        return;
                    }
                    if (self.ALLDATADisplayComponents.length == 0) {
                        self.showALLDATAComponents = false;
                        self.invalidSearchItem = true;
                        return;
                    }
                }, function (msg) {
                    self.spinner.end();
                });
            };
            Controller.prototype.ALLDATAComponentClicked = function (item) {
                var self = this;
                self.showALLDATAParts = false;
                self.ALLDATABreadcrumbs.push(item);
                self.retrieveALLDATAData(item.Value);
            };
            Controller.prototype.ALLDATABreadcrumbClicked = function (item) {
                var self = this;
                var newArray = [];
                var found = false;
                self.ALLDATAParts = [];
                self.ALLDATALabor = [];
                self.invalidSearchItem = false;
                self.ALLDATABreadcrumbs.forEach(function (element) {
                    if (item.Name == element.Name) {
                        found = true;
                    }
                    if (found == false) {
                        newArray.push(element);
                    }
                });
                self.ALLDATABreadcrumbs = newArray;
                self.ALLDATAComponentClicked(item);
            };
            Controller.prototype.ALLDATADisplayPartsData = function (data) {
                var self = this;
                self.showALLDATAComponents = false;
                self.showALLDATAParts = true;
                if (data.parts) {
                    self.ALLDATAParts = [];
                    data.parts.forEach(function (part) {
                        if (Object.keys(part).length !== 0) {
                            self.ALLDATAParts.push(part.Name);
                            var hierarchy = [];
                            hierarchy.push(part.Name);
                            self.flattenPartsHierarchy(part, self.ALLDATAParts, 1, hierarchy);
                        }
                    });
                }
                if (data.labor) {
                    self.ALLDATALabor = [];
                    data.labor.forEach(function (labor) {
                        if (Object.keys(labor).length !== 0) {
                            self.ALLDATALabor.push(labor.Name);
                            var hierarchy = [];
                            hierarchy.push(labor.Name);
                            self.flattenLaborHierarchy(labor, self.ALLDATALabor, 1, hierarchy);
                        }
                    });
                }
                self.componentName = data.ComponentName;
            };
            Controller.prototype.flattenPartsHierarchy = function (parts, partsArray, level, hierarchy) {
                var self = this;
                if (parts == null) {
                    return;
                }
                var padding = "";
                for (var i = 0; i < level; ++i) {
                    padding += "  ";
                }
                if (self.isArray(parts.Value)) {
                    parts.Value.forEach(function (part) {
                        partsArray.push(padding + part.Name.trim());
                        hierarchy.push(part.Name);
                        self.flattenPartsHierarchy(part, partsArray, level + 1, hierarchy);
                        hierarchy.pop();
                    });
                    return;
                }
                if (parts.Value.Price != null) {
                    parts.Value.Price = parts.Value.Price.replace("$", "");
                }
                parts.Value.quantitySelected = 0;
                parts.Value.Hierarchy = "";
                parts.Value.Name = "";
                hierarchy.forEach(function (element) {
                    if (parts.Value.Name.length == 0) {
                        parts.Value.Name = element;
                    }
                    else {
                        if (parts.Value.Hierarchy.length > 0) {
                            parts.Value.Hierarchy += " - ";
                        }
                        parts.Value.Hierarchy += element;
                    }
                });
                partsArray.push(parts);
            };
            Controller.prototype.addALLDATAPart = function (part, quantity, isTaxable, buttonLabel) {
                var self = this;
                var quote = self.quote;
                var line = new NgTsSeed.QuoteLineModel();
                self.ShellService.sendAnalyticsEvents('ALLDATA Catalog Part - Added', 'Quote from Mobile', 'ALLDATA Catalog Part is Added.');
                part.Value.quantitySelected = quantity;
                if (buttonLabel == 'Remove') {
                    var isRemovedFlag = false;
                    quote.datalines.reverse().forEach(function (tempLine, index, source) {
                        if (tempLine.lineType == 'ADPart' && tempLine.partNumber == part.Value.PartNumber && isRemovedFlag == false) {
                            isRemovedFlag = true;
                            self.partsTotal -= (tempLine.quantity_Hours * tempLine.price_Rate);
                            self.subTotal -= (tempLine.quantity_Hours * tempLine.price_Rate);
                            self.itemsAddedToCatalog -= tempLine.quantity_Hours;
                            source.splice(index, 1);
                        }
                    });
                    quote.datalines = quote.datalines.reverse();
                    return;
                }
                if (part.Value.quantitySelected == 0) {
                    return;
                }
                line.description = part.Value.Hierarchy;
                if (part.Value.Description.length > 0) {
                    line.description += "-" + part.Value.Description;
                }
                if (part.Value.PriceDescription.length > 0) {
                    line.description += "-" + part.Value.PriceDescription;
                }
                line.lineNumber = quote.datalines.length + 1;
                line.lineType = "ADPart";
                line.name = part.Value.Name;
                line.price_Rate = self.getPartMarkupPrice(part.Value.Price);
                line.taxable = isTaxable;
                line.quantity_Hours = part.Value.quantitySelected;
                line.partNumber = part.Value.PartNumber;
                line.displayPartSkill = part.Value.PartNumber;
                line.justAdded = true;
                if (buttonLabel == 'Add') {
                    quote.datalines.push(line);
                }
                else if (buttonLabel == 'Update') {
                    var isUpdatedFlag = false;
                    quote.datalines.reverse().forEach(function (tempLine, index, source) {
                        if (tempLine.lineType == 'ADPart' && tempLine.partNumber == part.Value.PartNumber && isUpdatedFlag == false) {
                            quote.datalines[index] = line;
                            isUpdatedFlag = true;
                        }
                    });
                    quote.datalines = quote.datalines.reverse();
                }
                self.partsTotal += (line.quantity * line.price);
                self.subTotal += (line.quantity * line.price);
                self.itemsAddedToCatalog += line.quantity;
            };
            Controller.prototype.flattenLaborHierarchy = function (labor, laborArray, level, hierarchy) {
                var self = this;
                if (labor == null) {
                    return;
                }
                var padding = "";
                for (var i = 0; i < level; ++i) {
                    padding += " ";
                }
                if (self.isArray(labor.Value)) {
                    labor.Value.forEach(function (line) {
                        laborArray.push(padding + line.Name.trim());
                        hierarchy.push(line.Name);
                        self.flattenLaborHierarchy(line, laborArray, level + 1, hierarchy);
                        hierarchy.pop();
                    });
                    return;
                }
                labor.Value.Hierarchy = "";
                labor.Value.Name = "";
                hierarchy.forEach(function (element) {
                    if (labor.Value.Name.length == 0) {
                        labor.Value.Name = element;
                    }
                    else {
                        if (labor.Value.Hierarchy.length > 0) {
                            labor.Value.Hierarchy += " - ";
                        }
                        labor.Value.Hierarchy += element;
                    }
                });
                laborArray.push(labor);
            };
            Controller.prototype.addALLDATALabor = function (labor, laborRateIndex, isTaxable, buttonLabel) {
                var self = this;
                var quote = self.quote;
                var line = new NgTsSeed.QuoteLineModel();
                line.description = labor.Value.Hierarchy;
                self.ShellService.sendAnalyticsEvents('ALLDATA Catalog Labor - Added', 'Quote from Mobile', 'ALLDATA Catalog Labor is Added.');
                if (labor.Value.Description.length > 0) {
                    line.description += "-" + labor.Value.Description;
                }
                if (buttonLabel == 'Remove') {
                    var isRemovedFlag = false;
                    quote.datalines.reverse().forEach(function (tempLine, index, source) {
                        if (tempLine.lineType == 'ADLabor' && tempLine.description == line.description && isRemovedFlag == false) {
                            isRemovedFlag = true;
                            self.laborTotal -= (tempLine.quantity * tempLine.price);
                            self.subTotal -= (tempLine.quantity * tempLine.price);
                            self.itemsAddedToCatalog -= 1;
                            source.splice(index, 1);
                        }
                    });
                    quote.datalines = quote.datalines.reverse();
                    return;
                }
                if (laborRateIndex == -1) {
                    return;
                }
                var laborRate = self.getLaborRates()[laborRateIndex];
                line.skill = labor.Value.Skill;
                line.displayPartSkill = labor.Value.Skill;
                line.lineNumber = quote.datalines.length + 1;
                line.lineType = "ADLabor";
                line.name = labor.Value.Name;
                line.quantity_Hours = parseFloat(labor.Value.StandardTime);
                line.taxable = isTaxable;
                line.price_Rate = laborRate;
                line.justAdded = true;
                if (buttonLabel == 'Add') {
                    quote.datalines.push(line);
                }
                else if (buttonLabel == 'Update') {
                    var isUpdatedFlag = false;
                    quote.datalines.reverse().forEach(function (tempLine, index, source) {
                        if (tempLine.lineType == 'ADLabor' && tempLine.description == line.description && isUpdatedFlag == false) {
                            quote.datalines[index] = line;
                            isUpdatedFlag = true;
                        }
                    });
                    quote.datalines = quote.datalines.reverse();
                }
                self.laborTotal += (line.quantity_Hours * line.price_Rate);
                self.subTotal += (line.quantity_Hours * line.price_Rate);
                self.itemsAddedToCatalog += 1;
            };
            Controller.prototype.AutoZoneSystemLoad = function () {
                var self = this;
                self.AutoZoneSearchTerm = "";
                self.AZDataNotFound = false;
                self.showALLDATAContent = false;
                self.showAutoZoneContent = true;
                self.invalidSearchItem = false;
                var defer = self.$q.defer();
                if (self.quote.vehicleALLDATAId == null || self.quote.vehicleALLDATAId == 0) {
                    this.DialogService.showAlert({
                        title: "ALLDATA Quote",
                        message: "Missing ALLDATA Vehicle ID!  No AutoZone catalog data available"
                    });
                    self.tabALLDATAActive = true;
                    self.tabAutoZoneActive = false;
                    return;
                }
                if (self.shop.azcatalogPIN == null || self.shop.azcatalogPIN == 0) {
                    this.DialogService.showAlert({
                        title: "ALLDATA Quote",
                        message: "PIN required to access AutoZone catalog. Please call 1-866-853-6459 for assistance or contact your local AutoZone Store."
                    });
                    self.tabALLDATAActive = true;
                    self.tabAutoZoneActive = false;
                    return;
                }
                self.catalogDataService.setAutoZonePIN(self.shop.azcatalogPIN);
                self.AutoZoneVehicleId = self.quote.vehicleAutoZoneId;
                if (self.tabAutoZoneActive === true && (self.AutoZoneVehicleId === undefined || self.AutoZoneVehicleId == 0)) {
                    self.spinner.start();
                    self.catalogDataService.requestAutoZoneVehicleId(self.quote.vehicleALLDATAId).then(function (data) {
                        self.spinner.end();
                        if (data == null || data.length == 0) {
                            this.DialogService.showAlert({
                                title: "ALLDATA Quote",
                                message: "Unable to map the ALLDATA vehicle to the AutoZone vehicle. No AutoZone catalog information will be available for this vehicle."
                            });
                            self.tabALLDATAActive = true;
                            self.tabAutoZoneActive = false;
                            return;
                        }
                        if (data.length == 1) {
                            var tokens = data[0].Value.split('/');
                            self.completeLoadingAutoZoneSystems(parseInt(tokens[tokens.length - 2]));
                            return;
                        }
                        if (data.length > 1) {
                            var dataIndex;
                            var acesVehicleIds = [];
                            var acesVehiclePartNames = [];
                            var acesVehiclePartUrls = [];
                            for (dataIndex in data) {
                                acesVehiclePartNames.push(data[dataIndex].Name);
                                acesVehiclePartUrls.push(data[dataIndex].Value);
                            }
                            self.selectACESVehicleParts(acesVehiclePartNames, acesVehiclePartUrls, defer);
                        }
                        self.AutoZoneMappingChoices = data;
                        self.showAutoZoneMapping = true;
                        return;
                    }, function (error) {
                        if (error == 'The part information requested is not available.') {
                            self.AZDataNotFound = true;
                        }
                        self.spinner.end();
                        self.tabALLDATAActive = false;
                        self.tabAutoZoneActive = true;
                    });
                }
                else if (self.tabAutoZoneActive === true) {
                    self.completeLoadingAutoZoneSystems(self.AutoZoneVehicleId);
                }
            };
            Controller.prototype.selectACESVehicleParts = function (acesVehiclePartNames, acesVehiclePartUrls, defer) {
                var self = this;
                self.DialogService.showDialog({
                    url: 'adCatalogControl/partials/selectMappedVehicleModal.html',
                    controller: 'SelectMappedVehicleModalController',
                    resolve: {
                        acesVehicleDescriptions: (function () {
                            return acesVehiclePartNames;
                        }),
                        vehiclePartsDisplayFlag: (function () {
                            return false;
                        }),
                        acesVehiclePartUrls: (function () {
                            return acesVehiclePartUrls;
                        })
                    }
                }).result.then(function (selectedVehiclePartIndex) {
                    var tokens = acesVehiclePartUrls[selectedVehiclePartIndex].split('/');
                    self.completeLoadingAutoZoneSystems(parseInt(tokens[tokens.length - 2]));
                }, function () {
                    defer.reject();
                });
            };
            Controller.prototype.AutoZoneMappingChoiceMade = function (index) {
                var self = this;
                self.showAutoZoneMapping = false;
                if (index < 0) {
                    this.DialogService.showAlert({
                        title: "ALLDATA Quote",
                        message: "AutoZone Vehicle Selection cancelled.  The AutoZone catalog will not be available"
                    });
                    self.tabALLDATAActive = true;
                    self.tabAutoZoneActive = false;
                    return;
                }
                var tokens = self.AutoZoneMappingChoices[index].Value.split('/');
                self.completeLoadingAutoZoneSystems(parseInt(tokens[tokens.length - 2]));
            };
            Controller.prototype.completeLoadingAutoZoneSystems = function (id) {
                var self = this;
                self.AutoZoneVehicleId = id;
                self.quote.vehicleAutoZoneId = id;
                if (self.AutoZoneVehicleId == 0) {
                    return;
                }
                self.AutoZoneBreadcrumbs = [];
                var url = "/vehicle/supplier_id/" + self.AutoZoneVehicleId + "/components/1";
                self.AutoZoneComponentClicked({ Name: 'Start', Value: url });
                self.showALLDATAContent = false;
                self.showAutoZoneContent = true;
            };
            Controller.prototype.retrieveAutoZoneData = function (url) {
                var self = this;
                self.spinner.start();
                self.catalogDataService.requestAutoZoneData(url)
                    .then(function (data) {
                    self.spinner.end();
                    self.noCatalogData = false;
                    if (data.parts != null) {
                        self.AutoZoneDisplayPartsData(data);
                        self.showAutoZoneComponents = false;
                    }
                    else {
                        if (data.Navigation == null) {
                            self.noCatalogData = true;
                            self.AutoZoneDisplayComponents = null;
                            return;
                        }
                        self.AutoZoneDisplayComponents = data.Navigation;
                        self.showAutoZoneComponents = true;
                    }
                })
                    .catch(function (msg) {
                    self.spinner.end();
                    if (msg == "The part information requested is not available.") {
                        self.noCatalogData = true;
                        self.AutoZoneDisplayComponents = null;
                    }
                });
            };
            Controller.prototype.AutoZoneStartCatalogSearch = function () {
                var self = this;
                self.AutoZoneParts = [];
                self.invalidSearchItem = false;
                self.showAutoZoneComponents = true;
                if (self.AutoZoneSearchTerm.length == 0) {
                    return;
                }
                self.showAutoZoneParts = false;
                while (self.AutoZoneBreadcrumbs.length > 1) {
                    self.AutoZoneBreadcrumbs.pop();
                }
                var url = "/vehicle/supplier_id/" + self.AutoZoneVehicleId + "/combinedSearch/" + encodeURIComponent(self.AutoZoneSearchTerm);
                self.spinner.start();
                self.catalogDataService.requestAutoZoneData(url).then(function (data) {
                    self.AutoZoneBreadcrumbs.push({ 'Name': 'search: ' + self.AutoZoneSearchTerm, "Value": url });
                    self.spinner.end();
                    if (data && data.Navigation.length == 1 && data.Navigation[0].Name == "Parts and Labor") {
                        var partsParent = Object();
                        partsParent.parts = data.Navigation;
                        self.AutoZoneDisplayPartsData(partsParent);
                        return;
                    }
                    self.AutoZoneDisplayComponents = data.Navigation;
                    if (self.AutoZoneDisplayComponents && self.AutoZoneDisplayComponents.length == 1 && self.AutoZoneDisplayComponents[0].Name == "Parts and Labor") {
                        self.AutoZoneComponentClicked(item);
                        return;
                    }
                    if (data.parts != null) {
                        self.AutoZoneDisplayPartsData(data);
                        return;
                    }
                    if (!data || self.AutoZoneDisplayComponents.length == 0) {
                        self.showAutoZoneComponents = false;
                        self.invalidSearchItem = true;
                        return;
                    }
                    self.showAutoZoneComponents = true;
                }, function (msg) {
                    self.showAutoZoneComponents = false;
                    self.invalidSearchItem = true;
                    self.spinner.end();
                });
            };
            Controller.prototype.AutoZoneComponentClicked = function (item) {
                var self = this;
                self.showAutoZoneParts = false;
                self.AutoZoneBreadcrumbs.push(item);
                self.retrieveAutoZoneData(item.Value);
            };
            Controller.prototype.AutoZoneBreadcrumbClicked = function (item) {
                var self = this;
                var newArray = [];
                var found = false;
                self.AutoZoneParts = [];
                self.AutoZoneBreadcrumbs.forEach(function (element) {
                    if (item.Name == element.Name) {
                        found = true;
                    }
                    if (found == false) {
                        newArray.push(element);
                    }
                });
                self.AutoZoneBreadcrumbs = newArray;
                self.AutoZoneComponentClicked(item);
            };
            Controller.prototype.AutoZoneDisplayPartsData = function (data) {
                var self = this;
                self.showAutoZoneComponents = false;
                self.showAutoZoneParts = true;
                self.AutoZoneParts = [];
                data.parts.forEach(function (partElement) {
                    partElement.Value.forEach(function (part) {
                        if (part.Value != null) {
                            part.Value.elementName = partElement.Name;
                            part.Value.Name = part.Name;
                            part.Value.quantitySelected = 1;
                            part.Value.price = part.Value.Cost;
                            self.AutoZoneParts.push(part.Value);
                        }
                        else {
                            part.elementName = partElement.Name;
                            part.quantitySelected = 1;
                            part.price = part.Cost;
                            self.AutoZoneParts.push(part);
                        }
                    });
                });
            };
            Controller.prototype.addAutoZonePart = function (part, quantity, isTaxable, buttonLabel) {
                var self = this;
                var quote = self.quote;
                part.quantitySelected = quantity;
                self.ShellService.sendAnalyticsEvents('AutoZone Catalog Part - Added', 'Quote from Mobile', 'AutoZone Catalog Part is Added.');
                if (buttonLabel == 'Remove') {
                    var isRemovedFlag = false;
                    quote.datalines.reverse().forEach(function (tempLine, index, source) {
                        if (tempLine.lineType == 'AZPart' && tempLine.partNumber == part.PartNumber && isRemovedFlag == false) {
                            isRemovedFlag = true;
                            self.partsTotal -= (tempLine.quantity_Hours * tempLine.price_Rate);
                            self.subTotal -= (tempLine.quantity_Hours * tempLine.price_Rate);
                            self.itemsAddedToCatalog -= tempLine.quantity_Hours;
                            source.splice(index, 1);
                        }
                    });
                    quote.datalines = quote.datalines.reverse();
                    return;
                }
                if (part.quantitySelected == 0) {
                    return;
                }
                var line = new NgTsSeed.QuoteLineModel();
                line.description = part.Description;
                if (part.Name != null) {
                    line.description += "-" + part.Name;
                }
                if (part.Notes != null) {
                    line.description += "-" + part.Notes;
                }
                line.lineNumber = quote.datalines.length + 1;
                line.lineType = "AZPart";
                line.name = part.elementName;
                line.price_Rate = self.getPartMarkupPrice(part.price);
                line.taxable = isTaxable;
                line.quantity_Hours = part.quantitySelected;
                line.partNumber = part.PartNumber;
                line.displayPartSkill = part.PartNumber;
                line.justAdded = true;
                if (buttonLabel == 'Add') {
                    quote.datalines.push(line);
                }
                else if (buttonLabel == 'Update') {
                    var isUpdatedFlag = false;
                    quote.datalines.reverse().forEach(function (tempLine, index, source) {
                        if (tempLine.lineType == 'AZPart' && tempLine.partNumber == part.PartNumber && isUpdatedFlag == false) {
                            quote.datalines[index] = line;
                            isUpdatedFlag = true;
                        }
                    });
                    quote.datalines = quote.datalines.reverse();
                }
                self.partsTotal += (line.quantity_Hours * line.price_Rate);
                self.subTotal += (line.quantity_Hours * line.price_Rate);
                self.itemsAddedToCatalog += line.quantity_Hours;
            };
            Controller.Make2DArray = function (list, width) {
                var rtnArray = [];
                for (var i = 0; i < list.length; i += width) {
                    rtnArray.push(list.slice(i, i + width));
                }
                return rtnArray;
            };
            Controller.prototype.isObject = function (source) {
                return source.constructor == Object;
            };
            Controller.prototype.isArray = function (source) {
                return source.constructor == Array;
            };
            Controller.prototype.getLaborRates = function () {
                var self = this;
                var laborRates = self.shop.laborRates || [];
                return laborRates;
            };
            Controller.prototype.getDefaultLaborRate = function (labor, rate) {
                var self = this;
                labor.Value.Rate = (this.shop.defaultLaborRate - 1);
                return labor.Value.Rate;
            };
            Controller.prototype.setLaborRate = function (event, labor) {
                labor.Value.Rate = $(event.target).val();
            };
            Controller.prototype.getPartMarkupPrice = function (price) {
                var self = this;
                return self.QuoteService.getPartMarkupPrice(price, self.shop);
            };
            Controller.prototype.onALLDATAComponentSelected = function (component, event, index) {
                this.ALLDATAComponentClicked(component);
            };
            Controller.prototype.onAutozoneComponentSelected = function (component, event, index) {
                this.AutoZoneComponentClicked(component);
            };
            Controller.$inject = ['$scope', 'CatalogDataService', 'adSpinnerService', '$timeout', 'QuoteService', '$modalInstance', 'adDialogService', 'quote', 'shop', '$rootScope', '$q', 'ShellService'];
            return Controller;
        }());
        Catalog.Controller = Controller;
    })(Catalog = NgTsSeed.Catalog || (NgTsSeed.Catalog = {}));
})(NgTsSeed || (NgTsSeed = {}));
var NgTsSeed;
(function (NgTsSeed) {
    var CatalogItemSelected;
    (function (CatalogItemSelected) {
        var CatalogItemSelectedController = (function () {
            function CatalogItemSelectedController($scope) {
                this.$scope = $scope;
                var self = this;
                this.$scope.vm = this;
                self.rates = self.$scope.rates;
                self.defaultRateIndex = self.$scope.defaultRateIndex;
                self.taxPartsDefault = self.$scope.taxPartsDefault;
                self.taxLaborDefault = self.$scope.taxLaborDefault;
                self.selectedLaborRate = -1;
                self.selectedLaborRateModel = 0;
                self.quantity = 0;
                self.hideQuantity = true;
                this.updateLabel();
            }
            CatalogItemSelectedController.prototype.updateLabel = function () {
                var self = this;
                if (self.rates == null) {
                    if (self.quantity == 0) {
                        self.buttonLabel = "Add";
                        self.hideQuantity = true;
                    }
                    else {
                        self.buttonLabel = "Remove";
                        self.hideQuantity = false;
                    }
                }
                else {
                    if (self.selectedLaborRate == -1) {
                        self.buttonLabel = "Add";
                    }
                    else {
                        self.buttonLabel = "Remove";
                    }
                }
            };
            CatalogItemSelectedController.prototype.updateQuantity = function () {
                var self = this;
                self.buttonLabel = "Update";
            };
            CatalogItemSelectedController.prototype.buttonClicked = function (item, isTaxable) {
                var self = this;
                if (self.buttonLabel == "Add") {
                    if (self.rates == null) {
                        self.quantity = 1;
                    }
                    else {
                        self.selectedLaborRate = self.defaultRateIndex;
                        self.selectedLaborRateModel = self.defaultRateIndex;
                    }
                }
                if (self.buttonLabel == "Remove") {
                    if (self.rates == null) {
                        self.quantity = 0;
                    }
                    else {
                        self.selectedLaborRate = -1;
                    }
                }
                if (self.rates == null) {
                    self.$scope.callback({ item: item, quantity: self.quantity, isTaxable: isTaxable, buttonLabel: self.buttonLabel });
                }
                else {
                    self.$scope.callback({ item: item, laborRateIndex: self.selectedLaborRateModel, isTaxable: isTaxable, buttonLabel: self.buttonLabel });
                }
                self.updateLabel();
            };
            CatalogItemSelectedController.prototype.updateTaxableFlag = function () {
                var self = this;
                if (self.buttonLabel != 'Add') {
                    self.buttonLabel = 'Update';
                }
            };
            CatalogItemSelectedController.$inject = ['$scope'];
            return CatalogItemSelectedController;
        }());
        CatalogItemSelected.CatalogItemSelectedController = CatalogItemSelectedController;
    })(CatalogItemSelected = NgTsSeed.CatalogItemSelected || (NgTsSeed.CatalogItemSelected = {}));
})(NgTsSeed || (NgTsSeed = {}));
var NgTsSeed;
(function (NgTsSeed) {
    var CatalogItemSelected;
    (function (CatalogItemSelected) {
        var Directive = (function () {
            function Directive($timeout, InputService) {
                this.$timeout = $timeout;
                this.InputService = InputService;
                this.restrict = 'AE';
                this.replace = true;
                this.templateUrl = 'adCatalogControl/partials/catalogItemSelected.html';
                this.controller = 'CatalogItemSelectedController';
                this.scope = {
                    'callback': '&',
                    'item': '=',
                    'rates': '=?',
                    'defaultRateIndex': '=?',
                    'taxPartsDefault': '=',
                    'taxLaborDefault': '=',
                    'rowIndex': '@'
                };
                this.link = LinkFunction.bind(this);
            }
            Directive.Factory = function ($timeout, InputService) {
                return new Directive($timeout, InputService);
            };
            Directive.$inject = ['$timeout', 'InputService'];
            return Directive;
        }());
        CatalogItemSelected.Directive = Directive;
        function LinkFunction(scope, element, attr, ctrl, transclude) {
            var _this = this, $timeout = this.$timeout;
        }
    })(CatalogItemSelected = NgTsSeed.CatalogItemSelected || (NgTsSeed.CatalogItemSelected = {}));
})(NgTsSeed || (NgTsSeed = {}));
var NgTsSeed;
(function (NgTsSeed) {
    var CommonDirectives = (function () {
        function CommonDirectives() {
        }
        CommonDirectives.Init = function (app, useScootch) {
            app.directive('redrawOnScroll', function () {
                function link(scope, element, attr) {
                    element.on('scroll', function () {
                        console.log('scrolling');
                        $('input').toggleClass('force-redraw');
                    });
                }
                return {
                    link: link
                };
            });
            app.directive('zoomToFit', function ($window) {
                var zoom = function (bodyWidth) {
                    var windowWidth = $(window).width();
                    var windowHeight = $(window).height();
                    var zoomLevel = windowWidth / bodyWidth;
                    if (windowHeight > windowWidth) {
                        zoomLevel = windowWidth / 800;
                    }
                    $('html').width(windowWidth);
                    $('body').css({ zoom: zoomLevel.toFixed(3) });
                    var zoomWindowHeight = (windowHeight / zoomLevel);
                    $('body').height(zoomWindowHeight);
                };
                function link(scope, element, attr) {
                    var targetWidth = attr.zoomToFit;
                    zoom(targetWidth);
                    $(window).resize(function () {
                        if (scope.resizeTO !== false) {
                            clearTimeout(scope.resizeTO);
                        }
                        scope.resizeTO = setTimeout(function () {
                            zoom(targetWidth);
                        }, 200);
                    });
                }
                return {
                    link: link
                };
            });
            if (useScootch) {
                app.directive('scootch', function () {
                    function findAncestor(elm, cls) {
                        while ((elm = elm.parentElement) && !elm.classList.contains((cls)))
                            ;
                        return elm;
                    }
                    function getDefaultElement(elm) {
                        var obj = findAncestor(elm, 'modal-body');
                        if (!obj) {
                            obj = elm.parentElement;
                        }
                        return obj;
                    }
                    var scootching = false;
                    var removeScootch = function () {
                        var space = document.getElementById('scootchSpace');
                        if (space) {
                            space.style.height = "0px";
                            space.remove();
                        }
                    };
                    var link = function link(scope, element, attr) {
                        var target = element[0];
                        var options = attr.scootch;
                        if (options !== "") {
                            options = JSON.parse(options);
                        }
                        if (!options) {
                            options = {
                                scrollTarget: getDefaultElement(target),
                                scrollClass: null,
                                spaceTarget: getDefaultElement(target),
                                spaceClass: null
                            };
                        }
                        else {
                            if (options.scrollClass) {
                                options.scrollTarget = findAncestor(target, options.scrollClass);
                            }
                            if (!options.scrollTarget) {
                                options.scrollTarget = getDefaultElement(target);
                            }
                            if (options.spaceClass) {
                                options.spaceTarget = findAncestor(target, options.spaceClass);
                            }
                            if (!options.spaceTarget) {
                                options.spaceTarget = options.scrollTarget;
                            }
                        }
                        target.addEventListener("focus", function () {
                            if (scootching) {
                                return;
                            }
                            scootching = true;
                            var holder = options.scrollTarget;
                            var targetRect = target.getBoundingClientRect();
                            var holderRect = holder.getBoundingClientRect();
                            var offset = targetRect.top + holder.scrollTop - holderRect.top;
                            var space = document.getElementById('scootchSpace');
                            if (space) {
                                space.style.height = offset + "px";
                            }
                            else {
                                space = document.createElement("div");
                                space.id = 'scootchSpace';
                                space.style.height = offset + "px";
                                options.spaceTarget.appendChild(space);
                            }
                            setTimeout(function () {
                                holder.style.maxHeight = holderRect.height + "px";
                                holder.scrollTop = offset;
                            }, 50);
                            setTimeout(function () {
                                scootching = false;
                            }, 500);
                        }, true);
                        target.addEventListener("blur", function () {
                            setTimeout(function () {
                                if (!scootching) {
                                    removeScootch();
                                }
                            }, 300);
                        }, true);
                    };
                    return {
                        link: link
                    };
                });
            }
        };
        return CommonDirectives;
    }());
    NgTsSeed.CommonDirectives = CommonDirectives;
})(NgTsSeed || (NgTsSeed = {}));
var NgTsSeed;
(function (NgTsSeed) {
    var CommonFilters = (function () {
        function CommonFilters() {
        }
        CommonFilters.Init = function (app) {
            app.filter('tel', function () {
                return function (tel) {
                    if (!tel) {
                        return '';
                    }
                    var value = tel.toString().trim().replace(/^\+/, '');
                    if (value.match(/[^0-9]/)) {
                        return tel;
                    }
                    var country, city, number;
                    switch (value.length) {
                        case 10:
                            country = 1;
                            city = value.slice(0, 3);
                            number = value.slice(3);
                            break;
                        case 11:
                            country = value[0];
                            city = value.slice(1, 4);
                            number = value.slice(4);
                            break;
                        case 12:
                            country = value.slice(0, 3);
                            city = value.slice(3, 5);
                            number = value.slice(5);
                            break;
                        default:
                            return tel;
                    }
                    if (country == 1) {
                        country = "";
                    }
                    number = number.slice(0, 3) + '-' + number.slice(3);
                    return (country + " (" + city + ") " + number).trim();
                };
            });
        };
        return CommonFilters;
    }());
    NgTsSeed.CommonFilters = CommonFilters;
})(NgTsSeed || (NgTsSeed = {}));
var Components;
(function (Components) {
    var Login;
    (function (Login) {
        var Controller = (function () {
            function Controller($scope, $cookies, AuthorizationService, DialogService) {
                this.$scope = $scope;
                this.$cookies = $cookies;
                this.AuthorizationService = AuthorizationService;
                this.DialogService = DialogService;
                this.betaCookieString = "quoteBetaAccepted";
                this.credentials = {};
                var self = this;
                var betaCookieValue = !!this.$cookies[this.betaCookieString] || false;
                self.showBetaNotice = !betaCookieValue;
                $scope.vm = self;
            }
            Controller.prototype.authenticate = function () {
                var self = this;
                self.credentials.username = $('input[name=username]').val();
                self.credentials.password = $('input[name=password]').val();
                if (self.showBetaNotice && !self.betaAccepted) {
                    self.DialogService.showAlert({
                        title: 'ALLDATA Quote',
                        message: 'Please accept the BETA agreement'
                    });
                }
                else if (self.credentials.username.length > 0 && self.credentials.password.length > 0) {
                    self.AuthorizationService.authenticate(self.credentials);
                    this.$cookies[this.betaCookieString] = 'true';
                }
                else {
                    alert('Please make sure that you filled the username and password fields.');
                }
            };
            Controller.$inject = ['$scope', '$cookies', 'AuthorizationService', 'adDialogService'];
            return Controller;
        }());
        Login.Controller = Controller;
    })(Login = Components.Login || (Components.Login = {}));
})(Components || (Components = {}));
var NgTsSeed;
(function (NgTsSeed) {
    var EmailModel = (function () {
        function EmailModel() {
        }
        return EmailModel;
    }());
    NgTsSeed.EmailModel = EmailModel;
})(NgTsSeed || (NgTsSeed = {}));
var NgTsSeed;
(function (NgTsSeed) {
    var QuoteLineModel = (function () {
        function QuoteLineModel() {
        }
        return QuoteLineModel;
    }());
    NgTsSeed.QuoteLineModel = QuoteLineModel;
})(NgTsSeed || (NgTsSeed = {}));
var NgTsSeed;
(function (NgTsSeed) {
    var QuoteModel = (function () {
        function QuoteModel(shopId) {
            this.datalines = [];
            this.quoteExpired = false;
            this.shopId = shopId;
            this.resetModel();
        }
        QuoteModel.prototype.resetModel = function () {
            this.totalParts = 0;
            this.totalLabor = 0;
            this.totalFee = 0;
            this.totalSub = 0;
            this.totalLaborTax = 0;
            this.totalPartsTax = 0;
            this.totalTax = 0;
            this.total = 0;
        };
        return QuoteModel;
    }());
    NgTsSeed.QuoteModel = QuoteModel;
})(NgTsSeed || (NgTsSeed = {}));
var NgTsSeed;
(function (NgTsSeed) {
    var ShopModel = (function () {
        function ShopModel() {
            this.setShopDisplayInfo();
        }
        ShopModel.prototype.setShopDisplayInfo = function () {
            var self = this;
            self.display_partsMarkup = Number((self.partsMarkup * 100).toFixed(2)) || 0;
            self.display_partsTax = Number((self.partsTax * 100).toFixed(1)) || 0;
            self.display_laborTax = Number((self.laborTax * 100).toFixed(1)) || 0;
            self.printDateTime = true;
            self.laborRates = [self.laborRate1, self.laborRate2, self.laborRate3];
        };
        ShopModel.prototype.setInitialSetupState = function () {
            var self = this;
            self.laborRate1 = null;
            self.laborRate2 = null;
            self.laborRate3 = null;
            self.partsMarkup = null;
            self.taxPartsDefault = null;
            self.taxLaborDefault = null;
            self.display_partsMarkup = null;
            self.display_partsTax = null;
            self.display_laborTax = null;
        };
        return ShopModel;
    }());
    NgTsSeed.ShopModel = ShopModel;
})(NgTsSeed || (NgTsSeed = {}));
var NgTsSeed;
(function (NgTsSeed) {
    var Quote;
    (function (Quote) {
        var FormErrorController = (function () {
            function FormErrorController($scope) {
                this.$scope = $scope;
                $scope.focusField = this.focusField;
            }
            FormErrorController.prototype.focusField = function ($event) {
                var element = $event.target, field = $(element).closest('.form-group').find('input').first();
                field.focus();
            };
            FormErrorController.$inject = ['$scope'];
            return FormErrorController;
        }());
        Quote.FormErrorController = FormErrorController;
    })(Quote = NgTsSeed.Quote || (NgTsSeed.Quote = {}));
})(NgTsSeed || (NgTsSeed = {}));
var NgTsSeed;
(function (NgTsSeed) {
    var AddCustomLaborModalController = (function () {
        function AddCustomLaborModalController($scope, $modalInstance, shop, adDialogService, ShellService) {
            this.$scope = $scope;
            this.$modalInstance = $modalInstance;
            this.shop = shop;
            this.adDialogService = adDialogService;
            this.ShellService = ShellService;
            var self = this;
            $scope.vm = self;
            self.buttons = self.initButtons();
            self.customLabor = new NgTsSeed.QuoteLineModel;
            self.customLaborRates = [];
            self.customLabor.description = "";
            self.customLabor.taxable = this.shop.taxLaborDefault;
            for (var i = 0; i < this.shop.laborRates.length; i++)
                self.customLaborRates[i] = { value: this.shop.laborRates[i], label: '$' + this.shop.laborRates[i] };
            self.customLabor.price_Rate = this.shop.laborRates[this.shop.defaultLaborRate - 1];
        }
        AddCustomLaborModalController.prototype.initButtons = function () {
            var self = this;
            return [
                {
                    title: 'Cancel',
                    onClick: function () {
                        self.$modalInstance.dismiss();
                    }
                },
                {
                    title: 'Add',
                    onClick: function () {
                        if (self.formScope.customLaborForm.$invalid === true) {
                            self.adDialogService.showAlert({
                                title: 'Custom Labor Information Not Saved',
                                message: "Please make sure the form does not contain any invalid entries and try again.",
                                buttons: ["OK"]
                            }).result.then(function () {
                                var field = "";
                                var errors = self.formScope.customLaborForm.$error;
                                for (var props in errors) {
                                    field = errors[props][0].$name;
                                    break;
                                }
                                console.log(field);
                                angular.element('*[name=' + field + ']').focus();
                            });
                            ;
                            return;
                        }
                        self.customLabor.quantity_Hours = self.customLabor.quantity_Hours || 0;
                        self.customLabor.price_Rate = self.customLabor.price_Rate || 0;
                        self.ShellService.sendAnalyticsEvents('Custom Labor - Added', 'Quote from Mobile', 'Custom labor is added');
                        self.$modalInstance.close(self.customLabor);
                    }
                }
            ];
        };
        AddCustomLaborModalController.prototype.setFormScope = function (formScope) {
            this.formScope = formScope;
        };
        AddCustomLaborModalController.prototype.updateLaborExtended = function () {
            var self = this;
            var rate = self.customLabor.price_Rate;
            var duration = self.customLabor.quantity_Hours;
            if (rate > 0 && duration > 0) {
                self.customLabor.extended = Number(rate * duration);
            }
        };
        AddCustomLaborModalController.$inject = ['$scope', '$modalInstance', 'shop', 'adDialogService', 'ShellService'];
        return AddCustomLaborModalController;
    }());
    NgTsSeed.AddCustomLaborModalController = AddCustomLaborModalController;
})(NgTsSeed || (NgTsSeed = {}));
var NgTsSeed;
(function (NgTsSeed) {
    var AddCustomPartModalController = (function () {
        function AddCustomPartModalController($scope, $modalInstance, shop, adDialogService, ShellService) {
            this.$scope = $scope;
            this.$modalInstance = $modalInstance;
            this.shop = shop;
            this.adDialogService = adDialogService;
            this.ShellService = ShellService;
            var self = this;
            $scope.vm = self;
            self.buttons = self.initButtons();
            self.customPart = {};
            self.customPart.description = "";
            self.customPart.taxable = this.shop.taxPartsDefault;
        }
        AddCustomPartModalController.prototype.initButtons = function () {
            var self = this;
            return [
                {
                    title: 'Cancel',
                    onClick: function () {
                        self.$modalInstance.dismiss();
                    }
                },
                {
                    title: 'Add',
                    onClick: function () {
                        if (self.formScope.customPartForm.$invalid === true) {
                            self.adDialogService.showAlert({
                                title: 'Custom Part Information Not Saved',
                                message: "Please make sure the form does not contain any invalid entries and try again.",
                                buttons: ["OK"]
                            }).result.then(function () {
                                var field = "";
                                var errors = self.formScope.customPartForm.$error;
                                for (var props in errors) {
                                    field = errors[props][0].$name;
                                    break;
                                }
                                console.log(field);
                                angular.element('*[name=' + field + ']').focus();
                            });
                            ;
                            return;
                        }
                        self.customPart.quantity_Hours = self.customPart.quantity_Hours || 0;
                        self.customPart.price_Rate = self.customPart.price_Rate || 0;
                        self.ShellService.sendAnalyticsEvents('Custom Part - Added', 'Quote from Mobile', 'Custom part is added');
                        self.$modalInstance.close(self.customPart);
                    }
                }
            ];
        };
        AddCustomPartModalController.prototype.updatePartExtended = function () {
            var self = this;
            if (self.customPart.price_Rate > 0 && self.customPart.quantity_Hours > 0) {
                self.customPart.extended = Number(self.customPart.price_Rate * self.customPart.quantity_Hours);
            }
        };
        AddCustomPartModalController.prototype.setFormScope = function (formScope) {
            this.formScope = formScope;
        };
        AddCustomPartModalController.$inject = ['$scope', '$modalInstance', 'shop', 'adDialogService', 'ShellService'];
        return AddCustomPartModalController;
    }());
    NgTsSeed.AddCustomPartModalController = AddCustomPartModalController;
})(NgTsSeed || (NgTsSeed = {}));
var NgTsSeed;
(function (NgTsSeed) {
    var AddFlatFeeModalController = (function () {
        function AddFlatFeeModalController($scope, $modalInstance, adDialogService, ShellService) {
            this.$scope = $scope;
            this.$modalInstance = $modalInstance;
            this.adDialogService = adDialogService;
            this.ShellService = ShellService;
            var self = this;
            $scope.vm = self;
            self.flatFee = new NgTsSeed.QuoteLineModel();
            self.flatFee.name = "";
            self.flatFee.description = "";
            self.buttons = self.initButtons();
        }
        AddFlatFeeModalController.prototype.initButtons = function () {
            var self = this;
            return [
                {
                    title: 'Cancel',
                    onClick: function () {
                        self.$modalInstance.dismiss();
                    }
                },
                {
                    title: 'Add',
                    onClick: function () {
                        if (self.formScope.flatFeeForm.$invalid === true) {
                            self.adDialogService.showAlert({
                                title: 'Flat Fee Form Information Not Saved',
                                message: "Please make sure the form does not contain any invalid entries and try again.",
                                buttons: ["OK"]
                            }).result.then(function () {
                                var field = "";
                                var errors = self.formScope.flatFeeForm.$error;
                                for (var props in errors) {
                                    field = errors[props][0].$name;
                                    break;
                                }
                                console.log(field);
                                angular.element('*[name=' + field + ']').focus();
                            });
                            ;
                            return;
                        }
                        self.ShellService.sendAnalyticsEvents('Flat Fee - Added', 'Quote from Mobile', 'Flat fee is added');
                        self.$modalInstance.close(self.flatFee);
                    }
                }
            ];
        };
        AddFlatFeeModalController.prototype.setFormScope = function (formScope) {
            this.formScope = formScope;
        };
        AddFlatFeeModalController.$inject = ['$scope', '$modalInstance', 'adDialogService', 'ShellService'];
        return AddFlatFeeModalController;
    }());
    NgTsSeed.AddFlatFeeModalController = AddFlatFeeModalController;
})(NgTsSeed || (NgTsSeed = {}));
var NgTsSeed;
(function (NgTsSeed) {
    var AddNoteModalController = (function () {
        function AddNoteModalController($scope, $modalInstance, ShellService) {
            this.$scope = $scope;
            this.$modalInstance = $modalInstance;
            this.ShellService = ShellService;
            var self = this;
            $scope.vm = self;
            self.buttons = self.initButtons();
        }
        AddNoteModalController.prototype.initButtons = function () {
            var self = this;
            return [
                {
                    title: 'Cancel',
                    onClick: function () {
                        self.$modalInstance.dismiss();
                    }
                },
                {
                    title: 'Add',
                    onClick: function () {
                        self.ShellService.sendAnalyticsEvents('Note - Added', 'Quote from Mobile', 'Custom note is added');
                        self.$modalInstance.close(self.note);
                    }
                }
            ];
        };
        AddNoteModalController.$inject = ['$scope', '$modalInstance', 'ShellService'];
        return AddNoteModalController;
    }());
    NgTsSeed.AddNoteModalController = AddNoteModalController;
})(NgTsSeed || (NgTsSeed = {}));
var NgTsSeed;
(function (NgTsSeed) {
    var AdditionalInfoModalController = (function () {
        function AdditionalInfoModalController($scope, $rootScope, $modalInstance, quote, adDialogService, ShellService, ManageLoginService, $filter, FlyoutDialogService, shop) {
            this.$scope = $scope;
            this.$rootScope = $rootScope;
            this.$modalInstance = $modalInstance;
            this.quote = quote;
            this.adDialogService = adDialogService;
            this.ShellService = ShellService;
            this.ManageLoginService = ManageLoginService;
            this.$filter = $filter;
            this.FlyoutDialogService = FlyoutDialogService;
            this.shop = shop;
            this.flyout = null;
            this.phoneEntered = false;
            this.firstNameEntered = false;
            this.lastNameEntered = false;
            var self = this;
            $scope.vm = self;
            self.buttons = self.initButtons();
            if (ShellService.moEnabled) {
                $scope.$watch('vm.quote.customerPhoneNumber', function (newVal, oldVal) {
                    if (newVal !== undefined && newVal != '')
                        newVal = self.$filter('tel')(newVal);
                    if (oldVal !== undefined && oldVal != '')
                        oldVal = self.$filter('tel')(oldVal);
                    if (newVal == '')
                        self.phoneEntered = false;
                    if (newVal !== undefined && newVal != oldVal && newVal != '' && self.phoneEntered == true) {
                        self.lookupByPhone(newVal);
                    }
                });
                $scope.$watch('vm.quote.customerFirstName', function (newVal, oldVal) {
                    if (newVal == '')
                        self.firstNameEntered = false;
                    if (newVal !== undefined && angular.lowercase(newVal) != angular.lowercase(oldVal) && newVal != '' && self.firstNameEntered == true) {
                        self.lookupByName(newVal, 'firstName');
                    }
                });
                $scope.$watch('vm.quote.customerLastName', function (newVal, oldVal) {
                    if (newVal == '')
                        self.lastNameEntered = false;
                    if (newVal !== undefined && angular.lowercase(newVal) != angular.lowercase(oldVal) && newVal != '' && self.lastNameEntered == true) {
                        self.lookupByName(newVal, 'lastName');
                    }
                });
            }
        }
        AdditionalInfoModalController.prototype.initButtons = function () {
            var self = this;
            return [
                {
                    title: 'Cancel',
                    onClick: function () {
                        self.$modalInstance.dismiss();
                    }
                },
                {
                    title: 'Save',
                    onClick: function () {
                        if (self.formScope.customerInfoForm.$invalid === true) {
                            self.adDialogService.showAlert({
                                title: 'Customer Information Not Saved',
                                message: "Please make sure the form does not contain any invalid entries and try again.",
                                buttons: ["OK"]
                            }).result.then(function () {
                                var field = "";
                                var errors = self.formScope.customerInfoForm.$error;
                                for (var props in errors) {
                                    field = errors[props][0].$name;
                                    break;
                                }
                                angular.element('*[name=' + field + ']').focus();
                            });
                            return;
                        }
                        self.ShellService.sendAnalyticsEvents('Customer Details - Saved', 'Quote from Mobile', 'Customer details saved.');
                        self.$modalInstance.close(self.quote);
                    }
                }
            ];
        };
        AdditionalInfoModalController.prototype.setFormScope = function (formScope) {
            this.formScope = formScope;
        };
        AdditionalInfoModalController.prototype.lookupByPhone = function (phone) {
            var self = this;
            if (self.flyout)
                self.flyout.hide();
            self.isLoading = true;
            self.flyout = self.FlyoutDialogService.showFlyout($('input[name=phone]'), 'quotes/partials/customerFlyout.html', {
                placement: 'bottom', anchor: 'left'
            }, self.$scope);
            var smsAuthorization = btoa(self.shop.manageOnlineUsername + ':::' + atob(self.shop.manageOnlinePassword));
            setTimeout(function () {
                self.ManageLoginService.lookupByPhone(phone, smsAuthorization)
                    .then(function (response) {
                    self.customerInfo = response.data;
                    self.isLoading = false;
                }, function (error) {
                    if (self.flyout)
                        self.flyout.hide();
                });
            }, 1000);
        };
        AdditionalInfoModalController.prototype.lookupByName = function (name, inputCtrlName) {
            var self = this;
            if (self.flyout)
                self.flyout.hide();
            self.isLoading = true;
            self.flyout = self.FlyoutDialogService.showFlyout($('input[name=' + inputCtrlName + ']'), 'quotes/partials/customerFlyout.html', {
                placement: 'bottom', anchor: 'left'
            }, self.$scope);
            var smsAuthorization = btoa(self.shop.manageOnlineUsername + ':::' + atob(self.shop.manageOnlinePassword));
            setTimeout(function () {
                self.ManageLoginService.lookupByName(name, smsAuthorization)
                    .then(function (response) {
                    self.customerInfo = response.data;
                    self.isLoading = false;
                }, function (error) {
                    if (self.flyout)
                        self.flyout.hide();
                });
            }, 1000);
        };
        AdditionalInfoModalController.prototype.selectInformation = function (info) {
            var self = this;
            self.flyout.hide();
            self.phoneEntered = false;
            self.quote.customerPhoneNumber = info.cellPhone;
            self.quote.customerFirstName = info.firstName;
            self.quote.customerLastName = info.lastName;
            self.quote.customerEmail = info.email;
            self.quote.customerStreetAddress1 = info.addressLine1;
            self.quote.customerStreetAddress2 = info.addressLine2;
            self.quote.customerCity = info.city;
            self.quote.customerState = info.state;
            self.quote.customerZipcode = info.zipcode;
        };
        AdditionalInfoModalController.prototype.phoneChange = function () {
            this.phoneEntered = true;
        };
        AdditionalInfoModalController.prototype.firstNameChange = function () {
            this.firstNameEntered = true;
        };
        AdditionalInfoModalController.prototype.lastNameChange = function () {
            this.lastNameEntered = true;
        };
        AdditionalInfoModalController.$inject = ['$scope', '$rootScope', '$modalInstance', 'quote', 'adDialogService', 'ShellService', 'ManageLoginService', '$filter', 'adFlyoutDialogService', 'shop'];
        return AdditionalInfoModalController;
    }());
    NgTsSeed.AdditionalInfoModalController = AdditionalInfoModalController;
})(NgTsSeed || (NgTsSeed = {}));
var NgTsSeed;
(function (NgTsSeed) {
    var customerInfoModalController = (function () {
        function customerInfoModalController($scope, $modalInstance, ShellService, $rootScope, $timeout) {
            this.$scope = $scope;
            this.$modalInstance = $modalInstance;
            this.ShellService = ShellService;
            this.$rootScope = $rootScope;
            this.$timeout = $timeout;
            this.isLoading = true;
            var self = this;
            $scope.vm = self;
            self.buttons = self.initButtons();
            self.$timeout(function () {
                self.isLoading = false;
            }, 4000);
        }
        customerInfoModalController.prototype.initButtons = function () {
            var self = this;
            return [
                {
                    title: 'Cancel',
                    onClick: function () {
                        self.$modalInstance.dismiss();
                    }
                }
            ];
        };
        customerInfoModalController.prototype.selectCustomer = function (customerInfo) {
            var self = this;
            console.log(customerInfo);
            self.$rootScope.$emit('CUSTOMER_INFO_DISPLAY', { data: customerInfo });
            self.$modalInstance.dismiss();
        };
        customerInfoModalController.$inject = ['$scope', '$modalInstance', 'ShellService', '$rootScope', '$timeout'];
        return customerInfoModalController;
    }());
    NgTsSeed.customerInfoModalController = customerInfoModalController;
})(NgTsSeed || (NgTsSeed = {}));
var NgTsSeed;
(function (NgTsSeed) {
    var EmailQuoteModalController = (function () {
        function EmailQuoteModalController($scope, $modalInstance, ShellService, shop, customerEmail) {
            this.$scope = $scope;
            this.$modalInstance = $modalInstance;
            this.ShellService = ShellService;
            this.shop = shop;
            this.customerEmail = customerEmail;
            var self = this;
            $scope.vm = self;
            self.emailModel = new NgTsSeed.EmailModel();
            self.emailModel.address = customerEmail;
            self.emailModel.message = self.shop.emailMessage;
            self.buttons = self.initButtons();
        }
        EmailQuoteModalController.prototype.initButtons = function () {
            var self = this;
            return [
                {
                    title: 'Cancel',
                    onClick: function () {
                        self.$modalInstance.dismiss();
                    }
                },
                {
                    title: 'Send',
                    onClick: function () {
                        if (self.formScope.form.$valid === true) {
                            self.ShellService.sendAnalyticsEvents('Quote Email - Sent', 'Quote from Mobile', 'Quote Email is sent.');
                            self.$modalInstance.close(self.emailModel);
                        }
                    }
                }
            ];
        };
        EmailQuoteModalController.prototype.setFormScope = function (formScope) {
            this.formScope = formScope;
        };
        EmailQuoteModalController.$inject = ['$scope', '$modalInstance', 'ShellService', 'shop', 'customerEmail'];
        return EmailQuoteModalController;
    }());
    NgTsSeed.EmailQuoteModalController = EmailQuoteModalController;
})(NgTsSeed || (NgTsSeed = {}));
var NgTsSeed;
(function (NgTsSeed) {
    var Preview;
    (function (Preview) {
        var Controller = (function () {
            function Controller($scope, spinner, PDF_VIEWER_HOST, PDF_VIEWER_PATH, ADCONNECT_HOST, QUOTE_PATH, $http, adSpinnerService, $modalInstance) {
                this.$scope = $scope;
                this.spinner = spinner;
                this.PDF_VIEWER_HOST = PDF_VIEWER_HOST;
                this.PDF_VIEWER_PATH = PDF_VIEWER_PATH;
                this.ADCONNECT_HOST = ADCONNECT_HOST;
                this.QUOTE_PATH = QUOTE_PATH;
                this.$http = $http;
                this.adSpinnerService = adSpinnerService;
                this.$modalInstance = $modalInstance;
                this.BASE_URL = '/shops/{shopId}/quotes/{quoteId}/pdf&auto=false';
                var self = this;
                $scope.host = PDF_VIEWER_HOST;
                $scope.path = PDF_VIEWER_PATH;
                $scope.quoteHost = ADCONNECT_HOST;
                $scope.quotePath = QUOTE_PATH;
                $scope.pdfViewerCallback = self.pdfViewerCallback.bind(self);
                $scope.buttons = [
                    { title: 'Cancel', onClick: self.dismiss.bind(this) },
                    { title: 'Print', onClick: self.printPDF }
                ];
            }
            Controller.prototype.getUrl = function (quote) {
                var self = this, url = self.$scope.quoteHost + self.$scope.quotePath + self.BASE_URL;
                url = url.replace('{shopId}', quote.shopId.toString());
                url = url.replace('{quoteId}', quote.quoteId.toString());
                return url;
            };
            Controller.prototype.dismiss = function () {
                var self = this;
                self.$modalInstance.dismiss();
            };
            Controller.prototype.printPDF = function () {
                var frame = $('iframe.pdf-viewer').get(0);
                if (frame === undefined)
                    return;
                frame = frame.contentWindow || frame;
                frame.print();
            };
            Controller.prototype.strToArrayBuffer = function (data) {
                var buf = new ArrayBuffer(data.length * 2), bufView = new Uint16Array(buf), strLen = data.length;
                for (var index = 0; index < strLen; index++) {
                    bufView[index] = data.charCodeAt(index);
                }
                return bufView;
            };
            Controller.prototype.pdfViewerCallback = function (quote) {
                var self = this, file = self.getUrl(quote);
                $('.modal-dialog').addClass('pdf-viewer');
                self.frame = window.frames[0].frameElement.contentWindow || null;
                if ('PDFViewerApplication' in self.frame === false) {
                    alert('An unexpected error ocurred.');
                    return;
                }
                self.frame.PDFViewerApplication.setTitleUsingUrl(file);
                file = file.substr(0, file.indexOf('&auto'));
                self.adSpinnerService.start();
                self.$http.get(file, { withCredentials: true, headers: { 'Accept': 'application/pdf' } })
                    .then(function (response) {
                    var data = self.strToArrayBuffer(response.data);
                    self.frame.PDFViewerApplication.open(new Uint8Array(data), 0);
                })
                    .catch(function (error) { })
                    .finally(function () {
                    self.adSpinnerService.end();
                });
            };
            Controller.$inject = ['$scope', 'adSpinnerService', 'PDF_VIEWER_HOST', 'PDF_VIEWER_PATH', 'ADCONNECT_HOST', 'QUOTE_PATH', '$http', 'adSpinnerService', '$modalInstance'];
            return Controller;
        }());
        Preview.Controller = Controller;
    })(Preview = NgTsSeed.Preview || (NgTsSeed.Preview = {}));
})(NgTsSeed || (NgTsSeed = {}));
var NgTsSeed;
(function (NgTsSeed) {
    var Preview;
    (function (Preview) {
        var Directive = (function () {
            function Directive($compile) {
                this.$compile = $compile;
                this.replace = true;
                this.restrict = 'E';
                this.templateUrl = 'quotes/partials/printPreviewTemplate.html';
                this.controller = 'PrintPreviewController';
                this.scope = {
                    vm: '='
                };
                this.link = LinkFunction.bind(this);
            }
            Directive.Factory = function ($compile) {
                return new Directive($compile);
            };
            Directive.$inject = ['$compile'];
            return Directive;
        }());
        Preview.Directive = Directive;
        function LinkFunction(scope, element, attr, ctrl) {
            scope.$watch('vm.printPreviewModalShown', function (newVal, oldVal, scope) {
                if (newVal === true && element) {
                    var el = '<pdf-viewer host="host" path="path" url="getUrl(vm.quote)" callback="pdfViewerCallback"></pdf-viewer>', compiledEl = this.$compile(el)(scope);
                    element.find('.modal-body').children().remove();
                    element.find('.modal-body').append(compiledEl);
                }
            }.bind(this));
        }
    })(Preview = NgTsSeed.Preview || (NgTsSeed.Preview = {}));
})(NgTsSeed || (NgTsSeed = {}));
var NgTsSeed;
(function (NgTsSeed) {
    var QuoteBannerController = (function () {
        function QuoteBannerController($scope, ShellService) {
            this.$scope = $scope;
            this.ShellService = ShellService;
            $scope.vm = this;
            var self = this;
        }
        QuoteBannerController.prototype.newQuote = function () {
            this.ShellService.goToNewQuote();
        };
        QuoteBannerController.$inject = ['$scope', 'ShellService'];
        return QuoteBannerController;
    }());
    NgTsSeed.QuoteBannerController = QuoteBannerController;
})(NgTsSeed || (NgTsSeed = {}));
var NgTsSeed;
(function (NgTsSeed) {
    var QuoteBannerDirective = (function () {
        function QuoteBannerDirective() {
        }
        QuoteBannerDirective.Init = function () {
            return {
                restrict: 'CE',
                replace: true,
                templateUrl: 'quotes/partials/quoteBannerDisplay.html',
                controller: 'QuoteBannerController',
                scope: {}
            };
        };
        return QuoteBannerDirective;
    }());
    NgTsSeed.QuoteBannerDirective = QuoteBannerDirective;
})(NgTsSeed || (NgTsSeed = {}));
var NgTsSeed;
(function (NgTsSeed) {
    var QuoteController = (function () {
        function QuoteController($scope, $rootScope, $timeout, $state, $stateParams, $http, $q, $filter, quoteDataSource, ngTableParams, ShellService, AuthorizationService, adDialogService, spinner, QuoteService, FlyoutDialogService) {
            this.$scope = $scope;
            this.$rootScope = $rootScope;
            this.$timeout = $timeout;
            this.$state = $state;
            this.$stateParams = $stateParams;
            this.$http = $http;
            this.$q = $q;
            this.$filter = $filter;
            this.quoteDataSource = quoteDataSource;
            this.ngTableParams = ngTableParams;
            this.ShellService = ShellService;
            this.AuthorizationService = AuthorizationService;
            this.adDialogService = adDialogService;
            this.spinner = spinner;
            this.QuoteService = QuoteService;
            this.FlyoutDialogService = FlyoutDialogService;
            this.readOnly = false;
            this.customPart = [];
            this.customLabor = [];
            this.flatFee = [];
            this.note = [];
            var self = this;
            self.showAddItems = false;
            self.headerButtonType = 'none';
            if (self.$state.is(NgTsSeed.Resources.Routes.newQuote)) {
                self.quote = new NgTsSeed.QuoteModel(self.ShellService.shop.shopId);
                if (self.$stateParams.engine) {
                    try {
                        var engine = JSON.parse(self.$stateParams.engine);
                        self.selectedCar = engine;
                        self.selectVehicle(engine);
                    }
                    catch (e) {
                    }
                }
            }
            else {
                self.loadQuoteData(self.$stateParams.quoteId);
            }
            self.initActions();
            self.ShellService.needsSave = false;
            $scope.vm = this;
            self.$timeout(function () {
                $scope.$watch('vm.quote.datalines', function () {
                    if (!self.quote)
                        return;
                    self.QuoteService.updateTotals(self.quote, self.ShellService.shop);
                }, true);
            }, 0);
            self.$scope.$watch('vm.readOnly', function (isReadOnly) {
                self.headerButtonType = isReadOnly ? 'none' : 'action';
            });
            if (typeof (self.quote) !== 'undefined') {
                if ($rootScope.customerInfoDialogFlag && self.quote.vehicleVIN && ShellService.moEnabled) {
                    self.showCustomerInformationDialog();
                    self.$rootScope.$emit('GET_CUSTOMER_INFO_USING_VIN', { data: self.quote.vehicleVIN });
                    $rootScope.customerInfoDialogFlag = false;
                }
            }
            $rootScope.$on('CUSTOMER_INFO_DISPLAY', function (event, response) {
                self.quote.customerFirstName = response.data.firstName;
                self.quote.customerLastName = response.data.lastName;
                self.quote.customerStreetAddress1 = response.data.addressLine1;
                self.quote.customerStreetAddress2 = response.data.addressLine2;
                self.quote.customerCity = response.data.city;
                self.quote.customerState = response.data.state;
                self.quote.customerZipcode = response.data.zipcode;
                self.quote.customerPhoneNumber = response.data.cellPhone;
                self.quote.customerEmail = response.data.email;
            });
        }
        QuoteController.prototype.initActions = function () {
            var self = this;
            self.quoteActions = [
                {
                    title: 'Share Quote',
                    icon: 'appIcon-email',
                    onSelect: function () {
                        self.ShellService.sendAnalyticsEvents('Quote Email - Tapped', 'Quote from Mobile', 'Quote Email is tapped.');
                        self.emailQuote();
                    }
                }
            ];
        };
        QuoteController.prototype.cancelQuote = function () {
            var self = this;
            self.ShellService.goToSavedQuotes();
        };
        QuoteController.prototype.loadQuoteData = function (quoteId) {
            var self = this;
            self.spinner.start();
            if (quoteId) {
                self.QuoteService.getQuote(self.ShellService.shop.shopId, quoteId)
                    .then(function (response) {
                    self.quote = response.data;
                    self.updateTotals();
                })
                    .catch(function (response) {
                    self.adDialogService.showAlert({
                        message: "An error ocurred while retrieving the quote information: " + response,
                        title: "An error occurred"
                    });
                })
                    .finally(function () {
                    self.spinner.end();
                });
            }
        };
        QuoteController.prototype.vehicleSelected = function () {
            var self = this;
            return (self.quote && self.quote.vehicleDescription && self.quote.vehicleDescription.length > 0);
        };
        QuoteController.prototype.saveQuote = function () {
            var self = this, shop = this.ShellService.shop;
            for (var i = 0; i < self.quote.datalines.length; ++i) {
                var dataline = self.quote.datalines[i], lineType = dataline.lineType.toLowerCase();
                dataline.lineNumber = i;
                dataline.itemName = dataline.name || dataline.itemName;
                dataline.price_Rate = dataline.price === undefined ? dataline.price_Rate : dataline.price;
                dataline.quantity_Hours = dataline.quantity === undefined ? dataline.quantity_Hours : dataline.quantity;
            }
            if (self.quote.quoteId == null || self.quote.quoteId < 0) {
                self.quoteDataSource.quoteService.storeNewQuote(self.quote).then(function (response) {
                    self.QuoteService.getQuote(self.ShellService.shop.shopId, self.quote.quoteId).then(function (response) {
                        self.quote.quoteNumber = response.data.quoteNumber;
                    });
                    self.adDialogService.showAlert({
                        title: "Quote Saved",
                        message: "Your quote has been saved."
                    });
                    self.ShellService.needsSave = false;
                }).catch(function (response) {
                    self.ShellService.needsSave = true;
                }).finally(function () {
                    self.updateTotals();
                });
            }
            else {
                self.quoteDataSource.quoteService.updateQuote(self.quote).then(function (data, headers) {
                    self.adDialogService.showAlert({
                        title: "Quote Saved",
                        message: "Your quote has been saved."
                    });
                    self.ShellService.needsSave = false;
                }).catch(function (failReason) {
                    self.ShellService.needsSave = true;
                }).finally(function () {
                    self.updateTotals();
                });
            }
        };
        QuoteController.prototype.printQuote = function () {
            var self = this;
            if (!self.quote.quoteId) {
                return;
            }
            Diagnostics.NativeService.getUUID(function (uuid) {
                window.open('https://' + uuid + ':ALLDATA@repairinfo.beta.alldata.com/ADConnect/v4/pro/quote/shops/' + self.ShellService.shop.shopId + '/quotes/' + self.quote.quoteId + '/pdf');
            });
        };
        QuoteController.prototype.viewQuote = function (quote) {
            var self = this;
            self.ShellService.goToQuote(quote);
        };
        QuoteController.prototype.deleteQuote = function () {
            var self = this;
            this.quoteDataSource.quoteService.deleteQuote(self.quote);
        };
        QuoteController.prototype.showAdditionalInformation = function () {
            var self = this;
            self.adDialogService.showDialog({
                url: 'quotes/partials/additionalInfoModal.html',
                controller: 'AdditionalInfoModalController',
                resolve: {
                    quote: function () {
                        return angular.copy(self.quote);
                    },
                    shop: function () {
                        return self.ShellService.shop;
                    }
                }
            }).result.then(function (quote) {
                angular.extend(self.quote, quote);
                self.ShellService.needsSave = true;
            });
        };
        QuoteController.prototype.addPartsLabor = function () {
            var self = this;
            if (self.quote.vehicleDescription == null || self.quote.vehicleDescription.length == 0) {
                self.adDialogService.showDialog({
                    url: 'quotes/partials/selectVehicleModal.html',
                    controller: 'SelectVehicleModalController',
                    size: 'lg',
                    resolve: {
                        quote: function () {
                            return angular.copy(self.quote);
                        }
                    }
                }).result.then(function (quote) {
                    angular.extend(self.quote, quote);
                    if (self.quote.vehicleDescription != null || self.quote.vehicleDescription.length != 0) {
                        self.ShellService.needsSave = true;
                        self.displayPartsLaborDialog();
                    }
                });
            }
            else {
                self.displayPartsLaborDialog();
            }
        };
        QuoteController.prototype.displayPartsLaborDialog = function () {
            var self = this;
            self.ShellService.sendAnalyticsEvents('ALLDATA Catalog - Tapped', 'Quote from Mobile', 'ALLDATA Catalog is tapped.');
            self.adDialogService.showDialog({
                url: 'adCatalogControl/partials/adCatalogModal.html',
                controller: 'ADCatalogControlController',
                size: 'lg',
                resolve: {
                    quote: function () {
                        return angular.copy(self.quote);
                    },
                    shop: function () { return self.ShellService.shop; }
                }
            }).result.then(function (quote) {
                self.quote = quote;
                self.ShellService.needsSave = true;
                self.updateTotals();
            })
                .catch(function (error) {
            });
        };
        QuoteController.prototype.changeVehicle = function () {
            var self = this;
            self.adDialogService.showDialog({
                url: 'quotes/partials/selectVehicleModal.html',
                controller: 'SelectVehicleModalController',
                size: 'lg',
                resolve: {
                    quote: function () {
                        return angular.copy(self.quote);
                    }
                }
            }).result.then(function (quote) {
                angular.extend(self.quote, quote);
                self.ShellService.needsSave = true;
            });
        };
        QuoteController.prototype.selectVehicle = function (engine) {
            var self = this;
            self.quote.vehicleDescription = engine.year + " " + engine.make + " " + engine.model + " " + engine.engine;
            self.quote.vehicleALLDATAId = engine.selectionLink;
            self.quote.vehicleVIN = engine.vin;
            self.ShellService.needsSave = true;
        };
        QuoteController.prototype.addNote = function () {
            var self = this;
            self.adDialogService.showDialog({
                url: 'quotes/partials/addNoteModal.html',
                controller: 'AddNoteModalController'
            }).result.then(function (response) {
                var noteQuoteLine = new NgTsSeed.QuoteLineModel();
                noteQuoteLine.lineType = "Note";
                noteQuoteLine.description = response;
                self.quote.datalines.push(noteQuoteLine);
                self.ShellService.needsSave = true;
            });
        };
        QuoteController.prototype.addCustomPart = function () {
            var self = this;
            self.adDialogService.showDialog({
                url: 'quotes/partials/addCustomPartModal.html',
                controller: 'AddCustomPartModalController',
                resolve: {
                    shop: function () { return self.ShellService.shop; }
                }
            }).result.then(function (customPart) {
                customPart.lineType = "CustomPart";
                self.quote.datalines.push(customPart);
                self.updateTotals();
                self.ShellService.needsSave = true;
            });
        };
        QuoteController.prototype.addCustomLabor = function () {
            var self = this;
            self.adDialogService.showDialog({
                url: 'quotes/partials/addCustomLaborModal.html',
                controller: 'AddCustomLaborModalController',
                resolve: {
                    shop: function () { return self.ShellService.shop; }
                }
            }).result.then(function (customLabor) {
                customLabor.lineType = "CustomLabor";
                self.quote.datalines.push(customLabor);
                self.updateTotals();
                self.ShellService.needsSave = true;
            });
        };
        QuoteController.prototype.addFlatFee = function () {
            var self = this;
            self.adDialogService.showDialog({
                url: 'quotes/partials/addFlatFeeModal.html',
                controller: 'AddFlatFeeModalController'
            }).result.then(function (flatFee) {
                flatFee.lineType = "FlatFee";
                self.quote.datalines.push(flatFee);
                self.updateTotals();
                self.ShellService.needsSave = true;
            });
            self.flatFee = [];
            self.flatFee.name = "";
            self.flatFee.description = "";
        };
        QuoteController.prototype.updateTotals = function () {
            var self = this;
            self.quote = self.QuoteService.normalizeQuote(self.quote);
            self.QuoteService.updateTotals(self.quote, self.ShellService.shop);
        };
        QuoteController.prototype.toCurrency = function (num) {
            var self = this, currencyFilter = self.$filter('currency'), currency = currencyFilter(num);
            return currency;
        };
        QuoteController.prototype.canShare = function () {
            var self = this;
            if (self.quote === null) {
                return true;
            }
            return self.ShellService.needsSave;
        };
        QuoteController.prototype.emailQuote = function () {
            var self = this;
            if (self.quote.quoteId === undefined || self.ShellService.needsSave) {
                return;
            }
            self.adDialogService.showDialog({
                url: 'quotes/partials/emailQuoteModal.html',
                controller: 'EmailQuoteModalController',
                resolve: {
                    shop: function () { return self.ShellService.shop; },
                    customerEmail: function () { return (self.quote.customerEmail === null) ? '' : self.quote.customerEmail; }
                }
            }).result.then(function (emailModel) {
                self.QuoteService.emailQuote(self.quote, emailModel);
            });
        };
        QuoteController.prototype.showCustomerInformationDialog = function () {
            var self = this;
            self.adDialogService.showDialog({
                url: 'quotes/partials/customerInfoModal.html',
                controller: 'customerInfoModalController'
            }).result.then(function () {
            });
        };
        QuoteController.$inject = ['$scope', '$rootScope', '$timeout', '$state', '$stateParams', '$http', '$q', '$filter', 'ADQuoteData', 'ngTableParams', 'ShellService', 'AuthorizationService', 'adDialogService', 'adSpinnerService', 'QuoteService', 'adFlyoutDialogService'];
        return QuoteController;
    }());
    NgTsSeed.QuoteController = QuoteController;
})(NgTsSeed || (NgTsSeed = {}));
var NgTsSeed;
(function (NgTsSeed) {
    var Quote;
    (function (Quote) {
        var Directive = (function () {
            function Directive($timeout, $filter, $document, $rootScope, InputService, ShellService, adDialogService) {
                this.$timeout = $timeout;
                this.$filter = $filter;
                this.$document = $document;
                this.$rootScope = $rootScope;
                this.InputService = InputService;
                this.ShellService = ShellService;
                this.adDialogService = adDialogService;
                this.replace = true;
                this.templateUrl = 'quotes/partials/quoteDisplay.html';
                this.scope = {
                    'readOnly': '=',
                    'quote': '='
                };
                this.link = LinkFunction.bind(this);
            }
            Directive.Factory = function ($timeout, $filter, $document, $rootScope, InputService, ShellService, adDialogService) {
                return new Directive($timeout, $filter, $document, $rootScope, InputService, ShellService, adDialogService);
            };
            Directive.$inject = ['$timeout', '$filter', '$document', '$rootScope', 'InputService', 'ShellService', 'adDialogService'];
            return Directive;
        }());
        Quote.Directive = Directive;
        function LinkFunction(scope, element, attr, ctrl, transclude) {
            var self = this, $timeout = this.$timeout;
            scope.selectedRow = {};
            scope.lineTemplate = function (quoteLine) {
                var templateLocationBase = 'quotes/partials/quoteLineTemplates/';
                switch (quoteLine.lineType) {
                    case 'Note':
                        return templateLocationBase + 'note.html';
                        break;
                    case 'ADLabor':
                    case 'CustomLabor':
                        return templateLocationBase + 'customLabor.html';
                        break;
                    case 'ADPart':
                    case 'AZPart':
                    case 'CustomPart':
                        return templateLocationBase + 'customPart.html';
                        break;
                    case 'FlatFee':
                        return templateLocationBase + 'flatFee.html';
                        break;
                    default:
                        return templateLocationBase + 'default.html';
                        break;
                }
            };
            var laborRates = [];
            angular.forEach(self.ShellService.shop.laborRates, function (item) {
                laborRates.push({
                    value: item,
                    label: self.$filter('currency')(item, '$', 2)
                });
            });
            scope.laborRates = laborRates;
            scope.selectRow = function (quoteLine, event) {
                if (event.target.className.indexOf('deleteQuoteLine') > -1 || event.target.className.indexOf('appIcon-close') > -1) {
                    var message = "Are you sure you want to delete this item?";
                    var lineType = quoteLine.lineType.toLowerCase();
                    if (lineType.indexOf('part') !== -1) {
                        message = "Delete Part?";
                    }
                    else if (lineType.indexOf('labor') !== -1) {
                        message = "Delete Labor?";
                    }
                    self.adDialogService.showAlert({
                        message: message,
                        title: "Confirm Delete",
                        buttons: ["Cancel", "OK"]
                    }).result.then(function (response) {
                        if (response.buttonIndex == 1) {
                            var lineIndex = scope.quote.datalines.indexOf(quoteLine);
                            scope.quote.datalines.splice(lineIndex, 1);
                        }
                    });
                }
                else {
                    scope.selectedRow = quoteLine;
                }
                self.ShellService.needsSave = true;
            };
            self.$document.on('click.quoteLine', function (e) {
                if (!e.target)
                    return;
                for (element = e.target; element; element = element.parentNode) {
                    var id = element.id;
                    var classNames = element.className;
                    if (classNames && classNames.indexOf('selected') > -1) {
                        return;
                    }
                }
                scope.selectedRow = {};
                scope.$digest();
            });
        }
    })(Quote = NgTsSeed.Quote || (NgTsSeed.Quote = {}));
})(NgTsSeed || (NgTsSeed = {}));
var NgTsSeed;
(function (NgTsSeed) {
    var QuoteHeader;
    (function (QuoteHeader) {
        var Directive = (function () {
            function Directive() {
                this.restrict = 'E';
                this.replace = true;
                this.templateUrl = 'quotes/partials/quoteHeader.html';
            }
            Directive.Factory = function () {
                return new Directive();
            };
            return Directive;
        }());
        QuoteHeader.Directive = Directive;
    })(QuoteHeader = NgTsSeed.QuoteHeader || (NgTsSeed.QuoteHeader = {}));
})(NgTsSeed || (NgTsSeed = {}));
var NgTsSeed;
(function (NgTsSeed) {
    var QuoteListController = (function () {
        function QuoteListController($scope, $q, $filter, quoteDataSource, spinner, adDialogService, QuoteService, ShellService, orderByFilter) {
            this.$scope = $scope;
            this.$q = $q;
            this.$filter = $filter;
            this.quoteDataSource = quoteDataSource;
            this.spinner = spinner;
            this.adDialogService = adDialogService;
            this.QuoteService = QuoteService;
            this.ShellService = ShellService;
            this.orderByFilter = orderByFilter;
            this.renderOrder = [
                ,
                'quoteNumber',
                'quoteAmount',
                'customerFirstName',
                'customerLastName',
                'customerPhoneNumber',
                'vehicleDescription',
                'creationDate'
            ];
            $scope.vm = this;
            var self = this;
            self.initQuoteActions();
            self.readOnly = true;
            $scope.$watch(function () {
                return ShellService.verified;
            }, function (verified) {
                if (verified === true) {
                    self.getOpenQuotes();
                }
            });
        }
        QuoteListController.prototype.initQuoteActions = function () {
            var self = this;
            self.quoteActions = [
                {
                    title: 'View Quote Details',
                    icon: 'appIcon-edit',
                    onSelect: function () {
                        self.ShellService.goToQuote(self.quote);
                    }
                },
                {
                    title: 'Delete Quote',
                    icon: 'appIcon-delete',
                    onSelect: function () {
                        self.deleteQuote(self.quote);
                    }
                }
            ];
        };
        QuoteListController.prototype.quoteSelected = function (quote) {
            var self = this;
            self.quote = quote;
        };
        QuoteListController.prototype.adListQuoteSelected = function (quote, event, index) {
            this.quoteSelected(quote);
        };
        QuoteListController.prototype.search = function (searchTerm) {
            var self = this;
            if (searchTerm.length == 0) {
                self.filteredQuoteList = self.quotes;
                return;
            }
            self.filteredQuoteList = [];
            self.quotes.forEach(function (quote) {
                var searchArray = ['#' + quote.quoteNumber, quote.customerFirstName, quote.customerLastName, quote.vehicleDescription], searchString = searchArray.join(' ').toLowerCase(), term = searchTerm.toLowerCase();
                if (searchString.indexOf(term) !== -1) {
                    self.filteredQuoteList.push(quote);
                }
            });
        };
        QuoteListController.prototype.getOpenQuotes = function () {
            var self = this;
            self.spinner.start();
            self.QuoteService.getOpenQuotes(self.ShellService.shop).then(function (response) {
                var quotes = [], shop = self.ShellService.shop, index;
                response.data = self.orderByFilter(response.data, 'lastUpdateDate', true);
                for (index = 0; index < response.data.length; index++) {
                    var quote = self.QuoteService.normalizeQuote(response.data[index]);
                    self.QuoteService.updateTotals(quote, shop);
                    quotes.push(quote);
                }
                self.quoteSelected(quotes[0]);
                self.spinner.end();
                self.quotes = quotes;
                self.filteredQuoteList = quotes;
            }, function (response) {
                self.spinner.end();
                self.adDialogService.showAlert({
                    title: 'An error occurred',
                    message: "Retrieving quote list failed due to " + response.data
                });
            });
        };
        QuoteListController.prototype.deleteQuote = function (quote) {
            var self = this;
            self.QuoteService.deleteQuote(quote).then(function (response) {
                self.getOpenQuotes();
            });
        };
        QuoteListController.$inject = ['$scope', '$q', '$filter', 'ADQuoteData', 'adSpinnerService', 'adDialogService', 'QuoteService', 'ShellService', 'orderByFilter'];
        return QuoteListController;
    }());
    NgTsSeed.QuoteListController = QuoteListController;
})(NgTsSeed || (NgTsSeed = {}));
var NgTsSeed;
(function (NgTsSeed) {
    var SelectVehicleModalController = (function () {
        function SelectVehicleModalController($scope, $modalInstance, quote, ShellService, ManageLoginService, adDialogService, $rootScope) {
            this.$scope = $scope;
            this.$modalInstance = $modalInstance;
            this.quote = quote;
            this.ShellService = ShellService;
            this.ManageLoginService = ManageLoginService;
            this.adDialogService = adDialogService;
            this.$rootScope = $rootScope;
            this.states = ["-", "AL", "AK", "AS", "AZ", "AR", "CA", "CO", "CT", "DE", "DC", "FM", "FL", "GA", "GU", "HI", "ID", "IL", "IN", "IA", "KS", "KY", "LA", "ME", "MH", "MD", "MA", "MI", "MN", "MS", "MO", "MT", "NE", "NV", "NH", "NJ", "NM", "NY", "NC", "ND", "MP", "OH", "OK", "OR", "PW", "PA", "PR", "RI", "SC", "SD", "TN", "TX", "UT", "VT", "VI", "VA", "WA", "WV", "WI", "WY"];
            this.isEngineSelected = false;
            var self = this;
            $scope.vm = this;
            self.buttons = self.initButtons();
            self.quote.vehicleLicenseState = self.quote.vehicleLicenseState ? self.quote.vehicleLicenseState : $scope.vm.states[0];
            if (self.quote.vehicleDescription && self.quote.vehicleALLDATAId) {
                self.isEngineSelected = true;
            }
        }
        SelectVehicleModalController.prototype.selectVehicle = function (engine) {
            var self = this;
            self.quote.selectedCar = engine;
            self.quote.vehicleDescription = [engine.year, engine.make, engine.model, engine.engine].join(' ');
            self.quote.vehicleALLDATAId = engine.selectionLink;
            self.quote.vehicleVIN = engine.vin;
            self.isEngineSelected = true;
        };
        SelectVehicleModalController.prototype.clearVehicleSelection = function () {
            var self = this;
            self.isEngineSelected = false;
        };
        SelectVehicleModalController.prototype.initButtons = function () {
            var self = this;
            return [
                {
                    title: 'Cancel',
                    onClick: function () {
                        self.$modalInstance.dismiss();
                    }
                },
                {
                    title: 'Done',
                    onClick: function () {
                        if (self.quote.vehicleVIN && self.ShellService.moEnabled) {
                            self.showCustomerInformationDialog();
                            self.$rootScope.$emit('GET_CUSTOMER_INFO_USING_VIN', { data: self.quote.vehicleVIN });
                        }
                        if (self.quote.vehicleDescription) {
                            self.ShellService.sendAnalyticsEvents('Vehicle Info - Saved', 'Quote from Mobile', 'Vehicle Info saved.');
                        }
                        self.$modalInstance.close(self.quote);
                    }
                }
            ];
        };
        SelectVehicleModalController.prototype.showCustomerInformationDialog = function () {
            var self = this;
            self.adDialogService.showDialog({
                url: 'quotes/partials/customerInfoModal.html',
                controller: 'customerInfoModalController'
            }).result.then(function () {
            });
        };
        SelectVehicleModalController.$inject = ['$scope', '$modalInstance', 'quote', 'ShellService', 'ManageLoginService', 'adDialogService', '$rootScope'];
        return SelectVehicleModalController;
    }());
    NgTsSeed.SelectVehicleModalController = SelectVehicleModalController;
})(NgTsSeed || (NgTsSeed = {}));
var NgTsSeed;
(function (NgTsSeed) {
    var Resources = (function () {
        function Resources() {
        }
        Resources.Routes = {
            main: 'main',
            quote: 'quote',
            newQuote: 'newQuote',
            landing: 'landing',
            quotes: 'quotes',
            setup: 'setup',
            login: 'login',
            shopCreation: 'shopCreation'
        };
        Resources.Views = {
            quote: {
                url: '/quote/:quoteId',
                templateUrl: 'quotes/partials/quoteHome.html',
                controller: 'QuoteController',
                resolve: {
                    shop: ['ShellService', function (ShellService) {
                            return ShellService.initShop();
                        }]
                }
            },
            newQuote: {
                url: '/quote/new?engine',
                templateUrl: 'quotes/partials/quoteHome.html',
                controller: 'QuoteController',
                resolve: {
                    shop: ['ShellService', function (ShellService) {
                            return ShellService.initShop();
                        }]
                }
            },
            quotes: {
                url: '/quotes',
                templateUrl: 'quotes/partials/quoteList.html',
                controller: "QuoteListController",
                resolve: {
                    shop: ['ShellService', function (ShellService) {
                            return ShellService.initShop();
                        }]
                }
            },
            setup: {
                url: '/setup',
                templateUrl: 'setup/partials/setup.html',
                controller: 'SetupController',
                resolve: {
                    shop: ['ShellService', function (ShellService) {
                            return ShellService.initShop();
                        }]
                }
            },
            login: {
                url: '/login',
                templateUrl: 'login/LoginTemplate.html',
                controller: 'LoginController'
            },
            shopCreation: {
                url: '/shop/create',
                templateUrl: 'setup/partials/ShopCreationTemplate.html',
                controller: 'ShopCreationController'
            }
        };
        return Resources;
    }());
    NgTsSeed.Resources = Resources;
})(NgTsSeed || (NgTsSeed = {}));
var Services;
(function (Services) {
    var ADConnectService = (function () {
        function ADConnectService($http, $q, host, path) {
            this.$http = $http;
            this.$q = $q;
            this.host = host;
            this.path = path;
            this.location = {
                checkAuthentication: '/auth-check',
                authenticate: '/login',
                endSession: '/logout',
                shopMapping: '/quote/account?type=OL2'
            };
        }
        ADConnectService.prototype.getBaseUrl = function (url) {
            return this.host + this.path + url;
        };
        ADConnectService.prototype.checkAuthentication = function () {
            var self = this, url = self.getBaseUrl(self.location.checkAuthentication), httpOptions = { withCredentials: true }, event = self.$q.defer();
            self.$http.get(url, httpOptions)
                .then(function (response) {
                event.resolve(true);
            }, function (response) {
                if (response.status === 401) {
                    event.resolve(false);
                }
                else {
                    event.reject();
                }
            });
            return event.promise;
        };
        ADConnectService.prototype.authenticate = function (username, password) {
            var self = this, url = self.getBaseUrl(self.location.authenticate), payload = "login={username}&password={password}", httpOptions = { withCredentials: true };
            payload = payload.replace('{username}', username);
            payload = payload.replace('{password}', password);
            return self.$http.post(url, payload, httpOptions);
        };
        ADConnectService.prototype.endSession = function () {
            var self = this, url = self.getBaseUrl(self.location.endSession), httpOptions = { withCredentials: true };
            return self.$http.get(url, httpOptions);
        };
        ADConnectService.$inject = ['$http', '$q', 'ADCONNECT_HOST', 'ADCONNECT_PATH'];
        return ADConnectService;
    }());
    Services.ADConnectService = ADConnectService;
})(Services || (Services = {}));
var NgTsSeed;
(function (NgTsSeed) {
    var AuthorizationService = (function () {
        function AuthorizationService($q, $state, $location, $http, adQuoteData, shellService, adConnectService, adSpinnerService, adDialogService) {
            this.$q = $q;
            this.$state = $state;
            this.$location = $location;
            this.$http = $http;
            this.adQuoteData = adQuoteData;
            this.shellService = shellService;
            this.adConnectService = adConnectService;
            this.adSpinnerService = adSpinnerService;
            this.adDialogService = adDialogService;
            this.shopRequestLocked = false;
        }
        AuthorizationService.prototype.verifyAuthorization = function () {
            var self = this, auth = self.$q.defer(), event = self.$q.defer();
            if (self.shopRequestLocked === false) {
                self.shopRequestLocked = true;
                self.checkAuthenticationToken()
                    .then(function (response) {
                    if (response === true) {
                    }
                    else {
                        auth.reject(200);
                        return auth.promise;
                    }
                })
                    .then(function (response) {
                    return self.getShop(response.data.shopId)
                        .then(function (response) {
                        self.shellService.setShop(response.data);
                        event.resolve(response);
                    });
                })
                    .catch(function (response) {
                    if (response === 200) {
                        event.resolve();
                        self.shellService.goToLogin();
                    }
                    else if (response.status === 404) {
                        event.reject();
                        self.shellService.goToShopCreation();
                    }
                    else {
                        event.reject();
                        self.shellService.goToLogin();
                        self.adDialogService.showAlert({
                            title: 'An error occurred',
                            message: "Unable to retrieve the shop information."
                        });
                    }
                })
                    .finally(function () {
                    self.shopRequestLocked = false;
                    self.adSpinnerService.end();
                });
            }
            else {
                event.resolve();
            }
            return event.promise;
        };
        AuthorizationService.prototype.authenticate = function (credentials) {
            var self = this, event = self.$q.defer(), auth = self.$q.defer();
            self.adSpinnerService.start();
            self.checkAuthenticationToken()
                .then(function (response) {
                if (response === true) {
                    auth.reject(200);
                    return auth.promise;
                }
                else {
                    return self.getAuthenticationToken(credentials);
                }
            })
                .then(function (response) {
                return self.verifyAuthorization();
            })
                .then(function (response) {
                self.redirectToLandingPage(response.data);
                event.resolve(response);
            })
                .catch(function (response) {
                if (typeof response === 'number' && response === 200) {
                    event.resolve();
                    self.verifyAuthorization();
                }
                else if (typeof response !== 'undefined' && response.status === 401) {
                    event.reject();
                    self.adDialogService.showAlert({
                        title: 'Login failed',
                        message: "The user name or password is incorrect."
                    });
                }
                else {
                    event.reject();
                }
            })
                .finally(function () {
                self.adSpinnerService.end();
            });
            return event.promise;
        };
        AuthorizationService.prototype.checkAuthenticationToken = function () {
            var self = this;
            return self.adConnectService.checkAuthentication();
        };
        AuthorizationService.prototype.getAuthenticationToken = function (credentials) {
            var self = this;
            return self.adConnectService.authenticate(credentials.username, credentials.password);
        };
        AuthorizationService.prototype.getShopMapping = function () {
            var self = this;
        };
        AuthorizationService.prototype.endSession = function () {
            var self = this;
            return self.adConnectService.endSession();
        };
        AuthorizationService.prototype.redirectToLandingPage = function (shop) {
            var self = this;
            if (shop.nextQuoteNumber > 1) {
                self.shellService.goToNewQuote();
            }
            else {
                self.shellService.goToSetup();
            }
        };
        AuthorizationService.prototype.checkShopSetup = function (shop) {
            return shop.nextQuoteNumber > 1;
        };
        AuthorizationService.prototype.getShop = function (shopId) {
            var self = this, event = self.$q.defer();
            self.adQuoteData.shopService.getShop(shopId)
                .then(function (response) {
                event.resolve(response);
            })
                .catch(function (response) {
                if (response.status === 404) {
                    self.shellService.goToShopCreation();
                    event.reject(response);
                }
                else {
                    self.shopRequestLocked = false;
                    self.shellService.goToLogin();
                    event.reject('Unable to retrieve the shop information');
                }
            });
            return event.promise;
        };
        AuthorizationService.$inject = ['$q', '$state', '$location', '$http', 'ADQuoteData', 'ShellService', 'ADConnectService', 'adSpinnerService', 'adDialogService'];
        return AuthorizationService;
    }());
    NgTsSeed.AuthorizationService = AuthorizationService;
})(NgTsSeed || (NgTsSeed = {}));
var NgTsSeed;
(function (NgTsSeed) {
    var CatalogDataContent = (function () {
        function CatalogDataContent() {
        }
        return CatalogDataContent;
    }());
    NgTsSeed.CatalogDataContent = CatalogDataContent;
    var AutoZonePartValue = (function () {
        function AutoZonePartValue() {
        }
        return AutoZonePartValue;
    }());
    NgTsSeed.AutoZonePartValue = AutoZonePartValue;
    var AutoZonePart = (function () {
        function AutoZonePart() {
        }
        return AutoZonePart;
    }());
    NgTsSeed.AutoZonePart = AutoZonePart;
    var ALLDATAPartValue = (function () {
        function ALLDATAPartValue() {
        }
        return ALLDATAPartValue;
    }());
    NgTsSeed.ALLDATAPartValue = ALLDATAPartValue;
    var ALLDATAPart = (function () {
        function ALLDATAPart() {
        }
        return ALLDATAPart;
    }());
    NgTsSeed.ALLDATAPart = ALLDATAPart;
    var ALLDATALaborValue = (function () {
        function ALLDATALaborValue() {
        }
        return ALLDATALaborValue;
    }());
    NgTsSeed.ALLDATALaborValue = ALLDATALaborValue;
    var ALLDATALabor = (function () {
        function ALLDATALabor() {
        }
        return ALLDATALabor;
    }());
    NgTsSeed.ALLDATALabor = ALLDATALabor;
    var CatalogDataService = (function () {
        function CatalogDataService($http, $q, carLookup, adDialogService, $rootScope, SpinnerService) {
            this.$http = $http;
            this.$q = $q;
            this.carLookup = carLookup;
            this.adDialogService = adDialogService;
            this.$rootScope = $rootScope;
            this.SpinnerService = SpinnerService;
            this.AutoZonePIN = 0;
            this.errors = {
                ConnectionError: 'CatalogDataConnectionError'
            };
        }
        CatalogDataService.prototype.requestALLDATAData = function (restURL) {
            var index = restURL.indexOf("/V0");
            if (index > 0) {
                restURL = restURL.substring(index + 3);
            }
            return this.requestData(CatalogDataService.ALLDATAUrl + restURL);
        };
        CatalogDataService.prototype.setAutoZonePIN = function (pin) {
            this.AutoZonePIN = pin;
        };
        CatalogDataService.prototype.requestAutoZoneData = function (restURL) {
            var index = restURL.indexOf("/V0");
            if (index > 0) {
                restURL = restURL.substring(index + 3);
            }
            if (restURL.substr(restURL.length - 1) == '/') {
                restURL = restURL.substr(0, restURL.length - 1);
            }
            return this.requestData(CatalogDataService.AutoZoneURL + restURL + this.pinURL());
        };
        CatalogDataService.prototype.pinURL = function () {
            var self = this;
            if (self.AutoZonePIN == 0) {
                return "";
            }
            else {
                return "/PIN/" + self.AutoZonePIN;
            }
        };
        CatalogDataService.prototype.requestAutoZoneVehicleId = function (alldataVehicleId) {
            var self = this;
            var defer = self.$q.defer();
            var urlSegment = "/carids/" + alldataVehicleId + ".json";
            self.carLookup.carLookupServer.getAjax(self.$http, urlSegment)
                .then(function (data) {
                if (data.data == null || data.data.acesVehicleNames == null || data.data.acesVehicleNames.length == 0) {
                    defer.reject("Unable to map to the AutoZone vehicle.");
                }
                var acesVehicleIds = [];
                var acesVehicleDescriptions = [];
                for (name in data.data.acesVehicleNames) {
                    var elements = data.data.acesVehicleNames[name].split(':');
                    acesVehicleIds.push(elements[0]);
                    acesVehicleDescriptions.push(elements[1]);
                }
                if (acesVehicleIds.length == 1) {
                    self.mapACESVehicleToAutoZone(acesVehicleIds[0], defer);
                }
                if (acesVehicleIds.length > 1) {
                    self.selectACESVehicle(acesVehicleIds, acesVehicleDescriptions, defer);
                }
            })
                .catch(function (error) {
                if (error.data == null && error.status == -1) {
                    self.SpinnerService.end();
                    self.$rootScope.$emit(self.errors.ConnectionError, {});
                }
                else {
                    defer.reject(error);
                }
            });
            return defer.promise;
        };
        CatalogDataService.prototype.mapACESVehicleToAutoZone = function (acesVehicleId, defer) {
            var self = this;
            var azUrl = "/vehicle/aces_vehicle_id/" + acesVehicleId;
            self.requestData(CatalogDataService.AutoZoneURL + azUrl)
                .then(function (data) {
                defer.resolve(data);
            })
                .catch(function (error) {
                defer.reject(error);
            });
        };
        CatalogDataService.prototype.selectACESVehicle = function (acesVehicleIds, acesVehicleDescriptions, defer) {
            var self = this;
            self.adDialogService.showDialog({
                url: 'adCatalogControl/partials/selectMappedVehicleModal.html',
                controller: 'SelectMappedVehicleModalController',
                resolve: {
                    acesVehicleDescriptions: (function () {
                        return acesVehicleDescriptions;
                    }),
                    vehiclePartsDisplayFlag: (function () {
                        return true;
                    }),
                    acesVehiclePartUrls: (function () {
                        return [];
                    })
                }
            }).result.then(function (selectedVehicleIndex) {
                self.mapACESVehicleToAutoZone(acesVehicleIds[selectedVehicleIndex], defer);
            }, function () {
                defer.reject();
            });
        };
        CatalogDataService.getItemId = function (url) {
            if (url[url.length - 1] == '/') {
                url = url.substring(0, url.length - 1);
            }
            var tokens = url.split('/');
            return tokens[tokens.length - 1];
        };
        CatalogDataService.prototype.requestData = function (url) {
            var self = this;
            var deferred = this.$q.defer();
            var httpOptions = {
                method: 'GET',
                url: url,
                headers: {
                    'Accept': 'application/hal+json',
                    'Authorization': ''
                }
            };
            var callAjax = function () {
                self.$http(httpOptions)
                    .success(function (data) {
                    deferred.resolve(data);
                })
                    .error(function (data, status, headers, config) {
                    if (status == -1 && data == null) {
                        self.$rootScope.$emit(self.errors.ConnectionError, { data: data, status: status });
                    }
                    else if (typeof status === 'number' && (status === 404 || status === 400)) {
                        deferred.reject('The part information requested is not available.');
                    }
                    else {
                        deferred.reject("Error retrieving from " + url + ".  Status is " + status);
                    }
                });
            };
            if (self.UUID) {
                httpOptions.headers.Authorization = 'Basic ' + btoa(self.UUID + ":ALLDATA");
                callAjax();
            }
            else {
                Diagnostics.NativeService.getUUID(function (_uuid) {
                    httpOptions.headers.Authorization = 'Basic ' + btoa(_uuid + ":ALLDATA");
                    callAjax();
                });
            }
            return deferred.promise;
        };
        CatalogDataService.$inject = ['$http', '$q', 'ADConnect', 'adDialogService', '$rootScope', 'adSpinnerService'];
        return CatalogDataService;
    }());
    NgTsSeed.CatalogDataService = CatalogDataService;
})(NgTsSeed || (NgTsSeed = {}));
var Services;
(function (Services) {
    var InputService = (function () {
        function InputService() {
        }
        InputService.prototype.parseToNumber = function (field) {
            var self = this, value = self.getParsedValue(field);
            $(field).val(value);
        };
        InputService.prototype.getParsedValue = function (field) {
            var value = $(field).val() || '', pattern = /[^0-9\.]/ig;
            if (value === '') {
                value = $(field).html() || '';
            }
            if (typeof value !== 'number') {
                value = value.toString().replace(pattern, '');
            }
            value = Number(value) || 0;
            if (value === NaN) {
                value = 0;
            }
            return value;
        };
        InputService.prototype.checkNumericInputKey = function (evt) {
            var char = String.fromCharCode(evt.charCode) || '', element = $(evt.target), number = element.val() + char, type = evt.data.type, pattern;
            if (type === 'floating') {
                pattern = /^[0-9]*(\.|(\.[0-9]+)){0,1}$/g;
            }
            else {
                pattern = /^[0-9]+$/g;
            }
            if (pattern.test(number) === false) {
                evt.preventDefault();
            }
        };
        return InputService;
    }());
    Services.InputService = InputService;
})(Services || (Services = {}));
var NgTsSeed;
(function (NgTsSeed) {
    var ManageLoginService = (function () {
        function ManageLoginService($q, manageOnlineService) {
            this.$q = $q;
            this.manageOnlineService = manageOnlineService;
        }
        ManageLoginService.prototype.lookupByName = function (name, smsAuthorization) {
            var self = this, event = self.$q.defer();
            self.manageOnlineService.lookupByName(name, smsAuthorization)
                .then(function (response) {
                event.resolve(response);
            }, function (error) {
                event.reject(error);
            });
            return event.promise;
        };
        ManageLoginService.prototype.lookupByPhone = function (phone, smsAuthorization) {
            var self = this, event = self.$q.defer();
            self.manageOnlineService.lookupByPhone(phone, smsAuthorization)
                .then(function (response) {
                event.resolve(response);
            }, function (error) {
                event.reject(error);
            });
            return event.promise;
        };
        ManageLoginService.prototype.verifyLogin = function (username, password) {
            var self = this, event = self.$q.defer();
            self.manageOnlineService.verifyLogin(username, password)
                .then(function (response) {
                event.resolve(response);
            })
                .catch(function (response) {
                if (response.status === 404) {
                    event.reject(response);
                }
                else {
                    event.reject('Unable to retrieve the user information');
                }
            });
            return event.promise;
        };
        ManageLoginService.prototype.lookupByVIN = function (vin, smsAuthorization) {
            var self = this, event = self.$q.defer();
            self.manageOnlineService.lookupByVIN(vin, smsAuthorization)
                .then(function (response) {
                event.resolve(response);
            });
            return event.promise;
        };
        ManageLoginService.prototype.lookupByCustomerAccNo = function (customerAccountNumber, smsAuthorization) {
            var self = this, event = self.$q.defer();
            self.manageOnlineService.lookupByCustomerAccNo(customerAccountNumber, smsAuthorization)
                .then(function (response) {
                event.resolve(response);
            });
            return event.promise;
        };
        ManageLoginService.prototype.lookupByLicensePlate = function (licensePlateNo) {
            var self = this, event = self.$q.defer();
            self.manageOnlineService.lookupByLicensePlate(licensePlateNo)
                .then(function (response) {
                event.resolve(response);
            });
            return event.promise;
        };
        ManageLoginService.$inject = ['$q', 'manageOnlineService'];
        return ManageLoginService;
    }());
    NgTsSeed.ManageLoginService = ManageLoginService;
})(NgTsSeed || (NgTsSeed = {}));
var Services;
(function (Services) {
    var QuoteService = (function () {
        function QuoteService(ADQuoteData, DialogService, $q, $filter) {
            this.ADQuoteData = ADQuoteData;
            this.DialogService = DialogService;
            this.$q = $q;
            this.$filter = $filter;
        }
        QuoteService.prototype.updateTotals = function (quote, shop) {
            var self = this, currencyFilter = self.$filter('currency');
            if (quote.datalines !== undefined) {
                quote.resetModel();
                quote.datalines.forEach(function (quoteLine) {
                    var name = quoteLine.name || quoteLine.itemName || '', description = quoteLine.description || '';
                    if (name !== '')
                        description = description.replace(name + ' - ', '');
                    if (name.length > 0 && description.length === 0) {
                        quoteLine.displayDescription = name;
                    }
                    else if (name.length > 0 && description.length > 0) {
                        quoteLine.displayDescription = name + ' - ' + description;
                    }
                    else {
                        quoteLine.displayDescription = description;
                    }
                    self.updateLineExtended(quote, quoteLine, shop);
                });
                var totalTax = quote.totalLaborTax + quote.totalPartsTax, totalSub = quote.totalLabor + quote.totalParts + quote.totalFee, total = totalTax + totalSub;
                quote.quoteAmount = total;
                quote.totalTax = totalTax;
                quote.totalSub = totalSub;
                quote.total = total;
            }
        };
        QuoteService.prototype.updateLineExtended = function (quote, quoteLine, shop) {
            var self = this, fee = 0, tax = 0, factor = 1, lineType = quoteLine.lineType.toLocaleLowerCase(), quantity = "", markup = 0, currencyFilter = self.$filter('currency');
            quoteLine.price_Rate = quoteLine.price !== undefined ? quoteLine.price : quoteLine.price_Rate || 0;
            quoteLine.quantity_Hours = quoteLine.quantity !== undefined ? quoteLine.quantity : quoteLine.quantity_Hours || 0;
            quoteLine.displayLineType = quoteLine.lineType || '';
            markup = quoteLine.price_Rate;
            if (lineType.indexOf('note') !== -1 || lineType.indexOf('flatfee') !== -1) {
                delete quoteLine.taxable;
            }
            if (lineType.indexOf('part') !== -1) {
                factor = Number(shop.partsTax);
                quoteLine.displayLineType = 'Part';
            }
            else if (lineType.indexOf('labor') !== -1) {
                factor = Number(shop.laborTax);
                quoteLine.displayLineType = 'Labor';
            }
            else if (lineType.indexOf('note') !== -1) {
                quoteLine.extended = 0;
                quoteLine.displayExtended = "";
                quoteLine.displayPrice = "";
                return quoteLine;
            }
            if (lineType.indexOf('flatfee') !== -1) {
                fee = Number(quoteLine.price_Rate);
                factor = 0;
                quote.totalFee += fee;
            }
            else if (quoteLine.price_Rate > 0 && quoteLine.quantity_Hours > 0) {
                fee = Number(markup * quoteLine.quantity_Hours);
                quantity = quoteLine.quantity_Hours.toString();
            }
            if ('taxable' in quoteLine && quoteLine.taxable && factor > 0) {
                tax = fee * factor;
            }
            if (lineType.indexOf('part') !== -1) {
                quote.totalParts += fee;
                quote.totalPartsTax += tax;
            }
            else if (lineType.indexOf('labor') !== -1) {
                quote.totalLabor += fee;
                quote.totalLaborTax += tax;
            }
            quoteLine.extended = fee;
            quoteLine.displayExtended = currencyFilter(quoteLine.extended);
            quoteLine.displayPrice = currencyFilter(quoteLine.price_Rate);
            quoteLine.displayQuantity = quantity;
            return quoteLine;
        };
        QuoteService.prototype.getOpenQuotes = function (shop) {
            var self = this, event;
            event = self.ADQuoteData.ADQuoteDataQuotes.getQuotes(shop.shopId);
            return event;
        };
        QuoteService.prototype.getQuote = function (shopId, quoteId) {
            var self = this, event = self.$q.defer();
            self.ADQuoteData.ADQuoteDataQuotes.getQuote(shopId, quoteId)
                .then(function (quote) {
                event.resolve(quote);
            })
                .catch(function () {
                event.resolve(null);
            });
            return event.promise;
        };
        QuoteService.prototype.deleteQuote = function (quote) {
            var self = this, dialog = {
                message: "Are you sure you want to delete this quote?",
                title: "Confirm Delete",
                buttons: ["Cancel", "OK"]
            }, event = self.$q.defer();
            self.DialogService.showAlert(dialog).result.then(function (response) {
                if (response.buttonIndex == 1) {
                    self.ADQuoteData.ADQuoteDataQuotes.deleteQuote(quote).then(function () {
                        event.resolve();
                    });
                }
                else {
                    event.reject();
                }
            });
            return event.promise;
        };
        QuoteService.prototype.getPartMarkupPrice = function (price, shop) {
            var self = this, factor = shop.partsMarkup || 0, markupPrice = 0, partPrice = price || 0, currencyFilter = self.$filter('currency');
            markupPrice = Number(partPrice) + (partPrice * factor);
            return markupPrice;
        };
        QuoteService.prototype.emailQuote = function (quote, emailModel) {
            var self = this;
            if (emailModel.message == null) {
                emailModel.message = "";
            }
            self.ADQuoteData.ADQuoteDataQuotes.emailQuote(quote, emailModel.address, emailModel.message).then(function () {
                var dialog = {
                    title: "ALLDATA Quote",
                    message: "Quote has been sent to " + emailModel.address
                };
                self.DialogService.showAlert(dialog);
            }, function (reason) {
                var dialog = {
                    title: "Quote Not Sent",
                    message: "Your message could not be sent at this time. Please try again."
                };
                self.DialogService.showAlert(dialog);
            });
        };
        QuoteService.prototype.normalizeQuote = function (newQuote) {
            var self = this, cleanModel = new NgTsSeed.QuoteModel(newQuote.shopId), datalines, quote;
            if (newQuote && 'resetModel' in newQuote === false) {
                datalines = newQuote.datalines;
                quote = angular.extend(cleanModel, newQuote);
                quote.datalines = datalines;
            }
            else {
                quote = newQuote || cleanModel;
            }
            quote.datalines.forEach(function (line) {
                line.displayPartSkill = line.partNumber;
            });
            return quote;
        };
        QuoteService.prototype.createShop = function (shop) {
            var self = this, event = self.$q.defer();
            self.ADQuoteData.ADQuoteDataShop.createShopMapping()
                .then(function (response) {
                var location = response.headers.location || '', index = location.lastIndexOf('/') + 1, accId = location.substr(index).split('?')[0], event = self.$q.defer();
                if (accId.length === 0) {
                    event.reject(response);
                    return event.promise;
                }
                else {
                    shop.accId = accId;
                    self.ADQuoteData.ADQuoteDataShop.getShopIdUsingAccId(shop.accId)
                        .then(function (response) {
                        shop.shopId = response.data.shopId;
                        return self.ADQuoteData.ADQuoteDataShop.updateShop(shop);
                    });
                }
            })
                .then(function (response) {
                event.resolve();
            })
                .catch(function (response) {
                event.reject(response);
            });
            return event.promise;
        };
        QuoteService.$inject = ['ADQuoteData', 'adDialogService', '$q', '$filter'];
        return QuoteService;
    }());
    Services.QuoteService = QuoteService;
})(Services || (Services = {}));
var NgTsSeed;
(function (NgTsSeed) {
    var Setup;
    (function (Setup) {
        var Controller = (function () {
            function Controller($scope, $rootScope, $state, $q, quoteDataSource, adDialogService, spinner, QuoteService, ShellService, shop, ManageLoginService) {
                this.$scope = $scope;
                this.$rootScope = $rootScope;
                this.$state = $state;
                this.$q = $q;
                this.quoteDataSource = quoteDataSource;
                this.adDialogService = adDialogService;
                this.spinner = spinner;
                this.QuoteService = QuoteService;
                this.ShellService = ShellService;
                this.shop = shop;
                this.ManageLoginService = ManageLoginService;
                this.MessageBoxContentLength = 200;
                this.displayPage = false;
                this.saveShopViaMOL = false;
                $scope.vm = this;
                var self = this;
                if (self.shop.manageOnlinePassword != '')
                    self.shop.manageOnlinePassword = atob(self.shop.manageOnlinePassword);
                self.ShellService.needsSave = false;
                self.firstSave = true;
                if (self.shop.expirationDays === 0) {
                    self.shop.expirationDays = 1;
                }
                if (self.shop.nextQuoteNumber === 0) {
                    self.fixedQuoteNumber = 1;
                    self.shop.nextQuoteNumber = 1;
                }
                else {
                    self.fixedQuoteNumber = self.shop.nextQuoteNumber;
                }
                self.shop.display_partsMarkup = self.shop.partsMarkup * 100;
                self.shop.display_partsTax = self.shop.partsTax * 100;
                self.shop.display_laborTax = self.shop.laborTax * 100;
                $scope.$watch('vm.shop', function (newValue) {
                    if (self.firstSave) {
                        self.formScope.setupForm.$pristine = true;
                    }
                });
                $scope.$watch('setupForm.$pristine', function (value) {
                    if (typeof value == 'undefined')
                        return;
                    self.ShellService.needsSave = !value;
                });
                $scope.$on('$viewContentLoaded', function () {
                    self.spinner.end();
                });
                if (self.shop.manageOnlineUsername && self.shop.manageOnlinePassword) {
                    ManageLoginService.verifyLogin(self.shop.manageOnlineUsername, self.shop.manageOnlinePassword)
                        .then(function (response) {
                        if (response == 'Success') {
                            self.molVerified = true;
                            self.ShellService.molLinked = true;
                        }
                        else {
                            self.molVerified = false;
                            self.ShellService.molLinked = false;
                        }
                    });
                }
            }
            Controller.prototype.verifyManageOnline = function () {
                var self = this;
                self.spinner.start();
                self.ManageLoginService.verifyLogin(self.shop.manageOnlineUsername, self.shop.manageOnlinePassword)
                    .then(function (response) {
                    if (response == 'Success') {
                        self.spinner.end();
                        self.molVerified = true;
                        self.ShellService.molLinked = true;
                        self.saveShopViaMOL = true;
                        self.saveShopInfo();
                    }
                    else {
                        self.spinner.end();
                        self.molVerified = false;
                        self.ShellService.molLinked = false;
                    }
                });
            };
            Controller.prototype.unlinkManageOnline = function () {
                var self = this;
                self.adDialogService.showAlert({
                    message: "Do you want to unlink Manage Online Account?",
                    title: "Manage Online",
                    buttons: ["No", "Yes"]
                }).result.then(function (response) {
                    if (response.buttonIndex === 1) {
                        self.shop.manageOnlineUsername = '';
                        self.shop.manageOnlinePassword = '';
                        self.saveShopViaMOL = true;
                        self.saveShopInfo();
                        self.$state.reload();
                        self.molVerified = false;
                        self.ShellService.molLinked = false;
                    }
                });
            };
            Controller.prototype.checkCurrentQuoteNumber = function (shop) {
                var self = this, quoteEvent;
                if (shop.nextQuoteNumber === 0) {
                    self.fixedQuoteNumber = 1;
                    shop.nextQuoteNumber = 1;
                }
                else {
                    self.fixedQuoteNumber = shop.nextQuoteNumber;
                }
            };
            Controller.prototype.openLearnMoreDialog = function () {
                var self = this;
                self.adDialogService.showAlert({
                    message: "Your manage online credentials will link your ALLDATA Mobile account to Manage Online in a secure manner, allowing data to be exchanged between the two systems on your behalf.",
                    title: "Learn more",
                    buttons: ["OK"]
                });
            };
            Controller.prototype.displayInitialMessage = function () {
                var self = this;
                self.adDialogService.showAlert({
                    message: "Please complete the shop setup to create quotes.",
                    title: "Information",
                    buttons: ["OK"]
                });
            };
            Controller.prototype.setInitialShopModel = function () {
                var self = this, shop = self.shop;
                shop.setInitialSetupState();
            };
            Controller.prototype.laborRateSelected = function (index) {
                var self = this;
                self.shop.defaultLaborRate = index;
                self.selectedLaborRate = "Hourly Rate " + (self.shop.defaultLaborRate + 1);
            };
            Controller.prototype.cancelShopInfo = function () {
                var self = this;
                self.ShellService.goToSavedQuotes();
            };
            Controller.prototype.setFormScope = function (formScope) {
                this.formScope = formScope;
            };
            Controller.prototype.setupFormIsPristine = function () {
                this.ShellService.needsSave = !this.formScope.setupForm.$pristine;
                return this.formScope.setupForm.$pristine;
            };
            Controller.prototype.saveShopInfo = function () {
                var self = this, input = $('input[name=quoteNumber]');
                self.firstSave = false;
                self.formScope.setupForm.$submitted = true;
                if (self.formScope.setupForm.$invalid === true) {
                    self.adDialogService.showAlert({
                        title: 'Shop Information Not Saved',
                        message: "Please make sure the form does not contain any invalid entries and try again.",
                        buttons: ["OK"]
                    }).result.then(function () {
                        var field = "";
                        var errors = self.formScope.setupForm.$error;
                        for (var props in errors) {
                            field = errors[props][0].$name;
                            break;
                        }
                        console.log(field);
                        angular.element('*[name=' + field + ']').focus();
                    });
                    return;
                }
                self.shop.partsMarkup = self.shop.display_partsMarkup / 100.0;
                self.shop.partsTax = self.shop.display_partsTax / 100.0;
                self.shop.laborTax = self.shop.display_laborTax / 100.0;
                self.fixedQuoteNumber = self.shop.nextQuoteNumber;
                if (self.molVerified) {
                    self.shop.manageOnlinePassword = btoa(self.shop.manageOnlinePassword);
                }
                else {
                    self.shop.manageOnlineUsername = '';
                    self.shop.manageOnlinePassword = '';
                }
                var shopModel = self.cleanModel(self.shop);
                self.quoteDataSource.shopService.updateShop(shopModel).then(function () {
                    self.ShellService.sendAnalyticsEvents('Quote Settings - Saved', 'Quote from Mobile', 'Quote setting is saved');
                    input.attr('readOnly', 'true');
                    if (self.currentQuoteWatcher !== undefined) {
                        self.currentQuoteWatcher();
                    }
                    self.ShellService.setShop(self.shop);
                    self.formScope.setupForm.$pristine = true;
                    if (!self.saveShopViaMOL) {
                        self.adDialogService.showAlert({
                            message: "Your shop settings have been saved.",
                            title: "Shop Update",
                            buttons: ["OK"]
                        });
                    }
                }, function (error) {
                });
            };
            Controller.prototype.cleanModel = function (shop) {
                var model = angular.copy(shop);
                delete model._embedded;
                delete model._links;
                delete model.currentQuoteNumber;
                delete model.setShopDisplayInfo;
                delete model.laborRates;
                delete model.display_laborTax;
                delete model.display_partsMarkup;
                delete model.display_partsTax;
                return model;
            };
            Controller.prototype.textareaUpdateCount = function (text, count, index) {
                var self = this;
                var textModified = false;
                text = text || '';
                if (text.length > self.MessageBoxContentLength) {
                    text = text.substring(0, self.MessageBoxContentLength);
                    textModified = true;
                }
                count = self.MessageBoxContentLength - text.length;
                switch (index) {
                    case 0:
                        self.headerCount = count;
                        if (textModified) {
                            self.shop.headerContents = text;
                        }
                        break;
                    case 1:
                        self.footerCount = count;
                        if (textModified) {
                            self.shop.footerContents = text;
                        }
                        break;
                    case 2:
                        self.emailMessageCount = count;
                        if (textModified) {
                            self.shop.emailMessage = text;
                        }
                        break;
                    case 3:
                        self.emailSignatureCount = count;
                        if (textModified) {
                            self.shop.emailSignature = text;
                        }
                        break;
                }
            };
            Controller.$inject = ['$scope', '$rootScope', '$state', '$q', 'ADQuoteData', 'adDialogService', 'adSpinnerService', 'QuoteService', 'ShellService', 'shop', 'ManageLoginService'];
            return Controller;
        }());
        Setup.Controller = Controller;
    })(Setup = NgTsSeed.Setup || (NgTsSeed.Setup = {}));
})(NgTsSeed || (NgTsSeed = {}));
var NgTsSeed;
(function (NgTsSeed) {
    var Setup;
    (function (Setup) {
        var Directive = (function () {
            function Directive() {
                this.restrict = 'E';
                this.templateUrl = 'setup/partials/setupContent.html';
                this.controller = 'SetupController';
                this.replace = true;
                this.scope = true;
                this.link = LinkFunction.bind(this);
            }
            Directive.Factory = function () {
                return new Directive();
            };
            return Directive;
        }());
        Setup.Directive = Directive;
        function LinkFunction($scope, element, attr, ctrl) { }
        ;
    })(Setup = NgTsSeed.Setup || (NgTsSeed.Setup = {}));
})(NgTsSeed || (NgTsSeed = {}));
var NgTsSeed;
(function (NgTsSeed) {
    var ShopCreation;
    (function (ShopCreation) {
        var Controller = (function () {
            function Controller($scope, $window, QuoteService, ShellService, adDialogService, adSpinnerService) {
                this.$scope = $scope;
                this.$window = $window;
                this.QuoteService = QuoteService;
                this.ShellService = ShellService;
                this.adDialogService = adDialogService;
                this.adSpinnerService = adSpinnerService;
                this.shop = {};
                $scope.vm = this;
            }
            Controller.prototype.setFormScope = function (formScope) {
                this.formScope = formScope;
            };
            Controller.prototype.cancelShopInfo = function () {
                var self = this;
                self.ShellService.goToDiagnostics();
            };
            Controller.prototype.submitShop = function () {
                var self = this;
                self.adSpinnerService.start();
                self.formScope.setupForm.$submitted = true;
                if (self.formScope.setupForm.$invalid === true) {
                    self.adDialogService.showAlert({
                        title: 'Shop Information Not Saved',
                        message: "Please make sure the form does not contain any invalid entries and try again.",
                        buttons: ["OK"]
                    }).result.then(function () {
                        self.adSpinnerService.end();
                        var field = "";
                        var errors = self.formScope.setupForm.$error;
                        for (var props in errors) {
                            for (var idx = 0; idx < errors[props].length; idx++) {
                                field = errors[props][idx].$name;
                                if (field) {
                                    break;
                                }
                            }
                            if (field) {
                                break;
                            }
                        }
                        if (field) {
                            console.log(field);
                            angular.element('*[name=' + field + ']').focus();
                        }
                    });
                    return;
                }
                self.QuoteService.createShop(self.shop)
                    .then(function (response) {
                    self.ShellService.goToSetup();
                })
                    .catch(function (response) {
                    self.adSpinnerService.end();
                    self.adDialogService.showAlert({
                        message: "An error occurred while creating the shop, please try again.",
                        title: "Error"
                    });
                });
            };
            Controller.$inject = ['$scope', '$window', 'QuoteService', 'ShellService', 'adDialogService', 'adSpinnerService'];
            return Controller;
        }());
        ShopCreation.Controller = Controller;
    })(ShopCreation = NgTsSeed.ShopCreation || (NgTsSeed.ShopCreation = {}));
})(NgTsSeed || (NgTsSeed = {}));
var context;
var NgTsSeed;
(function (NgTsSeed) {
    var ShellController = (function () {
        function ShellController($scope, $rootScope, $window, $state, $stateParams, $location, ShellService, AuthorizationService, adSpinnerService, adDialogService, ManageLoginService, $timeout) {
            this.$scope = $scope;
            this.$rootScope = $rootScope;
            this.$window = $window;
            this.$state = $state;
            this.$stateParams = $stateParams;
            this.$location = $location;
            this.ShellService = ShellService;
            this.AuthorizationService = AuthorizationService;
            this.adSpinnerService = adSpinnerService;
            this.adDialogService = adDialogService;
            this.ManageLoginService = ManageLoginService;
            this.$timeout = $timeout;
            var self = this;
            $scope.$state = $state;
            $rootScope.customerInfoDialogFlag = true;
            self.initHeader();
            $scope.$watch('vm.ShellService.shop', function (newValue, oldValue) {
                self.initHeader();
            });
            $scope.vm = self;
            $rootScope.$on('GET_CUSTOMER_INFO_USING_VIN', function (event, response) {
                var smsAuthorization = btoa(self.ShellService.shop.manageOnlineUsername + ':::' + atob(self.ShellService.shop.manageOnlinePassword));
                self.ManageLoginService.lookupByVIN(response.data, smsAuthorization)
                    .then(function (response) {
                    self.ManageLoginService.lookupByCustomerAccNo(response.data[0].customerAccountNumber, smsAuthorization)
                        .then(function (response) {
                        if (typeof (response.data) == "object") {
                            self.ShellService.molCustomerData[0] = response.data;
                        }
                        else
                            self.ShellService.molCustomerData = response.data;
                    });
                });
            });
            $scope.$on('$stateChangeStart', function (scope, toState, toParams, fromState, fromParams) {
                $scope.stateTransition = { from: fromState, to: toState };
                self.ShellService.verified = true;
                self.updateViewElements($scope.stateTransition);
            });
        }
        ShellController.prototype.initHeader = function () {
            var self = this;
            var goBackToDiagnostics = {
                title: 'Go to Mobile Diagnostics!',
                icon: 'appIcon-repair',
                onSelect: function () {
                    self.ShellService.goToDiagnostics();
                }
            };
            var settings = {
                title: 'Settings',
                icon: 'appIcon-cog',
                onSelect: function () {
                    if (!self.ShellService.needsSave) {
                        self.adSpinnerService.start();
                    }
                    self.ShellService.sendAnalyticsEvents('Quote Settings', 'Quote from Mobile', 'Quote settings page is opened.');
                    self.ShellService.goToSetup();
                }
            };
            if (self.ShellService.shop) {
                self.appData = {
                    appActions: [
                        goBackToDiagnostics,
                        settings
                    ]
                };
            }
            else {
                self.appData = {
                    appActions: [
                        goBackToDiagnostics
                    ]
                };
            }
        };
        ShellController.prototype.updateViewElements = function (stateTransition) {
            var self = this, collapsedState = stateTransition.to.name === 'login' || stateTransition.to.name === 'shopCreation';
            if (collapsedState === true) {
                $('.shell-content').addClass('login').removeClass('animation');
                $('.shell-navigation').hide();
            }
            else {
                $('.shell-content, .shell-navigation').addClass('animation').removeClass('login');
                $('.shell-navigation').show();
            }
        };
        ShellController.$inject = ['$scope', '$rootScope', '$window', '$state', '$stateParams', '$location', 'ShellService', 'AuthorizationService', 'adSpinnerService', 'adDialogService', 'ManageLoginService', '$timeout'];
        return ShellController;
    }());
    NgTsSeed.ShellController = ShellController;
})(NgTsSeed || (NgTsSeed = {}));
var NgTsSeed;
(function (NgTsSeed) {
    var ShellDirectives = (function () {
        function ShellDirectives() {
        }
        ShellDirectives.Init = function () {
            return {
                restrict: 'E',
                templateUrl: 'shell/partials/shell.html',
                replace: true,
                controller: 'ShellController'
            };
        };
        return ShellDirectives;
    }());
    NgTsSeed.ShellDirectives = ShellDirectives;
})(NgTsSeed || (NgTsSeed = {}));
var NgTsSeed;
(function (NgTsSeed) {
    var ShellService = (function () {
        function ShellService($q, $window, $state, $location, $timeout, $rootScope, $http, adQuoteData, ADConnectService, adDialogService, SpinnerService, CatalogDataService, nativeService, $injector) {
            this.$q = $q;
            this.$window = $window;
            this.$state = $state;
            this.$location = $location;
            this.$timeout = $timeout;
            this.$rootScope = $rootScope;
            this.$http = $http;
            this.adQuoteData = adQuoteData;
            this.ADConnectService = ADConnectService;
            this.adDialogService = adDialogService;
            this.SpinnerService = SpinnerService;
            this.CatalogDataService = CatalogDataService;
            this.nativeService = nativeService;
            this.$injector = $injector;
            this.verified = false;
            this.moEnabled = MO_Enabled_Flag;
            this.manageOnlineUser = '';
            this.molCustomerData = [];
            var self = this;
            self.needsSave = false;
            self.alertIsShowing = false;
            if (self.adQuoteData.baseHost.indexOf('layer7') == -1) {
                self.insecureData();
            }
            this.appWidth = 1360;
            if (nativeService.iOS) {
                $('head').append('<link href="css/styles-ios.css" rel="stylesheet" type="text/css"/>');
                this.appWidth = 1200;
            }
            $rootScope.$on('$stateChangeStart', function (event, toState, toParams, fromState, fromParams) {
                if (self.needsSave) {
                    event.preventDefault();
                    self.adDialogService.showAlert({
                        title: 'Unsaved changes',
                        message: "You haven't saved some changes. Do you want to leave without saving?",
                        buttons: ["Yes", "No"]
                    }).result.then(function (response) {
                        if (response.buttonIndex === 0) {
                            self.needsSave = false;
                            self.$state.go(toState, toParams);
                        }
                    });
                    ;
                }
            });
            $rootScope.$on(adQuoteData.errors.ConnectionError, function () {
                self.showConnectionFailedAlertIfNeeded();
            });
            $rootScope.$on(CatalogDataService.errors.ConnectionError, function () {
                self.showConnectionFailedAlertIfNeeded();
            });
            try {
                this.$analytics = $injector.get('$analytics');
            }
            catch (e) {
                console.log("Google analytics is not active within ymme module.");
            }
        }
        ShellService.prototype.initShop = function () {
            var _this = this;
            var self = this;
            var deferred = this.$q.defer();
            this.adQuoteData.ADQuoteDataShop.getShopId().then(function (shopId) {
                _this.adQuoteData.ADQuoteDataShop.getShop(shopId).then(function (response) {
                    var shop = response.data;
                    _this.setShop(shop);
                    deferred.resolve(shop);
                });
            }, function (error) {
                console.log(error);
                if (error.status === 404) {
                    self.goToShopCreation();
                }
            });
            return deferred.promise;
        };
        ShellService.prototype.setShop = function (shop) {
            this.shop = shop;
            this.shop.laborRates = [];
            if (this.shop.laborRate1 !== undefined)
                this.shop.laborRates.push(this.shop.laborRate1);
            if (this.shop.laborRate2 !== undefined)
                this.shop.laborRates.push(this.shop.laborRate2);
            if (this.shop.laborRate3 !== undefined)
                this.shop.laborRates.push(this.shop.laborRate3);
        };
        ShellService.prototype.insecureApplication = function () {
            var brand = $('.toolbar-placeholder');
            brand.prepend("<div style='float: right;color:#ffffff; padding-right:15px'>Insecure Application</div>");
        };
        ShellService.prototype.insecureData = function () {
            var brand = $('.toolbar-placeholder');
            brand.prepend("<div style='float: right;color:#ffffff; padding-right:15px'>Insecure Data</div>");
        };
        ShellService.prototype.checkDateExpiration = function (lastupdatedDate, quote) {
            var timeDifferenceInDays = Math.floor((Date.now() - lastupdatedDate) / (1000 * 60 * 60 * 24));
            if (this.shop.displayAfterExpirationDate && (timeDifferenceInDays > this.shop.expirationDays))
                quote.quoteExpired = true;
            return quote && quote.quoteExpired;
        };
        ShellService.prototype.goToQuote = function (quote) {
            this.$state.go(NgTsSeed.Resources.Routes.quote, { 'quoteId': quote.quoteId });
        };
        ShellService.prototype.goToNewQuote = function () {
            this.$state.go(NgTsSeed.Resources.Routes.newQuote);
        };
        ShellService.prototype.goToSavedQuotes = function () {
            this.$state.go(NgTsSeed.Resources.Routes.quotes);
        };
        ShellService.prototype.goToSetup = function () {
            if (!this.needsSave) {
                this.SpinnerService.start();
            }
            this.sendAnalyticsEvents('Quote Settings', 'Quote from Mobile', 'Quote settings page is opened.');
            this.$state.go(NgTsSeed.Resources.Routes.setup, {}, { reload: true });
        };
        ShellService.prototype.goToLogin = function () {
            this.$state.go(NgTsSeed.Resources.Routes.login);
        };
        ShellService.prototype.goToShopCreation = function () {
            this.$state.go(NgTsSeed.Resources.Routes.shopCreation);
        };
        ShellService.prototype.goToDiagnostics = function () {
            var self = this;
            var redirectPage = "";
            if (self.nativeService.win8) {
                redirectPage = "exitQuote";
            }
            else {
                redirectPage = "addiag://";
            }
            if (!self.needsSave) {
                self.$window.location = redirectPage;
            }
            else {
                self.adDialogService.showAlert({
                    title: 'Unsaved changes',
                    message: "You haven't saved some changes. Do you want to leave without saving?",
                    buttons: ["Yes", "No"]
                }).result.then(function (response) {
                    if (response.buttonIndex === 0) {
                        self.needsSave = false;
                        self.$window.location = redirectPage;
                    }
                });
            }
        };
        ShellService.prototype.showConnectionFailedAlertIfNeeded = function () {
            this.SpinnerService.end();
            this.adDialogService.showAlert({
                message: "A request could not be completed. Please verify that you are connected to the Internet and try again. If the problem persists, please contact ALLDATA for assistance.",
                title: "Error"
            });
        };
        ShellService.prototype.sendAnalyticsEvents = function (eventAction, eventCategory, eventLabel) {
            if (this.$analytics) {
                this.$analytics.eventTrack(eventAction, { category: eventCategory, label: eventLabel });
                if (this.nativeService.iOS) {
                    this.nativeEventNotification({ event: eventLabel, data: eventAction });
                }
            }
        };
        ShellService.prototype.nativeEventNotification = function (eventNotification) {
            this.nativeService.call(Diagnostics.NativeService.Tasks.nativeEventNotification, eventNotification);
        };
        ShellService.$inject = ['$q', '$window', '$state', '$location', '$timeout', '$rootScope', '$http', 'ADQuoteData', 'ADConnectService', 'adDialogService', 'adSpinnerService', 'CatalogDataService', 'NativeService', '$injector'];
        return ShellService;
    }());
    NgTsSeed.ShellService = ShellService;
})(NgTsSeed || (NgTsSeed = {}));
var NgTsSeed;
(function (NgTsSeed) {
    var Form;
    (function (Form) {
        var Directive = (function () {
            function Directive() {
                this.restrict = 'E';
                this.scope = true;
                this.link = LinkFunction.bind(this);
            }
            Directive.Factory = function () {
                return new Directive();
            };
            return Directive;
        }());
        Form.Directive = Directive;
        function LinkFunction(scope, element, attr) {
            var formName = attr.name || '', formScope = scope[formName] || {};
            scope.formScope = formScope;
        }
    })(Form = NgTsSeed.Form || (NgTsSeed.Form = {}));
})(NgTsSeed || (NgTsSeed = {}));
var NgTsSeed;
(function (NgTsSeed) {
    var NumField;
    (function (NumField) {
        var Directive = (function () {
            function Directive(InputService) {
                this.InputService = InputService;
                this.restrict = 'A';
                this.link = LinkFunction.bind(this);
            }
            Directive.Factory = function (InputService) {
                return new Directive(InputService);
            };
            Directive.$inject = ['InputService'];
            return Directive;
        }());
        NumField.Directive = Directive;
        function LinkFunction($scope, element, attrs) {
            var self = this, field = {
                type: attrs.hasOwnProperty('floating') ? 'floating' : 'numeric' || 'numeric'
            };
            element.on('paste', function (evt) {
                evt.preventDefault();
            });
            element.on('change, blur', function (evt) {
                self.InputService.parseToNumber(evt.target);
                $(evt.target).trigger('change');
            });
            element.on('keypress', field, function (evt) {
                self.InputService.checkNumericInputKey(evt);
            });
        }
    })(NumField = NgTsSeed.NumField || (NgTsSeed.NumField = {}));
})(NgTsSeed || (NgTsSeed = {}));
var NgTsSeed;
(function (NgTsSeed) {
    var ngApp = angular.module('app', ['config', 'ad.ui.bootstrap', 'ui.router', 'ad-ui-coyote-mobile-templates', 'ad.ws.quoteData', 'ad.ui.dialogService', 'ad.ui.ymme', 'ad.ws.repair', 'angulartics', 'angulartics.google.analytics', 'ad.ws.manageOnline']);
    ngApp.service('ShellService', NgTsSeed.ShellService);
    ngApp.service('CatalogDataService', NgTsSeed.CatalogDataService);
    ngApp.service('AuthorizationService', NgTsSeed.AuthorizationService);
    ngApp.service('ADConnectService', Services.ADConnectService);
    ngApp.service('QuoteService', Services.QuoteService);
    ngApp.service('InputService', Services.InputService);
    ngApp.service('ManageLoginService', NgTsSeed.ManageLoginService);
    ngApp.service('NativeService', Diagnostics.NativeService);
    ngApp.controller('ShellController', NgTsSeed.ShellController);
    ngApp.controller('LoginController', Components.Login.Controller);
    ngApp.controller('ADCatalogControlController', NgTsSeed.Catalog.Controller);
    ngApp.controller('QuoteListController', NgTsSeed.QuoteListController);
    ngApp.controller('QuoteController', NgTsSeed.QuoteController);
    ngApp.controller('SetupController', NgTsSeed.Setup.Controller);
    ngApp.controller('PrintPreviewController', NgTsSeed.Preview.Controller);
    ngApp.controller('FormErrorController', NgTsSeed.Quote.FormErrorController);
    ngApp.controller('SelectVehicleModalController', NgTsSeed.SelectVehicleModalController);
    ngApp.controller('AdditionalInfoModalController', NgTsSeed.AdditionalInfoModalController);
    ngApp.controller('AddNoteModalController', NgTsSeed.AddNoteModalController);
    ngApp.controller('AddCustomPartModalController', NgTsSeed.AddCustomPartModalController);
    ngApp.controller('AddCustomLaborModalController', NgTsSeed.AddCustomLaborModalController);
    ngApp.controller('AddFlatFeeModalController', NgTsSeed.AddFlatFeeModalController);
    ngApp.controller('EmailQuoteModalController', NgTsSeed.EmailQuoteModalController);
    ngApp.controller('SelectMappedVehicleModalController', NgTsSeed.SelectMappedVehicleModalController);
    ngApp.controller('ShopCreationController', NgTsSeed.ShopCreation.Controller);
    ngApp.controller('CatalogItemSelectedController', NgTsSeed.CatalogItemSelected.CatalogItemSelectedController);
    ngApp.controller('customerInfoModalController', NgTsSeed.customerInfoModalController);
    ngApp.directive('shell', NgTsSeed.ShellDirectives.Init);
    ngApp.directive('adQuote', NgTsSeed.Quote.Directive.Factory);
    ngApp.directive('adQuoteHeader', NgTsSeed.QuoteHeader.Directive.Factory);
    ngApp.directive('catalogItemSelected', NgTsSeed.CatalogItemSelected.Directive.Factory);
    NgTsSeed.CommonDirectives.Init(ngApp, Diagnostics.NativeService.UseScootch);
    NgTsSeed.CommonFilters.Init(ngApp);
    ngApp.config(['$stateProvider', '$urlRouterProvider', '$injector',
        function ($stateProvider, $urlRouterProvider, $injector) {
            $stateProvider
                .state(NgTsSeed.Resources.Routes.newQuote, NgTsSeed.Resources.Views.newQuote)
                .state(NgTsSeed.Resources.Routes.quote, NgTsSeed.Resources.Views.quote)
                .state(NgTsSeed.Resources.Routes.quotes, NgTsSeed.Resources.Views.quotes)
                .state(NgTsSeed.Resources.Routes.setup, NgTsSeed.Resources.Views.setup)
                .state(NgTsSeed.Resources.Routes.login, NgTsSeed.Resources.Views.login)
                .state(NgTsSeed.Resources.Routes.shopCreation, NgTsSeed.Resources.Views.shopCreation);
            $urlRouterProvider.otherwise(NgTsSeed.Resources.Views.quotes.url);
            if (GA_Enabled_Flag === true) {
                var analyticsService = $injector.get('$analyticsProvider');
                analyticsService.firstPageview(true);
                analyticsService.withAutoBase(true);
                analyticsService.virtualPageviews(false);
            }
        }]);
    ngApp.config(['ADConnectProvider', 'ADCONNECT_HOST', 'ADCONNECT_PATH', 'CarLookup_HOST', 'CarLookup_PATH',
        function (ADConnectProvider, ADCONNECT_HOST, ADCONNECT_PATH, CarLookup_HOST, CarLookup_PATH) {
            Diagnostics.NativeService.getUUID(function (_uuid) {
                ADConnectProvider.registerServers({
                    adConnect: {
                        host: ADCONNECT_HOST,
                        path: ADCONNECT_PATH,
                        uuid: _uuid
                    },
                    carLookup: {
                        host: CarLookup_HOST,
                        path: CarLookup_PATH
                    }
                });
            });
        }]);
    ngApp.config(['ADQuoteDataProvider', 'ADCONNECT_HOST', 'QUOTE_PATH', function (ADQuoteDataProvider, Host, Path) {
            Diagnostics.NativeService.getUUID(function (_uuid) {
                var authHeader = 'Basic ' + btoa(_uuid + ":ALLDATA");
                ADQuoteDataProvider.registerServer({
                    quoteServerLocation: {
                        host: Host,
                        path: Path
                    }
                });
                console.log(authHeader);
                ADQuoteDataProvider.setAuthorizationHeader(authHeader);
            });
        }]);
    ngApp.config(['manageOnlineServiceProvider', 'MANAGE_ONLINE_HOST', 'MANAGE_ONLINE_PATH', function (manageOnlineServiceProvider, Host, Path) {
            Diagnostics.NativeService.getUUID(function (_uuid) {
                var authHeader = 'Basic ' + btoa(_uuid + ":ALLDATA");
                manageOnlineServiceProvider.registerServers({
                    manageOnlineServer: {
                        host: Host,
                        path: Path
                    }
                });
                manageOnlineServiceProvider.setAuthorizationHeader(authHeader);
            });
        }]);
    ngApp.config(['$provide', '$httpProvider', function ($provide, $httpProvider) {
            $provide.factory('HttpInterceptor', function ($q, NativeService) {
                return {
                    responseError: function (rejection) {
                        if (NativeService.win8 && rejection.status === 401) {
                            var deferred = $q.defer();
                            NativeService.on("fail_" + rejection.config.url, function (result) {
                                deferred.reject(rejection);
                            });
                            NativeService.on(rejection.config.url, function (result) {
                                result = NativeService.cleanResult(result);
                                var response = {
                                    config: rejection.config,
                                    data: result,
                                    status: 200,
                                    statusText: "OK",
                                    headers: function (name) {
                                        if (name) {
                                            var value = rejection.config.headers[name];
                                            if (value === void 0) {
                                                value = null;
                                            }
                                            return value;
                                        }
                                        return rejection.config.headers;
                                    }
                                };
                                deferred.resolve(response);
                            });
                            NativeService.on("img_" + rejection.config.url, function (result) {
                                var binString = window.atob(result);
                                var bytes = new Uint8Array(binString.length);
                                for (var i = 0; i < binString.length; i++) {
                                    bytes[i] = binString.charCodeAt(i);
                                }
                                var response = {
                                    config: rejection.config,
                                    data: bytes.buffer,
                                    status: 200,
                                    statusText: "OK",
                                    headers: function (name) {
                                        if (name) {
                                            var value = rejection.config.headers[name];
                                            if (value === void 0) {
                                                value = null;
                                            }
                                            return value;
                                        }
                                        return rejection.config.headers;
                                    }
                                };
                                deferred.resolve(response);
                            });
                            NativeService.call('ajax', rejection.config);
                            return deferred.promise;
                        }
                        return $q.reject(rejection);
                    }
                };
            });
            $httpProvider.interceptors.push('HttpInterceptor');
        }]);
    ngApp.run(['ADCONNECT_HOST', 'AD_CATALOG_PATH', 'AZ_CATALOG_PATH', function (Host, AdCatalog, AzCatalog) {
            NgTsSeed.CatalogDataService.ALLDATAUrl = Host + AdCatalog;
            NgTsSeed.CatalogDataService.AutoZoneURL = Host + AzCatalog;
        }]);
    angular.bootstrap(document, ['app']);
})(NgTsSeed || (NgTsSeed = {}));
//# sourceMappingURL=ad-ui-coyote-mobile.js.map