/*
 * angular-ui-bootstrap
 * http://angular-ui.github.io/bootstrap/

 * Version: 0.13.4 - 2015-09-03
 * License: MIT
 */
angular.module("ui.bootstrap", ["ui.bootstrap.tpls", "ui.bootstrap.collapse","ui.bootstrap.accordion","ui.bootstrap.alert","ui.bootstrap.bindHtml","ui.bootstrap.buttons","ui.bootstrap.carousel","ui.bootstrap.dateparser","ui.bootstrap.position","ui.bootstrap.datepicker","ui.bootstrap.dropdown","ui.bootstrap.modal","ui.bootstrap.pagination","ui.bootstrap.tooltip","ui.bootstrap.popover","ui.bootstrap.progressbar","ui.bootstrap.rating","ui.bootstrap.tabs","ui.bootstrap.timepicker","ui.bootstrap.transition","ui.bootstrap.typeahead"]);
angular.module("ui.bootstrap.tpls", ["template/accordion/accordion-group.html","template/accordion/accordion.html","template/alert/alert.html","template/carousel/carousel.html","template/carousel/slide.html","template/datepicker/datepicker.html","template/datepicker/day.html","template/datepicker/month.html","template/datepicker/popup.html","template/datepicker/year.html","template/modal/backdrop.html","template/modal/window.html","template/pagination/pager.html","template/pagination/pagination.html","template/tooltip/tooltip-html-popup.html","template/tooltip/tooltip-html-unsafe-popup.html","template/tooltip/tooltip-popup.html","template/tooltip/tooltip-template-popup.html","template/popover/popover-html.html","template/popover/popover-template.html","template/popover/popover.html","template/progressbar/bar.html","template/progressbar/progress.html","template/progressbar/progressbar.html","template/rating/rating.html","template/tabs/tab.html","template/tabs/tabset.html","template/timepicker/timepicker.html","template/typeahead/typeahead-match.html","template/typeahead/typeahead-popup.html"]);
angular.module('ui.bootstrap.collapse', [])

  .directive('collapse', ['$animate', function($animate) {
    return {
      link: function(scope, element, attrs) {
        function expand() {
          element.removeClass('collapse')
            .addClass('collapsing')
            .attr('aria-expanded', true)
            .attr('aria-hidden', false);

          $animate.addClass(element, 'in', {
            to: { height: element[0].scrollHeight + 'px' }
          }).then(expandDone);
        }

        function expandDone() {
          element.removeClass('collapsing');
          element.css({height: 'auto'});
        }

        function collapse() {
          if (!element.hasClass('collapse') && !element.hasClass('in')) {
            return collapseDone();
          }

          element
            // IMPORTANT: The height must be set before adding "collapsing" class.
            // Otherwise, the browser attempts to animate from height 0 (in
            // collapsing class) to the given height here.
            .css({height: element[0].scrollHeight + 'px'})
            // initially all panel collapse have the collapse class, this removal
            // prevents the animation from jumping to collapsed state
            .removeClass('collapse')
            .addClass('collapsing')
            .attr('aria-expanded', false)
            .attr('aria-hidden', true);

          $animate.removeClass(element, 'in', {
            to: {height: '0'}
          }).then(collapseDone);
        }

        function collapseDone() {
          element.css({height: '0'}); // Required so that collapse works when animation is disabled
          element.removeClass('collapsing');
          element.addClass('collapse');
        }

        scope.$watch(attrs.collapse, function(shouldCollapse) {
          if (shouldCollapse) {
            collapse();
          } else {
            expand();
          }
        });
      }
    };
  }]);

angular.module('ui.bootstrap.accordion', ['ui.bootstrap.collapse'])

.constant('accordionConfig', {
  closeOthers: true
})

.controller('AccordionController', ['$scope', '$attrs', 'accordionConfig', function($scope, $attrs, accordionConfig) {
  // This array keeps track of the accordion groups
  this.groups = [];

  // Ensure that all the groups in this accordion are closed, unless close-others explicitly says not to
  this.closeOthers = function(openGroup) {
    var closeOthers = angular.isDefined($attrs.closeOthers) ?
      $scope.$eval($attrs.closeOthers) : accordionConfig.closeOthers;
    if (closeOthers) {
      angular.forEach(this.groups, function(group) {
        if (group !== openGroup) {
          group.isOpen = false;
        }
      });
    }
  };

  // This is called from the accordion-group directive to add itself to the accordion
  this.addGroup = function(groupScope) {
    var that = this;
    this.groups.push(groupScope);

    groupScope.$on('$destroy', function(event) {
      that.removeGroup(groupScope);
    });
  };

  // This is called from the accordion-group directive when to remove itself
  this.removeGroup = function(group) {
    var index = this.groups.indexOf(group);
    if (index !== -1) {
      this.groups.splice(index, 1);
    }
  };

}])

// The accordion directive simply sets up the directive controller
// and adds an accordion CSS class to itself element.
.directive('accordion', function() {
  return {
    restrict: 'EA',
    controller: 'AccordionController',
    controllerAs: 'accordion',
    transclude: true,
    replace: false,
    templateUrl: function(element, attrs) {
      return attrs.templateUrl || 'template/accordion/accordion.html';
    }
  };
})

// The accordion-group directive indicates a block of html that will expand and collapse in an accordion
.directive('accordionGroup', function() {
  return {
    require: '^accordion',         // We need this directive to be inside an accordion
    restrict: 'EA',
    transclude: true,              // It transcludes the contents of the directive into the template
    replace: true,                // The element containing the directive will be replaced with the template
    templateUrl: function(element, attrs) {
      return attrs.templateUrl || 'template/accordion/accordion-group.html';
    },
    scope: {
      heading: '@',               // Interpolate the heading attribute onto this scope
      isOpen: '=?',
      isDisabled: '=?'
    },
    controller: function() {
      this.setHeading = function(element) {
        this.heading = element;
      };
    },
    link: function(scope, element, attrs, accordionCtrl) {
      accordionCtrl.addGroup(scope);

      scope.openClass = attrs.openClass || 'panel-open';
      scope.panelClass = attrs.panelClass;
      scope.$watch('isOpen', function(value) {
        element.toggleClass(scope.openClass, value);
        if (value) {
          accordionCtrl.closeOthers(scope);
        }
      });

      scope.toggleOpen = function($event) {
        if (!scope.isDisabled) {
          if (!$event || $event.which === 32) {
            scope.isOpen = !scope.isOpen;
          }
        }
      };
    }
  };
})

// Use accordion-heading below an accordion-group to provide a heading containing HTML
// <accordion-group>
//   <accordion-heading>Heading containing HTML - <img src="..."></accordion-heading>
// </accordion-group>
.directive('accordionHeading', function() {
  return {
    restrict: 'EA',
    transclude: true,   // Grab the contents to be used as the heading
    template: '',       // In effect remove this element!
    replace: true,
    require: '^accordionGroup',
    link: function(scope, element, attr, accordionGroupCtrl, transclude) {
      // Pass the heading to the accordion-group controller
      // so that it can be transcluded into the right place in the template
      // [The second parameter to transclude causes the elements to be cloned so that they work in ng-repeat]
      accordionGroupCtrl.setHeading(transclude(scope, angular.noop));
    }
  };
})

// Use in the accordion-group template to indicate where you want the heading to be transcluded
// You must provide the property on the accordion-group controller that will hold the transcluded element
// <div class="accordion-group">
//   <div class="accordion-heading" ><a ... accordion-transclude="heading">...</a></div>
//   ...
// </div>
.directive('accordionTransclude', function() {
  return {
    require: '^accordionGroup',
    link: function(scope, element, attr, controller) {
      scope.$watch(function() { return controller[attr.accordionTransclude]; }, function(heading) {
        if (heading) {
          element.find('span').html('');
          element.find('span').append(heading);
        }
      });
    }
  };
})

;

angular.module('ui.bootstrap.alert', [])

.controller('AlertController', ['$scope', '$attrs', function($scope, $attrs) {
  $scope.closeable = !!$attrs.close;
  this.close = $scope.close;
}])

.directive('alert', function() {
  return {
    controller: 'AlertController',
    controllerAs: 'alert',
    templateUrl: function(element, attrs) {
      return attrs.templateUrl || 'template/alert/alert.html';
    },
    transclude: true,
    replace: true,
    scope: {
      type: '@',
      close: '&'
    }
  };
})

.directive('dismissOnTimeout', ['$timeout', function($timeout) {
  return {
    require: 'alert',
    link: function(scope, element, attrs, alertCtrl) {
      $timeout(function() {
        alertCtrl.close();
      }, parseInt(attrs.dismissOnTimeout, 10));
    }
  };
}]);

angular.module('ui.bootstrap.bindHtml', [])

  .value('$bindHtmlUnsafeSuppressDeprecated', false)

  .directive('bindHtmlUnsafe', ['$log', '$bindHtmlUnsafeSuppressDeprecated', function ($log, $bindHtmlUnsafeSuppressDeprecated) {
    return function (scope, element, attr) {
      if (!$bindHtmlUnsafeSuppressDeprecated) {
        $log.warn('bindHtmlUnsafe is now deprecated. Use ngBindHtml instead');
      }
      element.addClass('ng-binding').data('$binding', attr.bindHtmlUnsafe);
      scope.$watch(attr.bindHtmlUnsafe, function bindHtmlUnsafeWatchAction(value) {
        element.html(value || '');
      });
    };
  }]);
angular.module('ui.bootstrap.buttons', [])

.constant('buttonConfig', {
  activeClass: 'active',
  toggleEvent: 'click'
})

.controller('ButtonsController', ['buttonConfig', function(buttonConfig) {
  this.activeClass = buttonConfig.activeClass || 'active';
  this.toggleEvent = buttonConfig.toggleEvent || 'click';
}])

.directive('btnRadio', function() {
  return {
    require: ['btnRadio', 'ngModel'],
    controller: 'ButtonsController',
    controllerAs: 'buttons',
    link: function(scope, element, attrs, ctrls) {
      var buttonsCtrl = ctrls[0], ngModelCtrl = ctrls[1];

      element.find('input').css({display: 'none'});

      //model -> UI
      ngModelCtrl.$render = function() {
        element.toggleClass(buttonsCtrl.activeClass, angular.equals(ngModelCtrl.$modelValue, scope.$eval(attrs.btnRadio)));
      };

      //ui->model
      element.bind(buttonsCtrl.toggleEvent, function() {
        if (attrs.disabled) {
          return;
        }

        var isActive = element.hasClass(buttonsCtrl.activeClass);

        if (!isActive || angular.isDefined(attrs.uncheckable)) {
          scope.$apply(function() {
            ngModelCtrl.$setViewValue(isActive ? null : scope.$eval(attrs.btnRadio));
            ngModelCtrl.$render();
          });
        }
      });
    }
  };
})

.directive('btnCheckbox', ['$document', function($document) {
  return {
    require: ['btnCheckbox', 'ngModel'],
    controller: 'ButtonsController',
    controllerAs: 'button',
    link: function(scope, element, attrs, ctrls) {
      var buttonsCtrl = ctrls[0], ngModelCtrl = ctrls[1];

      element.find('input').css({display: 'none'});

      function getTrueValue() {
        return getCheckboxValue(attrs.btnCheckboxTrue, true);
      }

      function getFalseValue() {
        return getCheckboxValue(attrs.btnCheckboxFalse, false);
      }

      function getCheckboxValue(attributeValue, defaultValue) {
        var val = scope.$eval(attributeValue);
        return angular.isDefined(val) ? val : defaultValue;
      }

      //model -> UI
      ngModelCtrl.$render = function() {
        element.toggleClass(buttonsCtrl.activeClass, angular.equals(ngModelCtrl.$modelValue, getTrueValue()));
      };

      //ui->model
      element.bind(buttonsCtrl.toggleEvent, function() {
        if (attrs.disabled) {
          return;
        }

        scope.$apply(function() {
          ngModelCtrl.$setViewValue(element.hasClass(buttonsCtrl.activeClass) ? getFalseValue() : getTrueValue());
          ngModelCtrl.$render();
        });
      });

      //accessibility
      element.on('keypress', function(e) {
        if (attrs.disabled || e.which !== 32 || $document[0].activeElement !== element[0]) {
          return;
        }

        scope.$apply(function() {
          ngModelCtrl.$setViewValue(element.hasClass(buttonsCtrl.activeClass) ? getFalseValue() : getTrueValue());
          ngModelCtrl.$render();
        });
      });
    }
  };
}]);

/**
* @ngdoc overview
* @name ui.bootstrap.carousel
*
* @description
* AngularJS version of an image carousel.
*
*/
angular.module('ui.bootstrap.carousel', [])
.controller('CarouselController', ['$scope', '$element', '$interval', '$animate', function ($scope, $element, $interval, $animate) {
  var self = this,
    slides = self.slides = $scope.slides = [],
    NEW_ANIMATE = angular.version.minor >= 4,
    NO_TRANSITION = 'uib-noTransition',
    SLIDE_DIRECTION = 'uib-slideDirection',
    currentIndex = -1,
    currentInterval, isPlaying;
  self.currentSlide = null;

  var destroyed = false;
  /* direction: "prev" or "next" */
  self.select = $scope.select = function(nextSlide, direction) {
    var nextIndex = $scope.indexOfSlide(nextSlide);
    //Decide direction if it's not given
    if (direction === undefined) {
      direction = nextIndex > self.getCurrentIndex() ? 'next' : 'prev';
    }
    //Prevent this user-triggered transition from occurring if there is already one in progress
    if (nextSlide && nextSlide !== self.currentSlide && !$scope.$currentTransition) {
      goNext(nextSlide, nextIndex, direction);
    }
  };

  function goNext(slide, index, direction) {
    // Scope has been destroyed, stop here.
    if (destroyed) { return; }

    angular.extend(slide, {direction: direction, active: true});
    angular.extend(self.currentSlide || {}, {direction: direction, active: false});
    if ($animate.enabled() && !$scope.noTransition && !$scope.$currentTransition &&
      slide.$element && self.slides.length > 1) {
      slide.$element.data(SLIDE_DIRECTION, slide.direction);
      if (self.currentSlide && self.currentSlide.$element) {
        self.currentSlide.$element.data(SLIDE_DIRECTION, slide.direction);
      }

      $scope.$currentTransition = true;
      if (NEW_ANIMATE) {
        $animate.on('addClass', slide.$element, function (element, phase) {
          if (phase === 'close') {
            $scope.$currentTransition = null;
            $animate.off('addClass', element);
          }
        });
      } else {
        slide.$element.one('$animate:close', function closeFn() {
          $scope.$currentTransition = null;
        });
      }
    }

    self.currentSlide = slide;
    currentIndex = index;

    //every time you change slides, reset the timer
    restartTimer();
  }

  $scope.$on('$destroy', function () {
    destroyed = true;
  });

  function getSlideByIndex(index) {
    if (angular.isUndefined(slides[index].index)) {
      return slides[index];
    }
    var i, len = slides.length;
    for (i = 0; i < slides.length; ++i) {
      if (slides[i].index == index) {
        return slides[i];
      }
    }
  }

  self.getCurrentIndex = function() {
    if (self.currentSlide && angular.isDefined(self.currentSlide.index)) {
      return +self.currentSlide.index;
    }
    return currentIndex;
  };

  /* Allow outside people to call indexOf on slides array */
  $scope.indexOfSlide = function(slide) {
    return angular.isDefined(slide.index) ? +slide.index : slides.indexOf(slide);
  };

  $scope.next = function() {
    var newIndex = (self.getCurrentIndex() + 1) % slides.length;

    if (newIndex === 0 && $scope.noWrap()) {
      $scope.pause();
      return;
    }

    return self.select(getSlideByIndex(newIndex), 'next');
  };

  $scope.prev = function() {
    var newIndex = self.getCurrentIndex() - 1 < 0 ? slides.length - 1 : self.getCurrentIndex() - 1;

    if ($scope.noWrap() && newIndex === slides.length - 1){
      $scope.pause();
      return;
    }

    return self.select(getSlideByIndex(newIndex), 'prev');
  };

  $scope.isActive = function(slide) {
     return self.currentSlide === slide;
  };

  $scope.$watch('interval', restartTimer);
  $scope.$on('$destroy', resetTimer);

  function restartTimer() {
    resetTimer();
    var interval = +$scope.interval;
    if (!isNaN(interval) && interval > 0) {
      currentInterval = $interval(timerFn, interval);
    }
  }

  function resetTimer() {
    if (currentInterval) {
      $interval.cancel(currentInterval);
      currentInterval = null;
    }
  }

  function timerFn() {
    var interval = +$scope.interval;
    if (isPlaying && !isNaN(interval) && interval > 0 && slides.length) {
      $scope.next();
    } else {
      $scope.pause();
    }
  }

  $scope.play = function() {
    if (!isPlaying) {
      isPlaying = true;
      restartTimer();
    }
  };
  $scope.pause = function() {
    if (!$scope.noPause) {
      isPlaying = false;
      resetTimer();
    }
  };

  self.addSlide = function(slide, element) {
    slide.$element = element;
    slides.push(slide);
    //if this is the first slide or the slide is set to active, select it
    if(slides.length === 1 || slide.active) {
      self.select(slides[slides.length-1]);
      if (slides.length == 1) {
        $scope.play();
      }
    } else {
      slide.active = false;
    }
  };

  self.removeSlide = function(slide) {
    if (angular.isDefined(slide.index)) {
      slides.sort(function(a, b) {
        return +a.index > +b.index;
      });
    }
    //get the index of the slide inside the carousel
    var index = slides.indexOf(slide);
    slides.splice(index, 1);
    if (slides.length > 0 && slide.active) {
      if (index >= slides.length) {
        self.select(slides[index-1]);
      } else {
        self.select(slides[index]);
      }
    } else if (currentIndex > index) {
      currentIndex--;
    }
    
    //clean the currentSlide when no more slide
    if (slides.length === 0) {
      self.currentSlide = null;
    }
  };

  $scope.$watch('noTransition', function(noTransition) {
    $element.data(NO_TRANSITION, noTransition);
  });

}])

/**
 * @ngdoc directive
 * @name ui.bootstrap.carousel.directive:carousel
 * @restrict EA
 *
 * @description
 * Carousel is the outer container for a set of image 'slides' to showcase.
 *
 * @param {number=} interval The time, in milliseconds, that it will take the carousel to go to the next slide.
 * @param {boolean=} noTransition Whether to disable transitions on the carousel.
 * @param {boolean=} noPause Whether to disable pausing on the carousel (by default, the carousel interval pauses on hover).
 *
 * @example
<example module="ui.bootstrap">
  <file name="index.html">
    <carousel>
      <slide>
        <img src="http://placekitten.com/150/150" style="margin:auto;">
        <div class="carousel-caption">
          <p>Beautiful!</p>
        </div>
      </slide>
      <slide>
        <img src="http://placekitten.com/100/150" style="margin:auto;">
        <div class="carousel-caption">
          <p>D'aww!</p>
        </div>
      </slide>
    </carousel>
  </file>
  <file name="demo.css">
    .carousel-indicators {
      top: auto;
      bottom: 15px;
    }
  </file>
</example>
 */
.directive('carousel', [function() {
  return {
    restrict: 'EA',
    transclude: true,
    replace: true,
    controller: 'CarouselController',
    controllerAs: 'carousel',
    require: 'carousel',
    templateUrl: function(element, attrs) {
      return attrs.templateUrl || 'template/carousel/carousel.html';
    },
    scope: {
      interval: '=',
      noTransition: '=',
      noPause: '=',
      noWrap: '&'
    }
  };
}])

/**
 * @ngdoc directive
 * @name ui.bootstrap.carousel.directive:slide
 * @restrict EA
 *
 * @description
 * Creates a slide inside a {@link ui.bootstrap.carousel.directive:carousel carousel}.  Must be placed as a child of a carousel element.
 *
 * @param {boolean=} active Model binding, whether or not this slide is currently active.
 * @param {number=} index The index of the slide. The slides will be sorted by this parameter.
 *
 * @example
<example module="ui.bootstrap">
  <file name="index.html">
<div ng-controller="CarouselDemoCtrl">
  <carousel>
    <slide ng-repeat="slide in slides" active="slide.active" index="$index">
      <img ng-src="{{slide.image}}" style="margin:auto;">
      <div class="carousel-caption">
        <h4>Slide {{$index}}</h4>
        <p>{{slide.text}}</p>
      </div>
    </slide>
  </carousel>
  Interval, in milliseconds: <input type="number" ng-model="myInterval">
  <br />Enter a negative number to stop the interval.
</div>
  </file>
  <file name="script.js">
function CarouselDemoCtrl($scope) {
  $scope.myInterval = 5000;
}
  </file>
  <file name="demo.css">
    .carousel-indicators {
      top: auto;
      bottom: 15px;
    }
  </file>
</example>
*/

.directive('slide', function() {
  return {
    require: '^carousel',
    restrict: 'EA',
    transclude: true,
    replace: true,
    templateUrl: function(element, attrs) {
      return attrs.templateUrl || 'template/carousel/slide.html';
    },
    scope: {
      active: '=?',
      actual: '=?',
      index: '=?'
    },
    link: function (scope, element, attrs, carouselCtrl) {
      carouselCtrl.addSlide(scope, element);
      //when the scope is destroyed then remove the slide from the current slides array
      scope.$on('$destroy', function() {
        carouselCtrl.removeSlide(scope);
      });

      scope.$watch('active', function(active) {
        if (active) {
          carouselCtrl.select(scope);
        }
      });
    }
  };
})

.animation('.item', [
         '$injector', '$animate',
function ($injector, $animate) {
  var NO_TRANSITION = 'uib-noTransition',
    SLIDE_DIRECTION = 'uib-slideDirection',
    $animateCss = null;

  if ($injector.has('$animateCss')) {
    $animateCss = $injector.get('$animateCss');
  }

  function removeClass(element, className, callback) {
    element.removeClass(className);
    if (callback) {
      callback();
    }
  }

  return {
    beforeAddClass: function (element, className, done) {
      // Due to transclusion, noTransition property is on parent's scope
      if (className == 'active' && element.parent() &&
          !element.parent().data(NO_TRANSITION)) {
        var stopped = false;
        var direction = element.data(SLIDE_DIRECTION);
        var directionClass = direction == 'next' ? 'left' : 'right';
        var removeClassFn = removeClass.bind(this, element,
          directionClass + ' ' + direction, done);
        element.addClass(direction);

        if ($animateCss) {
          $animateCss(element, {addClass: directionClass})
            .start()
            .done(removeClassFn);
        } else {
          $animate.addClass(element, directionClass).then(function () {
            if (!stopped) {
              removeClassFn();
            }
            done();
          });
        }

        return function () {
          stopped = true;
        };
      }
      done();
    },
    beforeRemoveClass: function (element, className, done) {
      // Due to transclusion, noTransition property is on parent's scope
      if (className === 'active' && element.parent() &&
          !element.parent().data(NO_TRANSITION)) {
        var stopped = false;
        var direction = element.data(SLIDE_DIRECTION);
        var directionClass = direction == 'next' ? 'left' : 'right';
        var removeClassFn = removeClass.bind(this, element, directionClass, done);

        if ($animateCss) {
          $animateCss(element, {addClass: directionClass})
            .start()
            .done(removeClassFn);
        } else {
          $animate.addClass(element, directionClass).then(function () {
            if (!stopped) {
              removeClassFn();
            }
            done();
          });
        }
        return function () {
          stopped = true;
        };
      }
      done();
    }
  };

}])


;

angular.module('ui.bootstrap.dateparser', [])

.service('dateParser', ['$log', '$locale', 'orderByFilter', function($log, $locale, orderByFilter) {
  // Pulled from https://github.com/mbostock/d3/blob/master/src/format/requote.js
  var SPECIAL_CHARACTERS_REGEXP = /[\\\^\$\*\+\?\|\[\]\(\)\.\{\}]/g;

  this.parsers = {};

  var formatCodeToRegex = {
    'yyyy': {
      regex: '\\d{4}',
      apply: function(value) { this.year = +value; }
    },
    'yy': {
      regex: '\\d{2}',
      apply: function(value) { this.year = +value + 2000; }
    },
    'y': {
      regex: '\\d{1,4}',
      apply: function(value) { this.year = +value; }
    },
    'MMMM': {
      regex: $locale.DATETIME_FORMATS.MONTH.join('|'),
      apply: function(value) { this.month = $locale.DATETIME_FORMATS.MONTH.indexOf(value); }
    },
    'MMM': {
      regex: $locale.DATETIME_FORMATS.SHORTMONTH.join('|'),
      apply: function(value) { this.month = $locale.DATETIME_FORMATS.SHORTMONTH.indexOf(value); }
    },
    'MM': {
      regex: '0[1-9]|1[0-2]',
      apply: function(value) { this.month = value - 1; }
    },
    'M': {
      regex: '[1-9]|1[0-2]',
      apply: function(value) { this.month = value - 1; }
    },
    'dd': {
      regex: '[0-2][0-9]{1}|3[0-1]{1}',
      apply: function(value) { this.date = +value; }
    },
    'd': {
      regex: '[1-2]?[0-9]{1}|3[0-1]{1}',
      apply: function(value) { this.date = +value; }
    },
    'EEEE': {
      regex: $locale.DATETIME_FORMATS.DAY.join('|')
    },
    'EEE': {
      regex: $locale.DATETIME_FORMATS.SHORTDAY.join('|')
    },
    'HH': {
      regex: '(?:0|1)[0-9]|2[0-3]',
      apply: function(value) { this.hours = +value; }
    },
    'hh': {
      regex: '0[0-9]|1[0-2]',
      apply: function(value) { this.hours = +value; }
    },
    'H': {
      regex: '1?[0-9]|2[0-3]',
      apply: function(value) { this.hours = +value; }
    },
    'h': {
      regex: '[0-9]|1[0-2]',
      apply: function(value) { this.hours = +value; }
    },
    'mm': {
      regex: '[0-5][0-9]',
      apply: function(value) { this.minutes = +value; }
    },
    'm': {
      regex: '[0-9]|[1-5][0-9]',
      apply: function(value) { this.minutes = +value; }
    },
    'sss': {
      regex: '[0-9][0-9][0-9]',
      apply: function(value) { this.milliseconds = +value; }
    },
    'ss': {
      regex: '[0-5][0-9]',
      apply: function(value) { this.seconds = +value; }
    },
    's': {
      regex: '[0-9]|[1-5][0-9]',
      apply: function(value) { this.seconds = +value; }
    },
    'a': {
      regex: $locale.DATETIME_FORMATS.AMPMS.join('|'),
      apply: function(value) {
        if (this.hours === 12) {
          this.hours = 0;
        }

        if (value === 'PM') {
          this.hours += 12;
        }
      }
    }
  };

  function createParser(format) {
    var map = [], regex = format.split('');

    angular.forEach(formatCodeToRegex, function(data, code) {
      var index = format.indexOf(code);

      if (index > -1) {
        format = format.split('');

        regex[index] = '(' + data.regex + ')';
        format[index] = '$'; // Custom symbol to define consumed part of format
        for (var i = index + 1, n = index + code.length; i < n; i++) {
          regex[i] = '';
          format[i] = '$';
        }
        format = format.join('');

        map.push({ index: index, apply: data.apply });
      }
    });

    return {
      regex: new RegExp('^' + regex.join('') + '$'),
      map: orderByFilter(map, 'index')
    };
  }

  this.parse = function(input, format, baseDate) {
    if (!angular.isString(input) || !format) {
      return input;
    }

    format = $locale.DATETIME_FORMATS[format] || format;
    format = format.replace(SPECIAL_CHARACTERS_REGEXP, '\\$&');

    if (!this.parsers[format]) {
      this.parsers[format] = createParser(format);
    }

    var parser = this.parsers[format],
        regex = parser.regex,
        map = parser.map,
        results = input.match(regex);

    if (results && results.length) {
      var fields, dt;
      if (angular.isDate(baseDate) && !isNaN(baseDate.getTime())) {
        fields = {
          year: baseDate.getFullYear(),
          month: baseDate.getMonth(),
          date: baseDate.getDate(),
          hours: baseDate.getHours(),
          minutes: baseDate.getMinutes(),
          seconds: baseDate.getSeconds(),
          milliseconds: baseDate.getMilliseconds()
        };
      } else {
        if (baseDate) {
          $log.warn('dateparser:', 'baseDate is not a valid date');
        }
        fields = { year: 1900, month: 0, date: 1, hours: 0, minutes: 0, seconds: 0, milliseconds: 0 };
      }

      for (var i = 1, n = results.length; i < n; i++) {
        var mapper = map[i-1];
        if (mapper.apply) {
          mapper.apply.call(fields, results[i]);
        }
      }

      if (isValid(fields.year, fields.month, fields.date)) {
        dt = new Date(fields.year, fields.month, fields.date,
          fields.hours, fields.minutes, fields.seconds,
          fields.milliseconds || 0);
      }

      return dt;
    }
  };

  // Check if date is valid for specific month (and year for February).
  // Month: 0 = Jan, 1 = Feb, etc
  function isValid(year, month, date) {
    if (date < 1) {
      return false;
    }

    if (month === 1 && date > 28) {
      return date === 29 && ((year % 4 === 0 && year % 100 !== 0) || year % 400 === 0);
    }

    if (month === 3 || month === 5 || month === 8 || month === 10) {
      return date < 31;
    }

    return true;
  }
}]);

angular.module('ui.bootstrap.position', [])

/**
 * A set of utility methods that can be use to retrieve position of DOM elements.
 * It is meant to be used where we need to absolute-position DOM elements in
 * relation to other, existing elements (this is the case for tooltips, popovers,
 * typeahead suggestions etc.).
 */
  .factory('$position', ['$document', '$window', function($document, $window) {
    function getStyle(el, cssprop) {
      if (el.currentStyle) { //IE
        return el.currentStyle[cssprop];
      } else if ($window.getComputedStyle) {
        return $window.getComputedStyle(el)[cssprop];
      }
      // finally try and get inline style
      return el.style[cssprop];
    }

    /**
     * Checks if a given element is statically positioned
     * @param element - raw DOM element
     */
    function isStaticPositioned(element) {
      return (getStyle(element, 'position') || 'static' ) === 'static';
    }

    /**
     * returns the closest, non-statically positioned parentOffset of a given element
     * @param element
     */
    var parentOffsetEl = function(element) {
      var docDomEl = $document[0];
      var offsetParent = element.offsetParent || docDomEl;
      while (offsetParent && offsetParent !== docDomEl && isStaticPositioned(offsetParent) ) {
        offsetParent = offsetParent.offsetParent;
      }
      return offsetParent || docDomEl;
    };

    return {
      /**
       * Provides read-only equivalent of jQuery's position function:
       * http://api.jquery.com/position/
       */
      position: function(element) {
        var elBCR = this.offset(element);
        var offsetParentBCR = { top: 0, left: 0 };
        var offsetParentEl = parentOffsetEl(element[0]);
        if (offsetParentEl != $document[0]) {
          offsetParentBCR = this.offset(angular.element(offsetParentEl));
          offsetParentBCR.top += offsetParentEl.clientTop - offsetParentEl.scrollTop;
          offsetParentBCR.left += offsetParentEl.clientLeft - offsetParentEl.scrollLeft;
        }

        var boundingClientRect = element[0].getBoundingClientRect();
        return {
          width: boundingClientRect.width || element.prop('offsetWidth'),
          height: boundingClientRect.height || element.prop('offsetHeight'),
          top: elBCR.top - offsetParentBCR.top,
          left: elBCR.left - offsetParentBCR.left
        };
      },

      /**
       * Provides read-only equivalent of jQuery's offset function:
       * http://api.jquery.com/offset/
       */
      offset: function(element) {
        var boundingClientRect = element[0].getBoundingClientRect();
        return {
          width: boundingClientRect.width || element.prop('offsetWidth'),
          height: boundingClientRect.height || element.prop('offsetHeight'),
          top: boundingClientRect.top + ($window.pageYOffset || $document[0].documentElement.scrollTop),
          left: boundingClientRect.left + ($window.pageXOffset || $document[0].documentElement.scrollLeft)
        };
      },

      /**
       * Provides coordinates for the targetEl in relation to hostEl
       */
      positionElements: function(hostEl, targetEl, positionStr, appendToBody) {
        var positionStrParts = positionStr.split('-');
        var pos0 = positionStrParts[0], pos1 = positionStrParts[1] || 'center';

        var hostElPos,
          targetElWidth,
          targetElHeight,
          targetElPos;

        hostElPos = appendToBody ? this.offset(hostEl) : this.position(hostEl);

        targetElWidth = targetEl.prop('offsetWidth');
        targetElHeight = targetEl.prop('offsetHeight');

        var shiftWidth = {
          center: function() {
            return hostElPos.left + hostElPos.width / 2 - targetElWidth / 2;
          },
          left: function() {
            return hostElPos.left;
          },
          right: function() {
            return hostElPos.left + hostElPos.width;
          }
        };

        var shiftHeight = {
          center: function() {
            return hostElPos.top + hostElPos.height / 2 - targetElHeight / 2;
          },
          top: function() {
            return hostElPos.top;
          },
          bottom: function() {
            return hostElPos.top + hostElPos.height;
          }
        };

        switch (pos0) {
          case 'right':
            targetElPos = {
              top: shiftHeight[pos1](),
              left: shiftWidth[pos0]()
            };
            break;
          case 'left':
            targetElPos = {
              top: shiftHeight[pos1](),
              left: hostElPos.left - targetElWidth
            };
            break;
          case 'bottom':
            targetElPos = {
              top: shiftHeight[pos0](),
              left: shiftWidth[pos1]()
            };
            break;
          default:
            targetElPos = {
              top: hostElPos.top - targetElHeight,
              left: shiftWidth[pos1]()
            };
            break;
        }

        return targetElPos;
      }
    };
  }]);

angular.module('ui.bootstrap.datepicker', ['ui.bootstrap.dateparser', 'ui.bootstrap.position'])

.value('$datepickerSuppressError', false)

.constant('datepickerConfig', {
  formatDay: 'dd',
  formatMonth: 'MMMM',
  formatYear: 'yyyy',
  formatDayHeader: 'EEE',
  formatDayTitle: 'MMMM yyyy',
  formatMonthTitle: 'yyyy',
  datepickerMode: 'day',
  minMode: 'day',
  maxMode: 'year',
  showWeeks: true,
  startingDay: 0,
  yearRange: 20,
  minDate: null,
  maxDate: null,
  shortcutPropagation: false
})

.controller('DatepickerController', ['$scope', '$attrs', '$parse', '$interpolate', '$log', 'dateFilter', 'datepickerConfig', '$datepickerSuppressError', function($scope, $attrs, $parse, $interpolate, $log, dateFilter, datepickerConfig, $datepickerSuppressError) {
  var self = this,
      ngModelCtrl = { $setViewValue: angular.noop }; // nullModelCtrl;

  // Modes chain
  this.modes = ['day', 'month', 'year'];

  // Configuration attributes
  angular.forEach(['formatDay', 'formatMonth', 'formatYear', 'formatDayHeader', 'formatDayTitle', 'formatMonthTitle',
                   'showWeeks', 'startingDay', 'yearRange', 'shortcutPropagation'], function(key, index) {
    self[key] = angular.isDefined($attrs[key]) ? (index < 6 ? $interpolate($attrs[key])($scope.$parent) : $scope.$parent.$eval($attrs[key])) : datepickerConfig[key];
  });

  // Watchable date attributes
  angular.forEach(['minDate', 'maxDate'], function(key) {
    if ($attrs[key]) {
      $scope.$parent.$watch($parse($attrs[key]), function(value) {
        self[key] = value ? new Date(value) : null;
        self.refreshView();
      });
    } else {
      self[key] = datepickerConfig[key] ? new Date(datepickerConfig[key]) : null;
    }
  });

  angular.forEach(['minMode', 'maxMode'], function(key) {
    if ($attrs[key]) {
      $scope.$parent.$watch($parse($attrs[key]), function(value) {
        self[key] = angular.isDefined(value) ? value : $attrs[key];
        $scope[key] = self[key];
        if ((key == 'minMode' && self.modes.indexOf($scope.datepickerMode) < self.modes.indexOf(self[key])) || (key == 'maxMode' && self.modes.indexOf($scope.datepickerMode) > self.modes.indexOf(self[key]))) {
          $scope.datepickerMode = self[key];
        }
      });
    } else {
      self[key] = datepickerConfig[key] || null;
      $scope[key] = self[key];
    }
  });

  $scope.datepickerMode = $scope.datepickerMode || datepickerConfig.datepickerMode;
  $scope.uniqueId = 'datepicker-' + $scope.$id + '-' + Math.floor(Math.random() * 10000);

  if (angular.isDefined($attrs.initDate)) {
    this.activeDate = $scope.$parent.$eval($attrs.initDate) || new Date();
    $scope.$parent.$watch($attrs.initDate, function(initDate) {
      if (initDate && (ngModelCtrl.$isEmpty(ngModelCtrl.$modelValue) || ngModelCtrl.$invalid)) {
        self.activeDate = initDate;
        self.refreshView();
      }
    });
  } else {
    this.activeDate = new Date();
  }

  $scope.isActive = function(dateObject) {
    if (self.compare(dateObject.date, self.activeDate) === 0) {
      $scope.activeDateId = dateObject.uid;
      return true;
    }
    return false;
  };

  this.init = function(ngModelCtrl_) {
    ngModelCtrl = ngModelCtrl_;

    ngModelCtrl.$render = function() {
      self.render();
    };
  };

  this.render = function() {
    if (ngModelCtrl.$viewValue) {
      var date = new Date(ngModelCtrl.$viewValue),
          isValid = !isNaN(date);

      if (isValid) {
        this.activeDate = date;
      } else if (!$datepickerSuppressError) {
        $log.error('Datepicker directive: "ng-model" value must be a Date object, a number of milliseconds since 01.01.1970 or a string representing an RFC2822 or ISO 8601 date.');
      }
    }
    this.refreshView();
  };

  this.refreshView = function() {
    if (this.element) {
      this._refreshView();

      var date = ngModelCtrl.$viewValue ? new Date(ngModelCtrl.$viewValue) : null;
      ngModelCtrl.$setValidity('dateDisabled', !date || (this.element && !this.isDisabled(date)));
    }
  };

  this.createDateObject = function(date, format) {
    var model = ngModelCtrl.$viewValue ? new Date(ngModelCtrl.$viewValue) : null;
    return {
      date: date,
      label: dateFilter(date, format),
      selected: model && this.compare(date, model) === 0,
      disabled: this.isDisabled(date),
      current: this.compare(date, new Date()) === 0,
      customClass: this.customClass(date)
    };
  };

  this.isDisabled = function(date) {
    return ((this.minDate && this.compare(date, this.minDate) < 0) || (this.maxDate && this.compare(date, this.maxDate) > 0) || ($attrs.dateDisabled && $scope.dateDisabled({date: date, mode: $scope.datepickerMode})));
  };

  this.customClass = function(date) {
    return $scope.customClass({date: date, mode: $scope.datepickerMode});
  };

  // Split array into smaller arrays
  this.split = function(arr, size) {
    var arrays = [];
    while (arr.length > 0) {
      arrays.push(arr.splice(0, size));
    }
    return arrays;
  };

  // Fix a hard-reprodusible bug with timezones
  // The bug depends on OS, browser, current timezone and current date
  // i.e.
  // var date = new Date(2014, 0, 1);
  // console.log(date.getFullYear(), date.getMonth(), date.getDate(), date.getHours());
  // can result in "2013 11 31 23" because of the bug.
  this.fixTimeZone = function(date) {
    var hours = date.getHours();
    date.setHours(hours === 23 ? hours + 2 : 0);
  };

  $scope.select = function(date) {
    if ($scope.datepickerMode === self.minMode) {
      var dt = ngModelCtrl.$viewValue ? new Date(ngModelCtrl.$viewValue) : new Date(0, 0, 0, 0, 0, 0, 0);
      dt.setFullYear(date.getFullYear(), date.getMonth(), date.getDate());
      ngModelCtrl.$setViewValue(dt);
      ngModelCtrl.$render();
    } else {
      self.activeDate = date;
      $scope.datepickerMode = self.modes[self.modes.indexOf($scope.datepickerMode) - 1];
    }
  };

  $scope.move = function(direction) {
    var year = self.activeDate.getFullYear() + direction * (self.step.years || 0),
        month = self.activeDate.getMonth() + direction * (self.step.months || 0);
    self.activeDate.setFullYear(year, month, 1);
    self.refreshView();
  };

  $scope.toggleMode = function(direction) {
    direction = direction || 1;

    if (($scope.datepickerMode === self.maxMode && direction === 1) || ($scope.datepickerMode === self.minMode && direction === -1)) {
      return;
    }

    $scope.datepickerMode = self.modes[self.modes.indexOf($scope.datepickerMode) + direction];
  };

  // Key event mapper
  $scope.keys = { 13: 'enter', 32: 'space', 33: 'pageup', 34: 'pagedown', 35: 'end', 36: 'home', 37: 'left', 38: 'up', 39: 'right', 40: 'down' };

  var focusElement = function() {
    self.element[0].focus();
  };

  // Listen for focus requests from popup directive
  $scope.$on('datepicker.focus', focusElement);

  $scope.keydown = function(evt) {
    var key = $scope.keys[evt.which];

    if (!key || evt.shiftKey || evt.altKey) {
      return;
    }

    evt.preventDefault();
    if (!self.shortcutPropagation) {
      evt.stopPropagation();
    }

    if (key === 'enter' || key === 'space') {
      if (self.isDisabled(self.activeDate)) {
        return; // do nothing
      }
      $scope.select(self.activeDate);
      focusElement();
    } else if (evt.ctrlKey && (key === 'up' || key === 'down')) {
      $scope.toggleMode(key === 'up' ? 1 : -1);
      focusElement();
    } else {
      self.handleKeyDown(key, evt);
      self.refreshView();
    }
  };
}])

.directive('datepicker', function() {
  return {
    restrict: 'EA',
    replace: true,
    templateUrl: function(element, attrs) {
      return attrs.templateUrl || 'template/datepicker/datepicker.html';
    },
    scope: {
      datepickerMode: '=?',
      dateDisabled: '&',
      customClass: '&',
      shortcutPropagation: '&?'
    },
    require: ['datepicker', '^ngModel'],
    controller: 'DatepickerController',
    controllerAs: 'datepicker',
    link: function(scope, element, attrs, ctrls) {
      var datepickerCtrl = ctrls[0], ngModelCtrl = ctrls[1];

      datepickerCtrl.init(ngModelCtrl);
    }
  };
})

.directive('daypicker', ['dateFilter', function(dateFilter) {
  return {
    restrict: 'EA',
    replace: true,
    templateUrl: 'template/datepicker/day.html',
    require: '^datepicker',
    link: function(scope, element, attrs, ctrl) {
      scope.showWeeks = ctrl.showWeeks;

      ctrl.step = { months: 1 };
      ctrl.element = element;

      var DAYS_IN_MONTH = [31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31];
      function getDaysInMonth(year, month) {
        return ((month === 1) && (year % 4 === 0) && ((year % 100 !== 0) || (year % 400 === 0))) ? 29 : DAYS_IN_MONTH[month];
      }

      function getDates(startDate, n) {
        var dates = new Array(n), current = new Date(startDate), i = 0, date;
        while (i < n) {
          date = new Date(current);
          ctrl.fixTimeZone(date);
          dates[i++] = date;
          current.setDate(current.getDate() + 1);
        }
        return dates;
      }

      ctrl._refreshView = function() {
        var year = ctrl.activeDate.getFullYear(),
          month = ctrl.activeDate.getMonth(),
          firstDayOfMonth = new Date(year, month, 1),
          difference = ctrl.startingDay - firstDayOfMonth.getDay(),
          numDisplayedFromPreviousMonth = (difference > 0) ? 7 - difference : - difference,
          firstDate = new Date(firstDayOfMonth);

        if (numDisplayedFromPreviousMonth > 0) {
          firstDate.setDate(-numDisplayedFromPreviousMonth + 1);
        }

        // 42 is the number of days on a six-month calendar
        var days = getDates(firstDate, 42);
        for (var i = 0; i < 42; i ++) {
          days[i] = angular.extend(ctrl.createDateObject(days[i], ctrl.formatDay), {
            secondary: days[i].getMonth() !== month,
            uid: scope.uniqueId + '-' + i
          });
        }

        scope.labels = new Array(7);
        for (var j = 0; j < 7; j++) {
          scope.labels[j] = {
            abbr: dateFilter(days[j].date, ctrl.formatDayHeader),
            full: dateFilter(days[j].date, 'EEEE')
          };
        }

        scope.title = dateFilter(ctrl.activeDate, ctrl.formatDayTitle);
        scope.rows = ctrl.split(days, 7);

        if (scope.showWeeks) {
          scope.weekNumbers = [];
          var thursdayIndex = (4 + 7 - ctrl.startingDay) % 7,
              numWeeks = scope.rows.length;
          for (var curWeek = 0; curWeek < numWeeks; curWeek++) {
            scope.weekNumbers.push(
              getISO8601WeekNumber(scope.rows[curWeek][thursdayIndex].date));
          }
        }
      };

      ctrl.compare = function(date1, date2) {
        return (new Date(date1.getFullYear(), date1.getMonth(), date1.getDate()) - new Date(date2.getFullYear(), date2.getMonth(), date2.getDate()));
      };

      function getISO8601WeekNumber(date) {
        var checkDate = new Date(date);
        checkDate.setDate(checkDate.getDate() + 4 - (checkDate.getDay() || 7)); // Thursday
        var time = checkDate.getTime();
        checkDate.setMonth(0); // Compare with Jan 1
        checkDate.setDate(1);
        return Math.floor(Math.round((time - checkDate) / 86400000) / 7) + 1;
      }

      ctrl.handleKeyDown = function(key, evt) {
        var date = ctrl.activeDate.getDate();

        if (key === 'left') {
          date = date - 1;   // up
        } else if (key === 'up') {
          date = date - 7;   // down
        } else if (key === 'right') {
          date = date + 1;   // down
        } else if (key === 'down') {
          date = date + 7;
        } else if (key === 'pageup' || key === 'pagedown') {
          var month = ctrl.activeDate.getMonth() + (key === 'pageup' ? - 1 : 1);
          ctrl.activeDate.setMonth(month, 1);
          date = Math.min(getDaysInMonth(ctrl.activeDate.getFullYear(), ctrl.activeDate.getMonth()), date);
        } else if (key === 'home') {
          date = 1;
        } else if (key === 'end') {
          date = getDaysInMonth(ctrl.activeDate.getFullYear(), ctrl.activeDate.getMonth());
        }
        ctrl.activeDate.setDate(date);
      };

      ctrl.refreshView();
    }
  };
}])

.directive('monthpicker', ['dateFilter', function(dateFilter) {
  return {
    restrict: 'EA',
    replace: true,
    templateUrl: 'template/datepicker/month.html',
    require: '^datepicker',
    link: function(scope, element, attrs, ctrl) {
      ctrl.step = { years: 1 };
      ctrl.element = element;

      ctrl._refreshView = function() {
        var months = new Array(12),
            year = ctrl.activeDate.getFullYear(),
            date;

        for (var i = 0; i < 12; i++) {
          date = new Date(year, i, 1);
          ctrl.fixTimeZone(date);
          months[i] = angular.extend(ctrl.createDateObject(date, ctrl.formatMonth), {
            uid: scope.uniqueId + '-' + i
          });
        }

        scope.title = dateFilter(ctrl.activeDate, ctrl.formatMonthTitle);
        scope.rows = ctrl.split(months, 3);
      };

      ctrl.compare = function(date1, date2) {
        return new Date(date1.getFullYear(), date1.getMonth()) - new Date(date2.getFullYear(), date2.getMonth());
      };

      ctrl.handleKeyDown = function(key, evt) {
        var date = ctrl.activeDate.getMonth();

        if (key === 'left') {
          date = date - 1;   // up
        } else if (key === 'up') {
          date = date - 3;   // down
        } else if (key === 'right') {
          date = date + 1;   // down
        } else if (key === 'down') {
          date = date + 3;
        } else if (key === 'pageup' || key === 'pagedown') {
          var year = ctrl.activeDate.getFullYear() + (key === 'pageup' ? - 1 : 1);
          ctrl.activeDate.setFullYear(year);
        } else if (key === 'home') {
          date = 0;
        } else if (key === 'end') {
          date = 11;
        }
        ctrl.activeDate.setMonth(date);
      };

      ctrl.refreshView();
    }
  };
}])

.directive('yearpicker', ['dateFilter', function(dateFilter) {
  return {
    restrict: 'EA',
    replace: true,
    templateUrl: 'template/datepicker/year.html',
    require: '^datepicker',
    link: function(scope, element, attrs, ctrl) {
      var range = ctrl.yearRange;

      ctrl.step = { years: range };
      ctrl.element = element;

      function getStartingYear( year ) {
        return parseInt((year - 1) / range, 10) * range + 1;
      }

      ctrl._refreshView = function() {
        var years = new Array(range), date;

        for (var i = 0, start = getStartingYear(ctrl.activeDate.getFullYear()); i < range; i++) {
          date = new Date(start + i, 0, 1);
          ctrl.fixTimeZone(date);
          years[i] = angular.extend(ctrl.createDateObject(date, ctrl.formatYear), {
            uid: scope.uniqueId + '-' + i
          });
        }

        scope.title = [years[0].label, years[range - 1].label].join(' - ');
        scope.rows = ctrl.split(years, 5);
      };

      ctrl.compare = function(date1, date2) {
        return date1.getFullYear() - date2.getFullYear();
      };

      ctrl.handleKeyDown = function(key, evt) {
        var date = ctrl.activeDate.getFullYear();

        if (key === 'left') {
          date = date - 1;   // up
        } else if (key === 'up') {
          date = date - 5;   // down
        } else if (key === 'right') {
          date = date + 1;   // down
        } else if (key === 'down') {
          date = date + 5;
        } else if (key === 'pageup' || key === 'pagedown') {
          date += (key === 'pageup' ? - 1 : 1) * ctrl.step.years;
        } else if (key === 'home') {
          date = getStartingYear(ctrl.activeDate.getFullYear());
        } else if (key === 'end') {
          date = getStartingYear(ctrl.activeDate.getFullYear()) + range - 1;
        }
        ctrl.activeDate.setFullYear(date);
      };

      ctrl.refreshView();
    }
  };
}])

.constant('datepickerPopupConfig', {
  datepickerPopup: 'yyyy-MM-dd',
  datepickerPopupTemplateUrl: 'template/datepicker/popup.html',
  datepickerTemplateUrl: 'template/datepicker/datepicker.html',
  html5Types: {
    date: 'yyyy-MM-dd',
    'datetime-local': 'yyyy-MM-ddTHH:mm:ss.sss',
    'month': 'yyyy-MM'
  },
  currentText: 'Today',
  clearText: 'Clear',
  closeText: 'Done',
  closeOnDateSelection: true,
  appendToBody: false,
  showButtonBar: true,
  onOpenFocus: true
})

.directive('datepickerPopup', ['$compile', '$parse', '$document', '$rootScope', '$position', 'dateFilter', 'dateParser', 'datepickerPopupConfig', '$timeout',
function($compile, $parse, $document, $rootScope, $position, dateFilter, dateParser, datepickerPopupConfig, $timeout) {
  return {
    restrict: 'EA',
    require: 'ngModel',
    scope: {
      isOpen: '=?',
      currentText: '@',
      clearText: '@',
      closeText: '@',
      dateDisabled: '&',
      customClass: '&'
    },
    link: function(scope, element, attrs, ngModel) {
      var dateFormat,
          closeOnDateSelection = angular.isDefined(attrs.closeOnDateSelection) ? scope.$parent.$eval(attrs.closeOnDateSelection) : datepickerPopupConfig.closeOnDateSelection,
          appendToBody = angular.isDefined(attrs.datepickerAppendToBody) ? scope.$parent.$eval(attrs.datepickerAppendToBody) : datepickerPopupConfig.appendToBody,
          onOpenFocus = angular.isDefined(attrs.onOpenFocus) ? scope.$parent.$eval(attrs.onOpenFocus) : datepickerPopupConfig.onOpenFocus,
          datepickerPopupTemplateUrl = angular.isDefined(attrs.datepickerPopupTemplateUrl) ? attrs.datepickerPopupTemplateUrl : datepickerPopupConfig.datepickerPopupTemplateUrl,
          datepickerTemplateUrl = angular.isDefined(attrs.datepickerTemplateUrl) ? attrs.datepickerTemplateUrl : datepickerPopupConfig.datepickerTemplateUrl,
          cache = {};

      scope.showButtonBar = angular.isDefined(attrs.showButtonBar) ? scope.$parent.$eval(attrs.showButtonBar) : datepickerPopupConfig.showButtonBar;

      scope.getText = function(key) {
        return scope[key + 'Text'] || datepickerPopupConfig[key + 'Text'];
      };

      scope.isDisabled = function(date) {
        if (date === 'today') {
          date = new Date();
        }

        return ((scope.watchData.minDate && scope.compare(date, cache.minDate) < 0) ||
          (scope.watchData.maxDate && scope.compare(date, cache.maxDate) > 0));
      };

      scope.compare = function(date1, date2) {
        return (new Date(date1.getFullYear(), date1.getMonth(), date1.getDate()) - new Date(date2.getFullYear(), date2.getMonth(), date2.getDate()));
      };

      var isHtml5DateInput = false;
      if (datepickerPopupConfig.html5Types[attrs.type]) {
        dateFormat = datepickerPopupConfig.html5Types[attrs.type];
        isHtml5DateInput = true;
      } else {
        dateFormat = attrs.datepickerPopup || datepickerPopupConfig.datepickerPopup;
        attrs.$observe('datepickerPopup', function(value, oldValue) {
            var newDateFormat = value || datepickerPopupConfig.datepickerPopup;
            // Invalidate the $modelValue to ensure that formatters re-run
            // FIXME: Refactor when PR is merged: https://github.com/angular/angular.js/pull/10764
            if (newDateFormat !== dateFormat) {
              dateFormat = newDateFormat;
              ngModel.$modelValue = null;

              if (!dateFormat) {
                throw new Error('datepickerPopup must have a date format specified.');
              }
            }
        });
      }

      if (!dateFormat) {
        throw new Error('datepickerPopup must have a date format specified.');
      }

      if (isHtml5DateInput && attrs.datepickerPopup) {
        throw new Error('HTML5 date input types do not support custom formats.');
      }

      // popup element used to display calendar
      var popupEl = angular.element('<div datepicker-popup-wrap><div datepicker></div></div>');
      popupEl.attr({
        'ng-model': 'date',
        'ng-change': 'dateSelection(date)',
        'template-url': datepickerPopupTemplateUrl
      });

      function cameltoDash(string) {
        return string.replace(/([A-Z])/g, function($1) { return '-' + $1.toLowerCase(); });
      }

      // datepicker element
      var datepickerEl = angular.element(popupEl.children()[0]);
      datepickerEl.attr('template-url', datepickerTemplateUrl);

      if (isHtml5DateInput) {
        if (attrs.type === 'month') {
          datepickerEl.attr('datepicker-mode', '"month"');
          datepickerEl.attr('min-mode', 'month');
        }
      }

      if (attrs.datepickerOptions) {
        var options = scope.$parent.$eval(attrs.datepickerOptions);
        if (options && options.initDate) {
          scope.initDate = options.initDate;
          datepickerEl.attr('init-date', 'initDate');
          delete options.initDate;
        }
        angular.forEach(options, function(value, option) {
          datepickerEl.attr( cameltoDash(option), value );
        });
      }

      scope.watchData = {};
      angular.forEach(['minMode', 'maxMode', 'minDate', 'maxDate', 'datepickerMode', 'initDate', 'shortcutPropagation'], function(key) {
        if (attrs[key]) {
          var getAttribute = $parse(attrs[key]);
          scope.$parent.$watch(getAttribute, function(value) {
            scope.watchData[key] = value;
            if (key === 'minDate' || key === 'maxDate') {
              cache[key] = new Date(value);
            }
          });
          datepickerEl.attr(cameltoDash(key), 'watchData.' + key);

          // Propagate changes from datepicker to outside
          if (key === 'datepickerMode') {
            var setAttribute = getAttribute.assign;
            scope.$watch('watchData.' + key, function(value, oldvalue) {
              if (angular.isFunction(setAttribute) && value !== oldvalue) {
                setAttribute(scope.$parent, value);
              }
            });
          }
        }
      });
      if (attrs.dateDisabled) {
        datepickerEl.attr('date-disabled', 'dateDisabled({ date: date, mode: mode })');
      }

      if (attrs.showWeeks) {
        datepickerEl.attr('show-weeks', attrs.showWeeks);
      }

      if (attrs.customClass) {
        datepickerEl.attr('custom-class', 'customClass({ date: date, mode: mode })');
      }

      function parseDate(viewValue) {
        if (angular.isNumber(viewValue)) {
          // presumably timestamp to date object
          viewValue = new Date(viewValue);
        }

        if (!viewValue) {
          return null;
        } else if (angular.isDate(viewValue) && !isNaN(viewValue)) {
          return viewValue;
        } else if (angular.isString(viewValue)) {
          var date = dateParser.parse(viewValue, dateFormat, scope.date);
          if (isNaN(date)) {
            return undefined;
          } else {
            return date;
          }
        } else {
          return undefined;
        }
      }

      function validator(modelValue, viewValue) {
        var value = modelValue || viewValue;

        if (!attrs.ngRequired && !value) {
          return true;
        }

        if (angular.isNumber(value)) {
          value = new Date(value);
        }
        if (!value) {
          return true;
        } else if (angular.isDate(value) && !isNaN(value)) {
          return true;
        } else if (angular.isString(value)) {
          var date = dateParser.parse(value, dateFormat);
          return !isNaN(date);
        } else {
          return false;
        }
      }

      if (!isHtml5DateInput) {
        // Internal API to maintain the correct ng-invalid-[key] class
        ngModel.$$parserName = 'date';
        ngModel.$validators.date = validator;
        ngModel.$parsers.unshift(parseDate);
        ngModel.$formatters.push(function(value) {
          scope.date = value;
          return ngModel.$isEmpty(value) ? value : dateFilter(value, dateFormat);
        });
      } else {
        ngModel.$formatters.push(function(value) {
          scope.date = value;
          return value;
        });
      }

      // Inner change
      scope.dateSelection = function(dt) {
        if (angular.isDefined(dt)) {
          scope.date = dt;
        }
        var date = scope.date ? dateFilter(scope.date, dateFormat) : null; // Setting to NULL is necessary for form validators to function
        element.val(date);
        ngModel.$setViewValue(date);

        if (closeOnDateSelection) {
          scope.isOpen = false;
          element[0].focus();
        }
      };

      // Detect changes in the view from the text box
      ngModel.$viewChangeListeners.push(function() {
        scope.date = dateParser.parse(ngModel.$viewValue, dateFormat, scope.date);
      });

      var documentClickBind = function(event) {
        if (scope.isOpen && !(element[0].contains(event.target) || popupEl[0].contains(event.target))) {
          scope.$apply(function() {
            scope.isOpen = false;
          });
        }
      };

      var inputKeydownBind = function(evt) {
        if (evt.which === 27 && scope.isOpen) {
          evt.preventDefault();
          evt.stopPropagation();
          scope.$apply(function() {
            scope.isOpen = false;
          });
          element[0].focus();
        } else if (evt.which === 40 && !scope.isOpen) {
          evt.preventDefault();
          evt.stopPropagation();
          scope.$apply(function() {
            scope.isOpen = true;
          });
        }
      };
      element.bind('keydown', inputKeydownBind);

      scope.keydown = function(evt) {
        if (evt.which === 27) {
          scope.isOpen = false;
          element[0].focus();
        }
      };

      scope.$watch('isOpen', function(value) {
        if (value) {
          scope.position = appendToBody ? $position.offset(element) : $position.position(element);
          scope.position.top = scope.position.top + element.prop('offsetHeight');

          $timeout(function() {
            if (onOpenFocus) {
              scope.$broadcast('datepicker.focus');
            }
            $document.bind('click', documentClickBind);
          }, 0, false);
        } else {
          $document.unbind('click', documentClickBind);
        }
      });

      scope.select = function(date) {
        if (date === 'today') {
          var today = new Date();
          if (angular.isDate(scope.date)) {
            date = new Date(scope.date);
            date.setFullYear(today.getFullYear(), today.getMonth(), today.getDate());
          } else {
            date = new Date(today.setHours(0, 0, 0, 0));
          }
        }
        scope.dateSelection(date);
      };

      scope.close = function() {
        scope.isOpen = false;
        element[0].focus();
      };

      var $popup = $compile(popupEl)(scope);
      // Prevent jQuery cache memory leak (template is now redundant after linking)
      popupEl.remove();

      if (appendToBody) {
        $document.find('body').append($popup);
      } else {
        element.after($popup);
      }

      scope.$on('$destroy', function() {
        if (scope.isOpen === true) {
          if (!$rootScope.$$phase) {
            scope.$apply(function() {
              scope.isOpen = false;
            });
          }
        }

        $popup.remove();
        element.unbind('keydown', inputKeydownBind);
        $document.unbind('click', documentClickBind);
      });
    }
  };
}])

.directive('datepickerPopupWrap', function() {
  return {
    restrict:'EA',
    replace: true,
    transclude: true,
    templateUrl: function(element, attrs) {
      return attrs.templateUrl || 'template/datepicker/popup.html';
    }
  };
});

angular.module('ui.bootstrap.dropdown', ['ui.bootstrap.position'])

.constant('dropdownConfig', {
  openClass: 'open'
})

.service('dropdownService', ['$document', '$rootScope', function($document, $rootScope) {
  var openScope = null;

  this.open = function(dropdownScope) {
    if (!openScope) {
      $document.bind('click', closeDropdown);
      $document.bind('keydown', keybindFilter);
    }

    if (openScope && openScope !== dropdownScope) {
      openScope.isOpen = false;
    }

    openScope = dropdownScope;
  };

  this.close = function(dropdownScope) {
    if (openScope === dropdownScope) {
      openScope = null;
      $document.unbind('click', closeDropdown);
      $document.unbind('keydown', keybindFilter);
    }
  };

  var closeDropdown = function(evt) {
    // This method may still be called during the same mouse event that
    // unbound this event handler. So check openScope before proceeding.
    if (!openScope) { return; }

    if (evt && openScope.getAutoClose() === 'disabled')  { return ; }

    var toggleElement = openScope.getToggleElement();
    if (evt && toggleElement && toggleElement[0].contains(evt.target)) {
      return;
    }

    var dropdownElement = openScope.getDropdownElement();
    if (evt && openScope.getAutoClose() === 'outsideClick' &&
      dropdownElement && dropdownElement[0].contains(evt.target)) {
      return;
    }

    openScope.isOpen = false;

    if (!$rootScope.$$phase) {
      openScope.$apply();
    }
  };

  var keybindFilter = function(evt) {
    if (evt.which === 27) {
      openScope.focusToggleElement();
      closeDropdown();
    } else if (openScope.isKeynavEnabled() && /(38|40)/.test(evt.which) && openScope.isOpen) {
      evt.preventDefault();
      evt.stopPropagation();
      openScope.focusDropdownEntry(evt.which);
    }
  };
}])

.controller('DropdownController', ['$scope', '$attrs', '$parse', 'dropdownConfig', 'dropdownService', '$animate', '$position', '$document', '$compile', '$templateRequest', function($scope, $attrs, $parse, dropdownConfig, dropdownService, $animate, $position, $document, $compile, $templateRequest) {
  var self = this,
    scope = $scope.$new(), // create a child scope so we are not polluting original one
    templateScope,
    openClass = dropdownConfig.openClass,
    getIsOpen,
    setIsOpen = angular.noop,
    toggleInvoker = $attrs.onToggle ? $parse($attrs.onToggle) : angular.noop,
    appendToBody = false,
    keynavEnabled = false,
    selectedOption = null,
    body = $document.find('body');

  this.init = function(element) {
    self.$element = element;

    if ($attrs.isOpen) {
      getIsOpen = $parse($attrs.isOpen);
      setIsOpen = getIsOpen.assign;

      $scope.$watch(getIsOpen, function(value) {
        scope.isOpen = !!value;
      });
    }

    appendToBody = angular.isDefined($attrs.dropdownAppendToBody);
    keynavEnabled = angular.isDefined($attrs.keyboardNav);

    if (appendToBody && self.dropdownMenu) {
      body.append(self.dropdownMenu);
      body.addClass('dropdown');
      element.on('$destroy', function handleDestroyEvent() {
        self.dropdownMenu.remove();
      });
    }
  };

  this.toggle = function(open) {
    return scope.isOpen = arguments.length ? !!open : !scope.isOpen;
  };

  // Allow other directives to watch status
  this.isOpen = function() {
    return scope.isOpen;
  };

  scope.getToggleElement = function() {
    return self.toggleElement;
  };

  scope.getAutoClose = function() {
    return $attrs.autoClose || 'always'; //or 'outsideClick' or 'disabled'
  };

  scope.getElement = function() {
    return self.$element;
  };

  scope.isKeynavEnabled = function() {
    return keynavEnabled;
  };

  scope.focusDropdownEntry = function(keyCode) {
    var elems = self.dropdownMenu ? //If append to body is used.
      (angular.element(self.dropdownMenu).find('a')) :
      (angular.element(self.$element).find('ul').eq(0).find('a'));

    switch (keyCode) {
      case (40): {
        if (!angular.isNumber(self.selectedOption)) {
          self.selectedOption = 0;
        } else {
          self.selectedOption = (self.selectedOption === elems.length -1 ?
            self.selectedOption :
            self.selectedOption + 1);
        }
        break;
      }
      case (38): {
        if (!angular.isNumber(self.selectedOption)) {
          self.selectedOption = elems.length - 1;
        } else {
          self.selectedOption = self.selectedOption === 0 ?
            0 : self.selectedOption - 1;
        }
        break;
      }
    }
    elems[self.selectedOption].focus();
  };

  scope.getDropdownElement = function() {
    return self.dropdownMenu;
  };

  scope.focusToggleElement = function() {
    if (self.toggleElement) {
      self.toggleElement[0].focus();
    }
  };

  scope.$watch('isOpen', function(isOpen, wasOpen) {
    if (appendToBody && self.dropdownMenu) {
      var pos = $position.positionElements(self.$element, self.dropdownMenu, 'bottom-left', true);
      var css = {
        top: pos.top + 'px',
        display: isOpen ? 'block' : 'none'
      };

      var rightalign = self.dropdownMenu.hasClass('dropdown-menu-right');
      if (!rightalign) {
        css.left = pos.left + 'px';
        css.right = 'auto';
      } else {
        css.left = 'auto';
        css.right = (window.innerWidth - (pos.left + self.$element.prop('offsetWidth'))) + 'px';
      }

      self.dropdownMenu.css(css);
    }

    var openContainer = appendToBody ? body : self.$element;

    $animate[isOpen ? 'addClass' : 'removeClass'](openContainer, openClass).then(function() {
      if (angular.isDefined(isOpen) && isOpen !== wasOpen) {
        toggleInvoker($scope, { open: !!isOpen });
      }
    });

    if (isOpen) {
      if (self.dropdownMenuTemplateUrl) {
        $templateRequest(self.dropdownMenuTemplateUrl).then(function(tplContent) {
          templateScope = scope.$new();
          $compile(tplContent.trim())(templateScope, function(dropdownElement) {
            var newEl = dropdownElement;
            self.dropdownMenu.replaceWith(newEl);
            self.dropdownMenu = newEl;
          });
        });
      }

      scope.focusToggleElement();
      dropdownService.open(scope);
    } else {
      if (self.dropdownMenuTemplateUrl) {
        if (templateScope) {
          templateScope.$destroy();
        }
        var newEl = angular.element('<ul class="dropdown-menu"></ul>');
        self.dropdownMenu.replaceWith(newEl);
        self.dropdownMenu = newEl;
      }

      dropdownService.close(scope);
      self.selectedOption = null;
    }

    if (angular.isFunction(setIsOpen)) {
      setIsOpen($scope, isOpen);
    }
  });

  $scope.$on('$locationChangeSuccess', function() {
    if (scope.getAutoClose() !== 'disabled') {
      scope.isOpen = false;
    }
  });

  var offDestroy = $scope.$on('$destroy', function() {
    scope.$destroy();
  });
  scope.$on('$destroy', offDestroy);
}])

.directive('dropdown', function() {
  return {
    controller: 'DropdownController',
    link: function(scope, element, attrs, dropdownCtrl) {
      dropdownCtrl.init( element );
      element.addClass('dropdown');
    }
  };
})

.directive('dropdownMenu', function() {
  return {
    restrict: 'AC',
    require: '?^dropdown',
    link: function(scope, element, attrs, dropdownCtrl) {
      if (!dropdownCtrl) {
        return;
      }
      var tplUrl = attrs.templateUrl;
      if (tplUrl) {
        dropdownCtrl.dropdownMenuTemplateUrl = tplUrl;
      }
      if (!dropdownCtrl.dropdownMenu) {
        dropdownCtrl.dropdownMenu = element;
      }
    }
  };
})

.directive('keyboardNav', function() {
  return {
    restrict: 'A',
    require: '?^dropdown',
    link: function (scope, element, attrs, dropdownCtrl) {

      element.bind('keydown', function(e) {
        if ([38, 40].indexOf(e.which) !== -1) {
          e.preventDefault();
          e.stopPropagation();

          var elems = dropdownCtrl.dropdownMenu.find('a');

          switch (e.which) {
            case (40): { // Down
              if (!angular.isNumber(dropdownCtrl.selectedOption)) {
                dropdownCtrl.selectedOption = 0;
              } else {
                dropdownCtrl.selectedOption = dropdownCtrl.selectedOption === elems.length -1 ?
                  dropdownCtrl.selectedOption : dropdownCtrl.selectedOption + 1;
              }
              break;
            }
            case (38): { // Up
              if (!angular.isNumber(dropdownCtrl.selectedOption)) {
                dropdownCtrl.selectedOption = elems.length - 1;
              } else {
                dropdownCtrl.selectedOption = dropdownCtrl.selectedOption === 0 ?
                  0 : dropdownCtrl.selectedOption - 1;
              }
              break;
            }
          }
          elems[dropdownCtrl.selectedOption].focus();
        }
      });
    }
  };
})

.directive('dropdownToggle', function() {
  return {
    require: '?^dropdown',
    link: function(scope, element, attrs, dropdownCtrl) {
      if (!dropdownCtrl) {
        return;
      }

      element.addClass('dropdown-toggle');

      dropdownCtrl.toggleElement = element;

      var toggleDropdown = function(event) {
        event.preventDefault();

        if (!element.hasClass('disabled') && !attrs.disabled) {
          scope.$apply(function() {
            dropdownCtrl.toggle();
          });
        }
      };

      element.bind('click', toggleDropdown);

      // WAI-ARIA
      element.attr({ 'aria-haspopup': true, 'aria-expanded': false });
      scope.$watch(dropdownCtrl.isOpen, function( isOpen ) {
        element.attr('aria-expanded', !!isOpen);
      });

      scope.$on('$destroy', function() {
        element.unbind('click', toggleDropdown);
      });
    }
  };
});

angular.module('ui.bootstrap.modal', [])

/**
 * A helper, internal data structure that acts as a map but also allows getting / removing
 * elements in the LIFO order
 */
  .factory('$$stackedMap', function() {
    return {
      createNew: function() {
        var stack = [];

        return {
          add: function(key, value) {
            stack.push({
              key: key,
              value: value
            });
          },
          get: function(key) {
            for (var i = 0; i < stack.length; i++) {
              if (key == stack[i].key) {
                return stack[i];
              }
            }
          },
          keys: function() {
            var keys = [];
            for (var i = 0; i < stack.length; i++) {
              keys.push(stack[i].key);
            }
            return keys;
          },
          top: function() {
            return stack[stack.length - 1];
          },
          remove: function(key) {
            var idx = -1;
            for (var i = 0; i < stack.length; i++) {
              if (key == stack[i].key) {
                idx = i;
                break;
              }
            }
            return stack.splice(idx, 1)[0];
          },
          removeTop: function() {
            return stack.splice(stack.length - 1, 1)[0];
          },
          length: function() {
            return stack.length;
          }
        };
      }
    };
  })

/**
 * A helper, internal data structure that stores all references attached to key
 */
  .factory('$$multiMap', function() {
    return {
      createNew: function() {
        var map = {};

        return {
          entries: function() {
            return Object.keys(map).map(function(key) {
              return {
                key: key,
                value: map[key]
              };
            });
          },
          get: function(key) {
            return map[key];
          },
          hasKey: function(key) {
            return !!map[key];
          },
          keys: function() {
            return Object.keys(map);
          },
          put: function(key, value) {
            if (!map[key]) {
              map[key] = [];
            }

            map[key].push(value);
          },
          remove: function(key, value) {
            var values = map[key];

            if (!values) {
              return;
            }

            var idx = values.indexOf(value);

            if (idx !== -1) {
              values.splice(idx, 1);
            }

            if (!values.length) {
              delete map[key];
            }
          }
        };
      }
    };
  })

/**
 * A helper directive for the $modal service. It creates a backdrop element.
 */
  .directive('modalBackdrop', [
           '$animate', '$injector', '$modalStack',
  function($animate ,  $injector,   $modalStack) {
    var $animateCss = null;

    if ($injector.has('$animateCss')) {
      $animateCss = $injector.get('$animateCss');
    }

    return {
      restrict: 'EA',
      replace: true,
      templateUrl: 'template/modal/backdrop.html',
      compile: function(tElement, tAttrs) {
        tElement.addClass(tAttrs.backdropClass);
        return linkFn;
      }
    };

    function linkFn(scope, element, attrs) {
      if (attrs.modalInClass) {
        if ($animateCss) {
          $animateCss(element, {
            addClass: attrs.modalInClass
          }).start();
        } else {
          $animate.addClass(element, attrs.modalInClass);
        }

        scope.$on($modalStack.NOW_CLOSING_EVENT, function(e, setIsAsync) {
          var done = setIsAsync();
          if ($animateCss) {
            $animateCss(element, {
              removeClass: attrs.modalInClass
            }).start().then(done);
          } else {
            $animate.removeClass(element, attrs.modalInClass).then(done);
          }
        });
      }
    }
  }])

  .directive('modalWindow', [
           '$modalStack', '$q', '$animate', '$injector',
  function($modalStack ,  $q ,  $animate,   $injector) {
    var $animateCss = null;

    if ($injector.has('$animateCss')) {
      $animateCss = $injector.get('$animateCss');
    }

    return {
      restrict: 'EA',
      scope: {
        index: '@'
      },
      replace: true,
      transclude: true,
      templateUrl: function(tElement, tAttrs) {
        return tAttrs.templateUrl || 'template/modal/window.html';
      },
      link: function(scope, element, attrs) {
        element.addClass(attrs.windowClass || '');
        scope.size = attrs.size;

        scope.close = function(evt) {
          var modal = $modalStack.getTop();
          if (modal && modal.value.backdrop && modal.value.backdrop !== 'static' && (evt.target === evt.currentTarget)) {
            evt.preventDefault();
            evt.stopPropagation();
            $modalStack.dismiss(modal.key, 'backdrop click');
          }
        };

        // This property is only added to the scope for the purpose of detecting when this directive is rendered.
        // We can detect that by using this property in the template associated with this directive and then use
        // {@link Attribute#$observe} on it. For more details please see {@link TableColumnResize}.
        scope.$isRendered = true;

        // Deferred object that will be resolved when this modal is render.
        var modalRenderDeferObj = $q.defer();
        // Observe function will be called on next digest cycle after compilation, ensuring that the DOM is ready.
        // In order to use this way of finding whether DOM is ready, we need to observe a scope property used in modal's template.
        attrs.$observe('modalRender', function(value) {
          if (value == 'true') {
            modalRenderDeferObj.resolve();
          }
        });

        modalRenderDeferObj.promise.then(function() {
          var animationPromise = null;

          if (attrs.modalInClass) {
            if ($animateCss) {
              animationPromise = $animateCss(element, {
                addClass: attrs.modalInClass
              }).start();
            } else {
              animationPromise = $animate.addClass(element, attrs.modalInClass);
            }

            scope.$on($modalStack.NOW_CLOSING_EVENT, function(e, setIsAsync) {
              var done = setIsAsync();
              if ($animateCss) {
                $animateCss(element, {
                  removeClass: attrs.modalInClass
                }).start().then(done);
              } else {
                $animate.removeClass(element, attrs.modalInClass).then(done);
              }
            });
          }


          $q.when(animationPromise).then(function() {
            var inputsWithAutofocus = element[0].querySelectorAll('[autofocus]');
            /**
             * Auto-focusing of a freshly-opened modal element causes any child elements
             * with the autofocus attribute to lose focus. This is an issue on touch
             * based devices which will show and then hide the onscreen keyboard.
             * Attempts to refocus the autofocus element via JavaScript will not reopen
             * the onscreen keyboard. Fixed by updated the focusing logic to only autofocus
             * the modal element if the modal does not contain an autofocus element.
             */
            if (inputsWithAutofocus.length) {
              inputsWithAutofocus[0].focus();
            } else {
              element[0].focus();
            }
          });

          // Notify {@link $modalStack} that modal is rendered.
          var modal = $modalStack.getTop();
          if (modal) {
            $modalStack.modalRendered(modal.key);
          }
        });
      }
    };
  }])

  .directive('modalAnimationClass', [
    function () {
      return {
        compile: function(tElement, tAttrs) {
          if (tAttrs.modalAnimation) {
            tElement.addClass(tAttrs.modalAnimationClass);
          }
        }
      };
    }])

  .directive('modalTransclude', function() {
    return {
      link: function($scope, $element, $attrs, controller, $transclude) {
        $transclude($scope.$parent, function(clone) {
          $element.empty();
          $element.append(clone);
        });
      }
    };
  })

  .factory('$modalStack', [
             '$animate', '$timeout', '$document', '$compile', '$rootScope',
             '$q',
             '$injector',
             '$$multiMap',
             '$$stackedMap',
    function($animate ,  $timeout ,  $document ,  $compile ,  $rootScope ,
              $q,
              $injector,
              $$multiMap,
              $$stackedMap) {
      var $animateCss = null;

      if ($injector.has('$animateCss')) {
        $animateCss = $injector.get('$animateCss');
      }

      var OPENED_MODAL_CLASS = 'modal-open';

      var backdropDomEl, backdropScope;
      var openedWindows = $$stackedMap.createNew();
      var openedClasses = $$multiMap.createNew();
      var $modalStack = {
        NOW_CLOSING_EVENT: 'modal.stack.now-closing'
      };

      //Modal focus behavior
      var focusableElementList;
      var focusIndex = 0;
      var tababbleSelector = 'a[href], area[href], input:not([disabled]), ' +
        'button:not([disabled]),select:not([disabled]), textarea:not([disabled]), ' +
        'iframe, object, embed, *[tabindex], *[contenteditable=true]';

      function backdropIndex() {
        var topBackdropIndex = -1;
        var opened = openedWindows.keys();
        for (var i = 0; i < opened.length; i++) {
          if (openedWindows.get(opened[i]).value.backdrop) {
            topBackdropIndex = i;
          }
        }
        return topBackdropIndex;
      }

      $rootScope.$watch(backdropIndex, function(newBackdropIndex) {
        if (backdropScope) {
          backdropScope.index = newBackdropIndex;
        }
      });

      function removeModalWindow(modalInstance, elementToReceiveFocus) {
        var body = $document.find('body').eq(0);
        var modalWindow = openedWindows.get(modalInstance).value;

        //clean up the stack
        openedWindows.remove(modalInstance);

        removeAfterAnimate(modalWindow.modalDomEl, modalWindow.modalScope, function() {
          var modalBodyClass = modalWindow.openedClass || OPENED_MODAL_CLASS;
          openedClasses.remove(modalBodyClass, modalInstance);
          body.toggleClass(modalBodyClass, openedClasses.hasKey(modalBodyClass));
        });
        checkRemoveBackdrop();

        //move focus to specified element if available, or else to body
        if (elementToReceiveFocus && elementToReceiveFocus.focus) {
          elementToReceiveFocus.focus();
        } else {
          body.focus();
        }
      }

      function checkRemoveBackdrop() {
          //remove backdrop if no longer needed
          if (backdropDomEl && backdropIndex() == -1) {
            var backdropScopeRef = backdropScope;
            removeAfterAnimate(backdropDomEl, backdropScope, function() {
              backdropScopeRef = null;
            });
            backdropDomEl = undefined;
            backdropScope = undefined;
          }
      }

      function removeAfterAnimate(domEl, scope, done) {
        var asyncDeferred;
        var asyncPromise = null;
        var setIsAsync = function() {
          if (!asyncDeferred) {
            asyncDeferred = $q.defer();
            asyncPromise = asyncDeferred.promise;
          }

          return function asyncDone() {
            asyncDeferred.resolve();
          };
        };
        scope.$broadcast($modalStack.NOW_CLOSING_EVENT, setIsAsync);

        // Note that it's intentional that asyncPromise might be null.
        // That's when setIsAsync has not been called during the
        // NOW_CLOSING_EVENT broadcast.
        return $q.when(asyncPromise).then(afterAnimating);

        function afterAnimating() {
          if (afterAnimating.done) {
            return;
          }
          afterAnimating.done = true;

          if ($animateCss) {
            $animateCss(domEl, {
              event: 'leave'
            }).start().then(function() {
              domEl.remove();
            });
          } else {
            $animate.leave(domEl);
          }
          scope.$destroy();
          if (done) {
            done();
          }
        }
      }

      $document.bind('keydown', function(evt) {
        if (evt.isDefaultPrevented()) {
          return evt;
        }

        var modal = openedWindows.top();
        if (modal && modal.value.keyboard) {
          switch (evt.which){
            case 27: {
              evt.preventDefault();
              $rootScope.$apply(function() {
                $modalStack.dismiss(modal.key, 'escape key press');
              });
              break;
            }
            case 9: {
              $modalStack.loadFocusElementList(modal);
              var focusChanged = false;
              if (evt.shiftKey) {
                if ($modalStack.isFocusInFirstItem(evt)) {
                  focusChanged = $modalStack.focusLastFocusableElement();
                }
              } else {
                if ($modalStack.isFocusInLastItem(evt)) {
                  focusChanged = $modalStack.focusFirstFocusableElement();
                }
              }

              if (focusChanged) {
                evt.preventDefault();
                evt.stopPropagation();
              }
              break;
            }
          }
        }
      });

      $modalStack.open = function(modalInstance, modal) {
        var modalOpener = $document[0].activeElement,
          modalBodyClass = modal.openedClass || OPENED_MODAL_CLASS;

        openedWindows.add(modalInstance, {
          deferred: modal.deferred,
          renderDeferred: modal.renderDeferred,
          modalScope: modal.scope,
          backdrop: modal.backdrop,
          keyboard: modal.keyboard,
          openedClass: modal.openedClass
        });

        openedClasses.put(modalBodyClass, modalInstance);

        var body = $document.find('body').eq(0),
            currBackdropIndex = backdropIndex();

        if (currBackdropIndex >= 0 && !backdropDomEl) {
          backdropScope = $rootScope.$new(true);
          backdropScope.index = currBackdropIndex;
          var angularBackgroundDomEl = angular.element('<div modal-backdrop="modal-backdrop"></div>');
          angularBackgroundDomEl.attr('backdrop-class', modal.backdropClass);
          if (modal.animation) {
            angularBackgroundDomEl.attr('modal-animation', 'true');
          }
          backdropDomEl = $compile(angularBackgroundDomEl)(backdropScope);
          body.append(backdropDomEl);
        }

        var angularDomEl = angular.element('<div modal-window="modal-window"></div>');
        angularDomEl.attr({
          'template-url': modal.windowTemplateUrl,
          'window-class': modal.windowClass,
          'size': modal.size,
          'index': openedWindows.length() - 1,
          'animate': 'animate'
        }).html(modal.content);
        if (modal.animation) {
          angularDomEl.attr('modal-animation', 'true');
        }

        var modalDomEl = $compile(angularDomEl)(modal.scope);
        openedWindows.top().value.modalDomEl = modalDomEl;
        openedWindows.top().value.modalOpener = modalOpener;
        body.append(modalDomEl);
        body.addClass(modalBodyClass);

        $modalStack.clearFocusListCache();
      };

      function broadcastClosing(modalWindow, resultOrReason, closing) {
          return !modalWindow.value.modalScope.$broadcast('modal.closing', resultOrReason, closing).defaultPrevented;
      }

      $modalStack.close = function(modalInstance, result) {
        var modalWindow = openedWindows.get(modalInstance);
        if (modalWindow && broadcastClosing(modalWindow, result, true)) {
          modalWindow.value.modalScope.$$uibDestructionScheduled = true;
          modalWindow.value.deferred.resolve(result);
          removeModalWindow(modalInstance, modalWindow.value.modalOpener);
          return true;
        }
        return !modalWindow;
      };

      $modalStack.dismiss = function(modalInstance, reason) {
        var modalWindow = openedWindows.get(modalInstance);
        if (modalWindow && broadcastClosing(modalWindow, reason, false)) {
          modalWindow.value.modalScope.$$uibDestructionScheduled = true;
          modalWindow.value.deferred.reject(reason);
          removeModalWindow(modalInstance, modalWindow.value.modalOpener);
          return true;
        }
        return !modalWindow;
      };

      $modalStack.dismissAll = function(reason) {
        var topModal = this.getTop();
        while (topModal && this.dismiss(topModal.key, reason)) {
          topModal = this.getTop();
        }
      };

      $modalStack.getTop = function() {
        return openedWindows.top();
      };

      $modalStack.modalRendered = function(modalInstance) {
        var modalWindow = openedWindows.get(modalInstance);
        if (modalWindow) {
          modalWindow.value.renderDeferred.resolve();
        }
      };

      $modalStack.focusFirstFocusableElement = function() {
        if (focusableElementList.length > 0) {
          focusableElementList[0].focus();
          return true;
        }
        return false;
      };
      $modalStack.focusLastFocusableElement = function() {
        if (focusableElementList.length > 0) {
          focusableElementList[focusableElementList.length - 1].focus();
          return true;
        }
        return false;
      };

      $modalStack.isFocusInFirstItem = function(evt) {
        if (focusableElementList.length > 0) {
          return (evt.target || evt.srcElement) == focusableElementList[0];
        }
        return false;
      };

      $modalStack.isFocusInLastItem = function(evt) {
        if (focusableElementList.length > 0) {
          return (evt.target || evt.srcElement) == focusableElementList[focusableElementList.length - 1];
        }
        return false;
      };

      $modalStack.clearFocusListCache = function() {
        focusableElementList = [];
        focusIndex = 0;
      };

      $modalStack.loadFocusElementList = function(modalWindow) {
        if (focusableElementList === undefined || !focusableElementList.length0) {
          if (modalWindow) {
            var modalDomE1 = modalWindow.value.modalDomEl;
            if (modalDomE1 && modalDomE1.length) {
              focusableElementList = modalDomE1[0].querySelectorAll(tababbleSelector);
            }
          }
        }
      };

      return $modalStack;
    }])

  .provider('$modal', function() {
    var $modalProvider = {
      options: {
        animation: true,
        backdrop: true, //can also be false or 'static'
        keyboard: true
      },
      $get: ['$injector', '$rootScope', '$q', '$templateRequest', '$controller', '$modalStack',
        function ($injector, $rootScope, $q, $templateRequest, $controller, $modalStack) {
          var $modal = {};

          function getTemplatePromise(options) {
            return options.template ? $q.when(options.template) :
              $templateRequest(angular.isFunction(options.templateUrl) ? (options.templateUrl)() : options.templateUrl);
          }

          function getResolvePromises(resolves) {
            var promisesArr = [];
            angular.forEach(resolves, function(value) {
              if (angular.isFunction(value) || angular.isArray(value)) {
                promisesArr.push($q.when($injector.invoke(value)));
              } else if (angular.isString(value)) {
                promisesArr.push($q.when($injector.get(value)));
              } else {
                promisesArr.push($q.when(value));
              }
            });
            return promisesArr;
          }

          var promiseChain = null;
          $modal.getPromiseChain = function() {
            return promiseChain;
          };

          $modal.open = function (modalOptions) {

            var modalResultDeferred = $q.defer();
            var modalOpenedDeferred = $q.defer();
            var modalRenderDeferred = $q.defer();

            //prepare an instance of a modal to be injected into controllers and returned to a caller
            var modalInstance = {
              result: modalResultDeferred.promise,
              opened: modalOpenedDeferred.promise,
              rendered: modalRenderDeferred.promise,
              close: function (result) {
                return $modalStack.close(modalInstance, result);
              },
              dismiss: function (reason) {
                return $modalStack.dismiss(modalInstance, reason);
              }
            };

            //merge and clean up options
            modalOptions = angular.extend({}, $modalProvider.options, modalOptions);
            modalOptions.resolve = modalOptions.resolve || {};

            //verify options
            if (!modalOptions.template && !modalOptions.templateUrl) {
              throw new Error('One of template or templateUrl options is required.');
            }

            var templateAndResolvePromise =
              $q.all([getTemplatePromise(modalOptions)].concat(getResolvePromises(modalOptions.resolve)));

            // Wait for the resolution of the existing promise chain.
            // Then switch to our own combined promise dependency (regardless of how the previous modal fared).
            // Then add to $modalStack and resolve opened.
            // Finally clean up the chain variable if no subsequent modal has overwritten it.
            var samePromise;
            samePromise = promiseChain = $q.all([promiseChain])
              .then(function() { return templateAndResolvePromise; }, function() { return templateAndResolvePromise; })
              .then(function resolveSuccess(tplAndVars) {

                var modalScope = (modalOptions.scope || $rootScope).$new();
                modalScope.$close = modalInstance.close;
                modalScope.$dismiss = modalInstance.dismiss;

                modalScope.$on('$destroy', function() {
                  if (!modalScope.$$uibDestructionScheduled) {
                    modalScope.$dismiss('$uibUnscheduledDestruction');
                  }
                });

                var ctrlInstance, ctrlLocals = {};
                var resolveIter = 1;

                //controllers
                if (modalOptions.controller) {
                  ctrlLocals.$scope = modalScope;
                  ctrlLocals.$modalInstance = modalInstance;
                  angular.forEach(modalOptions.resolve, function(value, key) {
                    ctrlLocals[key] = tplAndVars[resolveIter++];
                  });

                  ctrlInstance = $controller(modalOptions.controller, ctrlLocals);
                  if (modalOptions.controllerAs) {
                    if (modalOptions.bindToController) {
                      angular.extend(ctrlInstance, modalScope);
                    }

                    modalScope[modalOptions.controllerAs] = ctrlInstance;
                  }
                }

                $modalStack.open(modalInstance, {
                  scope: modalScope,
                  deferred: modalResultDeferred,
                  renderDeferred: modalRenderDeferred,
                  content: tplAndVars[0],
                  animation: modalOptions.animation,
                  backdrop: modalOptions.backdrop,
                  keyboard: modalOptions.keyboard,
                  backdropClass: modalOptions.backdropClass,
                  windowClass: modalOptions.windowClass,
                  windowTemplateUrl: modalOptions.windowTemplateUrl,
                  size: modalOptions.size,
                  openedClass: modalOptions.openedClass
                });
                modalOpenedDeferred.resolve(true);

            }, function resolveError(reason) {
              modalOpenedDeferred.reject(reason);
              modalResultDeferred.reject(reason);
            })
            .finally(function() {
              if (promiseChain === samePromise) {
                promiseChain = null;
              }
            });

            return modalInstance;
          };

          return $modal;
        }]
    };

    return $modalProvider;
  });

angular.module('ui.bootstrap.pagination', [])
.controller('PaginationController', ['$scope', '$attrs', '$parse', function($scope, $attrs, $parse) {
  var self = this,
      ngModelCtrl = { $setViewValue: angular.noop }, // nullModelCtrl
      setNumPages = $attrs.numPages ? $parse($attrs.numPages).assign : angular.noop;

  this.init = function(ngModelCtrl_, config) {
    ngModelCtrl = ngModelCtrl_;
    this.config = config;

    ngModelCtrl.$render = function() {
      self.render();
    };

    if ($attrs.itemsPerPage) {
      $scope.$parent.$watch($parse($attrs.itemsPerPage), function(value) {
        self.itemsPerPage = parseInt(value, 10);
        $scope.totalPages = self.calculateTotalPages();
      });
    } else {
      this.itemsPerPage = config.itemsPerPage;
    }

    $scope.$watch('totalItems', function() {
      $scope.totalPages = self.calculateTotalPages();
    });

    $scope.$watch('totalPages', function(value) {
      setNumPages($scope.$parent, value); // Readonly variable

      if ( $scope.page > value ) {
        $scope.selectPage(value);
      } else {
        ngModelCtrl.$render();
      }
    });
  };

  this.calculateTotalPages = function() {
    var totalPages = this.itemsPerPage < 1 ? 1 : Math.ceil($scope.totalItems / this.itemsPerPage);
    return Math.max(totalPages || 0, 1);
  };

  this.render = function() {
    $scope.page = parseInt(ngModelCtrl.$viewValue, 10) || 1;
  };

  $scope.selectPage = function(page, evt) {
    if (evt) {
      evt.preventDefault();
    }

    var clickAllowed = !$scope.ngDisabled || !evt;
    if (clickAllowed && $scope.page !== page && page > 0 && page <= $scope.totalPages) {
      if (evt && evt.target) {
        evt.target.blur();
      }
      ngModelCtrl.$setViewValue(page);
      ngModelCtrl.$render();
    }
  };

  $scope.getText = function(key) {
    return $scope[key + 'Text'] || self.config[key + 'Text'];
  };

  $scope.noPrevious = function() {
    return $scope.page === 1;
  };

  $scope.noNext = function() {
    return $scope.page === $scope.totalPages;
  };
}])

.constant('paginationConfig', {
  itemsPerPage: 10,
  boundaryLinks: false,
  directionLinks: true,
  firstText: 'First',
  previousText: 'Previous',
  nextText: 'Next',
  lastText: 'Last',
  rotate: true
})

.directive('pagination', ['$parse', 'paginationConfig', function($parse, paginationConfig) {
  return {
    restrict: 'EA',
    scope: {
      totalItems: '=',
      firstText: '@',
      previousText: '@',
      nextText: '@',
      lastText: '@',
      ngDisabled:'='
    },
    require: ['pagination', '?ngModel'],
    controller: 'PaginationController',
    controllerAs: 'pagination',
    templateUrl: function(element, attrs) {
      return attrs.templateUrl || 'template/pagination/pagination.html';
    },
    replace: true,
    link: function(scope, element, attrs, ctrls) {
      var paginationCtrl = ctrls[0], ngModelCtrl = ctrls[1];

      if (!ngModelCtrl) {
         return; // do nothing if no ng-model
      }

      // Setup configuration parameters
      var maxSize = angular.isDefined(attrs.maxSize) ? scope.$parent.$eval(attrs.maxSize) : paginationConfig.maxSize,
          rotate = angular.isDefined(attrs.rotate) ? scope.$parent.$eval(attrs.rotate) : paginationConfig.rotate;
      scope.boundaryLinks = angular.isDefined(attrs.boundaryLinks) ? scope.$parent.$eval(attrs.boundaryLinks) : paginationConfig.boundaryLinks;
      scope.directionLinks = angular.isDefined(attrs.directionLinks) ? scope.$parent.$eval(attrs.directionLinks) : paginationConfig.directionLinks;

      paginationCtrl.init(ngModelCtrl, paginationConfig);

      if (attrs.maxSize) {
        scope.$parent.$watch($parse(attrs.maxSize), function(value) {
          maxSize = parseInt(value, 10);
          paginationCtrl.render();
        });
      }

      // Create page object used in template
      function makePage(number, text, isActive) {
        return {
          number: number,
          text: text,
          active: isActive
        };
      }

      function getPages(currentPage, totalPages) {
        var pages = [];

        // Default page limits
        var startPage = 1, endPage = totalPages;
        var isMaxSized = angular.isDefined(maxSize) && maxSize < totalPages;

        // recompute if maxSize
        if (isMaxSized) {
          if (rotate) {
            // Current page is displayed in the middle of the visible ones
            startPage = Math.max(currentPage - Math.floor(maxSize/2), 1);
            endPage   = startPage + maxSize - 1;

            // Adjust if limit is exceeded
            if (endPage > totalPages) {
              endPage   = totalPages;
              startPage = endPage - maxSize + 1;
            }
          } else {
            // Visible pages are paginated with maxSize
            startPage = ((Math.ceil(currentPage / maxSize) - 1) * maxSize) + 1;

            // Adjust last page if limit is exceeded
            endPage = Math.min(startPage + maxSize - 1, totalPages);
          }
        }

        // Add page number links
        for (var number = startPage; number <= endPage; number++) {
          var page = makePage(number, number, number === currentPage);
          pages.push(page);
        }

        // Add links to move between page sets
        if (isMaxSized && ! rotate) {
          if (startPage > 1) {
            var previousPageSet = makePage(startPage - 1, '...', false);
            pages.unshift(previousPageSet);
          }

          if (endPage < totalPages) {
            var nextPageSet = makePage(endPage + 1, '...', false);
            pages.push(nextPageSet);
          }
        }

        return pages;
      }

      var originalRender = paginationCtrl.render;
      paginationCtrl.render = function() {
        originalRender();
        if (scope.page > 0 && scope.page <= scope.totalPages) {
          scope.pages = getPages(scope.page, scope.totalPages);
        }
      };
    }
  };
}])

.constant('pagerConfig', {
  itemsPerPage: 10,
  previousText: '« Previous',
  nextText: 'Next »',
  align: true
})

.directive('pager', ['pagerConfig', function(pagerConfig) {
  return {
    restrict: 'EA',
    scope: {
      totalItems: '=',
      previousText: '@',
      nextText: '@',
      ngDisabled: '='
    },
    require: ['pager', '?ngModel'],
    controller: 'PaginationController',
    controllerAs: 'pagination',
    templateUrl: function(element, attrs) {
      return attrs.templateUrl || 'template/pagination/pager.html';
    },
    replace: true,
    link: function(scope, element, attrs, ctrls) {
      var paginationCtrl = ctrls[0], ngModelCtrl = ctrls[1];

      if (!ngModelCtrl) {
         return; // do nothing if no ng-model
      }

      scope.align = angular.isDefined(attrs.align) ? scope.$parent.$eval(attrs.align) : pagerConfig.align;
      paginationCtrl.init(ngModelCtrl, pagerConfig);
    }
  };
}]);

/**
 * The following features are still outstanding: animation as a
 * function, placement as a function, inside, support for more triggers than
 * just mouse enter/leave, html tooltips, and selector delegation.
 */
angular.module('ui.bootstrap.tooltip', ['ui.bootstrap.position', 'ui.bootstrap.bindHtml'])

/**
 * The $tooltip service creates tooltip- and popover-like directives as well as
 * houses global options for them.
 */
.provider('$tooltip', function() {
  // The default options tooltip and popover.
  var defaultOptions = {
    placement: 'top',
    animation: true,
    popupDelay: 0,
    useContentExp: false
  };

  // Default hide triggers for each show trigger
  var triggerMap = {
    'mouseenter': 'mouseleave',
    'click': 'click',
    'focus': 'blur',
    'none': ''
  };

  // The options specified to the provider globally.
  var globalOptions = {};

  /**
   * `options({})` allows global configuration of all tooltips in the
   * application.
   *
   *   var app = angular.module( 'App', ['ui.bootstrap.tooltip'], function( $tooltipProvider ) {
   *     // place tooltips left instead of top by default
   *     $tooltipProvider.options( { placement: 'left' } );
   *   });
   */
	this.options = function(value) {
		angular.extend(globalOptions, value);
	};

  /**
   * This allows you to extend the set of trigger mappings available. E.g.:
   *
   *   $tooltipProvider.setTriggers( 'openTrigger': 'closeTrigger' );
   */
  this.setTriggers = function setTriggers(triggers) {
    angular.extend(triggerMap, triggers);
  };

  /**
   * This is a helper function for translating camel-case to snake-case.
   */
  function snake_case(name) {
    var regexp = /[A-Z]/g;
    var separator = '-';
    return name.replace(regexp, function(letter, pos) {
      return (pos ? separator : '') + letter.toLowerCase();
    });
  }

  /**
   * Returns the actual instance of the $tooltip service.
   * TODO support multiple triggers
   */
  this.$get = ['$window', '$compile', '$timeout', '$document', '$position', '$interpolate', '$rootScope', '$parse', function($window, $compile, $timeout, $document, $position, $interpolate, $rootScope, $parse) {
    return function $tooltip(type, prefix, defaultTriggerShow, options) {
      options = angular.extend({}, defaultOptions, globalOptions, options);

      /**
       * Returns an object of show and hide triggers.
       *
       * If a trigger is supplied,
       * it is used to show the tooltip; otherwise, it will use the `trigger`
       * option passed to the `$tooltipProvider.options` method; else it will
       * default to the trigger supplied to this directive factory.
       *
       * The hide trigger is based on the show trigger. If the `trigger` option
       * was passed to the `$tooltipProvider.options` method, it will use the
       * mapped trigger from `triggerMap` or the passed trigger if the map is
       * undefined; otherwise, it uses the `triggerMap` value of the show
       * trigger; else it will just use the show trigger.
       */
      function getTriggers(trigger) {
        var show = (trigger || options.trigger || defaultTriggerShow).split(' ');
        var hide = show.map(function(trigger) {
          return triggerMap[trigger] || trigger;
        });
        return {
          show: show,
          hide: hide
        };
      }

      var directiveName = snake_case(type);

      var startSym = $interpolate.startSymbol();
      var endSym = $interpolate.endSymbol();
      var template =
        '<div '+ directiveName +'-popup '+
          'title="'+startSym+'title'+endSym+'" '+
          (options.useContentExp ?
            'content-exp="contentExp()" ' :
            'content="'+startSym+'content'+endSym+'" ') +
          'placement="'+startSym+'placement'+endSym+'" '+
          'popup-class="'+startSym+'popupClass'+endSym+'" '+
          'animation="animation" '+
          'is-open="isOpen"'+
          'origin-scope="origScope" '+
          '>'+
        '</div>';

      return {
        restrict: 'EA',
        compile: function(tElem, tAttrs) {
          var tooltipLinker = $compile( template );

          return function link(scope, element, attrs, tooltipCtrl) {
            var tooltip;
            var tooltipLinkedScope;
            var transitionTimeout;
            var popupTimeout;
            var positionTimeout;
            var appendToBody = angular.isDefined(options.appendToBody) ? options.appendToBody : false;
            var triggers = getTriggers(undefined);
            var hasEnableExp = angular.isDefined(attrs[prefix + 'Enable']);
            var ttScope = scope.$new(true);
            var repositionScheduled = false;
            var isOpenExp = angular.isDefined(attrs[prefix + 'IsOpen']) ? $parse(attrs[prefix + 'IsOpen']) : false;

            var positionTooltip = function() {
              if (!tooltip) { return; }

              if (!positionTimeout) {
                positionTimeout = $timeout(function() {
                  // Reset the positioning and box size for correct width and height values.
                  tooltip.css({ top: 0, left: 0, width: 'auto', height: 'auto' });

                  var ttBox = $position.position(tooltip);
                  var ttCss = $position.positionElements(element, tooltip, ttScope.placement, appendToBody);
                  ttCss.top += 'px';
                  ttCss.left += 'px';

                  ttCss.width = ttBox.width + 'px';
                  ttCss.height = ttBox.height + 'px';

                  // Now set the calculated positioning and size.
                  tooltip.css(ttCss);

                  positionTimeout = null;

                }, 0, false);
              }
            };

            // Set up the correct scope to allow transclusion later
            ttScope.origScope = scope;

            // By default, the tooltip is not open.
            // TODO add ability to start tooltip opened
            ttScope.isOpen = false;

            function toggleTooltipBind() {
              if (!ttScope.isOpen) {
                showTooltipBind();
              } else {
                hideTooltipBind();
              }
            }

            // Show the tooltip with delay if specified, otherwise show it immediately
            function showTooltipBind() {
              if (hasEnableExp && !scope.$eval(attrs[prefix + 'Enable'])) {
                return;
              }

              prepareTooltip();

              if (ttScope.popupDelay) {
                // Do nothing if the tooltip was already scheduled to pop-up.
                // This happens if show is triggered multiple times before any hide is triggered.
                if (!popupTimeout) {
                  popupTimeout = $timeout(show, ttScope.popupDelay, false);
                }
              } else {
                show();
              }
            }

            function hideTooltipBind () {
              hide();
              if (!$rootScope.$$phase) {
                $rootScope.$digest();
              }
            }

            // Show the tooltip popup element.
            function show() {
              popupTimeout = null;

              // If there is a pending remove transition, we must cancel it, lest the
              // tooltip be mysteriously removed.
              if (transitionTimeout) {
                $timeout.cancel(transitionTimeout);
                transitionTimeout = null;
              }

              // Don't show empty tooltips.
              if (!(options.useContentExp ? ttScope.contentExp() : ttScope.content)) {
                return angular.noop;
              }

              createTooltip();

              // And show the tooltip.
              ttScope.isOpen = true;
              if (isOpenExp) {
                isOpenExp.assign(ttScope.origScope, ttScope.isOpen);
              }

              if (!$rootScope.$$phase) {
                ttScope.$apply(); // digest required as $apply is not called
              }

              tooltip.css({ display: 'block' });

              positionTooltip();
            }

            // Hide the tooltip popup element.
            function hide() {
              // First things first: we don't show it anymore.
              ttScope.isOpen = false;
              if (isOpenExp) {
                isOpenExp.assign(ttScope.origScope, ttScope.isOpen);
              }

              //if tooltip is going to be shown after delay, we must cancel this
              $timeout.cancel(popupTimeout);
              popupTimeout = null;

              $timeout.cancel(positionTimeout);
              positionTimeout = null;

              // And now we remove it from the DOM. However, if we have animation, we
              // need to wait for it to expire beforehand.
              // FIXME: this is a placeholder for a port of the transitions library.
              if (ttScope.animation) {
                if (!transitionTimeout) {
                  transitionTimeout = $timeout(removeTooltip, 500);
                }
              } else {
                removeTooltip();
              }
            }

            function createTooltip() {
              // There can only be one tooltip element per directive shown at once.
              if (tooltip) {
                removeTooltip();
              }
              tooltipLinkedScope = ttScope.$new();
              tooltip = tooltipLinker(tooltipLinkedScope, function(tooltip) {
                if (appendToBody) {
                  $document.find('body').append(tooltip);
                } else {
                  element.after(tooltip);
                }
              });

              if (options.useContentExp) {
                tooltipLinkedScope.$watch('contentExp()', function(val) {
                  if (!val && ttScope.isOpen) {
                    hide();
                  }
                });

                tooltipLinkedScope.$watch(function() {
                  if (!repositionScheduled) {
                    repositionScheduled = true;
                    tooltipLinkedScope.$$postDigest(function() {
                      repositionScheduled = false;
                      if (ttScope.isOpen) {
                        positionTooltip();
                      }
                    });
                  }
                });

              }
            }

            function removeTooltip() {
              transitionTimeout = null;
              if (tooltip) {
                tooltip.remove();
                tooltip = null;
              }
              if (tooltipLinkedScope) {
                tooltipLinkedScope.$destroy();
                tooltipLinkedScope = null;
              }
            }

            function prepareTooltip() {
              prepPopupClass();
              prepPlacement();
              prepPopupDelay();
            }

            ttScope.contentExp = function() {
              return scope.$eval(attrs[type]);
            };

            /**
             * Observe the relevant attributes.
             */
            if (!options.useContentExp) {
              attrs.$observe(type, function(val) {
                ttScope.content = val;

                if (!val && ttScope.isOpen) {
                  hide();
                } else {
                  positionTooltip();
                }
              });
            }

            attrs.$observe('disabled', function(val) {
              if (popupTimeout && val) {
                $timeout.cancel(popupTimeout);
                popupTimeout = null;
              }

              if (val && ttScope.isOpen) {
                hide();
              }
            });

            attrs.$observe(prefix + 'Title', function(val) {
              ttScope.title = val;
              positionTooltip();
            });

            attrs.$observe(prefix + 'Placement', function() {
              if (ttScope.isOpen) {
                prepPlacement();
                positionTooltip();
              }
            });

            if (isOpenExp) {
              scope.$watch(isOpenExp, function(val) {
                if (val !== ttScope.isOpen) {
                  toggleTooltipBind();
                }
              });
            }

            function prepPopupClass() {
              ttScope.popupClass = attrs[prefix + 'Class'];
            }

            function prepPlacement() {
              var val = attrs[prefix + 'Placement'];
              ttScope.placement = angular.isDefined(val) ? val : options.placement;
            }

            function prepPopupDelay() {
              var val = attrs[prefix + 'PopupDelay'];
              var delay = parseInt(val, 10);
              ttScope.popupDelay = !isNaN(delay) ? delay : options.popupDelay;
            }

            var unregisterTriggers = function() {
              triggers.show.forEach(function(trigger) {
                element.unbind(trigger, showTooltipBind);
              });
              triggers.hide.forEach(function(trigger) {
                element.unbind(trigger, hideTooltipBind);
              });
            };

            function prepTriggers() {
              var val = attrs[prefix + 'Trigger'];
              unregisterTriggers();

              triggers = getTriggers(val);

              if (triggers.show !== 'none') {
                triggers.show.forEach(function(trigger, idx) {
                  // Using raw addEventListener due to jqLite/jQuery bug - #4060
                  if (trigger === triggers.hide[idx]) {
                    element[0].addEventListener(trigger, toggleTooltipBind);
                  } else if (trigger) {
                    element[0].addEventListener(trigger, showTooltipBind);
                    element[0].addEventListener(triggers.hide[idx], hideTooltipBind);
                  }
                });
              }
            }
            prepTriggers();

            var animation = scope.$eval(attrs[prefix + 'Animation']);
            ttScope.animation = angular.isDefined(animation) ? !!animation : options.animation;

            var appendToBodyVal = scope.$eval(attrs[prefix + 'AppendToBody']);
            appendToBody = angular.isDefined(appendToBodyVal) ? appendToBodyVal : appendToBody;

            // if a tooltip is attached to <body> we need to remove it on
            // location change as its parent scope will probably not be destroyed
            // by the change.
            if (appendToBody) {
              scope.$on('$locationChangeSuccess', function closeTooltipOnLocationChangeSuccess() {
                if (ttScope.isOpen) {
                  hide();
                }
              });
            }

            // Make sure tooltip is destroyed and removed.
            scope.$on('$destroy', function onDestroyTooltip() {
              $timeout.cancel(transitionTimeout);
              $timeout.cancel(popupTimeout);
              $timeout.cancel(positionTimeout);
              unregisterTriggers();
              removeTooltip();
              ttScope = null;
            });
          };
        }
      };
    };
  }];
})

// This is mostly ngInclude code but with a custom scope
.directive('tooltipTemplateTransclude', [
         '$animate', '$sce', '$compile', '$templateRequest',
function ($animate ,  $sce ,  $compile ,  $templateRequest) {
  return {
    link: function(scope, elem, attrs) {
      var origScope = scope.$eval(attrs.tooltipTemplateTranscludeScope);

      var changeCounter = 0,
        currentScope,
        previousElement,
        currentElement;

      var cleanupLastIncludeContent = function() {
        if (previousElement) {
          previousElement.remove();
          previousElement = null;
        }
        if (currentScope) {
          currentScope.$destroy();
          currentScope = null;
        }
        if (currentElement) {
          $animate.leave(currentElement).then(function() {
            previousElement = null;
          });
          previousElement = currentElement;
          currentElement = null;
        }
      };

      scope.$watch($sce.parseAsResourceUrl(attrs.tooltipTemplateTransclude), function(src) {
        var thisChangeId = ++changeCounter;

        if (src) {
          //set the 2nd param to true to ignore the template request error so that the inner
          //contents and scope can be cleaned up.
          $templateRequest(src, true).then(function(response) {
            if (thisChangeId !== changeCounter) { return; }
            var newScope = origScope.$new();
            var template = response;

            var clone = $compile(template)(newScope, function(clone) {
              cleanupLastIncludeContent();
              $animate.enter(clone, elem);
            });

            currentScope = newScope;
            currentElement = clone;

            currentScope.$emit('$includeContentLoaded', src);
          }, function() {
            if (thisChangeId === changeCounter) {
              cleanupLastIncludeContent();
              scope.$emit('$includeContentError', src);
            }
          });
          scope.$emit('$includeContentRequested', src);
        } else {
          cleanupLastIncludeContent();
        }
      });

      scope.$on('$destroy', cleanupLastIncludeContent);
    }
  };
}])

/**
 * Note that it's intentional that these classes are *not* applied through $animate.
 * They must not be animated as they're expected to be present on the tooltip on
 * initialization.
 */
.directive('tooltipClasses', function() {
  return {
    restrict: 'A',
    link: function(scope, element, attrs) {
      if (scope.placement) {
        element.addClass(scope.placement);
      }
      if (scope.popupClass) {
        element.addClass(scope.popupClass);
      }
      if (scope.animation()) {
        element.addClass(attrs.tooltipAnimationClass);
      }
    }
  };
})

.directive('tooltipPopup', function() {
  return {
    restrict: 'EA',
    replace: true,
    scope: { content: '@', placement: '@', popupClass: '@', animation: '&', isOpen: '&' },
    templateUrl: 'template/tooltip/tooltip-popup.html'
  };
})

.directive('tooltip', [ '$tooltip', function($tooltip) {
  return $tooltip('tooltip', 'tooltip', 'mouseenter');
}])

.directive('tooltipTemplatePopup', function() {
  return {
    restrict: 'EA',
    replace: true,
    scope: { contentExp: '&', placement: '@', popupClass: '@', animation: '&', isOpen: '&',
      originScope: '&' },
    templateUrl: 'template/tooltip/tooltip-template-popup.html'
  };
})

.directive('tooltipTemplate', ['$tooltip', function($tooltip) {
  return $tooltip('tooltipTemplate', 'tooltip', 'mouseenter', {
    useContentExp: true
  });
}])

.directive('tooltipHtmlPopup', function() {
  return {
    restrict: 'EA',
    replace: true,
    scope: { contentExp: '&', placement: '@', popupClass: '@', animation: '&', isOpen: '&' },
    templateUrl: 'template/tooltip/tooltip-html-popup.html'
  };
})

.directive('tooltipHtml', ['$tooltip', function($tooltip) {
  return $tooltip('tooltipHtml', 'tooltip', 'mouseenter', {
    useContentExp: true
  });
}])

/*
Deprecated
*/
.directive('tooltipHtmlUnsafePopup', function() {
  return {
    restrict: 'EA',
    replace: true,
    scope: { content: '@', placement: '@', popupClass: '@', animation: '&', isOpen: '&' },
    templateUrl: 'template/tooltip/tooltip-html-unsafe-popup.html'
  };
})

.value('tooltipHtmlUnsafeSuppressDeprecated', false)
.directive('tooltipHtmlUnsafe', [
          '$tooltip', 'tooltipHtmlUnsafeSuppressDeprecated', '$log',
function($tooltip ,  tooltipHtmlUnsafeSuppressDeprecated ,  $log) {
  if (!tooltipHtmlUnsafeSuppressDeprecated) {
    $log.warn('tooltip-html-unsafe is now deprecated. Use tooltip-html or tooltip-template instead.');
  }
  return $tooltip('tooltipHtmlUnsafe', 'tooltip', 'mouseenter');
}]);

/**
 * The following features are still outstanding: popup delay, animation as a
 * function, placement as a function, inside, support for more triggers than
 * just mouse enter/leave, and selector delegatation.
 */
angular.module( 'ui.bootstrap.popover', ['ui.bootstrap.tooltip'])

.directive('popoverTemplatePopup', function() {
  return {
    restrict: 'EA',
    replace: true,
    scope: { title: '@', contentExp: '&', placement: '@', popupClass: '@', animation: '&', isOpen: '&',
      originScope: '&' },
    templateUrl: 'template/popover/popover-template.html'
  };
})

.directive('popoverTemplate', ['$tooltip', function($tooltip) {
  return $tooltip('popoverTemplate', 'popover', 'click', {
    useContentExp: true
  });
}])

.directive('popoverHtmlPopup', function() {
  return {
    restrict: 'EA',
    replace: true,
    scope: { contentExp: '&', title: '@', placement: '@', popupClass: '@', animation: '&', isOpen: '&' },
    templateUrl: 'template/popover/popover-html.html'
  };
})

.directive('popoverHtml', ['$tooltip', function($tooltip) {
  return $tooltip( 'popoverHtml', 'popover', 'click', {
    useContentExp: true
  });
}])

.directive('popoverPopup', function() {
  return {
    restrict: 'EA',
    replace: true,
    scope: { title: '@', content: '@', placement: '@', popupClass: '@', animation: '&', isOpen: '&' },
    templateUrl: 'template/popover/popover.html'
  };
})

.directive('popover', ['$tooltip', function($tooltip) {
  return $tooltip( 'popover', 'popover', 'click' );
}]);

angular.module('ui.bootstrap.progressbar', [])

.constant('progressConfig', {
  animate: true,
  max: 100
})

.value('$progressSuppressWarning', false)

.controller('ProgressController', ['$scope', '$attrs', 'progressConfig', function($scope, $attrs, progressConfig) {
  var self = this,
      animate = angular.isDefined($attrs.animate) ? $scope.$parent.$eval($attrs.animate) : progressConfig.animate;

  this.bars = [];
  $scope.max = angular.isDefined($scope.max) ? $scope.max : progressConfig.max;

  this.addBar = function(bar, element) {
    if (!animate) {
      element.css({'transition': 'none'});
    }

    this.bars.push(bar);

    bar.max = $scope.max;

    bar.$watch('value', function(value) {
      bar.recalculatePercentage();
    });

    bar.recalculatePercentage = function() {
      bar.percent = +(100 * bar.value / bar.max).toFixed(2);

      var totalPercentage = self.bars.reduce(function(total, bar) {
        return total + bar.percent;
      }, 0);

      if (totalPercentage > 100) {
        bar.percent -= totalPercentage - 100;
      }
    };

    bar.$on('$destroy', function() {
      element = null;
      self.removeBar(bar);
    });
  };

  this.removeBar = function(bar) {
      this.bars.splice(this.bars.indexOf(bar), 1);
  };

  $scope.$watch('max', function(max) {
    self.bars.forEach(function(bar) {
      bar.max = $scope.max;
      bar.recalculatePercentage();
    });
  });
}])

.directive('uibProgress', function() {
  return {
    restrict: 'EA',
    replace: true,
    transclude: true,
    controller: 'ProgressController',
    require: 'uibProgress',
    scope: {
      max: '=?'
    },
    templateUrl: 'template/progressbar/progress.html'
  };
})

.directive('progress', ['$log', '$progressSuppressWarning', function($log, $progressSuppressWarning) {
  return {
    restrict: 'EA',
    replace: true,
    transclude: true,
    controller: 'ProgressController',
    require: 'progress',
    scope: {
      max: '=?'
    },
    templateUrl: 'template/progressbar/progress.html',
    link: function() {
      if ($progressSuppressWarning) {
        $log.warn('progress is now deprecated. Use uib-progress instead');
      }
    }
  };
}])

.directive('uibBar', function() {
  return {
    restrict: 'EA',
    replace: true,
    transclude: true,
    require: '^uibProgress',
    scope: {
      value: '=',
      type: '@'
    },
    templateUrl: 'template/progressbar/bar.html',
    link: function(scope, element, attrs, progressCtrl) {
      progressCtrl.addBar(scope, element);
    }
  };
})

.directive('bar', ['$log', '$progressSuppressWarning', function($log, $progressSuppressWarning) {
  return {
    restrict: 'EA',
    replace: true,
    transclude: true,
    require: '^progress',
    scope: {
      value: '=',
      type: '@'
    },
    templateUrl: 'template/progressbar/bar.html',
    link: function(scope, element, attrs, progressCtrl) {
      if ($progressSuppressWarning) {
        $log.warn('bar is now deprecated. Use uib-bar instead');
      }
      progressCtrl.addBar(scope, element);
    }
  };
}])

.directive('progressbar', function() {
  return {
    restrict: 'EA',
    replace: true,
    transclude: true,
    controller: 'ProgressController',
    scope: {
      value: '=',
      max: '=?',
      type: '@'
    },
    templateUrl: 'template/progressbar/progressbar.html',
    link: function(scope, element, attrs, progressCtrl) {
      progressCtrl.addBar(scope, angular.element(element.children()[0]));
    }
  };
});

angular.module('ui.bootstrap.rating', [])

.constant('ratingConfig', {
  max: 5,
  stateOn: null,
  stateOff: null,
  titles : ['one', 'two', 'three', 'four', 'five']
})

.controller('RatingController', ['$scope', '$attrs', 'ratingConfig', function($scope, $attrs, ratingConfig) {
  var ngModelCtrl  = { $setViewValue: angular.noop };

  this.init = function(ngModelCtrl_) {
    ngModelCtrl = ngModelCtrl_;
    ngModelCtrl.$render = this.render;

    ngModelCtrl.$formatters.push(function(value) {
      if (angular.isNumber(value) && value << 0 !== value) {
        value = Math.round(value);
      }
      return value;
    });

    this.stateOn = angular.isDefined($attrs.stateOn) ? $scope.$parent.$eval($attrs.stateOn) : ratingConfig.stateOn;
    this.stateOff = angular.isDefined($attrs.stateOff) ? $scope.$parent.$eval($attrs.stateOff) : ratingConfig.stateOff;
    var tmpTitles = angular.isDefined($attrs.titles)  ? $scope.$parent.$eval($attrs.titles) : ratingConfig.titles ;    
    this.titles = angular.isArray(tmpTitles) && tmpTitles.length > 0 ?
      tmpTitles : ratingConfig.titles;
    
    var ratingStates = angular.isDefined($attrs.ratingStates) ?
      $scope.$parent.$eval($attrs.ratingStates) :
      new Array(angular.isDefined($attrs.max) ? $scope.$parent.$eval($attrs.max) : ratingConfig.max);
    $scope.range = this.buildTemplateObjects(ratingStates);
  };

  this.buildTemplateObjects = function(states) {
    for (var i = 0, n = states.length; i < n; i++) {
      states[i] = angular.extend({ index: i }, { stateOn: this.stateOn, stateOff: this.stateOff, title: this.getTitle(i) }, states[i]);
    }
    return states;
  };
  
  this.getTitle = function(index) {
    if (index >= this.titles.length) {
      return index + 1;
    } else {
      return this.titles[index];
    }
  };
  
  $scope.rate = function(value) {
    if (!$scope.readonly && value >= 0 && value <= $scope.range.length) {
      ngModelCtrl.$setViewValue(ngModelCtrl.$viewValue === value ? 0 : value);
      ngModelCtrl.$render();
    }
  };

  $scope.enter = function(value) {
    if (!$scope.readonly) {
      $scope.value = value;
    }
    $scope.onHover({value: value});
  };

  $scope.reset = function() {
    $scope.value = ngModelCtrl.$viewValue;
    $scope.onLeave();
  };

  $scope.onKeydown = function(evt) {
    if (/(37|38|39|40)/.test(evt.which)) {
      evt.preventDefault();
      evt.stopPropagation();
      $scope.rate($scope.value + (evt.which === 38 || evt.which === 39 ? 1 : -1));
    }
  };

  this.render = function() {
    $scope.value = ngModelCtrl.$viewValue;
  };
}])

.directive('rating', function() {
  return {
    restrict: 'EA',
    require: ['rating', 'ngModel'],
    scope: {
      readonly: '=?',
      onHover: '&',
      onLeave: '&'
    },
    controller: 'RatingController',
    templateUrl: 'template/rating/rating.html',
    replace: true,
    link: function(scope, element, attrs, ctrls) {
      var ratingCtrl = ctrls[0], ngModelCtrl = ctrls[1];
      ratingCtrl.init( ngModelCtrl );
    }
  };
});


/**
 * @ngdoc overview
 * @name ui.bootstrap.tabs
 *
 * @description
 * AngularJS version of the tabs directive.
 */

angular.module('ui.bootstrap.tabs', [])

.controller('TabsetController', ['$scope', function TabsetCtrl($scope) {
  var ctrl = this,
      tabs = ctrl.tabs = $scope.tabs = [];

  ctrl.select = function(selectedTab) {
    angular.forEach(tabs, function(tab) {
      if (tab.active && tab !== selectedTab) {
        tab.active = false;
        tab.onDeselect();
        selectedTab.selectCalled = false;
      }
    });
    selectedTab.active = true;
    // only call select if it has not already been called
    if (!selectedTab.selectCalled) {
      selectedTab.onSelect();
      selectedTab.selectCalled = true;
    }
  };

  ctrl.addTab = function addTab(tab) {
    tabs.push(tab);
    // we can't run the select function on the first tab
    // since that would select it twice
    if (tabs.length === 1 && tab.active !== false) {
      tab.active = true;
    } else if (tab.active) {
      ctrl.select(tab);
    } else {
      tab.active = false;
    }
  };

  ctrl.removeTab = function removeTab(tab) {
    var index = tabs.indexOf(tab);
    //Select a new tab if the tab to be removed is selected and not destroyed
    if (tab.active && tabs.length > 1 && !destroyed) {
      //If this is the last tab, select the previous tab. else, the next tab.
      var newActiveIndex = index == tabs.length - 1 ? index - 1 : index + 1;
      ctrl.select(tabs[newActiveIndex]);
    }
    tabs.splice(index, 1);
  };

  var destroyed;
  $scope.$on('$destroy', function() {
    destroyed = true;
  });
}])

/**
 * @ngdoc directive
 * @name ui.bootstrap.tabs.directive:tabset
 * @restrict EA
 *
 * @description
 * Tabset is the outer container for the tabs directive
 *
 * @param {boolean=} vertical Whether or not to use vertical styling for the tabs.
 * @param {boolean=} justified Whether or not to use justified styling for the tabs.
 *
 * @example
<example module="ui.bootstrap">
  <file name="index.html">
    <tabset>
      <tab heading="Tab 1"><b>First</b> Content!</tab>
      <tab heading="Tab 2"><i>Second</i> Content!</tab>
    </tabset>
    <hr />
    <tabset vertical="true">
      <tab heading="Vertical Tab 1"><b>First</b> Vertical Content!</tab>
      <tab heading="Vertical Tab 2"><i>Second</i> Vertical Content!</tab>
    </tabset>
    <tabset justified="true">
      <tab heading="Justified Tab 1"><b>First</b> Justified Content!</tab>
      <tab heading="Justified Tab 2"><i>Second</i> Justified Content!</tab>
    </tabset>
  </file>
</example>
 */
.directive('tabset', function() {
  return {
    restrict: 'EA',
    transclude: true,
    replace: true,
    scope: {
      type: '@'
    },
    controller: 'TabsetController',
    templateUrl: 'template/tabs/tabset.html',
    link: function(scope, element, attrs) {
      scope.vertical = angular.isDefined(attrs.vertical) ? scope.$parent.$eval(attrs.vertical) : false;
      scope.justified = angular.isDefined(attrs.justified) ? scope.$parent.$eval(attrs.justified) : false;
    }
  };
})

/**
 * @ngdoc directive
 * @name ui.bootstrap.tabs.directive:tab
 * @restrict EA
 *
 * @param {string=} heading The visible heading, or title, of the tab. Set HTML headings with {@link ui.bootstrap.tabs.directive:tabHeading tabHeading}.
 * @param {string=} select An expression to evaluate when the tab is selected.
 * @param {boolean=} active A binding, telling whether or not this tab is selected.
 * @param {boolean=} disabled A binding, telling whether or not this tab is disabled.
 *
 * @description
 * Creates a tab with a heading and content. Must be placed within a {@link ui.bootstrap.tabs.directive:tabset tabset}.
 *
 * @example
<example module="ui.bootstrap">
  <file name="index.html">
    <div ng-controller="TabsDemoCtrl">
      <button class="btn btn-small" ng-click="items[0].active = true">
        Select item 1, using active binding
      </button>
      <button class="btn btn-small" ng-click="items[1].disabled = !items[1].disabled">
        Enable/disable item 2, using disabled binding
      </button>
      <br />
      <tabset>
        <tab heading="Tab 1">First Tab</tab>
        <tab select="alertMe()">
          <tab-heading><i class="icon-bell"></i> Alert me!</tab-heading>
          Second Tab, with alert callback and html heading!
        </tab>
        <tab ng-repeat="item in items"
          heading="{{item.title}}"
          disabled="item.disabled"
          active="item.active">
          {{item.content}}
        </tab>
      </tabset>
    </div>
  </file>
  <file name="script.js">
    function TabsDemoCtrl($scope) {
      $scope.items = [
        { title:"Dynamic Title 1", content:"Dynamic Item 0" },
        { title:"Dynamic Title 2", content:"Dynamic Item 1", disabled: true }
      ];

      $scope.alertMe = function() {
        setTimeout(function() {
          alert("You've selected the alert tab!");
        });
      };
    };
  </file>
</example>
 */

/**
 * @ngdoc directive
 * @name ui.bootstrap.tabs.directive:tabHeading
 * @restrict EA
 *
 * @description
 * Creates an HTML heading for a {@link ui.bootstrap.tabs.directive:tab tab}. Must be placed as a child of a tab element.
 *
 * @example
<example module="ui.bootstrap">
  <file name="index.html">
    <tabset>
      <tab>
        <tab-heading><b>HTML</b> in my titles?!</tab-heading>
        And some content, too!
      </tab>
      <tab>
        <tab-heading><i class="icon-heart"></i> Icon heading?!?</tab-heading>
        That's right.
      </tab>
    </tabset>
  </file>
</example>
 */
.directive('tab', ['$parse', '$log', function($parse, $log) {
  return {
    require: '^tabset',
    restrict: 'EA',
    replace: true,
    templateUrl: 'template/tabs/tab.html',
    transclude: true,
    scope: {
      active: '=?',
      heading: '@',
      onSelect: '&select', //This callback is called in contentHeadingTransclude
                          //once it inserts the tab's content into the dom
      onDeselect: '&deselect'
    },
    controller: function() {
      //Empty controller so other directives can require being 'under' a tab
    },
    link: function(scope, elm, attrs, tabsetCtrl, transclude) {
      scope.$watch('active', function(active) {
        if (active) {
          tabsetCtrl.select(scope);
        }
      });

      scope.disabled = false;
      if (attrs.disable) {
        scope.$parent.$watch($parse(attrs.disable), function(value) {
          scope.disabled = !! value;
        });
      }

      // Deprecation support of "disabled" parameter
      // fix(tab): IE9 disabled attr renders grey text on enabled tab #2677
      // This code is duplicated from the lines above to make it easy to remove once
      // the feature has been completely deprecated
      if (attrs.disabled) {
        $log.warn('Use of "disabled" attribute has been deprecated, please use "disable"');
        scope.$parent.$watch($parse(attrs.disabled), function(value) {
          scope.disabled = !! value;
        });
      }

      scope.select = function() {
        if (!scope.disabled) {
          scope.active = true;
        }
      };

      tabsetCtrl.addTab(scope);
      scope.$on('$destroy', function() {
        tabsetCtrl.removeTab(scope);
      });

      //We need to transclude later, once the content container is ready.
      //when this link happens, we're inside a tab heading.
      scope.$transcludeFn = transclude;
    }
  };
}])

.directive('tabHeadingTransclude', function() {
  return {
    restrict: 'A',
    require: '^tab',
    link: function(scope, elm, attrs, tabCtrl) {
      scope.$watch('headingElement', function updateHeadingElement(heading) {
        if (heading) {
          elm.html('');
          elm.append(heading);
        }
      });
    }
  };
})

.directive('tabContentTransclude', function() {
  return {
    restrict: 'A',
    require: '^tabset',
    link: function(scope, elm, attrs) {
      var tab = scope.$eval(attrs.tabContentTransclude);

      //Now our tab is ready to be transcluded: both the tab heading area
      //and the tab content area are loaded.  Transclude 'em both.
      tab.$transcludeFn(tab.$parent, function(contents) {
        angular.forEach(contents, function(node) {
          if (isTabHeading(node)) {
            //Let tabHeadingTransclude know.
            tab.headingElement = node;
          } else {
            elm.append(node);
          }
        });
      });
    }
  };

  function isTabHeading(node) {
    return node.tagName && (
      node.hasAttribute('tab-heading') ||
      node.hasAttribute('data-tab-heading') ||
      node.hasAttribute('x-tab-heading') ||
      node.tagName.toLowerCase() === 'tab-heading' ||
      node.tagName.toLowerCase() === 'data-tab-heading' ||
      node.tagName.toLowerCase() === 'x-tab-heading'
    );
  }
});

angular.module('ui.bootstrap.timepicker', [])

.constant('timepickerConfig', {
  hourStep: 1,
  minuteStep: 1,
  showMeridian: true,
  meridians: null,
  readonlyInput: false,
  mousewheel: true,
  arrowkeys: true,
  showSpinners: true
})

.controller('TimepickerController', ['$scope', '$attrs', '$parse', '$log', '$locale', 'timepickerConfig', function($scope, $attrs, $parse, $log, $locale, timepickerConfig) {
  var selected = new Date(),
      ngModelCtrl = { $setViewValue: angular.noop }, // nullModelCtrl
      meridians = angular.isDefined($attrs.meridians) ? $scope.$parent.$eval($attrs.meridians) : timepickerConfig.meridians || $locale.DATETIME_FORMATS.AMPMS;

  this.init = function(ngModelCtrl_, inputs) {
    ngModelCtrl = ngModelCtrl_;
    ngModelCtrl.$render = this.render;

    ngModelCtrl.$formatters.unshift(function(modelValue) {
      return modelValue ? new Date(modelValue) : null;
    });

    var hoursInputEl = inputs.eq(0),
        minutesInputEl = inputs.eq(1);

    var mousewheel = angular.isDefined($attrs.mousewheel) ? $scope.$parent.$eval($attrs.mousewheel) : timepickerConfig.mousewheel;
    if (mousewheel) {
      this.setupMousewheelEvents(hoursInputEl, minutesInputEl);
    }

    var arrowkeys = angular.isDefined($attrs.arrowkeys) ? $scope.$parent.$eval($attrs.arrowkeys) : timepickerConfig.arrowkeys;
    if (arrowkeys) {
      this.setupArrowkeyEvents(hoursInputEl, minutesInputEl);
    }

    $scope.readonlyInput = angular.isDefined($attrs.readonlyInput) ? $scope.$parent.$eval($attrs.readonlyInput) : timepickerConfig.readonlyInput;
    this.setupInputEvents(hoursInputEl, minutesInputEl);
  };

  var hourStep = timepickerConfig.hourStep;
  if ($attrs.hourStep) {
    $scope.$parent.$watch($parse($attrs.hourStep), function(value) {
      hourStep = parseInt(value, 10);
    });
  }

  var minuteStep = timepickerConfig.minuteStep;
  if ($attrs.minuteStep) {
    $scope.$parent.$watch($parse($attrs.minuteStep), function(value) {
      minuteStep = parseInt(value, 10);
    });
  }

  var min;
  $scope.$parent.$watch($parse($attrs.min), function(value) {
    var dt = new Date(value);
    min = isNaN(dt) ? undefined : dt;
  });

  var max;
  $scope.$parent.$watch($parse($attrs.max), function(value) {
    var dt = new Date(value);
    max = isNaN(dt) ? undefined : dt;
  });

  $scope.noIncrementHours = function() {
    var incrementedSelected = addMinutes(selected, hourStep * 60);
    return incrementedSelected > max ||
      (incrementedSelected < selected && incrementedSelected < min);
  };

  $scope.noDecrementHours = function() {
    var decrementedSelected = addMinutes(selected, -hourStep * 60);
    return decrementedSelected < min ||
      (decrementedSelected > selected && decrementedSelected > max);
  };

  $scope.noIncrementMinutes = function() {
    var incrementedSelected = addMinutes(selected, minuteStep);
    return incrementedSelected > max ||
      (incrementedSelected < selected && incrementedSelected < min);
  };

  $scope.noDecrementMinutes = function() {
    var decrementedSelected = addMinutes(selected, -minuteStep);
    return decrementedSelected < min ||
      (decrementedSelected > selected && decrementedSelected > max);
  };

  $scope.noToggleMeridian = function() {
    if (selected.getHours() < 13) {
      return addMinutes(selected, 12 * 60) > max;
    } else {
      return addMinutes(selected, -12 * 60) < min;
    }
  };

  // 12H / 24H mode
  $scope.showMeridian = timepickerConfig.showMeridian;
  if ($attrs.showMeridian) {
    $scope.$parent.$watch($parse($attrs.showMeridian), function(value) {
      $scope.showMeridian = !!value;

      if (ngModelCtrl.$error.time) {
        // Evaluate from template
        var hours = getHoursFromTemplate(), minutes = getMinutesFromTemplate();
        if (angular.isDefined(hours) && angular.isDefined(minutes)) {
          selected.setHours(hours);
          refresh();
        }
      } else {
        updateTemplate();
      }
    });
  }

  // Get $scope.hours in 24H mode if valid
  function getHoursFromTemplate() {
    var hours = parseInt($scope.hours, 10);
    var valid = $scope.showMeridian ? (hours > 0 && hours < 13) : (hours >= 0 && hours < 24);
    if (!valid) {
      return undefined;
    }

    if ($scope.showMeridian) {
      if (hours === 12) {
        hours = 0;
      }
      if ($scope.meridian === meridians[1]) {
        hours = hours + 12;
      }
    }
    return hours;
  }

  function getMinutesFromTemplate() {
    var minutes = parseInt($scope.minutes, 10);
    return (minutes >= 0 && minutes < 60) ? minutes : undefined;
  }

  function pad(value) {
    return (angular.isDefined(value) && value.toString().length < 2) ? '0' + value : value.toString();
  }

  // Respond on mousewheel spin
  this.setupMousewheelEvents = function(hoursInputEl, minutesInputEl) {
    var isScrollingUp = function(e) {
      if (e.originalEvent) {
        e = e.originalEvent;
      }
      //pick correct delta variable depending on event
      var delta = (e.wheelDelta) ? e.wheelDelta : -e.deltaY;
      return (e.detail || delta > 0);
    };

    hoursInputEl.bind('mousewheel wheel', function(e) {
      $scope.$apply(isScrollingUp(e) ? $scope.incrementHours() : $scope.decrementHours());
      e.preventDefault();
    });

    minutesInputEl.bind('mousewheel wheel', function(e) {
      $scope.$apply(isScrollingUp(e) ? $scope.incrementMinutes() : $scope.decrementMinutes());
      e.preventDefault();
    });

  };

  // Respond on up/down arrowkeys
  this.setupArrowkeyEvents = function(hoursInputEl, minutesInputEl) {
    hoursInputEl.bind('keydown', function(e) {
      if (e.which === 38) { // up
        e.preventDefault();
        $scope.incrementHours();
        $scope.$apply();
      } else if (e.which === 40) { // down
        e.preventDefault();
        $scope.decrementHours();
        $scope.$apply();
      }
    });

    minutesInputEl.bind('keydown', function(e) {
      if (e.which === 38) { // up
        e.preventDefault();
        $scope.incrementMinutes();
        $scope.$apply();
      } else if (e.which === 40) { // down
        e.preventDefault();
        $scope.decrementMinutes();
        $scope.$apply();
      }
    });
  };

  this.setupInputEvents = function(hoursInputEl, minutesInputEl) {
    if ($scope.readonlyInput) {
      $scope.updateHours = angular.noop;
      $scope.updateMinutes = angular.noop;
      return;
    }

    var invalidate = function(invalidHours, invalidMinutes) {
      ngModelCtrl.$setViewValue(null);
      ngModelCtrl.$setValidity('time', false);
      if (angular.isDefined(invalidHours)) {
        $scope.invalidHours = invalidHours;
      }
      if (angular.isDefined(invalidMinutes)) {
        $scope.invalidMinutes = invalidMinutes;
      }
    };

    $scope.updateHours = function() {
      var hours = getHoursFromTemplate(),
        minutes = getMinutesFromTemplate();

      if (angular.isDefined(hours) && angular.isDefined(minutes)) {
        selected.setHours(hours);
        if (selected < min || selected > max) {
          invalidate(true);
        } else {
          refresh('h');
        }
      } else {
        invalidate(true);
      }
    };

    hoursInputEl.bind('blur', function(e) {
      if (!$scope.invalidHours && $scope.hours < 10) {
        $scope.$apply(function() {
          $scope.hours = pad($scope.hours);
        });
      }
    });

    $scope.updateMinutes = function() {
      var minutes = getMinutesFromTemplate(),
        hours = getHoursFromTemplate();

      if (angular.isDefined(minutes) && angular.isDefined(hours)) {
        selected.setMinutes(minutes);
        if (selected < min || selected > max) {
          invalidate(undefined, true);
        } else {
          refresh('m');
        }
      } else {
        invalidate(undefined, true);
      }
    };

    minutesInputEl.bind('blur', function(e) {
      if (!$scope.invalidMinutes && $scope.minutes < 10) {
        $scope.$apply(function() {
          $scope.minutes = pad($scope.minutes);
        });
      }
    });

  };

  this.render = function() {
    var date = ngModelCtrl.$viewValue;

    if (isNaN(date)) {
      ngModelCtrl.$setValidity('time', false);
      $log.error('Timepicker directive: "ng-model" value must be a Date object, a number of milliseconds since 01.01.1970 or a string representing an RFC2822 or ISO 8601 date.');
    } else {
      if (date) {
        selected = date;
      }

      if (selected < min || selected > max) {
        ngModelCtrl.$setValidity('time', false);
        $scope.invalidHours = true;
        $scope.invalidMinutes = true;
      } else {
        makeValid();
      }
      updateTemplate();
    }
  };

  // Call internally when we know that model is valid.
  function refresh(keyboardChange) {
    makeValid();
    ngModelCtrl.$setViewValue(new Date(selected));
    updateTemplate(keyboardChange);
  }

  function makeValid() {
    ngModelCtrl.$setValidity('time', true);
    $scope.invalidHours = false;
    $scope.invalidMinutes = false;
  }

  function updateTemplate(keyboardChange) {
    var hours = selected.getHours(), minutes = selected.getMinutes();

    if ($scope.showMeridian) {
      hours = (hours === 0 || hours === 12) ? 12 : hours % 12; // Convert 24 to 12 hour system
    }

    $scope.hours = keyboardChange === 'h' ? hours : pad(hours);
    if (keyboardChange !== 'm') {
      $scope.minutes = pad(minutes);
    }
    $scope.meridian = selected.getHours() < 12 ? meridians[0] : meridians[1];
  }

  function addMinutes(date, minutes) {
    var dt = new Date(date.getTime() + minutes * 60000);
    var newDate = new Date(date);
    newDate.setHours(dt.getHours(), dt.getMinutes());
    return newDate;
  }

  function addMinutesToSelected(minutes) {
    selected = addMinutes(selected, minutes);
    refresh();
  }

  $scope.showSpinners = angular.isDefined($attrs.showSpinners) ?
    $scope.$parent.$eval($attrs.showSpinners) : timepickerConfig.showSpinners;

  $scope.incrementHours = function() {
    if (!$scope.noIncrementHours()) {
      addMinutesToSelected(hourStep * 60);
    }
  };

  $scope.decrementHours = function() {
    if (!$scope.noDecrementHours()) {
      addMinutesToSelected(-hourStep * 60);
    }
  };

  $scope.incrementMinutes = function() {
    if (!$scope.noIncrementMinutes()) {
      addMinutesToSelected(minuteStep);
    }
  };

  $scope.decrementMinutes = function() {
    if (!$scope.noDecrementMinutes()) {
      addMinutesToSelected(-minuteStep);
    }
  };

  $scope.toggleMeridian = function() {
    if (!$scope.noToggleMeridian()) {
      addMinutesToSelected(12 * 60 * (selected.getHours() < 12 ? 1 : -1));
    }
  };
}])

.directive('timepicker', function() {
  return {
    restrict: 'EA',
    require: ['timepicker', '?^ngModel'],
    controller:'TimepickerController',
    controllerAs: 'timepicker',
    replace: true,
    scope: {},
    templateUrl: function(element, attrs) {
      return attrs.templateUrl || 'template/timepicker/timepicker.html';
    },
    link: function(scope, element, attrs, ctrls) {
      var timepickerCtrl = ctrls[0], ngModelCtrl = ctrls[1];

      if (ngModelCtrl) {
        timepickerCtrl.init(ngModelCtrl, element.find('input'));
      }
    }
  };
});

angular.module('ui.bootstrap.transition', [])

.value('$transitionSuppressDeprecated', false)
/**
 * $transition service provides a consistent interface to trigger CSS 3 transitions and to be informed when they complete.
 * @param  {DOMElement} element  The DOMElement that will be animated.
 * @param  {string|object|function} trigger  The thing that will cause the transition to start:
 *   - As a string, it represents the css class to be added to the element.
 *   - As an object, it represents a hash of style attributes to be applied to the element.
 *   - As a function, it represents a function to be called that will cause the transition to occur.
 * @return {Promise}  A promise that is resolved when the transition finishes.
 */
.factory('$transition', [
        '$q', '$timeout', '$rootScope', '$log', '$transitionSuppressDeprecated',
function($q ,  $timeout ,  $rootScope ,  $log ,  $transitionSuppressDeprecated) {

  if (!$transitionSuppressDeprecated) {
    $log.warn('$transition is now deprecated. Use $animate from ngAnimate instead.');
  }

  var $transition = function(element, trigger, options) {
    options = options || {};
    var deferred = $q.defer();
    var endEventName = $transition[options.animation ? 'animationEndEventName' : 'transitionEndEventName'];

    var transitionEndHandler = function(event) {
      $rootScope.$apply(function() {
        element.unbind(endEventName, transitionEndHandler);
        deferred.resolve(element);
      });
    };

    if (endEventName) {
      element.bind(endEventName, transitionEndHandler);
    }

    // Wrap in a timeout to allow the browser time to update the DOM before the transition is to occur
    $timeout(function() {
      if ( angular.isString(trigger) ) {
        element.addClass(trigger);
      } else if ( angular.isFunction(trigger) ) {
        trigger(element);
      } else if ( angular.isObject(trigger) ) {
        element.css(trigger);
      }
      //If browser does not support transitions, instantly resolve
      if ( !endEventName ) {
        deferred.resolve(element);
      }
    });

    // Add our custom cancel function to the promise that is returned
    // We can call this if we are about to run a new transition, which we know will prevent this transition from ending,
    // i.e. it will therefore never raise a transitionEnd event for that transition
    deferred.promise.cancel = function() {
      if ( endEventName ) {
        element.unbind(endEventName, transitionEndHandler);
      }
      deferred.reject('Transition cancelled');
    };

    return deferred.promise;
  };

  // Work out the name of the transitionEnd event
  var transElement = document.createElement('trans');
  var transitionEndEventNames = {
    'WebkitTransition': 'webkitTransitionEnd',
    'MozTransition': 'transitionend',
    'OTransition': 'oTransitionEnd',
    'transition': 'transitionend'
  };
  var animationEndEventNames = {
    'WebkitTransition': 'webkitAnimationEnd',
    'MozTransition': 'animationend',
    'OTransition': 'oAnimationEnd',
    'transition': 'animationend'
  };
  function findEndEventName(endEventNames) {
    for (var name in endEventNames){
      if (transElement.style[name] !== undefined) {
        return endEventNames[name];
      }
    }
  }
  $transition.transitionEndEventName = findEndEventName(transitionEndEventNames);
  $transition.animationEndEventName = findEndEventName(animationEndEventNames);
  return $transition;
}]);

angular.module('ui.bootstrap.typeahead', ['ui.bootstrap.position'])

/**
 * A helper service that can parse typeahead's syntax (string provided by users)
 * Extracted to a separate service for ease of unit testing
 */
  .factory('typeaheadParser', ['$parse', function($parse) {

  //                      00000111000000000000022200000000000000003333333333333330000000000044000
  var TYPEAHEAD_REGEXP = /^\s*([\s\S]+?)(?:\s+as\s+([\s\S]+?))?\s+for\s+(?:([\$\w][\$\w\d]*))\s+in\s+([\s\S]+?)$/;

  return {
    parse: function(input) {
      var match = input.match(TYPEAHEAD_REGEXP);
      if (!match) {
        throw new Error(
          'Expected typeahead specification in form of "_modelValue_ (as _label_)? for _item_ in _collection_"' +
            ' but got "' + input + '".');
      }

      return {
        itemName:match[3],
        source:$parse(match[4]),
        viewMapper:$parse(match[2] || match[1]),
        modelMapper:$parse(match[1])
      };
    }
  };
}])

  .directive('typeahead', ['$compile', '$parse', '$q', '$timeout', '$document', '$window', '$rootScope', '$position', 'typeaheadParser',
    function($compile, $parse, $q, $timeout, $document, $window, $rootScope, $position, typeaheadParser) {
    var HOT_KEYS = [9, 13, 27, 38, 40];
    var eventDebounceTime = 200;

    return {
      require: ['ngModel', '^?ngModelOptions'],
      link: function(originalScope, element, attrs, ctrls) {
        var modelCtrl = ctrls[0];
        var ngModelOptions = ctrls[1];
        //SUPPORTED ATTRIBUTES (OPTIONS)

        //minimal no of characters that needs to be entered before typeahead kicks-in
        var minLength = originalScope.$eval(attrs.typeaheadMinLength);
        if (!minLength && minLength !== 0) {
          minLength = 1;
        }

        //minimal wait time after last character typed before typeahead kicks-in
        var waitTime = originalScope.$eval(attrs.typeaheadWaitMs) || 0;

        //should it restrict model values to the ones selected from the popup only?
        var isEditable = originalScope.$eval(attrs.typeaheadEditable) !== false;

        //binding to a variable that indicates if matches are being retrieved asynchronously
        var isLoadingSetter = $parse(attrs.typeaheadLoading).assign || angular.noop;

        //a callback executed when a match is selected
        var onSelectCallback = $parse(attrs.typeaheadOnSelect);

        //should it select highlighted popup value when losing focus?
        var isSelectOnBlur = angular.isDefined(attrs.typeaheadSelectOnBlur) ? originalScope.$eval(attrs.typeaheadSelectOnBlur) : false;

        //binding to a variable that indicates if there were no results after the query is completed
        var isNoResultsSetter = $parse(attrs.typeaheadNoResults).assign || angular.noop;

        var inputFormatter = attrs.typeaheadInputFormatter ? $parse(attrs.typeaheadInputFormatter) : undefined;

        var appendToBody =  attrs.typeaheadAppendToBody ? originalScope.$eval(attrs.typeaheadAppendToBody) : false;

        var focusFirst = originalScope.$eval(attrs.typeaheadFocusFirst) !== false;

        //If input matches an item of the list exactly, select it automatically
        var selectOnExact = attrs.typeaheadSelectOnExact ? originalScope.$eval(attrs.typeaheadSelectOnExact) : false;

        //INTERNAL VARIABLES

        //model setter executed upon match selection
        var parsedModel = $parse(attrs.ngModel);
        var invokeModelSetter = $parse(attrs.ngModel + '($$$p)');
        var $setModelValue = function(scope, newValue) {
          if (angular.isFunction(parsedModel(originalScope)) &&
            ngModelOptions && ngModelOptions.$options && ngModelOptions.$options.getterSetter) {
            return invokeModelSetter(scope, {$$$p: newValue});
          } else {
            return parsedModel.assign(scope, newValue);
          }
        };

        //expressions used by typeahead
        var parserResult = typeaheadParser.parse(attrs.typeahead);

        var hasFocus;

        //Used to avoid bug in iOS webview where iOS keyboard does not fire
        //mousedown & mouseup events
        //Issue #3699
        var selected;

        //create a child scope for the typeahead directive so we are not polluting original scope
        //with typeahead-specific data (matches, query etc.)
        var scope = originalScope.$new();
        var offDestroy = originalScope.$on('$destroy', function() {
			    scope.$destroy();
        });
        scope.$on('$destroy', offDestroy);

        // WAI-ARIA
        var popupId = 'typeahead-' + scope.$id + '-' + Math.floor(Math.random() * 10000);
        element.attr({
          'aria-autocomplete': 'list',
          'aria-expanded': false,
          'aria-owns': popupId
        });

        //pop-up element used to display matches
        var popUpEl = angular.element('<div typeahead-popup></div>');
        popUpEl.attr({
          id: popupId,
          matches: 'matches',
          active: 'activeIdx',
          select: 'select(activeIdx)',
          'move-in-progress': 'moveInProgress',
          query: 'query',
          position: 'position'
        });
        //custom item template
        if (angular.isDefined(attrs.typeaheadTemplateUrl)) {
          popUpEl.attr('template-url', attrs.typeaheadTemplateUrl);
        }

        if (angular.isDefined(attrs.typeaheadPopupTemplateUrl)) {
          popUpEl.attr('popup-template-url', attrs.typeaheadPopupTemplateUrl);
        }

        var resetMatches = function() {
          scope.matches = [];
          scope.activeIdx = -1;
          element.attr('aria-expanded', false);
        };

        var getMatchId = function(index) {
          return popupId + '-option-' + index;
        };

        // Indicate that the specified match is the active (pre-selected) item in the list owned by this typeahead.
        // This attribute is added or removed automatically when the `activeIdx` changes.
        scope.$watch('activeIdx', function(index) {
          if (index < 0) {
            element.removeAttr('aria-activedescendant');
          } else {
            element.attr('aria-activedescendant', getMatchId(index));
          }
        });

        var inputIsExactMatch = function(inputValue, index) {
          if (scope.matches.length > index && inputValue) {
            return inputValue.toUpperCase() === scope.matches[index].label.toUpperCase();
          }

          return false;
        };

        var getMatchesAsync = function(inputValue) {
          var locals = {$viewValue: inputValue};
          isLoadingSetter(originalScope, true);
          isNoResultsSetter(originalScope, false);
          $q.when(parserResult.source(originalScope, locals)).then(function(matches) {
            //it might happen that several async queries were in progress if a user were typing fast
            //but we are interested only in responses that correspond to the current view value
            var onCurrentRequest = (inputValue === modelCtrl.$viewValue);
            if (onCurrentRequest && hasFocus) {
              if (matches && matches.length > 0) {

                scope.activeIdx = focusFirst ? 0 : -1;
                isNoResultsSetter(originalScope, false);
                scope.matches.length = 0;

                //transform labels
                for (var i = 0; i < matches.length; i++) {
                  locals[parserResult.itemName] = matches[i];
                  scope.matches.push({
                    id: getMatchId(i),
                    label: parserResult.viewMapper(scope, locals),
                    model: matches[i]
                  });
                }

                scope.query = inputValue;
                //position pop-up with matches - we need to re-calculate its position each time we are opening a window
                //with matches as a pop-up might be absolute-positioned and position of an input might have changed on a page
                //due to other elements being rendered
                recalculatePosition();

                element.attr('aria-expanded', true);

                //Select the single remaining option if user input matches
                if (selectOnExact && scope.matches.length === 1 && inputIsExactMatch(inputValue, 0)) {
                  scope.select(0);
                }
              } else {
                resetMatches();
                isNoResultsSetter(originalScope, true);
              }
            }
            if (onCurrentRequest) {
              isLoadingSetter(originalScope, false);
            }
          }, function() {
            resetMatches();
            isLoadingSetter(originalScope, false);
            isNoResultsSetter(originalScope, true);
          });
        };

        // bind events only if appendToBody params exist - performance feature
        if (appendToBody) {
          angular.element($window).bind('resize', fireRecalculating);
          $document.find('body').bind('scroll', fireRecalculating);
        }

        // Declare the timeout promise var outside the function scope so that stacked calls can be cancelled later
        var timeoutEventPromise;

        // Default progress type
        scope.moveInProgress = false;

        function fireRecalculating() {
          if (!scope.moveInProgress) {
            scope.moveInProgress = true;
            scope.$digest();
          }

          // Cancel previous timeout
          if (timeoutEventPromise) {
            $timeout.cancel(timeoutEventPromise);
          }

          // Debounced executing recalculate after events fired
          timeoutEventPromise = $timeout(function() {
            // if popup is visible
            if (scope.matches.length) {
              recalculatePosition();
            }

            scope.moveInProgress = false;
            scope.$digest();
          }, eventDebounceTime);
        }

        // recalculate actual position and set new values to scope
        // after digest loop is popup in right position
        function recalculatePosition() {
          scope.position = appendToBody ? $position.offset(element) : $position.position(element);
          scope.position.top += element.prop('offsetHeight');
        }

        resetMatches();

        //we need to propagate user's query so we can higlight matches
        scope.query = undefined;

        //Declare the timeout promise var outside the function scope so that stacked calls can be cancelled later
        var timeoutPromise;

        var scheduleSearchWithTimeout = function(inputValue) {
          timeoutPromise = $timeout(function() {
            getMatchesAsync(inputValue);
          }, waitTime);
        };

        var cancelPreviousTimeout = function() {
          if (timeoutPromise) {
            $timeout.cancel(timeoutPromise);
          }
        };

        //plug into $parsers pipeline to open a typeahead on view changes initiated from DOM
        //$parsers kick-in on all the changes coming from the view as well as manually triggered by $setViewValue
        modelCtrl.$parsers.unshift(function(inputValue) {
          hasFocus = true;

          if (minLength === 0 || inputValue && inputValue.length >= minLength) {
            if (waitTime > 0) {
              cancelPreviousTimeout();
              scheduleSearchWithTimeout(inputValue);
            } else {
              getMatchesAsync(inputValue);
            }
          } else {
            isLoadingSetter(originalScope, false);
            cancelPreviousTimeout();
            resetMatches();
          }

          if (isEditable) {
            return inputValue;
          } else {
            if (!inputValue) {
              // Reset in case user had typed something previously.
              modelCtrl.$setValidity('editable', true);
              return null;
            } else {
              modelCtrl.$setValidity('editable', false);
              return undefined;
            }
          }
        });

        modelCtrl.$formatters.push(function(modelValue) {
          var candidateViewValue, emptyViewValue;
          var locals = {};

          // The validity may be set to false via $parsers (see above) if
          // the model is restricted to selected values. If the model
          // is set manually it is considered to be valid.
          if (!isEditable) {
            modelCtrl.$setValidity('editable', true);
          }

          if (inputFormatter) {
            locals.$model = modelValue;
            return inputFormatter(originalScope, locals);
          } else {
            //it might happen that we don't have enough info to properly render input value
            //we need to check for this situation and simply return model value if we can't apply custom formatting
            locals[parserResult.itemName] = modelValue;
            candidateViewValue = parserResult.viewMapper(originalScope, locals);
            locals[parserResult.itemName] = undefined;
            emptyViewValue = parserResult.viewMapper(originalScope, locals);

            return candidateViewValue!== emptyViewValue ? candidateViewValue : modelValue;
          }
        });

        scope.select = function(activeIdx) {
          //called from within the $digest() cycle
          var locals = {};
          var model, item;

          selected = true;
          locals[parserResult.itemName] = item = scope.matches[activeIdx].model;
          model = parserResult.modelMapper(originalScope, locals);
          $setModelValue(originalScope, model);
          modelCtrl.$setValidity('editable', true);
          modelCtrl.$setValidity('parse', true);

          onSelectCallback(originalScope, {
            $item: item,
            $model: model,
            $label: parserResult.viewMapper(originalScope, locals)
          });

          resetMatches();

          //return focus to the input element if a match was selected via a mouse click event
          // use timeout to avoid $rootScope:inprog error
          if (scope.$eval(attrs.typeaheadFocusOnSelect) !== false) {
            $timeout(function() { element[0].focus(); }, 0, false);
          }
        };

        //bind keyboard events: arrows up(38) / down(40), enter(13) and tab(9), esc(27)
        element.bind('keydown', function(evt) {
          //typeahead is open and an "interesting" key was pressed
          if (scope.matches.length === 0 || HOT_KEYS.indexOf(evt.which) === -1) {
            return;
          }

          // if there's nothing selected (i.e. focusFirst) and enter or tab is hit, clear the results
          if (scope.activeIdx === -1 && (evt.which === 9 || evt.which === 13)) {
            resetMatches();
            scope.$digest();
            return;
          }

          evt.preventDefault();

          if (evt.which === 40) {
            scope.activeIdx = (scope.activeIdx + 1) % scope.matches.length;
            scope.$digest();

          } else if (evt.which === 38) {
            scope.activeIdx = (scope.activeIdx > 0 ? scope.activeIdx : scope.matches.length) - 1;
            scope.$digest();

          } else if (evt.which === 13 || evt.which === 9) {
            scope.$apply(function () {
              scope.select(scope.activeIdx);
            });

          } else if (evt.which === 27) {
            evt.stopPropagation();

            resetMatches();
            scope.$digest();
          }
        });

        element.bind('blur', function() {
          if (isSelectOnBlur && scope.matches.length && scope.activeIdx !== -1 && !selected) {
            selected = true;
            scope.$apply(function() {
              scope.select(scope.activeIdx);
            });
          }
          hasFocus = false;
          selected = false;
        });

        // Keep reference to click handler to unbind it.
        var dismissClickHandler = function(evt) {
          // Issue #3973
          // Firefox treats right click as a click on document
          if (element[0] !== evt.target && evt.which !== 3 && scope.matches.length !== 0) {
            resetMatches();
            if (!$rootScope.$$phase) {
              scope.$digest();
            }
          }
        };

        $document.bind('click', dismissClickHandler);

        originalScope.$on('$destroy', function() {
          $document.unbind('click', dismissClickHandler);
          if (appendToBody) {
            $popup.remove();
          }
          // Prevent jQuery cache memory leak
          popUpEl.remove();
        });

        var $popup = $compile(popUpEl)(scope);

        if (appendToBody) {
          $document.find('body').append($popup);
        } else {
          element.after($popup);
        }
      }
    };

  }])

  .directive('typeaheadPopup', function() {
    return {
      restrict: 'EA',
      scope: {
        matches: '=',
        query: '=',
        active: '=',
        position: '&',
        moveInProgress: '=',
        select: '&'
      },
      replace: true,
      templateUrl: function(element, attrs) {
        return attrs.popupTemplateUrl || 'template/typeahead/typeahead-popup.html';
      },
      link: function(scope, element, attrs) {
        scope.templateUrl = attrs.templateUrl;

        scope.isOpen = function() {
          return scope.matches.length > 0;
        };

        scope.isActive = function(matchIdx) {
          return scope.active == matchIdx;
        };

        scope.selectActive = function(matchIdx) {
          scope.active = matchIdx;
        };

        scope.selectMatch = function(activeIdx) {
          scope.select({activeIdx:activeIdx});
        };
      }
    };
  })

  .directive('typeaheadMatch', ['$templateRequest', '$compile', '$parse', function($templateRequest, $compile, $parse) {
    return {
      restrict: 'EA',
      scope: {
        index: '=',
        match: '=',
        query: '='
      },
      link:function(scope, element, attrs) {
        var tplUrl = $parse(attrs.templateUrl)(scope.$parent) || 'template/typeahead/typeahead-match.html';
        $templateRequest(tplUrl).then(function(tplContent) {
          $compile(tplContent.trim())(scope, function(clonedElement) {
            element.replaceWith(clonedElement);
          });
        });
      }
    };
  }])

  .filter('typeaheadHighlight', ['$sce', '$injector', '$log', function($sce, $injector, $log) {
    var isSanitizePresent;
    isSanitizePresent = $injector.has('$sanitize');

    function escapeRegexp(queryToEscape) {
      // Regex: capture the whole query string and replace it with the string that will be used to match
      // the results, for example if the capture is "a" the result will be \a
      return queryToEscape.replace(/([.?*+^$[\]\\(){}|-])/g, '\\$1');
    }

    function containsHtml(matchItem) {
      return /<.*>/g.test(matchItem);
    }

    return function(matchItem, query) {
      if (!isSanitizePresent && containsHtml(matchItem)) {
        $log.warn('Unsafe use of typeahead please use ngSanitize'); // Warn the user about the danger
      }
      matchItem = query? ('' + matchItem).replace(new RegExp(escapeRegexp(query), 'gi'), '<strong>$&</strong>') : matchItem; // Replaces the capture string with a the same string inside of a "strong" tag
      if (!isSanitizePresent) {
        matchItem = $sce.trustAsHtml(matchItem); // If $sanitize is not present we pack the string in a $sce object for the ng-bind-html directive
      }
      return matchItem;
    };
  }]);

angular.module("template/accordion/accordion-group.html", []).run(["$templateCache", function($templateCache) {
  $templateCache.put("template/accordion/accordion-group.html",
    "<div class=\"panel {{panelClass || 'panel-default'}}\">\n" +
    "  <div class=\"panel-heading\" ng-keypress=\"toggleOpen($event)\">\n" +
    "    <h4 class=\"panel-title\">\n" +
    "      <a href tabindex=\"0\" class=\"accordion-toggle\" ng-click=\"toggleOpen()\" accordion-transclude=\"heading\"><span ng-class=\"{'text-muted': isDisabled}\">{{heading}}</span></a>\n" +
    "    </h4>\n" +
    "  </div>\n" +
    "  <div class=\"panel-collapse collapse\" collapse=\"!isOpen\">\n" +
    "	  <div class=\"panel-body\" ng-transclude></div>\n" +
    "  </div>\n" +
    "</div>\n" +
    "");
}]);

angular.module("template/accordion/accordion.html", []).run(["$templateCache", function($templateCache) {
  $templateCache.put("template/accordion/accordion.html",
    "<div class=\"panel-group\" ng-transclude></div>");
}]);

angular.module("template/alert/alert.html", []).run(["$templateCache", function($templateCache) {
  $templateCache.put("template/alert/alert.html",
    "<div class=\"alert\" ng-class=\"['alert-' + (type || 'warning'), closeable ? 'alert-dismissible' : null]\" role=\"alert\">\n" +
    "    <button ng-show=\"closeable\" type=\"button\" class=\"close\" ng-click=\"close($event)\">\n" +
    "        <span aria-hidden=\"true\">&times;</span>\n" +
    "        <span class=\"sr-only\">Close</span>\n" +
    "    </button>\n" +
    "    <div ng-transclude></div>\n" +
    "</div>\n" +
    "");
}]);

angular.module("template/carousel/carousel.html", []).run(["$templateCache", function($templateCache) {
  $templateCache.put("template/carousel/carousel.html",
    "<div ng-mouseenter=\"pause()\" ng-mouseleave=\"play()\" class=\"carousel\" ng-swipe-right=\"prev()\" ng-swipe-left=\"next()\">\n" +
    "    <ol class=\"carousel-indicators\" ng-show=\"slides.length > 1\">\n" +
    "        <li ng-repeat=\"slide in slides | orderBy:indexOfSlide track by $index\" ng-class=\"{active: isActive(slide)}\" ng-click=\"select(slide)\"></li>\n" +
    "    </ol>\n" +
    "    <div class=\"carousel-inner\" ng-transclude></div>\n" +
    "    <a class=\"left carousel-control\" ng-click=\"prev()\" ng-show=\"slides.length > 1\"><span class=\"glyphicon glyphicon-chevron-left\"></span></a>\n" +
    "    <a class=\"right carousel-control\" ng-click=\"next()\" ng-show=\"slides.length > 1\"><span class=\"glyphicon glyphicon-chevron-right\"></span></a>\n" +
    "</div>\n" +
    "");
}]);

angular.module("template/carousel/slide.html", []).run(["$templateCache", function($templateCache) {
  $templateCache.put("template/carousel/slide.html",
    "<div ng-class=\"{\n" +
    "    'active': active\n" +
    "  }\" class=\"item text-center\" ng-transclude></div>\n" +
    "");
}]);

angular.module("template/datepicker/datepicker.html", []).run(["$templateCache", function($templateCache) {
  $templateCache.put("template/datepicker/datepicker.html",
    "<div ng-switch=\"datepickerMode\" role=\"application\" ng-keydown=\"keydown($event)\">\n" +
    "  <daypicker ng-switch-when=\"day\" tabindex=\"0\"></daypicker>\n" +
    "  <monthpicker ng-switch-when=\"month\" tabindex=\"0\"></monthpicker>\n" +
    "  <yearpicker ng-switch-when=\"year\" tabindex=\"0\"></yearpicker>\n" +
    "</div>");
}]);

angular.module("template/datepicker/day.html", []).run(["$templateCache", function($templateCache) {
  $templateCache.put("template/datepicker/day.html",
    "<table role=\"grid\" aria-labelledby=\"{{::uniqueId}}-title\" aria-activedescendant=\"{{activeDateId}}\">\n" +
    "  <thead>\n" +
    "    <tr>\n" +
    "      <th><button type=\"button\" class=\"btn btn-default btn-sm pull-left\" ng-click=\"move(-1)\" tabindex=\"-1\"><i class=\"glyphicon glyphicon-chevron-left\"></i></button></th>\n" +
    "      <th colspan=\"{{::5 + showWeeks}}\"><button id=\"{{::uniqueId}}-title\" role=\"heading\" aria-live=\"assertive\" aria-atomic=\"true\" type=\"button\" class=\"btn btn-default btn-sm\" ng-click=\"toggleMode()\" ng-disabled=\"datepickerMode === maxMode\" tabindex=\"-1\" style=\"width:100%;\"><strong>{{title}}</strong></button></th>\n" +
    "      <th><button type=\"button\" class=\"btn btn-default btn-sm pull-right\" ng-click=\"move(1)\" tabindex=\"-1\"><i class=\"glyphicon glyphicon-chevron-right\"></i></button></th>\n" +
    "    </tr>\n" +
    "    <tr>\n" +
    "      <th ng-if=\"showWeeks\" class=\"text-center\"></th>\n" +
    "      <th ng-repeat=\"label in ::labels track by $index\" class=\"text-center\"><small aria-label=\"{{::label.full}}\">{{::label.abbr}}</small></th>\n" +
    "    </tr>\n" +
    "  </thead>\n" +
    "  <tbody>\n" +
    "    <tr ng-repeat=\"row in rows track by $index\">\n" +
    "      <td ng-if=\"showWeeks\" class=\"text-center h6\"><em>{{ weekNumbers[$index] }}</em></td>\n" +
    "      <td ng-repeat=\"dt in row track by dt.date\" class=\"text-center\" role=\"gridcell\" id=\"{{::dt.uid}}\" ng-class=\"::dt.customClass\">\n" +
    "        <button type=\"button\" style=\"min-width:100%;\" class=\"btn btn-default btn-sm\" ng-class=\"{'btn-info': dt.selected, active: isActive(dt)}\" ng-click=\"select(dt.date)\" ng-disabled=\"dt.disabled\" tabindex=\"-1\"><span ng-class=\"::{'text-muted': dt.secondary, 'text-info': dt.current}\">{{::dt.label}}</span></button>\n" +
    "      </td>\n" +
    "    </tr>\n" +
    "  </tbody>\n" +
    "</table>\n" +
    "");
}]);

angular.module("template/datepicker/month.html", []).run(["$templateCache", function($templateCache) {
  $templateCache.put("template/datepicker/month.html",
    "<table role=\"grid\" aria-labelledby=\"{{::uniqueId}}-title\" aria-activedescendant=\"{{activeDateId}}\">\n" +
    "  <thead>\n" +
    "    <tr>\n" +
    "      <th><button type=\"button\" class=\"btn btn-default btn-sm pull-left\" ng-click=\"move(-1)\" tabindex=\"-1\"><i class=\"glyphicon glyphicon-chevron-left\"></i></button></th>\n" +
    "      <th><button id=\"{{::uniqueId}}-title\" role=\"heading\" aria-live=\"assertive\" aria-atomic=\"true\" type=\"button\" class=\"btn btn-default btn-sm\" ng-click=\"toggleMode()\" ng-disabled=\"datepickerMode === maxMode\" tabindex=\"-1\" style=\"width:100%;\"><strong>{{title}}</strong></button></th>\n" +
    "      <th><button type=\"button\" class=\"btn btn-default btn-sm pull-right\" ng-click=\"move(1)\" tabindex=\"-1\"><i class=\"glyphicon glyphicon-chevron-right\"></i></button></th>\n" +
    "    </tr>\n" +
    "  </thead>\n" +
    "  <tbody>\n" +
    "    <tr ng-repeat=\"row in rows track by $index\">\n" +
    "      <td ng-repeat=\"dt in row track by dt.date\" class=\"text-center\" role=\"gridcell\" id=\"{{::dt.uid}}\" ng-class=\"::dt.customClass\">\n" +
    "        <button type=\"button\" style=\"min-width:100%;\" class=\"btn btn-default\" ng-class=\"{'btn-info': dt.selected, active: isActive(dt)}\" ng-click=\"select(dt.date)\" ng-disabled=\"dt.disabled\" tabindex=\"-1\"><span ng-class=\"::{'text-info': dt.current}\">{{::dt.label}}</span></button>\n" +
    "      </td>\n" +
    "    </tr>\n" +
    "  </tbody>\n" +
    "</table>\n" +
    "");
}]);

angular.module("template/datepicker/popup.html", []).run(["$templateCache", function($templateCache) {
  $templateCache.put("template/datepicker/popup.html",
    "<ul class=\"dropdown-menu\" ng-if=\"isOpen\" style=\"display: block\" ng-style=\"{top: position.top+'px', left: position.left+'px'}\" ng-keydown=\"keydown($event)\" ng-click=\"$event.stopPropagation()\">\n" +
    "	<li ng-transclude></li>\n" +
    "	<li ng-if=\"showButtonBar\" style=\"padding:10px 9px 2px\">\n" +
    "		<span class=\"btn-group pull-left\">\n" +
    "			<button type=\"button\" class=\"btn btn-sm btn-info\" ng-click=\"select('today')\" ng-disabled=\"isDisabled('today')\">{{ getText('current') }}</button>\n" +
    "			<button type=\"button\" class=\"btn btn-sm btn-danger\" ng-click=\"select(null)\">{{ getText('clear') }}</button>\n" +
    "		</span>\n" +
    "		<button type=\"button\" class=\"btn btn-sm btn-success pull-right\" ng-click=\"close()\">{{ getText('close') }}</button>\n" +
    "	</li>\n" +
    "</ul>\n" +
    "");
}]);

angular.module("template/datepicker/year.html", []).run(["$templateCache", function($templateCache) {
  $templateCache.put("template/datepicker/year.html",
    "<table role=\"grid\" aria-labelledby=\"{{::uniqueId}}-title\" aria-activedescendant=\"{{activeDateId}}\">\n" +
    "  <thead>\n" +
    "    <tr>\n" +
    "      <th><button type=\"button\" class=\"btn btn-default btn-sm pull-left\" ng-click=\"move(-1)\" tabindex=\"-1\"><i class=\"glyphicon glyphicon-chevron-left\"></i></button></th>\n" +
    "      <th colspan=\"3\"><button id=\"{{::uniqueId}}-title\" role=\"heading\" aria-live=\"assertive\" aria-atomic=\"true\" type=\"button\" class=\"btn btn-default btn-sm\" ng-click=\"toggleMode()\" ng-disabled=\"datepickerMode === maxMode\" tabindex=\"-1\" style=\"width:100%;\"><strong>{{title}}</strong></button></th>\n" +
    "      <th><button type=\"button\" class=\"btn btn-default btn-sm pull-right\" ng-click=\"move(1)\" tabindex=\"-1\"><i class=\"glyphicon glyphicon-chevron-right\"></i></button></th>\n" +
    "    </tr>\n" +
    "  </thead>\n" +
    "  <tbody>\n" +
    "    <tr ng-repeat=\"row in rows track by $index\">\n" +
    "      <td ng-repeat=\"dt in row track by dt.date\" class=\"text-center\" role=\"gridcell\" id=\"{{::dt.uid}}\">\n" +
    "        <button type=\"button\" style=\"min-width:100%;\" class=\"btn btn-default\" ng-class=\"{'btn-info': dt.selected, active: isActive(dt)}\" ng-click=\"select(dt.date)\" ng-disabled=\"dt.disabled\" tabindex=\"-1\"><span ng-class=\"::{'text-info': dt.current}\">{{::dt.label}}</span></button>\n" +
    "      </td>\n" +
    "    </tr>\n" +
    "  </tbody>\n" +
    "</table>\n" +
    "");
}]);

angular.module("template/modal/backdrop.html", []).run(["$templateCache", function($templateCache) {
  $templateCache.put("template/modal/backdrop.html",
    "<div class=\"modal-backdrop\"\n" +
    "     modal-animation-class=\"fade\"\n" +
    "     modal-in-class=\"in\"\n" +
    "     ng-style=\"{'z-index': 1040 + (index && 1 || 0) + index*10}\"\n" +
    "></div>\n" +
    "");
}]);

angular.module("template/modal/window.html", []).run(["$templateCache", function($templateCache) {
  $templateCache.put("template/modal/window.html",
    "<div modal-render=\"{{$isRendered}}\" tabindex=\"-1\" role=\"dialog\" class=\"modal\"\n" +
    "    modal-animation-class=\"fade\"\n" +
    "    modal-in-class=\"in\"\n" +
    "	ng-style=\"{'z-index': 1050 + index*10, display: 'block'}\" ng-click=\"close($event)\">\n" +
    "    <div class=\"modal-dialog\" ng-class=\"size ? 'modal-' + size : ''\"><div class=\"modal-content\" modal-transclude></div></div>\n" +
    "</div>\n" +
    "");
}]);

angular.module("template/pagination/pager.html", []).run(["$templateCache", function($templateCache) {
  $templateCache.put("template/pagination/pager.html",
    "<ul class=\"pager\">\n" +
    "  <li ng-class=\"{disabled: noPrevious()||ngDisabled, previous: align}\"><a href ng-click=\"selectPage(page - 1, $event)\">{{::getText('previous')}}</a></li>\n" +
    "  <li ng-class=\"{disabled: noNext()||ngDisabled, next: align}\"><a href ng-click=\"selectPage(page + 1, $event)\">{{::getText('next')}}</a></li>\n" +
    "</ul>\n" +
    "");
}]);

angular.module("template/pagination/pagination.html", []).run(["$templateCache", function($templateCache) {
  $templateCache.put("template/pagination/pagination.html",
    "<ul class=\"pagination\">\n" +
    "  <li ng-if=\"::boundaryLinks\" ng-class=\"{disabled: noPrevious()||ngDisabled}\" class=\"pagination-first\"><a href ng-click=\"selectPage(1, $event)\">{{::getText('first')}}</a></li>\n" +
    "  <li ng-if=\"::directionLinks\" ng-class=\"{disabled: noPrevious()||ngDisabled}\" class=\"pagination-prev\"><a href ng-click=\"selectPage(page - 1, $event)\">{{::getText('previous')}}</a></li>\n" +
    "  <li ng-repeat=\"page in pages track by $index\" ng-class=\"{active: page.active,disabled: ngDisabled&&!page.active}\" class=\"pagination-page\"><a href ng-click=\"selectPage(page.number, $event)\">{{page.text}}</a></li>\n" +
    "  <li ng-if=\"::directionLinks\" ng-class=\"{disabled: noNext()||ngDisabled}\" class=\"pagination-next\"><a href ng-click=\"selectPage(page + 1, $event)\">{{::getText('next')}}</a></li>\n" +
    "  <li ng-if=\"::boundaryLinks\" ng-class=\"{disabled: noNext()||ngDisabled}\" class=\"pagination-last\"><a href ng-click=\"selectPage(totalPages, $event)\">{{::getText('last')}}</a></li>\n" +
    "</ul>\n" +
    "");
}]);

angular.module("template/tooltip/tooltip-html-popup.html", []).run(["$templateCache", function($templateCache) {
  $templateCache.put("template/tooltip/tooltip-html-popup.html",
    "<div class=\"tooltip\"\n" +
    "  tooltip-animation-class=\"fade\"\n" +
    "  tooltip-classes\n" +
    "  ng-class=\"{ in: isOpen() }\">\n" +
    "  <div class=\"tooltip-arrow\"></div>\n" +
    "  <div class=\"tooltip-inner\" ng-bind-html=\"contentExp()\"></div>\n" +
    "</div>\n" +
    "");
}]);

angular.module("template/tooltip/tooltip-html-unsafe-popup.html", []).run(["$templateCache", function($templateCache) {
  $templateCache.put("template/tooltip/tooltip-html-unsafe-popup.html",
    "<div class=\"tooltip\"\n" +
    "  tooltip-animation-class=\"fade\"\n" +
    "  tooltip-classes\n" +
    "  ng-class=\"{ in: isOpen() }\">\n" +
    "  <div class=\"tooltip-arrow\"></div>\n" +
    "  <div class=\"tooltip-inner\" bind-html-unsafe=\"content\"></div>\n" +
    "</div>\n" +
    "");
}]);

angular.module("template/tooltip/tooltip-popup.html", []).run(["$templateCache", function($templateCache) {
  $templateCache.put("template/tooltip/tooltip-popup.html",
    "<div class=\"tooltip\"\n" +
    "  tooltip-animation-class=\"fade\"\n" +
    "  tooltip-classes\n" +
    "  ng-class=\"{ in: isOpen() }\">\n" +
    "  <div class=\"tooltip-arrow\"></div>\n" +
    "  <div class=\"tooltip-inner\" ng-bind=\"content\"></div>\n" +
    "</div>\n" +
    "");
}]);

angular.module("template/tooltip/tooltip-template-popup.html", []).run(["$templateCache", function($templateCache) {
  $templateCache.put("template/tooltip/tooltip-template-popup.html",
    "<div class=\"tooltip\"\n" +
    "  tooltip-animation-class=\"fade\"\n" +
    "  tooltip-classes\n" +
    "  ng-class=\"{ in: isOpen() }\">\n" +
    "  <div class=\"tooltip-arrow\"></div>\n" +
    "  <div class=\"tooltip-inner\"\n" +
    "    tooltip-template-transclude=\"contentExp()\"\n" +
    "    tooltip-template-transclude-scope=\"originScope()\"></div>\n" +
    "</div>\n" +
    "");
}]);

angular.module("template/popover/popover-html.html", []).run(["$templateCache", function($templateCache) {
  $templateCache.put("template/popover/popover-html.html",
    "<div class=\"popover\"\n" +
    "  tooltip-animation-class=\"fade\"\n" +
    "  tooltip-classes\n" +
    "  ng-class=\"{ in: isOpen() }\">\n" +
    "  <div class=\"arrow\"></div>\n" +
    "\n" +
    "  <div class=\"popover-inner\">\n" +
    "      <h3 class=\"popover-title\" ng-bind=\"title\" ng-if=\"title\"></h3>\n" +
    "      <div class=\"popover-content\" ng-bind-html=\"contentExp()\"></div>\n" +
    "  </div>\n" +
    "</div>\n" +
    "");
}]);

angular.module("template/popover/popover-template.html", []).run(["$templateCache", function($templateCache) {
  $templateCache.put("template/popover/popover-template.html",
    "<div class=\"popover\"\n" +
    "  tooltip-animation-class=\"fade\"\n" +
    "  tooltip-classes\n" +
    "  ng-class=\"{ in: isOpen() }\">\n" +
    "  <div class=\"arrow\"></div>\n" +
    "\n" +
    "  <div class=\"popover-inner\">\n" +
    "      <h3 class=\"popover-title\" ng-bind=\"title\" ng-if=\"title\"></h3>\n" +
    "      <div class=\"popover-content\"\n" +
    "        tooltip-template-transclude=\"contentExp()\"\n" +
    "        tooltip-template-transclude-scope=\"originScope()\"></div>\n" +
    "  </div>\n" +
    "</div>\n" +
    "");
}]);

angular.module("template/popover/popover.html", []).run(["$templateCache", function($templateCache) {
  $templateCache.put("template/popover/popover.html",
    "<div class=\"popover\"\n" +
    "  tooltip-animation-class=\"fade\"\n" +
    "  tooltip-classes\n" +
    "  ng-class=\"{ in: isOpen() }\">\n" +
    "  <div class=\"arrow\"></div>\n" +
    "\n" +
    "  <div class=\"popover-inner\">\n" +
    "      <h3 class=\"popover-title\" ng-bind=\"title\" ng-if=\"title\"></h3>\n" +
    "      <div class=\"popover-content\" ng-bind=\"content\"></div>\n" +
    "  </div>\n" +
    "</div>\n" +
    "");
}]);

angular.module("template/progressbar/bar.html", []).run(["$templateCache", function($templateCache) {
  $templateCache.put("template/progressbar/bar.html",
    "<div class=\"progress-bar\" ng-class=\"type && 'progress-bar-' + type\" role=\"progressbar\" aria-valuenow=\"{{value}}\" aria-valuemin=\"0\" aria-valuemax=\"{{max}}\" ng-style=\"{width: (percent < 100 ? percent : 100) + '%'}\" aria-valuetext=\"{{percent | number:0}}%\" style=\"min-width: 0;\" ng-transclude></div>\n" +
    "");
}]);

angular.module("template/progressbar/progress.html", []).run(["$templateCache", function($templateCache) {
  $templateCache.put("template/progressbar/progress.html",
    "<div class=\"progress\" ng-transclude></div>");
}]);

angular.module("template/progressbar/progressbar.html", []).run(["$templateCache", function($templateCache) {
  $templateCache.put("template/progressbar/progressbar.html",
    "<div class=\"progress\">\n" +
    "  <div class=\"progress-bar\" ng-class=\"type && 'progress-bar-' + type\" role=\"progressbar\" aria-valuenow=\"{{value}}\" aria-valuemin=\"0\" aria-valuemax=\"{{max}}\" ng-style=\"{width: (percent < 100 ? percent : 100) + '%'}\" aria-valuetext=\"{{percent | number:0}}%\" style=\"min-width: 0;\" ng-transclude></div>\n" +
    "</div>\n" +
    "");
}]);

angular.module("template/rating/rating.html", []).run(["$templateCache", function($templateCache) {
  $templateCache.put("template/rating/rating.html",
    "<span ng-mouseleave=\"reset()\" ng-keydown=\"onKeydown($event)\" tabindex=\"0\" role=\"slider\" aria-valuemin=\"0\" aria-valuemax=\"{{range.length}}\" aria-valuenow=\"{{value}}\">\n" +
    "    <span ng-repeat-start=\"r in range track by $index\" class=\"sr-only\">({{ $index < value ? '*' : ' ' }})</span>\n" +
    "    <i ng-repeat-end ng-mouseenter=\"enter($index + 1)\" ng-click=\"rate($index + 1)\" class=\"glyphicon\" ng-class=\"$index < value && (r.stateOn || 'glyphicon-star') || (r.stateOff || 'glyphicon-star-empty')\" ng-attr-title=\"{{r.title}}\" ></i>\n" +
    "</span>\n" +
    "");
}]);

angular.module("template/tabs/tab.html", []).run(["$templateCache", function($templateCache) {
  $templateCache.put("template/tabs/tab.html",
    "<li ng-class=\"{active: active, disabled: disabled}\">\n" +
    "  <a href ng-click=\"select()\" tab-heading-transclude>{{heading}}</a>\n" +
    "</li>\n" +
    "");
}]);

angular.module("template/tabs/tabset.html", []).run(["$templateCache", function($templateCache) {
  $templateCache.put("template/tabs/tabset.html",
    "<div>\n" +
    "  <ul class=\"nav nav-{{type || 'tabs'}}\" ng-class=\"{'nav-stacked': vertical, 'nav-justified': justified}\" ng-transclude></ul>\n" +
    "  <div class=\"tab-content\">\n" +
    "    <div class=\"tab-pane\" \n" +
    "         ng-repeat=\"tab in tabs\" \n" +
    "         ng-class=\"{active: tab.active}\"\n" +
    "         tab-content-transclude=\"tab\">\n" +
    "    </div>\n" +
    "  </div>\n" +
    "</div>\n" +
    "");
}]);

angular.module("template/timepicker/timepicker.html", []).run(["$templateCache", function($templateCache) {
  $templateCache.put("template/timepicker/timepicker.html",
    "<table>\n" +
    "  <tbody>\n" +
    "    <tr class=\"text-center\" ng-show=\"::showSpinners\">\n" +
    "      <td><a ng-click=\"incrementHours()\" ng-class=\"{disabled: noIncrementHours()}\" class=\"btn btn-link\"><span class=\"glyphicon glyphicon-chevron-up\"></span></a></td>\n" +
    "      <td>&nbsp;</td>\n" +
    "      <td><a ng-click=\"incrementMinutes()\" ng-class=\"{disabled: noIncrementMinutes()}\" class=\"btn btn-link\"><span class=\"glyphicon glyphicon-chevron-up\"></span></a></td>\n" +
    "      <td ng-show=\"showMeridian\"></td>\n" +
    "    </tr>\n" +
    "    <tr>\n" +
    "      <td class=\"form-group\" ng-class=\"{'has-error': invalidHours}\">\n" +
    "        <input style=\"width:50px;\" type=\"text\" ng-model=\"hours\" ng-change=\"updateHours()\" class=\"form-control text-center\" ng-readonly=\"::readonlyInput\" maxlength=\"2\">\n" +
    "      </td>\n" +
    "      <td>:</td>\n" +
    "      <td class=\"form-group\" ng-class=\"{'has-error': invalidMinutes}\">\n" +
    "        <input style=\"width:50px;\" type=\"text\" ng-model=\"minutes\" ng-change=\"updateMinutes()\" class=\"form-control text-center\" ng-readonly=\"::readonlyInput\" maxlength=\"2\">\n" +
    "      </td>\n" +
    "      <td ng-show=\"showMeridian\"><button type=\"button\" ng-class=\"{disabled: noToggleMeridian()}\" class=\"btn btn-default text-center\" ng-click=\"toggleMeridian()\">{{meridian}}</button></td>\n" +
    "    </tr>\n" +
    "    <tr class=\"text-center\" ng-show=\"::showSpinners\">\n" +
    "      <td><a ng-click=\"decrementHours()\" ng-class=\"{disabled: noDecrementHours()}\" class=\"btn btn-link\"><span class=\"glyphicon glyphicon-chevron-down\"></span></a></td>\n" +
    "      <td>&nbsp;</td>\n" +
    "      <td><a ng-click=\"decrementMinutes()\" ng-class=\"{disabled: noDecrementMinutes()}\" class=\"btn btn-link\"><span class=\"glyphicon glyphicon-chevron-down\"></span></a></td>\n" +
    "      <td ng-show=\"showMeridian\"></td>\n" +
    "    </tr>\n" +
    "  </tbody>\n" +
    "</table>\n" +
    "");
}]);

angular.module("template/typeahead/typeahead-match.html", []).run(["$templateCache", function($templateCache) {
  $templateCache.put("template/typeahead/typeahead-match.html",
    "<a href tabindex=\"-1\" ng-bind-html=\"match.label | typeaheadHighlight:query\"></a>\n" +
    "");
}]);

angular.module("template/typeahead/typeahead-popup.html", []).run(["$templateCache", function($templateCache) {
  $templateCache.put("template/typeahead/typeahead-popup.html",
    "<ul class=\"dropdown-menu\" ng-show=\"isOpen() && !moveInProgress\" ng-style=\"{top: position().top+'px', left: position().left+'px'}\" style=\"display: block;\" role=\"listbox\" aria-hidden=\"{{!isOpen()}}\">\n" +
    "    <li ng-repeat=\"match in matches track by $index\" ng-class=\"{active: isActive($index) }\" ng-mouseenter=\"selectActive($index)\" ng-click=\"selectMatch($index)\" role=\"option\" id=\"{{::match.id}}\">\n" +
    "        <div typeahead-match index=\"$index\" match=\"match\" query=\"query\" template-url=\"templateUrl\"></div>\n" +
    "    </li>\n" +
    "</ul>\n" +
    "");
}]);
!angular.$$csp() && angular.element(document).find('head').prepend('<style type="text/css">.ng-animate.item:not(.left):not(.right){-webkit-transition:0s ease-in-out left;transition:0s ease-in-out left}</style>');
/**
 * @license
 * Lo-Dash 2.4.2 (Custom Build) lodash.com/license | Underscore.js 1.5.2 underscorejs.org/LICENSE
 * Build: `lodash modern -o ./dist/lodash.js`
 */
;(function(){function n(n,r,t){for(var e=(t||0)-1,u=n?n.length:0;++e<u;)if(n[e]===r)return e;return-1}function r(r,t){var e=typeof t;if(r=r.cache,"boolean"==e||null==t)return r[t]?0:-1;"number"!=e&&"string"!=e&&(e="object");var u="number"==e?t:m+t;return r=(r=r[e])&&r[u],"object"==e?r&&n(r,t)>-1?0:-1:r?0:-1}function t(n){var r=this.cache,t=typeof n;if("boolean"==t||null==n)r[n]=!0;else{"number"!=t&&"string"!=t&&(t="object");var e="number"==t?n:m+n,u=r[t]||(r[t]={});"object"==t?(u[e]||(u[e]=[])).push(n):u[e]=!0;

}}function e(n){return n.charCodeAt(0)}function u(n,r){for(var t=n.criteria,e=r.criteria,u=-1,o=t.length;++u<o;){var a=t[u],i=e[u];if(a!==i){if(a>i||"undefined"==typeof a)return 1;if(a<i||"undefined"==typeof i)return-1}}return n.index-r.index}function o(n){var r=-1,e=n.length,u=n[0],o=n[e/2|0],a=n[e-1];if(u&&"object"==typeof u&&o&&"object"==typeof o&&a&&"object"==typeof a)return!1;var i=f();i["false"]=i["null"]=i["true"]=i.undefined=!1;var l=f();for(l.array=n,l.cache=i,l.push=t;++r<e;)l.push(n[r]);

return l}function a(n){return"\\"+G[n]}function i(){return h.pop()||[]}function f(){return g.pop()||{array:null,cache:null,criteria:null,"false":!1,index:0,"null":!1,number:null,object:null,push:null,string:null,"true":!1,undefined:!1,value:null}}function l(n){n.length=0,h.length<_&&h.push(n)}function c(n){var r=n.cache;r&&c(r),n.array=n.cache=n.criteria=n.object=n.number=n.string=n.value=null,g.length<_&&g.push(n)}function p(n,r,t){r||(r=0),"undefined"==typeof t&&(t=n?n.length:0);for(var e=-1,u=t-r||0,o=Array(u<0?0:u);++e<u;)o[e]=n[r+e];

return o}function s(t){function h(n){return n&&"object"==typeof n&&!Yt(n)&&Tt.call(n,"__wrapped__")?n:new g(n)}function g(n,r){this.__chain__=!!r,this.__wrapped__=n}function _(n){function r(){if(e){var n=p(e);$t.apply(n,arguments)}if(this instanceof r){var o=J(t.prototype),a=t.apply(o,n||arguments);return Sn(a)?a:o}return t.apply(u,n||arguments)}var t=n[0],e=n[2],u=n[4];return Xt(r,n),r}function G(n,r,t,e,u){if(t){var o=t(n);if("undefined"!=typeof o)return o}var a=Sn(n);if(!a)return n;var f=Nt.call(n);

if(!K[f])return n;var c=Jt[f];switch(f){case F:case B:return new c(+n);case q:case P:return new c(n);case L:return o=c(n.source,C.exec(n)),o.lastIndex=n.lastIndex,o}var s=Yt(n);if(r){var v=!e;e||(e=i()),u||(u=i());for(var h=e.length;h--;)if(e[h]==n)return u[h];o=s?c(n.length):{}}else o=s?p(n):oe({},n);return s&&(Tt.call(n,"index")&&(o.index=n.index),Tt.call(n,"input")&&(o.input=n.input)),r?(e.push(n),u.push(o),(s?Xn:fe)(n,function(n,a){o[a]=G(n,r,t,e,u)}),v&&(l(e),l(u)),o):o}function J(n,r){return Sn(n)?zt(n):{};

}function Q(n,r,t){if("function"!=typeof n)return Yr;if("undefined"==typeof r||!("prototype"in n))return n;var e=n.__bindData__;if("undefined"==typeof e&&(Qt.funcNames&&(e=!n.name),e=e||!Qt.funcDecomp,!e)){var u=At.call(n);Qt.funcNames||(e=!O.test(u)),e||(e=I.test(u),Xt(n,e))}if(e===!1||e!==!0&&1&e[1])return n;switch(t){case 1:return function(t){return n.call(r,t)};case 2:return function(t,e){return n.call(r,t,e)};case 3:return function(t,e,u){return n.call(r,t,e,u)};case 4:return function(t,e,u,o){
return n.call(r,t,e,u,o)}}return $r(n,r)}function X(n){function r(){var n=f?a:this;if(u){var h=p(u);$t.apply(h,arguments)}if((o||c)&&(h||(h=p(arguments)),o&&$t.apply(h,o),c&&h.length<i))return e|=16,X([t,s?e:-4&e,h,null,a,i]);if(h||(h=arguments),l&&(t=n[v]),this instanceof r){n=J(t.prototype);var g=t.apply(n,h);return Sn(g)?g:n}return t.apply(n,h)}var t=n[0],e=n[1],u=n[2],o=n[3],a=n[4],i=n[5],f=1&e,l=2&e,c=4&e,s=8&e,v=t;return Xt(r,n),r}function Y(t,e){var u=-1,a=ln(),i=t?t.length:0,f=i>=b&&a===n,l=[];

if(f){var p=o(e);p?(a=r,e=p):f=!1}for(;++u<i;){var s=t[u];a(e,s)<0&&l.push(s)}return f&&c(e),l}function nn(n,r,t,e){for(var u=(e||0)-1,o=n?n.length:0,a=[];++u<o;){var i=n[u];if(i&&"object"==typeof i&&"number"==typeof i.length&&(Yt(i)||vn(i))){r||(i=nn(i,r,t));var f=-1,l=i.length,c=a.length;for(a.length+=l;++f<l;)a[c++]=i[f]}else t||a.push(i)}return a}function rn(n,r,t,e,u,o){if(t){var a=t(n,r);if("undefined"!=typeof a)return!!a}if(n===r)return 0!==n||1/n==1/r;var f=typeof n,c=typeof r;if(n===n&&(!n||!V[f])&&(!r||!V[c]))return!1;

if(null==n||null==r)return n===r;var p=Nt.call(n),s=Nt.call(r);if(p==T&&(p=z),s==T&&(s=z),p!=s)return!1;switch(p){case F:case B:return+n==+r;case q:return n!=+n?r!=+r:0==n?1/n==1/r:n==+r;case L:case P:return n==jt(r)}var h=p==$;if(!h){var g=Tt.call(n,"__wrapped__"),y=Tt.call(r,"__wrapped__");if(g||y)return rn(g?n.__wrapped__:n,y?r.__wrapped__:r,t,e,u,o);if(p!=z)return!1;var m=n.constructor,b=r.constructor;if(m!=b&&!(In(m)&&m instanceof m&&In(b)&&b instanceof b)&&"constructor"in n&&"constructor"in r)return!1;

}var _=!u;u||(u=i()),o||(o=i());for(var d=u.length;d--;)if(u[d]==n)return o[d]==r;var w=0;if(a=!0,u.push(n),o.push(r),h){if(d=n.length,w=r.length,a=w==d,a||e)for(;w--;){var j=d,k=r[w];if(e)for(;j--&&!(a=rn(n[j],k,t,e,u,o)););else if(!(a=rn(n[w],k,t,e,u,o)))break}}else ie(r,function(r,i,f){return Tt.call(f,i)?(w++,a=Tt.call(n,i)&&rn(n[i],r,t,e,u,o)):v}),a&&!e&&ie(n,function(n,r,t){return Tt.call(t,r)?a=--w>-1:v});return u.pop(),o.pop(),_&&(l(u),l(o)),a}function tn(n,r,t,e,u){(Yt(r)?Xn:fe)(r,function(r,o){
var a,i,f=r,l=n[o];if(r&&((i=Yt(r))||le(r))){for(var c=e.length;c--;)if(a=e[c]==r){l=u[c];break}if(!a){var p;t&&(f=t(l,r),(p="undefined"!=typeof f)&&(l=f)),p||(l=i?Yt(l)?l:[]:le(l)?l:{}),e.push(r),u.push(l),p||tn(l,r,t,e,u)}}else t&&(f=t(l,r),"undefined"==typeof f&&(f=r)),"undefined"!=typeof f&&(l=f);n[o]=l})}function en(n,r){return n+St(Ht()*(r-n+1))}function un(t,e,u){var a=-1,f=ln(),p=t?t.length:0,s=[],v=!e&&p>=b&&f===n,h=u||v?i():s;if(v){var g=o(h);f=r,h=g}for(;++a<p;){var y=t[a],m=u?u(y,a,t):y;

(e?!a||h[h.length-1]!==m:f(h,m)<0)&&((u||v)&&h.push(m),s.push(y))}return v?(l(h.array),c(h)):u&&l(h),s}function on(n){return function(r,t,e){var u={};t=h.createCallback(t,e,3);var o=-1,a=r?r.length:0;if("number"==typeof a)for(;++o<a;){var i=r[o];n(u,i,t(i,o,r),r)}else fe(r,function(r,e,o){n(u,r,t(r,e,o),o)});return u}}function an(n,r,t,e,u,o){var a=1&r,i=2&r,f=4&r,l=16&r,c=32&r;if(!i&&!In(n))throw new kt;l&&!t.length&&(r&=-17,l=t=!1),c&&!e.length&&(r&=-33,c=e=!1);var s=n&&n.__bindData__;if(s&&s!==!0)return s=p(s),
s[2]&&(s[2]=p(s[2])),s[3]&&(s[3]=p(s[3])),!a||1&s[1]||(s[4]=u),!a&&1&s[1]&&(r|=8),!f||4&s[1]||(s[5]=o),l&&$t.apply(s[2]||(s[2]=[]),t),c&&Wt.apply(s[3]||(s[3]=[]),e),s[1]|=r,an.apply(null,s);var v=1==r||17===r?_:X;return v([n,r,t,e,u,o])}function fn(n){return re[n]}function ln(){var r=(r=h.indexOf)===br?n:r;return r}function cn(n){return"function"==typeof n&&Rt.test(n)}function pn(n){var r,t;return n&&Nt.call(n)==z&&(r=n.constructor,!In(r)||r instanceof r)?(ie(n,function(n,r){t=r}),"undefined"==typeof t||Tt.call(n,t)):!1;

}function sn(n){return te[n]}function vn(n){return n&&"object"==typeof n&&"number"==typeof n.length&&Nt.call(n)==T||!1}function hn(n,r,t,e){return"boolean"!=typeof r&&null!=r&&(e=t,t=r,r=!1),G(n,r,"function"==typeof t&&Q(t,e,1))}function gn(n,r,t){return G(n,!0,"function"==typeof r&&Q(r,t,1))}function yn(n,r){var t=J(n);return r?oe(t,r):t}function mn(n,r,t){var e;return r=h.createCallback(r,t,3),fe(n,function(n,t,u){return r(n,t,u)?(e=t,!1):v}),e}function bn(n,r,t){var e;return r=h.createCallback(r,t,3),
dn(n,function(n,t,u){return r(n,t,u)?(e=t,!1):v}),e}function _n(n,r,t){var e=[];ie(n,function(n,r){e.push(r,n)});var u=e.length;for(r=Q(r,t,3);u--&&r(e[u--],e[u],n)!==!1;);return n}function dn(n,r,t){var e=ne(n),u=e.length;for(r=Q(r,t,3);u--;){var o=e[u];if(r(n[o],o,n)===!1)break}return n}function wn(n){var r=[];return ie(n,function(n,t){In(n)&&r.push(t)}),r.sort()}function jn(n,r){return n?Tt.call(n,r):!1}function kn(n){for(var r=-1,t=ne(n),e=t.length,u={};++r<e;){var o=t[r];u[n[o]]=o}return u}function xn(n){
return n===!0||n===!1||n&&"object"==typeof n&&Nt.call(n)==F||!1}function Cn(n){return n&&"object"==typeof n&&Nt.call(n)==B||!1}function On(n){return n&&1===n.nodeType||!1}function Nn(n){var r=!0;if(!n)return r;var t=Nt.call(n),e=n.length;return t==$||t==P||t==T||t==z&&"number"==typeof e&&In(n.splice)?!e:(fe(n,function(){return r=!1}),r)}function Rn(n,r,t,e){return rn(n,r,"function"==typeof t&&Q(t,e,2))}function En(n){return Pt(n)&&!Kt(parseFloat(n))}function In(n){return"function"==typeof n}function Sn(n){
return!(!n||!V[typeof n])}function An(n){return Tn(n)&&n!=+n}function Dn(n){return null===n}function Tn(n){return"number"==typeof n||n&&"object"==typeof n&&Nt.call(n)==q||!1}function $n(n){return n&&"object"==typeof n&&Nt.call(n)==L||!1}function Fn(n){return"string"==typeof n||n&&"object"==typeof n&&Nt.call(n)==P||!1}function Bn(n){return"undefined"==typeof n}function Wn(n,r,t){var e={};return r=h.createCallback(r,t,3),fe(n,function(n,t,u){e[t]=r(n,t,u)}),e}function qn(n){var r=arguments,t=2;if(!Sn(n))return n;

if("number"!=typeof r[2]&&(t=r.length),t>3&&"function"==typeof r[t-2])var e=Q(r[--t-1],r[t--],2);else t>2&&"function"==typeof r[t-1]&&(e=r[--t]);for(var u=p(arguments,1,t),o=-1,a=i(),f=i();++o<t;)tn(n,u[o],e,a,f);return l(a),l(f),n}function zn(n,r,t){var e={};if("function"!=typeof r){var u=[];ie(n,function(n,r){u.push(r)}),u=Y(u,nn(arguments,!0,!1,1));for(var o=-1,a=u.length;++o<a;){var i=u[o];e[i]=n[i]}}else r=h.createCallback(r,t,3),ie(n,function(n,t,u){r(n,t,u)||(e[t]=n)});return e}function Ln(n){
for(var r=-1,t=ne(n),e=t.length,u=ht(e);++r<e;){var o=t[r];u[r]=[o,n[o]]}return u}function Pn(n,r,t){var e={};if("function"!=typeof r)for(var u=-1,o=nn(arguments,!0,!1,1),a=Sn(n)?o.length:0;++u<a;){var i=o[u];i in n&&(e[i]=n[i])}else r=h.createCallback(r,t,3),ie(n,function(n,t,u){r(n,t,u)&&(e[t]=n)});return e}function Kn(n,r,t,e){var u=Yt(n);if(null==t)if(u)t=[];else{var o=n&&n.constructor,a=o&&o.prototype;t=J(a)}return r&&(r=h.createCallback(r,e,4),(u?Xn:fe)(n,function(n,e,u){return r(t,n,e,u)})),
t}function Un(n){for(var r=-1,t=ne(n),e=t.length,u=ht(e);++r<e;)u[r]=n[t[r]];return u}function Mn(n){for(var r=arguments,t=-1,e=nn(r,!0,!1,1),u=r[2]&&r[2][r[1]]===n?1:e.length,o=ht(u);++t<u;)o[t]=n[e[t]];return o}function Vn(n,r,t){var e=-1,u=ln(),o=n?n.length:0,a=!1;return t=(t<0?Mt(0,o+t):t)||0,Yt(n)?a=u(n,r,t)>-1:"number"==typeof o?a=(Fn(n)?n.indexOf(r,t):u(n,r,t))>-1:fe(n,function(n){return++e<t?v:!(a=n===r)}),a}function Gn(n,r,t){var e=!0;r=h.createCallback(r,t,3);var u=-1,o=n?n.length:0;if("number"==typeof o)for(;++u<o&&(e=!!r(n[u],u,n)););else fe(n,function(n,t,u){
return e=!!r(n,t,u)});return e}function Hn(n,r,t){var e=[];r=h.createCallback(r,t,3);var u=-1,o=n?n.length:0;if("number"==typeof o)for(;++u<o;){var a=n[u];r(a,u,n)&&e.push(a)}else fe(n,function(n,t,u){r(n,t,u)&&e.push(n)});return e}function Jn(n,r,t){r=h.createCallback(r,t,3);var e=-1,u=n?n.length:0;if("number"!=typeof u){var o;return fe(n,function(n,t,e){return r(n,t,e)?(o=n,!1):v}),o}for(;++e<u;){var a=n[e];if(r(a,e,n))return a}}function Qn(n,r,t){var e;return r=h.createCallback(r,t,3),Yn(n,function(n,t,u){
return r(n,t,u)?(e=n,!1):v}),e}function Xn(n,r,t){var e=-1,u=n?n.length:0;if(r=r&&"undefined"==typeof t?r:Q(r,t,3),"number"==typeof u)for(;++e<u&&r(n[e],e,n)!==!1;);else fe(n,r);return n}function Yn(n,r,t){var e=n?n.length:0;if(r=r&&"undefined"==typeof t?r:Q(r,t,3),"number"==typeof e)for(;e--&&r(n[e],e,n)!==!1;);else{var u=ne(n);e=u.length,fe(n,function(n,t,o){return t=u?u[--e]:--e,r(o[t],t,o)})}return n}function Zn(n,r){var t=p(arguments,2),e=-1,u="function"==typeof r,o=n?n.length:0,a=ht("number"==typeof o?o:0);

return Xn(n,function(n){a[++e]=(u?r:n[r]).apply(n,t)}),a}function nr(n,r,t){var e=-1,u=n?n.length:0;if(r=h.createCallback(r,t,3),"number"==typeof u)for(var o=ht(u);++e<u;)o[e]=r(n[e],e,n);else o=[],fe(n,function(n,t,u){o[++e]=r(n,t,u)});return o}function rr(n,r,t){var u=-(1/0),o=u;if("function"!=typeof r&&t&&t[r]===n&&(r=null),null==r&&Yt(n))for(var a=-1,i=n.length;++a<i;){var f=n[a];f>o&&(o=f)}else r=null==r&&Fn(n)?e:h.createCallback(r,t,3),Xn(n,function(n,t,e){var a=r(n,t,e);a>u&&(u=a,o=n)});return o;

}function tr(n,r,t){var u=1/0,o=u;if("function"!=typeof r&&t&&t[r]===n&&(r=null),null==r&&Yt(n))for(var a=-1,i=n.length;++a<i;){var f=n[a];f<o&&(o=f)}else r=null==r&&Fn(n)?e:h.createCallback(r,t,3),Xn(n,function(n,t,e){var a=r(n,t,e);a<u&&(u=a,o=n)});return o}function er(n,r,t,e){if(!n)return t;var u=arguments.length<3;r=h.createCallback(r,e,4);var o=-1,a=n.length;if("number"==typeof a)for(u&&(t=n[++o]);++o<a;)t=r(t,n[o],o,n);else fe(n,function(n,e,o){t=u?(u=!1,n):r(t,n,e,o)});return t}function ur(n,r,t,e){
var u=arguments.length<3;return r=h.createCallback(r,e,4),Yn(n,function(n,e,o){t=u?(u=!1,n):r(t,n,e,o)}),t}function or(n,r,t){return r=h.createCallback(r,t,3),Hn(n,function(n,t,e){return!r(n,t,e)})}function ar(n,r,t){if(n&&"number"!=typeof n.length&&(n=Un(n)),null==r||t)return n?n[en(0,n.length-1)]:v;var e=ir(n);return e.length=Vt(Mt(0,r),e.length),e}function ir(n){var r=-1,t=n?n.length:0,e=ht("number"==typeof t?t:0);return Xn(n,function(n){var t=en(0,++r);e[r]=e[t],e[t]=n}),e}function fr(n){var r=n?n.length:0;

return"number"==typeof r?r:ne(n).length}function lr(n,r,t){var e;r=h.createCallback(r,t,3);var u=-1,o=n?n.length:0;if("number"==typeof o)for(;++u<o&&!(e=r(n[u],u,n)););else fe(n,function(n,t,u){return!(e=r(n,t,u))});return!!e}function cr(n,r,t){var e=-1,o=Yt(r),a=n?n.length:0,p=ht("number"==typeof a?a:0);for(o||(r=h.createCallback(r,t,3)),Xn(n,function(n,t,u){var a=p[++e]=f();o?a.criteria=nr(r,function(r){return n[r]}):(a.criteria=i())[0]=r(n,t,u),a.index=e,a.value=n}),a=p.length,p.sort(u);a--;){
var s=p[a];p[a]=s.value,o||l(s.criteria),c(s)}return p}function pr(n){return n&&"number"==typeof n.length?p(n):Un(n)}function sr(n){for(var r=-1,t=n?n.length:0,e=[];++r<t;){var u=n[r];u&&e.push(u)}return e}function vr(n){return Y(n,nn(arguments,!0,!0,1))}function hr(n,r,t){var e=-1,u=n?n.length:0;for(r=h.createCallback(r,t,3);++e<u;)if(r(n[e],e,n))return e;return-1}function gr(n,r,t){var e=n?n.length:0;for(r=h.createCallback(r,t,3);e--;)if(r(n[e],e,n))return e;return-1}function yr(n,r,t){var e=0,u=n?n.length:0;

if("number"!=typeof r&&null!=r){var o=-1;for(r=h.createCallback(r,t,3);++o<u&&r(n[o],o,n);)e++}else if(e=r,null==e||t)return n?n[0]:v;return p(n,0,Vt(Mt(0,e),u))}function mr(n,r,t,e){return"boolean"!=typeof r&&null!=r&&(e=t,t="function"!=typeof r&&e&&e[r]===n?null:r,r=!1),null!=t&&(n=nr(n,t,e)),nn(n,r)}function br(r,t,e){if("number"==typeof e){var u=r?r.length:0;e=e<0?Mt(0,u+e):e||0}else if(e){var o=Nr(r,t);return r[o]===t?o:-1}return n(r,t,e)}function _r(n,r,t){var e=0,u=n?n.length:0;if("number"!=typeof r&&null!=r){
var o=u;for(r=h.createCallback(r,t,3);o--&&r(n[o],o,n);)e++}else e=null==r||t?1:r||e;return p(n,0,Vt(Mt(0,u-e),u))}function dr(){for(var t=[],e=-1,u=arguments.length,a=i(),f=ln(),p=f===n,s=i();++e<u;){var v=arguments[e];(Yt(v)||vn(v))&&(t.push(v),a.push(p&&v.length>=b&&o(e?t[e]:s)))}var h=t[0],g=-1,y=h?h.length:0,m=[];n:for(;++g<y;){var _=a[0];if(v=h[g],(_?r(_,v):f(s,v))<0){for(e=u,(_||s).push(v);--e;)if(_=a[e],(_?r(_,v):f(t[e],v))<0)continue n;m.push(v)}}for(;u--;)_=a[u],_&&c(_);return l(a),l(s),
m}function wr(n,r,t){var e=0,u=n?n.length:0;if("number"!=typeof r&&null!=r){var o=u;for(r=h.createCallback(r,t,3);o--&&r(n[o],o,n);)e++}else if(e=r,null==e||t)return n?n[u-1]:v;return p(n,Mt(0,u-e))}function jr(n,r,t){var e=n?n.length:0;for("number"==typeof t&&(e=(t<0?Mt(0,e+t):Vt(t,e-1))+1);e--;)if(n[e]===r)return e;return-1}function kr(n){for(var r=arguments,t=0,e=r.length,u=n?n.length:0;++t<e;)for(var o=-1,a=r[t];++o<u;)n[o]===a&&(Bt.call(n,o--,1),u--);return n}function xr(n,r,t){n=+n||0,t="number"==typeof t?t:+t||1,
null==r&&(r=n,n=0);for(var e=-1,u=Mt(0,Et((r-n)/(t||1))),o=ht(u);++e<u;)o[e]=n,n+=t;return o}function Cr(n,r,t){var e=-1,u=n?n.length:0,o=[];for(r=h.createCallback(r,t,3);++e<u;){var a=n[e];r(a,e,n)&&(o.push(a),Bt.call(n,e--,1),u--)}return o}function Or(n,r,t){if("number"!=typeof r&&null!=r){var e=0,u=-1,o=n?n.length:0;for(r=h.createCallback(r,t,3);++u<o&&r(n[u],u,n);)e++}else e=null==r||t?1:Mt(0,r);return p(n,e)}function Nr(n,r,t,e){var u=0,o=n?n.length:u;for(t=t?h.createCallback(t,e,1):Yr,r=t(r);u<o;){
var a=u+o>>>1;t(n[a])<r?u=a+1:o=a}return u}function Rr(){return un(nn(arguments,!0,!0))}function Er(n,r,t,e){return"boolean"!=typeof r&&null!=r&&(e=t,t="function"!=typeof r&&e&&e[r]===n?null:r,r=!1),null!=t&&(t=h.createCallback(t,e,3)),un(n,r,t)}function Ir(n){return Y(n,p(arguments,1))}function Sr(){for(var n=-1,r=arguments.length;++n<r;){var t=arguments[n];if(Yt(t)||vn(t))var e=e?un(Y(e,t).concat(Y(t,e))):t}return e||[]}function Ar(){for(var n=arguments.length>1?arguments:arguments[0],r=-1,t=n?rr(ve(n,"length")):0,e=ht(t<0?0:t);++r<t;)e[r]=ve(n,r);

return e}function Dr(n,r){var t=-1,e=n?n.length:0,u={};for(r||!e||Yt(n[0])||(r=[]);++t<e;){var o=n[t];r?u[o]=r[t]:o&&(u[o[0]]=o[1])}return u}function Tr(n,r){if(!In(r))throw new kt;return function(){return--n<1?r.apply(this,arguments):v}}function $r(n,r){return arguments.length>2?an(n,17,p(arguments,2),null,r):an(n,1,null,null,r)}function Fr(n){for(var r=arguments.length>1?nn(arguments,!0,!1,1):wn(n),t=-1,e=r.length;++t<e;){var u=r[t];n[u]=an(n[u],1,null,null,n)}return n}function Br(n,r){return arguments.length>2?an(r,19,p(arguments,2),null,n):an(r,3,null,null,n);

}function Wr(){for(var n=arguments,r=n.length;r--;)if(!In(n[r]))throw new kt;return function(){for(var r=arguments,t=n.length;t--;)r=[n[t].apply(this,r)];return r[0]}}function qr(n,r){return r="number"==typeof r?r:+r||n.length,an(n,4,null,null,null,r)}function zr(n,r,t){var e,u,o,a,i,f,l,c=0,p=!1,s=!0;if(!In(n))throw new kt;if(r=Mt(0,r)||0,t===!0){var h=!0;s=!1}else Sn(t)&&(h=t.leading,p="maxWait"in t&&(Mt(r,t.maxWait)||0),s="trailing"in t?t.trailing:s);var g=function(){var t=r-(ge()-a);if(t>0)f=Ft(g,t);
else{u&&It(u);var p=l;u=f=l=v,p&&(c=ge(),o=n.apply(i,e),f||u||(e=i=null))}},y=function(){f&&It(f),u=f=l=v,(s||p!==r)&&(c=ge(),o=n.apply(i,e),f||u||(e=i=null))};return function(){if(e=arguments,a=ge(),i=this,l=s&&(f||!h),p===!1)var t=h&&!f;else{u||h||(c=a);var v=p-(a-c),m=v<=0;m?(u&&(u=It(u)),c=a,o=n.apply(i,e)):u||(u=Ft(y,v))}return m&&f?f=It(f):f||r===p||(f=Ft(g,r)),t&&(m=!0,o=n.apply(i,e)),!m||f||u||(e=i=null),o}}function Lr(n){if(!In(n))throw new kt;var r=p(arguments,1);return Ft(function(){n.apply(v,r);

},1)}function Pr(n,r){if(!In(n))throw new kt;var t=p(arguments,2);return Ft(function(){n.apply(v,t)},r)}function Kr(n,r){if(!In(n))throw new kt;var t=function(){var e=t.cache,u=r?r.apply(this,arguments):m+arguments[0];return Tt.call(e,u)?e[u]:e[u]=n.apply(this,arguments)};return t.cache={},t}function Ur(n){var r,t;if(!In(n))throw new kt;return function(){return r?t:(r=!0,t=n.apply(this,arguments),n=null,t)}}function Mr(n){return an(n,16,p(arguments,1))}function Vr(n){return an(n,32,null,p(arguments,1));

}function Gr(n,r,t){var e=!0,u=!0;if(!In(n))throw new kt;return t===!1?e=!1:Sn(t)&&(e="leading"in t?t.leading:e,u="trailing"in t?t.trailing:u),U.leading=e,U.maxWait=r,U.trailing=u,zr(n,r,U)}function Hr(n,r){return an(r,16,[n])}function Jr(n){return function(){return n}}function Qr(n,r,t){var e=typeof n;if(null==n||"function"==e)return Q(n,r,t);if("object"!=e)return tt(n);var u=ne(n),o=u[0],a=n[o];return 1!=u.length||a!==a||Sn(a)?function(r){for(var t=u.length,e=!1;t--&&(e=rn(r[u[t]],n[u[t]],null,!0)););
return e}:function(n){var r=n[o];return a===r&&(0!==a||1/a==1/r)}}function Xr(n){return null==n?"":jt(n).replace(ue,fn)}function Yr(n){return n}function Zr(n,r,t){var e=!0,u=r&&wn(r);r&&(t||u.length)||(null==t&&(t=r),o=g,r=n,n=h,u=wn(r)),t===!1?e=!1:Sn(t)&&"chain"in t&&(e=t.chain);var o=n,a=In(o);Xn(u,function(t){var u=n[t]=r[t];a&&(o.prototype[t]=function(){var r=this.__chain__,t=this.__wrapped__,a=[t];$t.apply(a,arguments);var i=u.apply(n,a);if(e||r){if(t===i&&Sn(i))return this;i=new o(i),i.__chain__=r;

}return i})})}function nt(){return t._=Ot,this}function rt(){}function tt(n){return function(r){return r[n]}}function et(n,r,t){var e=null==n,u=null==r;if(null==t&&("boolean"==typeof n&&u?(t=n,n=1):u||"boolean"!=typeof r||(t=r,u=!0)),e&&u&&(r=1),n=+n||0,u?(r=n,n=0):r=+r||0,t||n%1||r%1){var o=Ht();return Vt(n+o*(r-n+parseFloat("1e-"+((o+"").length-1))),r)}return en(n,r)}function ut(n,r){if(n){var t=n[r];return In(t)?n[r]():t}}function ot(n,r,t){var e=h.templateSettings;n=jt(n||""),t=ae({},t,e);var u,o=ae({},t.imports,e.imports),i=ne(o),f=Un(o),l=0,c=t.interpolate||E,p="__p += '",s=wt((t.escape||E).source+"|"+c.source+"|"+(c===N?x:E).source+"|"+(t.evaluate||E).source+"|$","g");

n.replace(s,function(r,t,e,o,i,f){return e||(e=o),p+=n.slice(l,f).replace(S,a),t&&(p+="' +\n__e("+t+") +\n'"),i&&(u=!0,p+="';\n"+i+";\n__p += '"),e&&(p+="' +\n((__t = ("+e+")) == null ? '' : __t) +\n'"),l=f+r.length,r}),p+="';\n";var g=t.variable,y=g;y||(g="obj",p="with ("+g+") {\n"+p+"\n}\n"),p=(u?p.replace(w,""):p).replace(j,"$1").replace(k,"$1;"),p="function("+g+") {\n"+(y?"":g+" || ("+g+" = {});\n")+"var __t, __p = '', __e = _.escape"+(u?", __j = Array.prototype.join;\nfunction print() { __p += __j.call(arguments, '') }\n":";\n")+p+"return __p\n}";

var m="\n/*\n//# sourceURL="+(t.sourceURL||"/lodash/template/source["+D++ +"]")+"\n*/";try{var b=mt(i,"return "+p+m).apply(v,f)}catch(_){throw _.source=p,_}return r?b(r):(b.source=p,b)}function at(n,r,t){n=(n=+n)>-1?n:0;var e=-1,u=ht(n);for(r=Q(r,t,1);++e<n;)u[e]=r(e);return u}function it(n){return null==n?"":jt(n).replace(ee,sn)}function ft(n){var r=++y;return jt(null==n?"":n)+r}function lt(n){return n=new g(n),n.__chain__=!0,n}function ct(n,r){return r(n),n}function pt(){return this.__chain__=!0,
this}function st(){return jt(this.__wrapped__)}function vt(){return this.__wrapped__}t=t?Z.defaults(H.Object(),t,Z.pick(H,A)):H;var ht=t.Array,gt=t.Boolean,yt=t.Date,mt=t.Function,bt=t.Math,_t=t.Number,dt=t.Object,wt=t.RegExp,jt=t.String,kt=t.TypeError,xt=[],Ct=dt.prototype,Ot=t._,Nt=Ct.toString,Rt=wt("^"+jt(Nt).replace(/[.*+?^${}()|[\]\\]/g,"\\$&").replace(/toString| for [^\]]+/g,".*?")+"$"),Et=bt.ceil,It=t.clearTimeout,St=bt.floor,At=mt.prototype.toString,Dt=cn(Dt=dt.getPrototypeOf)&&Dt,Tt=Ct.hasOwnProperty,$t=xt.push,Ft=t.setTimeout,Bt=xt.splice,Wt=xt.unshift,qt=function(){
try{var n={},r=cn(r=dt.defineProperty)&&r,t=r(n,n,n)&&r}catch(e){}return t}(),zt=cn(zt=dt.create)&&zt,Lt=cn(Lt=ht.isArray)&&Lt,Pt=t.isFinite,Kt=t.isNaN,Ut=cn(Ut=dt.keys)&&Ut,Mt=bt.max,Vt=bt.min,Gt=t.parseInt,Ht=bt.random,Jt={};Jt[$]=ht,Jt[F]=gt,Jt[B]=yt,Jt[W]=mt,Jt[z]=dt,Jt[q]=_t,Jt[L]=wt,Jt[P]=jt,g.prototype=h.prototype;var Qt=h.support={};Qt.funcDecomp=!cn(t.WinRTError)&&I.test(s),Qt.funcNames="string"==typeof mt.name,h.templateSettings={escape:/<%-([\s\S]+?)%>/g,evaluate:/<%([\s\S]+?)%>/g,interpolate:N,
variable:"",imports:{_:h}},zt||(J=function(){function n(){}return function(r){if(Sn(r)){n.prototype=r;var e=new n;n.prototype=null}return e||t.Object()}}());var Xt=qt?function(n,r){M.value=r,qt(n,"__bindData__",M),M.value=null}:rt,Yt=Lt||function(n){return n&&"object"==typeof n&&"number"==typeof n.length&&Nt.call(n)==$||!1},Zt=function(n){var r,t=n,e=[];if(!t)return e;if(!V[typeof n])return e;for(r in t)Tt.call(t,r)&&e.push(r);return e},ne=Ut?function(n){return Sn(n)?Ut(n):[]}:Zt,re={"&":"&amp;",
"<":"&lt;",">":"&gt;",'"':"&quot;","'":"&#39;"},te=kn(re),ee=wt("("+ne(te).join("|")+")","g"),ue=wt("["+ne(re).join("")+"]","g"),oe=function(n,r,t){var e,u=n,o=u;if(!u)return o;var a=arguments,i=0,f="number"==typeof t?2:a.length;if(f>3&&"function"==typeof a[f-2])var l=Q(a[--f-1],a[f--],2);else f>2&&"function"==typeof a[f-1]&&(l=a[--f]);for(;++i<f;)if(u=a[i],u&&V[typeof u])for(var c=-1,p=V[typeof u]&&ne(u),s=p?p.length:0;++c<s;)e=p[c],o[e]=l?l(o[e],u[e]):u[e];return o},ae=function(n,r,t){var e,u=n,o=u;

if(!u)return o;for(var a=arguments,i=0,f="number"==typeof t?2:a.length;++i<f;)if(u=a[i],u&&V[typeof u])for(var l=-1,c=V[typeof u]&&ne(u),p=c?c.length:0;++l<p;)e=c[l],"undefined"==typeof o[e]&&(o[e]=u[e]);return o},ie=function(n,r,t){var e,u=n,o=u;if(!u)return o;if(!V[typeof u])return o;r=r&&"undefined"==typeof t?r:Q(r,t,3);for(e in u)if(r(u[e],e,n)===!1)return o;return o},fe=function(n,r,t){var e,u=n,o=u;if(!u)return o;if(!V[typeof u])return o;r=r&&"undefined"==typeof t?r:Q(r,t,3);for(var a=-1,i=V[typeof u]&&ne(u),f=i?i.length:0;++a<f;)if(e=i[a],
r(u[e],e,n)===!1)return o;return o},le=Dt?function(n){if(!n||Nt.call(n)!=z)return!1;var r=n.valueOf,t=cn(r)&&(t=Dt(r))&&Dt(t);return t?n==t||Dt(n)==t:pn(n)}:pn,ce=on(function(n,r,t){Tt.call(n,t)?n[t]++:n[t]=1}),pe=on(function(n,r,t){(Tt.call(n,t)?n[t]:n[t]=[]).push(r)}),se=on(function(n,r,t){n[t]=r}),ve=nr,he=Hn,ge=cn(ge=yt.now)&&ge||function(){return(new yt).getTime()},ye=8==Gt(d+"08")?Gt:function(n,r){return Gt(Fn(n)?n.replace(R,""):n,r||0)};return h.after=Tr,h.assign=oe,h.at=Mn,h.bind=$r,h.bindAll=Fr,
h.bindKey=Br,h.chain=lt,h.compact=sr,h.compose=Wr,h.constant=Jr,h.countBy=ce,h.create=yn,h.createCallback=Qr,h.curry=qr,h.debounce=zr,h.defaults=ae,h.defer=Lr,h.delay=Pr,h.difference=vr,h.filter=Hn,h.flatten=mr,h.forEach=Xn,h.forEachRight=Yn,h.forIn=ie,h.forInRight=_n,h.forOwn=fe,h.forOwnRight=dn,h.functions=wn,h.groupBy=pe,h.indexBy=se,h.initial=_r,h.intersection=dr,h.invert=kn,h.invoke=Zn,h.keys=ne,h.map=nr,h.mapValues=Wn,h.max=rr,h.memoize=Kr,h.merge=qn,h.min=tr,h.omit=zn,h.once=Ur,h.pairs=Ln,
h.partial=Mr,h.partialRight=Vr,h.pick=Pn,h.pluck=ve,h.property=tt,h.pull=kr,h.range=xr,h.reject=or,h.remove=Cr,h.rest=Or,h.shuffle=ir,h.sortBy=cr,h.tap=ct,h.throttle=Gr,h.times=at,h.toArray=pr,h.transform=Kn,h.union=Rr,h.uniq=Er,h.values=Un,h.where=he,h.without=Ir,h.wrap=Hr,h.xor=Sr,h.zip=Ar,h.zipObject=Dr,h.collect=nr,h.drop=Or,h.each=Xn,h.eachRight=Yn,h.extend=oe,h.methods=wn,h.object=Dr,h.select=Hn,h.tail=Or,h.unique=Er,h.unzip=Ar,Zr(h),h.clone=hn,h.cloneDeep=gn,h.contains=Vn,h.escape=Xr,h.every=Gn,
h.find=Jn,h.findIndex=hr,h.findKey=mn,h.findLast=Qn,h.findLastIndex=gr,h.findLastKey=bn,h.has=jn,h.identity=Yr,h.indexOf=br,h.isArguments=vn,h.isArray=Yt,h.isBoolean=xn,h.isDate=Cn,h.isElement=On,h.isEmpty=Nn,h.isEqual=Rn,h.isFinite=En,h.isFunction=In,h.isNaN=An,h.isNull=Dn,h.isNumber=Tn,h.isObject=Sn,h.isPlainObject=le,h.isRegExp=$n,h.isString=Fn,h.isUndefined=Bn,h.lastIndexOf=jr,h.mixin=Zr,h.noConflict=nt,h.noop=rt,h.now=ge,h.parseInt=ye,h.random=et,h.reduce=er,h.reduceRight=ur,h.result=ut,h.runInContext=s,
h.size=fr,h.some=lr,h.sortedIndex=Nr,h.template=ot,h.unescape=it,h.uniqueId=ft,h.all=Gn,h.any=lr,h.detect=Jn,h.findWhere=Jn,h.foldl=er,h.foldr=ur,h.include=Vn,h.inject=er,Zr(function(){var n={};return fe(h,function(r,t){h.prototype[t]||(n[t]=r)}),n}(),!1),h.first=yr,h.last=wr,h.sample=ar,h.take=yr,h.head=yr,fe(h,function(n,r){var t="sample"!==r;h.prototype[r]||(h.prototype[r]=function(r,e){var u=this.__chain__,o=n(this.__wrapped__,r,e);return u||null!=r&&(!e||t&&"function"==typeof r)?new g(o,u):o;

})}),h.VERSION="2.4.2",h.prototype.chain=pt,h.prototype.toString=st,h.prototype.value=vt,h.prototype.valueOf=vt,Xn(["join","pop","shift"],function(n){var r=xt[n];h.prototype[n]=function(){var n=this.__chain__,t=r.apply(this.__wrapped__,arguments);return n?new g(t,n):t}}),Xn(["push","reverse","sort","unshift"],function(n){var r=xt[n];h.prototype[n]=function(){return r.apply(this.__wrapped__,arguments),this}}),Xn(["concat","slice","splice"],function(n){var r=xt[n];h.prototype[n]=function(){return new g(r.apply(this.__wrapped__,arguments),this.__chain__);

}}),h}var v,h=[],g=[],y=0,m=+new Date+"",b=75,_=40,d=" 	\f\xa0\ufeff\n\r\u2028\u2029\u1680\u180e\u2000\u2001\u2002\u2003\u2004\u2005\u2006\u2007\u2008\u2009\u200a\u202f\u205f\u3000",w=/\b__p \+= '';/g,j=/\b(__p \+=) '' \+/g,k=/(__e\(.*?\)|\b__t\)) \+\n'';/g,x=/\$\{([^\\}]*(?:\\.[^\\}]*)*)\}/g,C=/\w*$/,O=/^\s*function[ \n\r\t]+\w/,N=/<%=([\s\S]+?)%>/g,R=RegExp("^["+d+"]*0+(?=.$)"),E=/($^)/,I=/\bthis\b/,S=/['\n\r\t\u2028\u2029\\]/g,A=["Array","Boolean","Date","Function","Math","Number","Object","RegExp","String","_","attachEvent","clearTimeout","isFinite","isNaN","parseInt","setTimeout"],D=0,T="[object Arguments]",$="[object Array]",F="[object Boolean]",B="[object Date]",W="[object Function]",q="[object Number]",z="[object Object]",L="[object RegExp]",P="[object String]",K={};

K[W]=!1,K[T]=K[$]=K[F]=K[B]=K[q]=K[z]=K[L]=K[P]=!0;var U={leading:!1,maxWait:0,trailing:!1},M={configurable:!1,enumerable:!1,value:null,writable:!1},V={"boolean":!1,"function":!0,object:!0,number:!1,string:!1,undefined:!1},G={"\\":"\\","'":"'","\n":"n","\r":"r","	":"t","\u2028":"u2028","\u2029":"u2029"},H=V[typeof window]&&window||this,J=V[typeof exports]&&exports&&!exports.nodeType&&exports,Q=V[typeof module]&&module&&!module.nodeType&&module,X=Q&&Q.exports===J&&J,Y=V[typeof global]&&global;!Y||Y.global!==Y&&Y.window!==Y||(H=Y);

var Z=s();"function"==typeof define&&"object"==typeof define.amd&&define.amd?(H._=Z,define(function(){return Z})):J&&Q?X?(Q.exports=Z)._=Z:J._=Z:H._=Z}).call(this);
(function(angular, factory) {
    'use strict';

    if (typeof define === 'function' && define.amd) {
        define(['angular'], function(angular) {
            return factory(angular);
        });
    } else {
        return factory(angular);
    }
}(window.angular || null, function(angular) {
    'use strict';

/**
 * ngTable: Table + Angular JS
 *
 * @author Vitalii Savchuk <esvit666@gmail.com>
 * @url https://github.com/esvit/ng-table/
 * @license New BSD License <http://creativecommons.org/licenses/BSD/>
 */

/**
 * @ngdoc module
 * @name ngTable
 * @description ngTable: Table + Angular JS
 * @example
 <doc:example>
 <doc:source>
 <script>
 var app = angular.module('myApp', ['ngTable']);
 app.controller('MyCtrl', function($scope) {
                    $scope.users = [
                        {name: "Moroni", age: 50},
                        {name: "Tiancum", age: 43},
                        {name: "Jacob", age: 27},
                        {name: "Nephi", age: 29},
                        {name: "Enos", age: 34}
                    ];
                });
 </script>
 <table ng-table class="table">
 <tr ng-repeat="user in users">
 <td data-title="'Name'">{{user.name}}</td>
 <td data-title="'Age'">{{user.age}}</td>
 </tr>
 </table>
 </doc:source>
 </doc:example>
 */
var app = angular.module('ngTable', []);
/**
 * ngTable: Table + Angular JS
 *
 * @author Vitalii Savchuk <esvit666@gmail.com>
 * @url https://github.com/esvit/ng-table/
 * @license New BSD License <http://creativecommons.org/licenses/BSD/>
 */

(function(){
    'use strict';

    angular.module('ngTable')
        .factory('ngTableEventsChannel', ngTableEventsChannel);

    ngTableEventsChannel.$inject = ['$rootScope'];

    /**
     * @ngdoc service
     * @name ngTableEventsChannel
     * @description strongly typed pub/sub for `NgTableParams`
     *
     * Supported events:
     *
     * * afterCreated - raised when a new instance of `NgTableParams` has finished being constructed
     * * afterReloadData - raised when the `reload` event has finished loading new data
     * * datasetChanged - raised when `settings` receives a new data array
     * * pagesChanged - raised when a new pages array has been generated
     */
    function ngTableEventsChannel($rootScope){

        var events = {};
        events = addChangeEvent('afterCreated', events);
        events = addChangeEvent('afterReloadData', events);
        events = addChangeEvent('datasetChanged', events);
        events = addChangeEvent('pagesChanged', events);
        return events;

        //////////

        function addChangeEvent(eventName, target){
            var fnName = eventName.charAt(0).toUpperCase() + eventName.substring(1);
            var event = {};
            event['on' + fnName] = createEventSubscriptionFn(eventName);
            event['publish' + fnName] = createPublishEventFn(eventName);
            return angular.extend(target, event);
        }

        function createEventSubscriptionFn(eventName){

            return function subscription(handler/*[, eventSelector or $scope][, eventSelector]*/){
                var eventSelector = angular.identity;
                var scope = $rootScope;

                if (arguments.length === 2){
                    if (angular.isFunction(arguments[1].$new)) {
                        scope = arguments[1];
                    } else {
                        eventSelector = arguments[1]
                    }
                } else if (arguments.length > 2){
                    scope = arguments[1];
                    eventSelector = arguments[2];
                }

                // shorthand for subscriber to only receive events from a specific publisher instance
                if (angular.isObject(eventSelector)) {
                    var requiredPublisher = eventSelector;
                    eventSelector = function(publisher){
                        return publisher === requiredPublisher;
                    }
                }

                return scope.$on('ngTable:' + eventName, function(event, params/*, ...args*/){
                    // don't send events published by the internal NgTableParams created by ngTableController
                    if (params.isNullInstance) return;

                    var eventArgs = rest(arguments, 2);
                    var fnArgs = [params].concat(eventArgs);
                    if (eventSelector.apply(this, fnArgs)){
                        handler.apply(this, fnArgs);
                    }
                });
            }
        }

        function createPublishEventFn(eventName){
            return function publish(/*args*/){
                var fnArgs = ['ngTable:' + eventName].concat(Array.prototype.slice.call(arguments));
                $rootScope.$broadcast.apply($rootScope, fnArgs);
            }
        }

        function rest(array, n) {
            return Array.prototype.slice.call(array, n == null ? 1 : n);
        }
    }
})();

/**
 * ngTable: Table + Angular JS
 *
 * @author Vitalii Savchuk <esvit666@gmail.com>
 * @url https://github.com/esvit/ng-table/
 * @license New BSD License <http://creativecommons.org/licenses/BSD/>
 */

(function(){
    'use strict';

    angular.module('ngTable')
        .provider('ngTableFilterConfig', ngTableFilterConfigProvider);

    ngTableFilterConfigProvider.$inject = [];

    function ngTableFilterConfigProvider(){
        var config;
        var defaultConfig = {
            defaultBaseUrl: 'ng-table/filters/',
            defaultExt: '.html',
            aliasUrls: {}
        };

        this.$get = ngTableFilterConfig;
        this.resetConfigs = resetConfigs;
        this.setConfig = setConfig;

        init();

        /////////

        function init(){
            resetConfigs();
        }

        function resetConfigs(){
            config = defaultConfig;
        }

        function setConfig(customConfig){
            var mergeConfig = angular.extend({}, config, customConfig);
            mergeConfig.aliasUrls = angular.extend({}, config.aliasUrls, customConfig.aliasUrls);
            config = mergeConfig;
        }

        /////////

        ngTableFilterConfig.$inject = [];

        function ngTableFilterConfig(){

            var publicConfig;

            var service = {
                config: publicConfig,
                getTemplateUrl: getTemplateUrl,
                getUrlForAlias: getUrlForAlias
            };
            Object.defineProperty(service, "config", {
                get: function(){
                    return publicConfig = publicConfig || angular.copy(config);
                },
                enumerable: true
            });

            return service;

            /////////

            function getTemplateUrl(filterValue, filterKey){
                if (filterValue.indexOf('/') !== -1){
                    return filterValue;
                }

                return service.getUrlForAlias(filterValue, filterKey);
            }

            function getUrlForAlias(aliasName/*, filterKey*/){
                return config.aliasUrls[aliasName] || config.defaultBaseUrl + aliasName + config.defaultExt;
            }
        }
    }
})();

/**
 * ngTable: Table + Angular JS
 *
 * @author Vitalii Savchuk <esvit666@gmail.com>
 * @url https://github.com/esvit/ng-table/
 * @license New BSD License <http://creativecommons.org/licenses/BSD/>
 */

(function(){
    'use strict';


    angular.module('ngTable')
        .provider('ngTableDefaultGetData', ngTableDefaultGetDataProvider);

    ngTableDefaultGetDataProvider.$inject = [];

    /**
     * @ngdoc provider
     * @name ngTableDefaultGetDataProvider
     * @description Allows for the configuration of the {@link ngTable.ngTableDefaultGetData ngTableDefaultGetData}
     * service.
     *
     * Set filterFilterName to the name of a angular filter that knows how to take `NgTableParams.filter()`
     * to restrict an array of data.
     *
     * Set sortingFilterName to the name of a angular filter that knows how to take `NgTableParams.orderBy()`
     * to sort an array of data.
     *
     * Out of the box the `ngTableDefaultGetData` service will be configured to use the angular `filter` and `orderBy`
     * filters respectively
     */
    function ngTableDefaultGetDataProvider(){
        var provider = this;
        provider.$get = ngTableDefaultGetData;
        provider.filterFilterName = 'filter';
        provider.sortingFilterName = 'orderBy';

        ///////////

        ngTableDefaultGetData.$inject = ['$filter'];

        /**
         * @ngdoc service
         * @name ngTableDefaultGetData
         * @description A default implementation of the getData function that will apply the `filter`, `orderBy` and
         * paging values from the `NgTableParams` instance supplied to the data array supplied.
         *
         * The outcome will be to return the resulting array and to assign the total item count after filtering
         * to the `total` of the `NgTableParams` instance supplied
         */
        function ngTableDefaultGetData($filter) {

            return getData;

            function getData(data, params) {
                if (data == null){
                    return [];
                }

                var fData = params.hasFilter() ? $filter(provider.filterFilterName)(data, params.filter(true)) : data;
                var orderBy = params.orderBy();
                var orderedData = orderBy.length ? $filter(provider.sortingFilterName)(fData, orderBy) : fData;
                var pagedData = orderedData.slice((params.page() - 1) * params.count(), params.page() * params.count());
                params.total(orderedData.length); // set total for recalc pagination
                return pagedData;
            }
        }
    }
})();

/**
 * ngTable: Table + Angular JS
 *
 * @author Vitalii Savchuk <esvit666@gmail.com>
 * @url https://github.com/esvit/ng-table/
 * @license New BSD License <http://creativecommons.org/licenses/BSD/>
 */

(function(){
    'use strict';

    // todo: remove shim after an acceptable depreciation period

    angular.module('ngTable')
        .factory('ngTableGetDataBcShim', ngTableGetDataBcShim);

    ngTableGetDataBcShim.$inject = ['$q'];

    function ngTableGetDataBcShim($q){

        return createWrapper;

        function createWrapper(getDataFn){
            return function getDataShim(/*args*/){
                var $defer = $q.defer();
                var pData = getDataFn.apply(this, [$defer].concat(Array.prototype.slice.call(arguments)));
                if (!pData) {
                    // If getData resolved the $defer, and didn't promise us data,
                    //   create a promise from the $defer. We need to return a promise.
                    pData = $defer.promise;
                }
                return pData;
            }
        }
    }
})();

/**
 * ngTable: Table + Angular JS
 *
 * @author Vitalii Savchuk <esvit666@gmail.com>
 * @url https://github.com/esvit/ng-table/
 * @license New BSD License <http://creativecommons.org/licenses/BSD/>
 */

/**
 * @ngdoc object
 * @name ngTableDefaultParams
 * @module ngTable
 * @description Default Parameters for ngTable
 */
app.value('ngTableDefaults', {
    params: {},
    settings: {}
});

/**
 * @ngdoc service
 * @name NgTableParams
 * @module ngTable
 * @description Parameters manager for ngTable
 */

app.factory('NgTableParams', ['$q', '$log', 'ngTableDefaults', 'ngTableGetDataBcShim', 'ngTableDefaultGetData', 'ngTableEventsChannel', function($q, $log, ngTableDefaults, ngTableGetDataBcShim, ngTableDefaultGetData, ngTableEventsChannel) {
    var isNumber = function(n) {
        return !isNaN(parseFloat(n)) && isFinite(n);
    };
    var NgTableParams = function(baseParameters, baseSettings) {

        // the ngTableController "needs" to create a dummy/null instance and it's important to know whether an instance
        // is one of these
        if (typeof baseParameters === "boolean"){
            this.isNullInstance = true;
        }

        var self = this,
            committedParams,
            isCommittedDataset = false,
            log = function() {
                if (settings.debugMode && $log.debug) {
                    $log.debug.apply(this, arguments);
                }
            };

        this.data = [];

        /**
         * @ngdoc method
         * @name NgTableParams#parameters
         * @description Set new parameters or get current parameters
         *
         * @param {string} newParameters      New parameters
         * @param {string} parseParamsFromUrl Flag if parse parameters like in url
         * @returns {Object} Current parameters or `this`
         */
        this.parameters = function(newParameters, parseParamsFromUrl) {
            parseParamsFromUrl = parseParamsFromUrl || false;
            if (angular.isDefined(newParameters)) {
                for (var key in newParameters) {
                    var value = newParameters[key];
                    if (parseParamsFromUrl && key.indexOf('[') >= 0) {
                        var keys = key.split(/\[(.*)\]/).reverse()
                        var lastKey = '';
                        for (var i = 0, len = keys.length; i < len; i++) {
                            var name = keys[i];
                            if (name !== '') {
                                var v = value;
                                value = {};
                                value[lastKey = name] = (isNumber(v) ? parseFloat(v) : v);
                            }
                        }
                        if (lastKey === 'sorting') {
                            params[lastKey] = {};
                        }
                        params[lastKey] = angular.extend(params[lastKey] || {}, value[lastKey]);
                    } else {
                        params[key] = (isNumber(newParameters[key]) ? parseFloat(newParameters[key]) : newParameters[key]);
                    }
                }
                log('ngTable: set parameters', params);
                return this;
            }
            return params;
        };

        /**
         * @ngdoc method
         * @name NgTableParams#settings
         * @description Set new settings for table
         *
         * @param {string} newSettings New settings or undefined
         * @returns {Object} Current settings or `this`
         */
        this.settings = function(newSettings) {
            if (angular.isDefined(newSettings)) {
                if (angular.isArray(newSettings.data)) {
                    //auto-set the total from passed in data
                    newSettings.total = newSettings.data.length;
                }

                // todo: remove the backwards compatibility shim and the following two if blocks
                if (newSettings.getData && newSettings.getData.length > 1){
                    // support the old getData($defer, params) api
                    newSettings.getDataFnAdaptor = ngTableGetDataBcShim;
                }
                if (newSettings.getGroups && newSettings.getGroups.length > 2){
                    // support the old getGroups($defer, grouping, params) api
                    newSettings.getGroupsFnAdaptor = ngTableGetDataBcShim;
                }

                var originalDataset = settings.data;
                settings = angular.extend(settings, newSettings);

                // note: using != as want null and undefined to be treated the same
                var hasDatasetChanged = newSettings.hasOwnProperty('data') && (newSettings.data != originalDataset);
                if (hasDatasetChanged) {
                    if (isCommittedDataset){
                        this.page(1); // reset page as a new dataset has been supplied
                    }
                    isCommittedDataset = false;
                    ngTableEventsChannel.publishDatasetChanged(this, newSettings.data, originalDataset);
                }
                log('ngTable: set settings', settings);
                return this;
            }
            return settings;
        };

        /**
         * @ngdoc method
         * @name NgTableParams#page
         * @description If parameter page not set return current page else set current page
         *
         * @param {string} page Page number
         * @returns {Object|Number} Current page or `this`
         */
        this.page = function(page) {
            return angular.isDefined(page) ? this.parameters({
                'page': page
            }) : params.page;
        };

        /**
         * @ngdoc method
         * @name NgTableParams#total
         * @description If parameter total not set return current quantity else set quantity
         *
         * @param {string} total Total quantity of items
         * @returns {Object|Number} Current page or `this`
         */
        this.total = function(total) {
            return angular.isDefined(total) ? this.settings({
                'total': total
            }) : settings.total;
        };

        /**
         * @ngdoc method
         * @name NgTableParams#count
         * @description If parameter count not set return current count per page else set count per page
         *
         * @param {string} count Count per number
         * @returns {Object|Number} Count per page or `this`
         */
        this.count = function(count) {
            // reset to first page because can be blank page
            return angular.isDefined(count) ? this.parameters({
                'count': count,
                'page': 1
            }) : params.count;
        };

        /**
         * @ngdoc method
         * @name NgTableParams#filter
         * @description If 'filter' parameter not set return current filter else set current filter
         *
         * Note: when assigning a new filter, {@link NgTableParams#page page} will be set to 1
         *
         * @param {Object|Boolean} filter 'object': new filter to assign or
         * 'true': to return the current filter minus any insignificant values (null,  undefined and empty string); or
         * 'falsey': to return the current filter "as is"
         * @returns {Object} Current filter or `this`
         */
        this.filter = function(filter) {
            if (angular.isDefined(filter) && angular.isObject(filter)) {
                return this.parameters({
                    'filter': filter,
                    'page': 1
                });
            } else if (filter === true){
                var keys = Object.keys(params.filter);
                var significantFilter = {};
                for (var i=0; i < keys.length; i++){
                    var filterValue = params.filter[keys[i]];
                    if (filterValue != null && filterValue !== '') {
                        significantFilter[keys[i]] = filterValue;
                    }
                }
                return significantFilter;
            } else {
                return params.filter;
            }
        };

        /**
         * @ngdoc method
         * @name NgTableParams#sorting
         * @description If 'sorting' parameter is not set, return current sorting. Otherwise set current sorting.
         *
         * @param {string} sorting New sorting
         * @returns {Object} Current sorting or `this`
         */
        this.sorting = function(sorting) {
            if (arguments.length == 2) {
                var sortArray = {};
                sortArray[sorting] = arguments[1];
                this.parameters({
                    'sorting': sortArray
                });
                return this;
            }
            return angular.isDefined(sorting) ? this.parameters({
                'sorting': sorting
            }) : params.sorting;
        };

        /**
         * @ngdoc method
         * @name NgTableParams#isSortBy
         * @description Checks sort field
         *
         * @param {string} field     Field name
         * @param {string} direction Optional direction of sorting ('asc' or 'desc')
         * @returns {Array} Return true if field sorted by direction
         */
        this.isSortBy = function(field, direction) {
            if(direction !== undefined) {
                return angular.isDefined(params.sorting[field]) && params.sorting[field] == direction;
            } else {
                return angular.isDefined(params.sorting[field]);
            }
        };

        /**
         * @ngdoc method
         * @name NgTableParams#orderBy
         * @description Return object of sorting parameters for angular filter
         *
         * @returns {Array} Array like: [ '-name', '+age' ]
         */
        this.orderBy = function() {
            var sorting = [];
            for (var column in params.sorting) {
                sorting.push((params.sorting[column] === "asc" ? "+" : "-") + column);
            }
            return sorting;
        };

        /**
         * @ngdoc method
         * @name NgTableParams#getData
         * @description Called when updated some of parameters for get new data
         *
         * @param {Object} params New parameters
         */
        this.getData = function(params) {
            // note: this === settings
            return ngTableDefaultGetData(this.data, params);
        };

        /**
         * @ngdoc method
         * @name NgTableParams#getGroups
         * @description Return groups for table grouping
         */
        this.getGroups = function(column) {
            return runGetData().then(function(data) {
                var groups = {};
                angular.forEach(data, function(item) {
                    var groupName = angular.isFunction(column) ? column(item) : item[column];

                    groups[groupName] = groups[groupName] || {
                        data: []
                    };
                    groups[groupName]['value'] = groupName;
                    groups[groupName].data.push(item);
                });
                var result = [];
                for (var i in groups) {
                    result.push(groups[i]);
                }
                log('ngTable: refresh groups', result);
                return result;
            });
        };

        /**
         * @ngdoc method
         * @name NgTableParams#generatePagesArray
         * @description Generate array of pages
         *
         * When no arguments supplied, the current parameter state of this `NgTableParams` instance will be used
         *
         * @param {boolean} currentPage which page must be active
         * @param {boolean} totalItems  Total quantity of items
         * @param {boolean} pageSize    Quantity of items on page
         * @param {number} maxBlocks    Quantity of blocks for pagination
         * @returns {Array} Array of pages
         */
        this.generatePagesArray = function(currentPage, totalItems, pageSize, maxBlocks) {
            if (!arguments.length){
                currentPage = this.page();
                totalItems = this.total();
                pageSize = this.count();
            }

            var maxPage, maxPivotPages, minPage, numPages, pages;
            maxBlocks = maxBlocks && maxBlocks < 6 ? 6 : maxBlocks;

            pages = [];
            numPages = Math.ceil(totalItems / pageSize);
            if (numPages > 1) {
                pages.push({
                    type: 'prev',
                    number: Math.max(1, currentPage - 1),
                    active: currentPage > 1
                });
                pages.push({
                    type: 'first',
                    number: 1,
                    active: currentPage > 1,
                    current: currentPage === 1
                });
                maxPivotPages = Math.round((settings.paginationMaxBlocks - settings.paginationMinBlocks) / 2);
                minPage = Math.max(2, currentPage - maxPivotPages);
                maxPage = Math.min(numPages - 1, currentPage + maxPivotPages * 2 - (currentPage - minPage));
                minPage = Math.max(2, minPage - (maxPivotPages * 2 - (maxPage - minPage)));
                var i = minPage;
                while (i <= maxPage) {
                    if ((i === minPage && i !== 2) || (i === maxPage && i !== numPages - 1)) {
                        pages.push({
                            type: 'more',
                            active: false
                        });
                    } else {
                        pages.push({
                            type: 'page',
                            number: i,
                            active: currentPage !== i,
                            current: currentPage === i
                        });
                    }
                    i++;
                }
                pages.push({
                    type: 'last',
                    number: numPages,
                    active: currentPage !== numPages,
                    current: currentPage === numPages
                });
                pages.push({
                    type: 'next',
                    number: Math.min(numPages, currentPage + 1),
                    active: currentPage < numPages
                });
            }
            return pages;
        };

        /**
         * @ngdoc method
         * @name NgTableParams#isDataReloadRequired
         * @description Return true when a change to this `NgTableParams` instance should require the reload method
         * to be run so as to ensure the data presented to the user reflects the `NgTableParams`
         */
        this.isDataReloadRequired = function(){
            // note: using != as want to treat null and undefined the same
            return !isCommittedDataset || !angular.equals(params, committedParams);
        };

        /**
         * @ngdoc method
         * @name NgTableParams#hasFilter
         * @description Determines if NgTableParams#filter has significant filter value(s)
         * (any value except null, undefined, or empty string)
         * @returns {Boolean} true when NgTableParams#filter has at least one significant field value
         */
        this.hasFilter = function(){
            return Object.keys(this.filter(true)).length > 0;
        };

        /**
         * @ngdoc method
         * @name NgTableParams#hasFilterChanges
         * @description Return true when a change to `NgTableParams.filters`require the reload method
         * to be run so as to ensure the data presented to the user reflects these filters
         */
        this.hasFilterChanges = function(){
            return !angular.equals((params && params.filter), (committedParams && committedParams.filter));
        };

        /**
         * @ngdoc method
         * @name NgTableParams#url
         * @description Return groups for table grouping
         *
         * @param {boolean} asString flag indicates return array of string or object
         * @returns {Array} If asString = true will be return array of url string parameters else key-value object
         */
        this.url = function(asString) {
            asString = asString || false;
            var pairs = (asString ? [] : {});
            for (var key in params) {
                if (params.hasOwnProperty(key)) {
                    var item = params[key],
                        name = encodeURIComponent(key);
                    if (typeof item === "object") {
                        for (var subkey in item) {
                            if (!angular.isUndefined(item[subkey]) && item[subkey] !== "") {
                                var pname = name + "[" + encodeURIComponent(subkey) + "]";
                                if (asString) {
                                    pairs.push(pname + "=" + item[subkey]);
                                } else {
                                    pairs[pname] = item[subkey];
                                }
                            }
                        }
                    } else if (!angular.isFunction(item) && !angular.isUndefined(item) && item !== "") {
                        if (asString) {
                            pairs.push(name + "=" + encodeURIComponent(item));
                        } else {
                            pairs[name] = encodeURIComponent(item);
                        }
                    }
                }
            }
            return pairs;
        };

        /**
         * @ngdoc method
         * @name NgTableParams#reload
         * @description Reload table data
         */
        this.reload = function() {
            var self = this,
                pData = null;

            settings.$loading = true;

            committedParams = angular.copy(params);
            isCommittedDataset = true;

            if (settings.groupBy) {
                pData = runInterceptorPipeline(runGetGroups);
            } else {
                pData = runInterceptorPipeline(runGetData);
            }

            log('ngTable: reload data');

            var oldData = self.data;
            return pData.then(function(data) {
                settings.$loading = false;
                self.data = data;
                // note: I think it makes sense to publish this event even when data === oldData
                // subscribers can always set a filter to only receive the event when data !== oldData
                ngTableEventsChannel.publishAfterReloadData(self, data, oldData);
                self.reloadPages();

                // todo: remove after acceptable depreciation period
                if (settings.$scope) {
                    settings.$scope.$emit('ngTableAfterReloadData');
                }

                return data;
            }).catch(function(reason){
                committedParams = null;
                isCommittedDataset = false;
                // "rethrow"
                return $q.reject(reason);
            });
        };

        this.reloadPages = (function() {
            var currentPages;
            return function(){
                var oldPages = currentPages;
                var newPages = self.generatePagesArray(self.page(), self.total(), self.count());
                if (!angular.equals(oldPages, newPages)){
                    currentPages = newPages;
                    ngTableEventsChannel.publishPagesChanged(this, newPages, oldPages);
                }
            }
        })();

        function runGetData(){
            var getDataFn = settings.getDataFnAdaptor(settings.getData);
            return $q.when(getDataFn.call(settings, self));
        }

        function runGetGroups(){
            var getGroupsFn = settings.getGroupsFnAdaptor(settings.getGroups);
            return $q.when(getGroupsFn.call(settings, settings.groupBy, self));
        }

        function runInterceptorPipeline(fetchFn){
            var interceptors = settings.interceptors || [];

            return interceptors.reduce(function(result, interceptor){
                var thenFn = (interceptor.response && interceptor.response.bind(interceptor)) || $q.when;
                var rejectFn = (interceptor.responseError && interceptor.responseError.bind(interceptor)) || $q.reject;
                return result.then(function(data){
                    return thenFn(data, self);
                }, function(reason){
                    return rejectFn(reason, self);
                });
            }, fetchFn());
        }

        var params = {
            page: 1,
            count: 1,
            filter: {},
            sorting: {},
            group: {},
            groupBy: null
        };
        angular.extend(params, ngTableDefaults.params);

        var settings = {
            // todo: remove $scope after acceptable depreciation period as no longer required
            $scope: null, // set by ngTable controller
            $loading: false,
            data: null, //allows data to be set when table is initialized
            total: 0,
            defaultSort: 'desc',
            filterDelay: 750,
            counts: [10, 25, 50, 100],
            interceptors: [],
            paginationMaxBlocks: 11,
            paginationMinBlocks: 5,
            sortingIndicator: 'span',
            getDataFnAdaptor: angular.identity,
            getGroupsFnAdaptor: angular.identity,
            getGroups: this.getGroups,
            getData: this.getData
        };

        this.settings(ngTableDefaults.settings);
        this.settings(baseSettings);
        this.parameters(baseParameters, true);

        ngTableEventsChannel.publishAfterCreated(this);

        return this;
    };
    return NgTableParams;
}]);

/**
 * @ngdoc service
 * @name ngTableParams
 * @description Backwards compatible shim for lowercase 'n' in NgTableParams
 */
app.factory('ngTableParams', ['NgTableParams', function(NgTableParams) {
    return NgTableParams;
}]);

/**
 * ngTable: Table + Angular JS
 *
 * @author Vitalii Savchuk <esvit666@gmail.com>
 * @url https://github.com/esvit/ng-table/
 * @license New BSD License <http://creativecommons.org/licenses/BSD/>
 */

(function(){
    'use strict';

    angular.module('ngTable')
        .controller('ngTableFilterRowController', ngTableFilterRowController);

    ngTableFilterRowController.$inject = ['$scope', 'ngTableFilterConfig'];

    function ngTableFilterRowController($scope, ngTableFilterConfig){

        $scope.config = ngTableFilterConfig;
    }
})();

/**
 * ngTable: Table + Angular JS
 *
 * @author Vitalii Savchuk <esvit666@gmail.com>
 * @url https://github.com/esvit/ng-table/
 * @license New BSD License <http://creativecommons.org/licenses/BSD/>
 */

(function(){
    'use strict';

    angular.module('ngTable')
        .controller('ngTableSorterRowController', ngTableSorterRowController);

    ngTableSorterRowController.$inject = ['$scope'];

    function ngTableSorterRowController($scope){

        $scope.sortBy = sortBy;

        ///////////

        function sortBy($column, event) {
            var parsedSortable = $column.sortable && $column.sortable();
            if (!parsedSortable) {
                return;
            }
            var defaultSort = $scope.params.settings().defaultSort;
            var inverseSort = (defaultSort === 'asc' ? 'desc' : 'asc');
            var sorting = $scope.params.sorting() && $scope.params.sorting()[parsedSortable] && ($scope.params.sorting()[parsedSortable] === defaultSort);
            var sortingParams = (event.ctrlKey || event.metaKey) ? $scope.params.sorting() : {};
            sortingParams[parsedSortable] = (sorting ? inverseSort : defaultSort);
            $scope.params.parameters({
                sorting: sortingParams
            });
        }
    }
})();

/**
 * ngTable: Table + Angular JS
 *
 * @author Vitalii Savchuk <esvit666@gmail.com>
 * @url https://github.com/esvit/ng-table/
 * @license New BSD License <http://creativecommons.org/licenses/BSD/>
 */

/**
 * @ngdoc object
 * @name ngTableController
 *
 * @description
 * Each {@link ngTable ngTable} directive creates an instance of `ngTableController`
 */
app.controller('ngTableController', ['$scope', 'NgTableParams', '$timeout', '$parse', '$compile', '$attrs', '$element',
    'ngTableColumn', 'ngTableEventsChannel',
function($scope, NgTableParams, $timeout, $parse, $compile, $attrs, $element, ngTableColumn, ngTableEventsChannel) {
    var isFirstTimeLoad = true;
    $scope.$filterRow = {};
    $scope.$loading = false;

    // until such times as the directive uses an isolated scope, we need to ensure that the check for
    // the params field only consults the "own properties" of the $scope. This is to avoid seeing the params
    // field on a $scope higher up in the prototype chain
    if (!$scope.hasOwnProperty("params")) {
        $scope.params = new NgTableParams(true);
    }
    $scope.params.settings().$scope = $scope;

    var delayFilter = (function() {
        var timer = 0;
        return function(callback, ms) {
            $timeout.cancel(timer);
            timer = $timeout(callback, ms);
        };
    })();

    function onDataReloadStatusChange (newStatus/*, oldStatus*/) {
        if (!newStatus) {
            return;
        }

        $scope.params.settings().$scope = $scope;

        var currentParams = $scope.params;

        if (currentParams.hasFilterChanges()) {
            var applyFilter = function () {
                currentParams.page(1);
                currentParams.reload();
            };
            if (currentParams.settings().filterDelay) {
                delayFilter(applyFilter, currentParams.settings().filterDelay);
            } else {
                applyFilter();
            }
        } else {
            currentParams.reload();
        }
    }

    // watch for when a new NgTableParams is bound to the scope
    // CRITICAL: the watch must be for reference and NOT value equality; this is because NgTableParams maintains
    // the current data page as a field. Checking this for value equality would be terrible for performance
    // and potentially cause an error if the items in that array has circular references
    $scope.$watch('params', function(newParams, oldParams){
        if (newParams === oldParams || !newParams) {
            return;
        }

        newParams.reload();
    }, false);

    $scope.$watch('params.isDataReloadRequired()', onDataReloadStatusChange);

    this.compileDirectiveTemplates = function () {
        if (!$element.hasClass('ng-table')) {
            $scope.templates = {
                header: ($attrs.templateHeader ? $attrs.templateHeader : 'ng-table/header.html'),
                pagination: ($attrs.templatePagination ? $attrs.templatePagination : 'ng-table/pager.html')
            };
            $element.addClass('ng-table');
            var headerTemplate = null;

            // $element.find('> thead').length === 0 doesn't work on jqlite
            var theadFound = false;
            angular.forEach($element.children(), function(e) {
                if (e.tagName === 'THEAD') {
                    theadFound = true;
                }
            });
            if (!theadFound) {
                headerTemplate = angular.element(document.createElement('thead')).attr('ng-include', 'templates.header');
                $element.prepend(headerTemplate);
            }
            var paginationTemplate = angular.element(document.createElement('div')).attr({
                'ng-table-pagination': 'params',
                'template-url': 'templates.pagination'
            });
            $element.after(paginationTemplate);
            if (headerTemplate) {
                $compile(headerTemplate)($scope);
            }
            $compile(paginationTemplate)($scope);
        }
    };

    this.loadFilterData = function ($columns) {
        angular.forEach($columns, function ($column) {
            var def;
            def = $column.filterData($scope, {
                $column: $column
            });
            if (!def) {
                delete $column.filterData;
                return;
            }

            // if we're working with a deferred object, let's wait for the promise
            if ((angular.isObject(def) && angular.isObject(def.promise))) {
                delete $column.filterData;
                return def.promise.then(function(data) {
                    // our deferred can eventually return arrays, functions and objects
                    if (!angular.isArray(data) && !angular.isFunction(data) && !angular.isObject(data)) {
                        // if none of the above was found - we just want an empty array
                        data = [];
                    } else if (angular.isArray(data)) {
                        data.unshift({
                            title: '',
                            id: ''
                        });
                    }
                    $column.data = data;
                });
            }
            // otherwise, we just return what the user gave us. It could be a function, array, object, whatever
            else {
                return $column.data = def;
            }
        });
    };

    this.buildColumns = function (columns) {
        return columns.map(function(col){
            return ngTableColumn.buildColumn(col, $scope)
        })
    };

    this.parseNgTableDynamicExpr = function (attr) {
        if (!attr || attr.indexOf(" with ") > -1) {
            var parts = attr.split(/\s+with\s+/);
            return {
                tableParams: parts[0],
                columns: parts[1]
            };
        } else {
            throw new Error('Parse error (expected example: ng-table-dynamic=\'tableParams with cols\')');
        }
    };

    this.setupBindingsToInternalScope = function(tableParamsExpr){

        // note: this we're setting up watches to simulate angular's isolated scope bindings

        // note: is REALLY important to watch for a change to the ngTableParams *reference* rather than
        // $watch for value equivalence. This is because ngTableParams references the current page of data as
        // a field and it's important not to watch this
        var tableParamsGetter = $parse(tableParamsExpr);
        $scope.$watch(tableParamsGetter, (function (params) {
            if (angular.isUndefined(params)) {
                return;
            }
            $scope.paramsModel = tableParamsGetter;
            $scope.params = params;
        }), false);

        if ($attrs.showFilter) {
            $scope.$parent.$watch($attrs.showFilter, function(value) {
                $scope.show_filter = value;
            });
        }
        if ($attrs.disableFilter) {
            $scope.$parent.$watch($attrs.disableFilter, function(value) {
                $scope.$filterRow.disabled = value;
            });
        }
    };



    function commonInit(){
        ngTableEventsChannel.onAfterReloadData(bindDataToScope, $scope, isMyPublisher);
        ngTableEventsChannel.onPagesChanged(bindPagesToScope, $scope, isMyPublisher);

        function bindDataToScope(params, newDatapage){
            if (params.settings().groupBy) {
                $scope.$groups = newDatapage;
            } else {
                $scope.$data = newDatapage;
            }
        }

        function bindPagesToScope(params, newPages){
            $scope.pages = newPages
        }

        function isMyPublisher(publisher){
            return $scope.params === publisher;
        }
    }

    commonInit();
}]);


/**
 * @ngdoc service
 * @name ngTableColumn
 * @module ngTable
 * @description
 * Service to construct a $column definition used by {@link ngTable ngTable} directive
 */
app.factory('ngTableColumn', [function () {

    var defaults = {
        'class': function(){ return ''; },
        filter: function(){ return false; },
        filterData: angular.noop,
        headerTemplateURL: function(){ return false; },
        headerTitle: function(){ return ''; },
        sortable: function(){ return false; },
        show: function(){ return true; },
        title: function(){ return ''; },
        titleAlt: function(){ return ''; }
    };

    /**
     * @ngdoc method
     * @name ngTableColumn#buildColumn
     * @description Creates a $column for use within a header template
     *
     * @param {Object} column an existing $column or simple column data object
     * @param {Scope} defaultScope the $scope to supply to the $column getter methods when not supplied by caller
     * @returns {Object} a $column object
     */
    function buildColumn(column, defaultScope){
        // note: we're not modifying the original column object. This helps to avoid unintended side affects
        var extendedCol = Object.create(column);
        for (var prop in defaults) {
            if (extendedCol[prop] === undefined) {
                extendedCol[prop] = defaults[prop];
            }
            if(!angular.isFunction(extendedCol[prop])){
                // wrap raw field values with "getter" functions
                // - this is to ensure consistency with how ngTable.compile builds columns
                // - note that the original column object is being "proxied"; this is important
                //   as it ensure that any changes to the original object will be returned by the "getter"
                (function(prop1){
                    extendedCol[prop1] = function(){
                        return column[prop1];
                    };
                })(prop);
            }
            (function(prop1){
                // satisfy the arguments expected by the function returned by parsedAttribute in the ngTable directive
                var getterFn = extendedCol[prop1];
                extendedCol[prop1] = function(){
                    if (arguments.length === 0){
                        return getterFn.call(column, defaultScope);
                    } else {
                        return getterFn.apply(column, arguments);
                    }
                };
            })(prop);
        }
        return extendedCol;
    }

    return {
        buildColumn: buildColumn
    };
}]);

/**
 * ngTable: Table + Angular JS
 *
 * @author Vitalii Savchuk <esvit666@gmail.com>
 * @url https://github.com/esvit/ng-table/
 * @license New BSD License <http://creativecommons.org/licenses/BSD/>
 */

/**
 * @ngdoc directive
 * @name ngTable
 * @module ngTable
 * @restrict A
 *
 * @description
 * Directive that instantiates {@link ngTableController ngTableController}.
 */
app.directive('ngTable', ['$q', '$parse',
    function($q, $parse) {
        'use strict';

        return {
            restrict: 'A',
            priority: 1001,
            scope: true,
            controller: 'ngTableController',
            compile: function(element) {
                var columns = [],
                    i = 0,
                    row = null;

                // IE 8 fix :not(.ng-table-group) selector
                angular.forEach(angular.element(element.find('tr')), function(tr) {
                    tr = angular.element(tr);
                    if (!tr.hasClass('ng-table-group') && !row) {
                        row = tr;
                    }
                });
                if (!row) {
                    return;
                }
                angular.forEach(row.find('td'), function(item) {
                    var el = angular.element(item);
                    if (el.attr('ignore-cell') && 'true' === el.attr('ignore-cell')) {
                        return;
                    }

                    var getAttrValue = function(attr){
                        return el.attr('x-data-' + attr) || el.attr('data-' + attr) || el.attr(attr);
                    };

                    var parsedAttribute = function(attr) {
                        var expr = getAttrValue(attr);
                        if (!expr){
                            return undefined;
                        }
                        return function(scope, locals) {
                            return $parse(expr)(scope, angular.extend(locals || {}, {
                                $columns: columns
                            }));
                        };
                    };

                    var titleExpr = getAttrValue('title-alt') || getAttrValue('title');
                    if (titleExpr){
                        el.attr('data-title-text', '{{' + titleExpr + '}}'); // this used in responsive table
                    }
                    // NOTE TO MAINTAINERS: if you add extra fields to a $column be sure to extend ngTableColumn with
                    // a corresponding "safe" default
                    columns.push({
                        id: i++,
                        title: parsedAttribute('title'),
                        titleAlt: parsedAttribute('title-alt'),
                        headerTitle: parsedAttribute('header-title'),
                        sortable: parsedAttribute('sortable'),
                        'class': parsedAttribute('header-class'),
                        filter: parsedAttribute('filter'),
                        headerTemplateURL: parsedAttribute('header'),
                        filterData: parsedAttribute('filter-data'),
                        show: (el.attr("ng-if") ? function (scope) {
                            return $parse(el.attr("ng-if"))(scope);
                        } : undefined)
                    });
                });
                return function(scope, element, attrs, controller) {
                    scope.$columns = columns = controller.buildColumns(columns);

                    controller.setupBindingsToInternalScope(attrs.ngTable);
                    controller.loadFilterData(columns);
                    controller.compileDirectiveTemplates();
                };
            }
        }
    }
]);

/**
 * @ngdoc directive
 * @name ngTableDynamic
 * @module ngTable
 * @restrict A
 *
 * @description
 * A dynamic version of the {@link ngTable ngTable} directive that accepts a dynamic list of columns
 * definitions to render
 */
app.directive('ngTableDynamic', ['$parse', function ($parse){

    return {
        restrict: 'A',
        priority: 1001,
        scope: true,
        controller: 'ngTableController',
        compile: function(tElement) {
            var row;

            // IE 8 fix :not(.ng-table-group) selector
            angular.forEach(angular.element(tElement.find('tr')), function(tr) {
                tr = angular.element(tr);
                if (!tr.hasClass('ng-table-group') && !row) {
                    row = tr;
                }
            });
            if (!row) {
                return;
            }

            angular.forEach(row.find('td'), function(item) {
                var el = angular.element(item);
                var getAttrValue = function(attr){
                    return el.attr('x-data-' + attr) || el.attr('data-' + attr) || el.attr(attr);
                };

                // this used in responsive table
                var titleExpr = getAttrValue('title');
                if (!titleExpr){
                    el.attr('data-title-text', '{{$columns[$index].titleAlt(this) || $columns[$index].title(this)}}');
                }
                var showExpr = el.attr('ng-if');
                if (!showExpr){
                    el.attr('ng-if', '$columns[$index].show(this)');
                }
            });
            return function (scope, element, attrs, controller) {
                var expr = controller.parseNgTableDynamicExpr(attrs.ngTableDynamic);

                controller.setupBindingsToInternalScope(expr.tableParams);
                controller.compileDirectiveTemplates();

                scope.$watchCollection(expr.columns, function (newCols/*, oldCols*/) {
                    scope.$columns = controller.buildColumns(newCols);
                    controller.loadFilterData(scope.$columns);
                });
            };
        }
    };
}]);

(function(){
    'use strict';

    angular.module('ngTable')
        .directive('ngTableFilterRow', ngTableFilterRow);

    ngTableFilterRow.$inject = [];

    function ngTableFilterRow(){
        var directive = {
            restrict: 'E',
            replace: true,
            templateUrl: 'ng-table/filterRow.html',
            scope: true,
            controller: 'ngTableFilterRowController'
        };
        return directive;
    }
})();

(function(){
    'use strict';

    angular.module('ngTable')
        .directive('ngTableSorterRow', ngTableSorterRow);

    ngTableSorterRow.$inject = [];

    function ngTableSorterRow(){
        var directive = {
            restrict: 'E',
            replace: true,
            templateUrl: 'ng-table/sorterRow.html',
            scope: true,
            controller: 'ngTableSorterRowController'
        };
        return directive;
    }
})();

/**
 * ngTable: Table + Angular JS
 *
 * @author Vitalii Savchuk <esvit666@gmail.com>
 * @url https://github.com/esvit/ng-table/
 * @license New BSD License <http://creativecommons.org/licenses/BSD/>
 */

/**
 * @ngdoc directive
 * @name ngTablePagination
 * @module ngTable
 * @restrict A
 */
app.directive('ngTablePagination', ['$compile', 'ngTableEventsChannel',
    function($compile, ngTableEventsChannel) {
        'use strict';

        return {
            restrict: 'A',
            scope: {
                'params': '=ngTablePagination',
                'templateUrl': '='
            },
            replace: false,
            link: function(scope, element/*, attrs*/) {

                ngTableEventsChannel.onAfterReloadData(function(pubParams) {
                    scope.pages = pubParams.generatePagesArray();
                }, scope, function(pubParams){
                    return pubParams === scope.params;
                });

                scope.$watch('templateUrl', function(templateUrl) {
                    if (angular.isUndefined(templateUrl)) {
                        return;
                    }
                    var template = angular.element(document.createElement('div'));
                    template.attr({
                        'ng-include': 'templateUrl'
                    });
                    element.append(template);
                    $compile(template)(scope);
                });
            }
        };
    }
]);

angular.module('ngTable').run(['$templateCache', function ($templateCache) {
	$templateCache.put('ng-table/filterRow.html', '<tr ng-show="show_filter" class="ng-table-filters"> <th data-title-text="{{$column.titleAlt(this) || $column.title(this)}}" ng-repeat="$column in $columns" ng-if="$column.show(this)" class="filter"> <div ng-repeat="(name, filter) in $column.filter(this)"> <div ng-include="config.getTemplateUrl(filter)"></div> </div> </th> </tr> ');
	$templateCache.put('ng-table/filters/number.html', '<input type="number" name="{{name}}" ng-disabled="$filterRow.disabled" ng-model="params.filter()[name]" class="input-filter form-control"/> ');
	$templateCache.put('ng-table/filters/select-multiple.html', '<select ng-options="data.id as data.title for data in $column.data" ng-disabled="$filterRow.disabled" multiple ng-multiple="true" ng-model="params.filter()[name]" class="filter filter-select-multiple form-control" name="{{name}}"> </select> ');
	$templateCache.put('ng-table/filters/select.html', '<select ng-options="data.id as data.title for data in $column.data" ng-disabled="$filterRow.disabled" ng-model="params.filter()[name]" class="filter filter-select form-control" name="{{name}}"> <option style="display:none" value=""></option> </select> ');
	$templateCache.put('ng-table/filters/text.html', '<input type="text" name="{{name}}" ng-disabled="$filterRow.disabled" ng-model="params.filter()[name]" class="input-filter form-control"/> ');
	$templateCache.put('ng-table/header.html', '<ng-table-sorter-row></ng-table-sorter-row> <ng-table-filter-row></ng-table-filter-row> ');
	$templateCache.put('ng-table/pager.html', '<div class="ng-cloak ng-table-pager" ng-if="params.data.length"> <div ng-if="params.settings().counts.length" class="ng-table-counts btn-group pull-right"> <button ng-repeat="count in params.settings().counts" type="button" ng-class="{\'active\':params.count()==count}" ng-click="params.count(count)" class="btn btn-default"> <span ng-bind="count"></span> </button> </div> <ul class="pagination ng-table-pagination"> <li ng-class="{\'disabled\': !page.active && !page.current, \'active\': page.current}" ng-repeat="page in pages" ng-switch="page.type"> <a ng-switch-when="prev" ng-click="params.page(page.number)" href="">&laquo;</a> <a ng-switch-when="first" ng-click="params.page(page.number)" href=""><span ng-bind="page.number"></span></a> <a ng-switch-when="page" ng-click="params.page(page.number)" href=""><span ng-bind="page.number"></span></a> <a ng-switch-when="more" ng-click="params.page(page.number)" href="">&#8230;</a> <a ng-switch-when="last" ng-click="params.page(page.number)" href=""><span ng-bind="page.number"></span></a> <a ng-switch-when="next" ng-click="params.page(page.number)" href="">&raquo;</a> </li> </ul> </div> ');
	$templateCache.put('ng-table/sorterRow.html', '<tr> <th title="{{$column.headerTitle(this)}}" ng-repeat="$column in $columns" ng-class="{ \'sortable\': $column.sortable(this), \'sort-asc\': params.sorting()[$column.sortable(this)]==\'asc\', \'sort-desc\': params.sorting()[$column.sortable(this)]==\'desc\' }" ng-click="sortBy($column, $event)" ng-if="$column.show(this)" ng-init="template=$column.headerTemplateURL(this)" class="header {{$column.class(this)}}"> <div ng-if="!template" class="ng-table-header" ng-class="{\'sort-indicator\': params.settings().sortingIndicator==\'div\'}"> <span ng-bind="$column.title(this)" ng-class="{\'sort-indicator\': params.settings().sortingIndicator==\'span\'}"></span> </div> <div ng-if="template" ng-include="template"></div> </th> </tr> ');
}]);
    return app;
}));
/**
 * Created by robbie on 4/23/2014.
 */
angular.module('ad.ui.bootstrap',
    [
        'ui.bootstrap',
        'ui.bootstrap.dropdown',
        'ad.bootstrap.templates',
        'ad.ui.services',
        'ad.ui.dialogService',
        'ad.ui.bootstrap.searchBox',
        'ad.ui.bootstrap.navigation',
        'ad.ui.bootstrap.list',
        'ad.ui.bootstrap.table',
        'ad.ui.bootstrap.vehicleQuotes',
        'ad.ui.zoomContainer',
        'ad.ui.bootstrap.form-elements',
        'ad.ui.bootstrap.pdf',
        'angular-multiple-transclusion',
        // 'multi-transclude',
        'ad.ui.bootstrap.ad-action-bar',
        'ad.ui.bootstrap.ad-dropdown',
        'ad.ui.bootstrap.ad-list',
        'ad.ui.bootstrap.ad-simple-list',
        'ad.ui.bootstrap.ad-icon',
        'ad.ui.bootstrap.ad-vehicle-block',
        'ad.ui.bootstrap.ad-view',
        'ad.ui.bootstrap.ad-breadcrumb',
        'ad.ui.bootstrap.ad-flyout',
        'ad.ui.bootstrap.ad-flyoutDialog',
        'ad.ui.bootstrap.ad-rich-content-button',
        'ad.ui.bootstrap.conversion-calculator'
    ]
);
var ActionBar;
(function (ActionBar) {
    var ActionBarDirective = (function () {
        function ActionBarDirective() {
        }
        ActionBarDirective.init = function ($window, $filter, FlyoutDialogService) {
            var directive = {
                restrict: 'AE',
                replace: true,
                templateUrl: 'action-bar/action-bar.html',
                scope: {
                    actions: '=',
                    showTitles: '=',
                    alignment: '@',
                    overflowTreshold: '=?'
                },
                link: function ($scope, element, attr) {
                    $scope.overflowTreshold = $scope.overflowTreshold || { 'xs': 8, 'sm': 8, 'md': 8, 'lg': 8 };
                    var window = angular.element($window);
                    var calcAppSize = function (width) {
                        if (width > 1199) {
                            $scope.appSize = 'lg';
                        }
                        else if (width > 991) {
                            $scope.appSize = 'md';
                        }
                        else if (width > 767) {
                            $scope.appSize = 'sm';
                        }
                        else {
                            $scope.appSize = 'xs';
                        }
                    };
                    calcAppSize(window.width());
                    $scope.alignment = $scope.alignment || 'right';
                    $scope.selectAction = function (action, $event) {
                        if (!action.template) {
                            if (action.children) {
                                var subActionsScope = $scope.$new(true);
                                subActionsScope.selectSubAction = function (subAction) {
                                    subAction.onSelect();
                                    $('body').click();
                                };
                                subActionsScope.subActions = action.children;
                                FlyoutDialogService.showFlyout($event.target, 'subActions.html', {
                                    placement: 'bottom'
                                }, subActionsScope);
                            }
                            else {
                                action.onSelect();
                            }
                        }
                        else {
                            if ($event.target.parentElement.classList.contains('action')) {
                                FlyoutDialogService.showFlyout($event.target, action.template, {
                                    placement: action.templatePlacement,
                                    anchor: action.templateAnchor
                                }, action.scope);
                            }
                        }
                    };
                    $scope.showMore = function (actions, limit, $event) {
                        var subActionsScope = $scope.$new(true);
                        subActionsScope.selectSubAction = function (subAction) {
                            subAction.onSelect();
                            $('body').click();
                        };
                        subActionsScope.subActions = $filter('limitTo')(actions, actions.length - limit, limit);
                        FlyoutDialogService.showFlyout($event.target, 'subActions.html', {
                            placement: 'bottom'
                        }, subActionsScope);
                    };
                    window.bind('resize', function () {
                        calcAppSize(window.width());
                        $scope.$apply();
                    });
                }
            };
            return directive;
        };
        return ActionBarDirective;
    })();
    ActionBar.ActionBarDirective = ActionBarDirective;
    angular.module('ad.ui.bootstrap.ad-action-bar', []).directive('adActionBar', ['$window', '$filter', 'adFlyoutDialogService', ActionBarDirective.init]);
})(ActionBar || (ActionBar = {}));
var BreadCrumb;
(function (BreadCrumb) {
    var AdBreadcrumbDirective = (function () {
        function AdBreadcrumbDirective() {
        }
        AdBreadcrumbDirective.init = function () {
            var directive = {
                restrict: 'AE',
                replace: true,
                templateUrl: 'ad-breadcrumb/ad-breadcrumb.html',
                scope: {
                    trail: '=',
                    onSelect: '&?'
                },
                link: function ($scope, element, attr) {
                    $scope.$watch('trail', function (newValue) {
                        if (newValue) {
                            $scope.reverseTrail = newValue.slice(0).reverse();
                        }
                    });
                    $scope.breadCrumbSelect = function (item) {
                        $scope.onSelect && $scope.onSelect({ item: item });
                    };
                }
            };
            return directive;
        };
        return AdBreadcrumbDirective;
    })();
    BreadCrumb.AdBreadcrumbDirective = AdBreadcrumbDirective;
    angular.module('ad.ui.bootstrap.ad-breadcrumb', []).directive('adBreadcrumb', AdBreadcrumbDirective.init);
})(BreadCrumb || (BreadCrumb = {}));
angular.module('ad.ui.bootstrap.ad-flyout', ['ui.bootstrap.position', 'ui.bootstrap.bindHtml']);
angular.module('ad.ui.bootstrap.ad-flyoutDialog', ['ui.bootstrap']).service('adFlyoutDialogService', ['$window', '$compile', '$timeout', '$document', '$position', '$interpolate', '$rootScope', '$parse', '$templateCache', function ($window, $compile, $timeout, $document, $position, $interpolate, $rootScope, $parse, $templateCache) {
    this.showFlyout = function (_hostElement, templateUrl, options, scope) {
        if (scope === void 0) { scope = $rootScope.$new(); }
        var hostElement = angular.element(_hostElement);
        var template = angular.element('<div class="popover flyout"><div class="popover-inner"></div></div>)');
        var flyoutContent = angular.element('<div class="popover-content"></div>');
        flyoutContent.append($templateCache.get(templateUrl));
        template.append(flyoutContent);
        var placement = options.placement || 'bottom';
        var anchor = options.anchor;
        var appendToBody = options.appendToBody;
        var tooltipLinker = $compile(template);
        var tooltip;
        var repositionScheduled = false;
        var arrow = angular.element('<div class="arrow"></div>');
        showTooltipBind();
        function positionTooltip() {
            if (!tooltip) {
                return;
            }
            var ttPosition = $position.positionElements(hostElement, tooltip, placement, appendToBody);
            if (anchor) {
                var hostPosition = $position.position(hostElement);
                var tooltipPosition = $position.position(tooltip);
                if (anchor == 'left') {
                    ttPosition.left = hostPosition.left;
                }
                else if (anchor == 'right') {
                    ttPosition.left = hostPosition.left + hostPosition.width - tooltipPosition.width;
                }
            }
            tooltip.css({
                top: ttPosition.top + 'px',
                left: ttPosition.left + 'px'
            });
            var ttOffset = $position.offset(tooltip);
            var clientWidth = window.innerWidth;
            var clientHeight = window.innerHeight;
            var rightOffset = clientWidth - (ttOffset.left + ttOffset.width);
            if (rightOffset < 0) {
                ttPosition.left = ttPosition.left + rightOffset - 5;
            }
            if (ttOffset.left < 0) {
                ttPosition.left = 0;
            }
            tooltip.css({
                top: ttPosition.top + 'px',
                left: ttPosition.left + 'px'
            });
            var arrowPosition = $position.positionElements(hostElement, arrow, placement, false);
            arrowPosition.top += 'px';
            arrowPosition.left += 'px';
            arrow.css(arrowPosition);
            tooltip.addClass(placement);
            if (placement == 'left') {
                arrow.addClass('arrow-right');
            }
            else if (placement == 'right') {
                arrow.addClass('arrow-left');
            }
            else if (placement == 'top') {
                arrow.addClass('arrow-down');
            }
            else if (placement == 'bottom') {
                arrow.addClass('arrow-up');
            }
        }
        ;
        var positionTooltipAsync = function () {
            $timeout(positionTooltip, 0, false);
        };
        $timeout(function () {
            angular.element(document.body).bind('click', function (e) {
                if (tooltip) {
                    if (tooltip != e.target && hostElement != e.target && !hostElement[0].contains(e.target) && !tooltip[0].contains(e.target) && !(hostElement.attr('id') == e.target.id)) {
                        hide();
                    }
                }
            });
        }, 200);
        function showTooltipBind() {
            show()();
        }
        function hideTooltipBind() {
            hide();
            if (!$rootScope.$$phase) {
                $rootScope.$digest();
            }
        }
        function show() {
            createTooltip();
            tooltip.css({ top: 0, left: 0, display: 'block' });
            positionTooltip();
            return positionTooltip;
        }
        function hide() {
            removeTooltip();
        }
        function createTooltip() {
            if (tooltip) {
                removeTooltip();
            }
            tooltip = tooltipLinker(scope, function (tooltip) {
                if (appendToBody) {
                    $document.find('body').append(tooltip);
                }
                else {
                    hostElement.after(tooltip);
                }
            });
            hostElement.after(arrow);
            scope.$watch(function () {
                if (!repositionScheduled) {
                    repositionScheduled = true;
                    scope.$$postDigest(function () {
                        repositionScheduled = false;
                        positionTooltipAsync();
                    });
                }
            });
        }
        function removeTooltip() {
            if (tooltip) {
                arrow.remove();
                tooltip.remove();
                tooltip = null;
            }
        }
        return {
            hide: hide
        };
    };
    function snake_case(name) {
        var regexp = /[A-Z]/g;
        var separator = '-';
        return name.replace(regexp, function (letter, pos) {
            return (pos ? separator : '') + letter.toLowerCase();
        });
    }
}]);
angular.module('ad.ui.bootstrap.ad-flyout').directive('adFlyoutTemplatePopup', ['$document', function ($document) {
    return {
        restrict: 'EA',
        replace: true,
        scope: { title: '@', contentExp: '&', placement: '@', popupClass: '@', animation: '&', isOpen: '&', originScope: '&', anchor: '@' },
        templateUrl: 'ad-flyout/template/ad-flyout.html'
    };
}]).directive('adFlyoutTemplate', ['$flyout', function ($flyout) {
    return $flyout('adFlyoutTemplate', 'adFlyout', '', {
        useContentExp: true
    });
}]);
angular.module('ad.ui.bootstrap.ad-flyout').provider('$flyout', function () {
    var defaultOptions = {
        placement: 'top',
        animation: true,
        popupDelay: 0,
        useContentExp: false
    };
    var triggerMap = {
        'mouseenter': 'mouseleave',
        'click': 'click',
        'focus': 'blur',
        'none': ''
    };
    var globalOptions = {};
    this.options = function (value) {
        angular.extend(globalOptions, value);
    };
    this.setTriggers = function setTriggers(triggers) {
        angular.extend(triggerMap, triggers);
    };
    function snake_case(name) {
        var regexp = /[A-Z]/g;
        var separator = '-';
        return name.replace(regexp, function (letter, pos) {
            return (pos ? separator : '') + letter.toLowerCase();
        });
    }
    this.$get = ['$window', '$compile', '$timeout', '$document', '$position', '$interpolate', '$rootScope', '$parse', function ($window, $compile, $timeout, $document, $position, $interpolate, $rootScope, $parse) {
        return function $flyout(type, prefix, defaultTriggerShow, options, hostElement) {
            options = angular.extend({}, defaultOptions, globalOptions, options);
            function getTriggers(trigger) {
                var show = (trigger || options.trigger || defaultTriggerShow).split(' ');
                var hide = show.map(function (trigger) {
                    return triggerMap[trigger] || trigger;
                });
                return {
                    show: show,
                    hide: hide
                };
            }
            var directiveName = snake_case(type);
            var startSym = $interpolate.startSymbol();
            var endSym = $interpolate.endSymbol();
            var template = '<div ' + directiveName + '-popup ' + 'title="' + startSym + 'title' + endSym + '" ' + (options.useContentExp ? 'content-exp="contentExp()" ' : 'content="' + startSym + 'content' + endSym + '" ') + 'placement="' + startSym + 'placement' + endSym + '" ' + 'popup-class="' + startSym + 'popupClass' + endSym + '" ' + 'animation="animation" ' + 'is-open="isOpen"' + 'origin-scope="origScope" ' + '>' + '</div>';
            return {
                restrict: 'EA',
                compile: function (tElem, tAttrs) {
                    var tooltipLinker = $compile(template);
                    return function link(scope, element, attrs, tooltipCtrl) {
                        var tooltip;
                        var tooltipLinkedScope;
                        var transitionTimeout;
                        var popupTimeout;
                        var appendToBody = angular.isDefined(options.appendToBody) ? options.appendToBody : false;
                        var triggers = getTriggers(undefined);
                        var hasEnableExp = angular.isDefined(attrs[prefix + 'Enable']);
                        var ttScope = scope.$new(true);
                        var repositionScheduled = false;
                        var isOpenExp = angular.isDefined(attrs[prefix + 'IsOpen']) ? $parse(attrs[prefix + 'IsOpen']) : false;
                        var arrow = angular.element('<div class="arrow"></div>');
                        var positionTooltip = function () {
                            if (!tooltip) {
                                return;
                            }
                            var ttPosition = $position.positionElements(element, tooltip, ttScope.placement, appendToBody);
                            if (ttScope.anchor) {
                                var hostPosition = $position.position(element);
                                var tooltipPosition = $position.position(tooltip);
                                if (ttScope.anchor == 'left') {
                                    ttPosition.left = hostPosition.left;
                                }
                                else if (ttScope.anchor == 'right') {
                                    ttPosition.left = hostPosition.left + hostPosition.width - tooltipPosition.width;
                                }
                            }
                            ttPosition.top += 'px';
                            ttPosition.left += 'px';
                            tooltip.css(ttPosition);
                            var arrowPosition = $position.positionElements(element, arrow, ttScope.placement, appendToBody);
                            arrowPosition.top += 'px';
                            arrowPosition.left += 'px';
                            arrow.css(arrowPosition);
                            if (ttScope.placement == 'left') {
                                arrow.addClass('arrow-right');
                            }
                            else if (ttScope.placement == 'right') {
                                arrow.addClass('arrow-left');
                            }
                            else if (ttScope.placement == 'top') {
                                arrow.addClass('arrow-down');
                            }
                            else if (ttScope.placement == 'bottom') {
                                arrow.addClass('arrow-up');
                            }
                        };
                        var positionTooltipAsync = function () {
                            $timeout(positionTooltip, 0, false);
                        };
                        $timeout(function () {
                            angular.element(document.body).bind('click', function (e) {
                                if (tooltip) {
                                    if (tooltip != e.target && element != e.target && !element[0].contains(e.target) && !tooltip[0].contains(e.target) && !(element.attr('id') == e.target.id)) {
                                        hide();
                                    }
                                }
                            });
                        }, 200);
                        ttScope.origScope = scope;
                        ttScope.isOpen = false;
                        function toggleTooltipBind() {
                            if (!ttScope.isOpen) {
                                showTooltipBind();
                            }
                            else {
                                hideTooltipBind();
                            }
                        }
                        function showTooltipBind() {
                            if (hasEnableExp && !scope.$eval(attrs[prefix + 'Enable'])) {
                                return;
                            }
                            prepareTooltip();
                            if (ttScope.popupDelay) {
                                if (!popupTimeout) {
                                    popupTimeout = $timeout(show, ttScope.popupDelay, false);
                                    popupTimeout.then(function (reposition) {
                                        reposition();
                                    });
                                }
                            }
                            else {
                                show()();
                            }
                        }
                        function hideTooltipBind() {
                            hide();
                            if (!$rootScope.$$phase) {
                                $rootScope.$digest();
                            }
                        }
                        function show() {
                            popupTimeout = null;
                            if (transitionTimeout) {
                                $timeout.cancel(transitionTimeout);
                                transitionTimeout = null;
                            }
                            if (!(options.useContentExp ? ttScope.contentExp() : ttScope.content)) {
                                return angular.noop;
                            }
                            createTooltip();
                            tooltip.css({ top: 0, left: 0, display: 'block' });
                            positionTooltip();
                            ttScope.isOpen = true;
                            if (isOpenExp && isOpenExp.assign) {
                                isOpenExp.assign(ttScope.origScope, ttScope.isOpen);
                            }
                            if (!$rootScope.$$phase) {
                                ttScope.$apply();
                            }
                            return positionTooltip;
                        }
                        function hide() {
                            ttScope.isOpen = false;
                            if (isOpenExp && isOpenExp.assign) {
                                isOpenExp.assign(ttScope.origScope, ttScope.isOpen);
                            }
                            arrow.remove();
                            $timeout.cancel(popupTimeout);
                            popupTimeout = null;
                            removeTooltip();
                        }
                        function createTooltip() {
                            if (tooltip) {
                                removeTooltip();
                            }
                            tooltip = tooltipLinker(ttScope, function (tooltip) {
                                if (appendToBody) {
                                    $document.find('body').append(tooltip);
                                }
                                else {
                                    element.after(tooltip);
                                }
                            });
                            element.after(arrow);
                            if (options.useContentExp) {
                                ttScope.$watch(function () {
                                    if (!repositionScheduled) {
                                        repositionScheduled = true;
                                        ttScope.$$postDigest(function () {
                                            repositionScheduled = false;
                                            if (ttScope.isOpen) {
                                                positionTooltipAsync();
                                            }
                                        });
                                    }
                                });
                            }
                        }
                        function removeTooltip() {
                            transitionTimeout = null;
                            if (tooltip) {
                                tooltip.remove();
                                tooltip = null;
                            }
                            if (tooltipLinkedScope) {
                                tooltipLinkedScope.$destroy();
                                tooltipLinkedScope = null;
                            }
                        }
                        function prepareTooltip() {
                            prepPopupClass();
                            prepPlacement();
                            prepPopupDelay();
                            prepAnchor();
                        }
                        ttScope.contentExp = function () {
                            var content = scope.$eval(attrs[type]);
                            return content;
                        };
                        if (!options.useContentExp) {
                            attrs.$observe(type, function (val) {
                                ttScope.content = val;
                                if (!val && ttScope.isOpen) {
                                    hide();
                                }
                                else {
                                    positionTooltipAsync();
                                }
                            });
                        }
                        attrs.$observe('disabled', function (val) {
                            if (popupTimeout && val) {
                                $timeout.cancel(popupTimeout);
                                popupTimeout = null;
                            }
                            if (val && ttScope.isOpen) {
                                hide();
                            }
                        });
                        attrs.$observe(prefix + 'Title', function (val) {
                            ttScope.title = val;
                            positionTooltipAsync();
                        });
                        attrs.$observe(prefix + 'Placement', function () {
                            if (ttScope.isOpen) {
                                $timeout(function () {
                                    prepPlacement();
                                    show()();
                                }, 0, false);
                            }
                        });
                        if (isOpenExp) {
                            scope.$watch(isOpenExp, function (val) {
                                if (val !== ttScope.isOpen) {
                                    toggleTooltipBind();
                                }
                            });
                        }
                        function prepPopupClass() {
                            ttScope.popupClass = attrs[prefix + 'Class'];
                        }
                        function prepPlacement() {
                            var val = attrs[prefix + 'Placement'];
                            ttScope.placement = angular.isDefined(val) ? val : options.placement;
                        }
                        function prepAnchor() {
                            var val = attrs[prefix + 'Anchor'];
                            ttScope.anchor = angular.isDefined(val) ? val : options.anchor;
                        }
                        function prepPopupDelay() {
                            var val = attrs[prefix + 'PopupDelay'];
                            var delay = parseInt(val, 10);
                            ttScope.popupDelay = !isNaN(delay) ? delay : options.popupDelay;
                        }
                        var unregisterTriggers = function () {
                            triggers.show.forEach(function (trigger) {
                                element.unbind(trigger, showTooltipBind);
                            });
                            triggers.hide.forEach(function (trigger) {
                                element.unbind(trigger, hideTooltipBind);
                            });
                        };
                        function prepTriggers() {
                            var val = attrs[prefix + 'Trigger'];
                            unregisterTriggers();
                            triggers = getTriggers(val);
                            if (triggers.show !== 'none') {
                                triggers.show.forEach(function (trigger, idx) {
                                    if (trigger === triggers.hide[idx]) {
                                        element.bind(trigger, toggleTooltipBind);
                                    }
                                    else if (trigger) {
                                        element.bind(trigger, showTooltipBind);
                                        element.bind(triggers.hide[idx], hideTooltipBind);
                                    }
                                });
                            }
                        }
                        prepTriggers();
                        var animation = scope.$eval(attrs[prefix + 'Animation']);
                        ttScope.animation = angular.isDefined(animation) ? !!animation : options.animation;
                        var appendToBodyVal = scope.$eval(attrs[prefix + 'AppendToBody']);
                        appendToBody = angular.isDefined(appendToBodyVal) ? appendToBodyVal : appendToBody;
                        if (appendToBody) {
                            scope.$on('$locationChangeSuccess', function closeTooltipOnLocationChangeSuccess() {
                                if (ttScope.isOpen) {
                                    hide();
                                }
                            });
                        }
                        scope.$on('$destroy', function onDestroyTooltip() {
                            $timeout.cancel(transitionTimeout);
                            $timeout.cancel(popupTimeout);
                            unregisterTriggers();
                            removeTooltip();
                            ttScope = null;
                        });
                    };
                }
            };
        };
    }];
});
var ActionBar;
(function (ActionBar) {
    var AdListDirective = (function () {
        function AdListDirective() {
        }
        AdListDirective.init = function () {
            var directive = {
                restrict: 'AE',
                transclude: true,
                replace: true,
                templateUrl: 'ad-list/ad-list.html',
                scope: {
                    items: '=',
                    allowSearch: '=?',
                    orderBy: '=?',
                    orderReverse: '=?',
                    searchPlaceholder: '@',
                    icon: '@',
                    template: '=?',
                    onSelect: '&?',
                    startSelectedItem: '=?',
                    showDetailsPane: '=?'
                },
                link: function ($scope, element, attr) {
                    if ($scope.startSelectedItem) {
                        $scope.selectedItem = $scope.startSelectedItem;
                    }
                    $scope.$watch('startSelectedItem', function (newValue) {
                        $scope.selectedItem = newValue;
                    });
                    var options = {
                        searchTerm: ''
                    };
                    $scope.options = options;
                    $scope.template = $scope.template || 'ad-list/templates/default.html';
                    if ($scope.items && $scope.items.length > 0) {
                        $scope.itemIsString = typeof $scope.items[0] === 'string';
                    }
                    $scope.allowSearch = $scope.allowSearch || false;
                    $scope.select = function (item, $event, $index) {
                        $scope.selectedItem = item;
                        $scope.onSelect && $scope.onSelect({ item: item, $event: $event, $index: $index });
                    };
                }
            };
            return directive;
        };
        return AdListDirective;
    })();
    ActionBar.AdListDirective = AdListDirective;
    angular.module('ad.ui.bootstrap.ad-list', []).directive('adList', AdListDirective.init);
})(ActionBar || (ActionBar = {}));
var ADView;
(function (ADView) {
    var ADRichContentButtonDirective = (function () {
        function ADRichContentButtonDirective() {
        }
        ADRichContentButtonDirective.init = function () {
            var directive = {
                restrict: 'AE',
                replace: true,
                transclude: true,
                templateUrl: 'ad-rich-content-button/ad-rich-content-button.html',
                scope: {
                    linkType: '=',
                    icon: '=?'
                },
                link: function ($scope, element, attr) {
                    $scope.linkType = $scope.linkType || 'none';
                    $scope.noBorder = 'noBorder' in attr;
                },
            };
            return directive;
        };
        return ADRichContentButtonDirective;
    })();
    ADView.ADRichContentButtonDirective = ADRichContentButtonDirective;
    var ADRichContentButtonGroupDirective = (function () {
        function ADRichContentButtonGroupDirective() {
        }
        ADRichContentButtonGroupDirective.init = function () {
            var directive = {
                restrict: 'AE',
                replace: true,
                transclude: true,
                templateUrl: 'ad-rich-content-button/ad-rich-content-button-group.html',
                scope: {
                    direction: '@'
                },
                link: function ($scope, element, attr) {
                    $scope.direction = $scope.direction || 'horizontal';
                    element.find('.ad-rich-content-button').wrap('<div class="ad-rich-button-group-cell"></div>');
                }
            };
            return directive;
        };
        return ADRichContentButtonGroupDirective;
    })();
    ADView.ADRichContentButtonGroupDirective = ADRichContentButtonGroupDirective;
    angular.module('ad.ui.bootstrap.ad-rich-content-button', []).directive('adRichContentButton', ADRichContentButtonDirective.init).directive('adRichContentButtonGroup', ADRichContentButtonGroupDirective.init);
})(ADView || (ADView = {}));
var ADSimpleList;
(function (ADSimpleList) {
    var AdSimpleListDirective = (function () {
        function AdSimpleListDirective() {
        }
        AdSimpleListDirective.init = function () {
            var directive = {
                restrict: 'AE',
                replace: true,
                transclude: true,
                templateUrl: 'ad-simple-list/ad-simple-list.html',
                scope: {
                    icon: '=',
                    iconSize: '=',
                    listHeader: '='
                },
                link: function ($scope, element, attr) {
                }
            };
            return directive;
        };
        return AdSimpleListDirective;
    })();
    ADSimpleList.AdSimpleListDirective = AdSimpleListDirective;
    angular.module('ad.ui.bootstrap.ad-simple-list', []).directive('adSimpleList', AdSimpleListDirective.init);
})(ADSimpleList || (ADSimpleList = {}));
var VehicleBlock;
(function (VehicleBlock) {
    var VehicleBlockDirective = (function () {
        function VehicleBlockDirective() {
        }
        VehicleBlockDirective.init = function () {
            var directive = {
                restrict: 'AE',
                replace: true,
                templateUrl: 'ad-vehicle-block/ad-vehicle-block.html',
                scope: {
                    vehicle: '=',
                    size: '=',
                    inverse: '=?',
                    showIcon: '='
                },
                link: function ($scope, element, attr) {
                    $scope.showIcon = angular.isDefined($scope.showIcon) ? $scope.showIcon : true;
                }
            };
            return directive;
        };
        return VehicleBlockDirective;
    })();
    VehicleBlock.VehicleBlockDirective = VehicleBlockDirective;
    angular.module('ad.ui.bootstrap.ad-vehicle-block', []).directive('adVehicleBlock', VehicleBlockDirective.init);
})(VehicleBlock || (VehicleBlock = {}));
var AD;
(function (AD) {
    var UI;
    (function (UI) {
        var Bootstrap;
        (function (Bootstrap) {
            var ADView;
            (function (ADView) {
                var ADViewDirective = (function () {
                    function ADViewDirective() {
                    }
                    ADViewDirective.init = function () {
                        var directive = {
                            restrict: 'AE',
                            replace: true,
                            transclude: true,
                            templateUrl: 'ad-view/ad-view.html',
                            scope: {
                                headerSize: '='
                            },
                            link: function ($scope, element, attr) {
                                $scope.headerSize = $scope.headerSize || 'medium';
                            }
                        };
                        return directive;
                    };
                    return ADViewDirective;
                })();
                ADView.ADViewDirective = ADViewDirective;
                angular.module('ad.ui.bootstrap.ad-view', []).directive('adView', ADViewDirective.init);
            })(ADView = Bootstrap.ADView || (Bootstrap.ADView = {}));
        })(Bootstrap = UI.Bootstrap || (UI.Bootstrap = {}));
    })(UI = AD.UI || (AD.UI = {}));
})(AD || (AD = {}));
angular.module('ad.bootstrap.templates', ['action-bar/action-bar.html', 'ad-breadcrumb/ad-breadcrumb.html', 'ad-flyout/template/ad-flyout.html', 'ad-list/ad-list.html', 'ad-list/templates/default.html', 'ad-rich-content-button/ad-rich-content-button-group.html', 'ad-rich-content-button/ad-rich-content-button.html', 'ad-simple-list/ad-simple-list.html', 'ad-vehicle-block/ad-vehicle-block.html', 'ad-view/ad-view.html', 'conversion-calculator/conversionCalculator.html', 'dialog/dialog.html', 'dialog/dialogShell.html', 'dropdown/ad-dropdown.html', 'fonts/directive/ad-icon.html', 'form-elements/form-field-info/form-field-info.html', 'form-elements/form-input/form-input.html', 'list/partials/itemList.html', 'list/partials/itemShortcuts.html', 'list/partials/list.html', 'navigation/partials/ad-shell.html', 'navigation/partials/alldataLogo.html', 'navigation/partials/menu.html', 'navigation/partials/navigation.html', 'navigation/partials/profile.html', 'navigation/partials/shell-content.html', 'navigation/partials/shell-topnav.html', 'navigation/partials/subMenuItem.html', 'navigation/partials/topMenuItem.html', 'pdf/partials/pdfViewerTemplate.html', 'pdf/viewer/viewer.html', 'search-box/search-box.html', 'table/partials/pagination.html', 'table/partials/table.html', 'vehicles-quotes/partials/quote.html', 'vehicles-quotes/partials/vehicle.html', 'vehicles-quotes/partials/vehiclesQuotes.html', 'zoom-container/zoom-container.html']);
angular.module("action-bar/action-bar.html", []).run(["$templateCache", function ($templateCache) {
    $templateCache.put("action-bar/action-bar.html", "<ul class='ad-action-bar' ng-class=\"{'pull-left': alignment == 'left'}\">\n" + "	<li class=\"action\" ng-class=\"{'separator': action.separator}\" ng-repeat=\"action in actions | limitTo : overflowTreshold[appSize]\" ng-click=\"selectAction(action, $event)\" title=\"{{ action.title }}\">\n" + "		<i ng-class=\"action.icon\"></i>\n" + "		<span ng-bind=\"action.title\" ng-if=\"showTitles\" class=\"action-title\"></span>\n" + "		<span class=\"badge\" ng-bind=\"action.badge\"></span>\n" + "		<!--<i class=\"appIcon-arrowDown\" ng-if=\"action.children\"></i>-->\n" + "	</li>\n" + "    <li class=\"action\" ng-click=\"showMore(actions, overflowTreshold[appSize], $event)\" title=\"more\" ng-if=\"overflowTreshold[appSize] < actions.length\">\n" + "		<i class=\"appIcon-more\" title=\"more\"></i>\n" + "		<!--<span ng-if=\"showTitles\" class=\"action-title\">More</span>-->\n" + "		<!--<i class=\"appIcon-arrowDown\"></i>-->\n" + "	</li>\n" + "	<script type=\"text/ng-template\" id=\"subActions.html\" ng-if=\"action.children\">\n" + "		<ul class=\"ad-action-bar with-titles inverse sub-actions\">\n" + "			<li class=\"action no-float\" ng-class=\"{'separator': subAction.separator}\" ng-repeat=\"subAction in subActions\" ng-click=\"selectSubAction(subAction)\" title=\"{{ subAction.title }}\">\n" + "				<i ng-class=\"subAction.icon\"></i>\n" + "				<span ng-bind=\"subAction.title\"></span>\n" + "				<span class=\"badge\" ng-bind=\"action.badge\"></span>\n" + "			</li>\n" + "		</ul>\n" + "	</script>\n" + "</ul>");
}]);
angular.module("ad-breadcrumb/ad-breadcrumb.html", []).run(["$templateCache", function ($templateCache) {
    $templateCache.put("ad-breadcrumb/ad-breadcrumb.html", "<div class='ad-breadcrumb'>\n" + "  <i ng-if=\"trail.length == 1\" class=\"appIcon-arrow_back\" style=\"font-size:24px\" ng-click=\"breadCrumbSelect(trail[0])\"></i>\n" + "  <i ng-if=\"trail.length > 1\" class=\"appIcon-arrow_back\" style=\"font-size:24px\" ad-flyout-template=\"'breadCrumbTrail.html'\" ad-flyout-is-open=\"trail.show\"\n" + "    ad-flyout-placement=\"bottom\" ad-flyout-anchor=\"left\" ng-init=\"trail.show = false\" ng-click=\"trail.show = !trail.show\"></i>\n" + "  <!--<div class=\"btn-group btn-group-lg\" ng-repeat=\"crumb in trail | limitTo:-1 track by $index\">\n" + "    <button type=\"button\" class=\"btn btn-default\" ng-click=\"breadCrumbSelect(crumb)\" >\n" + "      <i class=\"appIcon-arrow_back\"></i>\n" + "    </button>\n" + "    <button type=\"button\" class=\"btn btn-default\" ad-flyout-template=\"'breadCrumbTrail.html'\" ad-flyout-placement=\"bottom\">\n" + "      <span>\n" + "        {{ crumb.title }}\n" + "      </span>\n" + "      <i class=\"appIcon-arrowDown\" style=\"margin-left: 15px;\"></i>\n" + "    </button>\n" + "  </div>-->\n" + "  <script type=\"text/ng-template\" id=\"breadCrumbTrail.html\">\n" + "    <ul>\n" + "      <li ng-repeat=\"crumb in $parent.reverseTrail track by $index\">\n" + "        <a ng-if=\"!crumb.isPassthrough\" ng-click=\"$parent.breadCrumbSelect(crumb)\" >{{ crumb.title }}</a>\n" + "        <span ng-if=\"crumb.isPassthrough\">{{ crumb.title }} </span>\n" + "      </li>\n" + "    </ul>\n" + "  </script>\n" + "</div>\n" + "");
}]);
angular.module("ad-flyout/template/ad-flyout.html", []).run(["$templateCache", function ($templateCache) {
    $templateCache.put("ad-flyout/template/ad-flyout.html", "<div class=\"popover flyout\"\n" + "  tooltip-animation-class=\"fade\"\n" + "  tooltip-classes\n" + "  ng-class=\"[{ in: isOpen },'anchor-' + anchor ]\">\n" + "  <!--{ in: isOpen() }, -->\n" + "  <div class=\"popover-inner\">\n" + "      <h3 class=\"popover-title\" ng-bind=\"title\" ng-if=\"title\"></h3>\n" + "      <div class=\"popover-content\"\n" + "        tooltip-template-transclude=\"contentExp()\"\n" + "        tooltip-template-transclude-scope=\"originScope()\"></div>\n" + "  </div>\n" + "</div>");
}]);
angular.module("ad-list/ad-list.html", []).run(["$templateCache", function ($templateCache) {
    $templateCache.put("ad-list/ad-list.html", "<div class='ad-list' ng-class=\"{'with-details-pane':showDetailsPane}\">\n" + "  <div class=\"ad-list-section\">\n" + "    <div class=\"search-box-wrapper\" ng-if=\"allowSearch\" >\n" + "      <search-box placeholder=\"{{ searchPlaceholder }}\" search-term=\"options.searchTerm\" number-of-character-delay=\"0\" time-delay=\"1\"></search-box>\n" + "    </div>\n" + "    <ul ng-class=\"{'with-search': allowSearch}\">\n" + "      <li ng-repeat=\"item in items | filter:options.searchTerm | orderBy:orderBy:orderReverse track by $index\" ng-click=\"select(item, $event, $index)\" >\n" + "        <div ng-include=\"template\" class=\"list-item-container\" ng-class=\"{'selected': item === selectedItem}\"></div>\n" + "      </li>\n" + "    </ul>\n" + "  </div>\n" + "  <div class=\"ad-list-details-pane\" ng-if=\"showDetailsPane\" ng-transclude>\n" + "    \n" + "  </div>\n" + "</div>");
}]);
angular.module("ad-list/templates/default.html", []).run(["$templateCache", function ($templateCache) {
    $templateCache.put("ad-list/templates/default.html", "<div class=\"list-item-default\">\n" + "	<i ng-if=\"item.icon\" ng-class=\"item.icon\" class=\"list-item-icon\"></i>\n" + "	<i ng-if=\" !item.icon && $parent.icon\" ng-class=\"$parent.icon\" class=\"list-item-icon\"></i>\n" + "	<div class=\"list-item\">\n" + "		<div class=\"list-item-title\">{{ itemIsString ? item : item.title }} </div>\n" + "		<div ng-if=\"item.subTitle\" class=\"list-item-sub-title\">{{ item.subTitle }}</div>\n" + "	</div>\n" + "</div>");
}]);
angular.module("ad-rich-content-button/ad-rich-content-button-group.html", []).run(["$templateCache", function ($templateCache) {
    $templateCache.put("ad-rich-content-button/ad-rich-content-button-group.html", "<div class=\"ad-rich-content-button-group\" ng-class=\"direction\">\n" + "	<div class=\"ad-rich-content-button-group-buttons\" ng-transclude>\n" + "		\n" + "	</div>\n" + "</div>\n" + "");
}]);
angular.module("ad-rich-content-button/ad-rich-content-button.html", []).run(["$templateCache", function ($templateCache) {
    $templateCache.put("ad-rich-content-button/ad-rich-content-button.html", "<div class=\"ad-rich-content-button\" ng-class=\"{'no-border': noBorder, 'with-icon': icon}\">\n" + "	<div class=\"ad-rich-content-icon\">\n" + "		<i ng-class=\"icon\" ng-if=\"icon\"></i>\n" + "	</div>\n" + "	<div class=\"ad-rich-content-button-content\" ng-transclude></div>\n" + "	<div class=\"ad-rich-content-button-link-icon\" ng-if=\"linkType != 'none'\">\n" + "		<i class=\"appIcon-arrowRight\" ng-if=\"linkType == 'link'\"></i>\n" + "		<i class=\"appIcon-arrowDown\" ng-if=\"linkType == 'action'\"></i>\n" + "	</div>\n" + "</div>\n" + "");
}]);
angular.module("ad-simple-list/ad-simple-list.html", []).run(["$templateCache", function ($templateCache) {
    $templateCache.put("ad-simple-list/ad-simple-list.html", "<div class='ad-simple-list'>\n" + "  <div ng-if=\"icon\" class=\"ad-simple-list-icon\">\n" + "    <ad-icon icon-class=\"icon\" size=\"iconSize\"></ad-icon>\n" + "  </div>\n" + "  <div class=\"ad-simple-list-content\">\n" + "    <div class=\"ad-simple-list-header\" ng-if=\"listHeader\">\n" + "      {{ listHeader }}\n" + "    </div>\n" + "    <ul ng-transclude>\n" + "      \n" + "    </ul>\n" + "  </div>\n" + "</div>");
}]);
angular.module("ad-vehicle-block/ad-vehicle-block.html", []).run(["$templateCache", function ($templateCache) {
    $templateCache.put("ad-vehicle-block/ad-vehicle-block.html", "<div class=\"ad-vehicle-block\" ng-class=\"['ad-vehicle-block-' + size, {'inverse' : inverse}]\">\n" + "	<ad-icon size=\"'micro'\" icon-class=\"'appIcon-car'\" ng-if=\"showIcon\"></ad-icon>\n" + "	<div class=\"vehicle-content\">\n" + "		<div class=\"YMM\">{{vehicle.year}} {{vehicle.make}} {{vehicle.model}}</div>\n" + "		<div class=\"engine\">{{ vehicle.engine }}</div>\n" + "		<div class=\"plate-miles\">\n" + "			<span ng-if=\"vehicle.licensePlate\">License Plate: {{ vehicle.licensePlate }}</span>\n" + "			<span ng-if=\"vehicle.licensePlate && vehicle.mileage\">, </span>\n" + "			<span ng-if=\"vehicle.mileage\">{{ vehicle.mileage }} miles</span>\n" + "		</div>\n" + "		<div class=\"vin\">{{ vehicle.vin }}</div>\n" + "	</div>\n" + "</div>");
}]);
angular.module("ad-view/ad-view.html", []).run(["$templateCache", function ($templateCache) {
    $templateCache.put("ad-view/ad-view.html", "<div class=\"ad-view\" ad-multi-transclude ng-class=\"'view-header-' + headerSize\">\n" + "	<div class=\"view-header\"  transclude-from='header'></div>\n" + "	<div class=\"view-content\" transclude-from='content'></div>\n" + "</div>\n" + "");
}]);
angular.module("conversion-calculator/conversionCalculator.html", []).run(["$templateCache", function ($templateCache) {
    $templateCache.put("conversion-calculator/conversionCalculator.html", "<div>\n" + "	  <ul class=\"conversionOptions\">\n" + "  		<li ng-class=\"{'selected': convVm.currentView == 'area'}\"><a ng-click=\"convVm.setCurrentView('area')\">Area</a></li>\n" + "      <li ng-class=\"{'selected': convVm.currentView == 'length'}\"><a ng-click=\"convVm.setCurrentView('length')\">Distance/Length</a></li>\n" + "      <li ng-class=\"{'selected': convVm.currentView == 'force'}\"><a ng-click=\"convVm.setCurrentView('force')\">Force</a></li>\n" + "      <li ng-class=\"{'selected': convVm.currentView == 'pressure'}\"><a ng-click=\"convVm.setCurrentView('pressure')\">Pressure</a></li>\n" + "      <li ng-class=\"{'selected': convVm.currentView == 'temperature'}\"><a ng-click=\"convVm.setCurrentView('temperature')\">Temperature</a></li>\n" + "      <li ng-class=\"{'selected': convVm.currentView == 'torque'}\"><a ng-click=\"convVm.setCurrentView('torque')\">Torque</a></li>\n" + "      <li ng-class=\"{'selected': convVm.currentView == 'vacuum'}\"><a ng-click=\"convVm.setCurrentView('vacuum')\">Vacuum</a></li>\n" + "      <li ng-class=\"{'selected': convVm.currentView == 'volume'}\"><a ng-click=\"convVm.setCurrentView('volume')\">Volume</a></li>\n" + "      <li ng-class=\"{'selected': convVm.currentView == 'weight'}\"><a ng-click=\"convVm.setCurrentView('weight')\">Weight</a></li>\n" + "  	</ul>\n" + "  	<div class=\"conversionView\">\n" + "      <div class=\"conversionUnit conversionFrom\">\n" + "        <div class=\"input-group\">\n" + "          <input type=\"number\" step=\"0.01\" class=\"form-control\" ng-model=\"convVm.fromConversion.value\" ng-change=\"convVm.convert(convVm.currentView,convVm.fromConversion.type)\">\n" + "          <div class=\"input-group-btn\">\n" + "            <ad-dropdown class=\"ad-dropdown\" items=\"convVm.conversions[convVm.currentView]\" input-size='md' display-property=\"'label'\" ng-model=\"convVm.fromConversion\" ></ad-dropdown>\n" + "          </div>\n" + "        </div>\n" + "        <span class=\"conversionEquals\">=</span>\n" + "\n" + "      </div>\n" + "      \n" + "      <div class=\"conversionUnit\">\n" + "        <div class=\"input-group\" ng-show=\"convVm.viewMode == 'collapsed'\">\n" + "          <input type=\"number\" step=\"0.01\" class=\"form-control\" ng-model=\"convVm.toConversion.value\" ng-change=\"convVm.convert(convVm.currentView,convVm.toConversion.type)\">\n" + "          <div class=\"input-group-btn\">\n" + "            <ad-dropdown class=\"ad-dropdown\" items=\"convVm.conversions[convVm.currentView]\" input-size='md' display-property=\"'label'\" ng-model=\"convVm.toConversion\" ></ad-dropdown>\n" + "          </div>\n" + "        </div>\n" + "      </div>\n" + "      \n" + "      <div class=\"conversionResults\">\n" + "        <div ng-repeat=\"conversion in convVm.conversions[convVm.currentView]\" class=\"conversionUnit\" ng-if=\"conversion.type !== convVm.fromConversion.type\">\n" + "          <input type=\"number\" step=\"0.01\" ng-model=\"conversion.value\" ng-change=\"convVm.convert(convVm.currentView,conversion.type)\">\n" + "          <label ng-bind=\"conversion.label\"></label>\n" + "        </div>\n" + "      </div>\n" + "    </div>\n" + "\n" + "  </div>");
}]);
angular.module("dialog/dialog.html", []).run(["$templateCache", function ($templateCache) {
    $templateCache.put("dialog/dialog.html", "<div class=\"modal-header\">\n" + "    <h3>\n" + "        {{ title }}\n" + "        <i ng-show=\"showDismissButton\" class=\"icon appIcon-close\" ng-click=\"dismiss()\" ></i>\n" + "    </h3>\n" + "</div>\n" + "<div class=\"modal-body\">\n" + "    {{ message }}\n" + "    <form role=\"form\">\n" + "        <div class=\"form-group\" ng-repeat=\"item in inputs\">\n" + "            <label>{{ item.label }}</label>\n" + "            <input stop-event=\"touchend\" type=\"text\" class=\"form-control\" ng-model=\"item.value\" maxlength=\"{{item.maxLength}}\">\n" + "        </div>\n" + "    </form>\n" + "    <div ng-include=\"templateUrl\"></div>\n" + "</div>\n" + "<div class=\"modal-footer\" ng-if=\"buttons\">\n" + "    <button class=\"btn btn-default\" ng-repeat=\"button in buttons\"  ng-click=\"click($index)\" ng-class=\"{'btn-primary': $last}\">{{ button }}</button>\n" + "</div>");
}]);
angular.module("dialog/dialogShell.html", []).run(["$templateCache", function ($templateCache) {
    $templateCache.put("dialog/dialogShell.html", "<div ng-class=\"{'no-footer': !buttons && !hasFooterContent}\">\n" + "    <div class=\"modal-header\">\n" + "        <h3>{{ title }}\n" + "            <i class=\"icon appIcon-close\" ng-click=\"close()\"></i>\n" + "        </h3>\n" + "    </div>\n" + "    <div class=\"modal-body\"></div>\n" + "    <div class=\"modal-footer\">\n" + "        <div class=\"footer-buttons\">\n" + "            <button class=\"btn btn-default\" ng-repeat=\"button in buttons\" ng-click=\"button.onClick()\" ng-class=\"{'btn-primary': $last}\" \n" + "                ng-disabled=\"button.disabled\">{{ button.title }}</button>\n" + "        </div>\n" + "    </div>\n" + "</div>");
}]);
angular.module("dropdown/ad-dropdown.html", []).run(["$templateCache", function ($templateCache) {
    $templateCache.put("dropdown/ad-dropdown.html", "<div>\n" + "    <input type=\"hidden\" ng-model=\"selectedValue\" ng-required=\"ngRequired\" name=\"{{name}}\">\n" + "    <div class=\"dropdown form-control\" ng-class=\"inputSize ? 'input-' + inputSize : ''\" dropdown ng-disabled=\"ngDisabled || items.length == 0\" \n" + "        is-open=\"open\" keyboard-nav on-toggle=\"toggled(open)\">\n" + "        <span class=\"ad-dropdown-toggle\" dropdown-toggle ng-disabled=\"ngDisabled || items.length == 0\">\n" + "            <span ng-if=\"selectedValue\" ng-bind=\"displayProperty ? selectedValue[displayProperty] : selectedValue\"></span>\n" + "            <span ng-if=\"!selectedValue\" ng-bind=\"placeholder\"></span>\n" + "            <i class=\"appIcon-arrow_down centered-right\"></i>\n" + "        </span>\n" + "        <ul class=\"dropdown-menu\" role=\"menu\">\n" + "            <li ng-repeat=\"item in items\" ng-click=\"onChange(item)\" role=\"menuitem\">\n" + "                <a>{{ displayProperty ? item[displayProperty] : item }}</a>\n" + "            </li>\n" + "        </ul>\n" + "    </div>\n" + "</div>");
}]);
angular.module("fonts/directive/ad-icon.html", []).run(["$templateCache", function ($templateCache) {
    $templateCache.put("fonts/directive/ad-icon.html", "<div class=\"ad-icon\" ng-class=\"['ad-icon-' + size, 'ad-icon-' + iconClass]\">\n" + "  <i ng-class=\"iconClass\"></i>\n" + "  <div class=\"ad-icon-title\" ng-bind=\"iconTitle\"></div>\n" + "</div>");
}]);
angular.module("form-elements/form-field-info/form-field-info.html", []).run(["$templateCache", function ($templateCache) {
    $templateCache.put("form-elements/form-field-info/form-field-info.html", "<div class=\"form-field-info\" ng-class=\"{active: active}\">\n" + "    <i class=\"appIcon-info\" ng-click=\"toggleActive()\"></i>\n" + "    <div class=\"info\">\n" + "        <span ng-transclude></span>\n" + "        <i class=\"appIcon-cancel\" ng-click=\"toggleActive()\"></i>\n" + "    </div>\n" + "\n" + "</div>");
}]);
angular.module("form-elements/form-input/form-input.html", []).run(["$templateCache", function ($templateCache) {
    $templateCache.put("form-elements/form-input/form-input.html", "<div class=\"form-group\" ng-switch on=\"type\">\n" + "    <label ng-if=\"formFieldOptions.label !== undefined\" ng-class=\"{'required':isRequired && !disabled}\">{{ formFieldOptions.label }}</label>\n" + "    <form-field-info ng-if=\"formFieldOptions.info\">{{ formFieldOptions.info }}</form-field-info>\n" + "\n" + "    <div ng-switch-default>\n" + "        <input type=\"text\" ng-disabled=\"disabled || ngDisabled\" class=\"form-control\" name=\"{{ formFieldOptions.name }}\"\n" + "           ng-model=\"$parent.ngModel\" ng-model-options=\"{updateOn: updateOn, debounce: debounce, allowInvalid: allowInvalid}\"\n" + "            ng-required=\"isRequired\" \n" + "            ng-minlength=\"minlength\" \n" + "            ng-maxlength=\"maxlength\" \n" + "            ng-pattern=\"pattern\"\n" + "            placeholder=\"{{$parent.placeholder}}\">\n" + "    </div>\n" + "    <div ng-switch-when=\"email\">\n" + "        <input type=\"text\" ng-disabled=\"disabled || ngDisabled\" class=\"form-control\" name=\"{{ formFieldOptions.name }}\"\n" + "           ng-model=\"$parent.ngModel\" ng-model-options=\"{updateOn: updateOn, debounce: debounce, allowInvalid: allowInvalid}\"\n" + "            ng-required=\"isRequired\" ng-pattern=\"pattern\"\n" + "            placeholder=\"{{$parent.placeholder}}\">\n" + "    </div>\n" + "    <div ng-switch-when=\"tel\">\n" + "        <input type=\"text\" ng-disabled=\"disabled || ngDisabled\" class=\"form-control\" name=\"{{ formFieldOptions.name }}\"\n" + "           ng-model=\"$parent.ngModel\" ng-model-options=\"{updateOn: updateOn, debounce: debounce, allowInvalid: allowInvalid}\"\n" + "            ng-required=\"isRequired\" \n" + "            ng-minlength=\"minlength\" \n" + "            ng-maxlength=\"maxlength\" \n" + "            ng-pattern=\"pattern\"\n" + "            placeholder=\"{{$parent.placeholder}}\">\n" + "    </div>\n" + "    <div ng-switch-when=\"textarea\">\n" + "        <textarea class=\"form-control\" ng-disabled=\"disabled || ngDisabled\" name=\"{{ formFieldOptions.name }}\" rows=\"{{ rows }}\"\n" + "           ng-model=\"$parent.ngModel\" ng-model-options=\"{updateOn: updateOn, debounce: debounce, allowInvalid: allowInvalid}\" placeholder=\"{{$parent.placeholder}}\"\n" + "            ng-required=\"isRequired\" maxlength=\"{{ formFieldOptions.maxlength}}\"></textarea>\n" + "        <div ng-if=\"formFieldOptions.maxlength\" class=\"form-field-notes right\">{{ formFieldOptions.maxlength - $parent.ngModel.length }} characters remaining</div>\n" + "    </div>\n" + "    <div ng-switch-when=\"number\">\n" + "        <div class=\"input-group\">\n" + "            <span ng-hide=\"formFieldOptions.symbolLocation === 'right'\" class=\"input-group-addon number-symbol\" ng-if=\"formFieldOptions.symbol\" ng-bind=\"formFieldOptions.symbol\"></span>\n" + "            <input ng-disabled=\"disabled || ngDisabled\" type=\"number\" class=\"form-control right \" name=\"{{ formFieldOptions.name }}\" ng-keydown=\"vm.limitToNumbers($event)\"\n" + "                ng-model=\"$parent.ngModel\" ng-model-options=\"{updateOn: updateOn, debounce: debounce, allowInvalid: allowInvalid}\"\n" + "                ng-required=\"isRequired\" min=\"{{ min }}\" max=\"{{max}}\" placeholder=\"{{$parent.placeholder}}\">\n" + "            <span ng-show=\"formFieldOptions.symbolLocation === 'right'\" class=\"input-group-addon number-symbol\" ng-if=\"formFieldOptions.symbol\" ng-bind=\"formFieldOptions.symbol\"></span>\n" + "        </div>\n" + "    </div>\n" + "    <div ng-switch-when=\"list\">\n" + "        <ad-dropdown ng-if=\"listType === 'select' && !selectMultiple\" items=\"items\" ng-disabled=\"disabled || ngDisabled\"\n" + "            ng-model=\"$parent.$parent.ngModel\" ng-required=\"isRequired\" name=\"formFieldOptions.name\"\n" + "            placeholder=\"{{placeholder}}\" display-property=\"formFieldOptions.itemLabel\" value-property=\"formFieldOptions.itemValue\"></ad-dropdown>\n" + "        <select ng-disabled=\"disabled || ngDisabled\" name=\"{{ formFieldOptions.name }}\" ng-if=\"listType === 'select' && selectMultiple\" multiple class=\"form-control\" ng-options=\"{{ selectOptions }}\" ng-model=\"$parent.$parent.ngModel\" ng-required=\"isRequired\">\n" + "            <option ng-if=\"placeholder\" value=\"\">{{placeholder}}</option>\n" + "        </select>\n" + "        <div ng-if=\"listType === 'radio'\" class=\"radio-inline\" ng-repeat=\"item in items\">\n" + "            <label>\n" + "                <input ng-disabled=\"disabled || ngDisabled\" type=\"radio\" ng-model=\"$parent.$parent.$parent.ngModel\" name=\"{{ formFieldOptions.name }}\" ng-value=\"getItemValue($index)\">\n" + "                <span ng-bind=\"getItemLabel($index)\"></span>\n" + "            </label>\n" + "        </div>\n" + "        <div ng-if=\"listType === 'checkbox'\" class=\"checkbox-inline\" ng-repeat=\"item in items\">\n" + "            <label>\n" + "                <input ng-disabled=\"disabled || ngDisabled\" type=\"checkbox\" checklist-model=\"$parent.$parent.$parent.ngModel\" checklist-value=\"{{ itemValueProperty }}\" name=\"{{ formFieldOptions.name }}\" ng-value=\"getItemValue($index)\">\n" + "                <span ng-bind=\"getItemLabel($index)\"></span>\n" + "            </label>\n" + "        </div>\n" + "    </div>\n" + "    \n" + "    <i class=\"ad-form-feedback\" ng-class=\"{'appIcon-cancel' : $parent[formName][formFieldOptions.name].$error }\"></i>\n" + "    <div ng-repeat=\"validator in validators\" ng-show=\"validator.show()\" class='validation-error'>\n" + "        <i class=\"appIcon-error\"></i><span>{{ validator.message }}</span>\n" + "    </div>\n" + "    <div class=\"form-field-notes\">{{ form[formFieldOptions.name].$error }}</div>\n" + "    <div class=\"form-field-notes {{ formFieldOptions.notesLocation }}\" ng-if=\"formFieldOptions.notes\">{{ formFieldOptions.notes }}</div>\n" + "</div>");
}]);
angular.module("list/partials/itemList.html", []).run(["$templateCache", function ($templateCache) {
    $templateCache.put("list/partials/itemList.html", "<div class=\"itemList\">\n" + "    <ul ng-repeat=\"(letter, items) in sortedItems\" ng-style=\"isNoShortcuts() && {'margin' : '0px'}\" >\n" + "        <li class=\"itemListHeader header-{{letter}}\" ng-if=\"isEmptyList(items) == false\">{{letter}}</li>\n" + "\n" + "        <li class=\"itemListBody\"\n" + "            ng-click=\"doAction(item)\"\n" + "            ng-repeat=\"item in items | filter:filterByTerm | orderBy:sortingField\"\n" + "            ng-style=\"isNoShortcuts() && {'border-top-width' : '0px'}\" >\n" + "            <div ng-if=\"isObject(item) === false\" >{{item}}</div>\n" + "\n" + "            <div ng-if=\"isObject(item) === true\" ng-repeat=\"(key, value) in item track by $index\" class=\"{{key}}\">\n" + "                {{value}}\n" + "            </div>\n" + "        </li>\n" + "    </ul>\n" + "</div>");
}]);
angular.module("list/partials/itemShortcuts.html", []).run(["$templateCache", function ($templateCache) {
    $templateCache.put("list/partials/itemShortcuts.html", "<div class=\"shortcuts\">\n" + "    <div ng-click=\"jumpTo($event, letter)\" ng-repeat=\"letter in filteredAlphabet\">\n" + "        <div>{{letter}}</div>\n" + "        <div class=\"separator\"></div>\n" + "    </div>\n" + "</div>");
}]);
angular.module("list/partials/list.html", []).run(["$templateCache", function ($templateCache) {
    $templateCache.put("list/partials/list.html", "<div class=\"listComponent\">\n" + "    <search-box ng-if=\"enableSearch\"\n" + "                placeholder=\"Search\"\n" + "                on-search=\"doSearch(searchItem, $parent)\"\n" + "                search-term=\"searchItem\">\n" + "     </search-box>\n" + "\n" + "    <div class=\"list\">\n" + "        <div class=\"itemListWrapper\">\n" + "            <item-list></item-list>\n" + "        </div>\n" + "\n" + "        <div class=\"itemShortcutsWrapper\" ng-show=\"enableShortcuts\">\n" + "            <item-shortcuts></item-shortcuts>\n" + "        </div>\n" + "    </div>\n" + "\n" + "</div>");
}]);
angular.module("navigation/partials/ad-shell.html", []).run(["$templateCache", function ($templateCache) {
    $templateCache.put("navigation/partials/ad-shell.html", "<div class=\"ad-shell\" ad-multi-transclude>\n" + "    <div class=\"shell-top-nav\">\n" + "        <div class=\"shell-top-nav-content\">\n" + "            <div class=\"brand\">\n" + "                <alldata-logo ng-click=\"onLogoSelect && onLogoSelect()\"></alldata-logo>\n" + "                <div class=\"app-name\" ng-click=\"onAppNameSelect && onAppNameSelect()\">\n" + "                    <span ng-bind=\"appName\"></span>\n" + "                </div>\n" + "            </div>\n" + "        </div>\n" + "    </div>\n" + "    <div class=\"shell-body\">\n" + "        <div class=\"header-content\" transclude-from='header'>\n" + "            <!--<div ></div>-->\n" + "        </div>\n" + "        <!--<ad-nav ng-if=\"navContent\" content=\"navContent\" class=\"default\"></ad-nav>-->\n" + "        <div class=\"app-nav\" ng-if=\"navContent\">\n" + "            <ad-action-bar actions=\"navContent.menu\" class=\"clear-after\" show-titles=\"true\" alignment=\"left\" overflow-treshold=\"overflowTreshold\"></ad-action-bar>\n" + "        </div>\n" + "        <div transclude-from='content' class=\"shell-content\" ng-class=\"{'no-nav':!navContent}\"></div>\n" + "    </div>\n" + "</div>");
}]);
angular.module("navigation/partials/alldataLogo.html", []).run(["$templateCache", function ($templateCache) {
    $templateCache.put("navigation/partials/alldataLogo.html", "<svg class=\"alldata-logo\" version=\"1.1\" id=\"Layer_1\" xmlns=\"http://www.w3.org/2000/svg\" xmlns:xlink=\"http://www.w3.org/1999/xlink\"\n" + "x=\"0px\" y=\"0px\" viewBox=\"-297.1 777.5 568.6 66.3\" enable-background=\"new -297.1 777.5 568.6 66.3\" xml:space=\"preserve\">\n" + "    <g>\n" + "        <path fill=\"#ffffff\" d=\"M-244.9,777.5h-16.6l-35.7,66.3h22.9l6.4-8.9h22.9h8.9l6.4-12.8l11.5,21.7h22.9l-35.7-66.3H-244.9z    M-244.9,819.6h-14l12.8-22.9h1.3h2.6l12.8,22.9H-244.9z\"\n" + "        />\n" + "        <polygon fill=\"#ffffff\" points=\"-174.8,777.5 -192.6,777.5 -192.6,843.8 -130.1,843.8 -130.1,828.5 -174.8,828.5 	\" />\n" + "        <polygon fill=\"#ffffff\" points=\"-105.9,777.5 -123.8,777.5 -123.8,843.8 -61.3,843.8 -61.3,828.5 -105.9,828.5 	\" />\n" + "        <path fill=\"#ffffff\" d=\"M31.8,795.3c0,0-1.3-17.8-15.3-17.8h-73.9v66.3h19.1v-49.7H1.2c0,0,10.2-1.3,10.2,11.5    c0,11.5,1.3,22.9-11.5,22.9c-11.5,0-38.3,0-38.3,0l11.5,15.3h42.1c0,0,14-2.6,15.3-15.3C31.8,817,31.8,795.3,31.8,795.3z\"\n" + "        />\n" + "        <polygon fill=\"#ffffff\" points=\"191.2,777.5 108.3,777.5 108.3,794.1 140.2,794.1 140.2,843.8 159.3,843.8 159.3,792.8    191.2,792.8 	\"\n" + "        />\n" + "        <path fill=\"#ffffff\" d=\"M80.3,777.5H63.7L28,843.8h22.9l5.1-8.9H79h10.2l5.1-12.8l11.5,21.7h22.9L93,777.5H80.3z M80.3,819.6h-14    L79,796.6h1.3h2.6l12.8,22.9H80.3z\"\n" + "        />\n" + "        <path fill=\"#ffffff\" d=\"M235.8,777.5h-14h-16.6l-35.7,66.3h22.9l6.4-8.9h22.9h8.9l6.4-12.8l11.5,21.7h22.9L235.8,777.5z    M221.8,819.6h-14l12.8-22.9h1.3h2.6l12.8,22.9H221.8z\"\n" + "        />\n" + "    </g>\n" + "</svg>");
}]);
angular.module("navigation/partials/menu.html", []).run(["$templateCache", function ($templateCache) {
    $templateCache.put("navigation/partials/menu.html", "<div class=\"menu\">\n" + "    <script type=\"text/ng-template\" id=\"navigationItemTemplate\">\n" + "        <li class=\"item closed\" ng-click=\"menuVm.clickSubMenuItem(item)\" ng-repeat=\"item in item.children\" ng-include=\"'navigationItemContentTemplate'\"></li>\n" + "    </script>\n" + "\n" + "    <script type=\"text/ng-template\" id=\"navigationItemContentTemplate\">\n" + "        <div class=\"item-wrapper\" ng-click=\"menuVm.clickTopLevelMenuItem(item)\">\n" + "            <div class=\"item-label\">\n" + "                <span class=\"icon {{item.icon}}\"></span>\n" + "                <span class=\"text\">{{item.title}}</span>\n" + "            </div>\n" + "\n" + "            <div class=\"item-arrows\" ng-if=\"item.children\">\n" + "                <i class=\"appIcon-arrow_down open\"></i>\n" + "                <i class=\"appIcon-arrow_right closed\"></i>\n" + "            </div>\n" + "        </div>\n" + "\n" + "        <ul class=\"list level-{{item.level + 1}}\" ng-if=\"item.children\" ng-include=\"'navigationItemTemplate'\"></ul>\n" + "    </script>\n" + "\n" + "    <ul class=\"list level-1\">\n" + "        <top-menu-item  ng-repeat=\"item in content.menu\" item=\"item\" class=\"item closed\"></top-menu-item>\n" + "    </ul>\n" + "</div>");
}]);
angular.module("navigation/partials/navigation.html", []).run(["$templateCache", function ($templateCache) {
    $templateCache.put("navigation/partials/navigation.html", "<div class=\"shell-navigation animation\" role=\"navigation\">\n" + "    <div class=\"navigation-body\">\n" + "        <navigation-menu></navigation-menu>\n" + "    </div>\n" + "\n" + "    <div class=\"navigation-footer\">\n" + "        <div class=\"resizeIconContainer\">\n" + "            <div class=\"resizeIcon\"></div>\n" + "        </div>\n" + "        <div class=\"alldata\">\n" + "            <span class=\"icon appIcon-AD_Logo\"></span>\n" + "            <span class=\"text\">powered by ALLDATA</span>\n" + "        </div>\n" + "    </div>\n" + "</div>");
}]);
angular.module("navigation/partials/profile.html", []).run(["$templateCache", function ($templateCache) {
    $templateCache.put("navigation/partials/profile.html", "<div class=\"profile\">\n" + "    <img ng-src=\"content.profile.picture\" ng-if=\"content.profile.picture\" />\n" + "    <div class=\"information\">\n" + "        <div class=\"name\" ng-if=\"content.profile.name\" ng-bind=\"content.profile.name\"></div>\n" + "        <div class=\"shopname\" ng-if=\"content.profile.shopname\" ng-bind=\"content.profile.shopname\"></div>\n" + "    </div>\n" + "</div>");
}]);
angular.module("navigation/partials/shell-content.html", []).run(["$templateCache", function ($templateCache) {
    $templateCache.put("navigation/partials/shell-content.html", "<div ng-transclude class=\"shell-content\">\n" + "\n" + "</div>");
}]);
angular.module("navigation/partials/shell-topnav.html", []).run(["$templateCache", function ($templateCache) {
    $templateCache.put("navigation/partials/shell-topnav.html", "<div class=\"shell-top-nav\">\n" + "    <div class=\"shell-top-nav-content\">\n" + "        <div class=\"brand\">\n" + "            <alldata-logo></alldata-logo>\n" + "            <div class=\"app-name\">\n" + "                <span ng-bind=\"appName\"></span>\n" + "            </div>\n" + "        </div>\n" + "        <div ng-transclude></div>\n" + "    </div>\n" + "</div>");
}]);
angular.module("navigation/partials/subMenuItem.html", []).run(["$templateCache", function ($templateCache) {
    $templateCache.put("navigation/partials/subMenuItem.html", "<li>\n" + "    <div class=\"item-wrapper\" ng-class=\"{'disabled':!item.enabled()}\" ng-disabled=\"!item.enabled()\">\n" + "        <div class=\"item-label\">\n" + "            <span class=\"icon {{item.icon}}\"></span>\n" + "            <span class=\"text\">{{item.title}}</span>\n" + "        </div>\n" + "    </div>\n" + "</li>");
}]);
angular.module("navigation/partials/topMenuItem.html", []).run(["$templateCache", function ($templateCache) {
    $templateCache.put("navigation/partials/topMenuItem.html", "<li>\n" + "    <div class=\"item-wrapper\" ng-class=\"{'disabled':!item.enabled()}\" ng-disabled=\"!item.enabled()\" ng-if=\"!item.separator\">\n" + "        <div class=\"item-label\">\n" + "            <span class=\"icon {{item.icon}}\"></span>\n" + "            <span class=\"text\">{{item.title}}</span>\n" + "        </div>\n" + "\n" + "        <div class=\"item-arrows\" ng-if=\"item.children\">\n" + "            <i class=\"appIcon-arrow_down open\"></i>\n" + "            <i class=\"appIcon-arrow_right closed\"></i>\n" + "        </div>\n" + "    </div>\n" + "    \n" + "    <hr ng-if=\"item.separator\" class=\"top-menu-separator\" />\n" + "    <ul class=\"list level-2\" ng-if=\"item.children\">\n" + "        <sub-menu-item ng-repeat=\"childItem in item.children\" item=\"childItem\" class=\"item\"></sub-menu-item>\n" + "    </ul>\n" + "</li>");
}]);
angular.module("pdf/partials/pdfViewerTemplate.html", []).run(["$templateCache", function ($templateCache) {
    $templateCache.put("pdf/partials/pdfViewerTemplate.html", "<iframe id=\"pdfViewer\" class=\"pdf-viewer\"></iframe>");
}]);
angular.module("pdf/viewer/viewer.html", []).run(["$templateCache", function ($templateCache) {
    $templateCache.put("pdf/viewer/viewer.html", "<!DOCTYPE html>\n" + "<!--\n" + "Copyright 2012 Mozilla Foundation\n" + "\n" + "Licensed under the Apache License, Version 2.0 (the \"License\");\n" + "you may not use this file except in compliance with the License.\n" + "You may obtain a copy of the License at\n" + "\n" + "    http://www.apache.org/licenses/LICENSE-2.0\n" + "\n" + "Unless required by applicable law or agreed to in writing, software\n" + "distributed under the License is distributed on an \"AS IS\" BASIS,\n" + "WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\n" + "See the License for the specific language governing permissions and\n" + "limitations under the License.\n" + "\n" + "Adobe CMap resources are covered by their own copyright and license:\n" + "http://sourceforge.net/adobe/cmap/wiki/License/\n" + "-->\n" + "<html dir=\"ltr\" mozdisallowselectionprint moznomarginboxes>\n" + "\n" + "<head>\n" + "    <meta charset=\"utf-8\">\n" + "    <meta name=\"viewport\" content=\"width=device-width, initial-scale=1, maximum-scale=1\">\n" + "    <meta name=\"google\" content=\"notranslate\">\n" + "    <title>PDF.js viewer</title>\n" + "\n" + "    <link rel=\"stylesheet\" href=\"viewer.css\" />\n" + "    <link rel=\"resource\" type=\"application/l10n\" href=\"locale/locale.properties\" />\n" + "\n" + "    <script src=\"viewer.min.js\"></script>\n" + "</head>\n" + "\n" + "<body tabindex=\"1\" class=\"loadingInProgress\">\n" + "    <div id=\"outerContainer\">\n" + "\n" + "        <div id=\"sidebarContainer\">\n" + "            <div id=\"toolbarSidebar\">\n" + "                <div class=\"splitToolbarButton toggled\">\n" + "                    <button id=\"viewThumbnail\" class=\"toolbarButton group toggled\" title=\"Show Thumbnails\" tabindex=\"2\" data-l10n-id=\"thumbs\">\n" + "                        <span data-l10n-id=\"thumbs_label\">Thumbnails</span>\n" + "                    </button>\n" + "                    <button id=\"viewOutline\" class=\"toolbarButton group\" title=\"Show Document Outline\" tabindex=\"3\" data-l10n-id=\"outline\">\n" + "                        <span data-l10n-id=\"outline_label\">Document Outline</span>\n" + "                    </button>\n" + "                    <button id=\"viewAttachments\" class=\"toolbarButton group\" title=\"Show Attachments\" tabindex=\"4\" data-l10n-id=\"attachments\">\n" + "                        <span data-l10n-id=\"attachments_label\">Attachments</span>\n" + "                    </button>\n" + "                </div>\n" + "            </div>\n" + "            <div id=\"sidebarContent\">\n" + "                <div id=\"thumbnailView\">\n" + "                </div>\n" + "                <div id=\"outlineView\" class=\"hidden\">\n" + "                </div>\n" + "                <div id=\"attachmentsView\" class=\"hidden\">\n" + "                </div>\n" + "            </div>\n" + "        </div>\n" + "        <!-- sidebarContainer -->\n" + "\n" + "        <div id=\"mainContainer\">\n" + "            <div class=\"findbar hidden doorHanger hiddenSmallView\" id=\"findbar\">\n" + "                <label for=\"findInput\" class=\"toolbarLabel\" data-l10n-id=\"find_label\">Find:</label>\n" + "                <input id=\"findInput\" class=\"toolbarField\" tabindex=\"91\">\n" + "                <div class=\"splitToolbarButton\">\n" + "                    <button class=\"toolbarButton findPrevious\" title=\"\" id=\"findPrevious\" tabindex=\"92\" data-l10n-id=\"find_previous\">\n" + "                        <span data-l10n-id=\"find_previous_label\">Previous</span>\n" + "                    </button>\n" + "                    <div class=\"splitToolbarButtonSeparator\"></div>\n" + "                    <button class=\"toolbarButton findNext\" title=\"\" id=\"findNext\" tabindex=\"93\" data-l10n-id=\"find_next\">\n" + "                        <span data-l10n-id=\"find_next_label\">Next</span>\n" + "                    </button>\n" + "                </div>\n" + "                <input type=\"checkbox\" id=\"findHighlightAll\" class=\"toolbarField\">\n" + "                <label for=\"findHighlightAll\" class=\"toolbarLabel\" tabindex=\"94\" data-l10n-id=\"find_highlight\">Highlight all</label>\n" + "                <input type=\"checkbox\" id=\"findMatchCase\" class=\"toolbarField\">\n" + "                <label for=\"findMatchCase\" class=\"toolbarLabel\" tabindex=\"95\" data-l10n-id=\"find_match_case_label\">Match case</label>\n" + "                <span id=\"findMsg\" class=\"toolbarLabel\"></span>\n" + "            </div>\n" + "            <!-- findbar -->\n" + "\n" + "            <div id=\"secondaryToolbar\" class=\"secondaryToolbar hidden doorHangerRight\">\n" + "                <div id=\"secondaryToolbarButtonContainer\">\n" + "                    <button id=\"secondaryPresentationMode\" class=\"secondaryToolbarButton presentationMode visibleLargeView\" title=\"Switch to Presentation Mode\" tabindex=\"51\" data-l10n-id=\"presentation_mode\">\n" + "                        <span data-l10n-id=\"presentation_mode_label\">Presentation Mode</span>\n" + "                    </button>\n" + "\n" + "                    <button id=\"secondaryOpenFile\" class=\"secondaryToolbarButton openFile visibleLargeView\" title=\"Open File\" tabindex=\"52\" data-l10n-id=\"open_file\">\n" + "                        <span data-l10n-id=\"open_file_label\">Open</span>\n" + "                    </button>\n" + "\n" + "                    <button id=\"secondaryPrint\" class=\"secondaryToolbarButton print visibleMediumView\" title=\"Print\" tabindex=\"53\" data-l10n-id=\"print\">\n" + "                        <span data-l10n-id=\"print_label\">Print</span>\n" + "                    </button>\n" + "\n" + "                    <button id=\"secondaryDownload\" class=\"secondaryToolbarButton download visibleMediumView\" title=\"Download\" tabindex=\"54\" data-l10n-id=\"download\">\n" + "                        <span data-l10n-id=\"download_label\">Download</span>\n" + "                    </button>\n" + "\n" + "                    <a href=\"#\" id=\"secondaryViewBookmark\" class=\"secondaryToolbarButton bookmark visibleSmallView\" title=\"Current view (copy or open in new window)\" tabindex=\"55\" data-l10n-id=\"bookmark\">\n" + "                        <span data-l10n-id=\"bookmark_label\">Current View</span>\n" + "                    </a>\n" + "\n" + "                    <div class=\"horizontalToolbarSeparator visibleLargeView\"></div>\n" + "\n" + "                    <button id=\"firstPage\" class=\"secondaryToolbarButton firstPage\" title=\"Go to First Page\" tabindex=\"56\" data-l10n-id=\"first_page\">\n" + "                        <span data-l10n-id=\"first_page_label\">Go to First Page</span>\n" + "                    </button>\n" + "                    <button id=\"lastPage\" class=\"secondaryToolbarButton lastPage\" title=\"Go to Last Page\" tabindex=\"57\" data-l10n-id=\"last_page\">\n" + "                        <span data-l10n-id=\"last_page_label\">Go to Last Page</span>\n" + "                    </button>\n" + "\n" + "                    <div class=\"horizontalToolbarSeparator\"></div>\n" + "\n" + "                    <button id=\"pageRotateCw\" class=\"secondaryToolbarButton rotateCw\" title=\"Rotate Clockwise\" tabindex=\"58\" data-l10n-id=\"page_rotate_cw\">\n" + "                        <span data-l10n-id=\"page_rotate_cw_label\">Rotate Clockwise</span>\n" + "                    </button>\n" + "                    <button id=\"pageRotateCcw\" class=\"secondaryToolbarButton rotateCcw\" title=\"Rotate Counterclockwise\" tabindex=\"59\" data-l10n-id=\"page_rotate_ccw\">\n" + "                        <span data-l10n-id=\"page_rotate_ccw_label\">Rotate Counterclockwise</span>\n" + "                    </button>\n" + "\n" + "                    <div class=\"horizontalToolbarSeparator\"></div>\n" + "\n" + "                    <button id=\"toggleHandTool\" class=\"secondaryToolbarButton handTool\" title=\"Enable hand tool\" tabindex=\"60\" data-l10n-id=\"hand_tool_enable\">\n" + "                        <span data-l10n-id=\"hand_tool_enable_label\">Enable hand tool</span>\n" + "                    </button>\n" + "\n" + "                    <div class=\"horizontalToolbarSeparator\"></div>\n" + "\n" + "                    <button id=\"documentProperties\" class=\"secondaryToolbarButton documentProperties\" title=\"Document Properties…\" tabindex=\"61\" data-l10n-id=\"document_properties\">\n" + "                        <span data-l10n-id=\"document_properties_label\">Document Properties…</span>\n" + "                    </button>\n" + "                </div>\n" + "            </div>\n" + "            <!-- secondaryToolbar -->\n" + "\n" + "            <div class=\"toolbar\">\n" + "                <div id=\"toolbarContainer\">\n" + "                    <div id=\"toolbarViewer\">\n" + "                        <div id=\"toolbarViewerLeft\">\n" + "                            <button id=\"sidebarToggle\" class=\"toolbarButton\" title=\"Toggle Sidebar\" tabindex=\"11\" data-l10n-id=\"toggle_sidebar\">\n" + "                                <span data-l10n-id=\"toggle_sidebar_label\">Toggle Sidebar</span>\n" + "                            </button>\n" + "                            <div class=\"toolbarButtonSpacer\"></div>\n" + "                            <button id=\"viewFind\" class=\"toolbarButton group hiddenSmallView\" title=\"Find in Document\" tabindex=\"12\" data-l10n-id=\"findbar\">\n" + "                                <span data-l10n-id=\"findbar_label\">Find</span>\n" + "                            </button>\n" + "                            <div class=\"splitToolbarButton\">\n" + "                                <button class=\"toolbarButton pageUp\" title=\"Previous Page\" id=\"previous\" tabindex=\"13\" data-l10n-id=\"previous\">\n" + "                                    <span data-l10n-id=\"previous_label\">Previous</span>\n" + "                                </button>\n" + "                                <div class=\"splitToolbarButtonSeparator\"></div>\n" + "                                <button class=\"toolbarButton pageDown\" title=\"Next Page\" id=\"next\" tabindex=\"14\" data-l10n-id=\"next\">\n" + "                                    <span data-l10n-id=\"next_label\">Next</span>\n" + "                                </button>\n" + "                            </div>\n" + "                            <label id=\"pageNumberLabel\" class=\"toolbarLabel\" for=\"pageNumber\" data-l10n-id=\"page_label\">Page: </label>\n" + "                            <input type=\"number\" id=\"pageNumber\" class=\"toolbarField pageNumber\" value=\"1\" size=\"4\" min=\"1\" tabindex=\"15\">\n" + "                            <span id=\"numPages\" class=\"toolbarLabel\"></span>\n" + "                        </div>\n" + "                        <div id=\"toolbarViewerRight\">\n" + "                            <button id=\"presentationMode\" class=\"toolbarButton presentationMode hiddenLargeView\" title=\"Switch to Presentation Mode\" tabindex=\"31\" data-l10n-id=\"presentation_mode\">\n" + "                                <span data-l10n-id=\"presentation_mode_label\">Presentation Mode</span>\n" + "                            </button>\n" + "\n" + "                            <button id=\"openFile\" class=\"toolbarButton openFile hiddenLargeView\" title=\"Open File\" tabindex=\"32\" data-l10n-id=\"open_file\">\n" + "                                <span data-l10n-id=\"open_file_label\">Open</span>\n" + "                            </button>\n" + "\n" + "                            <button id=\"print\" class=\"toolbarButton print hiddenMediumView\" title=\"Print\" tabindex=\"33\" data-l10n-id=\"print\">\n" + "                                <span data-l10n-id=\"print_label\">Print</span>\n" + "                            </button>\n" + "\n" + "                            <button id=\"download\" class=\"toolbarButton download hiddenMediumView\" title=\"Download\" tabindex=\"34\" data-l10n-id=\"download\">\n" + "                                <span data-l10n-id=\"download_label\">Download</span>\n" + "                            </button>\n" + "                            <!-- <div class=\"toolbarButtonSpacer\"></div> -->\n" + "                            <a href=\"#\" id=\"viewBookmark\" class=\"toolbarButton bookmark hiddenSmallView\" title=\"Current view (copy or open in new window)\" tabindex=\"35\" data-l10n-id=\"bookmark\">\n" + "                                <span data-l10n-id=\"bookmark_label\">Current View</span>\n" + "                            </a>\n" + "\n" + "                            <div class=\"verticalToolbarSeparator hiddenSmallView\"></div>\n" + "\n" + "                            <button id=\"secondaryToolbarToggle\" class=\"toolbarButton\" title=\"Tools\" tabindex=\"36\" data-l10n-id=\"tools\">\n" + "                                <span data-l10n-id=\"tools_label\">Tools</span>\n" + "                            </button>\n" + "                        </div>\n" + "                        <div class=\"outerCenter\">\n" + "                            <div class=\"innerCenter\" id=\"toolbarViewerMiddle\">\n" + "                                <div class=\"splitToolbarButton\">\n" + "                                    <button id=\"zoomOut\" class=\"toolbarButton zoomOut\" title=\"Zoom Out\" tabindex=\"21\" data-l10n-id=\"zoom_out\">\n" + "                                        <span data-l10n-id=\"zoom_out_label\">Zoom Out</span>\n" + "                                    </button>\n" + "                                    <div class=\"splitToolbarButtonSeparator\"></div>\n" + "                                    <button id=\"zoomIn\" class=\"toolbarButton zoomIn\" title=\"Zoom In\" tabindex=\"22\" data-l10n-id=\"zoom_in\">\n" + "                                        <span data-l10n-id=\"zoom_in_label\">Zoom In</span>\n" + "                                    </button>\n" + "                                </div>\n" + "                                <span id=\"scaleSelectContainer\" class=\"dropdownToolbarButton\">\n" + "                     <select id=\"scaleSelect\" title=\"Zoom\" tabindex=\"23\" data-l10n-id=\"zoom\">\n" + "                      <option id=\"pageAutoOption\" title=\"\" value=\"auto\" selected=\"selected\" data-l10n-id=\"page_scale_auto\">Automatic Zoom</option>\n" + "                      <option id=\"pageActualOption\" title=\"\" value=\"page-actual\" data-l10n-id=\"page_scale_actual\">Actual Size</option>\n" + "                      <option id=\"pageFitOption\" title=\"\" value=\"page-fit\" data-l10n-id=\"page_scale_fit\">Fit Page</option>\n" + "                      <option id=\"pageWidthOption\" title=\"\" value=\"page-width\" data-l10n-id=\"page_scale_width\">Full Width</option>\n" + "                      <option id=\"customScaleOption\" title=\"\" value=\"custom\"></option>\n" + "                      <option title=\"\" value=\"0.5\" data-l10n-id=\"page_scale_percent\" data-l10n-args='{ \"scale\": 50 }'>50%</option>\n" + "                      <option title=\"\" value=\"0.75\" data-l10n-id=\"page_scale_percent\" data-l10n-args='{ \"scale\": 75 }'>75%</option>\n" + "                      <option title=\"\" value=\"1\" data-l10n-id=\"page_scale_percent\" data-l10n-args='{ \"scale\": 100 }'>100%</option>\n" + "                      <option title=\"\" value=\"1.25\" data-l10n-id=\"page_scale_percent\" data-l10n-args='{ \"scale\": 125 }'>125%</option>\n" + "                      <option title=\"\" value=\"1.5\" data-l10n-id=\"page_scale_percent\" data-l10n-args='{ \"scale\": 150 }'>150%</option>\n" + "                      <option title=\"\" value=\"2\" data-l10n-id=\"page_scale_percent\" data-l10n-args='{ \"scale\": 200 }'>200%</option>\n" + "                      <option title=\"\" value=\"3\" data-l10n-id=\"page_scale_percent\" data-l10n-args='{ \"scale\": 300 }'>300%</option>\n" + "                      <option title=\"\" value=\"4\" data-l10n-id=\"page_scale_percent\" data-l10n-args='{ \"scale\": 400 }'>400%</option>\n" + "                    </select>\n" + "                  </span>\n" + "                            </div>\n" + "                        </div>\n" + "                    </div>\n" + "                    <div id=\"loadingBar\">\n" + "                        <div class=\"progress\">\n" + "                            <div class=\"glimmer\">\n" + "                            </div>\n" + "                        </div>\n" + "                    </div>\n" + "                </div>\n" + "            </div>\n" + "\n" + "            <menu type=\"context\" id=\"viewerContextMenu\">\n" + "                <menuitem id=\"contextFirstPage\" label=\"First Page\" data-l10n-id=\"first_page\"></menuitem>\n" + "                <menuitem id=\"contextLastPage\" label=\"Last Page\" data-l10n-id=\"last_page\"></menuitem>\n" + "                <menuitem id=\"contextPageRotateCw\" label=\"Rotate Clockwise\" data-l10n-id=\"page_rotate_cw\"></menuitem>\n" + "                <menuitem id=\"contextPageRotateCcw\" label=\"Rotate Counter-Clockwise\" data-l10n-id=\"page_rotate_ccw\"></menuitem>\n" + "            </menu>\n" + "\n" + "            <div id=\"viewerContainer\" tabindex=\"0\">\n" + "                <div id=\"viewer\" class=\"pdfViewer\"></div>\n" + "            </div>\n" + "\n" + "            <div id=\"errorWrapper\" hidden='true'>\n" + "                <div id=\"errorMessageLeft\">\n" + "                    <span id=\"errorMessage\"></span>\n" + "                    <button id=\"errorShowMore\" data-l10n-id=\"error_more_info\">\n" + "                        More Information\n" + "                    </button>\n" + "                    <button id=\"errorShowLess\" data-l10n-id=\"error_less_info\" hidden='true'>\n" + "                        Less Information\n" + "                    </button>\n" + "                </div>\n" + "                <div id=\"errorMessageRight\">\n" + "                    <button id=\"errorClose\" data-l10n-id=\"error_close\">\n" + "                        Close\n" + "                    </button>\n" + "                </div>\n" + "                <div class=\"clearBoth\"></div>\n" + "                <textarea id=\"errorMoreInfo\" hidden='true' readonly=\"readonly\"></textarea>\n" + "            </div>\n" + "        </div>\n" + "        <!-- mainContainer -->\n" + "\n" + "        <div id=\"overlayContainer\" class=\"hidden\">\n" + "            <div id=\"passwordOverlay\" class=\"container hidden\">\n" + "                <div class=\"dialog\">\n" + "                    <div class=\"row\">\n" + "                        <p id=\"passwordText\" data-l10n-id=\"password_label\">Enter the password to open this PDF file:</p>\n" + "                    </div>\n" + "                    <div class=\"row\">\n" + "                        <input type=\"password\" id=\"password\" class=\"toolbarField\" />\n" + "                    </div>\n" + "                    <div class=\"buttonRow\">\n" + "                        <button id=\"passwordCancel\" class=\"overlayButton\"><span data-l10n-id=\"password_cancel\">Cancel</span>\n" + "                        </button>\n" + "                        <button id=\"passwordSubmit\" class=\"overlayButton\"><span data-l10n-id=\"password_ok\">OK</span>\n" + "                        </button>\n" + "                    </div>\n" + "                </div>\n" + "            </div>\n" + "            <div id=\"documentPropertiesOverlay\" class=\"container hidden\">\n" + "                <div class=\"dialog\">\n" + "                    <div class=\"row\">\n" + "                        <span data-l10n-id=\"document_properties_file_name\">File name:</span>\n" + "                        <p id=\"fileNameField\">-</p>\n" + "                    </div>\n" + "                    <div class=\"row\">\n" + "                        <span data-l10n-id=\"document_properties_file_size\">File size:</span>\n" + "                        <p id=\"fileSizeField\">-</p>\n" + "                    </div>\n" + "                    <div class=\"separator\"></div>\n" + "                    <div class=\"row\">\n" + "                        <span data-l10n-id=\"document_properties_title\">Title:</span>\n" + "                        <p id=\"titleField\">-</p>\n" + "                    </div>\n" + "                    <div class=\"row\">\n" + "                        <span data-l10n-id=\"document_properties_author\">Author:</span>\n" + "                        <p id=\"authorField\">-</p>\n" + "                    </div>\n" + "                    <div class=\"row\">\n" + "                        <span data-l10n-id=\"document_properties_subject\">Subject:</span>\n" + "                        <p id=\"subjectField\">-</p>\n" + "                    </div>\n" + "                    <div class=\"row\">\n" + "                        <span data-l10n-id=\"document_properties_keywords\">Keywords:</span>\n" + "                        <p id=\"keywordsField\">-</p>\n" + "                    </div>\n" + "                    <div class=\"row\">\n" + "                        <span data-l10n-id=\"document_properties_creation_date\">Creation Date:</span>\n" + "                        <p id=\"creationDateField\">-</p>\n" + "                    </div>\n" + "                    <div class=\"row\">\n" + "                        <span data-l10n-id=\"document_properties_modification_date\">Modification Date:</span>\n" + "                        <p id=\"modificationDateField\">-</p>\n" + "                    </div>\n" + "                    <div class=\"row\">\n" + "                        <span data-l10n-id=\"document_properties_creator\">Creator:</span>\n" + "                        <p id=\"creatorField\">-</p>\n" + "                    </div>\n" + "                    <div class=\"separator\"></div>\n" + "                    <div class=\"row\">\n" + "                        <span data-l10n-id=\"document_properties_producer\">PDF Producer:</span>\n" + "                        <p id=\"producerField\">-</p>\n" + "                    </div>\n" + "                    <div class=\"row\">\n" + "                        <span data-l10n-id=\"document_properties_version\">PDF Version:</span>\n" + "                        <p id=\"versionField\">-</p>\n" + "                    </div>\n" + "                    <div class=\"row\">\n" + "                        <span data-l10n-id=\"document_properties_page_count\">Page Count:</span>\n" + "                        <p id=\"pageCountField\">-</p>\n" + "                    </div>\n" + "                    <div class=\"buttonRow\">\n" + "                        <button id=\"documentPropertiesClose\" class=\"overlayButton\"><span data-l10n-id=\"document_properties_close\">Close</span>\n" + "                        </button>\n" + "                    </div>\n" + "                </div>\n" + "            </div>\n" + "        </div>\n" + "        <!-- overlayContainer -->\n" + "\n" + "    </div>\n" + "    <!-- outerContainer -->\n" + "    <div id=\"printContainer\"></div>\n" + "    <div id=\"mozPrintCallback-shim\" hidden>\n" + "        <style>\n" + "            @media print {\n" + "                #printContainer div {\n" + "                    page-break-after: always;\n" + "                    page-break-inside: avoid;\n" + "                }\n" + "            }\n" + "        </style>\n" + "        <style scoped>\n" + "            #mozPrintCallback-shim {\n" + "                position: fixed;\n" + "                top: 0;\n" + "                left: 0;\n" + "                height: 100%;\n" + "                width: 100%;\n" + "                z-index: 9999999;\n" + "                display: block;\n" + "                text-align: center;\n" + "                background-color: rgba(0, 0, 0, 0.5);\n" + "            }\n" + "            \n" + "            #mozPrintCallback-shim[hidden] {\n" + "                display: none;\n" + "            }\n" + "            \n" + "            @media print {\n" + "                #mozPrintCallback-shim {\n" + "                    display: none;\n" + "                }\n" + "            }\n" + "            \n" + "            #mozPrintCallback-shim .mozPrintCallback-dialog-box {\n" + "                display: inline-block;\n" + "                margin: -50px auto 0;\n" + "                position: relative;\n" + "                top: 45%;\n" + "                left: 0;\n" + "                min-width: 220px;\n" + "                max-width: 400px;\n" + "                padding: 9px;\n" + "                border: 1px solid hsla(0, 0%, 0%, .5);\n" + "                border-radius: 2px;\n" + "                box-shadow: 0 1px 4px rgba(0, 0, 0, 0.3);\n" + "                background-color: #474747;\n" + "                color: hsl(0, 0%, 85%);\n" + "                font-size: 16px;\n" + "                line-height: 20px;\n" + "            }\n" + "            \n" + "            #mozPrintCallback-shim .progress-row {\n" + "                clear: both;\n" + "                padding: 1em 0;\n" + "            }\n" + "            \n" + "            #mozPrintCallback-shim progress {\n" + "                width: 100%;\n" + "            }\n" + "            \n" + "            #mozPrintCallback-shim .relative-progress {\n" + "                clear: both;\n" + "                float: right;\n" + "            }\n" + "            \n" + "            #mozPrintCallback-shim .progress-actions {\n" + "                clear: both;\n" + "            }\n" + "        </style>\n" + "        <div class=\"mozPrintCallback-dialog-box\">\n" + "            <!-- TODO: Localise the following strings -->\n" + "            Preparing document for printing...\n" + "            <div class=\"progress-row\">\n" + "                <progress value=\"0\" max=\"100\"></progress>\n" + "                <span class=\"relative-progress\">0%</span>\n" + "            </div>\n" + "            <div class=\"progress-actions\">\n" + "                <input type=\"button\" value=\"Cancel\" class=\"mozPrintCallback-cancel\">\n" + "            </div>\n" + "        </div>\n" + "    </div>\n" + "</body>\n" + "\n" + "</html>");
}]);
angular.module("search-box/search-box.html", []).run(["$templateCache", function ($templateCache) {
    $templateCache.put("search-box/search-box.html", "<div class=\"search-box\">\n" + "    <div class=\"input-group\">\n" + "        <span class=\"input-group-addon search-icon\"><i class=\"appIcon-search\"></i></span>\n" + "        <input type=\"search\" class=\"form-control\" placeholder=\"{{ placeholder }}\" ng-model=\"searchTerm\"/>\n" + "        <span class=\"input-group-addon search-trigger\" ng-hide=\"autoSearch\" ng-click=\"search(searchTerm)\">Search</span>\n" + "    </div>\n" + "</div>");
}]);
angular.module("table/partials/pagination.html", []).run(["$templateCache", function ($templateCache) {
    $templateCache.put("table/partials/pagination.html", "<div class=\"ng-cloak ng-table-pager paginationContainer\">\n" + "    <div class=\"itemCount\">\n" + "        <span>Showing {{((((params.page() * params.count()) - params.count()) + 1) > params.settings().total)? params.settings().total : ((params.page() * params.count()) - params.count()) + 1}}</span>\n" + "        <span> - </span>\n" + "        <span>{{(params.page() * params.count() > params.settings().total)? params.settings().total: params.page() * params.count()}}</span>\n" + "        <span>of {{params.settings().total}} entries</span>\n" + "    </div>\n" + "\n" + "    <ul class=\"pagination ng-table-pagination\">\n" + "        <li ng-class=\"{'disabled': !page.active}\" ng-repeat=\"page in pages\" ng-switch=\"page.type\">\n" + "            <a ng-switch-when=\"prev\" ng-click=\"params.page(page.number)\" href=\"\">&laquo;</a>\n" + "            <a ng-switch-when=\"first\" ng-click=\"params.page(page.number)\" href=\"\" class=\"{{(params.page() == page.number)? 'selected': ''}}\">\n" + "                <span ng-bind=\"page.number\"></span>\n" + "            </a>\n" + "            <a ng-switch-when=\"page\" ng-click=\"params.page(page.number)\" href=\"\" class=\"{{(params.page() == page.number)? 'selected': ''}}\">\n" + "                <span ng-bind=\"page.number\"></span>\n" + "            </a>\n" + "            <a ng-switch-when=\"more\" ng-click=\"params.page(page.number)\" href=\"\">&#8230;</a>\n" + "            <a ng-switch-when=\"last\" ng-click=\"params.page(page.number)\" href=\"\" class=\"{{(params.page() == page.number)? 'selected': ''}}\">\n" + "                <span ng-bind=\"page.number\"></span></a>\n" + "            <a ng-switch-when=\"next\" ng-click=\"params.page(page.number)\" href=\"\">&raquo;</a>\n" + "        </li>\n" + "    </ul>\n" + "</div>");
}]);
angular.module("table/partials/table.html", []).run(["$templateCache", function ($templateCache) {
    $templateCache.put("table/partials/table.html", "<div class=\"adTable\">\n" + "    <div class=\"wrapper\">\n" + "        <div class=\"toolbar\" ng-if=\"toolbar\">\n" + "            <h4 class=\"title\" ng-if=\"toolbar.title\" ng-bind=\"toolbar.title\"></h4>\n" + "            <div class=\"icons\" ng-if=\"toolbar.icons\">\n" + "                <div class=\"icon {{icon.name}}\" ng-repeat=\"icon in toolbar.icons\" ng-click=\"icon.do()\"></div>\n" + "            </div>\n" + "        </div>\n" + "\n" + "        <div class=\"searchbar\" ng-if=\"searchbar\">\n" + "            <div class=\"actions\" ng-if=\"searchbar.actions && searchbar.actions.length <= 1\">\n" + "                <button class=\"action\" ng-click=\"action.do()\" ng-repeat=\"action in searchbar.actions\" ng-bind=\"action.text\"></button>\n" + "            </div>\n" + "\n" + "            <div class=\"actions btn-group\" dropdown is-open=\"status.isopen\" ng-if=\"searchbar.actions && searchbar.actions.length > 1\">\n" + "                <button type=\"button\" class=\"action btn btn-primary dropdown-toggle\" ng-disabled=\"disabled\">\n" + "                    <span ng-bind=\"searchbar.actions[0].text\"></span>\n" + "                    <span class=\"caret\"></span>\n" + "                </button>\n" + "                <ul class=\"dropdown-menu\" role=\"menu\">\n" + "                    <li class=\"splitAction\" ng-repeat=\"action in searchbar.actions\">\n" + "                        <a ng-click=\"action.do()\" ng-bind=\"action.text\"></a>\n" + "                    </li>\n" + "                </ul>\n" + "            </div>\n" + "\n" + "            <div class=\"searchBox\" ng-if=\"searchbar.enableSearch\">\n" + "                <search-box placeholder=\"Search\" on-search=\"doFilter(filterItem, $parent)\" search-term=\"filterItem\" timeDelay=\"searchDelay\">\n" + "                </search-box>\n" + "            </div>\n" + "        </div>\n" + "    </div>\n" + "\n" + "    <table ng-table=\"tableObject\" template-pagination=\"table/partials/pagination.html\" class=\"table\">\n" + "        <thead>\n" + "            <tr>\n" + "                <th ng-repeat=\"field in fieldList\" name=\"{{ field.name }}\" class=\"header {{ field.bind || noBind }}\" ng-class=\"{\n" + "                            tdAlignRight: field.align == 'right',\n" + "                            tdAlignCenter: field.align == 'center',\n" + "                            'sort-asc': tableObject.isSortBy(field.element, 'asc'),\n" + "                            'sort-desc': tableObject.isSortBy(field.element, 'desc'),\n" + "                            sortable : sortClassText(field)\n" + "                        }\" ng-click=\"sortData(field, tableObject.isSortBy(field.element, 'asc') ? 'desc' : 'asc')\">\n" + "                    <div ng-bind=\"field.name\"></div>\n" + "                </th>\n" + "                <th ng-if=\"actions\" class=\"actions header\">\n" + "                    <div ng-bind=\"actions.title\"></div>\n" + "                </th>\n" + "            </tr>\n" + "        </thead>\n" + "\n" + "        <tbody>\n" + "            <tr class=\"tableRow\" ng-repeat=\"item in itemList | filter:filterText\">\n" + "                <td ng-repeat=\"field in fieldList\" name=\"{{ field.name }}\" class=\"{{ field.bind || noBind }}\" sortable=\"field.bind\" ng-class=\"{tdAlignRight: field.align == 'right', tdAlignCenter: field.align == 'center'}\" ng-click=\"internalItemClicked(item, $event)\">\n" + "                    <input ng-if=\"field.isCheckbox === true  && item[field.bind] != null\" type=\"checkbox\" ng-checked=\"item[field.bind]\" ng-disabled=\"isReadonly()\" ng-click=\"field.callback(item, field, $event)\">\n" + "                    <div ng-if=\"field.isCheckbox !== true\" ng-bind=\"item[field.bind]\"></div>\n" + "                </td>\n" + "\n" + "                <td ng-if=\"actions && actions.icons.length <= 2\" class=\"actions\">\n" + "                    <div ng-repeat=\"action in actions.icons\" class=\"icon {{action.name}}\" ng-click=\"action.do(item)\"></div>\n" + "                </td>\n" + "\n" + "                <td ng-if=\"actions && actions.texts.length <= 2\" class=\"actions\">\n" + "                    <span ng-repeat=\"action in actions.texts\" class=\"actionText\" ng-click=\"action.do(item)\" ng-bind=\"action.name\"></span>\n" + "                </td>\n" + "\n" + "                <td ng-if=\"actions && actions.icons.length > 2\" class=\"actions\">\n" + "                    <div class=\"btn-group\" dropdown is-open=\"status.isopen\">\n" + "                        <button type=\"button\" class=\"action btn btn-primary dropdown-toggle\" ng-disabled=\"disabled\">\n" + "                            <span ng-bind=\"actions.title\"></span>\n" + "                            <span class=\"caret\"></span>\n" + "                        </button>\n" + "                        <ul class=\"dropdown-menu\" role=\"menu\">\n" + "                            <li class=\"splitAction\" ng-repeat=\"action in actions.icons\" ng-click=\"action.do(item)\">\n" + "                                <div class=\"icon {{action.name}}\"></div>\n" + "                            </li>\n" + "                        </ul>\n" + "                    </div>\n" + "                </td>\n" + "\n" + "                <td ng-if=\"actions && actions.texts.length > 2\" class=\"actions\">\n" + "                    <div class=\"btn-group\" dropdown is-open=\"status.isopen\">\n" + "                        <button type=\"button\" class=\"action btn btn-primary dropdown-toggle\" ng-disabled=\"disabled\">\n" + "                            <span ng-bind=\"actions.title\"></span>\n" + "                            <span class=\"caret\"></span>\n" + "                        </button>\n" + "                        <ul class=\"dropdown-menu\" role=\"menu\">\n" + "                            <li class=\"splitAction\" ng-repeat=\"action in actions.texts\" ng-click=\"action.do(item)\">\n" + "                                <span class=\"actionText\" ng-bind=\"action.name\"></span>\n" + "                            </li>\n" + "                        </ul>\n" + "                    </div>\n" + "                </td>\n" + "            </tr>\n" + "        </tbody>\n" + "    </table>\n" + "</div>");
}]);
angular.module("vehicles-quotes/partials/quote.html", []).run(["$templateCache", function ($templateCache) {
    $templateCache.put("vehicles-quotes/partials/quote.html", "<div class=\"quoteContainer\">\n" + "    <div class=\"quoteHeader\">Quote History</div>\n" + "\n" + "    <div class=\"quoteBody\">\n" + "        <div class=\"notFound item\" ng-if=\"getQuoteCount(vehicle.quotes) === 0\">Not available</div>\n" + "        <div class=\"information\" ng-repeat=\"quote in vehicle.quotes | limitTo: limit\">\n" + "            <div class=\"date item\">{{quote.creationDate}}</div>\n" + "            <div class=\"name item\">{{quote.quoteName}}</div>\n" + "        </div>\n" + "    </div>\n" + "\n" + "    <div class=\"quoteFooter\">\n" + "        <div ng-click=\"displayMoreQuotes(vehicle.quotes)\" ng-if=\"getQuoteCount(vehicle.quotes) > limit\">More History</div>\n" + "        <div ng-click=\"displayLessQuotes(vehicle.quotes)\" ng-if=\"getQuoteCount(vehicle.quotes) == limit\">Less History</div>\n" + "    </div>\n" + "</div>");
}]);
angular.module("vehicles-quotes/partials/vehicle.html", []).run(["$templateCache", function ($templateCache) {
    $templateCache.put("vehicles-quotes/partials/vehicle.html", "<div class=\"vehicle\" id=\"vehicle-{{vehicle.id}}\">\n" + "    <div class=\"information\">\n" + "        <div class=\"ymmDescription\">{{vehicle.ymmDescription}}</div>\n" + "        <div class=\"engineDescription\">{{vehicle.engineDescription}}</div>\n" + "    </div>\n" + "\n" + "    <div class=\"quotes\">\n" + "        <quote></quote>\n" + "    </div>\n" + "</div>");
}]);
angular.module("vehicles-quotes/partials/vehiclesQuotes.html", []).run(["$templateCache", function ($templateCache) {
    $templateCache.put("vehicles-quotes/partials/vehiclesQuotes.html", "<div class=\"vehiclesQuotes\">\n" + "    <vehicle ng-repeat=\"vehicle in vehicles\"></vehicle>\n" + "</div>");
}]);
angular.module("zoom-container/zoom-container.html", []).run(["$templateCache", function ($templateCache) {
    $templateCache.put("zoom-container/zoom-container.html", "<div class=\"zoom-container\" ng-class=\"{'print-full-screen' : printFullScreen }\" >\n" + "    <ul class=\"zoom-container-buttons\">\n" + "        <li ng-click=\"zoomIn()\" title=\"Zoom In\"><i class=\"appIcon-zoom_in\"></i></li>\n" + "        <li ng-click=\"zoomOut()\" title=\"Zoom Out\"><i class=\"appIcon-zoom_out\"></i></li>\n" + "        <li ng-click=\"fit()\" title=\"Fit to Window\"><i class=\"appIcon-fullscreen_exit\"></i></li>\n" + "        <li ng-click=\"rotate(-90)\" title=\"Rotate Left\"><i class=\"appIcon-rotate_left\"></i></li>\n" + "        <li ng-click=\"rotate(90)\" title=\"Rotate Right\"><i class=\"appIcon-rotate_right\"></i></li>\n" + "        <li ng-click=\"print()\" title=\"Print\"><i class=\"appIcon-print\"></i></li>\n" + "    </ul>\n" + "    <div class=\"mainObject\" touch-action=\"pinch-and-zoom\">\n" + "        <div class=\"mainObjectContent\" ng-class=\"'rotate-' + rotation\" ng-transclude></div>\n" + "    </div>\n" + "</div>\n" + "");
}]);
var ConversionCalculator;
(function (ConversionCalculator) {
    var ConversionCalculatorDirective = (function () {
        function ConversionCalculatorDirective() {
        }
        ConversionCalculatorDirective.init = function () {
            var directive = {
                restrict: 'E',
                replace: true,
                templateUrl: 'conversion-calculator/conversionCalculator.html',
                scope: {},
                link: function ($scope, element, attr) {
                    var vm = {};
                    $scope.convVm = vm;
                    vm.currentView = "area";
                    vm.viewMode = 'collapsed';
                    vm.setCurrentView = function (currentView) {
                        vm.currentView = currentView;
                        var propIndex = 0;
                        for (var convProp in vm.conversions[currentView]) {
                            if (propIndex == 0) {
                                vm.fromConversion = vm.conversions[currentView][convProp];
                                propIndex++;
                            }
                            else if (propIndex == 1) {
                                vm.toConversion = vm.conversions[currentView][convProp];
                                break;
                            }
                        }
                    };
                    vm.conversions = {
                        area: {
                            sqMeters: { type: 'sqMeters', label: 'Sq. Meters', factor: 1, value: 0 },
                            sqYards: { type: 'sqYards', label: 'Sq. Yards', factor: 1.195990045562, value: 0 },
                            sqCentimeters: { type: 'sqCentimeters', label: 'Sq. Centimeters', factor: 10000.0, value: 0 },
                            sqFeet: { type: 'sqFeet', label: 'Sq. Feet', factor: 10.76391041662, value: 0 },
                            sqMillimeters: { type: 'sqMillimeters', label: 'Sq. Millimeters', factor: 1000000.0, value: 0 },
                            sqInches: { type: 'sqInches', label: 'Sq. Inches', factor: 1550.003099994, value: 0 },
                            sqKilometers: { type: 'sqKilometers', label: 'Sq. Kilometers', factor: 0.000001, value: 0 },
                            sqMiles: { type: 'sqMiles', label: 'Sq. Miles', factor: 0.0000003861006842, value: 0.000 }
                        },
                        length: {
                            meters: { type: 'meters', label: 'Meters', factor: 1, value: 0 },
                            centimeters: { type: 'centimeters', label: 'Centimeters', factor: 100, value: 0 },
                            millimeters: { type: 'millimeters', label: 'Millimeters', factor: 1000, value: 0 },
                            inches: { type: 'inches', label: 'Inches', factor: 39.37007874, value: 0 },
                            feet: { type: 'feet', label: 'Feet', factor: 3.280839895, value: 0 },
                            yards: { type: 'yards', label: 'Yards', factor: 1.093613298, value: 0 },
                            miles: { type: 'miles', label: 'Miles', factor: 0.0006213700339, value: 0 },
                            kilometers: { type: 'kilometers', label: 'Kilometers', factor: 0.001, value: 0 },
                        },
                        force: {
                            newtons: { type: 'newtons', label: 'Newtons', factor: 1, value: 0 },
                            pounds: { type: 'pounds', label: 'Pounds', factor: 0.2248089, value: 0 },
                            ounces: { type: 'ounces', label: 'Ounces', factor: 3.596942455, value: 0 }
                        },
                        pressure: {
                            bar: { type: 'bar', label: 'Bar', factor: 1, value: 0 },
                            Pascal: { type: 'Pascal', label: 'Pascal', factor: 100000, value: 0 },
                            kgcm: { type: 'kgcm', label: 'kg/cm', factor: 1.019716, value: 0 },
                            kPa: { type: 'kPa', label: 'kPa', factor: 100, value: 0 },
                            Atmospheres: { type: 'Atmospheres', label: 'Atmospheres', factor: 0.98692, value: 0 },
                            psi: { type: 'psi', label: 'psi', factor: 14.5038, value: 0 },
                            Millibar: { type: 'Millibar', label: 'Millibar', factor: 1000, value: 0 },
                        },
                        torque: {
                            newtonMeters: { type: 'newtonMeters', label: 'Newton Meters', factor: 1, value: 0 },
                            footLbs: { type: 'footLbs', label: 'Foot lbs', factor: .73756, value: 0 },
                            inchLbs: { type: 'inchLbs', label: 'Inch lbs', factor: 8.8507, value: 0 },
                            kgcm: { type: 'kgcm', label: 'KG cm', factor: 10.19716, value: 0 },
                            kgm: { type: 'kgm', label: 'KG M', factor: .1019716, value: 0 },
                        },
                        temperature: {
                            celsius: { type: 'celsius', label: 'Celsius', value: 0 },
                            fahrenheit: { type: 'fahrenheit', label: 'Fahrenheit', value: 0 },
                        },
                        vacuum: {
                            Pascal: { type: 'Pascal', label: 'Pascal', value: 0, factor: 1 },
                            kPa: { type: 'kPa', label: 'kPa', value: 0, factor: .001 },
                            InH2O: { type: 'InH2O', label: 'In H2O', value: 0, factor: .0040186 },
                            InHg: { type: 'InHg', label: 'In Hg', value: 0, factor: .00029613 },
                            Millibar: { type: 'Millibar', label: 'Millibar', value: 0, factor: .01 },
                            mmHg: { type: 'mmHg', label: 'mm Hg', value: 0, factor: .0075 },
                            Atmospheres: { type: 'Atmospheres', label: 'Atmospheres', value: 0, factor: 0.0000098692 },
                            kgsqcm: { type: 'kgsqcm', label: 'KG/sq. cm.', value: 0, factor: .00001019716 },
                            Bar: { type: 'Bar', label: 'Bar', value: 0, factor: .00001 },
                        },
                        volume: {
                            Liters: { type: 'Liters', label: 'Liters', value: 0, factor: 1 },
                            FluidOunces: { type: 'FluidOunces', label: 'Fluid Ounces', value: 0, factor: 33.8239926 },
                            Quarts: { type: 'Quarts', label: 'Quarts', value: 0, factor: 1.056688261 },
                            Gallons: { type: 'Gallons', label: 'Gallons', value: 0, factor: 0.264172037 },
                            CubicInches: { type: 'CubicInches', label: 'Cu. Inches', value: 0, factor: 61.02375899 },
                            CubicCentimeters: { type: 'CubicCentimeters', label: 'Cu. Centimeters', value: 0, factor: 1000 },
                            CubicFeet: { type: 'CubicFeet', label: 'Cu. Feet', value: 0, factor: 0.03531466668878 },
                            CubicYards: { type: 'CubicYards', label: 'Cu. Yards', value: 0, factor: 0.001307950618103 },
                            CubicMeters: { type: 'CubicMeters', label: 'Cu. Meters', value: 0, factor: 0.001 },
                        },
                        weight: {
                            Kilograms: { type: 'Kilograms', label: 'Kilograms', value: 0, factor: 1 },
                            Ounces: { type: 'Ounces', label: 'Ounces', value: 0, factor: 35.273944 },
                            Pounds: { type: 'Pounds', label: 'Pounds', value: 0, factor: 2.20462262 },
                            EnglishTons: { type: 'EnglishTons', label: 'English Tons', value: 0, factor: 0.00110231075 },
                            MetricTons: { type: 'MetricTons', label: 'Metric Tons', value: 0, factor: 0.001 },
                            Grams: { type: 'Grams', label: 'Grams', value: 0, factor: 1000 },
                        }
                    };
                    vm.fromConversion = vm.conversions.area.sqMeters;
                    vm.toConversion = vm.conversions.area.sqYards;
                    var round = function (value) {
                        if (value) {
                            return Math.round(value * 1000) / 1000;
                        }
                        else {
                            return value;
                        }
                    };
                    vm.convert = function (type, from) {
                        console.log(type, from);
                        if (type == 'temperature') {
                            if (from == 'celsius') {
                                vm.conversions.temperature.fahrenheit.value = round(vm.conversions.temperature.celsius.value * 1.8 + 32);
                            }
                            else {
                                vm.conversions.temperature.celsius.value = round((vm.conversions.temperature.fahrenheit.value - 32) / 1.8);
                            }
                            return;
                        }
                        var baseValue;
                        baseValue = vm.conversions[type][from].value / vm.conversions[type][from].factor;
                        for (var conversionType in vm.conversions[type]) {
                            vm.conversions[type][conversionType].value = round(baseValue * vm.conversions[type][conversionType].factor);
                        }
                    };
                }
            };
            return directive;
        };
        return ConversionCalculatorDirective;
    })();
    ConversionCalculator.ConversionCalculatorDirective = ConversionCalculatorDirective;
    angular.module('ad.ui.bootstrap.conversion-calculator', []).directive('conversionCalculator', ConversionCalculatorDirective.init);
})(ConversionCalculator || (ConversionCalculator = {}));
var ActionBar;
(function (ActionBar) {
    var DropdownDirective = (function () {
        function DropdownDirective() {
        }
        DropdownDirective.init = function () {
            var directive = {
                restrict: 'AE',
                require: 'ngModel',
                replace: false,
                templateUrl: 'dropdown/ad-dropdown.html',
                scope: {
                    items: '=',
                    placeholder: '@',
                    onSelect: '&?',
                    ngDisabled: '=?',
                    ngRequired: '=?',
                    name: '=?',
                    open: '=?',
                    inputSize: '@',
                    displayProperty: '=?',
                    valueProperty: '=?'
                },
                link: function ($scope, element, attr, ngModelController) {
                    var hiddenInput = element.find('input');
                    $scope.repeatExpression = $scope.repeatExpression || 'item in items';
                    $scope.selectedValue = null;
                    $scope.$watch(function () {
                        return ngModelController.$modelValue;
                    }, function (newValue, oldValue) {
                        setSelectedValue(newValue);
                    });
                    $scope.onChange = function (item) {
                        $scope.open = false;
                        var value;
                        if ($scope.valueProperty) {
                            value = item[$scope.valueProperty];
                        }
                        else {
                            value = item;
                        }
                        ngModelController.$setDirty();
                        ngModelController.$setViewValue(value);
                        ngModelController.$render();
                        setSelectedValue(value);
                        if ($scope.onSelect) {
                            $scope.onSelect({ item: value });
                        }
                    };
                    var setSelectedValue = function (value) {
                        if ($scope.valueProperty) {
                            $scope.selectedValue = _.find($scope.items, function (item) {
                                return item[$scope.valueProperty] === value;
                            });
                        }
                        else {
                            $scope.selectedValue = value;
                        }
                    };
                }
            };
            return directive;
        };
        return DropdownDirective;
    })();
    ActionBar.DropdownDirective = DropdownDirective;
    angular.module('ad.ui.bootstrap.ad-dropdown', []).directive('adDropdown', DropdownDirective.init);
})(ActionBar || (ActionBar = {}));
var Icon;
(function (_Icon) {
    var Icon = (function () {
        function Icon() {
        }
        Icon.init = function () {
            var directive = {
                restrict: 'AE',
                replace: true,
                templateUrl: 'fonts/directive/ad-icon.html',
                scope: {
                    title: '=?',
                    iconClass: '=',
                    size: '=?',
                    isFocused: '=?'
                },
                link: function ($scope, element, attr) {
                    $scope.size = $scope.size || 'medium';
                    $scope.iconTitle = $scope.title;
                    $scope.$watch("isFocused", function (currentValue, previousValue) {
                        if (currentValue === true && !previousValue) {
                            element[0].focus();
                        }
                        else if (currentValue === false && previousValue) {
                            element[0].blur();
                        }
                    });
                    switch ($scope.iconClass) {
                        case "appIcon-itype-5":
                            $scope.iconTitle = "Adjustments";
                            break;
                        case "appIcon-itype-2":
                            $scope.iconTitle = "Application & ID";
                            break;
                        case "appIcon-itype-3":
                            $scope.iconTitle = "Description & Operation";
                            break;
                        case "appIcon-itype-7":
                            $scope.iconTitle = "Diagrams";
                            break;
                        case "appIcon-itype-83":
                            $scope.iconTitle = "Exploded Parts Diagrams";
                            break;
                        case "appIcon-itype-8":
                            $scope.iconTitle = "Locations";
                            break;
                        case "appIcon-itype-189":
                            $scope.iconTitle = "Parts and Labor";
                            break;
                        case "appIcon-itype-10":
                            $scope.iconTitle = "Service & Repair";
                            break;
                        case "appIcon-itype-334":
                            if ($scope.title && $scope.title.indexOf('itype') > -1) {
                                $scope.iconTitle = "Service Intervals";
                            }
                            break;
                        case "appIcon-itype-1":
                            $scope.iconTitle = "Service Precautions";
                            break;
                        case "appIcon-itype-9":
                            $scope.iconTitle = "Specifications";
                            break;
                        case "appIcon-itype-6":
                            if ($scope.title && $scope.title.indexOf('itype') > -1) {
                                $scope.iconTitle = "Testing & Inspection";
                            }
                            break;
                        case "appIcon-itype-42":
                            $scope.iconTitle = "Tools & Equipment";
                            break;
                        case "appIcon-itype-13":
                            $scope.iconTitle = "TSBs ";
                            break;
                    }
                }
            };
            return directive;
        };
        return Icon;
    })();
    _Icon.Icon = Icon;
    angular.module('ad.ui.bootstrap.ad-icon', []).directive('adIcon', Icon.init);
})(Icon || (Icon = {}));
var FormElements;
(function (FormElements) {
    var ngApp = angular.module('ad.ui.bootstrap.form-elements', ['checklist-model']);
})(FormElements || (FormElements = {}));
var FormElements;
(function (FormElements) {
    var FormFieldInfoDirective = (function () {
        function FormFieldInfoDirective() {
        }
        FormFieldInfoDirective.init = function () {
            var directive = {
                restrict: 'E',
                replace: true,
                transclude: true,
                templateUrl: 'form-elements/form-field-info/form-field-info.html',
                scope: {},
                link: function ($scope, element, attr) {
                    $scope.active = false;
                    $scope.toggleActive = function () {
                        $scope.active = !$scope.active;
                    };
                }
            };
            return directive;
        };
        return FormFieldInfoDirective;
    })();
    FormElements.FormFieldInfoDirective = FormFieldInfoDirective;
    angular.module('ad.ui.bootstrap.form-elements').directive('formFieldInfo', FormFieldInfoDirective.init);
})(FormElements || (FormElements = {}));
var FormElements;
(function (FormElements) {
    var FormInputDirective = (function () {
        function FormInputDirective() {
        }
        FormInputDirective.init = function () {
            var directive = {
                restrict: 'AE',
                replace: true,
                templateUrl: 'form-elements/form-input/form-input.html',
                scope: {
                    formFieldOptions: '=',
                    ngModel: '=',
                    items: '=adFormFieldItems',
                    ngDisabled: '=',
                    ngBlur: '=?'
                },
                link: function ($scope, element, attr) {
                    $scope.formName = element.closest('form').attr('name');
                    $scope.updateOn = $scope.formFieldOptions.updateOn || 'blur';
                    $scope.debounce = $scope.formFieldOptions.debounce || { 'default': 0, 'blur': 0 };
                    $scope.allowInvalid = $scope.formFieldOptions.allowInvalid;
                    $scope.type = attr.type || $scope.formFieldOptions.type || 'text';
                    $scope.placeholder = attr.placeholder;
                    $scope.autofocus = attr.autofocus;
                    FormInputDirective.InitDisabledAttribute($scope, attr);
                    FormInputDirective.InitChildren($scope, attr);
                    $scope.$watch('ngDisabled', function (oldVal, newValue) {
                        FormInputDirective.InitChildren($scope, attr);
                    });
                    if ($scope.type == 'tel') {
                        $scope.$watch('ngModel', function (newValue, oldValue) {
                            if (!newValue) {
                                return;
                            }
                            var value = newValue.toString().trim().replace(/\D/g, '');
                            if (value.match(/[^0-9]/)) {
                                return;
                            }
                            var country, city, number;
                            switch (value.length) {
                                case 10:
                                    country = 1;
                                    city = value.slice(0, 3);
                                    number = value.slice(3);
                                    break;
                                case 11:
                                    country = value[0];
                                    city = value.slice(1, 4);
                                    number = value.slice(4);
                                    break;
                                case 12:
                                    country = value.slice(0, 3);
                                    city = value.slice(3, 5);
                                    number = value.slice(5);
                                    break;
                                default:
                                    return;
                            }
                            if (country == 1) {
                                country = "";
                            }
                            number = number.slice(0, 3) + '-' + number.slice(3);
                            $scope.ngModel = (country + " (" + city + ") " + number).trim();
                        });
                    }
                }
            };
            return directive;
        };
        FormInputDirective.InitChildren = function ($scope, attr) {
            $scope.validators = [];
            FormInputDirective.InitRequiredValidation($scope, attr);
            FormInputDirective.InitLengthValidation($scope, attr);
            FormInputDirective.InitNumberValidation($scope, attr);
            FormInputDirective.InitPatternValidation($scope, attr);
            FormInputDirective.InitTelValidation($scope, attr);
            FormInputDirective.InitEmailValidation($scope, attr);
            FormInputDirective.InitTextAreaValidation($scope, attr);
            FormInputDirective.InitListValidation($scope, attr);
        };
        FormInputDirective.InitDisabledAttribute = function ($scope, attr) {
            if (typeof attr.disabled !== "undefined") {
                $scope.disabled = attr.disabled === "" || attr.disabled === "true" || attr.disabled === "disabled" ? true : false;
            }
        };
        FormInputDirective.IsDisabled = function ($scope, attr) {
            return $scope.disabled || $scope.ngDisabled;
        };
        FormInputDirective.InitRequiredValidation = function ($scope, attr) {
            var isRequired = !FormInputDirective.IsDisabled($scope, attr) && attr.required;
            if (isRequired) {
                $scope.validators.push({
                    message: $scope.formFieldOptions.label + ' is required.',
                    type: 'required',
                    show: function () {
                        return ($scope.$parent[$scope.formName].$submitted || ($scope.$parent[$scope.formName][$scope.formFieldOptions.name] && $scope.$parent[$scope.formName][$scope.formFieldOptions.name].$touched)) && $scope.$parent[$scope.formName][$scope.formFieldOptions.name].$error['required'];
                    }
                });
            }
            $scope.isRequired = isRequired;
        };
        FormInputDirective.InitListValidation = function ($scope, attr) {
            if ($scope.formFieldOptions.type === 'list') {
                var itemLabel = $scope.formFieldOptions.itemLabel;
                var itemValue = $scope.formFieldOptions.itemValue;
                $scope.selectMultiple = $scope.formFieldOptions.selectMultiple || false;
                $scope.listType = $scope.formFieldOptions.listType || ($scope.items.length > 3 ? 'select' : ($scope.selectMultiple ? 'checkbox' : 'radio'));
                if ($scope.listType === 'select') {
                    if (typeof $scope.items[0] !== "object") {
                        $scope.selectOptions = "item for item in items";
                    }
                    else if (itemLabel && itemValue) {
                        $scope.selectOptions = 'item["' + itemValue + '"] as item["' + itemLabel + '"]  for item in items';
                    }
                    else {
                        $scope.selectOptions = 'item["' + itemLabel + '"] for item in items';
                    }
                }
                else {
                    $scope.getItemValue = function ($index) {
                        return (typeof $scope.items[0] !== "object") ? $scope.items[$index] : (itemValue ? $scope.items[$index][itemValue] : $scope.items[$index][itemLabel]);
                    };
                    $scope.getItemLabel = function ($index) {
                        return (typeof $scope.items[0] !== "object") ? $scope.items[$index] : $scope.items[$index][itemLabel];
                    };
                    if (typeof $scope.items[0] !== "object") {
                        $scope.itemValueProperty = 'item';
                    }
                    else {
                        $scope.itemValueProperty = itemValue ? 'item["' + itemValue + '"]' : 'item["' + itemLabel + '"]';
                    }
                }
            }
        };
        FormInputDirective.InitPatternValidation = function ($scope, attr) {
            if (FormInputDirective.IsDisabled($scope, attr)) {
                return;
            }
            if (attr.ngPattern) {
                $scope.pattern = attr.ngPattern;
                $scope.validators.push({
                    message: $scope.formFieldOptions.label + ' needs to be in the correct format.',
                    type: 'pattern',
                    show: function () {
                        return ($scope.$parent[$scope.formName].$submitted || $scope.$parent[$scope.formName][$scope.formFieldOptions.name].$touched) && $scope.$parent[$scope.formName][$scope.formFieldOptions.name].$error['pattern'];
                    }
                });
            }
        };
        FormInputDirective.InitEmailValidation = function ($scope, attr) {
            if (FormInputDirective.IsDisabled($scope, attr)) {
                return;
            }
            if ($scope.formFieldOptions.type === 'email') {
                $scope.pattern = FormInputDirective.EMAIL_REGEXP;
                $scope.validators.push({
                    message: $scope.formFieldOptions.label + ' needs to be in the correct format. (xxxx@yyy.zzz)',
                    type: 'pattern',
                    show: function () {
                        return ($scope.$parent[$scope.formName].$submitted || $scope.$parent[$scope.formName][$scope.formFieldOptions.name].$touched) && $scope.$parent[$scope.formName][$scope.formFieldOptions.name].$error['pattern'];
                    }
                });
            }
        };
        FormInputDirective.InitTelValidation = function ($scope, attr) {
            if (FormInputDirective.IsDisabled($scope, attr)) {
                return;
            }
            if ($scope.formFieldOptions.type === 'tel') {
                $scope.pattern = FormInputDirective.PHONE_REGEXP;
                $scope.validators.push({
                    message: $scope.formFieldOptions.label + ' needs to be in the correct format. (999)999-9999',
                    type: 'pattern',
                    show: function () {
                        return ($scope.$parent[$scope.formName].$submitted || $scope.$parent[$scope.formName][$scope.formFieldOptions.name].$touched) && $scope.$parent[$scope.formName][$scope.formFieldOptions.name].$error['pattern'];
                    }
                });
            }
        };
        FormInputDirective.InitTextAreaValidation = function ($scope, attr) {
            if ($scope.type === 'textarea') {
                $scope.rows = $scope.formFieldOptions.rows || 5;
            }
        };
        FormInputDirective.InitLengthValidation = function ($scope, attr) {
            if (FormInputDirective.IsDisabled($scope, attr)) {
                return;
            }
            if (attr.ngMaxlength) {
                $scope.maxlength = attr.ngMaxlength;
            }
            if (attr.ngMinlength) {
                $scope.minlength = attr.ngMinlength;
            }
            if (attr.ngMinlength && attr.ngMaxlength) {
                $scope.validators.push({
                    message: $scope.formFieldOptions.label + ' needs to contain between ' + attr.ngMinlength + ' and ' + attr.ngMaxlength + ' characters.',
                    type: 'minlength',
                    show: function () {
                        return $scope.$parent[$scope.formName][$scope.formFieldOptions.name].$error['minlength'] || $scope.$parent[$scope.formName][$scope.formFieldOptions.name].$error['maxlength'];
                    }
                });
            }
            else {
                if (attr.ngMinlength) {
                    $scope.validators.push({
                        message: $scope.formFieldOptions.label + ' needs to contain at least ' + attr.ngMinlength + ' characters.',
                        type: 'minlength',
                        show: function () {
                            return $scope.$parent[$scope.formName][$scope.formFieldOptions.name].$error['minlength'];
                        }
                    });
                }
                if (attr.ngMaxlength) {
                    $scope.validators.push({
                        message: $scope.formFieldOptions.label + ' needs to contain at most ' + attr.ngMaxlength + ' characters.',
                        type: 'maxlength',
                        show: function () {
                            return $scope.$parent[$scope.formName][$scope.formFieldOptions.name].$error['maxlength'];
                        }
                    });
                }
            }
        };
        FormInputDirective.InitNumberValidation = function ($scope, attr) {
            if (FormInputDirective.IsDisabled($scope, attr)) {
                return;
            }
            if ($scope.formFieldOptions.type !== 'number') {
                return;
            }
            $scope.precision = $scope.formFieldOptions.precision || 0;
            $scope.validators.push({
                message: $scope.formFieldOptions.label + ' needs to be a valid number.',
                type: 'number',
                show: function () {
                    if ($scope.$parent[$scope.formName] && $scope.$parent[$scope.formName][$scope.formFieldOptions.name])
                        return $scope.$parent[$scope.formName][$scope.formFieldOptions.name].$error['number'];
                    return false;
                }
            });
            if (attr.min) {
                $scope.min = attr.min;
            }
            if (attr.max) {
                $scope.max = attr.max;
            }
            if (attr.min && attr.max) {
                $scope.validators.push({
                    message: $scope.formFieldOptions.label + ' needs to have a value between ' + attr.min + ' and ' + attr.max + '.',
                    type: 'min',
                    show: function () {
                        return $scope.$parent[$scope.formName][$scope.formFieldOptions.name].$error['min'] || $scope.$parent[$scope.formName][$scope.formFieldOptions.name].$error['max'];
                    }
                });
            }
            else {
                if (attr.min) {
                    $scope.validators.push({
                        message: $scope.formFieldOptions.label + ' needs a minimum value of ' + attr.min + '.',
                        type: 'min',
                        show: function () {
                            return $scope.$parent[$scope.formName][$scope.formFieldOptions.name].$error['min'];
                        }
                    });
                }
                if (attr.max) {
                    $scope.validators.push({
                        message: $scope.formFieldOptions.label + ' needs a maximum value of ' + attr.max + '.',
                        type: 'max',
                        show: function () {
                            return $scope.$parent[$scope.formName][$scope.formFieldOptions.name].$error['max'];
                        }
                    });
                }
            }
            $scope.limitToNumbers = function (e) {
                if ($.inArray(e.keyCode, [46, 8, 9, 27, 13, 110, 190]) !== -1 || (e.keyCode == 65 && (e.ctrlKey === true || e.metaKey === true)) || (e.keyCode >= 35 && e.keyCode <= 40)) {
                    return;
                }
                if ((e.shiftKey || (e.keyCode < 48 || e.keyCode > 57)) && (e.keyCode < 96 || e.keyCode > 105)) {
                    e.preventDefault();
                }
            };
        };
        FormInputDirective.PHONE_REGEXP = /^[(]{0,1}[0-9]{3}[)\.\- ]{0,1}[ ]{0,1}[0-9]{3}[\.\- ]{0,1}[0-9]{4}$/;
        FormInputDirective.EMAIL_REGEXP = /[a-zA-Z0-9._%+-]+@(?:[a-zA-Z0-9-]+\.)+[a-zA-Z]{2,15}$/;
        return FormInputDirective;
    })();
    FormElements.FormInputDirective = FormInputDirective;
    angular.module('ad.ui.bootstrap.form-elements').directive('adFormInput', FormInputDirective.init);
})(FormElements || (FormElements = {}));
var List;
(function (List) {
    var ItemListController = (function () {
        function ItemListController($scope) {
            this.$scope = $scope;
            $scope.isObject = function (item) {
                return angular.isObject(item);
            };
            $scope.doAction = function (item) {
                $scope.onAction({ item: item });
            };
            $scope.isEmptyList = function (items) {
                if ($scope.enableShortcuts == false) {
                    return true;
                }
                return $scope.hideEmpty && items.length === 0;
            };
            $scope.isNoShortcuts = function () {
                return $scope.enableShortcuts == false;
            };
        }
        ItemListController.$inject = ['$scope'];
        return ItemListController;
    })();
    List.ItemListController = ItemListController;
})(List || (List = {}));
var List;
(function (List) {
    var ItemShortcutsController = (function () {
        function ItemShortcutsController($scope) {
            this.$scope = $scope;
            $scope.alphabet = "ABCDEFGHIJKLMNOPQRSTUVWXYZ".split('');
            $scope.filteredAlphabet = [];
            $scope.jumpTo = jumpTo;
            $scope.filterAlphabet = filterAlphabet;
            $scope.filterAlphabet();
            function jumpTo(event, letter) {
                var itemWrapper = $(event.toElement).closest('.list').find('.itemListWrapper');
                var itemList = itemWrapper.find('.itemList');
                var element = itemList.find('.header-' + letter);
                itemWrapper.scrollTop(element.position().top);
            }
            function filterAlphabet() {
                $scope.filteredAlphabet = [];
                $scope.alphabet.forEach(function (letter) {
                    $scope.sortedItems[letter] = $scope.itemList.filter(function (item) {
                        return sortBy(item, letter);
                    });
                    if ($scope.sortedItems[letter].length > 0)
                        $scope.filteredAlphabet.push(letter);
                });
            }
            function sortBy(item, letter) {
                if ($scope.sortingField === 'toString()') {
                    return String(item).toUpperCase().charAt(0) == letter;
                }
                else {
                    return String(item[$scope.sortingField]).toUpperCase().charAt(0) == letter;
                }
            }
        }
        ItemShortcutsController.$inject = ['$scope'];
        return ItemShortcutsController;
    })();
    List.ItemShortcutsController = ItemShortcutsController;
})(List || (List = {}));
var List;
(function (List) {
    var ListController = (function () {
        function ListController($scope) {
            this.$scope = $scope;
            $scope.enableSearch = $scope.enableSearch || false;
            $scope.enableShortcuts = $scope.enableShortcuts || false;
            $scope.hideEmpty = $scope.hideEmpty || false;
            $scope.itemList = $scope.itemList || [];
            $scope.onAction = $scope.onAction || {};
            $scope.sortedItems = {};
            $scope.renderOrder = $scope.renderOrder || [];
            $scope.sortingField = ($scope.sortingField == undefined) ? 'toString()' : $scope.sortingField;
            $scope.filterByTerm = '';
            $scope.doSearch = function (searchTerm, parentScope) {
                $scope.filterByTerm = searchTerm;
            };
        }
        ListController.$inject = ['$scope'];
        return ListController;
    })();
    List.ListController = ListController;
})(List || (List = {}));
var List;
(function (List) {
    var ItemListDirective = (function () {
        function ItemListDirective() {
        }
        ItemListDirective.init = function ($timeout) {
            var directive = {
                templateUrl: 'list/partials/itemList.html',
                replace: true,
                restrict: 'E',
                controller: 'itemListController',
                scope: true,
                link: function (scope, element) {
                    $timeout(function () {
                        doRender(this.scope, this.element);
                        this.scope.$watchCollection('itemList', updateRenderOrder.bind({ scope: this.scope, element: this.element }));
                        this.scope.$watchCollection('filterByTerm', updateRenderOrder.bind({ scope: this.scope, element: this.element }));
                        function doRender(scope, element) {
                            if (scope.renderOrder.length > 0 && scope.itemList.length > 0) {
                                renderInOrder(scope, element);
                            }
                        }
                        function renderInOrder(scope, element) {
                            element.find('li.itemListBody').each(function (index, item) {
                                for (var orderIndex = 1; orderIndex <= scope.renderOrder.length; orderIndex++) {
                                    var previousChildren = $(item).find('.' + scope.renderOrder[orderIndex - 1]);
                                    var children = $(item).find('.' + scope.renderOrder[orderIndex]);
                                    children.insertAfter(previousChildren);
                                }
                            }.bind(scope));
                        }
                        function updateRenderOrder() {
                            $timeout(function () {
                                doRender(this.scope, this.element);
                            }.bind({ scope: scope, element: element }));
                        }
                    }.bind({ scope: scope, element: element }));
                }
            };
            return directive;
        };
        return ItemListDirective;
    })();
    List.ItemListDirective = ItemListDirective;
})(List || (List = {}));
var List;
(function (List) {
    var ItemShortcutsDirective = (function () {
        function ItemShortcutsDirective() {
        }
        ItemShortcutsDirective.init = function ($timeout) {
            var directive = {
                templateUrl: 'list/partials/itemShortcuts.html',
                replace: true,
                restrict: 'E',
                controller: 'itemShortcutsController',
                scope: true,
                link: function (scope, element) {
                    $timeout(function () {
                        doCollapseShortcuts(this.scope, this.element);
                        this.scope.$watch('itemList', updateShortcuts.bind({ scope: this.scope, element: this.element }));
                        function doCollapseShortcuts(scope, element) {
                            if (scope.itemList.length > 0) {
                                var shortcuts = element.children();
                                resetShortcuts(shortcuts);
                                collapseShortcuts(shortcuts);
                                $(window).resize(function () {
                                    resetShortcuts(this.shortcuts);
                                    collapseShortcuts(this.shortcuts);
                                }.bind({ shortcuts: shortcuts }));
                            }
                        }
                        function resetShortcuts(shortcuts) {
                            shortcuts.css('padding-top', '0px');
                            shortcuts.show().find('div').not('.separator').show();
                            shortcuts.find('.separator').hide();
                        }
                        function collapseShortcuts(shortcuts) {
                            if (shortcuts.length > 0) {
                                var lastShortcut = shortcuts.last();
                                var parent = lastShortcut.closest('.itemShortcutsWrapper');
                                var lastShortcutTopOffset = parent.height() - lastShortcut.position().top;
                                var lastShortcutBottomOffset = lastShortcutTopOffset + lastShortcut.height();
                                if (lastShortcutTopOffset < 0 || lastShortcutBottomOffset < 0) {
                                    var evenShortcuts = filterShortcuts(shortcuts, true);
                                    var oddShortcuts = filterShortcuts(shortcuts, false);
                                    setShortcutSeparators(evenShortcuts);
                                    hideShortcuts(oddShortcuts);
                                    return collapseShortcuts(evenShortcuts);
                                }
                                else {
                                    updateShortcutMargins(shortcuts, parent);
                                    return;
                                }
                            }
                        }
                        function filterShortcuts(shortcuts, filterByEven) {
                            return shortcuts.filter(function (index) {
                                if (this.filterByEven)
                                    return index % 2 == 0;
                                else
                                    return index % 2 == 1;
                            }.bind({ filterByEven: filterByEven }));
                        }
                        function setShortcutSeparators(shortcuts) {
                            shortcuts.find('.separator').show();
                            shortcuts.last().find('.separator').hide();
                        }
                        function hideShortcuts(shortcuts) {
                            shortcuts.hide();
                            shortcuts.children().hide();
                        }
                        function updateShortcutMargins(shortcuts, parent) {
                            var separators = shortcuts.find('.separator:visible');
                            var elementCount = shortcuts.length + separators.length;
                            shortcuts.css('padding-top', '0px');
                            separators.css('padding-top', '0px');
                            var availableHeight = parent.height() - (shortcuts.last().position().top + shortcuts.last().height());
                            var padding = Math.floor(availableHeight / elementCount);
                            shortcuts.css('padding-top', padding + 'px');
                            separators.css('padding-top', padding + 'px');
                            shortcuts.first().css('padding-top', (padding / 2) + 'px');
                        }
                        function updateShortcuts() {
                            this.scope.filterAlphabet();
                            $timeout(function () {
                                doCollapseShortcuts(this.scope, this.element);
                            }.bind({ scope: scope, element: element }));
                        }
                    }.bind({ scope: scope, element: element }));
                }
            };
            return directive;
        };
        return ItemShortcutsDirective;
    })();
    List.ItemShortcutsDirective = ItemShortcutsDirective;
})(List || (List = {}));
var List;
(function (List) {
    var ListDirective = (function () {
        function ListDirective() {
        }
        ListDirective.init = function () {
            var directive = {
                templateUrl: 'list/partials/list.html',
                replace: true,
                restrict: 'E',
                controller: 'listController',
                scope: {
                    onAction: '&?',
                    itemList: '=',
                    enableSearch: '=?',
                    enableShortcuts: '=?',
                    sortingField: '=?',
                    hideEmpty: '=?',
                    renderOrder: '=?'
                }
            };
            return directive;
        };
        return ListDirective;
    })();
    List.ListDirective = ListDirective;
})(List || (List = {}));
var List;
(function (List) {
    var app = angular.module('ad.ui.bootstrap.list', ['ad.ui.bootstrap.searchBox']);
    app.controller('listController', List.ListController);
    app.controller('itemListController', List.ItemListController);
    app.controller('itemShortcutsController', List.ItemShortcutsController);
    app.directive('list', List.ListDirective.init);
    app.directive('itemList', ['$timeout', List.ItemListDirective.init]);
    app.directive('itemShortcuts', ['$timeout', List.ItemShortcutsDirective.init]);
})(List || (List = {}));
var Navigation;
(function (Navigation) {
    var MenuController = (function () {
        function MenuController($scope) {
            this.$scope = $scope;
            $scope.menuVm = this;
            $scope.content = $scope.content || {};
        }
        MenuController.$inject = ['$scope'];
        return MenuController;
    })();
    Navigation.MenuController = MenuController;
})(Navigation || (Navigation = {}));
var Navigation;
(function (Navigation) {
    var NavigationController = (function () {
        function NavigationController($scope) {
            this.$scope = $scope;
            $scope.content = $scope.content || {};
            $scope.content.selectItem = function (item) {
                $scope.onSelection({ item: item.id });
            };
            $scope.onResize = $scope.onResize || function () {
            };
        }
        NavigationController.$inject = ['$scope'];
        return NavigationController;
    })();
    Navigation.NavigationController = NavigationController;
})(Navigation || (Navigation = {}));
var Navigation;
(function (Navigation) {
    var ngApp = angular.module('ad.ui.bootstrap.navigation', []);
    ngApp.controller('navigationController', Navigation.NavigationController);
    ngApp.controller('menuController', Navigation.MenuController);
})(Navigation || (Navigation = {}));
var Navigation;
(function (Navigation) {
    var AlldataLogoDirective = (function () {
        function AlldataLogoDirective() {
        }
        AlldataLogoDirective.init = function () {
            var directive = {
                restrict: 'E',
                replace: true,
                templateUrl: 'navigation/partials/alldataLogo.html',
                scope: {
                    onSelect: '&'
                },
                link: function (scope, element) {
                    element.on('click', function (evt) {
                        scope.onSelect && scope.onSelect();
                    });
                }
            };
            return directive;
        };
        return AlldataLogoDirective;
    })();
    Navigation.AlldataLogoDirective = AlldataLogoDirective;
    angular.module('ad.ui.bootstrap.navigation').directive('alldataLogo', [AlldataLogoDirective.init]);
})(Navigation || (Navigation = {}));
var Navigation;
(function (Navigation) {
    var MenuDirective = (function () {
        function MenuDirective() {
        }
        MenuDirective.init = function (NavigationService) {
            var directive = {
                restrict: 'E',
                replace: true,
                templateUrl: 'navigation/partials/menu.html',
                controller: 'menuController'
            };
            return directive;
        };
        return MenuDirective;
    })();
    Navigation.MenuDirective = MenuDirective;
    angular.module('ad.ui.bootstrap.navigation').directive('navigationMenu', ['NavigationService', MenuDirective.init]);
})(Navigation || (Navigation = {}));
var Navigation;
(function (Navigation) {
    var NavigationDirective = (function () {
        function NavigationDirective() {
        }
        NavigationDirective.init = function (NavigationService) {
            var directive = {
                restrict: 'E',
                replace: true,
                scope: {
                    content: '='
                },
                templateUrl: 'navigation/partials/navigation.html',
                link: function (scope, element) {
                    NavigationService.registerMenu(element);
                    var icon = element.find('.resizeIcon');
                    var toggleMenu = function () {
                        NavigationService.toggleMenu(element);
                    };
                    icon.click(toggleMenu);
                }
            };
            return directive;
        };
        return NavigationDirective;
    })();
    Navigation.NavigationDirective = NavigationDirective;
    angular.module('ad.ui.bootstrap.navigation').directive('adNav', ['NavigationService', NavigationDirective.init]);
})(Navigation || (Navigation = {}));
var Navigation;
(function (Navigation) {
    var ProfileDirective = (function () {
        function ProfileDirective() {
        }
        ProfileDirective.init = function () {
            var directive = {
                restrict: 'E',
                replace: true,
                templateUrl: 'navigation/partials/profile.html'
            };
            return directive;
        };
        return ProfileDirective;
    })();
    Navigation.ProfileDirective = ProfileDirective;
})(Navigation || (Navigation = {}));
var Navigation;
(function (Navigation) {
    var ShellContentDirective = (function () {
        function ShellContentDirective() {
        }
        ShellContentDirective.init = function () {
            var directive = {
                restrict: 'E',
                replace: true,
                transclude: true,
                templateUrl: 'navigation/partials/shell-content.html',
                link: function (scope, element) {
                }
            };
            return directive;
        };
        return ShellContentDirective;
    })();
    Navigation.ShellContentDirective = ShellContentDirective;
    angular.module('ad.ui.bootstrap.navigation').directive('adContent', [ShellContentDirective.init]);
})(Navigation || (Navigation = {}));
var Navigation;
(function (Navigation) {
    var ShellDirective = (function () {
        function ShellDirective() {
        }
        ShellDirective.init = function ($compile) {
            var directive = {
                restrict: 'E',
                replace: true,
                transclude: true,
                templateUrl: 'navigation/partials/ad-shell.html',
                scope: {
                    appName: '@',
                    navContent: '=',
                    onLogoSelect: '&?',
                    onAppNameSelect: '&?',
                    overflowTreshold: '=?'
                },
                link: function (scope, element, attr, ctrl, transclude) {
                }
            };
            return directive;
        };
        return ShellDirective;
    })();
    Navigation.ShellDirective = ShellDirective;
    angular.module('ad.ui.bootstrap.navigation').directive('adShell', [ShellDirective.init]);
})(Navigation || (Navigation = {}));
var Navigation;
(function (Navigation) {
    var ShellTopNavDirective = (function () {
        function ShellTopNavDirective() {
        }
        ShellTopNavDirective.init = function () {
            var directive = {
                restrict: 'E',
                replace: true,
                transclude: true,
                templateUrl: 'navigation/partials/shell-topnav.html',
                scope: {
                    appName: '@'
                },
                link: function (scope, element) {
                }
            };
            return directive;
        };
        return ShellTopNavDirective;
    })();
    Navigation.ShellTopNavDirective = ShellTopNavDirective;
    angular.module('ad.ui.bootstrap.navigation').directive('adHeader', [ShellTopNavDirective.init]);
})(Navigation || (Navigation = {}));
var Navigation;
(function (Navigation) {
    var SubMenuDirective = (function () {
        function SubMenuDirective() {
        }
        SubMenuDirective.init = function (NavigationService) {
            var directive = {
                restrict: 'E',
                replace: true,
                templateUrl: 'navigation/partials/subMenuItem.html',
                scope: {
                    item: '='
                },
                link: function (scope, element) {
                    if (typeof scope.item.enabled === 'undefined') {
                        scope.item.enabled = function () {
                            return true;
                        };
                    }
                    element.on('click', function (evt) {
                        if (scope.item.enabled()) {
                            scope.item.onSelect && scope.$apply(function () {
                                scope.item.onSelect();
                            });
                            evt.preventDefault();
                            return false;
                        }
                        else {
                            return false;
                        }
                    });
                }
            };
            return directive;
        };
        return SubMenuDirective;
    })();
    Navigation.SubMenuDirective = SubMenuDirective;
    angular.module('ad.ui.bootstrap.navigation').directive('subMenuItem', ['NavigationService', SubMenuDirective.init]);
})(Navigation || (Navigation = {}));
var Navigation;
(function (Navigation) {
    var TopMenuDirective = (function () {
        function TopMenuDirective() {
        }
        TopMenuDirective.init = function (NavigationService) {
            var directive = {
                restrict: 'E',
                replace: true,
                templateUrl: 'navigation/partials/topMenuItem.html',
                scope: {
                    item: '='
                },
                link: function (scope, element) {
                    if (typeof scope.item.enabled === 'undefined') {
                        scope.item.enabled = function () {
                            return true;
                        };
                    }
                    element.on('click', function (evt) {
                        if (scope.item.enabled()) {
                            if (scope.item.children) {
                                NavigationService.expandMenu(element.closest('.shell-navigation'), false);
                                if (element.hasClass('closed')) {
                                    NavigationService.hideSubMenus(element, true);
                                    element.removeClass('closed').addClass('open');
                                }
                                else {
                                    NavigationService.hideSubMenus(element, true);
                                }
                            }
                            else {
                                NavigationService.hideSubMenus(element, true);
                                scope.item.onSelect && scope.item.onSelect();
                            }
                        }
                    }.bind({ scope: scope }));
                }
            };
            return directive;
        };
        return TopMenuDirective;
    })();
    Navigation.TopMenuDirective = TopMenuDirective;
    angular.module('ad.ui.bootstrap.navigation').directive('topMenuItem', ['NavigationService', TopMenuDirective.init]);
})(Navigation || (Navigation = {}));
var Navigation;
(function (Navigation) {
    var NavigationService = (function () {
        function NavigationService($document, $timeout) {
            this.$document = $document;
            this.$timeout = $timeout;
        }
        NavigationService.prototype.registerMenu = function (menuElement) {
            var _this = this;
            this.$timeout(function () {
                _this.setupMenu(menuElement);
                $(window).on('resize', function () {
                    _this.setupMenu(menuElement);
                });
            });
        };
        NavigationService.prototype.toggleMenu = function (menuElement) {
            if (menuElement.hasClass('collapsed')) {
                this.expandMenu(menuElement, true);
            }
            else {
                this.collapseMenu(menuElement, true);
            }
        };
        NavigationService.prototype.setupMenu = function (navigation) {
            var viewPortWidth = Math.max(document.documentElement.clientWidth, window.innerWidth || 0);
            var icon = navigation.find('.resizeIcon');
            if (viewPortWidth <= 1280) {
                this.collapseMenu(navigation, true);
            }
            else {
                this.expandMenu(navigation, true);
            }
        };
        NavigationService.prototype.expandMenu = function (navigation, animate) {
            var viewPortWidth = Math.max(document.documentElement.clientWidth, window.innerWidth || 0);
            var icon = navigation.find('.resizeIcon');
            var siblings = navigation.siblings('.shell-content');
            icon.addClass('appIcon-circle-left').removeClass('appIcon-circle-right');
            navigation.addClass('expanded').removeClass('collapsed');
            if (viewPortWidth > 1280) {
                siblings.addClass('expanded').removeClass('collapsed');
            }
            if (animate) {
                navigation.addClass('animation');
                siblings.addClass('animation');
            }
            else {
                navigation.removeClass('animation');
                siblings.removeClass('animation');
            }
        };
        NavigationService.prototype.collapseMenu = function (navigation, animate) {
            var icon = navigation.find('.resizeIcon');
            var subMenuItems = $('.navigation').find('.navigation-body li.item').get(0);
            var siblings = navigation.siblings('.shell-content');
            icon.addClass('appIcon-circle-right').removeClass('appIcon-circle-left');
            navigation.addClass('collapsed').removeClass('expanded');
            siblings.addClass('collapsed').removeClass('expanded');
            this.hideSubMenus($(subMenuItems), true);
        };
        NavigationService.prototype.hideSubMenus = function (element, hideAll) {
            var openMenuItems = element.closest('.level-1').find('> .item.open');
            openMenuItems.removeClass('open').addClass('closed');
        };
        NavigationService.$inject = ['$document', '$timeout'];
        return NavigationService;
    })();
    angular.module('ad.ui.bootstrap.navigation').service('NavigationService', ['$document', '$timeout', NavigationService]);
})(Navigation || (Navigation = {}));
var PDF;
(function (PDF) {
    var Viewer;
    (function (Viewer) {
        var Directive = (function () {
            function Directive(adSpinnerService) {
                this.adSpinnerService = adSpinnerService;
                this.restrict = 'E';
                this.templateUrl = 'pdf/partials/pdfViewerTemplate.html';
                this.replace = true;
                this.scope = {
                    url: '=',
                    host: '=?',
                    path: '=?',
                    callback: '&?'
                };
                this.link = LinkFunction.bind(this);
            }
            Directive.Factory = function (adSpinnerService) {
                return new Directive(adSpinnerService);
            };
            Directive.$inject = ['adSpinnerService'];
            return Directive;
        })();
        Viewer.Directive = Directive;
        function LinkFunction(scope, element, attrs, ctrl) {
            var self = this, host = scope.host || 'http://vmd-mblweb-001.alldata.com', path = scope.path || '/build/viewer/viewer.html?file={file}', viewerUrl = (host + path).replace('{file}', scope.url), binds = {
                scope: scope,
                adSpinnerService: self.adSpinnerService
            };
            element.attr('src', viewerUrl);
            element.on('load', binds, function (evt) {
                var binds = evt.data;
                if ('callback' in binds.scope && typeof binds.scope.callback === 'function') {
                    binds.scope.callback();
                }
                binds.adSpinnerService.end();
            });
            this.adSpinnerService.start();
        }
    })(Viewer = PDF.Viewer || (PDF.Viewer = {}));
})(PDF || (PDF = {}));
var PDF;
(function (PDF) {
    var ngApp = angular.module('ad.ui.bootstrap.pdf', ['ad.ui.services']);
    ngApp.directive('pdfViewer', PDF.Viewer.Directive.Factory);
})(PDF || (PDF = {}));
var Services;
(function (Services) {
    var ModalService;
    (function (ModalService) {
        var Factory = (function () {
            function Factory() {
            }
            Factory.prototype.start = function (context) {
                var $body = $('body');
                if ($('.modal' + context).length === 0) {
                    $body.append($('<div class="modal-backdrop fade in modal' + context + '"></div>'));
                    $body.addClass('hasModal');
                }
            };
            Factory.prototype.end = function (context) {
                $(".modal" + context).remove();
                if ($('.modal-backdrop').length === 0) {
                    $('body').removeClass('hasModal');
                }
            };
            Factory.prototype.endAll = function () {
                $(".modal-backdrop").remove();
                if ($('.modal-backdrop').length === 0) {
                    $('body').removeClass('hasModal');
                }
            };
            return Factory;
        })();
        ModalService.Factory = Factory;
    })(ModalService = Services.ModalService || (Services.ModalService = {}));
})(Services || (Services = {}));
var Services;
(function (Services) {
    var RecursionHelperService;
    (function (RecursionHelperService) {
        var Factory = (function () {
            function Factory($compile) {
                this.$compile = $compile;
            }
            Factory.prototype.compile = function (element, link) {
                var self = this;
                if (angular.isFunction(link)) {
                    link = { post: link };
                }
                var contents = element.contents().remove();
                var compiledContents;
                return {
                    pre: (link && link.pre) ? link.pre : null,
                    post: function (scope, element) {
                        if (!compiledContents) {
                            compiledContents = self.$compile(contents);
                        }
                        compiledContents(scope, function (clone) {
                            element.append(clone);
                        });
                        if (link && link.post) {
                            link.post.apply(null, arguments);
                        }
                    }
                };
            };
            Factory.$inject = ['$compile'];
            return Factory;
        })();
        RecursionHelperService.Factory = Factory;
    })(RecursionHelperService = Services.RecursionHelperService || (Services.RecursionHelperService = {}));
})(Services || (Services = {}));
var Services;
(function (Services) {
    var ScrollService;
    (function (ScrollService) {
        var Factory = (function () {
            function Factory() {
            }
            Factory.prototype.scrollTo = function (top, $elem) {
                $($elem).scrollTop(top);
            };
            Factory.prototype.scrollAllDivsToTop = function () {
                $('div').scrollTop(0);
            };
            return Factory;
        })();
        ScrollService.Factory = Factory;
    })(ScrollService = Services.ScrollService || (Services.ScrollService = {}));
})(Services || (Services = {}));
var Services;
(function (Services) {
    var SpinnerService;
    (function (SpinnerService) {
        var Factory = (function () {
            function Factory(adModalService) {
                this.adModalService = adModalService;
            }
            Factory.prototype.start = function () {
                this.end();
                this.adModalService.start('spinner');
                var spinner = $('<div class="modal"><div class="spinner centered-middle"><div class="rect1"></div><div class="rect2"></div><div class="rect3"></div><div class="rect4"></div><div class="rect5"></div></div></div>');
                $('body').append(spinner);
            };
            Factory.prototype.end = function () {
                var currentSpinner = $('.spinner');
                if (currentSpinner.length > 0) {
                    currentSpinner.parent().remove();
                }
                this.adModalService.end('spinner');
            };
            Factory.$inject = ['adModalService'];
            return Factory;
        })();
        SpinnerService.Factory = Factory;
    })(SpinnerService = Services.SpinnerService || (Services.SpinnerService = {}));
})(Services || (Services = {}));
var Services;
(function (Services) {
    var ValidationService;
    (function (ValidationService) {
        var ValidatorTypes = ['required', 'minLength', 'maxLength'], ValidatorMessages = ['This is a required field.', 'The minimun length for this field is {value}.', 'The maximum length for this field is {value}.'];
        var Factory = (function () {
            function Factory() {
            }
            Factory.prototype.getValidatorTypes = function () {
                return ValidatorTypes;
            };
            Factory.prototype.getValidatorMessages = function () {
                return ValidatorMessages;
            };
            Factory.prototype.validateField = function (value, validators) {
                var errorList = [], index, validator, error;
                for (index in validators) {
                    validator = validators[index];
                    error = '';
                    if (validator.name === 'required')
                        error = ValidateRequired(value, validator);
                    else if (validator.name === 'minLength')
                        error = ValidateMinLength(value, validator);
                    else if (validator.name === 'maxLength')
                        error = ValidateMaxLength(value, validator);
                    if (error.length !== 0)
                        errorList.push(error);
                }
                return errorList;
            };
            return Factory;
        })();
        ValidationService.Factory = Factory;
        function ValidateRequired(value, validator) {
            var error = '';
            if (value.length === 0)
                error = validator.message;
            return error;
        }
        function ValidateMinLength(value, validator) {
            var error = '';
            if (value.length < validator.value)
                error = validator.message.replace('{value}', validator.value);
            return error;
        }
        function ValidateMaxLength(value, validator) {
            var error = '';
            if (value.length > validator.value)
                error = validator.message.replace('{value}', validator.value);
            return error;
        }
    })(ValidationService = Services.ValidationService || (Services.ValidationService = {}));
})(Services || (Services = {}));
var Services;
(function (Services) {
    'use strict';
    var ngApp = angular.module('ad.ui.services', []);
    ngApp.service('adModalService', Services.ModalService.Factory);
    ngApp.service('adSpinnerService', Services.SpinnerService.Factory);
    ngApp.service('adScrollService', Services.ScrollService.Factory);
    ngApp.service('RecursionHelper', Services.RecursionHelperService.Factory);
})(Services || (Services = {}));
var AD;
(function (AD) {
    var UI;
    (function (UI) {
        var Bootstrap;
        (function (Bootstrap) {
            var Table;
            (function (Table) {
                var Controller = (function () {
                    function Controller($scope, $filter, ngTableParams) {
                        this.$scope = $scope;
                        this.$filter = $filter;
                        this.ngTableParams = ngTableParams;
                        if ($scope.readonly == null) {
                            $scope.readonly = true;
                        }
                        var defaultSort;
                        $scope.fieldList.forEach(function (field) {
                            if (defaultSort == undefined) {
                                if (field.enableSort == true) {
                                    defaultSort = field.element;
                                }
                            }
                        });
                        $scope.tableObject = new ngTableParams({
                            page: $scope.page,
                            count: $scope.count
                        }, {
                            getData: function ($defer, params) {
                                var filteredData = $scope.filterData($scope.itemList);
                                var orderBy = params.orderBy();
                                if (orderBy.length == 0 && defaultSort != undefined) {
                                    orderBy.push("+" + defaultSort);
                                }
                                var orderedData = $scope.enableSort == true ? $filter('orderBy')(filteredData, orderBy) : filteredData;
                                if (orderedData != null) {
                                    params.total(orderedData.length);
                                }
                                else {
                                    params.total(0);
                                }
                                if (orderedData != null) {
                                    $defer.resolve(orderedData.slice((params.page() - 1) * params.count(), params.page() * params.count()));
                                }
                            }
                        });
                        $scope.tableObject.settings().$scope = $scope;
                        $scope.filterData = function (itemList) {
                            if ($scope.filterText == null || $scope.filterText.length == 0) {
                                return itemList;
                            }
                            var searchText = $scope.filterText.toLowerCase();
                            var rtn = [];
                            itemList.forEach(function (item) {
                                var added = false;
                                $scope.fieldList.forEach(function (field) {
                                    if (field.enableSearch == true && added == false) {
                                        if (item[field.element].toLowerCase().indexOf(searchText) != -1) {
                                            rtn.push(item);
                                            added = true;
                                        }
                                    }
                                });
                            });
                            return rtn;
                        };
                        $scope.$watchCollection('itemList', function () {
                            $scope.tableObject.reload();
                        });
                        $scope.doFilter = function (filterText) {
                            $scope.filterText = filterText;
                            $scope.tableObject.reload();
                        };
                        $scope.internalItemClicked = function (item, $event) {
                            if ($event.originalEvent.prevent != true) {
                                if ($scope.itemClicked != null) {
                                    $scope.itemClicked({ item: item, event: $event });
                                }
                            }
                        };
                        $scope.sortData = function (field, direction) {
                            if ($scope.enableSort == true && field.enableSort == true) {
                                this.tableObject.sorting(field.element, direction);
                            }
                        };
                        $scope.sortClassText = function (field) {
                            return $scope.enableSort == true && field.enableSort == true;
                        };
                        $scope.isReadonly = function () {
                            return $scope.readonly;
                        };
                    }
                    Controller.$inject = ['$scope', '$filter', 'ngTableParams'];
                    return Controller;
                })();
                Table.Controller = Controller;
            })(Table = Bootstrap.Table || (Bootstrap.Table = {}));
        })(Bootstrap = UI.Bootstrap || (UI.Bootstrap = {}));
    })(UI = AD.UI || (AD.UI = {}));
})(AD || (AD = {}));
var AD;
(function (AD) {
    var UI;
    (function (UI) {
        var Bootstrap;
        (function (Bootstrap) {
            var Table;
            (function (Table) {
                var Directives = (function () {
                    function Directives() {
                    }
                    Directives.Init = function (ngApp) {
                        ngApp.directive('adTable', function ($timeout) {
                            return {
                                templateUrl: 'table/partials/table.html',
                                replace: true,
                                restrict: 'E',
                                controller: 'tableController',
                                scope: {
                                    toolbar: '=?',
                                    searchbar: '=?',
                                    itemClicked: '&?',
                                    actions: '=?',
                                    itemList: '=',
                                    fieldList: '=',
                                    enableSort: '=?',
                                    page: '=?',
                                    count: '=?',
                                    paging: '=?',
                                    readonly: "=?"
                                }
                            };
                        });
                    };
                    return Directives;
                })();
                Table.Directives = Directives;
            })(Table = Bootstrap.Table || (Bootstrap.Table = {}));
        })(Bootstrap = UI.Bootstrap || (UI.Bootstrap = {}));
    })(UI = AD.UI || (AD.UI = {}));
})(AD || (AD = {}));
var AD;
(function (AD) {
    var UI;
    (function (UI) {
        var Bootstrap;
        (function (Bootstrap) {
            var Table;
            (function (Table) {
                var app = angular.module('ad.ui.bootstrap.table', ['ad.ui.bootstrap.searchBox', 'ngTable']);
                app.controller('tableController', Table.Controller);
                Table.Directives.Init(app);
            })(Table = Bootstrap.Table || (Bootstrap.Table = {}));
        })(Bootstrap = UI.Bootstrap || (UI.Bootstrap = {}));
    })(UI = AD.UI || (AD.UI = {}));
})(AD || (AD = {}));
var VehicleQuotes;
(function (VehicleQuotes) {
    var QuoteController = (function () {
        function QuoteController($scope) {
            this.$scope = $scope;
            $scope.limit = $scope.displayLimit || $scope.quotes.length || 0;
            $scope.getQuoteCount = function (quotes) {
                if (quotes == undefined)
                    return 0;
                return quotes.length;
            };
            $scope.displayMoreQuotes = function () {
                $scope.limit = $scope.vehicle.quotes.length || 0;
            };
            $scope.displayLessQuotes = function () {
                $scope.limit = $scope.displayLimit || $scope.quotes.length || 0;
            };
        }
        QuoteController.$inject = ['$scope'];
        return QuoteController;
    })();
    VehicleQuotes.QuoteController = QuoteController;
})(VehicleQuotes || (VehicleQuotes = {}));
var VehicleQuotes;
(function (VehicleQuotes) {
    var QuoteDirective = (function () {
        function QuoteDirective() {
        }
        QuoteDirective.init = function () {
            var directive = {
                restrict: 'E',
                replace: true,
                templateUrl: 'vehicles-quotes/partials/quote.html',
                controller: 'quoteController'
            };
            return directive;
        };
        return QuoteDirective;
    })();
    VehicleQuotes.QuoteDirective = QuoteDirective;
})(VehicleQuotes || (VehicleQuotes = {}));
var VehicleQuotes;
(function (VehicleQuotes) {
    var VehicleDirective = (function () {
        function VehicleDirective() {
        }
        VehicleDirective.init = function () {
            var directive = {
                restrict: 'E',
                replace: true,
                templateUrl: 'vehicles-quotes/partials/vehicle.html'
            };
            return directive;
        };
        return VehicleDirective;
    })();
    VehicleQuotes.VehicleDirective = VehicleDirective;
})(VehicleQuotes || (VehicleQuotes = {}));
var VehicleQuotes;
(function (VehicleQuotes) {
    var VehicleQuotesDirective = (function () {
        function VehicleQuotesDirective() {
        }
        VehicleQuotesDirective.init = function () {
            var directive = {
                restrict: 'E',
                replace: true,
                templateUrl: 'vehicles-quotes/partials/vehiclesQuotes.html',
                scope: {
                    displayLimit: '=',
                    vehicles: '='
                }
            };
            return directive;
        };
        return VehicleQuotesDirective;
    })();
    VehicleQuotes.VehicleQuotesDirective = VehicleQuotesDirective;
})(VehicleQuotes || (VehicleQuotes = {}));
var VehicleQuotes;
(function (VehicleQuotes) {
    var ngApp = angular.module('ad.ui.bootstrap.vehicleQuotes', []);
    ngApp.controller('quoteController', VehicleQuotes.QuoteController);
    ngApp.directive('vehicleQuotes', VehicleQuotes.VehicleQuotesDirective.init);
    ngApp.directive('vehicle', VehicleQuotes.VehicleDirective.init);
    ngApp.directive('quote', VehicleQuotes.QuoteDirective.init);
})(VehicleQuotes || (VehicleQuotes = {}));
//# sourceMappingURL=ad.ui.bootstrap-ts.js.map
/**
 * Created by rroussea on 8/28/2014.
 */

angular.module('ad.ui.dialogService', ['ui.bootstrap'])
    /// A service to display modal dialogs
    /// based on http://angular-ui.github.io/bootstrap/#/modal
    .service('adDialogService', ['$modal', '$window', function ($modal, $window) {
        
        this.showDialog = function(options){
            return $modal.open({
                templateUrl: options.url,
                template: options.template,
                controller: options.controller,
                controllerAs: options.controllerAs,
                backdrop: 'static',
                resolve: options.resolve,
                size: options.size,
                windowClass: options.windowClass || 'default',
                animation: options.animation
            });
        };
        
        
        /// Options
        /// message: The message displayed within the dialog
        /// title: The title displayed in the dialog header
        /// buttons: [optional] An array of button labels. Default button is "OK"
        /// inputs: [optional] An array of input objects containing { label, value }
        /// templateUrl: [optional] An HTML template reference to include in the body of the dialog.
        /// showDismissButton: [optional] A boolean value indicating whether or not we show the close button in the header (X).
        this.showAlert = function(options){
            if(!options.buttons && !options.templateUrl){options.buttons = ['OK'];}
            if(!options.inputs){options.inputs = [];}
            if(typeof options.showDismissButton === 'undefined'){
                options.showDismissButton = false;
            }

            var rootContext=this; 

            return $modal.open({
                templateUrl: 'dialog/dialog.html',
                controller: ('modalController', ['$scope', '$modalInstance',
                    function($scope, $modalInstance) {
                        $scope.message = options.message;
                        $scope.title = options.title;
                        $scope.buttons = options.buttons;
                        $scope.inputs = options.inputs;
                        $scope.templateUrl = options.templateUrl;
                        $scope.showDismissButton = options.showDismissButton;
                        var context = $scope;
                        $scope.click = function (index) {
                            var response = {buttonIndex: index, inputs: context.inputs};
                            $modalInstance.close(response);
                        };
                        $scope.dismiss = function (){
                            $modalInstance.dismiss();
                        };
                    }
                ]),
                backdrop: 'static',
                size: options.size,
                resolve: {
                    message: function () {
                        return options.message;
                    },
                    title: function(){
                        return options.title;
                    },
                    buttons: function(){
                        return options.buttons;
                    },
                    inputs:function(){
                        return options.inputs;
                    }
                },
                windowClass: options.windowClass || 'default'
            });
        };
    }])
    .directive('adDialogShell',[function(){
        return {
            restrict: 'E',
            templateUrl: 'dialog/dialogShell.html',
            transclude: true,
            scope: {
                title: '@',
                buttons: '=',
                hasFooterContent: '=?'
            },
            link: function(scope, element, attrs, ctrl, transclude) {
                scope.close = function(){
                    dismiss(scope);
                };
                
                function dismiss(currentScope){
                    if(!currentScope.$dismiss){
                        dismiss(currentScope.$parent);
                    } else {
                        currentScope.$dismiss();
                    }
                }
                
                transclude(scope.$parent, function(clone, childScope){
                    var zoomLevel = $('body').css('zoom');
                    if(typeof zoomLevel === "string"){
                        zoomLevel = parseFloat(zoomLevel.replace('%',''));
                    }
                    if(zoomLevel > 10){
                        zoomLevel = zoomLevel / 100
                    }
                    console.log(zoomLevel);
                    if(!zoomLevel){
                        zoomLevel = 1;
                    }
                    
                    var modalBody = element.find('.modal-body');
                    var modalDialog = element.find('.modal-dialog');
                    
                    modalBody.append(clone);
                    modalBody.css('max-height', ($(window).height() / zoomLevel) - 180);
                    
                    // move footer-content to footer area.
                    if(scope.hasFooterContent){
                        var footerContent = modalBody.find('ad-dialog-footer');
                        var modalFooter = element.find('.modal-footer');
                        modalFooter.prepend(footerContent);
                        footerContent.wrap('<div class="footer-content"></div>');
                        footerContent.addClass('ad-dialog-footer');
                    }

                    // set height
                    $(window).on('resize', function(){
                        var zoomLevel = $('body').css('zoom');
                        modalBody.css('max-height', ($(window).height() / zoomLevel) - 180);
                    })
                });
            }
        };
    }]);


/**
 * Checklist-model
 * AngularJS directive for list of checkboxes
 * Source: http://vitalets.github.io/checklist-model/
 * With modifications to allow for dynamic property definitions (see line55)
 */

angular.module('checklist-model', [])
.directive('checklistModel', ['$parse', '$compile','$interpolate', function($parse, $compile, $interpolate) {
  // contains
  function contains(arr, item, comparator) {
    if (angular.isArray(arr)) {
      for (var i = arr.length; i--;) {
        if (comparator(arr[i], item)) {
          return true;
        }
      }
    }
    return false;
  }

  // add
  function add(arr, item, comparator) {
    arr = angular.isArray(arr) ? arr : [];
      if(!contains(arr, item, comparator)) {
          arr.push(item);
      }
    return arr;
  }  

  // remove
  function remove(arr, item, comparator) {
    if (angular.isArray(arr)) {
      for (var i = arr.length; i--;) {
        if (comparator(arr[i], item)) {
          arr.splice(i, 1);
          break;
        }
      }
    }
    return arr;
  }

  // http://stackoverflow.com/a/19228302/1458162
  function postLinkFn(scope, elem, attrs) {
    // compile with `ng-model` pointing to `checked`
    $compile(elem)(scope);

    // getter / setter for original model
    var getter = $parse(attrs.checklistModel);
    var setter = getter.assign;
    var checklistChange = $parse(attrs.checklistChange);

    // value added to list
    var checklistValue = attrs.checklistValue.startsWith('{{') ? $interpolate(attrs.checklistValue)(scope.$parent) : attrs.checklistValue;
    
    var value = $parse(checklistValue)(scope.$parent);
    // var value = $parse(attrs.checklistValue)(scope.$parent);


  var comparator = angular.equals;

  if (attrs.hasOwnProperty('checklistComparator')){
    comparator = $parse(attrs.checklistComparator)(scope.$parent);
  }

    // watch UI checked change
    scope.$watch('checked', function(newValue, oldValue) {
      if (newValue === oldValue) { 
        return;
      } 
      var current = getter(scope.$parent);
      if (newValue === true) {
        setter(scope.$parent, add(current, value, comparator));
      } else {
        setter(scope.$parent, remove(current, value, comparator));
      }

      if (checklistChange) {
        checklistChange(scope);
      }
    });
    
    // declare one function to be used for both $watch functions
    function setChecked(newArr, oldArr) {
        scope.checked = contains(newArr, value, comparator);
    }

    // watch original model change
    // use the faster $watchCollection method if it's available
    if (angular.isFunction(scope.$parent.$watchCollection)) {
        scope.$parent.$watchCollection(attrs.checklistModel, setChecked);
    } else {
        scope.$parent.$watch(attrs.checklistModel, setChecked, true);
    }
  }

  return {
    restrict: 'A',
    priority: 1000,
    terminal: true,
    scope: true,
    compile: function(tElement, tAttrs) {
      if (tElement[0].tagName !== 'INPUT' || tAttrs.type !== 'checkbox') {
        throw 'checklist-model should be applied to `input[type="checkbox"]`.';
      }

      if (!tAttrs.checklistValue) {
        throw 'You should provide `checklist-value`.';
      }

      // exclude recursion
      tElement.removeAttr('checklist-model');
      
      // local scope var storing individual checkbox model
      tElement.attr('ng-model', 'checked');

      return postLinkFn;
    }
  };
}]);
(function (root, factory) {
  if (typeof define === 'function' && define.amd) {
    define(['angular'], factory);
  } else if (typeof exports === 'object') {
    factory(require('angular'));
  } else {
    factory(root.angular);
  }
}(this, function (angular) {
  'use strict';

  angular.module('angular-multiple-transclusion', [])

  // .directive('ngMultiTransclude', function(){
  .directive('adMultiTransclude', function(){
    return {
      priority: 1, // run after ng-transclude

      link: function(scope, elem, attrs, ctrl, transclude) {

        // if (attrs.transcludeFrom) {
        //   var children = elem.children();
        //   for (var i = 0; i < children.length; i++) {
        //     var child = angular.element(children[i]);
        //     var transcludeAs = child.attr('transclude-to');
        //     if (transcludeAs !== attrs.transcludeFrom) {
        //       child.remove();
        //     }
        //   }
        // }
        
        transclude(function(clone){
          angular.forEach(clone, function(cloneEl){
            if(cloneEl.attributes && cloneEl.attributes["transclude-to"]){
              var destinationId = cloneEl.attributes["transclude-to"].value;
              var destination = angular.element(elem[0].querySelector('[transclude-from="' + destinationId + '"]'));
              if(destination.length){
                destination.append(cloneEl);
              } else {
                cloneEl.remove();
              }
            }
          })
        })

      }
    };
  });

}));
/**
 * Created by robbie on 4/23/2014.
 */

angular.module('ad.ui.bootstrap.searchBox', [])
    .constant('searchBoxConfig', {

    })
    .controller('searchBoxController',['searchBoxConfig', function(searchBoxConfig){

    }])
    .directive('searchBox', ['$timeout',function($timeout){
        return {
            restrict: 'E',
            templateUrl: 'search-box/search-box.html',
//            require: ['ngModel'],
            scope: {
                onSearch: '&',
                onClear: '&',
                placeholder: '@',
                searchTerm: '=?',
                control: '=',
                timeDelay: '=?',
                numberOfCharacterDelay: '=?',
                isFocused: '=?',
                autoSearch: '=?',
                onFocus: '&?',
                onBlur: '&?'

//                clearSearchTextButton: '=?',
//                lostFocus: '=?',
//                setFocus: '=?'
            },
            link: function(scope, element, attrs) {
                var self = this;
                var inputField = element.find('input');
                if(!attrs.isFocused){
                    inputField.focus();
                }
                inputField.bind('keyup', function(event){
                    if ( event.which == 13 ) {
                        scope.onSearch({searchTerm: scope.searchTerm});
                        inputField.blur();
                        scope.$apply();
                    } else {
                        scope.change();
                    }
                });
                inputField.bind('blur', function(event){
                    if(scope.onBlur){
                        scope.onBlur();
                    }
                });
                inputField.bind('focus', function(event){
                    if(scope.onFocus){
                        scope.onFocus();
                    }
                });

                if (scope.autoSearch == undefined) { scope.autoSearch = true; }
                if(!scope.searchTerm) { scope.searchTerm = ""; }
                if(!scope.timeDelay) { scope.timeDelay = 10; }
                if(!scope.numberOfCharacterDelay){ scope.numberOfCharacterDelay = 0; }
                if(!scope.isFocused){scope.isFocused = false;}

                scope.clearSearchText = function() {
                    scope.searchTerm =null;
                };
                scope.$watch('searchTerm', function(){
                    scope.change();
                });
                scope.keypressCallback = function($event) {
                    $event.target.blur();
                };
                scope.$watch('isFocused', function(value){
//                    console.log(value);
                    if(value === true){
                        element.find('input').focus();
                        self.isFocused = false;
                    }

                });
                scope.search = function(searchTerm){
                    if(searchTerm && searchTerm.length > 0){
                        scope.onSearch({searchTerm: scope.searchTerm});
                    }
                };

                scope.change = function(){
                    if (scope.autoSearch) {
                        if((scope.numberOfCharacterDelay - 1) < scope.searchTerm.length) {
                            // if there is a timeout, cancel it.
                            if (scope.waitForTyping) {
                                $timeout.cancel(scope.waitForTyping);
                            }
                            if (scope.onSearch) {
                                scope.waitForTyping = $timeout(function(){scope.clearSearchTextButton=true; scope.onSearch({searchTerm: scope.searchTerm})}, scope.timeDelay);
                            }
                        } else {
                            if(scope.onClear){
                                $timeout(scope.onClear, 10);
                            }
                        }
                    }
                }

            }
        }
    }])
/**
 * Created by robbie on 8/17/2014.
 * This is a polymer entry from (http://polymer.github.io/PointerEvents/pointerevents.js)
 */

// Copyright (c) 2013 The Polymer Authors. All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are
// met:
//
//    * Redistributions of source code must retain the above copyright
// notice, this list of conditions and the following disclaimer.
//    * Redistributions in binary form must reproduce the above
// copyright notice, this list of conditions and the following disclaimer
// in the documentation and/or other materials provided with the
// distribution.
//    * Neither the name of Google Inc. nor the names of its
// contributors may be used to endorse or promote products derived from
// this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
    "undefined"==typeof WeakMap&&!function(){var a=Object.defineProperty,b=Date.now()%1e9,c=function(){this.name="__st"+(1e9*Math.random()>>>0)+(b++ +"__")};c.prototype={set:function(b,c){var d=b[this.name];d&&d[0]===b?d[1]=c:a(b,this.name,{value:[b,c],writable:!0})},get:function(a){var b;return(b=a[this.name])&&b[0]===a?b[1]:void 0},"delete":function(a){this.set(a,void 0)}},window.WeakMap=c}(),function(a){a=a||{},a.external=a.external||{};var b={shadow:function(a){return a?a.shadowRoot||a.webkitShadowRoot:void 0},canTarget:function(a){return a&&Boolean(a.elementFromPoint)},targetingShadow:function(a){var b=this.shadow(a);return this.canTarget(b)?b:void 0},olderShadow:function(a){var b=a.olderShadowRoot;if(!b){var c=a.querySelector("shadow");c&&(b=c.olderShadowRoot)}return b},allShadows:function(a){for(var b=[],c=this.shadow(a);c;)b.push(c),c=this.olderShadow(c);return b},searchRoot:function(a,b,c){if(a){var d,e,f=a.elementFromPoint(b,c);for(e=this.targetingShadow(f);e;){if(d=e.elementFromPoint(b,c)){var g=this.targetingShadow(d);return this.searchRoot(g,b,c)||d}e=this.olderShadow(e)}return f}},owner:function(a){for(var b=a;b.parentNode;)b=b.parentNode;return b},findTarget:function(a){var b=a.clientX,c=a.clientY,d=this.owner(a.target);return d.elementFromPoint(b,c)||(d=document),this.searchRoot(d,b,c)}};a.targetFinding=b,a.findTarget=b.findTarget.bind(b),window.PointerEventsPolyfill=a}(window.PointerEventsPolyfill),function(){function a(a){return"body ^^ "+b(a)}function b(a){return'[touch-action="'+a+'"]'}function c(a){return"{ -ms-touch-action: "+a+"; touch-action: "+a+"; touch-action-delay: none; }"}var d=["none","auto","pan-x","pan-y",{rule:"pan-x pan-y",selectors:["pan-x pan-y","pan-y pan-x"]}],e="";d.forEach(function(d){String(d)===d?(e+=b(d)+c(d)+"\n",e+=a(d)+c(d)+"\n"):(e+=d.selectors.map(b)+c(d.rule)+"\n",e+=d.selectors.map(a)+c(d.rule)+"\n")});var f=document.createElement("style");f.textContent=e,document.head.appendChild(f)}(),function(a){function b(a,e){e=e||{};var f=e.buttons;if(void 0===f)switch(e.which){case 1:f=1;break;case 2:f=4;break;case 3:f=2;break;default:f=0}var i;if(c)i=new MouseEvent(a,e);else{i=document.createEvent("MouseEvent");for(var j,k={},l=0;l<g.length;l++)j=g[l],k[j]=e[j]||h[l];i.initMouseEvent(a,k.bubbles,k.cancelable,k.view,k.detail,k.screenX,k.screenY,k.clientX,k.clientY,k.ctrlKey,k.altKey,k.shiftKey,k.metaKey,k.button,k.relatedTarget)}i.__proto__=b.prototype,d||Object.defineProperty(i,"buttons",{get:function(){return f},enumerable:!0});var m=0;return m=e.pressure?e.pressure:f?.5:0,Object.defineProperties(i,{pointerId:{value:e.pointerId||0,enumerable:!0},width:{value:e.width||0,enumerable:!0},height:{value:e.height||0,enumerable:!0},pressure:{value:m,enumerable:!0},tiltX:{value:e.tiltX||0,enumerable:!0},tiltY:{value:e.tiltY||0,enumerable:!0},pointerType:{value:e.pointerType||"",enumerable:!0},hwTimestamp:{value:e.hwTimestamp||0,enumerable:!0},isPrimary:{value:e.isPrimary||!1,enumerable:!0}}),i}var c=!1,d=!1;try{var e=new MouseEvent("click",{buttons:1});c=!0,d=1===e.buttons}catch(f){}var g=["bubbles","cancelable","view","detail","screenX","screenY","clientX","clientY","ctrlKey","altKey","shiftKey","metaKey","button","relatedTarget"],h=[!1,!1,null,null,0,0,0,0,!1,!1,!1,!1,0,null];b.prototype=Object.create(MouseEvent.prototype),a.PointerEvent||(a.PointerEvent=b)}(window),function(a){function b(){if(c){var a=new Map;return a.pointers=d,a}this.keys=[],this.values=[]}var c=window.Map&&window.Map.prototype.forEach,d=function(){return this.size};b.prototype={set:function(a,b){var c=this.keys.indexOf(a);c>-1?this.values[c]=b:(this.keys.push(a),this.values.push(b))},has:function(a){return this.keys.indexOf(a)>-1},"delete":function(a){var b=this.keys.indexOf(a);b>-1&&(this.keys.splice(b,1),this.values.splice(b,1))},get:function(a){var b=this.keys.indexOf(a);return this.values[b]},clear:function(){this.keys.length=0,this.values.length=0},forEach:function(a,b){this.values.forEach(function(c,d){a.call(b,c,this.keys[d],this)},this)},pointers:function(){return this.keys.length}},a.PointerMap=b}(window.PointerEventsPolyfill),function(a){var b=["bubbles","cancelable","view","detail","screenX","screenY","clientX","clientY","ctrlKey","altKey","shiftKey","metaKey","button","relatedTarget","buttons","pointerId","width","height","pressure","tiltX","tiltY","pointerType","hwTimestamp","isPrimary","type","target","currentTarget","which"],c=[!1,!1,null,null,0,0,0,0,!1,!1,!1,!1,0,null,void 0,0,0,0,0,0,0,"",0,!1,"",null,null,0],d={targets:new WeakMap,handledEvents:new WeakMap,pointermap:new a.PointerMap,eventMap:{},eventSources:{},eventSourceList:[],registerSource:function(a,b){var c=b,d=c.events;d&&(d.forEach(function(a){c[a]&&(this.eventMap[a]=c[a].bind(c))},this),this.eventSources[a]=c,this.eventSourceList.push(c))},register:function(a){for(var b,c=this.eventSourceList.length,d=0;c>d&&(b=this.eventSourceList[d]);d++)b.register.call(b,a)},unregister:function(a){for(var b,c=this.eventSourceList.length,d=0;c>d&&(b=this.eventSourceList[d]);d++)b.unregister.call(b,a)},contains:a.external.contains||function(a,b){return a.contains(b)},down:function(a){this.fireEvent("pointerdown",a)},move:function(a){this.fireEvent("pointermove",a)},up:function(a){this.fireEvent("pointerup",a)},enter:function(a){a.bubbles=!1,this.fireEvent("pointerenter",a)},leave:function(a){a.bubbles=!1,this.fireEvent("pointerleave",a)},over:function(a){a.bubbles=!0,this.fireEvent("pointerover",a)},out:function(a){a.bubbles=!0,this.fireEvent("pointerout",a)},cancel:function(a){this.fireEvent("pointercancel",a)},leaveOut:function(a){this.contains(a.target,a.relatedTarget)||this.leave(a),this.out(a)},enterOver:function(a){this.contains(a.target,a.relatedTarget)||this.enter(a),this.over(a)},eventHandler:function(a){if(!this.handledEvents.get(a)){var b=a.type,c=this.eventMap&&this.eventMap[b];c&&c(a),this.handledEvents.set(a,!0)}},listen:function(a,b){b.forEach(function(b){this.addEvent(a,b)},this)},unlisten:function(a,b){b.forEach(function(b){this.removeEvent(a,b)},this)},addEvent:a.external.addEvent||function(a,b){a.addEventListener(b,this.boundHandler)},removeEvent:a.external.removeEvent||function(a,b){a.removeEventListener(b,this.boundHandler)},makeEvent:function(a,b){this.captureInfo&&(b.relatedTarget=null);var c=new PointerEvent(a,b);return b.preventDefault&&(c.preventDefault=b.preventDefault),this.targets.set(c,this.targets.get(b)||b.target),c},fireEvent:function(a,b){var c=this.makeEvent(a,b);return this.dispatchEvent(c)},cloneEvent:function(a){for(var d,e={},f=0;f<b.length;f++)d=b[f],e[d]=a[d]||c[f];return a.preventDefault&&(e.preventDefault=function(){a.preventDefault()}),e},getTarget:function(a){return this.captureInfo&&this.captureInfo.id===a.pointerId?this.captureInfo.target:this.targets.get(a)},setCapture:function(a,b){this.captureInfo&&this.releaseCapture(this.captureInfo.id),this.captureInfo={id:a,target:b};var c=new PointerEvent("gotpointercapture",{bubbles:!0});this.implicitRelease=this.releaseCapture.bind(this,a),document.addEventListener("pointerup",this.implicitRelease),document.addEventListener("pointercancel",this.implicitRelease),this.targets.set(c,b),this.asyncDispatchEvent(c)},releaseCapture:function(a){if(this.captureInfo&&this.captureInfo.id===a){var b=new PointerEvent("lostpointercapture",{bubbles:!0}),c=this.captureInfo.target;this.captureInfo=null,document.removeEventListener("pointerup",this.implicitRelease),document.removeEventListener("pointercancel",this.implicitRelease),this.targets.set(b,c),this.asyncDispatchEvent(b)}},dispatchEvent:a.external.dispatchEvent||function(a){var b=this.getTarget(a);return b?b.dispatchEvent(a):void 0},asyncDispatchEvent:function(a){setTimeout(this.dispatchEvent.bind(this,a),0)}};d.boundHandler=d.eventHandler.bind(d),a.dispatcher=d,a.register=d.register.bind(d),a.unregister=d.unregister.bind(d)}(window.PointerEventsPolyfill),function(a){function b(a,b,c,d){this.addCallback=a.bind(d),this.removeCallback=b.bind(d),this.changedCallback=c.bind(d),g&&(this.observer=new g(this.mutationWatcher.bind(this)))}var c=Array.prototype.forEach.call.bind(Array.prototype.forEach),d=Array.prototype.map.call.bind(Array.prototype.map),e=Array.prototype.slice.call.bind(Array.prototype.slice),f=Array.prototype.filter.call.bind(Array.prototype.filter),g=window.MutationObserver||window.WebKitMutationObserver,h="[touch-action]",i={subtree:!0,childList:!0,attributes:!0,attributeOldValue:!0,attributeFilter:["touch-action"]};b.prototype={watchSubtree:function(b){a.targetFinding.canTarget(b)&&this.observer.observe(b,i)},enableOnSubtree:function(a){this.watchSubtree(a),a===document&&"complete"!==document.readyState?this.installOnLoad():this.installNewSubtree(a)},installNewSubtree:function(a){c(this.findElements(a),this.addElement,this)},findElements:function(a){return a.querySelectorAll?a.querySelectorAll(h):[]},removeElement:function(a){this.removeCallback(a)},addElement:function(a){this.addCallback(a)},elementChanged:function(a,b){this.changedCallback(a,b)},concatLists:function(a,b){return a.concat(e(b))},installOnLoad:function(){document.addEventListener("DOMContentLoaded",this.installNewSubtree.bind(this,document))},isElement:function(a){return a.nodeType===Node.ELEMENT_NODE},flattenMutationTree:function(a){var b=d(a,this.findElements,this);return b.push(f(a,this.isElement)),b.reduce(this.concatLists,[])},mutationWatcher:function(a){a.forEach(this.mutationHandler,this)},mutationHandler:function(a){if("childList"===a.type){var b=this.flattenMutationTree(a.addedNodes);b.forEach(this.addElement,this);var c=this.flattenMutationTree(a.removedNodes);c.forEach(this.removeElement,this)}else"attributes"===a.type&&this.elementChanged(a.target,a.oldValue)}},g||(b.prototype.watchSubtree=function(){console.warn("PointerEventsPolyfill: MutationObservers not found, touch-action will not be dynamically detected")}),a.Installer=b}(window.PointerEventsPolyfill),function(a){var b=a.dispatcher,c=b.pointermap,d=25,e={POINTER_ID:1,POINTER_TYPE:"mouse",events:["mousedown","mousemove","mouseup","mouseover","mouseout"],register:function(a){b.listen(a,this.events)},unregister:function(a){b.unlisten(a,this.events)},lastTouches:[],isEventSimulatedFromTouch:function(a){for(var b,c=this.lastTouches,e=a.clientX,f=a.clientY,g=0,h=c.length;h>g&&(b=c[g]);g++){var i=Math.abs(e-b.x),j=Math.abs(f-b.y);if(d>=i&&d>=j)return!0}},prepareEvent:function(a){var c=b.cloneEvent(a),d=c.preventDefault;return c.preventDefault=function(){a.preventDefault(),d()},c.pointerId=this.POINTER_ID,c.isPrimary=!0,c.pointerType=this.POINTER_TYPE,c},mousedown:function(a){if(!this.isEventSimulatedFromTouch(a)){var d=c.has(this.POINTER_ID);d&&this.cancel(a);var e=this.prepareEvent(a);c.set(this.POINTER_ID,a),b.down(e)}},mousemove:function(a){if(!this.isEventSimulatedFromTouch(a)){var c=this.prepareEvent(a);b.move(c)}},mouseup:function(a){if(!this.isEventSimulatedFromTouch(a)){var d=c.get(this.POINTER_ID);if(d&&d.button===a.button){var e=this.prepareEvent(a);b.up(e),this.cleanupMouse()}}},mouseover:function(a){if(!this.isEventSimulatedFromTouch(a)){var c=this.prepareEvent(a);b.enterOver(c)}},mouseout:function(a){if(!this.isEventSimulatedFromTouch(a)){var c=this.prepareEvent(a);b.leaveOut(c)}},cancel:function(a){var c=this.prepareEvent(a);b.cancel(c),this.cleanupMouse()},cleanupMouse:function(){c["delete"](this.POINTER_ID)}};a.mouseEvents=e}(window.PointerEventsPolyfill),function(a){var b,c=a.dispatcher,d=a.findTarget,e=a.targetFinding.allShadows.bind(a.targetFinding),f=c.pointermap,g=Array.prototype.map.call.bind(Array.prototype.map),h=2500,i=200,j="touch-action",k=!1,l={scrollType:new WeakMap,events:["touchstart","touchmove","touchend","touchcancel"],register:function(a){k?c.listen(a,this.events):b.enableOnSubtree(a)},unregister:function(a){k&&c.unlisten(a,this.events)},elementAdded:function(a){var b=a.getAttribute(j),d=this.touchActionToScrollType(b);d&&(this.scrollType.set(a,d),c.listen(a,this.events),e(a).forEach(function(a){this.scrollType.set(a,d),c.listen(a,this.events)},this))},elementRemoved:function(a){this.scrollType["delete"](a),c.unlisten(a,this.events),e(a).forEach(function(a){this.scrollType["delete"](a),c.unlisten(a,this.events)},this)},elementChanged:function(a,b){var c=a.getAttribute(j),d=this.touchActionToScrollType(c),f=this.touchActionToScrollType(b);d&&f?(this.scrollType.set(a,d),e(a).forEach(function(a){this.scrollType.set(a,d)},this)):f?this.elementRemoved(a):d&&this.elementAdded(a)},scrollTypes:{EMITTER:"none",XSCROLLER:"pan-x",YSCROLLER:"pan-y",SCROLLER:/^(?:pan-x pan-y)|(?:pan-y pan-x)|auto$/},touchActionToScrollType:function(a){var b=a,c=this.scrollTypes;return"none"===b?"none":b===c.XSCROLLER?"X":b===c.YSCROLLER?"Y":c.SCROLLER.exec(b)?"XY":void 0},POINTER_TYPE:"touch",firstTouch:null,isPrimaryTouch:function(a){return this.firstTouch===a.identifier},setPrimaryTouch:function(a){(0===f.pointers()||1===f.pointers()&&f.has(1))&&(this.firstTouch=a.identifier,this.firstXY={X:a.clientX,Y:a.clientY},this.scrolling=!1,this.cancelResetClickCount())},removePrimaryPointer:function(a){a.isPrimary&&(this.firstTouch=null,this.firstXY=null,this.resetClickCount())},clickCount:0,resetId:null,resetClickCount:function(){var a=function(){this.clickCount=0,this.resetId=null}.bind(this);this.resetId=setTimeout(a,i)},cancelResetClickCount:function(){this.resetId&&clearTimeout(this.resetId)},touchToPointer:function(a){var b=c.cloneEvent(a);return b.pointerId=a.identifier+2,b.target=d(b),b.bubbles=!0,b.cancelable=!0,b.detail=this.clickCount,b.button=0,b.buttons=1,b.width=a.webkitRadiusX||a.radiusX||0,b.height=a.webkitRadiusY||a.radiusY||0,b.pressure=a.webkitForce||a.force||.5,b.isPrimary=this.isPrimaryTouch(a),b.pointerType=this.POINTER_TYPE,b},processTouches:function(a,b){var c=a.changedTouches,d=g(c,this.touchToPointer,this);d.forEach(function(b){b.preventDefault=function(){this.scrolling=!1,this.firstXY=null,a.preventDefault()}},this),d.forEach(b,this)},shouldScroll:function(a){if(this.firstXY){var b,c=this.scrollType.get(a.currentTarget);if("none"===c)b=!1;else if("XY"===c)b=!0;else{var d=a.changedTouches[0],e=c,f="Y"===c?"X":"Y",g=Math.abs(d["client"+e]-this.firstXY[e]),h=Math.abs(d["client"+f]-this.firstXY[f]);b=g>=h}return this.firstXY=null,b}},findTouch:function(a,b){for(var c,d=0,e=a.length;e>d&&(c=a[d]);d++)if(c.identifier===b)return!0},vacuumTouches:function(a){var b=a.touches;if(f.pointers()>=b.length){var c=[];f.forEach(function(a,d){if(1!==d&&!this.findTouch(b,d-2)){var e=a.out;c.push(this.touchToPointer(e))}},this),c.forEach(this.cancelOut,this)}},touchstart:function(a){this.vacuumTouches(a),this.setPrimaryTouch(a.changedTouches[0]),this.dedupSynthMouse(a),this.scrolling||(this.clickCount++,this.processTouches(a,this.overDown))},overDown:function(a){f.set(a.pointerId,{target:a.target,out:a,outTarget:a.target});c.over(a),c.down(a)},touchmove:function(a){this.scrolling||(this.shouldScroll(a)?(this.scrolling=!0,this.touchcancel(a)):(a.preventDefault(),this.processTouches(a,this.moveOverOut)))},moveOverOut:function(a){var b=a,d=f.get(b.pointerId);if(d){var e=d.out,g=d.outTarget;c.move(b),e&&g!==b.target&&(e.relatedTarget=b.target,b.relatedTarget=g,e.target=g,b.target?(c.leaveOut(e),c.enterOver(b)):(b.target=g,b.relatedTarget=null,this.cancelOut(b))),d.out=b,d.outTarget=b.target}},touchend:function(a){this.dedupSynthMouse(a),this.processTouches(a,this.upOut)},upOut:function(a){this.scrolling||(c.up(a),c.out(a)),this.cleanUpPointer(a)},touchcancel:function(a){this.processTouches(a,this.cancelOut)},cancelOut:function(a){c.cancel(a),c.out(a),this.cleanUpPointer(a)},cleanUpPointer:function(a){f["delete"](a.pointerId),this.removePrimaryPointer(a)},dedupSynthMouse:function(b){var c=a.mouseEvents.lastTouches,d=b.changedTouches[0];if(this.isPrimaryTouch(d)){var e={x:d.clientX,y:d.clientY};c.push(e);var f=function(a,b){var c=a.indexOf(b);c>-1&&a.splice(c,1)}.bind(null,c,e);setTimeout(f,h)}}};k||(b=new a.Installer(l.elementAdded,l.elementRemoved,l.elementChanged,l)),a.touchEvents=l}(window.PointerEventsPolyfill),function(a){var b=a.dispatcher,c=b.pointermap,d=window.MSPointerEvent&&"number"==typeof window.MSPointerEvent.MSPOINTER_TYPE_MOUSE,e={events:["MSPointerDown","MSPointerMove","MSPointerUp","MSPointerOut","MSPointerOver","MSPointerCancel","MSGotPointerCapture","MSLostPointerCapture"],register:function(a){b.listen(a,this.events)},unregister:function(a){b.unlisten(a,this.events)},POINTER_TYPES:["","unavailable","touch","pen","mouse"],prepareEvent:function(a){var c=a;return d&&(c=b.cloneEvent(a),c.pointerType=this.POINTER_TYPES[a.pointerType]),c},cleanup:function(a){c["delete"](a)},MSPointerDown:function(a){c.set(a.pointerId,a);var d=this.prepareEvent(a);b.down(d)},MSPointerMove:function(a){var c=this.prepareEvent(a);b.move(c)},MSPointerUp:function(a){var c=this.prepareEvent(a);b.up(c),this.cleanup(a.pointerId)},MSPointerOut:function(a){var c=this.prepareEvent(a);b.leaveOut(c)},MSPointerOver:function(a){var c=this.prepareEvent(a);b.enterOver(c)},MSPointerCancel:function(a){var c=this.prepareEvent(a);b.cancel(c),this.cleanup(a.pointerId)},MSLostPointerCapture:function(a){var c=b.makeEvent("lostpointercapture",a);b.dispatchEvent(c)},MSGotPointerCapture:function(a){var c=b.makeEvent("gotpointercapture",a);b.dispatchEvent(c)}};a.msEvents=e}(window.PointerEventsPolyfill),function(a){var b=a.dispatcher;if(void 0===window.navigator.pointerEnabled){if(Object.defineProperty(window.navigator,"pointerEnabled",{value:!0,enumerable:!0}),window.navigator.msPointerEnabled){var c=window.navigator.msMaxTouchPoints;Object.defineProperty(window.navigator,"maxTouchPoints",{value:c,enumerable:!0}),b.registerSource("ms",a.msEvents)}else b.registerSource("mouse",a.mouseEvents),void 0!==window.ontouchstart&&b.registerSource("touch",a.touchEvents);b.register(document)}}(window.PointerEventsPolyfill),function(a){function b(a){if(!e.pointermap.has(a))throw new Error("InvalidPointerId")}var c,d,e=a.dispatcher,f=window.navigator;f.msPointerEnabled?(c=function(a){b(a),this.msSetPointerCapture(a)},d=function(a){b(a),this.msReleasePointerCapture(a)}):(c=function(a){b(a),e.setCapture(a,this)},d=function(a){b(a),e.releaseCapture(a,this)}),window.Element&&!Element.prototype.setPointerCapture&&Object.defineProperties(Element.prototype,{setPointerCapture:{value:c},releasePointerCapture:{value:d}})}(window.PointerEventsPolyfill);

/**
 * Created by robbie on 8/9/2014.
 */
angular.module('ad.ui.zoomContainer', [])
    .service('zoomContainerService', function () {
        this.prepEvent = function(e, pageZoomLevel, $object){
            // adjust coordinates for zoom level
            if(e.pageX) {
                e._pageX = e.pageX / pageZoomLevel;
                e._pageY = e.pageY / pageZoomLevel;
            }
            if(e.originalEvent && e.originalEvent.touches){
                var touches = e.originalEvent.touches;
                for(var i=0; i < touches.length; i++){
                    var touch = touches[i];
                    touch._pageX = touch.pageX / pageZoomLevel;
                    touch._pageY = touch.pageY / pageZoomLevel;
                }
            }
            e.$object = $object;
            e.pageZoomLevel = pageZoomLevel;
            if(e.stopPropagation) e.stopPropagation();
            if(e.preventDefault) e.preventDefault();
            e.cancelBubble=true;
            e.returnValue=false;
            return false;
        };
        this.calculateLine = function(x1, y1, x2, y2){
            var sideX = Math.abs(Math.pow(x1-x2,2));
            var sideY = Math.abs(Math.pow(y1-y2,2));
            return Math.sqrt(sideX + sideY)
        };
        this.mapMouseToTouch = function(e){
            var currentPos = this.getObjectCoordinates(e);
            return {
                start: {
                    _pageX: e._pageX ,
                    _pageY: e._pageY,
                    translateX: currentPos.translateX,
                    translateY: currentPos.translateY,
                    scale: currentPos.scale,
                    object: currentPos.pageOffset,
                    timestamp: new Date()
                },
                $object: e.$object
            }
        };
        this.getObjectCoordinates = function(e){
            var objectOffset = e.$object.offset();

            var matrix = e.$object.css("transform");
            if(!matrix){
                matrix = e.$object.css("-webkit-transform");
            }
            matrix = matrix ? matrix.substr(7, matrix.length - 8).split(', ') : "";
            var hasTransform = matrix.length > 1;
            return {
                $object : e.$object,
                translateX : hasTransform ? parseFloat(matrix[4]) : 0,
                translateY : hasTransform ? parseFloat(matrix[5]) : 0,
                scale : hasTransform ? parseFloat(matrix[0]) : 1,
                pageOffset: {
                    x: objectOffset.left,
                    y: objectOffset.top,
                    width: e.$object.width(),
                    height: e.$object.height()
                }
            };
        };
        this.zoom = function(touch, scale){
            // calculate transformation
            var v = {};
            v.objectPointX = touch.start._pageX - touch.start.object.x;
            v.objectPointY = touch.start._pageY - touch.start.object.y;
            v.offsetX = v.objectPointX - (v.objectPointX * scale) + (touch.deltaX ? touch.deltaX : 0);
            v.offsetY = v.objectPointY - (v.objectPointY * scale) + (touch.deltaY ? touch.deltaY : 0);
            v.translateX = touch.start.translateX;
            v.translateY = touch.start.translateY;

            var newScale = touch.start.scale * scale;

            this.transform(touch.$object, touch.start.translateX + v.offsetX, touch.start.translateY + v.offsetY, newScale);
            return newScale;
        };
        this.transform = function($object, translateX, translateY, scale, rotate){
            rotate = rotate || 0;
            var rect = $object[0];
            var transformValue = "translate("+ translateX +"px,"+ translateY + "px) " + "scale("+scale+","+scale+") rotate(" + rotate + "deg)";
            rect.style.transform = transformValue;
            rect.style.oTransform = transformValue;
            rect.style.msTransform = transformValue;
            rect.style.mozTransform = transformValue;
            rect.style.webkitTransform = transformValue;
        };
        this.calculateZoomLevel = function(elem){
            var pageZoomLevel = 1;
            var parents = elem.parents();
            parents.each(function(){
                var parentZoom = $(this).css('zoom') || 1;
                if(isNaN(parentZoom)) {
                    parentZoom = parentZoom === "normal" ? 1 : (parseFloat(parentZoom.replace("%")) / 100);
                }
                pageZoomLevel = pageZoomLevel * (parentZoom ? parentZoom : 0);
            });
            return pageZoomLevel;
        };

        this.reset = function($object, scale){
            $object.addClass('transition');
            this.transform($object, 0, 0, scale);
        };
    })
    .controller('zoomContainerController', function(){

    })
    .directive('zoomContainer', ['zoomContainerService',function(zcSvc){
        return {
            restrict: 'EA',
            transclude: true,
            replace: true,
            scope: {
                useCtrlForMouseValue: '@useCtrlForMouse',
                zoomToFitValue: '@zoomToFit',
                printFullScreen: '=?'
            },
            templateUrl: 'zoom-container/zoom-container.html',
            link: function(scope, element, attrs) {
                angular.element('body').addClass('full-screen-image');
                if((typeof scope.useCtrlForMouseValue === 'undefined') || scope.useCtrlForMouseValue.toUpperCase() ==='FALSE' || scope.useCtrlForMouseValue == '0'){
                    scope.useCtrlForMouse = false;
                } else {
                    scope.useCtrlForMouse = true;
                }
                if((typeof scope.zoomToFitValue === 'undefined') || scope.zoomToFitValue.toUpperCase() ==='TRUE' || scope.zoomToFitValue == '1'){
                    scope.zoomToFit = true;
                } else {
                    scope.zoomToFit = false;
                }

                var initialScale = 1;
                var defaultZoom = 2;
                var $object = element.find('.mainObject');
                var $container = $object.parent();
                var pageZoomLevel = zcSvc.calculateZoomLevel($object);
                scope.rotation = 0;
                
                var getTouchEvent = function(){
                    var currentPos = zcSvc.getObjectCoordinates({$object: $container});
                    
                    var e = {
                        pageX: currentPos.pageOffset.x + currentPos.pageOffset.width / 2,
                        pageY:  currentPos.pageOffset.y + currentPos.pageOffset.height / 2
                    }
                    
                    // var currentPos = zcSvc.getObjectCoordinates({$object: $object});
                    
                    // var e = {
                    //     pageX: currentPos.pageOffset.x + (currentPos.pageOffset.width * currentPos.scale) / 2,
                    //     pageY:  currentPos.pageOffset.y + (currentPos.pageOffset.height * currentPos.scale) / 2,
                    // }
                    zcSvc.prepEvent(e, pageZoomLevel, $object);
                    $object.addClass('transition');
                    var ev = zcSvc.mapMouseToTouch(e);
                    return ev;
                }
                
                scope.zoomIn = function(){
                    // mock center of page event
                    var ev = getTouchEvent();
                    zcSvc.zoom(ev, defaultZoom);
                }
                scope.zoomOut = function(){
                    var ev = getTouchEvent();
                    // zcSvc.zoom(ev, defaultZoom);
                    zcSvc.zoom(ev, Math.pow(defaultZoom, -1));
                }
                scope.fit = function(){
                    var containerWidth = $container.width() / pageZoomLevel;
                    var containerHeight = $container.height() / pageZoomLevel;
                    var newObjectWidth = $object.width() / pageZoomLevel;
                    var newObjectHeight = $object.height() / pageZoomLevel;
                    
                    var scaleWidth = containerWidth / newObjectWidth;
                    var scaleHeight = containerHeight / newObjectHeight;
                    
                    if(scaleWidth < scaleHeight){
                        var diffHeight = (containerHeight - newObjectHeight*scaleWidth) / 2;
                         zcSvc.transform($object,  0, diffHeight,  scaleWidth);
                    } else {
                        
                        var diffWidth = (containerWidth - newObjectWidth*scaleHeight) / 2;
                        zcSvc.transform($object,  diffWidth, 0,  scaleHeight);
                    }
                    
                   
                }
                
                scope.rotate = function(degrees){
                    scope.rotation = scope.rotation + degrees;
                    while(scope.rotation > 360){
                        scope.rotation = scope.rotation - 360;
                    }
                    while(scope.rotation < -360){
                        scope.rotation = scope.rotation + 360;
                    }
                }
                
                scope.print = function(){
                    window.print();
                }

                
                if(scope.zoomToFit){
                    // var lastWidth = 0;
                    // var scaleObject = function(){
                    //     var containerWidth = $container.width() / pageZoomLevel;
                    //     var newObjectWidth = $object.width() / pageZoomLevel;
                    //     if(newObjectWidth > lastWidth){
                    //         lastWidth = newObjectWidth;
                    //         initialScale = containerWidth / newObjectWidth;
                    //         zcSvc.transform($object,  0, 0, initialScale);
                    //     }
                    // };
                    // scaleObject();
                    scope.fit();
                    // make sure we reevaluate the size, every time an image is loaded
                    $("img").on("load", function() {
                        // scaleObject();
                        scope.fit();
                    });
                }
                
                var scake

                if(!window.navigator.msPointerEnabled){  // Separating IE pointer events from touch events.

                    var checkCtrlKey = function(ctrlKey){
                        return !scope.useCtrlForMouse || (scope.useCtrlForMouse && ctrlKey)
                    }

                    //************** Zoom Out on Double Right Click ****************************//
                    var lastContextMenu = null;
                    var handleContextMenu = function(e){
                        if(checkCtrlKey(e.ctrlKey)) {
                            zcSvc.prepEvent(e, pageZoomLevel, $object);
                            $object.addClass('transition');
                            if (lastContextMenu) {
                                lastContextMenu = null;
                                var ev = zcSvc.mapMouseToTouch(e);
                                zcSvc.zoom(ev, Math.pow(defaultZoom, -1));
                            } else {
                                lastContextMenu = e;
                                setTimeout(function () {
                                    lastContextMenu = null;
                                }, 300);
                            }
                            return false;
                        }
                    };
                    $object.on('contextmenu', handleContextMenu);

                    //************** Zoom In on Double Click ****************************//
                    var handleDoubleClick = function(e){
                        if(checkCtrlKey(e.ctrlKey)) {
                            zcSvc.prepEvent(e, pageZoomLevel, $object);
                            $object.addClass('transition');
                            var ev = zcSvc.mapMouseToTouch(e);
                            zcSvc.zoom(ev, defaultZoom);
                        }
                    }
                    $object.on('dblclick',handleDoubleClick);

                    //************** Pan on Mouse Drag ****************************//
                    var activeMouse = null;
                    var handleMouseDown = function(e){
                        if(checkCtrlKey(e.ctrlKey)) {
                            zcSvc.prepEvent(e, pageZoomLevel, $object);
                            activeMouse = zcSvc.mapMouseToTouch(e);
                        }
                    };
                    var handleMouseMove = function(e){
                        if(checkCtrlKey(e.ctrlKey)) {
                            if (activeMouse) {
                                zcSvc.prepEvent(e, pageZoomLevel, $object);
                                $object.removeClass('transition');
                                activeMouse.deltaX = e._pageX - activeMouse.start._pageX;
                                activeMouse.deltaY = e._pageY - activeMouse.start._pageY;
                                zcSvc.transform($object, activeMouse.start.translateX + activeMouse.deltaX, activeMouse.start.translateY + activeMouse.deltaY, activeMouse.start.scale);
                            }
                        }
                    };
                    var handleMouseUp = function(e){
                        if(checkCtrlKey(e.ctrlKey)) {
                            zcSvc.prepEvent(e, pageZoomLevel, $object);
                            activeMouse = null;
                        }
                    };
                    $object.on('mousedown', handleMouseDown);
                    $object.on('mousemove', handleMouseMove);
                    $object.on('mouseup', handleMouseUp);



                    var activeTouches = [];
                    var getActiveTouch = function(id){
                        return activeTouches[id.toString()];
                    };
                    var setActiveTouch = function(id, touch){
                        activeTouches[id.toString()] = touch;
                    };
                    var clearActiveTouch = function(id){
                        delete activeTouches[id.toString()];
                    }
                    var touchActionLocked = null;
                    var pendingDoubleDouble = false;
                    var firstDoubleDoubleTapTime = null;
                    var pendingTap = false;
                    var tapTimer = null;
                    var handleTouchMove = function(e){
                        zcSvc.prepEvent(e, pageZoomLevel, $object);

                        $object.removeClass('transition');
                        var largestDelta = 0 ;
                        var touches = e.originalEvent.touches;
                        if(touches.length > 2){
                            zcSvc.reset($object, initialScale);
                        } else {
                            for (var i = 0; i < touches.length; i++) {
                                var touch = touches[i];
                                var activeTouch = getActiveTouch(touch.identifier);
                                if (!activeTouch || !activeTouch.start) {
                                    return;
                                }
                                activeTouch.deltaX = touch._pageX - activeTouch.start._pageX;
                                activeTouch.deltaY = touch._pageY - activeTouch.start._pageY;
                                activeTouch._pageX = touch._pageX;
                                activeTouch._pageY = touch._pageY;

                                largestDelta = Math.max(largestDelta, Math.abs(activeTouch.deltaX), Math.abs(activeTouch.deltaY));
                            }

                            var touchA = getActiveTouch(touches[0].identifier);

                            if (largestDelta < 20 && touchA.state === 'init') {
                                return;
                            }

                            if (touches.length === 1) { // DRAG
                                if (!touchActionLocked) {
                                    touchA.state = 'moving';
                                    zcSvc.transform($object, touchA.start.translateX + touchA.deltaX, touchA.start.translateY + touchA.deltaY, touchA.start.scale);
                                }
                            } else if (touches.length === 2) { // PINCH
                                touchActionLocked = 'pinch';
                                var touchB = getActiveTouch(touches[1].identifier);

                                // calculate scale
                                var originalLineLength = zcSvc.calculateLine(touchA.start._pageX, touchA.start._pageY, touchB.start._pageX, touchB.start._pageY);
                                var currentLineLength = zcSvc.calculateLine(touchA._pageX, touchA._pageY, touchB._pageX, touchB._pageY);
                                var scale = currentLineLength / originalLineLength;

                                zcSvc.zoom(touchA, scale);
                            }
                        }
                    };
                    var handleTouchStart = function(e){
                        zcSvc.prepEvent(e, pageZoomLevel, $object);
                        var currentPos = zcSvc.getObjectCoordinates(e);
                        var touches = e.originalEvent.touches;
                        for(var i=0; i < touches.length; i++){
                            var touch = touches[i];
                            if(!getActiveTouch(touch.identifier)){
                                setActiveTouch(touch.identifier, {
                                    identifier: touch.identifier,
                                    state: 'init',
                                    start: {
                                        timestamp: new Date(),
                                        pageX: touch.pageX,
                                        pageY: touch.pageY,
                                        _pageX: touch._pageX,
                                        _pageY: touch._pageY,
                                        translateX: currentPos.translateX,
                                        translateY: currentPos.translateY,
                                        scale: currentPos.scale,
                                        object: currentPos.pageOffset
                                    },
                                    $object: $object,
                                    pageZoomLevel: pageZoomLevel
                                });
                            }
                        }

                        // check if this could be double double tap.
                        var activeTouchesArray = [];
                        for(touch in activeTouches){
                            activeTouchesArray.push(activeTouches[touch]);
                        }
                        if(touches.length > 1 && Math.abs(activeTouchesArray[0].start.timestamp - activeTouchesArray[1].start.timestamp) > 100){
                            if(pendingDoubleDouble && (firstDoubleDoubleTapTime - activeTouchesArray[1].start.timestamp) < 400){
                                $object.addClass('transition');
                                zcSvc.zoom(activeTouchesArray[1],Math.pow(defaultZoom,-1));
                                pendingDoubleDouble = false;
                            } else {
                                pendingDoubleDouble = true;
                                firstDoubleDoubleTapTime = new Date()
                            }
                        } else {
                            pendingDoubleDouble = false;
                            firstDoubleDoubleTapTime = null;
                        }
                    };
                    var handleTouchEnd = function(e){
                        $targetElement = e.target;
                        zcSvc.prepEvent(e, pageZoomLevel, $object);
                        var changedTouches = e.originalEvent.changedTouches;

                        if(!touchActionLocked && !pendingDoubleDouble){
                            var changedTouch = changedTouches[0];
                            var activeTouch = getActiveTouch(changedTouch.identifier);
                            if(activeTouch.start){
                                var touchDuration = new Date() - activeTouch.start.timestamp;
                                var touchDelta = activeTouch.deltaX ? (activeTouch.deltaX + activeTouch.deltaY) : 0;
                                if(touchDuration < 150 && touchDelta < 10){
                                    if(pendingTap){
                                        pendingTap = false;
                                        clearTimeout(tapTimer);
                                        $object.addClass('transition');
                                        zcSvc.zoom(activeTouch,defaultZoom);
                                    } else {
                                        pendingTap = true;
                                        tapTimer = setTimeout(function(){
                                            // This is a single tap.
                                            pendingTap = false;
                                            $targetElement.click();
                                        }, 250);
                                    }
                                }
                            }
                        }

                        for(var i=0; i < changedTouches.length; i++){
                            var touch = changedTouches[i];
                            clearActiveTouch(touch.identifier);
                        }
                        if(e.originalEvent.touches.length === 0){
                            touchActionLocked = null;
                            pendingDoubleDouble = null;
                        }
                    }

                    $object.on('touchmove', handleTouchMove);
                    $object.on('touchstart', handleTouchStart);
                    $object.on('touchend', handleTouchEnd);
                } else {
                    var pointers = {};
                    var pointersLength = 0;
                    var currentScale;

                    var handlePointerMove = function(e){
                        zcSvc.prepEvent(e, pageZoomLevel, $object);
                        $object.removeClass('transition');

                        // Calculate current delta
                        var pointer = pointers[e.pointerId];
                        if(!pointer){
                            return;
                        }
                        pointer._pageX = e._pageX;
                        pointer._pageY = e._pageY;
                        pointer.deltaX = e._pageX - pointer.start._pageX;
                        pointer.deltaY = e._pageY - pointer.start._pageY;

                        if(pointersLength === 1){
                            console.log(currentScale);
                            zcSvc.transform($object, pointer.start.translateX + pointer.deltaX, pointer.start.translateY + pointer.deltaY, currentScale ? currentScale : pointer.start.scale);
                        } else if (pointersLength === 2){
                            var pointerA;
                            var pointerB;
                            var firstPointer = true;
                            for(var pointerId in pointers){
                                if(firstPointer){
                                    pointerA = pointers[pointerId];
                                    firstPointer = false;
                                } else {
                                    pointerB = pointers[pointerId];
                                }
                            }

                            // calculate scale
                            var originalLineLength = zcSvc.calculateLine(pointerA.start._pageX, pointerA.start._pageY, pointerB.start._pageX, pointerB.start._pageY);
                            var currentLineLength = zcSvc.calculateLine(pointerA._pageX, pointerA._pageY, pointerB._pageX, pointerB._pageY);
                            var scale = currentLineLength / originalLineLength;

                            currentScale = zcSvc.zoom(pointerA, scale);
                        }
                    };
                    var handlePointerStart = function(e){
                        zcSvc.prepEvent(e, pageZoomLevel, $object);
                        var currentPos = zcSvc.getObjectCoordinates(e);
                        pointers[e.pointerId] = {
                            identifier: e.pointerId,
                            state: 'init',
                            start : {
                                timestamp: new Date(),
                                pageX: e.pageX,
                                pageY: e.pageY,
                                _pageX: e._pageX,
                                _pageY: e._pageY,
                                translateX: currentPos.translateX,
                                translateY: currentPos.translateY,
                                scale: currentPos.scale,
                                object: currentPos.pageOffset
                            },
                            $object: $object,
                            pageZoomLevel: pageZoomLevel
                        };
                        pointersLength++;
                    };
                    var handlePointerEnd = function(e){
                        delete pointers[e.pointerId];
                        pointersLength--;
                    };

                    var objectElem = $object[0];
                    objectElem.addEventListener( 'pointerdown', handlePointerStart, false );
                    objectElem.addEventListener( 'pointermove', handlePointerMove, false );
                    objectElem.addEventListener( 'pointerup', handlePointerEnd, false );
                    objectElem.addEventListener( 'pointercancel', handlePointerEnd, false );
                }



                scope.$on('$destroy', function() {
                    $object.off('contextmenu', handleContextMenu);
                    $object.off('handleDoubleClick', handleDoubleClick);
                    $object.off('mousedown', handleMouseDown);
                    $object.off('mousemove', handleMouseMove);
                    $object.off('mouseup', handleMouseUp);
                    $object.off('touchmove', handleTouchMove);
                    $object.off('touchstart', handleTouchStart);
                    $object.off('touchend', handleTouchEnd);
                });


                //**************** Init width if necessary **********************//
            }
        };
    }]);
var Dictionary = (function () {
    function Dictionary() {
        this.itemArray = [];
    }
    Dictionary.prototype.add = function (key, item) {
        this[key] = item;
        this.itemArray.push(item);
    };
    Dictionary.prototype.containsKey = function (key) {
        return !!this[key];
    };
    Dictionary.prototype.getValue = function (key) {
        return this[key];
    };
    Dictionary.prototype.values = function () {
        return this.itemArray;
    };
    Dictionary.prototype.length = function () {
        return this.itemArray.length;
    };
    Dictionary.prototype.getFirst = function () {
        return this.itemArray.length > 0 ? this.itemArray[0] : undefined;
    };
    Dictionary.prototype.remove = function (key) {
        var item = this[key];
        this.itemArray.slice(this.itemArray.indexOf(item), 1);
        delete this[key];
    };
    return Dictionary;
})();

var Stack = (function () {
    function Stack() {
        this.stack = [];
    }
    Stack.prototype.push = function (item) {
        this.stack.push(item);
    };
    Stack.prototype.pop = function () {
        return this.stack.pop();
    };
    Stack.prototype.count = function () {
        return this.stack.length;
    };
    Stack.prototype.values = function () {
        return this.stack;
    };
    return Stack;
})();

var Utils = (function () {
    function Utils() {
    }
    Utils.map = function (list, iterator) {
        var resultingList = [];
        for (var i = 0; i < list.length; i++) {
            resultingList.push(iterator(list[i]));
        }
        return resultingList;
    };
    Utils.mergeRecursive = function (obj1, obj2) {
        for (var p in obj2) {
            try  {
                if (obj2[p].constructor == Object) {
                    obj1[p] = this.mergeRecursive(obj1[p], obj2[p]);
                } else {
                    obj1[p] = obj2[p];
                }
            } catch (e) {
                obj1[p] = obj2[p];
            }
        }

        return obj1;
    };
    Utils.replaceAll = function (str, find, replace) {
        return str.replace(new RegExp(find, 'g'), replace);
    };
    Utils.equals = function (a, b) {
        var aProps = Object.getOwnPropertyNames(a);
        var bProps = Object.getOwnPropertyNames(b);

        if (aProps.length !== bProps.length) {
            return false;
        }

        for (var i = 0; i < aProps.length; i++) {
            var propName = aProps[i];

            if (a[propName] !== b[propName]) {
                return false;
            }
        }

        return true;
    };
    return Utils;
})();


var ADQuoteData;
(function (ADQuoteData) {
    var adQuoteData = angular.module('ad.ws.quoteData', []);

    adQuoteData.provider('ADQuoteData', function () {
        this.quoteServer = null;

        this.registerServer = function (options) {
            this.quoteServer = new ADQuoteData.ADQuoteServer(options.quoteServerLocation.host, options.quoteServerLocation.path);

            this.quoteServer.rootScope = new Object();
            this.quoteServer.rootScope.customerId = -1;
            this.quoteServer.rootScope.vehicleId = -1;
        };

        this.$get = [
            '$http', '$q', '$rootScope', function ($http, $q, $rootScope) {
                this.quoteServer.rootScope = $rootScope;
                this.quoteServer.setQ($q);

                this.quoteServer.ADQuoteDataQuotes = new ADQuoteData.ADQuoteDataQuotes($http, $q, this.quoteServer);
                this.quoteServer.ADQuoteDataShop = new ADQuoteData.ADQuoteDataShop($http, $q, this.quoteServer);

                this.quoteServer.quoteService = this.quoteServer.ADQuoteDataQuotes;
                this.quoteServer.shopService = this.quoteServer.ADQuoteDataShop;

                return this.quoteServer;
            }];

        this.setAuthorizationHeader = function (header) {
            this.quoteServer.setAuthorizationHeader(header);
        };
    });
})(ADQuoteData || (ADQuoteData = {}));
var ADQuoteData;
(function (ADQuoteData) {
    var ADQuoteDataQuotes = (function () {
        function ADQuoteDataQuotes($http, $q, adQuoteDataServer) {
            this.$http = $http;
            this.$q = $q;
            this.adQuoteDataServer = adQuoteDataServer;
            this.urls = {
                quoteListVehicle: function (vehicleId) {
                    return "/vehicles/" + vehicleId + "/quotes";
                },
                quote: function (shopId, quoteId) {
                    return "/shops/" + shopId + "/quotes/" + quoteId;
                },
                quotes: function (shopId) {
                    return "/shops/" + shopId + "/quotes";
                }
            };
        }
        ADQuoteDataQuotes.prototype.getQuotes = function (shopId) {
            var self = this, deferred = self.$q.defer();

            self.adQuoteDataServer.getAjax(this.$http, this.urls.quotes(shopId)).then(function (response) {
                var quotes = response.data;

                quotes.forEach(function (quote) {
                    quote.datalines = quote.hasOwnProperty('datalines') ? quote.datalines : quote._embedded.datalinesInQuotes;
                });

                response.data = quotes;
                deferred.resolve(response);
            }, function (response) {
                deferred.reject(response);
            });

            return deferred.promise;
        };

        ADQuoteDataQuotes.prototype.getQuotesWithExpiration = function (shopId, daysWorth) {
            var self = this, deferred = self.$q.defer();

            var url = this.urls.quotes(shopId);

            self.adQuoteDataServer.getAjax(this.$http, url).then(function (response) {
                var quotes = response.data;

                quotes.forEach(function (quote) {
                    quote.datalines = quote.hasOwnProperty('datalines') ? quote.datalines : quote._embedded.datalinesInQuotes;
                });

                response.data = quotes;
                deferred.resolve(response);
            }, function (response) {
                deferred.reject(response);
            });

            return deferred.promise;
        };

        ADQuoteDataQuotes.prototype.getQuote = function (shopId, quoteId) {
            var self = this, deferred = self.$q.defer();

            self.adQuoteDataServer.getAjax(this.$http, this.urls.quote(shopId, quoteId)).then(function (response) {
                var quote = response.data;
                quote.datalines = quote.hasOwnProperty('datalines') ? quote.datalines : quote._embedded.datalinesInQuotes;
                response.data = quote;
                deferred.resolve(response);
            }, function (response) {
                deferred.reject(response);
            });

            return deferred.promise;
        };

        ADQuoteDataQuotes.prototype.storeNewQuote = function (quote) {
            var self = this, deferred = self.$q.defer();

            self.adQuoteDataServer.postAjax(this.$http, this.urls.quotes(quote.shopId), quote).then(function (response) {
                var location = response.headers.location || '', index = location.lastIndexOf('/');

                if (index > 0) {
                    quote.quoteId = location.substr(index + 1);
                    deferred.resolve();
                } else {
                    deferred.reject('An error ocurred while saving the quote.');
                }
            }, function (response) {
                deferred.reject("Storing Quotes failed: " + response.data);
            });

            return deferred.promise;
        };

        ADQuoteDataQuotes.prototype.updateQuote = function (quote) {
            var self = this, deferred = self.$q.defer();

            self.adQuoteDataServer.putAjax(self.$http, self.urls.quote(quote.shopId, quote.quoteId), quote).then(function (response) {
                deferred.resolve(response);
            }, function (response) {
                deferred.reject(response);
            });

            return deferred.promise;
        };

        ADQuoteDataQuotes.prototype.deleteQuote = function (quote) {
            var self = this, deferred = self.$q.defer();

            self.adQuoteDataServer.deleteAjax(self.$http, self.urls.quote(quote.shopId, quote.quoteId), quote).then(function (response) {
                deferred.resolve(response);
            }, function (response) {
                deferred.reject(response);
            });

            return deferred.promise;
        };

        ADQuoteDataQuotes.prototype.emailQuote = function (quote, address, message) {
            var self = this;

            var defer = self.$q.defer();

            var content = {
                "customerEmailAddress": address,
                "message": message
            };

            self.adQuoteDataServer.putAjax(self.$http, self.urls.quote(quote.shopId, quote.quoteId) + "/email", content).then(function (data) {
                defer.resolve();
            }, function (failReason) {
                defer.reject(failReason);
            });

            return defer.promise;
        };
        ADQuoteDataQuotes.$inject = ['$http', '$q', 'ADQuoteData.ADQuoteServer'];
        return ADQuoteDataQuotes;
    })();
    ADQuoteData.ADQuoteDataQuotes = ADQuoteDataQuotes;
})(ADQuoteData || (ADQuoteData = {}));
var ADQuoteData;
(function (ADQuoteData) {
    var ADQuoteDataShop = (function () {
        function ADQuoteDataShop($http, $q, adQuoteDataServer) {
            this.$http = $http;
            this.$q = $q;
            this.adQuoteDataServer = adQuoteDataServer;
            this.urls = {
                shopList: function () {
                    return "/shops/";
                },
                shop: function (shopId) {
                    return "/shops/" + shopId;
                },
                shopMapping: function () {
                    return '/account?type=OL2';
                },
                shopUsingAccId: function (accId) {
                    return "/account/" + accId;
                }
            };
        }
        ADQuoteDataShop.prototype.getShops = function () {
            return this.adQuoteDataServer.getAjax(this.$http, this.urls.shopList());
        };

        ADQuoteDataShop.prototype.getShop = function (shopId) {
            return this.adQuoteDataServer.getAjax(this.$http, this.urls.shop(shopId));
        };

        ADQuoteDataShop.prototype.storeNewShop = function (shop) {
            return this.adQuoteDataServer.postAjax(this.$http, this.urls.shopList(), shop);
        };

        ADQuoteDataShop.prototype.updateShop = function (shop) {
            return this.adQuoteDataServer.putAjax(this.$http, this.urls.shop(shop.shopId), shop);
        };

        ADQuoteDataShop.prototype.deleteShop = function (shop) {
            return this.adQuoteDataServer.deleteAjax(this.$http, this.urls.shop(shop.shopId), shop);
        };

        ADQuoteDataShop.prototype.createShopMapping = function () {
            return this.adQuoteDataServer.postAjax(this.$http, this.urls.shopMapping(), {});
        };

        ADQuoteDataShop.prototype.getShopMapping = function () {
            return this.adQuoteDataServer.getAjax(this.$http, this.urls.shopMapping(), {});
        };

        ADQuoteDataShop.prototype.getShopId = function () {
            var deferred = this.$q.defer();
            this.adQuoteDataServer.getAjax(this.$http, this.urls.shopMapping(), {
                headers: {
                    'Accept': 'application/json'
                }
            }).then(function (response) {
                deferred.resolve(response.data.shopId);
            }, function (error) {
                deferred.reject(error);
            });
            return deferred.promise;
        };

        ADQuoteDataShop.prototype.getShopIdUsingAccId = function (accId) {
            var options = {
                headers: { 'Accept': 'application/json' }
            };
            return this.adQuoteDataServer.getAjax(this.$http, this.urls.shopUsingAccId(accId), options);
        };
        ADQuoteDataShop.$inject = ['$http', '$q', 'ADQuoteData.ADQuoteServer'];
        return ADQuoteDataShop;
    })();
    ADQuoteData.ADQuoteDataShop = ADQuoteDataShop;
})(ADQuoteData || (ADQuoteData = {}));

var ADQuoteData;
(function (ADQuoteData) {
    var ADQuoteServer = (function () {
        function ADQuoteServer(baseHost, basePath) {
            this.baseHost = baseHost;
            this.basePath = basePath;
            this.errors = {
                ConnectionError: 'CommunityConnectionError'
            };
            this.baseUrl = baseHost + basePath;
        }
        ADQuoteServer.prototype.setQ = function (q) {
            this.$q = q;
        };

        ADQuoteServer.prototype.setAuthorizationHeader = function (header) {
            this.authorizationHeader = header;
        };

        ADQuoteServer.prototype.getAjax = function ($http, url, options) {
            var self = this;
            var deferred = this.$q.defer(), httpOptions = {
                withCredentials: !this.authorizationHeader,
                headers: {
                    'Accept': 'application/hal+json',
                    'Authorization': this.authorizationHeader
                }
            };

            url = this.baseUrl + this.stripURL(url);

            if (options) {
                Utils.mergeRecursive(httpOptions, options);
            }

            $http.get(url, httpOptions).success(function (data, status, headers) {
                deferred.resolve({ data: data, headers: headers() });
            }).error(function (data, status, headers, config) {
                if (status == -1 && data == null) {
                    self.showConnectionAlert(data, status);
                } else {
                    deferred.reject({ data: data, status: status });
                }
            });

            return deferred.promise;
        };

        ADQuoteServer.prototype.postAjax = function ($http, url, postData, options) {
            var self = this;
            var deferred = this.$q.defer(), httpOptions = {
                withCredentials: !this.authorizationHeader,
                headers: {
                    'Content-Type': 'application/json;charset=utf-8',
                    'Accept': 'application/hal+json',
                    'Authorization': this.authorizationHeader
                }
            };

            url = this.baseUrl + this.stripURL(url);

            if (options) {
                Utils.mergeRecursive(httpOptions, options);
            }

            $http.post(url, postData, httpOptions).success(function (data, status, headers) {
                deferred.resolve({ data: data, headers: headers() });
            }).error(function (data, status, headers, config) {
                if (status == -1 && data == null) {
                    self.showConnectionAlert(data, status);
                } else {
                    deferred.reject({ data: data, status: status });
                }
            });

            return deferred.promise;
        };

        ADQuoteServer.prototype.putAjax = function ($http, url, putData, options) {
            var self = this;
            var deferred = this.$q.defer(), httpOptions = {
                withCredentials: !this.authorizationHeader,
                headers: {
                    'Content-Type': 'application/json;charset=utf-8',
                    'Accept': 'application/hal+json',
                    'Authorization': this.authorizationHeader
                }
            };

            url = this.baseUrl + this.stripURL(url);

            if (options) {
                Utils.mergeRecursive(httpOptions, options);
            }

            $http.put(url, putData, httpOptions).success(function (data, status, headers) {
                deferred.resolve({ data: data, headers: headers() });
            }).error(function (data, status, headers, config) {
                if (status == -1 && data == null) {
                    self.showConnectionAlert(data, status);
                } else {
                    deferred.reject({ data: data, status: status });
                }
            });

            return deferred.promise;
        };

        ADQuoteServer.prototype.deleteAjax = function ($http, url, deleteData, options) {
            var self = this;
            var deferred = this.$q.defer(), httpOptions = {
                data: deleteData || {},
                withCredentials: !this.authorizationHeader,
                headers: {
                    'Content-Type': 'application/json;charset=utf-8',
                    'Accept': 'application/hal+json',
                    'Authorization': this.authorizationHeader
                }
            };

            url = this.baseUrl + this.stripURL(url);

            if (options) {
                Utils.mergeRecursive(httpOptions, options);
            }

            $http.delete(url, httpOptions).success(function (data, status, headers) {
                deferred.resolve({ data: data, headers: headers() });
            }).error(function (data, status, headers, config) {
                if (status == -1 && data == null) {
                    self.showConnectionAlert(data, status);
                } else {
                    deferred.reject({ data: data, status: status });
                }
            });

            return deferred.promise;
        };
        ADQuoteServer.prototype.showConnectionAlert = function (data, status) {
            this.rootScope.$emit(this.errors.ConnectionError, { data: data, status: status });
        };

        ADQuoteServer.prototype.stripURL = function (url) {
            return url.replace(this.baseHost, '').replace(this.basePath, '');
        };
        return ADQuoteServer;
    })();
    ADQuoteData.ADQuoteServer = ADQuoteServer;
})(ADQuoteData || (ADQuoteData = {}));

var AD;
(function (AD) {
    (function (WS) {
        (function (ManageOnline) {
            var mol = angular.module('ad.ws.manageOnline', []);
            mol.provider('manageOnlineService', function () {
                this.rootScope;
                this.manageOnlineServer = {};
                this.registerServers = function (options) {
                    this.manageOnlineServer = new ManageOnline.ManageOnlineServer(options.manageOnlineServer.host, options.manageOnlineServer.path);
                };
                this.$get = function ($http, $sce, $q, $rootScope) {
                    this.manageOnlineServer.rootScope = $rootScope;
                    this.manageOnlineServer.setQ($q);
                    var manageOnlineService = new ManageOnline.ManageOnlineService($http, $q, this.manageOnlineServer);
                    return manageOnlineService;
                };
                this.setAuthorizationHeader = function (header) {
                    this.manageOnlineServer.setAuthorizationHeader(header);
                };
            });
        })(WS.ManageOnline || (WS.ManageOnline = {}));
        var ManageOnline = WS.ManageOnline;
    })(AD.WS || (AD.WS = {}));
    var WS = AD.WS;
})(AD || (AD = {}));

var AD;
(function (AD) {
    (function (WS) {
        (function (ManageOnline) {
            var ManageOnlineService = (function () {
                function ManageOnlineService($http, $q, manageOnlineServer) {
                    this.$http = $http;
                    this.$q = $q;
                    this.manageOnlineServer = manageOnlineServer;
                    this.urls = {
                        verifyManageLogin: function (username, password) {
                            return '/adsms/authorization/validate';
                        },
                        lookupByName: function (name) {
                            return '/adsms/customers/name/' + name;
                        },
                        lookupByPhone: function (phone) {
                            return '/adsms/customers/phone/' + phone;
                        },
                        lookupByVIN: function (vin) {
                            return '/adsms/vehicles/vin/' + vin;
                        },
                        lookupByCustomerAccNo: function (accno) {
                            return '/adsms/customers/' + accno;
                        }
                    };
                }
                ManageOnlineService.prototype.verifyLogin = function (username, password) {
                    var self = this, deferred = self.$q.defer();
                    var httpOptions = {
                        headers: {
                            'SMS-Authorization': btoa(username + ':::' + password)
                        }
                    };
                    self.manageOnlineServer.getAjax(this.$http, this.urls.verifyManageLogin(username, password), httpOptions).then(function (response) {
                        console.log('SUCCESS RESPONSE');
                        console.log(response);
                        deferred.resolve('Success');
                    }, function (response) {
                        console.log('FAILURE RESPONSE');
                        console.log(response);
                        deferred.resolve('Failed');
                    });
                    return deferred.promise;
                };

                ManageOnlineService.prototype.lookupByName = function (name, smsAuthorization) {
                    var self = this, deferred = self.$q.defer();
                    var httpOptions = {
                        headers: {
                            'SMS-Authorization': smsAuthorization
                        }
                    };
                    self.manageOnlineServer.getAjax(this.$http, this.urls.lookupByName(name), httpOptions).then(function (response) {
                        deferred.resolve(response);
                    }, function (response) {
                        deferred.reject(response);
                    });
                    return deferred.promise;
                };

                ManageOnlineService.prototype.lookupByPhone = function (phone, smsAuthorization) {
                    var self = this, deferred = self.$q.defer();
                    var httpOptions = {
                        headers: {
                            'SMS-Authorization': smsAuthorization
                        }
                    };
                    self.manageOnlineServer.getAjax(this.$http, this.urls.lookupByPhone(phone), httpOptions).then(function (response) {
                        deferred.resolve(response);
                    }, function (response) {
                        deferred.reject(response);
                    });
                    return deferred.promise;
                };
                ManageOnlineService.prototype.lookupByCustomerAccNo = function (customerAccNo, smsAuthorization) {
                    var self = this, deferred = self.$q.defer();
                    var httpOptions = {
                        headers: {
                            'SMS-Authorization': smsAuthorization
                        }
                    };
                    self.manageOnlineServer.getAjax(this.$http, this.urls.lookupByCustomerAccNo(customerAccNo), httpOptions).then(function (response) {
                        deferred.resolve(response);
                    }, function (response) {
                        deferred.reject(response);
                    });
                    return deferred.promise;
                };

                ManageOnlineService.prototype.lookupByVIN = function (vin, smsAuthorization) {
                    var self = this, deferred = self.$q.defer();
                    var httpOptions = {
                        headers: {
                            'SMS-Authorization': smsAuthorization
                        }
                    };
                    self.manageOnlineServer.getAjax(this.$http, this.urls.lookupByVIN(vin), httpOptions).then(function (response) {
                        deferred.resolve(response);
                    }, function (response) {
                        deferred.reject(response);
                    });
                    return deferred.promise;
                };

                ManageOnlineService.prototype.lookupByLicensePlate = function (licensePlateNo) {
                    var self = this, deferred = self.$q.defer();
                    var url = '../src/manageOnlineLogin/license-state-info.json';
                    self.$http.get(url, { 'Accept': 'application/json' }).success(function (data, status, headers) {
                        console.log(data);
                        deferred.resolve({ data: data, headers: headers() });
                    }).error(function (data, status, headers, config) {
                        if (status == -1 && data == null) {
                        } else {
                            deferred.reject({ data: data, status: status });
                        }
                    });
                    return deferred.promise;
                };
                ManageOnlineService.$inject = ['$http', '$q', 'AD.WS.ManageOnline.ManageOnlineServer'];
                return ManageOnlineService;
            })();
            ManageOnline.ManageOnlineService = ManageOnlineService;
        })(WS.ManageOnline || (WS.ManageOnline = {}));
        var ManageOnline = WS.ManageOnline;
    })(AD.WS || (AD.WS = {}));
    var WS = AD.WS;
})(AD || (AD = {}));
var AD;
(function (AD) {
    (function (WS) {
        (function (ManageOnline) {
            var ManageOnlineServer = (function () {
                function ManageOnlineServer(baseHost, basePath) {
                    this.baseHost = baseHost;
                    this.basePath = basePath;
                    this.errors = {
                        ConnectionError: 'ManageOnlineConnectionError'
                    };
                    this.baseUrl = baseHost + basePath;
                }
                ManageOnlineServer.prototype.setQ = function (q) {
                    this.$q = q;
                };

                ManageOnlineServer.prototype.setAuthorizationHeader = function (header) {
                    this.authorizationHeader = header;
                };

                ManageOnlineServer.prototype.getAjax = function ($http, url, options) {
                    var self = this;
                    var deferred = this.$q.defer(), httpOptions = {
                        withCredentials: !this.authorizationHeader,
                        headers: {
                            'Accept': 'application/hal+json',
                            'Authorization': this.authorizationHeader
                        }
                    };

                    url = this.baseUrl + this.stripURL(url);

                    if (options) {
                        Utils.mergeRecursive(httpOptions, options);
                    }

                    $http.get(url, httpOptions).success(function (data, status, headers) {
                        deferred.resolve({ data: data, headers: headers() });
                    }).error(function (data, status, headers, config) {
                        if (status == -1 && data == null) {
                            self.showConnectionAlert(data, status);
                        } else {
                            deferred.reject({ data: data, status: status });
                        }
                    });

                    return deferred.promise;
                };

                ManageOnlineServer.prototype.postAjax = function ($http, url, postData, options) {
                    var self = this;
                    var deferred = this.$q.defer(), httpOptions = {
                        withCredentials: !this.authorizationHeader,
                        headers: {
                            'Content-Type': 'application/json;charset=utf-8',
                            'Accept': 'application/hal+json',
                            'Authorization': this.authorizationHeader
                        }
                    };

                    url = this.baseUrl + this.stripURL(url);

                    if (options) {
                        Utils.mergeRecursive(httpOptions, options);
                    }

                    $http.post(url, postData, httpOptions).success(function (data, status, headers) {
                        deferred.resolve({ data: data, headers: headers() });
                    }).error(function (data, status, headers, config) {
                        if (status == -1 && data == null) {
                            self.showConnectionAlert(data, status);
                        } else {
                            deferred.reject({ data: data, status: status });
                        }
                    });

                    return deferred.promise;
                };

                ManageOnlineServer.prototype.putAjax = function ($http, url, putData, options) {
                    var self = this;
                    var deferred = this.$q.defer(), httpOptions = {
                        withCredentials: !this.authorizationHeader,
                        headers: {
                            'Content-Type': 'application/json;charset=utf-8',
                            'Accept': 'application/hal+json',
                            'Authorization': this.authorizationHeader
                        }
                    };

                    url = this.baseUrl + this.stripURL(url);

                    if (options) {
                        Utils.mergeRecursive(httpOptions, options);
                    }

                    $http.put(url, putData, httpOptions).success(function (data, status, headers) {
                        deferred.resolve({ data: data, headers: headers() });
                    }).error(function (data, status, headers, config) {
                        if (status == -1 && data == null) {
                            self.showConnectionAlert(data, status);
                        } else {
                            deferred.reject({ data: data, status: status });
                        }
                    });

                    return deferred.promise;
                };

                ManageOnlineServer.prototype.deleteAjax = function ($http, url, deleteData, options) {
                    var self = this;
                    var deferred = this.$q.defer(), httpOptions = {
                        data: deleteData || {},
                        withCredentials: !this.authorizationHeader,
                        headers: {
                            'Content-Type': 'application/json;charset=utf-8',
                            'Accept': 'application/hal+json',
                            'Authorization': this.authorizationHeader
                        }
                    };

                    url = this.baseUrl + this.stripURL(url);

                    if (options) {
                        Utils.mergeRecursive(httpOptions, options);
                    }

                    $http.delete(url, httpOptions).success(function (data, status, headers) {
                        deferred.resolve({ data: data, headers: headers() });
                    }).error(function (data, status, headers, config) {
                        if (status == -1 && data == null) {
                            self.showConnectionAlert(data, status);
                        } else {
                            deferred.reject({ data: data, status: status });
                        }
                    });

                    return deferred.promise;
                };
                ManageOnlineServer.prototype.showConnectionAlert = function (data, status) {
                    this.rootScope.$emit(this.errors.ConnectionError, { data: data, status: status });
                };

                ManageOnlineServer.prototype.stripURL = function (url) {
                    return url.replace(this.baseHost, '').replace(this.basePath, '');
                };
                return ManageOnlineServer;
            })();
            ManageOnline.ManageOnlineServer = ManageOnlineServer;
        })(WS.ManageOnline || (WS.ManageOnline = {}));
        var ManageOnline = WS.ManageOnline;
    })(AD.WS || (AD.WS = {}));
    var WS = AD.WS;
})(AD || (AD = {}));

angular.module('ad.ui.ymme.templates', ['partials/ymmeSearch.html']);

angular.module("partials/ymmeSearch.html", []).run(["$templateCache", function($templateCache) {
  $templateCache.put("partials/ymmeSearch.html",
    "<div class=\"ymme-container\">\n" +
    "    <!--DROID-546 - Creating a new UI component which holds the Barcode Scan Icon, upon tapping on it, it turns on the Camera and upon pointing on a Barcode or QR code, we get the vin filled\n" +
    "    in the search box-->\n" +
    "    <span class=\"btn-scan\" ng-click=\"vm.onBarCodeScan()\" ng-show=\"requiresBarcodeTrigger\"><i class=\"appIcon-scan-search before\"></i></span>\n" +
    "    <search-box placeholder=\"Search by Year, Make, Model, Engine, or VIN\" size=\"'lg'\" on-search=\"vm.search(searchTerm)\" on-clear=\"vm.search('')\" search-term=\"vin\" number-of-character-delay=\"2\" time-delay=\"500\" is-focused=\"isFocused\"></search-box>\n" +
    "    <br>\n" +
    "    <div class=\"ymme-selectors row\">\n" +
    "        <div class=\"col-md-2 year-column\">\n" +
    "            <ad-dropdown items=\"vm.years\" input-size=\"lg\" ng-model=\"vm.selectedYear\" display-property=\"'title'\" \n" +
    "                on-select=\"vm.yearSelected(item)\" open=\"vm.yearIsOpen\" placeholder=\"Year\"></ad-dropdown>\n" +
    "        </div>\n" +
    "        <div class=\"col-md-5 make-column\">\n" +
    "            <ad-dropdown items=\"vm.makes\" input-size=\"lg\" ng-model=\"vm.selectedMake\" display-property=\"'title'\" \n" +
    "                on-select=\"vm.makeSelected(item)\" open=\"vm.makeIsOpen\" placeholder=\"Make\"></ad-dropdown>\n" +
    "        </div>\n" +
    "        <div class=\"col-md-5 model-column\">\n" +
    "            <ad-dropdown items=\"vm.models\" input-size=\"lg\" ng-model=\"vm.selectedModel\" display-property=\"'title'\" \n" +
    "                on-select=\"vm.modelSelected(item)\" open=\"vm.modelIsOpen\" placeholder=\"Model\"></ad-dropdown>\n" +
    "        </div>\n" +
    "    </div>\n" +
    "\n" +
    "    <ul class=\"engine-selection list-group\">\n" +
    "        <a ng-repeat=\"engine in vm.engines | filter: vm.minimumYear\" class=\"list-group-item expand-right\" ng-click=\"vm.selectCar(engine)\">\n" +
    "            <div class=\"ymme\">\n" +
    "                <div class=\"engine\">{{ engine.engine }} <span ng-show=\"engine.vin\">- ({{ engine.vin }})</span></div>\n" +
    "                <div class=\"ymm\">{{ engine.year }} {{ engine.make }} {{ engine.model }}</div>\n" +
    "            </div>\n" +
    "\n" +
    "        </a>\n" +
    "    </ul>\n" +
    "\n" +
    "    <div ng-show=\"vm.engineNotFound && vm.vinNotFound && !vm.invalidVin\" class=\"warning text-center\">\n" +
    "        No matching vehicle was found.\n" +
    "    </div>\n" +
    "\n" +
    "    <div ng-show=\"vm.invalidVin\" class=\"warning text-center\">\n" +
    "        Entered VIN is invalid.\n" +
    "    </div>\n" +
    "\n" +
    "    <div ng-show=\"vm.isSearching\" class=\"warning text-center\">\n" +
    "        Searching...\n" +
    "    </div>\n" +
    "</div>");
}]);
;var AD;
(function (AD) {
    (function (UI) {
        (function (YMME) {
            var Controller = (function () {
                function Controller($scope, $injector, ADConnect) {
                    var _this = this;
                    this.$scope = $scope;
                    this.$injector = $injector;
                    this.ADConnect = ADConnect;
                    this.engineNotFound = false;
                    this.vinNotFound = false;
                    this.invalidVin = false;
                    this.isSearching = false;
                    this.yearIsOpen = false;
                    this.makeIsOpen = false;
                    this.modelIsOpen = false;
                    this.hasYears = false;
                    this.hasMakes = false;
                    this.hasModels = false;
                    var self = this;

                    self.makes = [];
                    self.models = [];

                    this.searchControl = {};
                    console.log($scope.$parent.vin);
                    if ($scope.$parent.vin) {
                        this.search($scope.$parent.vin);
                    }
                    $scope.callYMME = function (term) {
                        self.search(term);
                    };

                    $scope.vm = self;
                    this.ADConnect.getYears().success(function (data) {
                        self.years = data.items;
                        self.hasYears = true;
                        if (self.engines && self.engines.length > 0) {
                            self.yearIsOpen = false;
                        } else {
                            self.yearIsOpen = true;
                        }
                    });
                    $scope.$on('resetYMME', function () {
                        _this.resetYears();
                        _this.resetMakes();
                        _this.resetModels();
                        _this.resetEngines();
                        _this.resetSearch();
                    });

                    try  {
                        this.$analytics = $injector.get('$analytics');
                    } catch (e) {
                        console.log("Google analytics is not active within ymme module.");
                    }
                }
                Controller.prototype.yearSelected = function (year) {
                    var self = this;
                    this.resetMakes();
                    this.resetModels();
                    this.resetEngines();
                    this.resetSearch();

                    this.ADConnect.getMakesByLink(year.makesLink).success(function (data) {
                        self.makes = data.items;
                        self.makeIsOpen = true;
                        self.hasMakes = true;
                    });
                };
                Controller.prototype.makeSelected = function (make) {
                    var self = this;
                    this.resetModels();
                    this.resetEngines();
                    this.resetSearch();
                    this.ADConnect.getModelsByLink(make.modelsLink).success(function (data) {
                        self.models = data.items;
                        self.modelIsOpen = true;
                        self.hasModels = true;
                    });
                };
                Controller.prototype.modelSelected = function (model) {
                    var self = this;
                    this.resetEngines();
                    this.resetSearch();

                    this.ADConnect.getEnginesByLink(model.enginesLink).success(function (data) {
                        var engines = [];
                        for (var i = 0; i < data.items.length; i++) {
                            var searchResult = data.items[i];
                            engines.push(new engine(self.selectedYear.title, self.selectedMake.title, self.selectedModel.title, searchResult.title, searchResult.carLink ? searchResult.carLink.carId : searchResult.selectionLink, null));
                        }
                        self.engines = engines;
                        self.modelIsOpen = false;
                    });

                    if (this.$analytics) {
                        this.track('Tapped on YMME drop downs', 'Vehicle Selection', 'Searched for vehicle by selecting year, make, model, and engine');
                    }
                };

                Controller.prototype.search = function (searchTerm) {
                    var _this = this;
                    var self = this;
                    var engines = [];
                    this.engineNotFound = false;
                    this.vinNotFound = false;
                    this.invalidVin = false;
                    this.isSearching = false;

                    if (searchTerm.length > 0) {
                        this.isSearching = true;
                        if (/^[a-zA-Z0-9]*$/.test(searchTerm) == false || searchTerm.length > 17) {
                            this.invalidVin = true;
                        } else {
                            this.invalidVin = false;
                        }

                        this.yearIsOpen = false;
                        this.makeIsOpen = false;
                        this.modelIsOpen = false;
                        this.resetYears();
                        this.resetMakes();
                        this.resetModels();
                        this.resetEngines();

                        this.ADConnect.searchEngines(searchTerm).success(function (data) {
                            for (var i = 0; i < data.results.length; i++) {
                                var searchResult = data.results[i];
                                engines.push(new engine(searchResult.year, searchResult.make, searchResult.model, searchResult.engine, searchResult.carid, null));
                            }
                            self.engines = engines;
                            self.isSearching = false;
                            self.invalidVin = false;
                            if (engines.length === 0) {
                                self.engineNotFound = true;
                                self.vinNotFound = true;
                            }
                        }).error(function (data, status, headers, config) {
                            if (engines.length === 0) {
                                self.engineNotFound = true;
                                self.vinNotFound = true;
                            }
                        });

                        if (!this.invalidVin) {
                            this.ADConnect.searchVins(searchTerm).success(function (data) {
                                for (var i = 0; i < data.results.length; i++) {
                                    var searchResult = data.results[i];
                                    _this.isSearching = false;
                                    engines.push(new engine(searchResult.year, searchResult.make, searchResult.model, searchResult.engine, searchResult.carid, searchTerm));
                                }
                                self.engineNotFound = engines.length === 0;
                                if (engines.length === 0) {
                                    self.vinNotFound = true;
                                } else if (engines.length === 1 && searchTerm.length === 17) {
                                    self.selectCar(engines[0]);
                                } else {
                                    self.engines = engines;
                                    self.vinNotFound = false;
                                    self.yearIsOpen = false;
                                    self.isSearching = false;
                                }
                            }).error(function (data, status, headers, config) {
                                if (engines.length === 0) {
                                    self.vinNotFound = true;
                                    self.isSearching = false;
                                }
                            });
                        } else {
                            this.isSearching = false;
                        }
                        this.track('Entered Text in search bar', 'Vehicle Selection', 'Searched for vehicle by entering text in search bar');
                    } else {
                        self.engines = engines;
                    }
                };

                Controller.prototype.selectCar = function (engine) {
                    var _this = this;
                    if (engine.selectionLink && engine.selectionLink.length > 7) {
                        this.ADConnect.getCarDetails(engine.selectionLink).then(function (vehicle) {
                            engine.selectionLink = vehicle.carId;
                            _this.$scope.onCarSelected({ engine: engine });
                        });
                    } else {
                        this.$scope.onCarSelected({ engine: engine });
                    }
                    this.track('Tapped on vehicle from YMME list', 'Vehicle Selection', 'Vehicle selected from YMME list');
                };

                Controller.prototype.resetYears = function () {
                    this.selectedYear = null;
                };
                Controller.prototype.resetMakes = function () {
                    this.makes = [];
                    this.selectedMake = null;
                    this.hasMakes = false;
                };
                Controller.prototype.resetModels = function () {
                    this.models = [];
                    this.selectedModel = null;
                    this.hasModels = false;
                };
                Controller.prototype.resetEngines = function () {
                    this.engines = [];
                };
                Controller.prototype.resetSearch = function () {
                    this.engineNotFound = false;
                    this.vinNotFound = false;
                    this.invalidVin = false;
                    this.isSearching = false;
                    this.$scope.vin = "";
                };

                Controller.prototype.onBarCodeScan = function () {
                    if (this.$scope.onBarcodeTriggered) {
                        this.track('Tapped on bar code button', 'Vehicle Selection', 'Bar code scan initiated');
                        this.$scope.onBarcodeTriggered();
                    }
                };

                Controller.prototype.track = function (message, category, label) {
                    if (this.$analytics) {
                        this.$analytics.eventTrack(message, { category: category, label: label });
                    }
                };
                Controller.$inject = ['$scope', '$injector', 'ADConnect'];
                return Controller;
            })();
            YMME.Controller = Controller;

            var engine = (function () {
                function engine(year, make, model, engine, selectionLink, vin) {
                    this.year = year;
                    this.make = make;
                    this.model = model;
                    this.engine = engine;
                    this.selectionLink = selectionLink;
                    this.vin = vin;
                }
                return engine;
            })();
        })(UI.YMME || (UI.YMME = {}));
        var YMME = UI.YMME;
    })(AD.UI || (AD.UI = {}));
    var UI = AD.UI;
})(AD || (AD = {}));
var AD;
(function (AD) {
    (function (UI) {
        (function (YMME) {
            var Directives = (function () {
                function Directives() {
                }
                Directives.Init = function (app) {
                    app.directive('ymmeSearch', function () {
                        return {
                            restrict: 'AECM',
                            templateUrl: 'partials/ymmeSearch.html',
                            scope: {
                                onCarSelected: '&',
                                vin: '=?',
                                onBarcodeTriggered: '&?',
                                isFocused: '=?',
                                disableBarcodeTrigger: '=?'
                            },
                            controller: AD.UI.YMME.Controller,
                            link: function (scope, elem, attrs) {
                                scope.requiresBarcodeTrigger = angular.isDefined(attrs.onBarcodeTriggered);
                                if (angular.isDefined(elem[0].attributes['disable-barcode-trigger'])) {
                                    scope.$watch('disableBarcodeTrigger', function (value) {
                                        scope.requiresBarcodeTrigger = angular.isDefined(attrs.onBarcodeTriggered) && !value;
                                    });
                                }
                            }
                        };
                    });
                };
                return Directives;
            })();
            YMME.Directives = Directives;
        })(UI.YMME || (UI.YMME = {}));
        var YMME = UI.YMME;
    })(AD.UI || (AD.UI = {}));
    var UI = AD.UI;
})(AD || (AD = {}));

var AD;
(function (AD) {
    (function (UI) {
        var ymme = angular.module('ad.ui.ymme', ['ad.ui.bootstrap', 'ad.ws.repair', 'ad.ui.ymme.templates']);
        ymme.controller('ymmeController', AD.UI.YMME.Controller);
        AD.UI.YMME.Directives.Init(ymme);
    })(AD.UI || (AD.UI = {}));
    var UI = AD.UI;
})(AD || (AD = {}));

var ADConnect;
(function (ADConnect) {
    var adRepair = angular.module('ad.ws.repair', []);
    adRepair.provider('ADConnect', function () {
        this.rootScope;
        this.adConnectServer = {};
        this.carLookupServer = {};
        this.registerServers = function (options) {
            this.adConnectServer = new ADConnect.ADConnectServer(options.adConnect.host, options.adConnect.path, options.adConnect.uuid, options.adConnect.gridRoot);
            this.carLookupServer = new ADConnect.CarLookupServer(options.carLookup.host, options.carLookup.path, options.carLookup.audience, options.carLookup.minYear, options.carLookup.maxYear);
        };
        this.$get = function ($http, $sce, $q, $rootScope) {
            this.adConnectServer.rootScope = $rootScope;
            this.carLookupServer.rootScope = $rootScope;
            var adConnectService = new ADConnect.ADConnectService($http, $sce, $q, $rootScope, this.adConnectServer, this.carLookupServer);
            return adConnectService;
        };
    });
})(ADConnect || (ADConnect = {}));
var ADConnect;
(function (ADConnect) {
    var ADConnectURI = (function () {
        function ADConnectURI(adConnectServer, url) {
            this.url = url;
            url = url.replace(adConnectServer.baseUrl, '');
            var urlElems = url.split('/');
            var index = 0;
            while (index < urlElems.length) {
                var nextValue = urlElems[index];
                switch (nextValue) {
                    case "carids":
                        index++;
                        this.carId = parseInt(urlElems[index]);
                        index++;
                        break;
                    case "components":
                        index++;
                        this.componentId = parseInt(urlElems[index]);
                        index++;
                        break;
                    case "itypes":
                        index++;
                        this.iTypeId = parseInt(urlElems[index]);
                        index++;
                        break;
                    case "tsbs":
                        index++;
                        this.tsbId = parseInt(urlElems[index]);
                        index++;
                        break;
                    case "nonstandards":
                        index++;
                        this.nonStandardId = parseInt(urlElems[index]);
                        index++;
                        break;
                    default:
                        index++;
                }
            }
        }
        ADConnectURI.prototype.getkey = function () {
            var key = this.carId + '/c' + this.componentId;
            if (this.iTypeId)
                key = key + '/i' + this.iTypeId;
            if (this.nonStandardId)
                key = key + '/n' + this.nonStandardId;
            if (this.tsbId)
                key = key + '/t' + this.tsbId;
            return key;
        };
        ADConnectURI.getUrl = function (adConnectServer, carId, componentId, iTypeId, nonStandardId, tsbId) {
            if (!componentId) {
                componentId = 1;
            }
            var uri = '/carids/' + carId + '/components/' + componentId + '/';
            if (iTypeId)
                uri = uri + 'itypes/' + iTypeId.toString() + '/';
            if (nonStandardId)
                uri = uri + 'nonstandards/' + nonStandardId.toString() + '/';
            if (tsbId)
                uri = uri + 'tsbs/' + tsbId + '/';
            return new ADConnectURI(adConnectServer, uri);
        };
        ADConnectURI.cleanADUrl = function (adConnectServer, url) {
            var newUrl = url.replace(adConnectServer.baseHost, '').replace(adConnectServer.basePath, '');
            newUrl = Utils.replaceAll(newUrl, '//', '/');
            return newUrl;
        };
        ADConnectURI.convertADUrlToGridUrl = function (adConnectServer, url) {
            url = url.replace(adConnectServer.baseHost, '');
            url = url.replace('//', '/');
            var newUrl = url.replace(adConnectServer.baseHost, '').replace(adConnectServer.basePath, '').replace('/carids/', adConnectServer.gridRoot).replace('/components/', '?ComponentId=').replace('/itypes/', '&InformationTypeId=').replace('/nonstandards/', '&NonStandardId=').replace('/tsbs/', '&TsbId=').replace('/image_article/', '&ImageIndex=');
            if (newUrl.substr(newUrl.length - 1, 1) === '/') {
                newUrl = newUrl.substr(0, newUrl.length - 1);
            }
            return newUrl;
        };
        ADConnectURI.convertGridUrlToADUrl = function (adConnectServer, url) {
            var indexOfCar = url.lastIndexOf('/car/');
            var url = url.substring(indexOfCar);
            var newUrl = (adConnectServer.basePath + url).replace('/car/', '/carids/').replace('?ComponentId=', '/components/').replace('&InformationTypeId=', '/itypes/').replace('&NonStandardId=', '/nonstandards/').replace('&TsbId=', '/tsbs/').replace('&ImageIndex=', '/image_article/');
            if (newUrl.substr(newUrl.length - 1, 1) !== '/') {
                newUrl = newUrl + '/';
            }
            return newUrl;
        };
        ADConnectURI.convertADUrlToAndroidUrl = function (adConnectServer, url) {
            return ADConnectURI.convertADUrlToGridUrl(adConnectServer, url).replace(adConnectServer.gridRoot, "file:///android_asset/RepairGrid/index.html#/car/");
        };
        return ADConnectURI;
    })();
    ADConnect.ADConnectURI = ADConnectURI;
})(ADConnect || (ADConnect = {}));
var ADConnect;
(function (ADConnect) {
    var ADConnectObject = (function () {
        function ADConnectObject(carId, componentId, url, dataService) {
            this.carId = carId;
            this.componentId = componentId;
            this.url = url;
            this.dataService = dataService;
            this.isPassthrough = false;
            this.bookmarkId = null;
            this.isExpanded = false;
        }
        ADConnectObject.prototype.expand = function () {
            var deferred = this.dataService.$q.defer();
            deferred.resolve();
            return deferred.promise;
        };

        ADConnectObject.prototype.parseChildren = function ($data, selector) {
            var _this = this;
            var self = this;
            return $.map($data.find(selector), function (item, index) {
                var uri = new ADConnect.ADConnectURI(_this.dataService.adConnectServer, item.pathname);
                var childObject = self.dataService.getADConnectObject(uri);
                childObject.title = item.text;
                return childObject;
            });
        };

        ADConnectObject.prototype.parseITypes = function ($data, selector) {
            var iTypes = new Dictionary();
            var self = this;
            $.each($data.find(selector), function (index) {
                var uri = new ADConnect.ADConnectURI(self.dataService.adConnectServer, this.pathname);
                var childObject = self.dataService.getADConnectObject(uri);
                childObject.title = this.text;
                iTypes.add("iType" + uri.iTypeId, childObject);
            });
            return iTypes;
        };

        ADConnectObject.prototype.parseParentPath = function ($data) {
            var parentRef = $data.find("dt:contains('Parent')").next().find('a')[0];
            if (parentRef) {
                this.parent = this.dataService.getADConnectObject(new ADConnect.ADConnectURI(this.dataService.adConnectServer, parentRef.pathname));
                return this.parent.expand();
            }
            var deferred = this.dataService.$q.defer();
            deferred.resolve();
            return deferred.promise;
        };

        ADConnectObject.prototype.ancestors = function () {
            var parentStack = [];
            var currentParent = this.parent;
            while (currentParent) {
                parentStack.unshift(currentParent);
                currentParent = currentParent.parent;
            }
            return parentStack;
        };

        ADConnectObject.prototype.getPassthroughContext = function () {
            return null;
        };

        ADConnectObject.prototype.getParent = function () {
            if (this.parent.isPassthrough) {
                return this.parent.getParent();
            }
            return this.parent;
        };
        return ADConnectObject;
    })();
    ADConnect.ADConnectObject = ADConnectObject;
})(ADConnect || (ADConnect = {}));
var ADConnect;
(function (ADConnect) {
    var ADConnectServer = (function () {
        function ADConnectServer(baseHost, basePath, uuid, gridRoot) {
            this.baseHost = baseHost;
            this.basePath = basePath;
            this.uuid = uuid;
            this.gridRoot = gridRoot;
            this.errors = {
                ConnectionError: 'ADConnectConnectionError'
            };
            this.baseUrl = baseHost + basePath;
            var urlElems = baseHost.split('://');
            this.baseUrlWithAuthorization = urlElems[0] + "://" + uuid + ":ALLDATA@" + urlElems[1] + basePath;
            this.authorizationHeader = 'Basic ' + btoa(uuid + ":ALLDATA");
        }
        ADConnectServer.prototype.setUUID = function (uuid) {
            var urlElems = this.baseHost.split('://');
            this.baseUrlWithAuthorization = urlElems[0] + "://" + uuid + ":ALLDATA@" + urlElems[1] + this.basePath;
            this.authorizationHeader = 'Basic ' + btoa(uuid + ":ALLDATA");
        };

        ADConnectServer.prototype.isLoggedIn = function () {
            return this.authorizationHeader !== 'Basic ' + btoa(":ALLDATA");
        };

        ADConnectServer.prototype.getAjax = function ($http, url, options, ignoreError) {
            var _this = this;
            url = url.replace(this.baseHost, '').replace(this.basePath, '');
            var httpOptions = { method: 'GET', url: this.baseUrl + url, headers: { 'Authorization': this.authorizationHeader, 'x-access-token': this.uuid } };
            if (options) {
                Utils.mergeRecursive(httpOptions, options);
            }
            return $http(httpOptions).error(function (data, status, headers, config) {
                if (ignoreError === false) {
                    console.log("No Error to be Shown");
                } else {
                    _this.rootScope.$emit(_this.errors.ConnectionError, { data: data, status: status });
                }
            });
        };
        ADConnectServer.prototype.postAjax = function ($http, url, postData) {
            var _this = this;
            url = url.replace(this.baseHost, '').replace(this.basePath, '');
            return $http({ method: 'POST', url: this.baseUrl + url, data: postData, headers: { 'Authorization': this.authorizationHeader, 'x-access-token': this.uuid } }).error(function (data, status, headers, config) {
                _this.rootScope.$emit(_this.errors.ConnectionError, { data: data, status: status });
            });
        };
        ADConnectServer.prototype.deleteAjax = function ($http, url) {
            var _this = this;
            url = url.replace(this.baseHost, '').replace(this.basePath, '');
            return $http({ method: 'DELETE', url: this.baseUrl + url, headers: { 'Authorization': this.authorizationHeader, 'x-access-token': this.uuid } }).error(function (data, status, header) {
                _this.rootScope.$emit(_this.errors.ConnectionError, { data: data, status: status });
            });
        };
        return ADConnectServer;
    })();
    ADConnect.ADConnectServer = ADConnectServer;

    var CarLookupServer = (function () {
        function CarLookupServer(baseHost, basePath, audience, minYear, maxYear) {
            this.baseHost = baseHost;
            this.basePath = basePath;
            this.audience = audience;
            this.minYear = minYear;
            this.maxYear = maxYear;
            this.errors = {
                ConnectionError: 'CarLookupConnectionError'
            };
            this.baseUrl = baseHost + basePath;
            this.params = {
                audience: audience,
                minYear: minYear,
                maxYear: maxYear
            };
        }
        CarLookupServer.prototype.getAjax = function ($http, url) {
            var _this = this;
            url = url.replace(this.baseHost, '').replace(this.basePath, '');
            return $http({
                method: 'GET',
                url: this.baseUrl + url,
                params: this.params,
                headers: { 'ACCEPT': 'application/json' }
            }).error(function (data, status, headers, config) {
                if (status !== 404) {
                    _this.rootScope.$emit(_this.errors.ConnectionError, { data: data, status: status });
                }
            });
        };
        return CarLookupServer;
    })();
    ADConnect.CarLookupServer = CarLookupServer;
})(ADConnect || (ADConnect = {}));
var __extends = this.__extends || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
var ADConnect;
(function (ADConnect) {
    var Component = (function (_super) {
        __extends(Component, _super);
        function Component() {
            _super.apply(this, arguments);
        }
        Component.prototype.expand = function () {
            var _this = this;
            var deferred = this.dataService.$q.defer();
            if (!this.isExpanded) {
                this.dataService.getXHTML(this.url).then(function (response) {
                    var $data = $(response);
                    _this.carTitle = $data.find(".vehicle")[0].textContent;
                    _this.title = $data.find(".name")[0].textContent;
                    _this.components = _this.parseChildren($data, '.children .component');
                    _this.informationTypes = _this.parseITypes($data, '.children .itype');
                    _this.isPassthrough = (_this.components.length === 1 && _this.informationTypes.length() === 0) || (_this.informationTypes.length() === 1 && _this.components.length === 0);
                    _this.parseParentPath($data).then(function () {
                        _this.isExpanded = true;
                        deferred.resolve();
                    });
                });
            } else {
                deferred.resolve();
            }

            return deferred.promise;
        };
        Component.prototype.expandChildComponents = function () {
            if (this.components) {
                for (var i = 0; i < this.components.length; i++) {
                    this.components[i].expand();
                }
            }
        };
        Component.prototype.hasInformationTypes = function () {
            return this.informationTypes && this.informationTypes.length() > 0;
        };
        Component.prototype.hasComponents = function () {
            return this.components && this.components.length > 0;
        };
        Component.prototype.hasComponent = function (componentId) {
            for (var i = 0; i < this.components.length; i++) {
                if (this.components[i].componentId === componentId) {
                    return true;
                }
            }
            return false;
        };
        Component.prototype.getPassthroughContext = function () {
            if (this.isPassthrough && this.hasInformationTypes()) {
                return this.informationTypes.getFirst();
            }
            if (this.isPassthrough && this.hasComponents()) {
                return this.components[0];
            }
            return null;
        };
        return Component;
    })(ADConnect.ADConnectObject);
    ADConnect.Component = Component;
})(ADConnect || (ADConnect = {}));
var ADConnect;
(function (ADConnect) {
    var Vehicle = (function () {
        function Vehicle(year, make, model, engine, vin, carId, vehicleComponent, dataService) {
            this.year = year;
            this.make = make;
            this.model = model;
            this.engine = engine;
            this.vin = vin;
            this.carId = carId;
            this.vehicleComponent = vehicleComponent;
            this.dataService = dataService;
            this.isExpanded = false;
            this.componentBank1 = [];
            this.componentBank2 = [];
        }
        Vehicle.prototype.expand = function () {
            var _this = this;
            var deferred = this.dataService.$q.defer();
            if (!this.isExpanded) {
                this.vehicleComponent.expand().then(function () {
                    var compLength = _this.vehicleComponent.components.length;

                    for (var i = 0; i < compLength; i++) {
                        if (i < Math.ceil((compLength - 1) / 2)) {
                            _this.componentBank1.push(_this.vehicleComponent.components[i]);
                        } else {
                            _this.componentBank2.push(_this.vehicleComponent.components[i]);
                        }
                    }
                    deferred.resolve();
                });
            } else {
                deferred.resolve();
            }

            return deferred.promise;
        };
        return Vehicle;
    })();
    ADConnect.Vehicle = Vehicle;
})(ADConnect || (ADConnect = {}));
var ADConnect;
(function (ADConnect) {
    var InformationType = (function (_super) {
        __extends(InformationType, _super);
        function InformationType(carId, componentId, informationTypeId, url, dataService) {
            _super.call(this, carId, componentId, url, dataService);
            this.carId = carId;
            this.componentId = componentId;
            this.informationTypeId = informationTypeId;
            this.url = url;
            this.dataService = dataService;
        }
        InformationType.prototype.expand = function () {
            var _this = this;
            var deferred = this.dataService.$q.defer();
            if (!this.isExpanded) {
                this.dataService.getXHTML(this.url).then(function (response) {
                    var $data = $(response);

                    var currentSelection = $data.find('.current_selection');
                    _this.componentTitle = currentSelection.find('.component_name')[0].textContent;
                    _this.iTypeTitle = currentSelection.find('.itype_name')[0].textContent;

                    _this.informationTypes = _this.parseITypes($data, '.children .itype');
                    _this.nonStandards = _this.parseChildren($data, '.children .nonstandard');
                    _this.tsbs = _this.parseTsbs($data);
                    if ($data.find('ul.article a').length > 0) {
                        _this.article = _this.parseArticle($data);
                        _this.article.expand();
                    }
                    _this.isPassthrough = _this.informationTypes.length() === 1 || _this.nonStandards.length === 1 || _this.tsbs.length === 1;
                    _this.parseParentPath($data).then(function () {
                        _this.isExpanded = true;
                        deferred.resolve();
                    });
                });
            } else {
                deferred.resolve();
            }
            return deferred.promise;
        };

        InformationType.prototype.parseTsbs = function ($data) {
            var tsbs = new Array();
            var self = this;
            $.each($data.find('.tsb_info'), function (index) {
                var $this = $(this);
                var tsbNumber = $this.find('.tsb_number')[0].textContent;
                var tsbDate = $this.find('.tsb_date')[0].textContent;
                var tsbTitleCell = $this.find('.tsb_title .tsb')[0];
                var tsbTitle = tsbTitleCell.textContent;
                var tsb = self.dataService.getADConnectObject(new ADConnect.ADConnectURI(self.dataService.adConnectServer, tsbTitleCell.pathname));
                tsb.number = tsbNumber;
                tsb.date = tsbDate;
                tsb.title = tsbTitle;

                var siblingLink = $this.next().find('ul.ul_arr_dwn a');
                if (siblingLink && siblingLink.length > 0) {
                    tsb.superseededBy = {
                        title: siblingLink[0].innerText,
                        url: siblingLink[0].pathname
                    };
                }

                tsbs.push(tsb);
            });
            return tsbs;
        };

        InformationType.prototype.parseArticle = function ($data) {
            var parsedArticle;
            var $article = $data.find('ul.article a');

            if ($article.hasClass('html_article')) {
                parsedArticle = new ADConnect.Article(this.carId, this.componentId, this.informationTypeId, this.url, this.dataService);
                parsedArticle.title = $article[0].textContent;
            }
            if ($article.hasClass('parts_and_labor_article')) {
                parsedArticle = new ADConnect.PartsAndLaborArticle(this.carId, this.componentId, this.informationTypeId, this.url, this.dataService);
                parsedArticle.title = $article[0].textContent;
            }
            if ($article.hasClass('maintenance_table')) {
                parsedArticle = new ADConnect.MaintenanceTableArticle(this.carId, this.componentId, this.informationTypeId, this.url, this.dataService);
                parsedArticle.title = $article[0].textContent;
            }

            return parsedArticle;
        };

        InformationType.prototype.hasInformationTypes = function () {
            return this.informationTypes && this.informationTypes.length() > 0;
        };
        InformationType.prototype.hasNonStandards = function () {
            return this.nonStandards && this.nonStandards.length > 0;
        };
        InformationType.prototype.hasTsbs = function () {
            return this.tsbs && this.tsbs.length > 0;
        };
        InformationType.prototype.getPassthroughContext = function () {
            if (this.isPassthrough && this.hasInformationTypes()) {
                return this.informationTypes.getFirst();
            }
            if (this.isPassthrough && this.hasNonStandards()) {
                return this.nonStandards[0];
            }
            if (this.isPassthrough && this.hasTsbs()) {
                return this.tsbs[0];
            }
            return null;
        };
        return InformationType;
    })(ADConnect.ADConnectObject);
    ADConnect.InformationType = InformationType;
})(ADConnect || (ADConnect = {}));
var ADConnect;
(function (ADConnect) {
    var NonStandard = (function (_super) {
        __extends(NonStandard, _super);
        function NonStandard(carId, componentId, informationTypeId, nonStandardId, url, dataService) {
            _super.call(this, carId, componentId, url, dataService);
            this.carId = carId;
            this.componentId = componentId;
            this.informationTypeId = informationTypeId;
            this.nonStandardId = nonStandardId;
            this.url = url;
            this.dataService = dataService;
        }
        NonStandard.prototype.expand = function () {
            var _this = this;
            var deferred = this.dataService.$q.defer();
            if (!this.isExpanded) {
                this.url = this.url.replace('/maintenance_schedule', '');
                this.dataService.getXHTML(this.url).then(function (response) {
                    var $data = $(response);

                    _this.title = $data.find(".name")[0].textContent;

                    var currentSelection = $data.find('.current_selection');
                    _this.componentTitle = currentSelection.find('.component_name')[0].textContent;
                    _this.iTypeTitle = currentSelection.find('.itype_name')[0].textContent;
                    _this.nonStandardTitle = currentSelection.find('.nonstandard_name')[0].textContent;

                    _this.nonStandards = _this.parseChildren($data, '.children .nonstandard');
                    var articleLink = $data.find('ul.article a');
                    if (articleLink.length > 0) {
                        if (articleLink[0].pathname.indexOf('/prom_article/') > 0) {
                            _this.article = new ADConnect.PromArticle(_this.carId, _this.componentId, _this.informationTypeId, _this.nonStandardId, _this.url, _this.dataService);
                            _this.article.title = articleLink[0].textContent;
                            _this.article.expand();
                        } else {
                            _this.article = new ADConnect.NonStandardArticle(_this.carId, _this.componentId, _this.informationTypeId, _this.nonStandardId, _this.url, _this.dataService);
                            _this.article.title = articleLink[0].textContent;
                            _this.article.expand();
                        }
                    }
                    _this.isPassthrough = _this.nonStandards.length === 1;
                    _this.parseParentPath($data).then(function () {
                        _this.isExpanded = true;
                        deferred.resolve();
                    });
                });
            } else {
                deferred.resolve();
            }
            return deferred.promise;
        };

        NonStandard.prototype.hasNonStandards = function () {
            return this.nonStandards && this.nonStandards.length > 0;
        };
        NonStandard.prototype.getPassthroughContext = function () {
            if (this.isPassthrough && this.hasNonStandards()) {
                return this.nonStandards[0];
            }
            return null;
        };
        return NonStandard;
    })(ADConnect.ADConnectObject);
    ADConnect.NonStandard = NonStandard;
})(ADConnect || (ADConnect = {}));
var ADConnect;
(function (ADConnect) {
    var Tsb = (function (_super) {
        __extends(Tsb, _super);
        function Tsb(carId, componentId, informationTypeId, tsbId, url, dataService) {
            _super.call(this, carId, componentId, url, dataService);
            this.carId = carId;
            this.componentId = componentId;
            this.informationTypeId = informationTypeId;
            this.tsbId = tsbId;
            this.url = url;
            this.dataService = dataService;
        }
        Tsb.prototype.expand = function () {
            var _this = this;
            var deferred = this.dataService.$q.defer();
            if (!this.isExpanded) {
                return this.dataService.getXHTML(this.url).then(function (response) {
                    var $data = $(response);
                    _this.nonStandards = _this.parseChildren($data, '.children .nonstandard');
                    _this.article = new ADConnect.TsbArticle(_this.carId, _this.componentId, _this.informationTypeId, _this.tsbId, _this.url, _this.dataService);
                    _this.article.parse($data);
                    _this.parseParentPath($data);
                    _this.isExpanded = true;
                });
            }
            deferred.resolve();
            return deferred.promise;
        };
        return Tsb;
    })(ADConnect.ADConnectObject);
    ADConnect.Tsb = Tsb;
})(ADConnect || (ADConnect = {}));
var ADConnect;
(function (ADConnect) {
    var Article = (function (_super) {
        __extends(Article, _super);
        function Article(carId, componentId, informationTypeId, url, dataService) {
            _super.call(this, carId, componentId, url, dataService);
            this.carId = carId;
            this.componentId = componentId;
            this.informationTypeId = informationTypeId;
            this.url = url;
            this.dataService = dataService;
            this.hasContent = true;
        }
        Article.prototype.expand = function (urlSuffix) {
            var _this = this;
            var deferred = this.dataService.$q.defer();
            if (!urlSuffix) {
                urlSuffix = 'html_article/';
            }
            this.dataService.getXHTML(this.url + urlSuffix).then(function (response) {
                var $data = $(response);

                var $articleBody = $data.siblings('.article_body');
                if (urlSuffix === 'maintenance_schedule/') {
                    $articleBody = $data.find('.article_body');
                }

                _this.parseImages($articleBody);
                Article.parseLinks($articleBody, _this.dataService.adConnectServer);

                var html = _this.dataService.$sce.trustAsHtml(_this.cleanHtml($articleBody[0].innerHTML));
                _this.content = html;
                _this.isExpanded = true;
            });
            deferred.resolve();
            return deferred.promise;
        };

        Article.prototype.parseImages = function ($articleBody) {
            var images = $articleBody.find('img');
            var self = this;
            self.images = [];
            images.each(function ($index) {
                var $this = $(this);
                var srcAttr = $this.attr('src');
                var useMapValue = $this.attr('usemap');
                var mapId = useMapValue ? useMapValue.replace(/\./g, '\\.') : '';
                var $map = $this.siblings('map' + mapId);
                $this.attr('ng-src', '{{ vm.context.article.images[' + $index + '].src }}');
                $this.attr('src', '');
                $this.attr('img-index', $index);
                var imgName = $this.attr('img_name');
                var image = new Image(srcAttr, $map, parseInt($this.attr('img_width')), parseInt($this.attr('width').replace('px', '')));
                self.images.push(image);
                if (srcAttr) {
                    self.dataService.loadImage(srcAttr, function (blob_uri, requested_uri) {
                        var imgFound = false;
                        var retries = 0;
                        var updateImageSrc = function () {
                            var imgElem = $('img[img-index="' + $index + '"]')[0];
                            if (!imgElem && retries < 100) {
                                setTimeout(function () {
                                    updateImageSrc();
                                }, 300);
                                retries++;
                            } else if (imgElem) {
                                imgElem.src = blob_uri;
                                image.src = blob_uri;
                            } else {
                                console.log('given up after ' + retries + ' retries.');
                            }
                        };
                        updateImageSrc();
                    });
                }

                var $imageAnchorElem = $this.nextAll('a.image').first();
                $imageAnchorElem.html('').append($this);
                $imageAnchorElem.attr('href', ADConnect.ADConnectURI.convertADUrlToGridUrl(self.dataService.adConnectServer, self.url) + "&ImageIndex=" + $index);
            });
        };
        Article.parseLinks = function ($articleBody, adConnectServer) {
            var links = $articleBody.find('a');
            links.each(function () {
                var $this = $(this);
                if ($this.hasClass('image')) {
                } else {
                    var url = $this.attr('href');
                    if (url && url.substr(0, 1) !== "#") {
                        var urlElems = url.split('/');
                        var lastUrlElem = urlElems[urlElems.length - 2];
                        if (lastUrlElem.substr(0, 1) !== '-') {
                            $this.attr('href', ADConnect.ADConnectURI.convertADUrlToGridUrl(adConnectServer, url));
                        }
                    }
                }
            });
            var areaLinks = $articleBody.find('area');
            areaLinks.each(function () {
                var $this = $(this);
                var url = $this.attr('href');
                if (url && url.substr(0, 1) !== "#") {
                    var urlElems = url.split('/');
                    var lastUrlElem = urlElems[urlElems.length - 2];
                    if (lastUrlElem.substr(0, 1) !== '-') {
                        $this.attr('href', ADConnect.ADConnectURI.convertADUrlToGridUrl(adConnectServer, url));
                    }
                }
            });
        };

        Article.prototype.cleanHtml = function (html) {
            return html.replace(/<!--[\s\S]*?-->/g, '').replace(/\r/g, '').replace(/\n{2,}/g, '');
        };

        Article.prototype.loadFullImage = function () {
            var currentImage = this.images[this.currentImageIndex];
            this.dataService.loadImage(currentImage.fullSrc.replace('thumbnails', 'images'), function (blob_uri, requested_uri) {
                var imgFound = false;
                var retries = 0;
                if (!imgFound && retries < 50) {
                    setTimeout(function () {
                        var imgElem = $('.full-image img')[0];
                        if (imgElem) {
                            imgElem.src = blob_uri;
                        }
                        retries++;
                    }, 10);
                }
            });
        };
        return Article;
    })(ADConnect.ADConnectObject);
    ADConnect.Article = Article;

    var NonStandardArticle = (function (_super) {
        __extends(NonStandardArticle, _super);
        function NonStandardArticle(carId, componentId, informationTypeId, nonStandardId, url, dataService) {
            _super.call(this, carId, componentId, informationTypeId, url, dataService);
            this.carId = carId;
            this.componentId = componentId;
            this.informationTypeId = informationTypeId;
            this.nonStandardId = nonStandardId;
            this.url = url;
            this.dataService = dataService;
        }
        NonStandardArticle.prototype.expand = function () {
            if (this.nonStandardId < 0) {
                return _super.prototype.expand.call(this, 'maintenance_schedule/');
            }
            return _super.prototype.expand.call(this);
        };
        return NonStandardArticle;
    })(Article);
    ADConnect.NonStandardArticle = NonStandardArticle;

    var PromArticle = (function (_super) {
        __extends(PromArticle, _super);
        function PromArticle(carId, componentId, informationTypeId, nonStandardId, url, dataService) {
            _super.call(this, carId, componentId, informationTypeId, url, dataService);
            this.carId = carId;
            this.componentId = componentId;
            this.informationTypeId = informationTypeId;
            this.nonStandardId = nonStandardId;
            this.url = url;
            this.dataService = dataService;
            this.promUpdates = [];
        }
        PromArticle.prototype.expand = function () {
            var _this = this;
            var self = this;
            var deferred = this.dataService.$q.defer();
            this.dataService.getXHTML(this.url + 'html_article/').then(function (response) {
                var $data = $(response);
                var $articleBody = $data.siblings('.article_body');

                var promUpdateElems = $articleBody.find('update');
                promUpdateElems.each(function ($index) {
                    var $this = $(this);
                    self.promUpdates.push(new PromUpdate(self.getValue('scan_id', $this), self.getValue('prom_code', $this), self.getValue('prom_bc_code', $this), self.getValue('part_number', $this), self.getValue('note', $this), self.getValue('super_part_num', $this), self.getValue('super_scan_id', $this), self.getValue('prom_tsb_ref_no', $this), self.getValue('tsb_number', $this), self.getValue('tsb_division', $this)));
                });

                _this.isExpanded = true;
            });
            deferred.resolve();
            return deferred.promise;
        };

        PromArticle.prototype.getValue = function (elemTag, parentElem) {
            var valueElem = parentElem.find(elemTag);
            if (valueElem.length > 0)
                return valueElem[0].textContent;
            else
                return "";
        };
        return PromArticle;
    })(Article);
    ADConnect.PromArticle = PromArticle;

    var PromUpdate = (function () {
        function PromUpdate(scan_id, prom_code, prom_bc_code, part_number, note, super_part_num, super_scan_id, prom_tsb_ref_no, tsb_number, tsb_division) {
            this.scan_id = scan_id;
            this.prom_code = prom_code;
            this.prom_bc_code = prom_bc_code;
            this.part_number = part_number;
            this.note = note;
            this.super_part_num = super_part_num;
            this.super_scan_id = super_scan_id;
            this.prom_tsb_ref_no = prom_tsb_ref_no;
            this.tsb_number = tsb_number;
            this.tsb_division = tsb_division;
        }
        return PromUpdate;
    })();
    ADConnect.PromUpdate = PromUpdate;

    var TsbArticle = (function (_super) {
        __extends(TsbArticle, _super);
        function TsbArticle(carId, componentId, informationTypeId, tsbId, url, dataService) {
            _super.call(this, carId, componentId, informationTypeId, url, dataService);
            this.carId = carId;
            this.componentId = componentId;
            this.informationTypeId = informationTypeId;
            this.tsbId = tsbId;
            this.url = url;
            this.dataService = dataService;
        }
        TsbArticle.prototype.expand = function () {
        };
        TsbArticle.prototype.parse = function ($data) {
            this.title = $data.find('.tsb_name')[0].textContent;
            var $articleBody = $data.find('.article_body');
            if ($articleBody.length > 0) {
                this.parseImages($articleBody);
                Article.parseLinks($articleBody, this.dataService.adConnectServer);
                var html = this.dataService.$sce.trustAsHtml(this.cleanHtml($articleBody[0].innerHTML));
                this.content = html;
            } else {
                this.hasContent = false;
            }
            this.isExpanded = true;
        };
        return TsbArticle;
    })(Article);
    ADConnect.TsbArticle = TsbArticle;

    var PartsAndLaborArticle = (function (_super) {
        __extends(PartsAndLaborArticle, _super);
        function PartsAndLaborArticle(carId, componentId, informationTypeId, url, dataService) {
            _super.call(this, carId, componentId, informationTypeId, url, dataService);
            this.carId = carId;
            this.componentId = componentId;
            this.informationTypeId = informationTypeId;
            this.url = url;
            this.dataService = dataService;
        }
        PartsAndLaborArticle.prototype.expand = function () {
            var _this = this;
            var deferred = this.dataService.$q.defer();
            this.dataService.getXHTML(this.url + 'parts_and_labor_article/').then(function (response) {
                var $data = $(response.replace(new RegExp('visibility: hidden', 'g'), 'display:none'));
                var $articleBody = $data.find('.article_body');
                _this.parseImages($articleBody);
                Article.parseLinks($articleBody, _this.dataService.adConnectServer);

                var html = _this.dataService.$sce.trustAsHtml(_this.cleanHtml($articleBody[0].innerHTML));
                _this.content = html;
                _this.isExpanded = true;
            });
            deferred.resolve();
            return deferred.promise;
        };
        return PartsAndLaborArticle;
    })(Article);
    ADConnect.PartsAndLaborArticle = PartsAndLaborArticle;

    var MaintenanceTableArticle = (function (_super) {
        __extends(MaintenanceTableArticle, _super);
        function MaintenanceTableArticle(carId, componentId, informationTypeId, url, dataService) {
            _super.call(this, carId, componentId, informationTypeId, url, dataService);
            this.carId = carId;
            this.componentId = componentId;
            this.informationTypeId = informationTypeId;
            this.url = url;
            this.dataService = dataService;
        }
        MaintenanceTableArticle.prototype.expand = function () {
            var _this = this;
            var deferred = this.dataService.$q.defer();
            this.dataService.getXHTML(this.url + 'maintenance_table/').then(function (response) {
                var self = _this;
                var $data = $(response);
                var $articleBody = $data.find('.article_body');
                Article.parseLinks($articleBody, _this.dataService.adConnectServer);
                _this.pages = [];
                var $pages = $articleBody.find('.mileage_page');
                $pages.each(function () {
                    var $this = $(this);
                    var mileagePage = new MaintenanceMileagePage(self.carId, self.componentId, self.informationTypeId, self.url, self.dataService);
                    mileagePage.parse($this);
                    self.pages.push(mileagePage);
                });
                _this.setActivePage(self.pages[0]);
                var html = _this.dataService.$sce.trustAsHtml(_this.cleanHtml($articleBody[0].innerHTML));
                _this.content = html;
                _this.isExpanded = true;
            });
            deferred.resolve();
            return deferred.promise;
        };

        MaintenanceTableArticle.prototype.setActivePage = function (page) {
            if (this.activePage) {
                this.activePage.isActive = false;
            }
            this.activePage = page;
            page.isActive = true;
        };
        return MaintenanceTableArticle;
    })(Article);
    ADConnect.MaintenanceTableArticle = MaintenanceTableArticle;

    var MaintenanceMileagePage = (function () {
        function MaintenanceMileagePage(carId, componentId, informationTypeId, url, dataService) {
            this.carId = carId;
            this.componentId = componentId;
            this.informationTypeId = informationTypeId;
            this.url = url;
            this.dataService = dataService;
        }
        MaintenanceMileagePage.prototype.parse = function ($data) {
            var self = this;
            var title = $data.find('table.captions td strong')[0].textContent.split('-');
            var minimum = this.convertToThousands(title[0]);
            var maximum = '';
            if (title.length > 1) {
                maximum = ' - ' + this.convertToThousands(title[1]);
            }
            this.title = minimum + maximum;

            var $entries = $data.find('table.entries tr');
            var rows;
            this.columnHeaders = [];
            $entries.each(function (index) {
                var $this = $(this);
                if (index === 0) {
                    var columns = $this.find('td');

                    for (var i = 1; i < columns.length; i++) {
                        var link = $(columns[i]).find('a')[0];
                        if (link) {
                            var columnHeader = self.convertToThousands(link.textContent);
                            var articleUrl = link.pathname;
                            self.columnHeaders.push(new MaintenanceMileagePageColumn(columnHeader, articleUrl, self.dataService));
                        } else {
                            self.columnHeaders.push(new MaintenanceMileagePageColumn("", "", self.dataService));
                        }
                    }
                } else {
                    rows = rows + '<tr>' + $this.html() + '</tr>';
                }
            });
            this.content = this.dataService.$sce.trustAsHtml(rows);
        };

        MaintenanceMileagePage.prototype.showDetails = function (column) {
            if (this.activeColumn)
                this.activeColumn.isActive = false;
            if (column === this.activeColumn) {
                this.activeColumn = null;
            } else {
                column.expand();
                this.activeColumn = column;
                this.activeColumn.isActive = true;
            }
        };
        MaintenanceMileagePage.prototype.convertToThousands = function (mileage) {
            return (parseFloat(mileage) / 1000).toString() + 'k';
        };
        return MaintenanceMileagePage;
    })();
    ADConnect.MaintenanceMileagePage = MaintenanceMileagePage;

    var MaintenanceMileagePageColumn = (function () {
        function MaintenanceMileagePageColumn(title, url, dataService) {
            this.title = title;
            this.url = url;
            this.dataService = dataService;
            this.isEmpty = (title === '');
        }
        MaintenanceMileagePageColumn.prototype.expand = function () {
            var _this = this;
            var deferred = this.dataService.$q.defer();
            this.dataService.getXHTML(this.dataService.adConnectServer.baseHost + this.url + 'maintenance_schedule/').then(function (response) {
                var self = _this;
                var $data = $(response);
                var $articleBody = $data.find('.article_body');
                Article.parseLinks($articleBody, _this.dataService.adConnectServer);
                var html = _this.dataService.$sce.trustAsHtml($articleBody[0].innerHTML);
                _this.details = html;
            });
            deferred.resolve();
            return deferred.promise;
        };
        return MaintenanceMileagePageColumn;
    })();
    ADConnect.MaintenanceMileagePageColumn = MaintenanceMileagePageColumn;

    var Image = (function () {
        function Image(src, $map, width, thumbWidth) {
            this.src = src;
            this.$map = $map;
            this.width = width;
            this.thumbWidth = thumbWidth;
            if ($map) {
                this.map = new Map($map, width / thumbWidth);
            }
            this.fullSrc = src;
        }
        return Image;
    })();
    ADConnect.Image = Image;
    var Map = (function () {
        function Map($map, scale) {
            this.$map = $map;
            this.scale = scale;
            this.name = $map.attr('name');
            this.areas = [];
            var $areas = $map.children('area');
            for (var i = 0; i < $areas.length; i++) {
                var $area = $($areas[i]);
                this.areas.push(new MapArea($area, scale));
            }
        }
        return Map;
    })();
    ADConnect.Map = Map;
    var MapArea = (function () {
        function MapArea($area, scale) {
            this.$area = $area;
            this.scale = scale;
            var c = $area.attr('coords').split(',');
            this.coords = {
                top: parseInt(c[1]) * scale,
                left: parseInt(c[0]) * scale,
                width: (parseInt(c[2]) * scale) - (parseInt(c[0]) * scale),
                height: (parseInt(c[3]) * scale) - (parseInt(c[1]) * scale)
            };
            this.positionStyle = { 'top': this.coords.top + "px", 'left': this.coords.left + "px", 'width': this.coords.width + "px", 'height': this.coords.height + "px" };
            this.ADUrl = $area.attr('href');
        }
        return MapArea;
    })();
    ADConnect.MapArea = MapArea;
})(ADConnect || (ADConnect = {}));
var ADConnect;
(function (ADConnect) {
    var Bookmark = (function () {
        function Bookmark(url, title, carid, bookmarkid, engine, make, model, timestamp, componentId, year) {
            this.url = url;
            this.title = title;
            this.carid = carid;
            this.bookmarkid = bookmarkid;
            this.engine = engine;
            this.make = make;
            this.model = model;
            this.timestamp = timestamp;
            this.componentId = componentId;
            this.year = year;
        }
        return Bookmark;
    })();
    ADConnect.Bookmark = Bookmark;
})(ADConnect || (ADConnect = {}));

var cache;
var ADConnect;
(function (ADConnect) {
    var ADConnectService = (function () {
        function ADConnectService($http, $sce, $q, $rootScope, adConnectServer, carLookupServer) {
            this.$http = $http;
            this.$sce = $sce;
            this.$q = $q;
            this.$rootScope = $rootScope;
            this.adConnectServer = adConnectServer;
            this.carLookupServer = carLookupServer;
            this.urls = {
                recent_cars: "/account/recent_cars",
                car_lookup_search: "/find/",
                vin_lookup_search: "/vins/",
                car_lookup_carid: "/carids/",
                car_lookup_years: "/years.json",
                car_lookup_makes: function (year) {
                    return "/years/" + year + "/makes.json";
                },
                car_lookup_models: function (year, make) {
                    return "/years/" + year + "/makes/" + make + "/models.json";
                },
                car_lookup_engines: function (year, make, model) {
                    return "/years/" + year + "/makes/" + make + "/models/" + model + "/engines.json";
                },
                search: function (carId, searchTerm, filter) {
                    return "/carids/" + carId + "/?search=" + searchTerm + (filter !== "All" ? "&search-select=" + filter : "");
                },
                dtcs: function (carId, vin) {
                    return "/account/carids/" + carId + "/dtcs/?count=100&vin=" + vin;
                },
                dtcsDescription: function (carId, dtcCodes) {
                    return "/carids/" + carId + "/dtc_descriptions/" + dtcCodes + '/';
                },
                postDtcs: "/account/dtcs/",
                clearDtcs: function (carId) {
                    return "/account/carids/" + carId + "/clear-codes/";
                },
                ff: function (carId, vin) {
                    return "/account/carids/" + carId + "/freeze-frame/" + "?query_params&vin=" + vin + "&count=" + 100;
                },
                bookmarks: "/account/bookmarks/",
                deleteBookmark: function (bookmarkId) {
                    return "/account/bookmarks/?bookmarkid=" + bookmarkId;
                },
                postFFs: function (carId) {
                    return "/account/carids/" + carId + "/freeze-frame";
                },
                register: "/register",
                authenticate: "/auth-check/"
            };
            this.iTypeCategories = [
                { iTypeId: 1, title: "Service Precautions" },
                { iTypeId: 2, title: "Application and ID" },
                { iTypeId: 3, title: "Description and Operation" },
                { iTypeId: 5, title: "Adjustments" },
                { iTypeId: 6, title: "Testing and Inspection" },
                { iTypeId: 7, title: "Diagrams" },
                { iTypeId: 8, title: "Locations" },
                { iTypeId: 9, title: "Specifications" },
                { iTypeId: 10, title: "Service and Repair" },
                { iTypeId: 13, title: "Technical Service Bulletins" },
                { iTypeId: 42, title: "Tools and Equipment" },
                { iTypeId: 83, title: "Exploded Parts Diagram" },
                { iTypeId: 189, title: "Parts and Labor" },
                { iTypeId: 334, title: "Service Intervals" },
                { iTypeId: 344, title: "Diagnostic Trouble Codes" }
            ];
            this.iTypeCategoriesDictionary = new Dictionary();
            this.componentCache = new Dictionary();
            cache = {
                components: this.componentCache
            };
            this.getEntireBookmarkList();

            for (var catIndex = 0; catIndex < this.iTypeCategories.length; catIndex++) {
                this.iTypeCategoriesDictionary.add(this.iTypeCategories[catIndex].title, this.iTypeCategories[catIndex].iTypeId.toString());
            }
        }
        ADConnectService.prototype.checkifArticleExists = function (url) {
            var deferred = this.$q.defer();
            this.adConnectServer.getAjax(this.$http, url, null, false).success(function (data, status, headers, config) {
                deferred.resolve(data);
            }).error(function (data, status, headers, config) {
                deferred.reject(data);
            });
            return deferred.promise;
        };

        ADConnectService.prototype.getEntireBookmarkList = function () {
            var _this = this;
            var deferred = this.$q.defer();

            if (!this.adConnectServer.isLoggedIn()) {
                deferred.reject('UUID has not been set yet.');
                return deferred.promise;
            }

            this.adConnectServer.getAjax(this.$http, this.urls.bookmarks).success(function (data, status, headers, config) {
                var bookmarks = _.map(data, function (bookmark) {
                    var indexOfCar = bookmark.url.lastIndexOf('/car/');
                    if (indexOfCar > 0) {
                        var appUrl = bookmark.url.substring(indexOfCar);
                        bookmark.url = ADConnect.ADConnectURI.convertGridUrlToADUrl(_this.adConnectServer, bookmark.url);
                    }
                    return bookmark;
                });
                _this.bookmarkList = _.groupBy(data, function (bookmark) {
                    return bookmark.carid;
                });
                var currentThis = _this;
                _this.bookmarkArray = Object.keys(_this.bookmarkList).map(function (k) {
                    return currentThis.bookmarkList[k];
                });

                _this.sortBookmarkArray();
                deferred.resolve(_this.bookmarkList);
            }).error(function (data, status, headers, config) {
                deferred.reject(data);
            });
            return deferred.promise;
        };

        ADConnectService.prototype.getBookmark = function (carId, url) {
            return _.find(this.bookmarkList[carId], function (bookmark) {
                return bookmark.url === url;
            });
        };

        ADConnectService.prototype.sortBookmarkArray = function () {
            this.bookmarkArray = this.bookmarkArray.sort(function (a, b) {
                if (a.length > 1) {
                    a.sort(function (a, b) {
                        return b.timestamp > a.timestamp ? 1 : -1;
                    });
                }

                return b[0].timestamp > a[0].timestamp ? 1 : -1;
            });
        };

        ADConnectService.prototype.convertURL = function (url) {
            var convertedURL = ADConnect.ADConnectURI.convertGridUrlToADUrl(this.adConnectServer, url);
            return convertedURL;
        };

        ADConnectService.prototype.saveBookmark = function (data) {
            var _this = this;
            var deferred = this.$q.defer();
            var carBookmarks = this.bookmarkList[data.carid];
            var foundBookmark = _.find(carBookmarks, function (bookmark) {
                return bookmark.title === data.title;
            });
            if (foundBookmark) {
                deferred.reject("A bookmark with this title already exists for this vehicle.");
            } else {
                this.adConnectServer.postAjax(this.$http, this.urls.bookmarks, data).success(function (response) {
                    _this.getEntireBookmarkList().then(function (list) {
                        carBookmarks = list[data.carid];
                        data.url = ADConnect.ADConnectURI.convertGridUrlToADUrl(_this.adConnectServer, data.url);
                        var foundBookmark = _.find(carBookmarks, function (bookmark) {
                            return bookmark.url === data.url;
                        });
                        deferred.resolve(foundBookmark);
                    });
                }).error(function (response) {
                    deferred.resolve(response);
                });
            }
            return deferred.promise;
        };

        ADConnectService.prototype.deleteBookmark = function (carid, bookmarkId) {
            var _this = this;
            var deferred = this.$q.defer();
            this.adConnectServer.deleteAjax(this.$http, this.urls.deleteBookmark(bookmarkId)).success(function (data, status, headers, config) {
                _.remove(_this.bookmarkList[carid], function (bookmark) {
                    return bookmark.bookmarkid === bookmarkId;
                });
                _this.getEntireBookmarkList().then(function (response) {
                    deferred.resolve(data);
                }, (function (error) {
                    console.log("Get Failed but Deleting the bookmark Succeeded, hence, resolving the promise");
                    console.log(error);
                    deferred.resolve(data);
                }));
            }).error(function (data, status, headers, config) {
                deferred.reject(data);
            });
            return deferred.promise;
        };

        ADConnectService.prototype.clearCache = function () {
            this.componentCache = new Dictionary();
        };

        ADConnectService.prototype.getCarDetails = function (carLookupLink) {
            var _this = this;
            var deferred = this.$q.defer();
            var url = carLookupLink.replace('.json', "/car").replace(this.carLookupServer.basePath, '');

            this.carLookupServer.getAjax(this.$http, url).success(function (data) {
                var vehicle = new ADConnect.Vehicle(data.year, data.make, data.model, data.engine, null, data.carid, _this.getADConnectObject(ADConnect.ADConnectURI.getUrl(_this.adConnectServer, data.carid, 1)), _this);
                deferred.resolve(vehicle);
            }).error(function (response, status) {
                return deferred.reject(status);
            });

            return deferred.promise;
        };

        ADConnectService.prototype.searchCarByLink = function (carLookupLink) {
            var _this = this;
            var deferred = this.$q.defer();
            var url = carLookupLink.replace('.json', "/car").replace(this.carLookupServer.basePath, '');

            this.carLookupServer.getAjax(this.$http, url).success(function (data) {
                var vehicle = new ADConnect.Vehicle(data.year, data.make, data.model, data.engine, null, data.carid, _this.getADConnectObject(ADConnect.ADConnectURI.getUrl(_this.adConnectServer, data.carid, 1)), _this);
                vehicle.expand().then(function () {
                    deferred.resolve(vehicle);
                });
            }).error(function (response, status) {
                return deferred.reject(status);
            });

            return deferred.promise;
        };

        ADConnectService.prototype.selectCarById = function (carId, vin) {
            var _this = this;
            var deferred = this.$q.defer();
            var url = this.urls.car_lookup_carid + carId;

            this.carLookupServer.getAjax(this.$http, url).success(function (data) {
                var vehicle = new ADConnect.Vehicle(data.year, data.make, data.model, data.engine, vin, data.carid, _this.getADConnectObject(ADConnect.ADConnectURI.getUrl(_this.adConnectServer, data.carid, 1)), _this);
                vehicle.expand().then(function () {
                    deferred.resolve(vehicle);
                });
            }).error(function (response, status) {
                return deferred.reject(status);
            });

            return deferred.promise;
        };

        ADConnectService.prototype.addCarToRecentList = function (carId, vin) {
            var url = this.urls.recent_cars;
            var postData = { carid: carId };
            if (vin) {
                postData["vin"] = vin;
            }
            return this.adConnectServer.postAjax(this.$http, url, postData);
        };

        ADConnectService.prototype.getYears = function () {
            return this.carLookupServer.getAjax(this.$http, this.urls.car_lookup_years);
        };
        ADConnectService.prototype.getMakes = function (year) {
            return this.carLookupServer.getAjax(this.$http, this.urls.car_lookup_makes(year));
        };
        ADConnectService.prototype.getMakesByLink = function (makesLink) {
            return this.carLookupServer.getAjax(this.$http, makesLink);
        };
        ADConnectService.prototype.getModels = function (year, make) {
            make = encodeURIComponent(make);
            return this.carLookupServer.getAjax(this.$http, this.urls.car_lookup_models(year, make));
        };
        ADConnectService.prototype.getModelsByLink = function (modelsLink) {
            return this.carLookupServer.getAjax(this.$http, modelsLink);
        };
        ADConnectService.prototype.getEngines = function (year, make, model) {
            make = encodeURIComponent(make);
            model = encodeURIComponent(model);
            return this.carLookupServer.getAjax(this.$http, this.urls.car_lookup_engines(year, make, model));
        };
        ADConnectService.prototype.getEnginesByLink = function (enginesLink) {
            return this.carLookupServer.getAjax(this.$http, enginesLink);
        };
        ADConnectService.prototype.searchEngines = function (searchTerms) {
            searchTerms = encodeURIComponent(Utils.replaceAll(searchTerms, '/', '\''));
            return this.carLookupServer.getAjax(this.$http, this.urls.car_lookup_search + searchTerms);
        };
        ADConnectService.prototype.searchVins = function (searchTerms) {
            searchTerms = encodeURIComponent(searchTerms);
            return this.carLookupServer.getAjax(this.$http, this.urls.vin_lookup_search + searchTerms);
        };
        ADConnectService.prototype.getRecentVehicles = function () {
            var self = this;
            return this.adConnectServer.getAjax(this.$http, this.urls.recent_cars);
        };

        ADConnectService.prototype.searchArticle = function (carId, searchTerm, filter) {
            return this.adConnectServer.getAjax(this.$http, this.urls.search(carId, searchTerm, filter));
        };

        ADConnectService.prototype.createADConnectObject = function (uri) {
            if (!uri.iTypeId)
                return new ADConnect.Component(uri.carId, uri.componentId, uri.url, this);
            if (uri.iTypeId && !uri.nonStandardId && !uri.tsbId)
                return new ADConnect.InformationType(uri.carId, uri.componentId, uri.iTypeId, uri.url, this);
            if (uri.iTypeId && uri.nonStandardId)
                return new ADConnect.NonStandard(uri.carId, uri.componentId, uri.iTypeId, uri.nonStandardId, uri.url, this);
            if (uri.iTypeId && uri.tsbId)
                return new ADConnect.Tsb(uri.carId, uri.componentId, uri.iTypeId, uri.tsbId, uri.url, this);
        };

        ADConnectService.prototype.getADConnectObject = function (adConnectUrl) {
            var adObject = this.componentCache.getValue(adConnectUrl.getkey());
            if (adObject) {
                return adObject;
            }
            adObject = this.createADConnectObject(adConnectUrl);
            this.componentCache.add(adConnectUrl.getkey(), adObject);
            return adObject;
        };

        ADConnectService.prototype.getADConnectObjectByUrl = function (url) {
            var uri = new ADConnect.ADConnectURI(this.adConnectServer, url);
            return this.getADConnectObject(uri);
        };

        ADConnectService.prototype.getXHTML = function (url) {
            var deferred = this.$q.defer();

            this.adConnectServer.getAjax(this.$http, ADConnect.ADConnectURI.cleanADUrl(this.adConnectServer, url), { headers: { Accept: 'application/xhtml+xml' } }).success(function (response, status, headers, config) {
                response = response.substr(response.indexOf('</table>'));
                deferred.resolve(response);
            }).error(function (response, status) {
                return deferred.reject(status);
            });

            return deferred.promise;
        };

        ADConnectService.prototype.getITypeCategory = function (ADObjectTitle) {
            var elems = ADObjectTitle.split('>>');
            for (var i = elems.length - 1; i >= 0; i--) {
                var elem = elems[i].trim();
                if (this.iTypeCategoriesDictionary.containsKey(elem)) {
                    return { iTypeId: this.iTypeCategoriesDictionary.getValue(elem), title: elem };
                }
            }
            return null;
        };

        ADConnectService.prototype.getDTCHistory = function (vehicle) {
            var deferred = this.$q.defer();
            if (vehicle && vehicle.carId && vehicle.vin) {
                var self = this;
                this.adConnectServer.getAjax(this.$http, this.urls.dtcs(vehicle.carId, vehicle.vin)).success(function (data) {
                    deferred.resolve(data);
                });
            }
            return deferred.promise;
        };

        ADConnectService.prototype.postDTCHistory = function (carId, vin, activeDtcs, pendingDtcs) {
            var deferred = this.$q.defer();
            if (carId && vin) {
                var postData = [];
                for (var i = 0; i < activeDtcs.length; i++) {
                    postData.push({
                        carid: carId,
                        vin: vin,
                        dtc_id: activeDtcs[i]
                    });
                }
                for (var i = 0; i < pendingDtcs.length; i++) {
                    postData.push({
                        carid: carId,
                        vin: vin,
                        dtc_id: pendingDtcs[i],
                        pending: true
                    });
                }
                this.adConnectServer.postAjax(this.$http, this.urls.postDtcs, postData).success(function (data) {
                    deferred.resolve(data);
                });
            }
            return deferred.promise;
        };

        ADConnectService.prototype.postClearDTCCodes = function (carId, vin) {
            var deferred = this.$q.defer();
            if (carId && vin) {
                var postData = {
                    carid: carId,
                    vin: vin
                };
                this.adConnectServer.postAjax(this.$http, this.urls.clearDtcs(carId), postData).success(function (data) {
                    deferred.resolve(data);
                });
            }
            return deferred.promise;
        };

        ADConnectService.prototype.getDTCDecriptions = function (carId, dtcIds) {
            var deferred = this.$q.defer();
            if (dtcIds && dtcIds.length > 0) {
                var self = this;
                this.adConnectServer.getAjax(this.$http, this.urls.dtcsDescription(carId, dtcIds.join(','))).success(function (data) {
                    deferred.resolve(data);
                }).error(function (data) {
                    deferred.reject(data);
                });
            } else {
                deferred.resolve([]);
            }
            return deferred.promise;
        };

        ADConnectService.prototype.loadImage = function (uri, callback) {
            this.adConnectServer.getAjax(this.$http, uri, { responseType: 'arraybuffer' }).then(function (response) {
                var encodeImage = function (input) {
                    var keyStr = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=";
                    var output = "";
                    var chr1, chr2, chr3, enc1, enc2, enc3, enc4;
                    var i = 0;

                    while (i < input.length) {
                        chr1 = input[i++];
                        chr2 = i < input.length ? input[i++] : Number.NaN;
                        chr3 = i < input.length ? input[i++] : Number.NaN;

                        enc1 = chr1 >> 2;
                        enc2 = ((chr1 & 3) << 4) | (chr2 >> 4);
                        enc3 = ((chr2 & 15) << 2) | (chr3 >> 6);
                        enc4 = chr3 & 63;

                        if (isNaN(chr2)) {
                            enc3 = enc4 = 64;
                        } else if (isNaN(chr3)) {
                            enc4 = 64;
                        }
                        output += keyStr.charAt(enc1) + keyStr.charAt(enc2) + keyStr.charAt(enc3) + keyStr.charAt(enc4);
                    }
                    return output;
                };

                var bytes = new Uint8Array(response.data);
                var url = 'data:image/png;base64,' + encodeImage(bytes);
                callback(url, uri);
            });
        };

        ADConnectService.prototype.getFFHistory = function (vehicle) {
            var deferred = this.$q.defer();
            if (vehicle && vehicle.carId && vehicle.vin) {
                var self = this;
                this.adConnectServer.getAjax(this.$http, this.urls.ff(vehicle.carId, vehicle.vin)).success(function (data) {
                    deferred.resolve(data);
                });
            }
            return deferred.promise;
        };
        ADConnectService.prototype.postFFHistory = function (carId, history) {
            var deferred = this.$q.defer();
            if (carId && history) {
                this.adConnectServer.postAjax(this.$http, this.urls.postFFs(carId), history).success(function (data) {
                    deferred.resolve(data);
                });
            }
            return deferred.promise;
        };
        ADConnectService.prototype.updateUUID = function (uuid) {
            this.adConnectServer.setUUID(uuid);
        };
        ADConnectService.prototype.register = function (deviceName, pin, uuid) {
            var deferred = this.$q.defer();
            var data = { deviceIdentifier: uuid, deviceName: deviceName, activationPIN: pin };
            this.adConnectServer.postAjax(this.$http, this.urls.register, data).success(function (data) {
                deferred.resolve(data);
            }).error(function (data, status, headers, config) {
                deferred.reject(data, status);
            });
            return deferred.promise;
        };
        ADConnectService.prototype.authenticate = function (uuid) {
            var deferred = this.$q.defer();
            this.adConnectServer.getAjax(this.$http, this.urls.authenticate, {}, false).success(function (data) {
                deferred.resolve(data);
            }).error(function (data, status, headers, config) {
                deferred.reject(data, status);
            });
            return deferred.promise;
        };

        ADConnectService.prototype.getApplicationLogo = function (url) {
            var deferred = this.$q.defer();
            this.adConnectServer.getAjax(this.$http, url, null, false).success(function (data, status, headers, config) {
                deferred.resolve(data);
            }).error(function (data, status, headers, config) {
                deferred.reject(data);
            });
            return deferred.promise;
        };
        return ADConnectService;
    })();
    ADConnect.ADConnectService = ADConnectService;
})(ADConnect || (ADConnect = {}));
